/*========================= DATA FROM DESY3  ========================*/
#ifndef D3VER
#define D3VER 101

/*
  D3VER 101: Added D3MODUS
  D3VER 100: Inaugural version with basic information

*/

#ifndef MSDOS
#define INT_32 unsigned int
#else
#define INT_32 unsigned long
#endif
#define CF_INT_32 CF_LONG


/****** DESY3 Machine data ** nominal update rate 1 Hz unless otherwise noted ****/

/*      Timestamps categories:

          T0 - Timestamp refers to the time the data were taken.

          T1 - Timestamp refers to the time the data were last 
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.

          T2 - Timestamp refers to the time the data are being 
               requested from the (data) server.  
*/ 

/* below: DESY3 State Parameters */
/* Timestamp:  Category  T1 */

#define D3STATE_NO 32
char D3STATE_str[D3STATE_NO];
NETKEY D3STATE={ "DESYDATA","D3STATE",CF_TEXT,D3STATE_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3STATE_str };


/* below: beam parameters */
/* Timestamp:  Category  T1 */

double D3EBottom_d[1];                 /* Normalized Energy at Flat Bottom - GeV (set-value) */
NETKEY D3EBottom={ "DESYDATA","D3EBottomN",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3EBottom_d };

double D3ETop_d[1];                 /* Normalized Energy at Flat Top - GeV (set-value) */
NETKEY D3ETop={ "DESYDATA","D3ETopN",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3ETop_d };

double D3PBottom_d[1];                 /* Normalized Momemtum at flat bottom - GeV/c (set-value) */
NETKEY D3PBottom={ "DESYDATA","D3PBottom",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3PBottom_d };

double D3PTop_d[1];                 /* Normalized Momemtum at flat Top - GeV/c (set-value) */
NETKEY D3PTop={ "DESYDATA","D3PTop",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3PBottom_d };

float D3IBottom_f[1];                 /* Current at Injection Energy - mA */
NETKEY D3IBottom={ "DESYDATA","D3IBottom",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3IBottom_f };

float D3ITop_f[1];                 /* Current at Top Energy - mA */
NETKEY D3ITop={ "DESYDATA","D3ITop",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3ITop_f };

#define D3RAMP_NO 100
float D3IRamp_f[D3RAMP_NO];                 /* Current during the ramp - mA */
NETKEY D3IRamp={ "DESYDATA","D3IRamp",CF_FLOAT,D3RAMP_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3IRamp_f };

float D3IScaled_f[D3RAMP_NO];                 /* Scaled Current during the ramp - mA */
NETKEY D3IScaled={ "DESYDATA","D3IScaled",CF_FLOAT,D3RAMP_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3IScaled_f };

#define D3BPMave_NO 3

float D3BPMMeanX_f[D3BPMave_NO];                 /* DESY-3 BPM Orbit Mean: X - mm */
NETKEY D3BPMMeanX={ "DESYDATA","D3BPMMean.X",CF_FLOAT,D3BPMave_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3BPMMeanX_f };

float D3BPMMeanY_f[D3BPMave_NO];                 /* DESY-3 BPM Orbit Mean: Y - mm */
NETKEY D3BPMMeanY={ "DESYDATA","D3BPMMean.Y",CF_FLOAT,D3BPMave_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3BPMMeanY_f };

float D3BPMrmsX_f[D3BPMave_NO];                 /* Desy-3 BPM Orbit RMS: X - mm */
NETKEY D3BPMrmsX={ "DESYDATA","D3BPMRMS.X",CF_FLOAT,D3BPMave_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3BPMrmsX_f };

float D3BPMrmsY_f[D3BPMave_NO];                 /* Desy-3 BPM Orbit RMS: Y - mm */
NETKEY D3BPMrmsY={ "DESYDATA","D3BPMRMS.Y",CF_FLOAT,D3BPMave_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3BPMrmsY_f };

#define D3BPM_NO 31

float D3OrbitInjectX_f[D3BPM_NO];                 /* D3 Orbit-X: Injection - mm */
NETKEY D3OrbitInjectX={ "DESYDATA","D3OrbitInject.X",CF_FLOAT,D3BPM_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3OrbitInjectX_f };

float D3OrbitInjectY_f[D3BPM_NO];                 /* D3 Orbit-Y: Injection - mm */
NETKEY D3OrbitInjectY={ "DESYDATA","D3OrbitInject.Y",CF_FLOAT,D3BPM_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3OrbitInjectY_f };

float D3OrbitEjectX_f[D3BPM_NO];                 /* D3 Orbit-X: Ejection - mm */
NETKEY D3OrbitEjectX={ "DESYDATA","D3OrbitEject.X",CF_FLOAT,D3BPM_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3OrbitEjectX_f };

float D3OrbitEjectY_f[D3BPM_NO];                 /* D3 Orbit-Y: Ejection - mm */
NETKEY D3OrbitEjectY={ "DESYDATA","D3OrbitEject.Y",CF_FLOAT,D3BPM_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3OrbitEjectY_f };


/* Below: RF Data  */
/* Timestamp:  Category  T1 */

float D3RFPowerFwd_f[1];                   /* Desy 3 RF Forward Power kW */
NETKEY D3RFPowerFwd={ "DESYDATA","D3RFPowerFwd",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3RFPowerFwd_f };

float D3RFPowerRef_f[1];                   /* Desy 3 RF Reflected Power kW */
NETKEY D3RFPowerRef={ "DESYDATA","D3RFPowerRef",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3RFPowerRef_f };


/* Below: Vacuum data mbar  */
/* Timestamp:  Category  T1 */

#define D3VAC_NO 67
float D3VacPressure_f[D3VAC_NO];    /* DESY3 Vacuum Pressures */
NETKEY D3VacPressure={ "DESY3DATA","D3VacPressure",CF_FLOAT,D3VAC_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3VacPressure_f };

#define D3HEBTVVAC_NO 7
float D3HEBTVacPressure_f[D3HEBTVVAC_NO];    /* HEBT Vacuum Pressures */
NETKEY D3HEBTVacPressure={ "DESY3DATA","D3HEBTVacPressure",CF_FLOAT,D3HEBTVVAC_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)D3HEBTVacPressure_f };

/* Below: Status Strings  */
/* Timestamp:  Category  T1 */

#define D3MODUS_NO 32
char gcD3MODUS[D3MODUS_NO];    /* DESY3 MODUS String */
NETKEY D3MODUS={ "DESY3DATA","D3MODUS",CF_FLOAT,D3MODUS_NO,1,RC_INI,T_INIT,ST_INI,D3VER,(void*)gcD3MODUS };

#endif /* D3VER */
