/***************************  h1data_2002.h  *****************************************/
 
/*
  WWW access http://www-h1.desy.de/h1/www/sc/netmex/h12002/h1data_2002.h
  Questions:   jan.olsson@desy.de,   sheviako@mail.desy.de    
*/
 
/*      Timestamps categories:
 
          T0 - Timestamp refers to the time the data were taken.                         T1 - Timestamp refers to the time the data were last
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.
 
          T2 - Timestamp refers to the time the data are being
               requested from the (data) server.
 
                e.g. If the dataserver does not explicitly set
                the timestamp for a keyword entry, the timestamp will
                automatically be set to the time of the call (i.e. now).
*/                                     

 #define VER 103
 #define N_Dat 32

 /* ========  THE DATA FROM THE H1 LUMINOSITY SYSTEM  ==================  */

 /* H1LUMIData: it includes the data measured by H1 Lumi system itself    */
 /*             and related data measured by another  systems.            */
 /*                                                                       */
 /*         Measurement time, UTC seconds       [0]                       */
 /*         Proton   Current, mA                [1]                       */
 /*         Lepton Current,   mA                [2]                       */
 /*         proton background rate, kHz         [3]                       */
 /*         e+/-   background rate, kHz         [4]                       */
 /*         Etag_6 background rate, kHz         [5]                       */
 /*         Water Cherenkov Counter rate, kHz   [6]                       */
 /*         Photon Arm rate,      kHz           [7]                       */
 /*         Photon Beam position X, cm          [8]                       */
 /*         Photon Beam position Y, cm          [9]                       */
 /*         Radiation Monitor, kHz             [10]                       */
 /*         PD Collision rate,     kHz         [11]                       */
 /*         PD e-pilot rate,      kHz          [12]                       */
 /*         PD p-pilot rate,      kHz          [13]                       */
 /*         intgr HERA delivered lumi,  nb-1   [14]                       */
 /*         intgr H1 Run gated lumi,  nb-1     [15]                       */
 /*         intgr H1 data taken lumi  nb-1     [16]                       */
 /*         Annual HERA delivered lumi, nb-1   [17]                       */
 /*         Annual H1 Run gated lumi. nb-1     [18]                       */
 /*         Annual H1 data taken lumi nb-1     [19]                       */
 /*         H1 Solenoid current, A             [20]                       */
 /*         H1 Toroid current,   A             [21]                       */
 /*         PD status  [0.0 - 1.0]             [22]                       */
 /*         PD platform position, mm           [23]                       */
 /*         H1 SPACAL bkg left,   kHz          [24]                       */
 /*         H1 SPACAL bkg right,  kHz          [25]                       */
 /*         H1 SPACAL bkg top,    kHz          [26]                       */
 /*         H1 SPACAL bkg bottom, kHz          [27]                       */
 /*         H1 DAQ run number,                 [28]                       */
 /*         ----------- Spare -------------    [29]                       */
 /*         Lumi,  cm-2 s-1                    [30]                       */
 /*         Specific Lumi, cm-2 s-1 mA-2       [31]                       */


 /*     Timestamp:  Category  T1                                          */

 float Lumidat_f[N_Dat];
 NETKEY H1LUMIData={"H1LDATA","LUMIDATA",CF_FLOAT,N_Dat,0,RC_INI,T_INIT,ST_INI,VER,Lumidat_f};





 /* =========  THE DATA FROM THE H1 TRACKER DETECTORS  =================  */

 /* H1HVData: it includes the  HV systems data from CJC1, CJC2, FMD, FTD  */
 /*           BPC, CIC, COP, COZ and data measured by another systems.    */
 /*                                                                       */
 /*         Measurement time, UTC seconds       [0]                       */
 /*         BPC  HV Current, mkA                [1]                       */
 /*         CIC  HV Current, mkA                [2]                       */
 /*         CJC1 HV Current, mkA                [3]                       */
 /*         CJC2 HV Current, mkA                [4]                       */
 /*         COP  HV Current, mkA                [5]                       */
 /*         COZ  HV Current, mkA                [6]                       */
 /*         FMD  HV Current, mkA                [7]                       */
 /*         FTD  HV Current, mkA                [8]                       */
 /*         ----------- Spare -------------     [9]                       */
 /*         ----------- Spare -------------    [10]                       */
 /*         ----------- Spare -------------    [11]                       */
 /*         ----------- Spare -------------    [12]                       */
 /*         ----------- Spare -------------    [13]                       */
 /*         ----------- Spare -------------    [14]                       */
 /*         ----------- Spare -------------    [15]                       */
 /*         ----------- Spare -------------    [16]                       */
 /*         ----------- Spare -------------    [17]                       */
 /*         ----------- Spare -------------    [18]                       */
 /*         ----------- Spare -------------    [19]                       */
 /*         ----------- Spare -------------    [20]                       */
 /*         ----------- Spare -------------    [21]                       */
 /*         ----------- Spare -------------    [22]                       */
 /*         ----------- Spare -------------    [23]                       */
 /*         ----------- Spare -------------    [24]                       */
 /*         ----------- Spare -------------    [25]                       */
 /*         ----------- Spare -------------    [26]                       */
 /*         ----------- Spare -------------    [27]                       */
 /*         ----------- Spare -------------    [28]                       */
 /*         ----------- Spare -------------    [29]                       */
 /*         ----------- Spare -------------    [30]                       */
 /*         ----------- Spare -------------    [31]                       */


 /*     Timestamp:  Category  T1                                          */



 float HVdat_f[N_Dat];
 NETKEY H1HVData={"H1LDATA","HVDATA",CF_FLOAT,N_Dat,0,RC_INI,T_INIT,ST_INI,VER,HVdat_f};
