/***************************  h1data_2002.h  *****************************************/
 
/*
  WWW access http://www-h1.desy.de/h1/www/sc/netmex/h12002/h1data_2002.h
  Questions:   jan.olsson@desy.de,   sheviako@mail.desy.de    
*/
 
/*      Timestamps categories:
 
          T0 - Timestamp refers to the time the data were taken.                

          T1 - Timestamp refers to the time the data were last
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.
 
          T2 - Timestamp refers to the time the data are being
               requested from the (data) server.
 
                e.g. If the dataserver does not explicitly set
                the timestamp for a keyword entry, the timestamp will
                automatically be set to the time of the call (i.e. now).
*/                                     

 #define VER 105
 #define N_Dat1 32
 #define N_Dat2 40

 /* ========  THE DATA FROM THE H1 LUMINOSITY SYSTEM  ==================  */

 /* H1LUMIData: it includes the data measured by H1 Lumi system itself    */
 /*             and related data measured by  other systems.              */
 /*                                                                       */
 /*         Measurement time, UTC seconds       [0]                       */
 /*         Proton   Current, mA                [1]                       */
 /*         Lepton Current,   mA                [2]                       */
 /*         proton background rate, kHz         [3]                       */
 /*         e+/-   background rate, kHz         [4]                       */
 /*         Etag_6 background rate, kHz         [5]                       */
 /*         Water Cherenkov Counter rate, kHz   [6]                       */
 /*         Photon Arm rate,      kHz           [7]                       */
 /*         Photon Beam position X, cm          [8]                       */
 /*         Photon Beam position Y, cm          [9]                       */
 /*         Radiation Monitor, kHz             [10]                       */
 /*         PD Collision rate,     kHz         [11]                       */
 /*         PD e-pilot rate,      kHz          [12]                       */
 /*         PD p-pilot rate,      kHz          [13]                       */
 /*         intgr HERA delivered lumi,  nb-1   [14]                       */
 /*         intgr H1 Run gated lumi,  nb-1     [15]                       */
 /*         intgr H1 data taken lumi  nb-1     [16]                       */
 /*         Annual HERA delivered lumi, nb-1   [17]                       */
 /*         Annual H1 Run gated lumi. nb-1     [18]                       */
 /*         Annual H1 data taken lumi nb-1     [19]                       */
 /*         H1 Solenoid current, A             [20]                       */
 /*         H1 Toroid current,   A             [21]                       */
 /*         PD status  [0.0 - 1.0]             [22]                       */
 /*         PD platform position, mm           [23]                       */
 /*         H1 SPACAL bkg left,   kHz          [24]                       */
 /*         H1 SPACAL bkg right,  kHz          [25]                       */
 /*         H1 SPACAL bkg top,    kHz          [26]                       */
 /*         H1 SPACAL bkg bottom, kHz          [27]                       */
 /*         H1 DAQ run number,                 [28]                       */
 /*         ----------- Spare -------------    [29]                       */
 /*         Lumi,  cm-2 s-1                    [30]                       */
 /*         Specific Lumi, cm-2 s-1 mA-2       [31]                       */

 /*     Timestamp:  Category  T1                                          */

 float Lumidat_f[N_Dat1];
 NETKEY H1LUMIData={"H1LDATA","LUMIDATA",CF_FLOAT,N_Dat1,0,RC_INI,T_INIT,ST_INI,VER,Lumidat_f};





 /* =========  THE DATA FROM THE H1 TRACKER DETECTORS  =================  */

 /* H1HVData: it includes the  HV systems data from CJC1, CJC2, FMD, FTD  */
 /*           BPC, CIC, COP, COZ and data measured by another systems.    */
 /*                                                                       */
 /*         Measurement time, UTC seconds       [0]                       */
 /*         BPC  HV Current, mkA                [1]                       */
 /*         CIC  HV Current, mkA                [2]                       */
 /*         CJC1 HV Current, mkA                [3]                       */
 /*         CJC2 HV Current, mkA                [4]                       */
 /*         COP  HV Current, mkA                [5]                       */
 /*         COZ  HV Current, mkA                [6]                       */
 /*         FMD  HV Current, mkA                [7]                       */
 /*         FTD  HV Current, mkA                [8]                       */
 /*         TEMP Position C4a R Innen           [9]                       */
 /*         TEMP Position C4b R Innen          [10]                       */
 /*         TEMP Position C5a R Innen          [11]                       */
 /*         TEMP Position C5b R Innen          [12]                       */
 /*         TEMP GO flange Oben                [13]                       */
 /*         TEMP GG flange Oben                [14]                       */
 /*         TEMP Key at +3570 mm Oben          [15]                       */
 /*         TEMP GG trans.flange R Innen       [16]                       */
 /*         TEMP AlBe Metall Oben              [17]                       */
 /*         TEMP GO Pick-up Balg Oben # 1      [18]                       */
 /*         TEMP  GG Pick-up Balg Oben # 2     [19]                       */
 /*         ----------- Spare -------------    [20]                       */
 /*         ----------- Spare -------------    [21]                       */
 /*         ----------- Spare -------------    [22]                       */
 /*         ----------- Spare -------------    [23]                       */
 /*         ----------- Spare -------------    [24]                       */
 /*         ----------- Spare -------------    [25]                       */
 /*         ----------- Spare -------------    [26]                       */
 /*         ----------- Spare -------------    [27]                       */
 /*         ----------- Spare -------------    [28]                       */
 /*         ----------- Spare -------------    [29]                       */
 /*         ----------- Spare -------------    [30]                       */
 /*         ----------- Spare -------------    [31]                       */

 /*     Timestamp:  Category  T1                                          */



 float HVdat_f[N_Dat1];
 NETKEY H1HVData={"H1LDATA","HVDATA",CF_FLOAT,N_Dat1,0,RC_INI,T_INIT,ST_INI,VER,HVdat_f};


 /* =========    DATA FROM THE   HERA PILOT   PROJECT   =================  */
 /*          August 2003                                                   */

 /* Luminosity System:                                                     */ 
 /* L[0]  H1Lu_Time_Fast       // time stamp for fast H1 lumi info         */
 /* L[1]  H1Lu_Rate_Fast       // photon detector rate, short int. time    */
 /* L[2]  H1Lu_Rate_Fast_Error // stat. error (sqrt(N)) on above variable  */
 /* L[3]  H1Lu_Inst_Fast       // instantaneous luminosity, short int. time*/
 /* L[4]  H1Lu_Inst_Fast_Error // stat. error (sqrt(N)) on above variable  */
 /* L[5]  H1Lu_Spec_Fast       // specific luminosity, short int. time     */
 /* L[6]  H1Lu_Spec_Fast_Error // stat. error (sqrt(N)) on above variable  */
 /* L[7]  [Spare]                                                          */
 /* L[8]  H1Lu_Time_Slow       // time stamp for slow H1 lumi info         */
 /* L[9]  H1Lu_Rate_Slow       // photon detector rate,  long int. time    */
 /* L[10] H1Lu_Rate_Slow_Error // stat. error (sqrt(N)) on above variable  */
 /* L[11] H1Lu_Inst_Slow       // instantaneous luminosity, long int. time */
 /* L[12] H1Lu_Inst_Slow_Error // stat. error (sqrt(N)) on above variable  */
 /* L[13] H1Lu_Spec_Slow       // specific luminosity,  long int. time     */
 /* L[14] H1Lu_Spec_Slow_Error // stat. error (sqrt(N)) on above variable  */
 /* L[15] H1Lu_Xave            // average X position of photon beamspit    */
 /* L[16] H1Lu_Yave            // average Y position of photon beam        */
 /* L[17] H1Lu_XRMS            // RMS X position of photon beam            */
 /* L[18] H1Lu_YRMS            // RMS Y position of photon beam            */
 /* L[19] H1Lu_NHit            // number of reconstructed photons          */
 /* L[20] [Spare]                                                          */
 /* Background Rates:                                                      */
 /* B[21] H1BG_time            // time stamp for H1 backgrounds            */
 /* B[22] H1BG_Veto_e          // Veto-Wall rate, e-gated                  */
 /* B[23] H1BG_Veto_p          // Veto-Wall rate, p-gated                  */
 /* B[24] H1BG_BTOF_e          // Backward TOF rate, e-gated               */
 /* B[25] H1BG_BTOF_p          // Backward TOF rate, p-gated               */
 /* B[26] H1BG_STOF_e          // SpaCal TOF rate, e-gated                 */
 /* B[27] H1BG_STOF_p          // SpaCal TOF rate, p-gated                 */
 /* B[28] H1BG_BTS_e           // Backward tagging rate, e-gated           */
 /* B[29] H1BG_BTS_p           // Backward tagging rate, p-gated           */
 /* B[30] [Spare]                                                          */
 /* Radiation Monitors:                                                    */
 /* R[31] H1RM_time            // time stamp for H1 Radiation Monitors     */
 /* R[32] H1RM_BST_Pad         // BST Pad rate                             */
 /* R[33] H1RM_Diode_B         // Radiation Monitor diodes, pos. B         */
 /* R[34] H1RM_Diode_C         // Radiation Monitor diodes, pos. C         */
 /* R[35] [Spare]                                                          */
 /* R[36] [Spare]                                                          */
 /* R[37] [Spare]                                                          */
 /* R[38] [Spare]                                                          */
 /* R[39] [Spare]                                                          */


 double Pilotdat_d[N_Dat2];

 NETKEY
H1LUMIData={"H1LDATA","PILOTDATA",CF_DOUBLE,N_Dat2,0,RC_INI,T_INIT,ST_INI,VER,Pilotdat_d};
