/***************************  h1lumidata.h  ******************************************/
#define H1LUMIVER 101

/*
  WWW access http://www-h1.desy.de/h1/www/sc/netmex/lumi/mexsw/lumicln/clnmod/
	general: http://www-h1.desy.de/h1/www/sc/netmex/lumi/lumisrv.html

  H1LUMIVER 101: QVT keyword was added
*/

/*      Timestamps categories:

          T0 - Timestamp refers to the time the data were taken.


          T1 - Timestamp refers to the time the data were last 
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.

          T2 - Timestamp refers to the time the data are being 
               requested from the (data) server.  
                
                e.g. If the dataserver does not explicitly set
                the timestamp for a keyword entry, the timestamp will 
                automatically be set to the time of the call (i.e. now).
*/ 

#define VER 101
#define NO_MYDATA 32

                /* H1lumidata: it includes the data measured by lumi system itself,      */ 
                /*             measured by another systems, but available for MEXworld   */
                /*             only on this server and                                   */
                /*             measured by another systems and used in lumisystem        */
                /*                    (crosscheck)                                       */
                /*                                                                       */
                /*         Lumi measurement time, UTC seconds  [0]                       */
                /*         Fixed Cherenkov Counter rate, kHz   [1]                       */
                /*         Total Lumi Trigger*ETaccept., kHz   [2]                       */
                /*         proton background rate, kHz         [3]                       */
                /*         e+/-   background rate, kHz         [4]                       */
                /*         Lumi*ETaccept., cm-2s-1             [5]                       */
                /*         Specific Lumi*ETaccept.,cm-2s-1mA-2 [6]                       */
                /*         Bkg Lumi Trigger Rate*ETaccept.,kHz [7]                       */
                /*         Photon Beam X, cm                   [8]                       */
                /*         Photon Beam Y, cm                   [9]                       */
                /*         RadiationMonitor, kHz              [10]                       */
                /*         Movable Cherenkov Counter, kHz     [11]                       */
                /*         Positron Current, mA               [12]                       */
                /*         Proton   Current, mA               [13]                       */
                /*         HERA_lumi,          cm-2           [14]                       */
                /*         H1_Run_lumi,        cm-2           [15]                       */
                /*         H1_data_taken_lumi, cm-2           [16]                       */
                /*         e+ pilot Current, mA               [17]                       */
                /*         Etag44, kHz                        [18]                       */
                /*         Etag33, kHz                        [19]                       */
                /*         Photon Arm, kHz                    [20]                       */
                /*         Proton tagger, kHz                 [21]                       */
                /*         Etag8 total rate, kHz              [22]                       */
                /*         Etag8 e-rate, kHz                  [23]                       */
                /*         dead time, [0-1]                   [24]                       */
                /*         H1 DAQ run number,                 [25]                       */
                /*         Zeus Luminosity, cm-2s-1           [26]                       */
                /*         ZeusSpecificLuminosity,cm-2s-1mA-2 [27]                       */
                /*         Total Lumi Trigger,kHz             [28]                       */
                /*         Bkg Lumi Trigger Rate, kHz         [29]                       */
                /*         Lumi,  cm-2s-1                     [30]                       */
                /*         Specific Lumi, cm-2s-1mA-2         [31]                       */
                /*                                                                       */
                /*     Timestamp:  Category  T0                                          */
								

                
float FLUMI_f[NO_MYDATA];                
NETKEY H1FLUMI={ "H1LUMI","FLUMI",CF_FLOAT,NO_MYDATA,0,RC_INI,T_INIT,ST_INI,VER,(void*)FLUMI_f };


                /* H1QVT: QVT plot from TOF - 960 long word array                       */ 
                /*                    updated every 30 seconds                           */
                /*     Timestamp:  Category  T0                                          */

#define NO_QVT 960               
long QVT_l[NO_QVT];                
NETKEY H1QVT={ "H1LUMI","QVT",CF_LONG,NO_QVT,0,RC_INI,T_INIT,ST_INI,VER,(void*)QVT_l };



/*============================================================*/
/* NOT MEX part - you need it if use sw example */

typedef struct {
    char   title[60];
    char   units[20];
    }      spec;
    
 spec    luspec[32]= {
         {"Lumi Event Time",          "UTC seconds"},
         {"Fixed Cherenkov Counter",  "kHz"},
         {"Total Lumi Trigger*ET_Acc","kHz"},
         {"p -Bkg",                   "kHz"},
         {"e+-Bkg",                   "kHz"},
         {"Lumi*ET_Acc",              "cm-2s-1"},
         {"Specific Lumi*ET_Acc",     "cm-2s-1mA-2"},
         {"Bkg Lumi Trigger*ET_Acc",  "kHz"},
         {"Photon Beam X",            "cm"},
         {"Photon Beam Y",            "cm"},
         {"RadiationMonitor",         "kHz"},
         {"Movable Cherenkov Counter","kHz"},
         {"Positron Current",         "mA"},
         {"Proton   Current",         "mA"},
         {"HERA_lumi",                "cm-2"},
         {"H1_Run_lumi",              "cm-2"},
         {"H1_data_taken_lumi",       "cm-2"},
         {"e+ pilot Current",         "mA"},
         {"Etag44",                   "kHz"},
         {"Etag33",                   "kHz"},
         {"Photon Arm",               "kHz"},
         {"Proton tagger",            "kHz"},
         {"Etag8 total rate",         "kHz"},
         {"Etag8 e-rate,",            "kHz"},
         {"dead time",                "[0-1]"},
         {"H1 DAQ run number",        "kHz"},
         {"Zeus Luminosity",          "cm-2s-1"},
         {"Zeus Specific Luminosity", "cm-2s-1mA-2"},
         {"Total Lumi Trigger",       "kHz"},
         {"Bkg Lumi Trigger Rate",    "kHz"},
         {"Lumi",                 "cm-2s-1"},
         {"Specific Lumi",        "cm-2s-1mA-2"},
 };
 

