/****************************************************************************
*
*   name of file:  h1data.h
*
*   purpose:       header file for NextMEX client code, version 1.02
*                  including NETKEYs for H1 data and information
*                  about run status of the H1 detector, status, rates and
*                  position of the Roman pots ( forward proton
*                  spectrometer = FPS ) and data of the H1 Lopez monitor
*
*   created:       Jul, 1996
*
*   author:        A. Usik
*
*   changes:       May 10, 1997 by T. Wilksen
*
*                  * included new horizontal stations of FPS @ 63m and 80m
*
*                  * included data of H1 Lopez monitor, see also for a
*                    description
*
*                            http:/www-h1.desy.de/~sven/ppu/ppu.html
*
*   mail:          wilksen@mail.desy.de
*
*   remarks:       The web page of A. Usik for the H1-MEX server is NOT
*                  up to date !
*
*
*                  Timestamp conventions:
*
*                  data belonging to the H1 run status ( all except FPS
*                  and beam pick up data ):
*
*                  the data is recorded by the H1 online logging task,
*                  running on dice1, which gets the data from the frontend
*                  devices, resp. filter farm and buffering machines. It is
*                  written every 20 secs to disk and collected by the H1 MEX
*                  server then. The timestamps of each Netkey item belongs
*                  to the time of writing which is within 1-2 secs the
*                  time of recording/measuring and fetching via bus systems
*                  and net. If the logging task stops, e.g. during no lumi,
*                  the timestamps of course may be outdated.
*
*                  The FPS data has got timestamps which represent the
*                  time of measuring within an accuracy of 500 ms.
*                  These timestamps should be up to date all the time,
*                  unless the front end PC or the net connection is down.
*
*                  Refresh times, resp. update times are currently 20 secs
*                  for the H1 detector status and 10 secs for the FPS data.
*                  The Lopez monitor data is updated every 30 min ! See also
*                  remark below.
*
*****************************************************************************/

/* version number of header file */

#define VER 101

/* run and logging status of H1 detector */

/* current H1 run number, no unit  */

long H1_RunNr_l[1];

/* character string with value:   RUNSTART
                                  RUNNING
                                  PAUSED
                                  RUNEND
                                  STOPPED
                                  UNDEFINED
                                  WARMSTART
*/

char H1Status_c[20];

/* character string giving starting date of run dd/mm/yy */

char H1_RSDat_c[20];

/* character string giving starting time of run hh:mm:ss */

char H1_RSTim_c[20];

/* variable giving the current event number in run */

long H1_Ev_Nr_l[1];

/* average event size, unit is kB */

float H1_ESize_f[1];

/* average logging rate, unit is Hz */

float H1_LRate_f[1];

/* the meaning of the following bit patterns could be asked for
   at Jan Olsson, olsson@mail.desy.de */

/* bit-pattern for detector components included in the read out */

long H1_XIMSK_l[1];

/* bit-pattern for active Full_Event_Buffer tasks */

long H1_FBMSK_l[1];

/* H1 detector system mode as a bit=pattern */

long H1_SYSMD_l[1];

/* H1 CDAQ mode, bit-pattern */

long H1_DAQMD_l[1];

/* current dead time of logging task, unit is % */

float H1_DTime_f[1];

/* current of H1 compensator magnet, unit is A  */

long H1_I_Com_l[1];

/* current of H1 solenoid, unit is A  */

long H1_I_Sol_l[1];

/* current of forward muon toroid, unit is A */

long H1_I_Tor_l[1];

/* central trigger set up, character string */

char H1_Tr_Ph_c[20];

/* trigger mode specification, bit-patterns */

long H1_Tr_W1_l[1];
long H1_Tr_W2_l[1];

/* active subtriggers 0 - 31, bit-pattern */

long H1_SubT0_l[1];

/* active subtriggers 31 - 63, bit-pattern */

long H1_SubT1_l[1];

/* active subtriggers 64 - 95, bit-pattern */

long H1_SubT2_l[1];

/* active subtriggers 96 - 127, bit-pattern */

long H1_SubT3_l[1];

/* status of BBL3 central slow control system, bit-pattern */

long H1_BBL3_l[1];

/* status of HV settings, subdetectors, bit-pattern */

long H1_Sta1_l[1];
long H1_Sta2_l[1];

/* names of shift leader on shift crew, character strings */

char H1_CDAQ1_c[20];
char H1_CDAQ2_c[20];
char H1_CDAQ3_c[20];
char H1_CDAQ4_c[20];

/* mode of data logging, usually SGI */

char H1_LogTo_c[20];

/* current luminosity, unit is  mb-1/s */

/* for accessing more information H1 lumi, please have a look on

   http://dice2.desy.de/%7Eusik/netmex/lumisrv.html

*/

float H1_Lumi_f[1];

/* integrated lumi for run, ungated, unit is  mb-1 */

float H1_IntuL_f[1];

/* integrated lumi for run, gated, unit is  mb-1 */

float H1_IntgL_f[1];

/*
The following data contains some slow control information about the H1
forward proton spectrometer (FPS), located in the forward region of the
H1 detector. It consists of four Roman pot stations at z=+90m, 81m, 80m
and 63m of the proton beam pipe. A Roman pot is a moveable vacuum insert
which can be moved close to the proton beam to detect off-momentum protons
stemming from diffractive events in the H1 detector.

The pots are controlled by one PC. All slow control information which is
used by this PC is available also in a FPSINFO file of the old MEX format,
however only on dice machines. In addition to that this data is also provided
on the general H1 NextMEX server. One can call this server with RPC like NextMEX
calls. Some information can be found on the web (http://dice2.desy.de/~usik/netmex
 NOT UP TO DATE ! and http://www-mpy.desy.de/netmex ).

There is also information about the two synchrotron radiation masks C4
and C6 in the H1 detector area. They are steered via SEDAQ-electronics
with the FPS-PC too.
 */

/*
distance between the bottom of the vacuum insert and the nominal axis
of the proton beam (in the retracted position it should be
about 60mm; the position close to the beam is for the 90m station of about 16mm).
This value is the more accurate value coming from a position measurement device.
The accuracy is better than 50 microns.  Unit is mm
*/

float H1_FPS90y_f[1];

/* same as for the 90m station, but the position of the vertical 81m pot
close to the beam corresponds to a distance of about 12mm. */

float H1_FPS81y_f[1];

/* same as for the 90m station, but the position of the horizontal 80m pot
close to the beam corresponds to a distance of about ? mm. */

float H1_FPS80x_f[1];

/* same as for the 90m station, but the position of the horizontal 63m pot
close to the beam corresponds to a distance of about  ? mm. */

float H1_FPS63x_f[1];

/* counting rate of the beam loss monitor
mounted at the proton beam pipe at 92m.
Used as a monitor during the insertion of the 90m pot.
Unit is Hz. */

float H1_FPSblm92_f[1];

/* counting rate of a coincidence signal in four trigger tiles. Choosen are
those trigger tiles which are most in the middle of each plane and most
close to the beam axis. This rate should roughly indicate a clear track
through one station.  Unit is Hz. */

float H1_FPStcn90_f[1];

/*  trigger counting rate which is provided by an internal trigger logic
and is one of the contributions to the FPS triggerelement.
This rate may change drastically due to changes in the FPS DAQ trigger
setting, so do not use this rate for safety monitoring or other purposes !
Unit is Hz. */

float H1_FPStin90_f[1];

/* trigger rate from the forward neutron calorimeter with low threshold
(the FNC is located at z = +107m); this rate is used to monitor the moving
of the 90m pot Unit is Hz */

float H1_NCounter_f[1];

/* The following data items are similar, four items per Roman pot station,
the last one of each station is not really used, unit is for all Hz */

float H1_FPSblm84_f[1];

float H1_FPStcn81_f[1];

float H1_FPStin81_f[1];

float H1_FPStrig_f[1];

float H1_FPSblm80_f[1];

float H1_FPStcn80_f[1];

float H1_FPStin80_f[1];

float H1_FPSnused_f[1];

float H1_FPSblm63_f[1];

float H1_FPStcn63_f[1];

float H1_FPStin63_f[1];

/* means, that the Roman pots have reached a region where the pots ar close to
the proton beam. This integer represents a bit pattern: 0 means no pot is in
the critical zone, 1 means only pot 81, 2 means only pot 90 and 3 both pots are
in the critical region. */

long H1_CritFlag_l[1];

/* represents a status, where the pots are in their final position close to the
beam. This in fact a bit pattern. 0 means no DAQ flag is set, 1 means it is. */

long H1_DaqFlag_l[1];

/* Next to the C3 collimator which is steered and controlled
by the HERA machine group the C4 and C6 collimators are
steered by the Roman pot PC of H1. The C4 collimtor is
located under the forward tracker and C6 behind the Spacal.

position of the C4 collimator yoke in negative y-direction (lower one) relative
to the beampipe).  Unit is mm.  */

float H1_SColC4yd_f[1];

/* position of the C4 collimator yoke in the positive  y-direction (upper
one relative to the beampipe) */

float H1_SColC4yu_f[1];

/* position of the inner (with respect to the x-axis) C4 collimator yoke. */

float H1_SColC4i_f[1];

/* position of the outer (with respect to the x-axis) C4 collimator yoke. */

float H1_SColC4o_f[1];

/* position of the inner (with respect to the x-axis) C6 collimator yoke. */

float H1_SColC6i_f[1];

/* position of the outer (with respect to the x-axis) C6 collimator yoke. */

float H1_SColC6o_f[1];

NETKEY H1_RunNr={ "H1SERVER","H1_RunNr",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RunNr_l };
NETKEY H1Status={ "H1SERVER","H1Status",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1Status_c };
NETKEY H1_RSDat={ "H1SERVER","H1_RSDat",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RSDat_c };
NETKEY H1_RSTim={ "H1SERVER","H1_RSTim",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RSTim_c };
NETKEY H1_Ev_Nr={ "H1SERVER","H1_Ev_Nr",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Ev_Nr_l };
NETKEY H1_ESize={ "H1SERVER","H1_ESize",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_ESize_f };
NETKEY H1_LRate={ "H1SERVER","H1_LRate",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_LRate_f };
NETKEY H1_XIMSK={ "H1SERVER","H1_XIMSK",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_XIMSK_l };
NETKEY H1_FBMSK={ "H1SERVER","H1_FBMSK",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FBMSK_l };
NETKEY H1_SYSMD={ "H1SERVER","H1_SYSMD",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SYSMD_l };
NETKEY H1_DAQMD={ "H1SERVER","H1_DAQMD",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DAQMD_l };
NETKEY H1_DTime={ "H1SERVER","H1_DTime",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DTime_f };
NETKEY H1_I_Com={ "H1SERVER","H1_I_Com",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Com_l };
NETKEY H1_I_Sol={ "H1SERVER","H1_I_Sol",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Sol_l };
NETKEY H1_I_Tor={ "H1SERVER","H1_I_Tor",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Tor_l };
NETKEY H1_Tr_Ph={ "H1SERVER","H1_Tr_Ph",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Tr_Ph_c };
NETKEY H1_Tr_W1={ "H1SERVER","H1_Tr_W1",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Tr_W1_l };
NETKEY H1_SubT0={ "H1SERVER","H1_SubT0",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT0_l };
NETKEY H1_SubT1={ "H1SERVER","H1_SubT1",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT1_l };
NETKEY H1_SubT2={ "H1SERVER","H1_SubT2",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT2_l };
NETKEY H1_SubT3={ "H1SERVER","H1_SubT3",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT3_l };
NETKEY H1_BBL3={ "H1SERVER","H1_BBL3",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BBL3_l };
NETKEY H1_Sta1={ "H1SERVER","H1_Sta1",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Sta1_l };
NETKEY H1_Sta2={ "H1SERVER","H1_Sta2",CF_LONG,        1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Sta2_l };
NETKEY H1_CDAQ1={ "H1SERVER","H1_CDAQ1",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ1_c };
NETKEY H1_CDAQ2={ "H1SERVER","H1_CDAQ2",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ2_c };
NETKEY H1_CDAQ3={ "H1SERVER","H1_CDAQ3",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ3_c };
NETKEY H1_CDAQ4={ "H1SERVER","H1_CDAQ4",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ4_c };
NETKEY H1_LogTo={ "H1SERVER","H1_LogTo",CF_TEXT,       20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_LogTo_c };
NETKEY H1_Lumi={ "H1SERVER","H1_Lumi",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Lumi_f };
NETKEY H1_IntuL={ "H1SERVER","H1_IntuL",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_IntuL_f };
NETKEY H1_IntgL={ "H1SERVER","H1_IntgL",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_IntgL_f };
NETKEY H1_FPS90y={"H1SERVER","H1_FPS90y",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS90y_f };
NETKEY H1_FPS81y={"H1SERVER","H1_FPS81y",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS81y_f };
NETKEY H1_FPS80x={"H1SERVER","H1_FPS80x",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS80x_f };
NETKEY H1_FPS63x={"H1SERVER","H1_FPS63x",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS63x_f };
NETKEY H1_FPSblm92={"H1SERVER","H1_FPSblm92",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm92_f };
NETKEY H1_FPStcn90={"H1SERVER","H1_FPStcn90",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn90_f };
NETKEY H1_FPStin90={"H1SERVER","H1_FPStin90",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin90_f };
NETKEY H1_NCounter={"H1SERVER","H1_NCounter",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_NCounter_f };
NETKEY H1_FPSblm84={"H1SERVER","H1_FPSblm84",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm84_f };
NETKEY H1_FPStcn81={"H1SERVER","H1_FPStcn81",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn81_f };
NETKEY H1_FPStin81={"H1SERVER","H1_FPStin81",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin81_f };
NETKEY H1_FPStrig={"H1SERVER","H1_FPStrig",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStrig_f };
NETKEY H1_FPSblm80={"H1SERVER","H1_FPSblm80",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm80_f };
NETKEY H1_FPStcn80={"H1SERVER","H1_FPStcn80",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn80_f };
NETKEY H1_FPStin80={"H1SERVER","H1_FPStin80",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin80_f };
NETKEY H1_FPSnused={"H1SERVER","H1_FPSnused",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSnused_f };
NETKEY H1_FPSblm63={"H1SERVER","H1_FPSblm63",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm63_f };
NETKEY H1_FPStcn63={"H1SERVER","H1_FPStcn63",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn63_f };
NETKEY H1_FPStin63={"H1SERVER","H1_FPStin63",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin63_f };
NETKEY H1_CritFlag={"H1SERVER","H1_CritFlag",CF_LONG,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CritFlag_l };
NETKEY H1_DaqFlag={"H1SERVER","H1_DaqFlag",CF_LONG,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DaqFlag_l };
NETKEY H1_SColC4yd={"H1SERVER","H1_SColC4yd",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4yd_f };
NETKEY H1_SColC4yu={"H1SERVER","H1_SColC4yu",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4yu_f };
NETKEY H1_SColC4i={"H1SERVER","H1_SColC4i",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4i_f };
NETKEY H1_SColC4o={"H1SERVER","H1_SColC4o",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4o_f };
NETKEY H1_SColC6i={"H1SERVER","H1_SColC6i",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC6i_f };
NETKEY H1_SColC6o={"H1SERVER","H1_SColC6o",CF_FLOAT,       1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC6o_f };


/*
   Data of the H1 Lopez monitor, see also the web page mentioned in the header.
   As explained on the web the data is currently updated every 30 min, the measurement
   takes place for at maximum 220 bunches with 300 data points per bunch.
   Every data point is an averaged value of 200 measurments.

*/

#define H1_BpuValNo 300

double H1_BPU001_d[H1_BpuValNo];
double H1_BPU002_d[H1_BpuValNo];
double H1_BPU003_d[H1_BpuValNo];
double H1_BPU004_d[H1_BpuValNo];
double H1_BPU005_d[H1_BpuValNo];
double H1_BPU006_d[H1_BpuValNo];
double H1_BPU007_d[H1_BpuValNo];
double H1_BPU008_d[H1_BpuValNo];
double H1_BPU009_d[H1_BpuValNo];
double H1_BPU010_d[H1_BpuValNo];
double H1_BPU011_d[H1_BpuValNo];
double H1_BPU012_d[H1_BpuValNo];
double H1_BPU013_d[H1_BpuValNo];
double H1_BPU014_d[H1_BpuValNo];
double H1_BPU015_d[H1_BpuValNo];
double H1_BPU016_d[H1_BpuValNo];
double H1_BPU017_d[H1_BpuValNo];
double H1_BPU018_d[H1_BpuValNo];
double H1_BPU019_d[H1_BpuValNo];
double H1_BPU020_d[H1_BpuValNo];
double H1_BPU021_d[H1_BpuValNo];
double H1_BPU022_d[H1_BpuValNo];
double H1_BPU023_d[H1_BpuValNo];
double H1_BPU024_d[H1_BpuValNo];
double H1_BPU025_d[H1_BpuValNo];
double H1_BPU026_d[H1_BpuValNo];
double H1_BPU027_d[H1_BpuValNo];
double H1_BPU028_d[H1_BpuValNo];
double H1_BPU029_d[H1_BpuValNo];
double H1_BPU030_d[H1_BpuValNo];
double H1_BPU031_d[H1_BpuValNo];
double H1_BPU032_d[H1_BpuValNo];
double H1_BPU033_d[H1_BpuValNo];
double H1_BPU034_d[H1_BpuValNo];
double H1_BPU035_d[H1_BpuValNo];
double H1_BPU036_d[H1_BpuValNo];
double H1_BPU037_d[H1_BpuValNo];
double H1_BPU038_d[H1_BpuValNo];
double H1_BPU039_d[H1_BpuValNo];
double H1_BPU040_d[H1_BpuValNo];
double H1_BPU041_d[H1_BpuValNo];
double H1_BPU042_d[H1_BpuValNo];
double H1_BPU043_d[H1_BpuValNo];
double H1_BPU044_d[H1_BpuValNo];
double H1_BPU045_d[H1_BpuValNo];
double H1_BPU046_d[H1_BpuValNo];
double H1_BPU047_d[H1_BpuValNo];
double H1_BPU048_d[H1_BpuValNo];
double H1_BPU049_d[H1_BpuValNo];
double H1_BPU050_d[H1_BpuValNo];
double H1_BPU051_d[H1_BpuValNo];
double H1_BPU052_d[H1_BpuValNo];
double H1_BPU053_d[H1_BpuValNo];
double H1_BPU054_d[H1_BpuValNo];
double H1_BPU055_d[H1_BpuValNo];
double H1_BPU056_d[H1_BpuValNo];
double H1_BPU057_d[H1_BpuValNo];
double H1_BPU058_d[H1_BpuValNo];
double H1_BPU059_d[H1_BpuValNo];
double H1_BPU060_d[H1_BpuValNo];
double H1_BPU061_d[H1_BpuValNo];
double H1_BPU062_d[H1_BpuValNo];
double H1_BPU063_d[H1_BpuValNo];
double H1_BPU064_d[H1_BpuValNo];
double H1_BPU065_d[H1_BpuValNo];
double H1_BPU066_d[H1_BpuValNo];
double H1_BPU067_d[H1_BpuValNo];
double H1_BPU068_d[H1_BpuValNo];
double H1_BPU069_d[H1_BpuValNo];
double H1_BPU070_d[H1_BpuValNo];
double H1_BPU071_d[H1_BpuValNo];
double H1_BPU072_d[H1_BpuValNo];
double H1_BPU073_d[H1_BpuValNo];
double H1_BPU074_d[H1_BpuValNo];
double H1_BPU075_d[H1_BpuValNo];
double H1_BPU076_d[H1_BpuValNo];
double H1_BPU077_d[H1_BpuValNo];
double H1_BPU078_d[H1_BpuValNo];
double H1_BPU079_d[H1_BpuValNo];
double H1_BPU080_d[H1_BpuValNo];
double H1_BPU081_d[H1_BpuValNo];
double H1_BPU082_d[H1_BpuValNo];
double H1_BPU083_d[H1_BpuValNo];
double H1_BPU084_d[H1_BpuValNo];
double H1_BPU085_d[H1_BpuValNo];
double H1_BPU086_d[H1_BpuValNo];
double H1_BPU087_d[H1_BpuValNo];
double H1_BPU088_d[H1_BpuValNo];
double H1_BPU089_d[H1_BpuValNo];
double H1_BPU090_d[H1_BpuValNo];
double H1_BPU091_d[H1_BpuValNo];
double H1_BPU092_d[H1_BpuValNo];
double H1_BPU093_d[H1_BpuValNo];
double H1_BPU094_d[H1_BpuValNo];
double H1_BPU095_d[H1_BpuValNo];
double H1_BPU096_d[H1_BpuValNo];
double H1_BPU097_d[H1_BpuValNo];
double H1_BPU098_d[H1_BpuValNo];
double H1_BPU099_d[H1_BpuValNo];
double H1_BPU100_d[H1_BpuValNo];
double H1_BPU101_d[H1_BpuValNo];
double H1_BPU102_d[H1_BpuValNo];
double H1_BPU103_d[H1_BpuValNo];
double H1_BPU104_d[H1_BpuValNo];
double H1_BPU105_d[H1_BpuValNo];
double H1_BPU106_d[H1_BpuValNo];
double H1_BPU107_d[H1_BpuValNo];
double H1_BPU108_d[H1_BpuValNo];
double H1_BPU109_d[H1_BpuValNo];
double H1_BPU110_d[H1_BpuValNo];
double H1_BPU111_d[H1_BpuValNo];
double H1_BPU112_d[H1_BpuValNo];
double H1_BPU113_d[H1_BpuValNo];
double H1_BPU114_d[H1_BpuValNo];
double H1_BPU115_d[H1_BpuValNo];
double H1_BPU116_d[H1_BpuValNo];
double H1_BPU117_d[H1_BpuValNo];
double H1_BPU118_d[H1_BpuValNo];
double H1_BPU119_d[H1_BpuValNo];
double H1_BPU120_d[H1_BpuValNo];
double H1_BPU121_d[H1_BpuValNo];
double H1_BPU122_d[H1_BpuValNo];
double H1_BPU123_d[H1_BpuValNo];
double H1_BPU124_d[H1_BpuValNo];
double H1_BPU125_d[H1_BpuValNo];
double H1_BPU126_d[H1_BpuValNo];
double H1_BPU127_d[H1_BpuValNo];
double H1_BPU128_d[H1_BpuValNo];
double H1_BPU129_d[H1_BpuValNo];
double H1_BPU130_d[H1_BpuValNo];
double H1_BPU131_d[H1_BpuValNo];
double H1_BPU132_d[H1_BpuValNo];
double H1_BPU133_d[H1_BpuValNo];
double H1_BPU134_d[H1_BpuValNo];
double H1_BPU135_d[H1_BpuValNo];
double H1_BPU136_d[H1_BpuValNo];
double H1_BPU137_d[H1_BpuValNo];
double H1_BPU138_d[H1_BpuValNo];
double H1_BPU139_d[H1_BpuValNo];
double H1_BPU140_d[H1_BpuValNo];
double H1_BPU141_d[H1_BpuValNo];
double H1_BPU142_d[H1_BpuValNo];
double H1_BPU143_d[H1_BpuValNo];
double H1_BPU144_d[H1_BpuValNo];
double H1_BPU145_d[H1_BpuValNo];
double H1_BPU146_d[H1_BpuValNo];
double H1_BPU147_d[H1_BpuValNo];
double H1_BPU148_d[H1_BpuValNo];
double H1_BPU149_d[H1_BpuValNo];
double H1_BPU150_d[H1_BpuValNo];
double H1_BPU151_d[H1_BpuValNo];
double H1_BPU152_d[H1_BpuValNo];
double H1_BPU153_d[H1_BpuValNo];
double H1_BPU154_d[H1_BpuValNo];
double H1_BPU155_d[H1_BpuValNo];
double H1_BPU156_d[H1_BpuValNo];
double H1_BPU157_d[H1_BpuValNo];
double H1_BPU158_d[H1_BpuValNo];
double H1_BPU159_d[H1_BpuValNo];
double H1_BPU160_d[H1_BpuValNo];
double H1_BPU161_d[H1_BpuValNo];
double H1_BPU162_d[H1_BpuValNo];
double H1_BPU163_d[H1_BpuValNo];
double H1_BPU164_d[H1_BpuValNo];
double H1_BPU165_d[H1_BpuValNo];
double H1_BPU166_d[H1_BpuValNo];
double H1_BPU167_d[H1_BpuValNo];
double H1_BPU168_d[H1_BpuValNo];
double H1_BPU169_d[H1_BpuValNo];
double H1_BPU170_d[H1_BpuValNo];
double H1_BPU171_d[H1_BpuValNo];
double H1_BPU172_d[H1_BpuValNo];
double H1_BPU173_d[H1_BpuValNo];
double H1_BPU174_d[H1_BpuValNo];
double H1_BPU175_d[H1_BpuValNo];
double H1_BPU176_d[H1_BpuValNo];
double H1_BPU177_d[H1_BpuValNo];
double H1_BPU178_d[H1_BpuValNo];
double H1_BPU179_d[H1_BpuValNo];
double H1_BPU180_d[H1_BpuValNo];
double H1_BPU181_d[H1_BpuValNo];
double H1_BPU182_d[H1_BpuValNo];
double H1_BPU183_d[H1_BpuValNo];
double H1_BPU184_d[H1_BpuValNo];
double H1_BPU185_d[H1_BpuValNo];
double H1_BPU186_d[H1_BpuValNo];
double H1_BPU187_d[H1_BpuValNo];
double H1_BPU188_d[H1_BpuValNo];
double H1_BPU189_d[H1_BpuValNo];
double H1_BPU190_d[H1_BpuValNo];
double H1_BPU191_d[H1_BpuValNo];
double H1_BPU192_d[H1_BpuValNo];
double H1_BPU193_d[H1_BpuValNo];
double H1_BPU194_d[H1_BpuValNo];
double H1_BPU195_d[H1_BpuValNo];
double H1_BPU196_d[H1_BpuValNo];
double H1_BPU197_d[H1_BpuValNo];
double H1_BPU198_d[H1_BpuValNo];
double H1_BPU199_d[H1_BpuValNo];
double H1_BPU200_d[H1_BpuValNo];
double H1_BPU201_d[H1_BpuValNo];
double H1_BPU202_d[H1_BpuValNo];
double H1_BPU203_d[H1_BpuValNo];
double H1_BPU204_d[H1_BpuValNo];
double H1_BPU205_d[H1_BpuValNo];
double H1_BPU206_d[H1_BpuValNo];
double H1_BPU207_d[H1_BpuValNo];
double H1_BPU208_d[H1_BpuValNo];
double H1_BPU209_d[H1_BpuValNo];
double H1_BPU210_d[H1_BpuValNo];
double H1_BPU211_d[H1_BpuValNo];
double H1_BPU212_d[H1_BpuValNo];
double H1_BPU213_d[H1_BpuValNo];
double H1_BPU214_d[H1_BpuValNo];
double H1_BPU215_d[H1_BpuValNo];
double H1_BPU216_d[H1_BpuValNo];
double H1_BPU217_d[H1_BpuValNo];
double H1_BPU218_d[H1_BpuValNo];
double H1_BPU219_d[H1_BpuValNo];
double H1_BPU220_d[H1_BpuValNo];

#define H1_BunchNO 220

double H1_BeamPup_d[H1_BunchNO];

NETKEY H1_BPU001 ={"H1SERVER","H1_BPU001",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU001_d };
NETKEY H1_BPU002 ={"H1SERVER","H1_BPU002",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU002_d };
NETKEY H1_BPU003 ={"H1SERVER","H1_BPU003",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU003_d };
NETKEY H1_BPU004 ={"H1SERVER","H1_BPU004",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU004_d };
NETKEY H1_BPU005 ={"H1SERVER","H1_BPU005",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU005_d };
NETKEY H1_BPU006 ={"H1SERVER","H1_BPU006",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU006_d };
NETKEY H1_BPU007 ={"H1SERVER","H1_BPU007",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU007_d };
NETKEY H1_BPU008 ={"H1SERVER","H1_BPU008",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU008_d };
NETKEY H1_BPU009 ={"H1SERVER","H1_BPU009",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU009_d };
NETKEY H1_BPU010 ={"H1SERVER","H1_BPU010",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU010_d };
NETKEY H1_BPU011 ={"H1SERVER","H1_BPU011",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU011_d };
NETKEY H1_BPU012 ={"H1SERVER","H1_BPU012",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU012_d };
NETKEY H1_BPU013 ={"H1SERVER","H1_BPU013",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU013_d };
NETKEY H1_BPU014 ={"H1SERVER","H1_BPU014",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU014_d };
NETKEY H1_BPU015 ={"H1SERVER","H1_BPU015",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU015_d };
NETKEY H1_BPU016 ={"H1SERVER","H1_BPU016",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU016_d };
NETKEY H1_BPU017 ={"H1SERVER","H1_BPU017",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU017_d };
NETKEY H1_BPU018 ={"H1SERVER","H1_BPU018",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU018_d };
NETKEY H1_BPU019 ={"H1SERVER","H1_BPU019",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU019_d };
NETKEY H1_BPU020 ={"H1SERVER","H1_BPU020",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU020_d };
NETKEY H1_BPU021 ={"H1SERVER","H1_BPU021",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU021_d };
NETKEY H1_BPU022 ={"H1SERVER","H1_BPU022",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU022_d };
NETKEY H1_BPU023 ={"H1SERVER","H1_BPU023",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU023_d };
NETKEY H1_BPU024 ={"H1SERVER","H1_BPU024",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU024_d };
NETKEY H1_BPU025 ={"H1SERVER","H1_BPU025",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU025_d };
NETKEY H1_BPU026 ={"H1SERVER","H1_BPU026",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU026_d };
NETKEY H1_BPU027 ={"H1SERVER","H1_BPU027",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU027_d };
NETKEY H1_BPU028 ={"H1SERVER","H1_BPU028",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU028_d };
NETKEY H1_BPU029 ={"H1SERVER","H1_BPU029",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU029_d };
NETKEY H1_BPU030 ={"H1SERVER","H1_BPU030",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU030_d };
NETKEY H1_BPU031 ={"H1SERVER","H1_BPU031",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU031_d };
NETKEY H1_BPU032 ={"H1SERVER","H1_BPU032",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU032_d };
NETKEY H1_BPU033 ={"H1SERVER","H1_BPU033",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU033_d };
NETKEY H1_BPU034 ={"H1SERVER","H1_BPU034",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU034_d };
NETKEY H1_BPU035 ={"H1SERVER","H1_BPU035",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU035_d };
NETKEY H1_BPU036 ={"H1SERVER","H1_BPU036",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU036_d };
NETKEY H1_BPU037 ={"H1SERVER","H1_BPU037",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU037_d };
NETKEY H1_BPU038 ={"H1SERVER","H1_BPU038",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU038_d };
NETKEY H1_BPU039 ={"H1SERVER","H1_BPU039",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU039_d };
NETKEY H1_BPU040 ={"H1SERVER","H1_BPU040",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU040_d };
NETKEY H1_BPU041 ={"H1SERVER","H1_BPU041",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU041_d };
NETKEY H1_BPU042 ={"H1SERVER","H1_BPU042",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU042_d };
NETKEY H1_BPU043 ={"H1SERVER","H1_BPU043",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU043_d };
NETKEY H1_BPU044 ={"H1SERVER","H1_BPU044",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU044_d };
NETKEY H1_BPU045 ={"H1SERVER","H1_BPU045",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU045_d };
NETKEY H1_BPU046 ={"H1SERVER","H1_BPU046",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU046_d };
NETKEY H1_BPU047 ={"H1SERVER","H1_BPU047",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU047_d };
NETKEY H1_BPU048 ={"H1SERVER","H1_BPU048",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU048_d };
NETKEY H1_BPU049 ={"H1SERVER","H1_BPU049",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU049_d };
NETKEY H1_BPU050 ={"H1SERVER","H1_BPU050",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU050_d };
NETKEY H1_BPU051 ={"H1SERVER","H1_BPU051",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU051_d };
NETKEY H1_BPU052 ={"H1SERVER","H1_BPU052",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU052_d };
NETKEY H1_BPU053 ={"H1SERVER","H1_BPU053",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU053_d };
NETKEY H1_BPU054 ={"H1SERVER","H1_BPU054",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU054_d };
NETKEY H1_BPU055 ={"H1SERVER","H1_BPU055",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU055_d };
NETKEY H1_BPU056 ={"H1SERVER","H1_BPU056",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU056_d };
NETKEY H1_BPU057 ={"H1SERVER","H1_BPU057",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU057_d };
NETKEY H1_BPU058 ={"H1SERVER","H1_BPU058",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU058_d };
NETKEY H1_BPU059 ={"H1SERVER","H1_BPU059",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU059_d };
NETKEY H1_BPU060 ={"H1SERVER","H1_BPU060",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU060_d };
NETKEY H1_BPU061 ={"H1SERVER","H1_BPU061",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU061_d };
NETKEY H1_BPU062 ={"H1SERVER","H1_BPU062",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU062_d };
NETKEY H1_BPU063 ={"H1SERVER","H1_BPU063",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU063_d };
NETKEY H1_BPU064 ={"H1SERVER","H1_BPU064",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU064_d };
NETKEY H1_BPU065 ={"H1SERVER","H1_BPU065",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU065_d };
NETKEY H1_BPU066 ={"H1SERVER","H1_BPU066",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU066_d };
NETKEY H1_BPU067 ={"H1SERVER","H1_BPU067",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU067_d };
NETKEY H1_BPU068 ={"H1SERVER","H1_BPU068",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU068_d };
NETKEY H1_BPU069 ={"H1SERVER","H1_BPU069",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU069_d };
NETKEY H1_BPU070 ={"H1SERVER","H1_BPU070",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU070_d };
NETKEY H1_BPU071 ={"H1SERVER","H1_BPU071",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU071_d };
NETKEY H1_BPU072 ={"H1SERVER","H1_BPU072",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU072_d };
NETKEY H1_BPU073 ={"H1SERVER","H1_BPU073",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU073_d };
NETKEY H1_BPU074 ={"H1SERVER","H1_BPU074",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU074_d };
NETKEY H1_BPU075 ={"H1SERVER","H1_BPU075",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU075_d };
NETKEY H1_BPU076 ={"H1SERVER","H1_BPU076",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU076_d };
NETKEY H1_BPU077 ={"H1SERVER","H1_BPU077",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU077_d };
NETKEY H1_BPU078 ={"H1SERVER","H1_BPU078",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU078_d };
NETKEY H1_BPU079 ={"H1SERVER","H1_BPU079",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU079_d };
NETKEY H1_BPU080 ={"H1SERVER","H1_BPU080",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU080_d };
NETKEY H1_BPU081 ={"H1SERVER","H1_BPU081",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU081_d };
NETKEY H1_BPU082 ={"H1SERVER","H1_BPU082",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU082_d };
NETKEY H1_BPU083 ={"H1SERVER","H1_BPU083",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU083_d };
NETKEY H1_BPU084 ={"H1SERVER","H1_BPU084",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU084_d };
NETKEY H1_BPU085 ={"H1SERVER","H1_BPU085",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU085_d };
NETKEY H1_BPU086 ={"H1SERVER","H1_BPU086",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU086_d };
NETKEY H1_BPU087 ={"H1SERVER","H1_BPU087",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU087_d };
NETKEY H1_BPU088 ={"H1SERVER","H1_BPU088",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU088_d };
NETKEY H1_BPU089 ={"H1SERVER","H1_BPU089",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU089_d };
NETKEY H1_BPU090 ={"H1SERVER","H1_BPU090",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU090_d };
NETKEY H1_BPU091 ={"H1SERVER","H1_BPU091",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU091_d };
NETKEY H1_BPU092 ={"H1SERVER","H1_BPU092",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU092_d };
NETKEY H1_BPU093 ={"H1SERVER","H1_BPU093",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU093_d };
NETKEY H1_BPU094 ={"H1SERVER","H1_BPU094",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU094_d };
NETKEY H1_BPU095 ={"H1SERVER","H1_BPU095",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU095_d };
NETKEY H1_BPU096 ={"H1SERVER","H1_BPU096",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU096_d };
NETKEY H1_BPU097 ={"H1SERVER","H1_BPU097",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU097_d };
NETKEY H1_BPU098 ={"H1SERVER","H1_BPU098",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU098_d };
NETKEY H1_BPU099 ={"H1SERVER","H1_BPU099",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU099_d };
NETKEY H1_BPU100 ={"H1SERVER","H1_BPU100",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU100_d };
NETKEY H1_BPU101 ={"H1SERVER","H1_BPU101",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU101_d };
NETKEY H1_BPU102 ={"H1SERVER","H1_BPU102",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU102_d };
NETKEY H1_BPU103 ={"H1SERVER","H1_BPU103",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU103_d };
NETKEY H1_BPU104 ={"H1SERVER","H1_BPU104",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU104_d };
NETKEY H1_BPU105 ={"H1SERVER","H1_BPU105",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU105_d };
NETKEY H1_BPU106 ={"H1SERVER","H1_BPU106",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU106_d };
NETKEY H1_BPU107 ={"H1SERVER","H1_BPU107",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU107_d };
NETKEY H1_BPU108 ={"H1SERVER","H1_BPU108",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU108_d };
NETKEY H1_BPU109 ={"H1SERVER","H1_BPU109",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU109_d };
NETKEY H1_BPU110 ={"H1SERVER","H1_BPU110",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU110_d };
NETKEY H1_BPU111 ={"H1SERVER","H1_BPU111",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU111_d };
NETKEY H1_BPU112 ={"H1SERVER","H1_BPU112",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU112_d };
NETKEY H1_BPU113 ={"H1SERVER","H1_BPU113",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU113_d };
NETKEY H1_BPU114 ={"H1SERVER","H1_BPU114",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU114_d };
NETKEY H1_BPU115 ={"H1SERVER","H1_BPU115",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU115_d };
NETKEY H1_BPU116 ={"H1SERVER","H1_BPU116",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU116_d };
NETKEY H1_BPU117 ={"H1SERVER","H1_BPU117",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU117_d };
NETKEY H1_BPU118 ={"H1SERVER","H1_BPU118",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU118_d };
NETKEY H1_BPU119 ={"H1SERVER","H1_BPU119",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU119_d };
NETKEY H1_BPU120 ={"H1SERVER","H1_BPU120",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU120_d };
NETKEY H1_BPU121 ={"H1SERVER","H1_BPU121",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU121_d };
NETKEY H1_BPU122 ={"H1SERVER","H1_BPU122",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU122_d };
NETKEY H1_BPU123 ={"H1SERVER","H1_BPU123",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU123_d };
NETKEY H1_BPU124 ={"H1SERVER","H1_BPU124",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU124_d };
NETKEY H1_BPU125 ={"H1SERVER","H1_BPU125",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU125_d };
NETKEY H1_BPU126 ={"H1SERVER","H1_BPU126",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU126_d };
NETKEY H1_BPU127 ={"H1SERVER","H1_BPU127",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU127_d };
NETKEY H1_BPU128 ={"H1SERVER","H1_BPU128",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU128_d };
NETKEY H1_BPU129 ={"H1SERVER","H1_BPU129",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU129_d };
NETKEY H1_BPU130 ={"H1SERVER","H1_BPU130",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU130_d };
NETKEY H1_BPU131 ={"H1SERVER","H1_BPU131",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU131_d };
NETKEY H1_BPU132 ={"H1SERVER","H1_BPU132",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU132_d };
NETKEY H1_BPU133 ={"H1SERVER","H1_BPU133",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU133_d };
NETKEY H1_BPU134 ={"H1SERVER","H1_BPU134",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU134_d };
NETKEY H1_BPU135 ={"H1SERVER","H1_BPU135",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU135_d };
NETKEY H1_BPU136 ={"H1SERVER","H1_BPU136",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU136_d };
NETKEY H1_BPU137 ={"H1SERVER","H1_BPU137",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU137_d };
NETKEY H1_BPU138 ={"H1SERVER","H1_BPU138",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU138_d };
NETKEY H1_BPU139 ={"H1SERVER","H1_BPU139",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU139_d };
NETKEY H1_BPU140 ={"H1SERVER","H1_BPU140",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU140_d };
NETKEY H1_BPU141 ={"H1SERVER","H1_BPU141",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU141_d };
NETKEY H1_BPU142 ={"H1SERVER","H1_BPU142",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU142_d };
NETKEY H1_BPU143 ={"H1SERVER","H1_BPU143",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU143_d };
NETKEY H1_BPU144 ={"H1SERVER","H1_BPU144",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU144_d };
NETKEY H1_BPU145 ={"H1SERVER","H1_BPU145",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU145_d };
NETKEY H1_BPU146 ={"H1SERVER","H1_BPU146",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU146_d };
NETKEY H1_BPU147 ={"H1SERVER","H1_BPU147",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU147_d };
NETKEY H1_BPU148 ={"H1SERVER","H1_BPU148",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU148_d };
NETKEY H1_BPU149 ={"H1SERVER","H1_BPU149",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU149_d };
NETKEY H1_BPU150 ={"H1SERVER","H1_BPU150",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU150_d };
NETKEY H1_BPU151 ={"H1SERVER","H1_BPU151",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU151_d };
NETKEY H1_BPU152 ={"H1SERVER","H1_BPU152",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU152_d };
NETKEY H1_BPU153 ={"H1SERVER","H1_BPU153",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU153_d };
NETKEY H1_BPU154 ={"H1SERVER","H1_BPU154",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU154_d };
NETKEY H1_BPU155 ={"H1SERVER","H1_BPU155",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU155_d };
NETKEY H1_BPU156 ={"H1SERVER","H1_BPU156",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU156_d };
NETKEY H1_BPU157 ={"H1SERVER","H1_BPU157",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU157_d };
NETKEY H1_BPU158 ={"H1SERVER","H1_BPU158",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU158_d };
NETKEY H1_BPU159 ={"H1SERVER","H1_BPU159",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU159_d };
NETKEY H1_BPU160 ={"H1SERVER","H1_BPU160",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU160_d };
NETKEY H1_BPU161 ={"H1SERVER","H1_BPU161",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU161_d };
NETKEY H1_BPU162 ={"H1SERVER","H1_BPU162",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU162_d };
NETKEY H1_BPU163 ={"H1SERVER","H1_BPU163",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU163_d };
NETKEY H1_BPU164 ={"H1SERVER","H1_BPU164",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU164_d };
NETKEY H1_BPU165 ={"H1SERVER","H1_BPU165",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU165_d };
NETKEY H1_BPU166 ={"H1SERVER","H1_BPU166",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU166_d };
NETKEY H1_BPU167 ={"H1SERVER","H1_BPU167",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU167_d };
NETKEY H1_BPU168 ={"H1SERVER","H1_BPU168",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU168_d };
NETKEY H1_BPU169 ={"H1SERVER","H1_BPU169",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU169_d };
NETKEY H1_BPU170 ={"H1SERVER","H1_BPU170",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU170_d };
NETKEY H1_BPU171 ={"H1SERVER","H1_BPU171",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU171_d };
NETKEY H1_BPU172 ={"H1SERVER","H1_BPU172",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU172_d };
NETKEY H1_BPU173 ={"H1SERVER","H1_BPU173",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU173_d };
NETKEY H1_BPU174 ={"H1SERVER","H1_BPU174",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU174_d };
NETKEY H1_BPU175 ={"H1SERVER","H1_BPU175",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU175_d };
NETKEY H1_BPU176 ={"H1SERVER","H1_BPU176",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU176_d };
NETKEY H1_BPU177 ={"H1SERVER","H1_BPU177",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU177_d };
NETKEY H1_BPU178 ={"H1SERVER","H1_BPU178",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU178_d };
NETKEY H1_BPU179 ={"H1SERVER","H1_BPU179",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU179_d };
NETKEY H1_BPU180 ={"H1SERVER","H1_BPU180",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU180_d };
NETKEY H1_BPU181 ={"H1SERVER","H1_BPU181",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU181_d };
NETKEY H1_BPU182 ={"H1SERVER","H1_BPU182",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU182_d };
NETKEY H1_BPU183 ={"H1SERVER","H1_BPU183",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU183_d };
NETKEY H1_BPU184 ={"H1SERVER","H1_BPU184",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU184_d };
NETKEY H1_BPU185 ={"H1SERVER","H1_BPU185",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU185_d };
NETKEY H1_BPU186 ={"H1SERVER","H1_BPU186",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU186_d };
NETKEY H1_BPU187 ={"H1SERVER","H1_BPU187",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU187_d };
NETKEY H1_BPU188 ={"H1SERVER","H1_BPU188",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU188_d };
NETKEY H1_BPU189 ={"H1SERVER","H1_BPU189",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU189_d };
NETKEY H1_BPU190 ={"H1SERVER","H1_BPU190",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU190_d };
NETKEY H1_BPU191 ={"H1SERVER","H1_BPU191",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU191_d };
NETKEY H1_BPU192 ={"H1SERVER","H1_BPU192",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU192_d };
NETKEY H1_BPU193 ={"H1SERVER","H1_BPU193",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU193_d };
NETKEY H1_BPU194 ={"H1SERVER","H1_BPU194",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU194_d };
NETKEY H1_BPU195 ={"H1SERVER","H1_BPU195",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU195_d };
NETKEY H1_BPU196 ={"H1SERVER","H1_BPU196",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU196_d };
NETKEY H1_BPU197 ={"H1SERVER","H1_BPU197",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU197_d };
NETKEY H1_BPU198 ={"H1SERVER","H1_BPU198",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU198_d };
NETKEY H1_BPU199 ={"H1SERVER","H1_BPU199",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU199_d };
NETKEY H1_BPU200 ={"H1SERVER","H1_BPU200",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU200_d };
NETKEY H1_BPU201 ={"H1SERVER","H1_BPU201",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU201_d };
NETKEY H1_BPU202 ={"H1SERVER","H1_BPU202",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU202_d };
NETKEY H1_BPU203 ={"H1SERVER","H1_BPU203",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU203_d };
NETKEY H1_BPU204 ={"H1SERVER","H1_BPU204",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU204_d };
NETKEY H1_BPU205 ={"H1SERVER","H1_BPU205",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU205_d };
NETKEY H1_BPU206 ={"H1SERVER","H1_BPU206",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU206_d };
NETKEY H1_BPU207 ={"H1SERVER","H1_BPU207",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU207_d };
NETKEY H1_BPU208 ={"H1SERVER","H1_BPU208",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU208_d };
NETKEY H1_BPU209 ={"H1SERVER","H1_BPU209",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU209_d };
NETKEY H1_BPU210 ={"H1SERVER","H1_BPU210",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU210_d };
NETKEY H1_BPU211 ={"H1SERVER","H1_BPU211",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU211_d };
NETKEY H1_BPU212 ={"H1SERVER","H1_BPU212",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU212_d };
NETKEY H1_BPU213 ={"H1SERVER","H1_BPU213",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU213_d };
NETKEY H1_BPU214 ={"H1SERVER","H1_BPU214",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU214_d };
NETKEY H1_BPU215 ={"H1SERVER","H1_BPU215",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU215_d };
NETKEY H1_BPU216 ={"H1SERVER","H1_BPU216",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU216_d };
NETKEY H1_BPU217 ={"H1SERVER","H1_BPU217",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU217_d };
NETKEY H1_BPU218 ={"H1SERVER","H1_BPU218",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU218_d };
NETKEY H1_BPU219 ={"H1SERVER","H1_BPU219",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU219_d };
NETKEY H1_BPU220 ={"H1SERVER","H1_BPU220",CF_DOUBLE,H1_BpuValNo,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BPU220_d };



NETKEY *H1_BeamPup[H1_BunchNO] = {
&H1_BPU001,
&H1_BPU002,
&H1_BPU003,
&H1_BPU004,
&H1_BPU005,
&H1_BPU006,
&H1_BPU007,
&H1_BPU008,
&H1_BPU009,
&H1_BPU010,
&H1_BPU011,
&H1_BPU012,
&H1_BPU013,
&H1_BPU014,
&H1_BPU015,
&H1_BPU016,
&H1_BPU017,
&H1_BPU018,
&H1_BPU019,
&H1_BPU020,
&H1_BPU021,
&H1_BPU022,
&H1_BPU023,
&H1_BPU024,
&H1_BPU025,
&H1_BPU026,
&H1_BPU027,
&H1_BPU028,
&H1_BPU029,
&H1_BPU030,
&H1_BPU031,
&H1_BPU032,
&H1_BPU033,
&H1_BPU034,
&H1_BPU035,
&H1_BPU036,
&H1_BPU037,
&H1_BPU038,
&H1_BPU039,
&H1_BPU040,
&H1_BPU041,
&H1_BPU042,
&H1_BPU043,
&H1_BPU044,
&H1_BPU045,
&H1_BPU046,
&H1_BPU047,
&H1_BPU048,
&H1_BPU049,
&H1_BPU050,
&H1_BPU051,
&H1_BPU052,
&H1_BPU053,
&H1_BPU054,
&H1_BPU055,
&H1_BPU056,
&H1_BPU057,
&H1_BPU058,
&H1_BPU059,
&H1_BPU060,
&H1_BPU061,
&H1_BPU062,
&H1_BPU063,
&H1_BPU064,
&H1_BPU065,
&H1_BPU066,
&H1_BPU067,
&H1_BPU068,
&H1_BPU069,
&H1_BPU070,
&H1_BPU071,
&H1_BPU072,
&H1_BPU073,
&H1_BPU074,
&H1_BPU075,
&H1_BPU076,
&H1_BPU077,
&H1_BPU078,
&H1_BPU079,
&H1_BPU080,
&H1_BPU081,
&H1_BPU082,
&H1_BPU083,
&H1_BPU084,
&H1_BPU085,
&H1_BPU086,
&H1_BPU087,
&H1_BPU088,
&H1_BPU089,
&H1_BPU090,
&H1_BPU091,
&H1_BPU092,
&H1_BPU093,
&H1_BPU094,
&H1_BPU095,
&H1_BPU096,
&H1_BPU097,
&H1_BPU098,
&H1_BPU099,
&H1_BPU100,
&H1_BPU101,
&H1_BPU102,
&H1_BPU103,
&H1_BPU104,
&H1_BPU105,
&H1_BPU106,
&H1_BPU107,
&H1_BPU108,
&H1_BPU109,
&H1_BPU110,
&H1_BPU111,
&H1_BPU112,
&H1_BPU113,
&H1_BPU114,
&H1_BPU115,
&H1_BPU116,
&H1_BPU117,
&H1_BPU118,
&H1_BPU119,
&H1_BPU120,
&H1_BPU121,
&H1_BPU122,
&H1_BPU123,
&H1_BPU124,
&H1_BPU125,
&H1_BPU126,
&H1_BPU127,
&H1_BPU128,
&H1_BPU129,
&H1_BPU130,
&H1_BPU131,
&H1_BPU132,
&H1_BPU133,
&H1_BPU134,
&H1_BPU135,
&H1_BPU136,
&H1_BPU137,
&H1_BPU138,
&H1_BPU139,
&H1_BPU140,
&H1_BPU141,
&H1_BPU142,
&H1_BPU143,
&H1_BPU144,
&H1_BPU145,
&H1_BPU146,
&H1_BPU147,
&H1_BPU148,
&H1_BPU149,
&H1_BPU150,
&H1_BPU151,
&H1_BPU152,
&H1_BPU153,
&H1_BPU154,
&H1_BPU155,
&H1_BPU156,
&H1_BPU157,
&H1_BPU158,
&H1_BPU159,
&H1_BPU160,
&H1_BPU161,
&H1_BPU162,
&H1_BPU163,
&H1_BPU164,
&H1_BPU165,
&H1_BPU166,
&H1_BPU167,
&H1_BPU168,
&H1_BPU169,
&H1_BPU170,
&H1_BPU171,
&H1_BPU172,
&H1_BPU173,
&H1_BPU174,
&H1_BPU175,
&H1_BPU176,
&H1_BPU177,
&H1_BPU178,
&H1_BPU179,
&H1_BPU180,
&H1_BPU181,
&H1_BPU182,
&H1_BPU183,
&H1_BPU184,
&H1_BPU185,
&H1_BPU186,
&H1_BPU187,
&H1_BPU188,
&H1_BPU189,
&H1_BPU190,
&H1_BPU191,
&H1_BPU192,
&H1_BPU193,
&H1_BPU194,
&H1_BPU195,
&H1_BPU196,
&H1_BPU197,
&H1_BPU198,
&H1_BPU199,
&H1_BPU200,
&H1_BPU201,
&H1_BPU202,
&H1_BPU203,
&H1_BPU204,
&H1_BPU205,
&H1_BPU206,
&H1_BPU207,
&H1_BPU208,
&H1_BPU209,
&H1_BPU210,
&H1_BPU211,
&H1_BPU212,
&H1_BPU213,
&H1_BPU214,
&H1_BPU215,
&H1_BPU216,
&H1_BPU217,
&H1_BPU218,
&H1_BPU219,
&H1_BPU220
};

NETKEY *netkeyp[61]= {
    &H1_RunNr,
    &H1Status,
    &H1_RSDat,
    &H1_RSTim,
    &H1_Ev_Nr,
    &H1_ESize,
    &H1_LRate,
    &H1_XIMSK,
    &H1_FBMSK,
    &H1_SYSMD,
    &H1_DAQMD,
    &H1_DTime,
    &H1_I_Com,
    &H1_I_Sol,
    &H1_I_Tor,
    &H1_Tr_Ph,
    &H1_Tr_W1,
    &H1_SubT0,
    &H1_SubT1,
    &H1_SubT2,
    &H1_SubT3,
    &H1_BBL3,
    &H1_Sta1,
    &H1_Sta2,
    &H1_CDAQ1,
    &H1_CDAQ2,
    &H1_CDAQ3,
    &H1_CDAQ4,
    &H1_LogTo,
    &H1_Lumi,
    &H1_IntuL,
    &H1_IntgL,
    &H1_FPS90y,
    &H1_FPS81y,
    &H1_FPS80x,
    &H1_FPS63x,
    &H1_FPSblm92,
    &H1_FPStcn90,
    &H1_FPStin90,
    &H1_NCounter,
    &H1_FPSblm84,
    &H1_FPStcn81,
    &H1_FPStin81,
    &H1_FPStrig,
    &H1_FPSblm80,
    &H1_FPStcn80,
    &H1_FPStin80,
    &H1_FPSnused,
    &H1_FPSblm63,
    &H1_FPStcn63,
    &H1_FPStin63,
    &H1_CritFlag,
    &H1_DaqFlag,
    &H1_SColC4yd,
    &H1_SColC4yu,
    &H1_SColC4i,
    &H1_SColC4o,
    &H1_SColC6i,
    &H1_SColC6o
};

