/****************************************************************************
*
*   name of file:  h1server.h
*
*   purpose:       header file for NextMEX client code including 
*                  NETKEYs for H1 data and information about run 
*                  status of the H1 detector, status, rates and
*                  position of the Roman pots ( forward proton
*                  spectrometer = FPS )
*                     
*                  The server runs on dice1, 131.169.10.110, and has
*                  FEC name MEX-H1. If one uses the HERA name server
*                  via herans.csv you will get automatically the 
*                  correct NETKEYs from H1SERVER.
*
*   created:       Jul, 1996
*
*   author:        A. Usik/T. Wilksen
*
*   changes:       May 10, 1997 by T. Wilksen
*
*                  
* included new horizontal stations of FPS @ 63m and 
*                    80m
*
*                  November 24th, 1999 T. Wilksen
*                    
*                  
* removed luminosity related keys, use instead
*                    the h1lumi NextMEX server.
*                
*                  
* removed the H1 Lopez monitor related keys
*
*   mail:          wilksen@mail.desy.de
*
*   remarks:       
*                  Timestamp conventions:
*
*                  data belonging to the H1 run status ( all except FPS
*                  data ):
*
*                  the data is recorded by the H1 online logging task,
*                  running on dice1, which gets the data from the frontend
*                  devices, resp. filter farm and buffering machines. It is
*                  written every 20 secs to disk and collected by the H1 MEX
*                  server then. The timestamps of each Netkey item belongs
*                  to the time of writing which is within 1-2 secs the
*                  time of recording/measuring and fetching via bus systems
*                  and net. If the logging task stops, e.g. during no lumi,
*                  the timestamps of course may be outdated.
*
*                  The FPS data has got timestamps which represent the
*                  time of measuring within an accuracy of 500 ms.
*                  These timestamps should be up to date all the time,
*                  unless the front end PC or the net connection is down.
*
*                  Refresh times, resp. update times are currently 20 secs
*                  for the H1 detector status and 20 secs for the FPS data.
******************************************************************************/
/* version number of header file */
#define VER 103
/* run and logging status of H1 detector */
/* current H1 run number, no unit  */
long H1_RunNr_l[1];
/* character string with value:   RUNSTART
                                  RUNNING
                                  PAUSED
                                  RUNEND
                                  STOPPED
                                  UNDEFINED
                                  WARMSTART
*/
char H1Status_c[20];
/* character string giving starting date of run dd/mm/yy */
char H1_RSDat_c[20];
/* character string giving starting time of run hh:mm:ss */
char H1_RSTim_c[20];
/* variable giving the current event number in run */
long H1_Ev_Nr_l[1];
/* average event size, unit is kB */
float H1_ESize_f[1];
/* average logging rate, unit is Hz */
float H1_LRate_f[1];
/* the meaning of the following bit patterns could be asked for
   at Jan Olsson, olsson@mail.desy.de */
/* bit-pattern for detector components included in the read out */
long H1_XIMSK_l[1];
/* bit-pattern for active Full_Event_Buffer tasks */
long H1_FBMSK_l[1];
/* H1 detector system mode as a bit=pattern */
long H1_SYSMD_l[1];
/* H1 CDAQ mode, bit-pattern */
long H1_DAQMD_l[1];
/* current dead time of logging task, unit is % */
float H1_DTime_f[1];
/* current of H1 compensator magnet, unit is A  */
long H1_I_Com_l[1];
/* current of H1 solenoid, unit is A  */
long H1_I_Sol_l[1];
/* current of forward muon toroid, unit is A */
long H1_I_Tor_l[1];
/* central trigger set up, character string */
char H1_Tr_Ph_c[20];
/* trigger mode specification, bit-patterns */
long H1_Tr_W1_l[1];
long H1_Tr_W2_l[1];
/* active subtriggers 0 - 31, bit-pattern */
long H1_SubT0_l[1];
/* active subtriggers 31 - 63, bit-pattern */
long H1_SubT1_l[1];
/* active subtriggers 64 - 95, bit-pattern */
long H1_SubT2_l[1];
/* active subtriggers 96 - 127, bit-pattern */
long H1_SubT3_l[1];
/* status of BBL3 central slow control system, bit-pattern */
long H1_BBL3_l[1];
/* status of HV settings, subdetectors, bit-pattern */
long H1_Sta1_l[1];
long H1_Sta2_l[1];
/* names of shift leader on shift crew, character strings */
char H1_CDAQ1_c[20];
char H1_CDAQ2_c[20];
char H1_CDAQ3_c[20];
char H1_CDAQ4_c[20];
/* mode of data logging, usually SGI */
char H1_LogTo_c[20];
/* The following data contains some slow control information 
   about the H1forward proton spectrometer (FPS), located in 
   the forward region of theH1 detector. It consists of four 
   Roman pot stations at z=+90m, 81m, 80mand 63m of the proton 
   beam pipe. A Roman pot is a moveable vacuum insertwhich can 
   be moved close to the proton beam to detect off-momentum 
   protonsstemming from diffractive events in the H1 detector. 
   The pots are controlled by one PC. All slow control information 
   which isused by this PC is available also in a FPSINFO file of 
   the old MEX format,however only on dice machines. In addition 
   to that this data is also providedon the general H1 NextMEX server. 
   One can call this server with RPC like NextMEXcalls. Some 
   information can be found on the web 
   (http://www-h1.desy.de/h1/www/sc/netmex/index.html 
    and http://www-mpy.desy.de/netmex ).
   There is also information about the two synchrotron radiation 
   masks C4and C6 in the H1 detector area. They are steered via 
   SEDAQ-electronicswith the FPS-PC too. 
*/

/* distance between the bottom of the vacuum insert
   and the nominal axisof the proton beam (in the retracted 
   position it should beabout 60mm; the position close to the 
   beam is for the 90m station of about 16mm).This value is 
   the more accurate value coming from a position measurement 
   device.  The accuracy is better than 50 microns.  Unit is mm.
*/
float H1_FPS90y_f[1];
/* same as for the 90m station, but the position of the vertical 
   81m potclose to the beam corresponds to a distance of about 12mm. 
*/
float H1_FPS81y_f[1];
/* same as for the 90m station, but the position of the horizontal 
   80m potclose to the beam corresponds to a distance of about ? mm. 
*/
float H1_FPS80x_f[1];
/* same as for the 90m station, but the position of the horizontal 
   63m potclose to the beam corresponds to a distance of about  ? mm. 
*/
float H1_FPS63x_f[1];
/* counting rate of the beam loss monitormounted at the proton 
   beam pipe at 92m.Used as a monitor during the insertion of the 
   90m pot.Unit is Hz. 
*/
float H1_FPSblm92_f[1];
/* counting rate of a coincidence signal in four trigger tiles. 
   Choosen arethose trigger tiles which are most in the middle 
   of each plane and mostclose to the beam axis. This rate should 
   roughly indicate a clear trackthrough one station.  Unit is Hz. 
*/
float H1_FPStcn90_f[1];
/* trigger counting rate which is provided by an internal trigger 
   logicand is one of the contributions to the FPS triggerelement.
   This rate may change drastically due to changes in the FPS DAQ 
   triggersetting, so do not use this rate for safety monitoring 
   or other purposes !Unit is Hz. */
float H1_FPStin90_f[1];
/* trigger rate from the forward neutron calorimeter with low 
   threshold(the FNC is located at z = +107m); this rate is used 
   to monitor the movingof the 90m pot Unit is Hz 
*/
float H1_NCounter_f[1];
/* The following data items are similar, four items per Roman 
   pot station,the last one of each station is not really used, 
   unit is for all Hz 
*/
float H1_FPSblm84_f[1];
float H1_FPStcn81_f[1];
float H1_FPStin81_f[1];
float H1_FPStrig_f[1];
float H1_FPSblm80_f[1];
float H1_FPStcn80_f[1];
float H1_FPStin80_f[1];
float H1_FPStrig2_f[1];
float H1_FPSblm63_f[1];
float H1_FPStcn63_f[1];
float H1_FPStin63_f[1];
float H1_FPSsin63_f[1];
/* means, that the Roman pots have reached a region where the 
   pots are close tothe proton beam. This integer represents a 
   bit pattern: 0 means no pot is inthe critical zone, 1 means 
   only pot 81, 2 means only pot 90 and 3 both pots arein the 
   critical region. 
*/
long H1_CritFlag_l[1];
/* represents a status, where the pots are in their final position 
   close to thebeam. This in fact a bit pattern. 0 means no DAQ 
   flag is set, 1 means it is. 
*/
long H1_DaqFlag_l[1];
/* Next to the C3 collimator which is steered and controlled by 
   the HERA machine group the C4 and C6 collimators aresteered 
   by the Roman pot PC of H1. The C4 collimtor islocated under 
   the forward tracker and C6 behind the Spacal.position of the 
   C4 collimator yoke in negative y-direction (lower one) relative
   to the beampipe).  Unit is mm.  
*/
float H1_SColC4yd_f[1];
/* position of the C4 collimator yoke in the positive  y-direction 
   (upperone relative to the beampipe) 
*/
float H1_SColC4yu_f[1];
/* position of the inner (with respect to the x-axis) C4 
   collimator yoke. 
*/
float H1_SColC4i_f[1];
/* position of the outer (with respect to the x-axis) C4 
   collimator yoke. 
*/
float H1_SColC4o_f[1];
/* position of the inner (with respect to the x-axis) C6 
   collimator yoke. 
*/
float H1_SColC6i_f[1];
/* position of the outer (with respect to the x-axis) C6 
   collimator yoke. 
*/
float H1_SColC6o_f[1];
NETKEY H1_RunNr={ "H1SERVER","H1_RunNr",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RunNr_l };
NETKEY H1Status={ "H1SERVER","H1Status",CF_TEXT, 20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1Status_c };
NETKEY H1_RSDat={ "H1SERVER","H1_RSDat",CF_TEXT, 20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RSDat_c };
NETKEY H1_RSTim={ "H1SERVER","H1_RSTim",CF_TEXT, 20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_RSTim_c };
NETKEY H1_Ev_Nr={ "H1SERVER","H1_Ev_Nr",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Ev_Nr_l };
NETKEY H1_ESize={ "H1SERVER","H1_ESize",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_ESize_f };
NETKEY H1_LRate={ "H1SERVER","H1_LRate",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_LRate_f };
NETKEY H1_XIMSK={ "H1SERVER","H1_XIMSK",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_XIMSK_l };
NETKEY H1_FBMSK={ "H1SERVER","H1_FBMSK",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FBMSK_l };
NETKEY H1_SYSMD={ "H1SERVER","H1_SYSMD",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SYSMD_l };
NETKEY H1_DAQMD={ "H1SERVER","H1_DAQMD",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DAQMD_l };
NETKEY H1_DTime={ "H1SERVER","H1_DTime",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DTime_f };
NETKEY H1_I_Com={ "H1SERVER","H1_I_Com",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Com_l };
NETKEY H1_I_Sol={ "H1SERVER","H1_I_Sol",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Sol_l };
NETKEY H1_I_Tor={ "H1SERVER","H1_I_Tor",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_I_Tor_l };
NETKEY H1_Tr_Ph={ "H1SERVER","H1_Tr_Ph",CF_TEXT, 20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Tr_Ph_c };
NETKEY H1_Tr_W1={ "H1SERVER","H1_Tr_W1",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Tr_W1_l };
NETKEY H1_Tr_W2={ "H1SERVER","H1_Tr_W2",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Tr_W2_l };
NETKEY H1_SubT0={ "H1SERVER","H1_SubT0",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT0_l };
NETKEY H1_SubT1={ "H1SERVER","H1_SubT1",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT1_l };
NETKEY H1_SubT2={ "H1SERVER","H1_SubT2",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT2_l };
NETKEY H1_SubT3={ "H1SERVER","H1_SubT3",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SubT3_l };
NETKEY H1_BBL3={ "H1SERVER","H1_BBL3",CF_LONG,    1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_BBL3_l };
NETKEY H1_Sta1={ "H1SERVER","H1_Sta1",CF_LONG,    1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Sta1_l };
NETKEY H1_Sta2={ "H1SERVER","H1_Sta2",CF_LONG,    1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_Sta2_l };
NETKEY H1_CDAQ1={ "H1SERVER","H1_CDAQ1",CF_TEXT,  20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ1_c };
NETKEY H1_CDAQ2={ "H1SERVER","H1_CDAQ2",CF_TEXT,  20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ2_c };
NETKEY H1_CDAQ3={ "H1SERVER","H1_CDAQ3",CF_TEXT,  20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ3_c };
NETKEY H1_CDAQ4={ "H1SERVER","H1_CDAQ4",CF_TEXT,  20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CDAQ4_c };
NETKEY H1_LogTo={ "H1SERVER","H1_LogTo",CF_TEXT,  20,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_LogTo_c };
NETKEY H1_FPS90y={"H1SERVER","H1_FPS90y",CF_FLOAT,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS90y_f };
NETKEY H1_FPS81y={"H1SERVER","H1_FPS81y",CF_FLOAT,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS81y_f };
NETKEY H1_FPS80x={"H1SERVER","H1_FPS80x",CF_FLOAT,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS80x_f };
NETKEY H1_FPS63x={"H1SERVER","H1_FPS63x",CF_FLOAT,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPS63x_f };
NETKEY H1_FPSblm92={"H1SERVER","H1_FPSblm92",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm92_f };
NETKEY H1_FPStcn90={"H1SERVER","H1_FPStcn90",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn90_f };
NETKEY H1_FPStin90={"H1SERVER","H1_FPStin90",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin90_f };
NETKEY H1_NCounter={"H1SERVER","H1_NCounter",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_NCounter_f };
NETKEY H1_FPSblm84={"H1SERVER","H1_FPSblm84",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm84_f };
NETKEY H1_FPStcn81={"H1SERVER","H1_FPStcn81",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn81_f };
NETKEY H1_FPStin81={"H1SERVER","H1_FPStin81",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin81_f };
NETKEY H1_FPStrig={"H1SERVER","H1_FPStrig",CF_FLOAT,   1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStrig_f };
NETKEY H1_FPSblm80={"H1SERVER","H1_FPSblm80",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm80_f };
NETKEY H1_FPStcn80={"H1SERVER","H1_FPStcn80",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn80_f };
NETKEY H1_FPStin80={"H1SERVER","H1_FPStin80",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin80_f };
NETKEY H1_FPStrig2={"H1SERVER","H1_FPStrig2",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStrig2_f };
NETKEY H1_FPSblm63={"H1SERVER","H1_FPSblm63",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSblm63_f };
NETKEY H1_FPStcn63={"H1SERVER","H1_FPStcn63",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStcn63_f };
NETKEY H1_FPStin63={"H1SERVER","H1_FPStin63",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPStin63_f };
NETKEY H1_FPSsin63={"H1SERVER","H1_FPSsin63",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_FPSsin63_f };
NETKEY H1_CritFlag={"H1SERVER","H1_CritFlag",CF_LONG,  1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_CritFlag_l };
NETKEY H1_DaqFlag={"H1SERVER","H1_DaqFlag",CF_LONG,    1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_DaqFlag_l };
NETKEY H1_SColC4yd={"H1SERVER","H1_SColC4yd",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4yd_f };
NETKEY H1_SColC4yu={"H1SERVER","H1_SColC4yu",CF_FLOAT, 1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4yu_f };
NETKEY H1_SColC4i={"H1SERVER","H1_SColC4i",CF_FLOAT,   1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4i_f };
NETKEY H1_SColC4o={"H1SERVER","H1_SColC4o",CF_FLOAT,   1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC4o_f };
NETKEY H1_SColC6i={"H1SERVER","H1_SColC6i",CF_FLOAT,   1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC6i_f };
NETKEY H1_SColC6o={"H1SERVER","H1_SColC6o",CF_FLOAT,   1,0,RC_INI,T_INIT,ST_INI,VER,(void*)H1_SColC6o_f };
