#ifndef HERABVER
#define HERABVER 200

/* HERA-B Target Data   Oct 2001  K. Ehret  

   HB_TTime:  HB_Target.TimeStamp is the Unix-Timestamp when the data
              were send by the server
              -----------------------------------        
              0: Unix-Timestamp when the target data were recorded
              -----------------------------------        


  HB_TStatus:  HB_TStatus.TimeStamp is the Unix-Timestamp when the
              target data were recorded
              -----------------------------------        
              0: Selection byte of the 8 wires: 
                 bit 0...7:  0 - not selected
                             1 - selected
                  (Ia,Ib,Ii,Io/ IIa,IIb,IIi,IIo)
               1: Status of the rates - not used (-99)

              -----------------------------------        

   HB_Target: HB_Target.TimeStamp is set to the Unix-Timestamp when the 
              target data were recorded
              -----------------------------------        
              0-7: Target Positions (Ia,Ib,Ii,Io/ IIa,IIb,IIi,IIo) 
                   All Positions are given in mm. 
                   The coordinate system is like this: 
                       A wire is maximal out of the beam at approx
-25mm.
                       The nominal beam center is at 0mm.
                       If the beam is not centered in the beam pipe 
                       it may happen that different wires move to 
                       positive values, crossing the center of the beam
pipe.
              -----------------------------------        
              8-39:  32 different Rates or derived values 
                        Rates are given in MHz.
                   
                     
 -------->     8:    background rate
               9:    background rate
              10-17: wire rates (Ia,Ib,Ii,Io/ IIa,IIb,IIi,IIo) 
              18:    anticipated rate
 -------->    19:    interaction rate
              20:    relative rate fluctuations in %
              21-30: counter rates: F1, H1 ...H4, Lumi, L1...L4
              31-32: Hodoscope Counters: CB fraction, 
                               number of effective bunches
              33-34: Lumi Counters: CB fraction, 
                               number of effective bunches
              35-38: spare: 99, -99, 1, -1
              39:    bunch crossing frequency 

*/

#define NO_HB_TTime 1
long int HB_TTime_l[NO_HB_TTime];                
NETKEY HB_TTime={ "HB_DATA","HB_TTime",CF_LONG,NO_HB_TTime,0,RC_INI,T_INIT,ST_INI,HERABVER,(void*)HB_TTime_l};


#define NO_HB_Target 40
float HB_Target_f[NO_HB_Target];
NETKEY HB_Target={"HB_DATA","HB_Target",CF_FLOAT,NO_HB_Target,0,RC_INI,T_INIT,ST_INI,HERABVER,(void*)HB_Target_f};

#define NO_HB_TStatus 2
long int HB_TStatus_l[NO_HB_TStatus];                
NETKEY HB_TStatus={"HB_DATA","HB_TStatus",CF_LONG,NO_HB_TStatus,0,RC_INI,T_INIT,ST_INI,HERABVER,(void*)HB_TStatus_l};

#endif /* HERABVER */
