/*========================= DATA FROM PKTR NET ========================*/
#ifndef HPVER
#define HPVER 127

/*
  HPVER 127: 09.01.04
	     added extra monitor for HEbpmXHE and HEbpmYHE

  HPVER 126: 18.09.03
	     updated file to reflect new HERA E collimators and new HERA P GP Pumps

  HPVER 125: 11.03.02  9:22
             corrected typos in H1OrbitX, H1OrbitY, HBOrbitX, HBOrbitY keywords

  HPVER 124: 07.03.02 18:14
	     added new electron collimators to definition of HEColPZU and HEColPH1

  HPVER 123: 17.12.01 10:45
	     fixed an offset problem with the e and p BPMs (HERA netmex server counts
             from 1 and not 0!)

  HPVER 122: 12.11.01 12:39
             updated the comment fields for the Electron Collimators 

  HPVER 121: added HPVAC HPVACQ, HPvacAPR

  HPVER 120: changed HEcolPH1 (etc.) to HEColPH1 ('col' to 'Col')

  HPVER 119: adjusted HPbpm's and HEbpm's for new monitors.  Included HPOrbit.X and HPOrbit.Y for the entire ring.
             removed keywords such as HBintX and HBstatX which were always coming with the 'packed' calls to the
             BPM server.  If there is any interest in the monitor intensities or status, they can be re-introduced.
             Adjusted the HEColP's and HEmst's for the new number of jaws.

  HPVER 118: (1st update for the HERA 2001 Run start: new collimators, BPM monitors, etc.)
             HPColPos_f redefined: 18 Jaws instead of 16 (see order information below) 
             HPColRat_f redefined: 30 values -> 5 collimators X (inner, outer, top, bottom, max, mean)
                                   Note: there are 20 diodes and 18 collimators
             HPColCust, HPColTemp have been removed -> HPColCust was impossible to understand, and
                                                       HPColTemp never worked
             HERATEXT replaces HERATXT1 and HERATXT2

             HERFFREQ and HPRFFREQ replace HEHFREQ, HPHFREQ

  HPVER 117: add HPbpmNL7MX ... HPbpmSR7MX and
                 HPbpmNL7MY ... HPbpmSR7MY
  HPVER 116: add HETIME and HPTIME
  HPVER 115: expand keyword HEbpmXH1 and HEbpmYH1
  HPVER 114: add HEHFREQ, HPHFREQ, HEvacAPR, HERATXT1, HERATXT2, HERUNNUM
             HPRUNNUM

  HPVER 113: add HETYPE (-1 = electrons, +1 = positrons)

  HPVER 112: add HERFREQ, HEinjPha, HEVACNW, HEVACSO, HEVACNWQ,
             HEVACSOQ, HEBUNCH, HPBUNCH, HEBUNCHL, HPBUNCHL

  HPVER 111: add HEbpmXH1, HEbpmYH1, HEbpmXZU, HEbpmYZU, 
             HEbpmXHE, HEbpmYHE, HEbpmXHB, HEbpmYHB
             HEColPH1, HEColPZU, HEColPHE, HEColPHB
             HEDCCur, HEDCTau, HPDCCur, HPDCTau,
             HEBunCur, HPBunCur

             HESTATE, HPSTATE

             remove HECurMKI, HPCurMKI

  HPVER 110: remove TunTemp, TunHumid, TunTempR7 keywords (the source
             for these parameters has been terminated)

  HPVER 109: add HEMAGEN and HPMAGEN keywords
  
  HPVER 108: Updated the Hera-b blm list to reflect new BLM monitors
  for 1997.
  
  HPVER 107: fixed a mixup with the Collimator keywords 
  (WL19 / WR33 were incorrectly labled)

  HPVER 106: add BLM keyword
  HPblmHB  

  HPVER 105: add BPM keywords
  KOLbpmX,KOLbpmY,KOLstatX,KOLstatY,KOLintX,KOLintY
  Correct BPM descriptions to reflect current magnet positions
  Add timestamp information
  
  HPVER 104: add HF keyword
  HEHFWPW

  HPVER 103: add HF keywords:
  HEHFNRPW, HEHFNLPW, HEHFSRPW, HEHFSLPW, HEHFORPW, HEHFOLPW

*/

#ifndef MSDOS
#define INT_32 unsigned int
#else
#define INT_32 unsigned long
#endif
#define CF_INT_32 CF_LONG


/****** HERA Proton Machine data ** nominal update rate 1 Hz unless otherwise noted ****/

/*      Timestamps categories:

          T0 - Timestamp refers to the time the data were taken.

                e.g. HPEmit is calculated only when the BKR operators
                initiate a wire scan.  Its timestamp refers to the
                time of the scan, and it can never be assumed that 
                by merely reading this value one has the present emittance.

          T1 - Timestamp refers to the time the data were last 
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.

                e.g. A request from the HERAP dataserver to get the
                collimator positions retrieves the collimator positions
                and a timestamp from the collimator server indicating
                when the call was made, which should be within milliseconds
                of when the position data were actually obtained.

          T2 - Timestamp refers to the time the data are being 
               requested from the (data) server.  
                
                e.g. If the HERAP dataserver does not explicitly set
                the timestamp for a keyword entry, the timestamp will 
                automatically be set to the time of the call (i.e. now).
                There is NO such case for keywords on the HERAP dataserver.

*/ 
float HPMAGEN_f[1];                 /* proton momentum -GeV/c (set-value) */
float HEMAGEN_f[1];                 /* electron momentum - GeV/c (set-value) */

NETKEY HPMAGEN={ "HERADATA","HPMAGEN",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPMAGEN_f };
NETKEY HEMAGEN={ "HERADATA","HEMAGEN",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEMAGEN_f };

float HPENERGY_f[1];                /* Proton Energy - to nearest GeV (!) */
                                    /* Timestamp:  Category  T1 */
NETKEY HPENERGY={ "HERADATA","HPENERGY",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPENERGY_f };

float HPDipCur_f[1];                /* HP dipole current - to nearest ampere */
                                    /* Timestamp:  Category  T1 */
NETKEY HPDipCur={ "HERADATA","HPDipCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDipCur_f };

#define HPCOLPOS_NO 18              /*  WR94(out,in,top,bottom) WL105(o,i,t,b) WL150(o,i,t,b) WR33(o,i,t,b) WL19(o,i) */
#define HPCOLRAT_NO 30              /*  WR94(out,in,top,bottom,max,mean) WL105 WL150 WR33 WL19 */
                                    /* Timestamp:  Category  T1 */
float HPColPos_f[HPCOLPOS_NO];      /* Collimator Position mm */
float HPColRat_f[HPCOLRAT_NO];      /* Collimator Rates */

NETKEY HPColPos={ "HERADATA","HPColPos",CF_FLOAT,HPCOLPOS_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPColPos_f };
NETKEY HPColRat={ "HERADATA","HPColRat",CF_FLOAT,HPCOLRAT_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPColRat_f };

/******* HERA Proton BEAM  data ********/

#define HPORBIT_NO 141                    /* All HERA P BPMs */
                                          /* Timestamp:  Category  T0 */
float HPOrbitX_f[HPORBIT_NO];             /* x - mm */
float HPOrbitY_f[HPORBIT_NO];             /* y - mm */
NETKEY HPOrbitX={ "HERADATA","HPOrbitX",CF_FLOAT,HPORBIT_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPOrbitX_f };
NETKEY HPOrbitY={ "HERADATA","HPOrbitY",CF_FLOAT,HPORBIT_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPOrbitY_f };

#define HPZUBPM_NO 12            /* X */  /* SR144 MX SR 61 MX SR 41 MX SR 27 MX SR  2 MX    */
					  /* SL  2 MX SL  6 MX SL 27 MX SL 41 MX SL 61 MX SL148 SL201 MX SL253 MX */
				 /* Y */  /* SR114 MY SR 61 MY SR 41 MY SR 27 MY SR  2 MY  */
					  /* SL  2 MY SL  6 MY SL 27 MY SL 41 MY SL 61 MY SL118 MY SL176 MY SL229 MY */
                                          /* Timestamp:  Category  T0 */
float HPZUbpmX_f[HPZUBPM_NO];               /* x - mm */
float HPZUbpmY_f[HPZUBPM_NO];               /* y - mm */
NETKEY HPZUbpmX={ "HERADATA","HPOrbitX",CF_FLOAT,HPZUBPM_NO,27,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPZUbpmX_f };
NETKEY HPZUbpmY={ "HERADATA","HPOrbitY",CF_FLOAT,HPZUBPM_NO,27,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPZUbpmY_f };


#define H1BPM_NO 15              /* X */  /* NR144 MX NR 61 MX NR 41 MX NR 27 MX NR  2 MX */
					  /* NL  2 MX NL  6 MX NL 27 MX NL 41 MX NL 61 MX
					     NL148 MX NL201 MX NL253 MX NL300 MX NL347 MX */
				 /* Y */  /* NR114 MY NR 61 MY NR 41 MY NR 27 MY NR  2 MY */
					  /* NL  2 MY NL  6 MY NL 27 MY NL 41 MY NL 61 MY
					     NL118 MY NL176 MY NL229 MY NL276 MY NL323 MY */
                                          /* Timestamp:  Category  T0 */
float H1bpmX_f[H1BPM_NO];                 /* x - mm */
float H1bpmY_f[H1BPM_NO];                 /* y - mm */
NETKEY H1bpmX={ "HERADATA","HPOrbitX",CF_FLOAT,H1BPM_NO,96,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1bpmX_f };
NETKEY H1bpmY={ "HERADATA","HPOrbitY",CF_FLOAT,H1BPM_NO,96,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1bpmY_f };


#define HBBPM_NO 2                        /* BPM data  mm - HERA B   */
				/* X */	  /* WR126 MX WR 86 MX */
				/* Y */   /* WR126 MY WR 86 MY */
                                          /* Timestamp:  Category  T0 */
float HBbpmX_f[HBBPM_NO];                 /* x - mm : WR 126X 86X  */
float HBbpmY_f[HBBPM_NO];                 /* y - mm : WR 126Y 86Y  */
NETKEY HBbpmX={ "HERADATA","HPOrbitX",CF_FLOAT,HBBPM_NO,133,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBbpmX_f };
NETKEY HBbpmY={ "HERADATA","HPOrbitY",CF_FLOAT,HBBPM_NO,133,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBbpmY_f };

#define KOLBPM_NO 9                       /* BPM data  mm - HERA P Kollimator region */
				/* X */	  /* WR126 MX WR 86 MX WR 68 MX WR 36 MX WR 10 MX */
					  /* WL 24 MX WL 44 MX WL 96 MX WL128 MX */
				/* Y */   /* WR126 MY WR 86 MY WR 68 MY WR 36 MY WR 10 MY */
					  /* WL 24 MY WL 44 MY WL 96 MY WL128 MY */
                                          /* Timestamp:  Category  T0 */
float KOLbpmX_f[KOLBPM_NO];               /* x - mm */
float KOLbpmY_f[KOLBPM_NO];               /* y - mm */
float KOLintX_f[KOLBPM_NO];               /* int - prot * e8 very rough */
float KOLintY_f[KOLBPM_NO];               /* int - prot * e8 very rough */
INT_32 KOLstatX_i[KOLBPM_NO];             /* x - status */
INT_32 KOLstatY_i[KOLBPM_NO];             /* y - status */
NETKEY KOLbpmX={ "HERADATA","KOLbpmX",CF_FLOAT,KOLBPM_NO,133,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLbpmX_f };
NETKEY KOLbpmY={ "HERADATA","KOLbpmY",CF_FLOAT,KOLBPM_NO,133,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLbpmY_f };

                           /* information from 2 GHz Bunch Shape Measurement */
                           /* Timestamp:  Category  T1 */
short HPINJDL_s[1];        /* Delay in turns*64 after injection */
short HPBUCKT_s[1];        /* HIT RF Bucket # */
short HPflags_s[1];        /* 0=Normal 1=1rst Trace 2=Background */ 
INT_32 HPlastT_i[1];       /* Time of most recent Injection - sec */
float HPFWHM_f[1];         /* Hera P bunch Length - psec */
float HPphase_f[1];        /* Hera P Bunch Phase wrt HIT - psec */
float HParea_f[1];         /* Hear P bunch area  nA ?? */
NETKEY HPINJDL={ "HERADATA","HPINJDL",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPINJDL_s };
NETKEY HPBUCKT={ "HERADATA","HPBUCKT",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUCKT_s };
NETKEY HPflags={ "HERADATA","HPflags",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPINJDL_s };
NETKEY HPlastT={ "HERADATA","HPlastT",CF_INT_32,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPlastT_i };
NETKEY HPFWHM={ "HERADATA","HPFWHM",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPFWHM_f };
NETKEY HPphase={ "HERADATA","HPphase",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPphase_f };
NETKEY HParea={ "HERADATA","HParea",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HParea_f };

                           /* information from Hera P wire scanner  30 sec nominal update */
                           /* Timestamp:  Category  T0 */
float HPEmit_f[1];         /* HP emittance */
float HPNormEmit_f[1];     /* HP Norm emittance */
float HPlumi_f[1];         /* HP specific lum (est from wire scan */
NETKEY HPEmit={ "HERADATA","HPEmit",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPEmit_f };
NETKEY HPNormEmit={ "HERADATA","HPNormEmit",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPNormEmit_f };
NETKEY HPlumi={ "HERADATA","HPlumi",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPlumi_f };

                      /* information from Beam Current monitors */
                                    /* Timestamp:  Category  T0 */
float HPCur_f[1];             /* Legacy: Hera P DC current - mA */ 
float HPtau_f[1];                      /* Legacy: lifetime - hr */
float HECur_f[1];             /* Legacy: Hera E DC current - mA */
float HEtau_f[1];                   /* Legacy: HE lifetime - hr */
NETKEY HPCur={ "HERADATA","HPCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPCur_f };
NETKEY HPtau={ "HERADATA","HPtau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPtau_f };
NETKEY HECur={ "HERADATA","HECur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HECur_f };
NETKEY HEtau={ "HERADATA","HEtau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEtau_f };

float HPDCCur_f[1];                   /* Hera P DC current - mA */ 
float HPDCTau_f[1];                            /* lifetime - hr */
float HPBunCur_f[1];        /* Hera P integrated Bunch cur - mA */
float HEDCCur_f[1];                   /* Hera E DC current - mA */
float HEDCTau_f[1];                            /* lifetime - hr */
float HEBunCur_f[1];        /* Hera E integrated Bunch cur - mA */
#define HEBUNCH_NO 220
float HEBUNCH_f[HEBUNCH_NO];             /* Hera E Bunch Currents - uA */
float HPBUNCH_f[HEBUNCH_NO];             /* Hera P Bunch Currents - uA */
float HEBUNCHL_f[HEBUNCH_NO];     /* Legacy Hera E Bunch Currents - uA */
float HPBUNCHL_f[HEBUNCH_NO];     /* Legacy Hera P Bunch Currents - uA */
NETKEY HPDCCur={ "HERADATA","HPDCCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDCCur_f };
NETKEY HPDCTau={ "HERADATA","HPDCTau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDCTau_f };
NETKEY HPBunCur={ "HERADATA","HPBunCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBunCur_f };
NETKEY HEDCCur={ "HERADATA","HEDCCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEDCCur_f };
NETKEY HEDCTau={ "HERADATA","HEDCTau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEDCTau_f };
NETKEY HEBunCur={ "HERADATA","HEBunCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBunCur_f };
NETKEY HEBUNCH={ "HERADATA","HEBUNCH",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBUNCH_f };
NETKEY HPBUNCH={ "HERADATA","HPBUNCH",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUNCH_f };
NETKEY HEBUNCHL={ "HERADATA","HEBUNCHL",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBUNCHL_f };
NETKEY HPBUNCHL={ "HERADATA","HPBUNCHL",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUNCHL_f };

float HPQX_f[1];              /* horz betatron tune for Hera P */
float HPQY_f[1];              /* vert betatron tune for Hera P */
                              /* Timestamp:  Category  T1 */
NETKEY HPQX={ "HERADATA","HPQX",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPQX_f };
NETKEY HPQY={ "HERADATA","HPQY",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPQY_f };

float HEHFNRPW_f[1];          /* HERA E HF Power in KWatt for NR */
float HEHFNLPW_f[1];          /* HERA E HF Power in KWatt for NL */
float HEHFSRPW_f[1];          /* HERA E HF Power in KWatt for SR */
float HEHFSLPW_f[1];          /* HERA E HF Power in KWatt for SL */
float HEHFORPW_f[1];          /* HERA E HF Power in KWatt for OR */
float HEHFOLPW_f[1];          /* HERA E HF Power in KWatt for OL */
                              /* Timestamp:  Category  T1 */
NETKEY HEHFNRPW={ "HERADATA","HEHFNRPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFNRPW_f };
NETKEY HEHFNLPW={ "HERADATA","HEHFNLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFNLPW_f };
NETKEY HEHFSRPW={ "HERADATA","HEHFSRPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFSRPW_f };
NETKEY HEHFSLPW={ "HERADATA","HEHFSLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFSLPW_f };
NETKEY HEHFORPW={ "HERADATA","HEHFORPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFORPW_f };
NETKEY HEHFOLPW={ "HERADATA","HEHFOLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFOLPW_f };

float HEHFWPW_f[1];           /* HERA E HF Power in KWatt for West */
                              /* Timestamp:  Category  T1 */
NETKEY HEHFWPW={ "HERADATA","HEHFWPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFWPW_f };

#define HPBLMHB_NO 29         /* 29 BLMs in the HERA B IR (Protons/sec): */  
float HPblmHB_f[HPBLMHB_NO];  /* WR: 252 225 196 163 158 133 94K2 
                                 WR: 94K1 61 40 33K2 33K1 18
                                     19K1 19K2 23 43 58 79D 91 123   :WL 
                                     105K1 153K1 158 126CERN 162 166 :WL
                                     195 218                         :WL */
                              /* Timestamp:  Category  T1 */
NETKEY HPblmHB={ "HERADATA","HBblm",CF_FLOAT,HPBLMHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPblmHB_f };

/* Below: Electron BPM data mm  */
/* Timestamp:  Category  T1 */

#define HEbpm_NO 287
#define HEbpmH1_NO 27
#define HEbpmZU_NO 14
#define HEbpmHE_NO 11
#define HEbpmHB_NO 2
float HEbpmX_f[HEbpm_NO];          /* X pos for E ring mm (WL791 -> SR 796) - that's right: all of 'em */
float HEbpmY_f[HEbpm_NO];          /* Y pos for E ring mm (WL791 -> SR 796) - that's right: all of 'em */
float HEbpmXH1_f[HEbpmH1_NO];      /* X pos at int point H1 mm */
float HEbpmYH1_f[HEbpmH1_NO];      /* Y pos at int point H1 mm */
				   /* NL   251  NL   227 NL   204 NL   196  NL   126 NL   107  NL    94 
                                      NL    81  NL    54 NL    44 NL    27  NL    11 NL     6  NL     2 
				      NR     2  NR    15 NR    27 NR    44  NR    54 NR    81  NR    94 
				      NR   107  NR   126 NR   196 NR   205  NR   228 NR   252 */

float HEbpmXZU_f[HEbpmZU_NO];      /* X pos at int point ZEUS mm */
float HEbpmYZU_f[HEbpmZU_NO];      /* Y pos at int point ZEUS mm */
				   /* SL    81 SL    54 SL    44 SL    27 SL    11  SL     6 SL     2 
				      SR     2 SR    15 SR    27 SR    44 SR    54 SR    81 SR    94 */

float HEbpmXHE_f[HEbpmHE_NO];      /* X pos at int point HERMES mm */
float HEbpmYHE_f[HEbpmHE_NO];      /* Y pos at int point HERMES mm */
				   /* OL    54 OL    34 OL    10 OL     7 OL     3  OR     8 OR    34 	
				      OR    46 OR    51 OR    68  OR    81 */

float HEbpmXB_f[HEbpmHB_NO];       /* Hera-E X BPM at Hera B Interaction in mm, WR133 */
float HEbpmYB_f[HEbpmHB_NO];       /* Hera-E Y BPM at Hera B Interaction in mm, WR133 */

NETKEY HEbpmX={ "HERADATA","HEOrbitX",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmX_f };
NETKEY HEbpmY={ "HERADATA","HEOrbitY",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmY_f };
NETKEY HEbpmXH1={ "HERADATA","HEOrbitX",CF_FLOAT,HEbpmH1_NO,95,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXH1_f };
NETKEY HEbpmYH1={ "HERADATA","HEOrbitY",CF_FLOAT,HEbpmH1_NO,95,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYH1_f };
NETKEY HEbpmXZU={ "HERADATA","HEOrbitX",CF_FLOAT,HEbpmZU_NO,246,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXZU_f };
NETKEY HEbpmYZU={ "HERADATA","HEOrbitY",CF_FLOAT,HEbpmZU_NO,246,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYZU_f };
NETKEY HEbpmXHE={ "HERADATA","HEOrbitX",CF_FLOAT,HEbpmHE_NO,176,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXHE_f };
NETKEY HEbpmYHE={ "HERADATA","HEOrbitY",CF_FLOAT,HEbpmHE_NO,176,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYHE_f };
NETKEY HEbpmXB={ "HERADATA","HEOrbitX",CF_FLOAT,HEbpmHB_NO,45,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXB_f };
NETKEY HEbpmYB={ "HERADATA","HEOrbitY",CF_FLOAT,HEbpmHB_NO,45,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYB_f };


/* Below: Electron Collimator data mm  */
/* Timestamp:  Category  T1 */

#define HEColPHE_NO 4                                  /* Electron Collimators: ABS(position)>1000 No Value */
#define HEColPZU_NO 8
#define HEColPH1_NO 8
#define HEColPHB_NO 2                 
float HEColPHE_f[HEColPHE_NO];        /* E cols at HERMES mm (OST 01o 01u 02a 02i)   */
float HEColPZU_f[HEColPZU_NO];        /* E cols at ZEUS   mm (SL 52a 40i 19i 15a 6o 6u 5a) */
float HEColPH1_f[HEColPH1_NO];        /* E cols at H1     mm (NL 52a 40i 19i 15a 6o 6u 5a) */
float HEColPHB_f[HEColPHB_NO];        /* E cols at herab  mm (WL 11a 11i)            */

NETKEY HEColPHE={ "HERADATA","HEColPHE",CF_FLOAT,HEColPHE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEColPHE_f };
NETKEY HEColPZU={ "HERADATA","HEColPZU",CF_FLOAT,HEColPZU_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEColPZU_f };
NETKEY HEColPH1={ "HERADATA","HEColPH1",CF_FLOAT,HEColPH1_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEColPH1_f };
NETKEY HEColPHB={ "HERADATA","HEColPHB",CF_FLOAT,HEColPHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEColPHB_f };

#define Motor_NO 2
float HEmstZU_f[Motor_NO];        /* vacuum chamber motor Steering at Zeus (SR 36 39) */
float HEmstH1_f[Motor_NO];        /* vacuum chamber motor Steering at H1   (NR 36 39) */
NETKEY HEmstZU={ "HERADATA","HEmstZU",CF_FLOAT,Motor_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEmstZU_f };
NETKEY HEmstH1={ "HERADATA","HEmstH1",CF_FLOAT,Motor_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEmstH1_f };

/* Below: HERA P and E States */
/* Timestamp:  Category  T1 */

#define STATE_NO 1
char HESTATE_str[32];
char HPSTATE_str[32];
NETKEY HESTATE={ "HERADATA","HESTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HESTATE_str };
NETKEY HPSTATE={ "HERADATA","HPSTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPSTATE_str };

#define HETYPE_NO 1
INT_32 HETYPE_i[HETYPE_NO];
NETKEY HETYPE={ "HERADATA","HETYPE",CF_LONG,HETYPE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HETYPE_i };

/* Below: Electron Vacuum data mbar  */
/* Timestamp:  Category  T1 */

#define HEVAC_NO 280
float HEVACNW_f[HEVAC_NO];    /* HERA E Vacuum Pressures for North and West Quadrants */
float HEVACSO_f[HEVAC_NO];    /* HERA E Vacuum Pressures for South and East Quadrants */
                              /* Timestamp:  Category  T1 */
NETKEY HEVACNW={ "HERADATA","HEVACN",CF_FLOAT,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACNW_f };
NETKEY HEVACSO={ "HERADATA","HEVACS",CF_FLOAT,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACSO_f };

NAME16 HEVACNW_str[HEVAC_NO];    /* HERA E Vacuum Locations for North and West Quadrants */
NAME16 HEVACSO_str[HEVAC_NO];    /* HERA E Vacuum Locations for South and East Quadrants */

NETKEY HEVACNWQ={ "HERADATA","HEVACN.NAM",CF_NAME16,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACNW_str };
NETKEY HEVACSOQ={ "HERADATA","HEVACS.NAM",CF_NAME16,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACSO_str };

float HEvacAPR_f[1];        /* mki file value for Hera-E avg. pressure, mBar */
NETKEY HEvacAPR={ "HERADATA","HEvacAPR",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEvacAPR_f };

/* Below: Electron & Proton Injection Data  */
/* Timestamp:  Category  T1 */

double HERFFREQ_d[1];                   /* HERA E RF Frequency MHz */
NETKEY HERFFREQ={ "HERADATA","HERFFREQ",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HERFFREQ_d };

float HEinjPha_f[1];                   /* HERA E Injection Phase */
NETKEY HEinjPha={ "HERADATA","HEinjPha",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEinjPha_f };

double HPRFFREQ_d[1];                   /* HERA P RF Frequency MHz */
NETKEY HPRFFREQ={ "HERADATA","HPRFFREQ",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPRFFREQ_d };

float HPinjPha_f[1];                   /* HERA E Injection Phase */
NETKEY HPinjPha={ "HERADATA","HPinjPha",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPinjPha_f };

/* Below: Electron Injection Data (Legacy) */
/* Timestamp:  Category  T1 */

/* Below: Text and Run information */
/* Timestamp:  Category  T1 */

#define HERATEXT_NO 64
char  HERATEXT_c[HERATEXT_NO];       /* Hera Text status */
NETKEY HERATEXT={ "HERADATA","HERATEXT",CF_TEXT,HERATEXT_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HERATEXT_c };

#define HERARUN_NO 1
short HERUNNUM_s[HERARUN_NO];
NETKEY HERUNNUM={ "HERADATA","HERUNNUM",CF_SHORT,HERARUN_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HERUNNUM_s };
short HPRUNNUM_s[HERARUN_NO];
NETKEY HPRUNNUM={ "HERADATA","HPRUNNUM",CF_SHORT,HERARUN_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPRUNNUM_s };

#define HETIME_NO 1
float HETIME_f[HETIME_NO];                   /* HERA E Bunch-Marker Timing */
NETKEY HETIME={ "HERADATA","HETIME",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HETIME_f };

#define HPTIME_NO 1
float HPTIME_f[HPTIME_NO];                   /* HERA P Bunch-Marker Timing */
NETKEY HPTIME={ "HERADATA","HPTIME",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPTIME_f };

#define HPVAC_NO 333
float HPVAC_f[HPVAC_NO];    /* HERA P Vacuum Pressures for whole ring */
float HPVAC_f[HPVAC_NO];    /* HERA P Vacuum Pressures for whole ring */
                              /* Timestamp:  Category  T1 */
NETKEY HPVAC={ "HERADATA","HPVACPR",CF_FLOAT,HPVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPVAC_f };

NAME16 HPVAC_str[HPVAC_NO];    /* HERA P Vacuum Locations for whole ring */
NETKEY HPVACQ={ "HERADATA","HPVACPR.NAM",CF_NAME16,HPVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPVAC_str };

float HPvacAPR_f[1];        /* value for Hera-P avg. pressure, mBar */
NETKEY HPvacAPR={ "HERADATA","HPvacAPR",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPvacAPR_f };

#endif /* HPVER */
