/*========================= DATA FROM PKTR NET ========================*/
#ifndef HPVER
#define HPVER 113

/*
  HPVER 113: add HETYPE (-1 = electrons, +1 = positrons)

  HPVER 112: add HERFREQ, HEinjPha, HEVACNW, HEVACSO, HEVACNWQ,
             HEVACSOQ, HEBUNCH, HPBUNCH, HEBUNCHL, HPBUNCHL

  HPVER 111: add HEbpmXH1, HEbpmYH1, HEbpmXZU, HEbpmYZU, 
             HEbpmXHE, HEbpmYHE, HEbpmXHB, HEbpmYHB
             HEcolPH1, HEcolPZU, HEcolPHE, HEcolPHB
             HEDCCur, HEDCTau, HPDCCur, HPDCTau,
             HEBunCur, HPBunCur

             HESTATE, HPSTATE

             remove HECurMKI, HPCurMKI

  HPVER 110: remove TunTemp, TunHumid, TunTempR7 keywords (the source
             for these parameters has been terminated)

  HPVER 109: add HEMAGEN and HPMAGEN keywords
  
  HPVER 108: Updated the Hera-b blm list to reflect new BLM monitors
  for 1997.
  
  HPVER 107: fixed a mixup with the Collimator keywords 
  (WL19 / WR33 were incorrectly labled)

  HPVER 106: add BLM keyword
  HPblmHB  

  HPVER 105: add BPM keywords
  KOLbpmX,KOLbpmY,KOLstatX,KOLstatY,KOLintX,KOLintY
  Correct BPM descriptions to reflect current magnet positions
  Add timestamp information
  
  HPVER 104: add HF keyword
  HEHFWPW

  HPVER 103: add HF keywords:
  HEHFNRPW, HEHFNLPW, HEHFSRPW, HEHFSLPW, HEHFORPW, HEHFOLPW

*/

#ifndef MSDOS
#define INT_32 unsigned int
#else
#define INT_32 unsigned long
#endif
#define CF_INT_32 CF_LONG


/****** HERA Proton Machine data ** nominal update rate 1 Hz unless otherwise noted ****/

/*      Timestamps categories:

          T0 - Timestamp refers to the time the data were taken.

                e.g. HPEmit is calculated only when the BKR operators
                initiate a wire scan.  Its timestamp refers to the
                time of the scan, and it can never be assumed that 
                by merely reading this value one has the present emittance.

          T1 - Timestamp refers to the time the data were last 
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.

                e.g. A request from the HERAP dataserver to get the
                collimator positions retrieves the collimator positions
                and a timestamp from the collimator server indicating
                when the call was made, which should be within milliseconds
                of when the position data were actually obtained.

          T2 - Timestamp refers to the time the data are being 
               requested from the (data) server.  
                
                e.g. If the HERAP dataserver does not explicitly set
                the timestamp for a keyword entry, the timestamp will 
                automatically be set to the time of the call (i.e. now).
                There is NO such case for keywords on the HERAP dataserver.

*/ 
float HPMAGEN_f[1];                 /* proton momentum -GeV/c (set-value) */
float HEMAGEN_f[1];                 /* electron momentum - GeV/c (set-value) */

NETKEY HPMAGEN={ "HERADATA","HPMAGEN",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPMAGEN_f };
NETKEY HEMAGEN={ "HERADATA","HEMAGEN",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEMAGEN_f };

float HPENERGY_f[1];                /* Proton Energy - to nearest GeV (!) */
                                    /* Timestamp:  Category  T1 */
NETKEY HPENERGY={ "HERADATA","HPENERGY",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPENERGY_f };

float HPDipCur_f[1];                /* HP dipole current - to nearest ampere */
                                    /* Timestamp:  Category  T1 */
NETKEY HPDipCur={ "HERADATA","HPDipCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDipCur_f };

#define HPCOL_NO 16                 /*  WR94(out,in,top,bottom) WL105(o,i,t,b) WL150(o,i,t,b) WL19(o,i) WR33(t,b) */
                                    /* Timestamp:  Category  T1 */
float HPColPos_f[HPCOL_NO];         /* Collimator Position mm */
float HPColTmp_f[HPCOL_NO];         /* Collimator Temp C */
float HPColCus_f[HPCOL_NO];         /* #0,4,8,12 Max Rate, #1,5,9,13 Avg Rate -  since last Readout */
                                    /* other values are coded and not of general interest */
NETKEY HPColPos={ "HERADATA","HPColPos",CF_FLOAT,HPCOL_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPColPos_f };
NETKEY HPColTmp={ "HERADATA","HPColTmp",CF_FLOAT,HPCOL_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPColTmp_f };
NETKEY HPColCus={ "HERADATA","HPColCus",CF_FLOAT,HPCOL_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPColCus_f };

/******* HERA Proton BEAM  data ********/
                                          /* SR: 144X 061X 045X 045X 061X 148X 201X 253X :SL */
#define HPBPM_NO 8                        /* SR: 114Y 061Y 045Y 045Y 061Y 118Y 176Y 229Y :SL */
                                          /* Timestamp:  Category  T0 */
float HPbpmX_f[HPBPM_NO];                 /* x - mm */
float HPbpmY_f[HPBPM_NO];                 /* y - mm */
float HPintX_f[HPBPM_NO];                 /* int - prot * e8 very rough */
float HPintY_f[HPBPM_NO];                 /* int - prot * e8 very rough */
INT_32 HPstatX_i[HPBPM_NO];               /* x - status */
INT_32 HPstatY_i[HPBPM_NO];               /* y - status */
NETKEY HPbpmX={ "HERADATA","HPbpmX",CF_FLOAT,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPbpmX_f };
NETKEY HPbpmY={ "HERADATA","HPbpmY",CF_FLOAT,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPbpmY_f };
NETKEY HPintX={ "HERADATA","HPintX",CF_FLOAT,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPintX_f };
NETKEY HPintY={ "HERADATA","HPintY",CF_FLOAT,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPintY_f };
NETKEY HPstatX={ "HERADATA","HPstatX",CF_INT_32,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPstatX_i };
NETKEY HPstatY={ "HERADATA","HPstatY",CF_INT_32,HPBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPstatY_i };

                                          /* NR: 144X 061X 045X 045X 061X 148X 201X 253X 300X 347X :NL */
#define H1BPM_NO 10                       /* NR: 114Y 061Y 045Y 045Y 061Y 118Y 176Y 229Y 276Y 323Y :NL */
                                          /* Timestamp:  Category  T0 */
float H1bpmX_f[H1BPM_NO];                 /* x - mm */
float H1bpmY_f[H1BPM_NO];                 /* y - mm */
float H1intX_f[H1BPM_NO];                 /* int - prot * e8 very rough */
float H1intY_f[H1BPM_NO];                 /* int - prot * e8 very rough */
INT_32 H1statX_i[H1BPM_NO];               /* x - status */
INT_32 H1statY_i[H1BPM_NO];               /* y - status */
NETKEY H1bpmX={ "HERADATA","H1bpmX",CF_FLOAT,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1bpmX_f };
NETKEY H1bpmY={ "HERADATA","H1bpmY",CF_FLOAT,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1bpmY_f };
NETKEY H1intX={ "HERADATA","H1intX",CF_FLOAT,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1intX_f };
NETKEY H1intY={ "HERADATA","H1intY",CF_FLOAT,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1intY_f };
NETKEY H1statX={ "HERADATA","H1statX",CF_INT_32,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1statX_i };
NETKEY H1statY={ "HERADATA","H1statY",CF_INT_32,H1BPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)H1statY_i };

#define HBBPM_NO 2                        /* BPM data  mm - HERA B   */
                                          /* Timestamp:  Category  T0 */
float HBbpmX_f[HBBPM_NO];                 /* x - mm : WR 126X 86X  */
float HBbpmY_f[HBBPM_NO];                 /* y - mm : WR 126Y 86Y  */
float HBintX_f[HBBPM_NO];                 /* int - prot * e8 very rough */
float HBintY_f[HBBPM_NO];                 /* int - prot * e8 very rough */
INT_32 HBstatX_i[HBBPM_NO];               /* x - status */
INT_32 HBstatY_i[HBBPM_NO];               /* y - status */
NETKEY HBbpmX={ "HERADATA","HBbpmX",CF_FLOAT,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBbpmX_f };
NETKEY HBbpmY={ "HERADATA","HBbpmY",CF_FLOAT,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBbpmY_f };
NETKEY HBintX={ "HERADATA","HBintX",CF_FLOAT,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBintX_f };
NETKEY HBintY={ "HERADATA","HBintY",CF_FLOAT,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBintY_f };
NETKEY HBstatX={ "HERADATA","HBstatX",CF_INT_32,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBstatX_i };
NETKEY HBstatY={ "HERADATA","HBstatY",CF_INT_32,HBBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HBstatY_i };

                                          /* BPM data  mm - HERA P Kollimator region */
                                          /* WR: 126X 86X 68X 36X 10X 24X 44X 96X 128X :WL*/
#define KOLBPM_NO 9                       /* WR: 126Y 86Y 68Y 36Y 10Y 24Y 44Y 96Y 128Y :WL */
                                          /* Timestamp:  Category  T0 */
float KOLbpmX_f[KOLBPM_NO];               /* x - mm */
float KOLbpmY_f[KOLBPM_NO];               /* y - mm */
float KOLintX_f[KOLBPM_NO];               /* int - prot * e8 very rough */
float KOLintY_f[KOLBPM_NO];               /* int - prot * e8 very rough */
INT_32 KOLstatX_i[KOLBPM_NO];             /* x - status */
INT_32 KOLstatY_i[KOLBPM_NO];             /* y - status */
NETKEY KOLbpmX={ "HERADATA","KOLbpmX",CF_FLOAT,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLbpmX_f };
NETKEY KOLbpmY={ "HERADATA","KOLbpmY",CF_FLOAT,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLbpmY_f };
NETKEY KOLintX={ "HERADATA","KOLintX",CF_FLOAT,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLintX_f };
NETKEY KOLintY={ "HERADATA","KOLintY",CF_FLOAT,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLintY_f };
NETKEY KOLstatX={ "HERADATA","KOLstatX",CF_INT_32,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLstatX_i };
NETKEY KOLstatY={ "HERADATA","KOLstatY",CF_INT_32,KOLBPM_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)KOLstatY_i };
                           
                           /* information from 2 GHz Bunch Shape Measurement */
                           /* Timestamp:  Category  T1 */
short HPINJDL_s[1];        /* Delay in turns*64 after injection */
short HPBUCKT_s[1];        /* HIT RF Bucket # */
short HPflags_s[1];        /* 0=Normal 1=1rst Trace 2=Background */ 
INT_32 HPlastT_i[1];       /* Time of most recent Injection - sec */
float HPFWHM_f[1];         /* Hera P bunch Length - psec */
float HPphase_f[1];        /* Hera P Bunch Phase wrt HIT - psec */
float HParea_f[1];         /* Hear P bunch area  nA ?? */
NETKEY HPINJDL={ "HERADATA","HPINJDL",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPINJDL_s };
NETKEY HPBUCKT={ "HERADATA","HPBUCKT",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUCKT_s };
NETKEY HPflags={ "HERADATA","HPflags",CF_SHORT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPINJDL_s };
NETKEY HPlastT={ "HERADATA","HPlastT",CF_INT_32,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPlastT_i };
NETKEY HPFWHM={ "HERADATA","HPFWHM",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPFWHM_f };
NETKEY HPphase={ "HERADATA","HPphase",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPphase_f };
NETKEY HParea={ "HERADATA","HParea",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HParea_f };

                           /* information from Hera P wire scanner  30 sec nominal update */
                           /* Timestamp:  Category  T0 */
float HPEmit_f[1];         /* HP emittance */
float HPNormEmit_f[1];     /* HP Norm emittance */
float HPlumi_f[1];         /* HP specific lum (est from wire scan */
NETKEY HPEmit={ "HERADATA","HPEmit",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPEmit_f };
NETKEY HPNormEmit={ "HERADATA","HPNormEmit",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPNormEmit_f };
NETKEY HPlumi={ "HERADATA","HPlumi",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPlumi_f };

                      /* information from Beam Current monitors */
                                    /* Timestamp:  Category  T0 */
float HPCur_f[1];             /* Legacy: Hera P DC current - mA */ 
float HPtau_f[1];                      /* Legacy: lifetime - hr */
float HECur_f[1];             /* Legacy: Hera E DC current - mA */
float HEtau_f[1];                   /* Legacy: HE lifetime - hr */
NETKEY HPCur={ "HERADATA","HPCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPCur_f };
NETKEY HPtau={ "HERADATA","HPtau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPtau_f };
NETKEY HECur={ "HERADATA","HECur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HECur_f };
NETKEY HEtau={ "HERADATA","HEtau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEtau_f };

float HPDCCur_f[1];                   /* Hera P DC current - mA */ 
float HPDCTau_f[1];                            /* lifetime - hr */
float HPBunCur_f[1];        /* Hera P integrated Bunch cur - mA */
float HEDCCur_f[1];                   /* Hera E DC current - mA */
float HEDCTau_f[1];                            /* lifetime - hr */
float HEBunCur_f[1];        /* Hera E integrated Bunch cur - mA */
#define HEBUNCH_NO 220
float HEBUNCH_f[HEBUNCH_NO];             /* Hera E Bunch Currents - uA */
float HPBUNCH_f[HEBUNCH_NO];             /* Hera P Bunch Currents - uA */
float HEBUNCHL_f[HEBUNCH_NO];     /* Legacy Hera E Bunch Currents - uA */
float HPBUNCHL_f[HEBUNCH_NO];     /* Legacy Hera P Bunch Currents - uA */
NETKEY HPDCCur={ "HERADATA","HPDCCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDCCur_f };
NETKEY HPDCTau={ "HERADATA","HPDCTau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPDCTau_f };
NETKEY HPBunCur={ "HERADATA","HPBunCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBunCur_f };
NETKEY HEDCCur={ "HERADATA","HEDCCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEDCCur_f };
NETKEY HEDCTau={ "HERADATA","HEDCTau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEDCTau_f };
NETKEY HEBunCur={ "HERADATA","HEBunCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBunCur_f };
NETKEY HEBUNCH={ "HERADATA","HEBUNCH",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBUNCH_f };
NETKEY HPBUNCH={ "HERADATA","HPBUNCH",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUNCH_f };
NETKEY HEBUNCHL={ "HERADATA","HEBUNCHL",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEBUNCHL_f };
NETKEY HPBUNCHL={ "HERADATA","HPBUNCHL",CF_FLOAT,HEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPBUNCHL_f };

float HPQX_f[1];              /* horz betatron tune for Hera P */
float HPQY_f[1];              /* vert betatron tune for Hera P */
                              /* Timestamp:  Category  T1 */
NETKEY HPQX={ "HERADATA","HPQX",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPQX_f };
NETKEY HPQY={ "HERADATA","HPQY",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPQY_f };

float HEHFNRPW_f[1];          /* HERA E HF Power in KWatt for NR */
float HEHFNLPW_f[1];          /* HERA E HF Power in KWatt for NL */
float HEHFSRPW_f[1];          /* HERA E HF Power in KWatt for SR */
float HEHFSLPW_f[1];          /* HERA E HF Power in KWatt for SL */
float HEHFORPW_f[1];          /* HERA E HF Power in KWatt for OR */
float HEHFOLPW_f[1];          /* HERA E HF Power in KWatt for OL */
                              /* Timestamp:  Category  T1 */
NETKEY HEHFNRPW={ "HERADATA","HEHFNRPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFNRPW_f };
NETKEY HEHFNLPW={ "HERADATA","HEHFNLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFNLPW_f };
NETKEY HEHFSRPW={ "HERADATA","HEHFSRPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFSRPW_f };
NETKEY HEHFSLPW={ "HERADATA","HEHFSLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFSLPW_f };
NETKEY HEHFORPW={ "HERADATA","HEHFORPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFORPW_f };
NETKEY HEHFOLPW={ "HERADATA","HEHFOLPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFOLPW_f };

float HEHFWPW_f[1];           /* HERA E HF Power in KWatt for West */
                              /* Timestamp:  Category  T1 */
NETKEY HEHFWPW={ "HERADATA","HEHFWPW",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEHFWPW_f };

#define HPBLMHB_NO 29         /* 29 BLMs in the HERA B IR (Protons/sec): */  
float HPblmHB_f[HPBLMHB_NO];  /* WR: 252 225 196 163 158 133 94K2 
                                 WR: 94K1 61 40 33K2 33K1 18
                                     19K1 19K2 23 43 58 79D 91 123   :WL 
                                     105K1 153K1 158 126CERN 162 166 :WL
                                     195 218                         :WL */
                              /* Timestamp:  Category  T1 */
NETKEY HPblmHB={ "HERADATA","HBblm",CF_FLOAT,HPBLMHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPblmHB_f };

/* Below: Electron BPM data mm  */
/* Timestamp:  Category  T1 */

#define HEbpm_NO 6
#define HEbpmHE_NO 9
#define HEbpmHB_NO 2
float HEbpmXH1_f[HEbpm_NO];          /* X pos at int point H1 mm (NL 68 54 7 7 54 68 NR)*/
float HEbpmYH1_f[HEbpm_NO];          /* Y pos at int point H1 mm (NL 68 54 7 7 54 68 NR)*/
float HEbpmXZU_f[HEbpm_NO];          /* X pos at int point ZEUS mm (SL 68 54 7 7 54 68 SR)*/
float HEbpmYZU_f[HEbpm_NO];          /* Y pos at int point ZEUS mm (SL 68 54 7 7 54 68 SR)*/
float HEbpmXHE_f[HEbpmHE_NO];        /* X pos at int point HERMES mm (OL 54 34 10 8 3 10 34 46 52 OR)*/
float HEbpmYHE_f[HEbpmHE_NO];        /* Y pos at int point HERMES mm (OL 54 34 10 8 3 10 34 46 52 OR)*/
float HEbpmXB_f[HEbpmHB_NO];       /* Hera-E X BPM at Hera B Interaction in mm, WR133 */
float HEbpmYB_f[HEbpmHB_NO];       /* Hera-E Y BPM at Hera B Interaction in mm, WR133 */

NETKEY HEbpmXH1={ "HERADATA","HEbpmXH1",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXH1_f };
NETKEY HEbpmYH1={ "HERADATA","HEbpmYH1",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYH1_f };
NETKEY HEbpmXZU={ "HERADATA","HEbpmXZU",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXZU_f };
NETKEY HEbpmYZU={ "HERADATA","HEbpmYZU",CF_FLOAT,HEbpm_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYZU_f };
NETKEY HEbpmXHE={ "HERADATA","HEbpmXHE",CF_FLOAT,HEbpmHE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXHE_f };
NETKEY HEbpmYHE={ "HERADATA","HEbpmYHE",CF_FLOAT,HEbpmHE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYHE_f };
NETKEY HEbpmXB={ "HERADATA","HEbpmXB",CF_FLOAT,HEbpmHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmXB_f };
NETKEY HEbpmYB={ "HERADATA","HEbpmYB",CF_FLOAT,HEbpmHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEbpmYB_f };


/* Below: Electron Collimator data mm  */
/* Timestamp:  Category  T1 */

#define HEcolPHE_NO 4                                  /* Electron Collimators: ABS(position)>1000 No Value */
#define HEcolPZU_NO 4
#define HEcolPH1_NO 4
#define HEcolPHB_NO     2                     /* Electron Collimators: at herab in mm; WL11a, WL11i     */
float HEcolPHE_f[HEcolPHE_NO];        /* mki file: E cols at HERMES mm (HeC1Ko HeC1Ku HeC1Ka HeC1Ki) */
float HEcolPZU_f[HEcolPZU_NO];        /* mki file: E cols at ZEUS   mm (SL 12 8 5a 5i 2a 2i) */
float HEcolPH1_f[HEcolPH1_NO];        /* mki file: E cols at H1     mm (NL 12 8 5a 5i ) */
float HEcolPHB_f[HEcolPHB_NO];

NETKEY HEcolPHE={ "HERADATA","HEcolPHE",CF_FLOAT,HEcolPHE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEcolPHE_f };
NETKEY HEcolPZU={ "HERADATA","HEcolPZU",CF_FLOAT,HEcolPZU_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEcolPZU_f };
NETKEY HEcolPH1={ "HERADATA","HEcolPH1",CF_FLOAT,HEcolPH1_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEcolPH1_f };
NETKEY HEcolPHB={ "HERADATA","HEcolPHB",CF_FLOAT,HEcolPHB_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEcolPHB_f };

#define Motor_NO 2
float HEmstZU_f[Motor_NO];        /* vacuum chamber motor Steering at Zeus */
float HEmstH1_f[Motor_NO];        /* vacuum chamber motor Steering at H1 */
NETKEY HEmstZU={ "HERADATA","HEmstZU",CF_FLOAT,Motor_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEmstZU_f };
NETKEY HEmstH1={ "HERADATA","HEmstH1",CF_FLOAT,Motor_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEmstH1_f };

/* Below: HERA P and E States */
/* Timestamp:  Category  T1 */

#define STATE_NO 1
char HESTATE_str[32];
char HPSTATE_str[32];
NETKEY HESTATE={ "HERADATA","HESTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HESTATE_str };
NETKEY HPSTATE={ "HERADATA","HPSTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HPSTATE_str };

#define HETYPE_NO 1
INT_32 HETYPE_i[HETYPE_NO];
NETKEY HETYPE={ "HERADATA","HETYPE",CF_LONG,HETYPE_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HETYPE_i };


/* Below: Electron Vacuum data mbar  */
/* Timestamp:  Category  T1 */

#define HEVAC_NO 400
float HEVACNW_f[HEVAC_NO];    /* HERA E Vacuum Pressures for North and West Quadrants */
float HEVACSO_f[HEVAC_NO];    /* HERA E Vacuum Pressures for South and East Quadrants */
                              /* Timestamp:  Category  T1 */
NETKEY HEVACNW={ "HERADATA","HEVACN",CF_FLOAT,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACNW_f };
NETKEY HEVACSO={ "HERADATA","HEVACS",CF_FLOAT,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACSO_f };

NAME16 HEVACNW_str[HEVAC_NO];    /* HERA E Vacuum Locations for North and West Quadrants */
NAME16 HEVACSO_str[HEVAC_NO];    /* HERA E Vacuum Locations for South and East Quadrants */

NETKEY HEVACNWQ={ "HERADATA","HEVACNQ",CF_NAME16,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACNW_str };
NETKEY HEVACSOQ={ "HERADATA","HEVACSQ",CF_NAME16,HEVAC_NO,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEVACSO_str };

/* Below: Electron Injection Data  */
/* Timestamp:  Category  T1 */

double HERFREQ_d[1];                   /* HERA E Frequency */
NETKEY HERFREQ={ "HERADATA","HERFREQ",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HERFREQ_d };

float HEinjPha_f[1];                   /* HERA E Injection Phase */
NETKEY HEinjPha={ "HERADATA","HEinjPha",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,HPVER,(void*)HEinjPha_f };

#endif /* HPVER */
