/*$*********************************************************************\

Source File     : hermes.h

Description     : contains keyword definitions for hermexsever

Comments        : Keyword length has been cut down to 8 characters
                  to please DOS users ...
                                                
Date            : 14.06.1996

Updates         : ver102  02.09.1996  woller
                          - added target magnet current and density
		  ver103  08.08.1997  woller
                          - added p/DIS ratio
		  ver104  20.07.1999  T. Lindemann
                          - added bunchpol polarisation
      ver200  31.10.2001 M. Bouwhuis
                          - Removed all of the Transverse Polarimeter
                          - Added HXPVeto 
      var201  11.01.2004 M. Bouwhuis
                          - Added HXVacPrP, HXVacStat, HXVacStatP, HXVacGv

At this moment the netmex computer is lxher1.desy.de
  
As requested, a note on timestamps: Each of these keywords comes with
a timestamp indicating the last time it was updated by the frontend
device, usually a VME computer. Each of the HERMES online computers is
synchronized with the DESY time server at the beginning of each data
taking run, that is every 10 to 15 minutes while we have (positron)
beam. Deviations are supposed to be in the millisecand range. This is
ok for all 'instantaneous' measurements, like HXSpMCur, HXTgMCur,
HXVacPr, HXDaqSta, HXRunSta, HXRunNum.

However, this is kind of meaningless for all measurements of rates, as
they are averaged over a longer time interval:

HXLumRat, HXLumRio, HXChrgSc, HXPhotSc, HXPVCoin and HXPVeto are calculated 
based on scaler values of a ten second interval preceding the timestamp.

HXTgDens, HXTraPol, HXLonPol are measured by averaging over longer
(maybe minutes) and not even constant periods of time. The timestamp
will indicate the time of the last calculation.

\*************************************************************************/

/* version number of this header file*/
#define VER 201

/*-----------------------------------------------------------------------*\
  Hermes spectrometer magnet and compensation coil currents  
    HXSpMCur_f[0] : Spectrometer Magnet current in A
    HXSpMCur_f[1] : Correction coil current in A
\*-----------------------------------------------------------------------*/
float HXSpMCur_f[2];                
NETKEY HXSpMCur={ "HERMEX","HXSpMCur",CF_FLOAT,2,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXSpMCur_f };

/*-----------------------------------------------------------------------*\
  Hermes target magnet current
    HXTgMCur_f[0] : Target Magnet current in A
\*-----------------------------------------------------------------------*/
float HXTgMCur_f[1];
NETKEY HXTgMCur={ "HERMEX","HXTgMCur",CF_FLOAT,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXTgMCur_f };

/*-----------------------------------------------------------------------*\
  Hermes calculated target density
    HXTgDens_f[0] : target density in [nucleons/cm^2]
    HXTgDens_f[1] : statistical error of measurement 
    HXTgDens_f[2] : systematic error of measurement 
\*-----------------------------------------------------------------------*/
float HXTgDens_f[3];
NETKEY HXTgDens={ "HERMEX","HXTgDens",CF_FLOAT,3,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXTgDens_f };

/*-----------------------------------------------------------------------*\
  Longitudinal Polarization in percent with error 
\*-----------------------------------------------------------------------*/
float HXLonPol_f[2];                
NETKEY HXLonPol={ "HERMEX","HXLonPol",CF_FLOAT,2,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXLonPol_f };

/*-----------------------------------------------------------------------*\
  Bunch Polarization in percent with error 
\*-----------------------------------------------------------------------*/
float HXBunPol_f[440];                
NETKEY HXBunPol={ "HERMEX","HXBunPol",CF_FLOAT,440,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXBunPol_f };

/*-----------------------------------------------------------------------*\
  Vacuum Pressures at Hermes Target Pumpstands 1-5
\*-----------------------------------------------------------------------*/
#define NO_PUMPSTAND (5)
float HXVacPr_f[NO_PUMPSTAND];  /* Beamline Ion Gauge IG2*/              
float HXVacPrP_f[NO_PUMPSTAND]; /* Ion Gauge in PumpStand IG1 */               

long  HXVacStat_l[NO_PUMPSTAND]; /* Status IG2 */               
long  HXVacStatP_l[NO_PUMPSTAND]; /* Status IG1 */               
long  HXVacGv_l[NO_PUMPSTAND]; /* Gate Valve between PumpStand and BeamLine */             
/* For Status and GateValve: 0=off/closed, 1=on/open, 2=Unknown */

NETKEY HXVacPr   ={ "HERMEX","HXVacPr",   CF_FLOAT,NO_PUMPSTAND,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXVacPr_f };
NETKEY HXVacPrP  ={ "HERMEX","HXVacPrP",  CF_FLOAT,NO_PUMPSTAND,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXVacPrP_f };
NETKEY HXVacStat ={ "HERMEX","HXVacStat", CF_LONG, NO_PUMPSTAND,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXVacStat_l };
NETKEY HXVacStatP={ "HERMEX","HXVacStatP",CF_LONG, NO_PUMPSTAND,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXVacStatP_l };
NETKEY HXVacGv   ={ "HERMEX","HXVacGv",   CF_LONG, NO_PUMPSTAND,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXVacGv_l };

/*-----------------------------------------------------------------------*\
  Hermes electron luminosity 
\*-----------------------------------------------------------------------*/
float HXLumRat_f[1];                
NETKEY HXLumRat={ "HERMEX","HXLumRat",CF_FLOAT,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXLumRat_f };

/*-----------------------------------------------------------------------*\
  Lumi right/left ratio 
\*-----------------------------------------------------------------------*/
float HXLumRio_f[1];                
NETKEY HXLumRio={ "HERMEX","HXLumRio",CF_FLOAT,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXLumRio_f };

/*-----------------------------------------------------------------------*\
  Charged particle tuning coincidences  
\*-----------------------------------------------------------------------*/
long HXChrgSc_l[12];                
NETKEY HXChrgSc={ "HERMEX","HXChrgSc",CF_LONG,12,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXChrgSc_l };

/*-----------------------------------------------------------------------*\
  Photon tuning anticoincidences 
\*-----------------------------------------------------------------------*/
long HXPhotSc_l[12];                
NETKEY HXPhotSc={ "HERMEX","HXPhotSc",CF_LONG,12,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXPhotSc_l };

/*-----------------------------------------------------------------------*\
  Hermes main physics trigger rate
\*-----------------------------------------------------------------------*/
long HXTrgRat_l[1];                
NETKEY HXTrgRat={ "HERMEX","HXTrgRat",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXTrgRat_l };

/*-----------------------------------------------------------------------*\
  Proton veto trigger coincidence rate 
\*-----------------------------------------------------------------------*/
long HXPVCoin_l[1];                
NETKEY HXPVCoin={ "HERMEX","HXPVCoin",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXPVCoin_l };

/*-----------------------------------------------------------------------*\
  Proton veto trigger rate From new detectors that are always on
    HXPVeto_l[0]  : rate in Hz for detector above proton beam line
    HXPVeto_l[1]  : rate in Hz for detector below proton beam line
\*-----------------------------------------------------------------------*/
long HXPVeto_l[2];                
NETKEY HXPVeto={ "HERMEX","HXPVeto",CF_LONG,2,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXPVeto_l };

/*-----------------------------------------------------------------------*\
  Proton over DIS trigger rate (just another background number)
\*-----------------------------------------------------------------------*/
long HXPovDIS_l[1];                
NETKEY HXPovDIS={ "HERMEX","HXPovDIS",CF_FLOAT,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXPovDIS_l };

/*-----------------------------------------------------------------------*\
  Data acquisition status 
    Status 0 = Halted / Restarting
           1 = Running
          >1 = Unknown
\*-----------------------------------------------------------------------*/
long HXDaqSta_l[1];                
NETKEY HXDaqSta={ "HERMEX","HXDaqSta",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXDaqSta_l };

/*-----------------------------------------------------------------------*\
  Hermes run status   
    Status 0 or 1 = Idle
                2 = Running
                3 = Paused
\*-----------------------------------------------------------------------*/
long HXRunSta_l[1];                
NETKEY HXRunSta={ "HERMEX","HXRunSta",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXRunSta_l };

/*-----------------------------------------------------------------------*\
  Hermes run number  
\*-----------------------------------------------------------------------*/
long HXRunNum_l[1];                
NETKEY HXRunNum={ "HERMEX","HXRunNum",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,VER,(void*)HXRunNum_l };

