/*================================== irsurvey.h ====================================*/

#ifndef SURVVER
/*#define SURVVER 100 */ /*     Version 1.00: 11.09.01  Steve.Herb@desy.de (3049) */
/*#define SURVVER 120 */     /*  Version 1.20: 09.11.01  Steve.Herb@desy.de (3049) */
#define SURVVER 130   /*  Version 1.30: 13.06.02  Steve.Herb@desy.de (3049) (ADD ZEUS POSITION DATA) */

/*--------- I.-- DATA from ZMEA (Survey Group) PCs in Halle NORD, Halle SUED -------*/
/*  
* The ZMEA measurement systems in the North and South IR are ~identical and are
* now read out by PCs running Win98 with a Dos Program, Labview, TINE Server, and
* FTP Server.  They are at present not stable and at least some of the programs
* must be restarted locally in the halls every few days, with the result that 
* measurement data can be stale for hours or days.  Data produced are:  
* 1. "LEV-NORD","LEV-SUED" A system using 'water levels' joined by common pipes to 
* measure RELATIVE displacement at 6 positions.  These are, in array order:
* 
* [ Left Bridge, Detector (In Left,Out Left,In Right,Out Right), Right Bridge ]
*
* 2. "TEMP-NORD","TEMP-SUED" Temp at the positions of the water levels listed above   
*/
#define IRLEV_NO 6    
float NirLevel_f[IRLEV_NO];         /* Water Level Readouts: mm  */
float NirTemp_f[IRLEV_NO];          /* Water Level System Temp C */
NETKEY NirLevel={ "HERADATA","LEV-NORD",CF_FLOAT,IRLEV_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)NirLevel_f };
NETKEY NirTemp= { "HERADATA","TEMP-NORD",CF_FLOAT,IRLEV_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)NirTemp_f };

float SirLevel_f[IRLEV_NO];         /* Water Level Readouts: mm  */
float SirTemp_f[IRLEV_NO];          /* Water Level System Temp C */
NETKEY SirLevel={ "HERADATA","LEV-SUED",CF_FLOAT,IRLEV_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)SirLevel_f };
NETKEY SirTemp={ "HERADATA","TEMP-SUED",CF_FLOAT,IRLEV_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)SirTemp_f };
/*
* 3. "POS-BRKNORD","POS-BRKSUED" The position at each bridge water level is extended 
* to the bridge girder with an Invar Rod attached to an 'dial gauge' with electronic 
* readout (respectively Left and Right in the data array). Thus the position of the 
* bridge girder wrt the other water levels is the sum of water level + Gauge readout.
*/
#define GAUGE_NO 2
float NirGauge_f[GAUGE_NO];         /* Dial Gauge: Left,Right: mm */
NETKEY NirGauge={ "HERADATA","POS-BRKNORD",CF_FLOAT,GAUGE_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)NirGauge_f };

float SirGauge_f[GAUGE_NO];         /* Dial Gauge: Left,Right: mm */
NETKEY SirGauge={ "HERADATA","POS-BRKSUED",CF_FLOAT,GAUGE_NO,1,RC_INI,T_INIT,ST_INI,
                   SURVVER,(void*)SirGauge_f };
/*
*  4. "TILT-NORD"  Four 'Tilt meters' are installed at each IR.  They measure the 
*  LONGITUDINAL tilt (??) at four positions, in array order: 
*  [ Left Bridge, GO quad, GG quad, Right Bridge ]
*/
#define TILT_NO  4          
float NirTilt_f[TILT_NO];          /* Tilt Left Brk,GO, */
NETKEY NirTilt={ "HERADATA","TILT-NORD",CF_FLOAT,TILT_NO,1,RC_INI,T_INIT,ST_INI,
                  SURVVER,(void*)NirTilt_f };

float SirTilt_f[TILT_NO];     
NETKEY SirTilt={ "HERADATA","TILT-SUED",CF_FLOAT,TILT_NO,1,RC_INI,T_INIT,ST_INI,
                  SURVVER,(void*)SirTilt_f };
/*
*  5. "POS-GOGGNORD" Five positions sensors each measure the position of the GO 
* and GG quadrupoles wrt the inner end of the respective Bridge.  Array elements are: 
* [ GO X, GO Y, GO Z1, GO Z2, GO Z3, GG X, GG Y, GG Z1, GG Z2, GG Z3 ] where the
* three Z positions should permit deriving both vertical and horizontal longitudinal
* tilt of each quad wrt the bridge
*/
#define GOGG_NO 10
float NirGOGG_f[GOGG_NO];           /* Tilt Meters, mrad??, order */
NETKEY NirGOGG={ "HERADATA","POS-GOGGNORD",CF_FLOAT,GOGG_NO,1,RC_INI,T_INIT,ST_INI,
                  SURVVER,(void*)NirGOGG_f };

float SirGOGG_f[GOGG_NO];           /* Tilt Meters, mrad??, order */
NETKEY SirGOGG={ "HERADATA","POS-GOGGSUED",CF_FLOAT,GOGG_NO,1,RC_INI,T_INIT,ST_INI,
                  SURVVER,(void*)SirGOGG_f };

/*------II-- DATA from MST PCs in Building 30 (Sedac connection to Halls  ----*/
/*
*  Each Bridge can be moved via a stepping motor system, and the # of steps moved
* is converted to mm and made available.  It is expected that these positions will
* not be changed very often. The array order is, where 
* R=Right,L=Left,I = Vertical Ring Inside, A = Vertical Ring Outside, X = horizontal
* , and the number is the approximate distance from the IR:
*  [ L 11X, L 11I, L 11A, L 7I, L 7A, R 5I, R 5A, R 5X, R 10I, R 10A, R 10X] 
*/
#define MOTBRK_NO 11
float NirBrkMot_f[MOTBRK_NO];    
NETKEY NirBrkMot={ "HERADATA","MOT-BRKNORD",CF_FLOAT,MOTBRK_NO,1,RC_INI,T_INIT,ST_INI,
                    SURVVER,(void*)NirBrkMot_f };

float SirBrkMot_f[MOTBRK_NO];  
NETKEY SirBrkMot={ "HERADATA","MOT-BRKSUED",CF_FLOAT,MOTBRK_NO,1,RC_INI,T_INIT,ST_INI,
                    SURVVER,(void*)SirBrkMot_f };

/*-------------------------------- III -- Data from Experiments --------------------------------------------*/
/* The following data are for the transverse position at the supports near the INNER end of the GO and GG
*  magnets in the ZEUS detector, relative to the ZEUS detector, from linear transducers mounted there.  
*  SirZeusX:  X GO, X GG (mm) 
*  SirZeusY:  Y GO, Y GG (mm)
*  The values are as read out by M. Werner, ie not corrected to absolute spatial positions                  */                                                                                                           */
#define POSZEUS_NO 2
float SirZeusX_f[POSZEUS_NO];
float SirZeusY_f[POSZEUS_NO];
NETKEY SirZeusX ={ "HERADATA","POS-ZEUSX",CF_FLOAT,POSZEUS_NO,1,RC_INI,T_INIT,ST_INI,
                    SURVVER,(void*)SirZeusX_f };
NETKEY SirZeusY ={ "HERADATA","POS-ZEUSY",CF_FLOAT,POSZEUS_NO,1,RC_INI,T_INIT,ST_INI,
                    SURVVER,(void*)SirZeusY_f };
                    
/*   Positions from H1       - not yet available 11.09.01 */
/*-------------------------------- IV Data from Stretched Wire Systems -(90.11.01 S. Herb)----------------- */
/*  X,Y positions from Stretched wire systems in the N and S IRs                                            */
/*  There are a total of 8 wires planned: an Inside and Outside wire for N Left,N Right, S Left, S Right    */
/*  At present only the OUTSIDE wires are active, but the INSIDE are also included in the data block.       */
/*  The Monitor Positions are:                                                                              */
/*  LEFT  (m)     7, 9, 13, 17, 20, 22, 25, 30, 35, 38, 45, 50, 53, 59 (14 X, 14 Y values) x2 (In/Out)      */
/*  RIGHT (m)  5, 7, 9, 13, 17, 20, 22, 25, 30, 35, 38, 45, 50, 53, 59 (15 X, 15 Y values) x2 (In/Out)      */
/*                                                                                                          */
/*  Each array contains data (units of mm) from an Inside and and Outside wire, for example                 */
/*  NirStWireLX = { 7IX,7OX,9IX,9OX,13IX,13OX ..... 59IX,59OX }                                             */
/*  The values of the not yet installed Inside monitors are set to 32000., for better or worse              */
/*  
*/
#define STWIREL_NO  28
#define STWIRER_NO  30
float NirStWireLX_f[STWIREL_NO];
float NirStWireLY_f[STWIREL_NO];
float NirStWireRX_f[STWIRER_NO];
float NirStWireRY_f[STWIRER_NO];
float SirStWireLX_f[STWIREL_NO];
float SirStWireLY_f[STWIREL_NO];
float SirStWireRX_f[STWIRER_NO];
float SirStWireRY_f[STWIRER_NO];

NETKEY NirStWireLX = { "HERADATA","Str-Wire-NL.X",CF_FLOAT,STWIREL_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)NirStWireLX_f };
NETKEY NirStWireLY = { "HERADATA","Str-Wire-NL.Y",CF_FLOAT,STWIREL_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)NirStWireLY_f };
NETKEY NirStWireRX = { "HERADATA","Str-Wire-NR.X",CF_FLOAT,STWIRER_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)NirStWireRX_f };
NETKEY NirStWireRY = { "HERADATA","Str-Wire-NR.Y",CF_FLOAT,STWIRER_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)NirStWireRY_f };                        
NETKEY SirStWireLX = { "HERADATA","Str-Wire-SL.X",CF_FLOAT,STWIREL_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)NirStWireLX_f };
NETKEY SirStWireLY = { "HERADATA","Str-Wire-SL.Y",CF_FLOAT,STWIREL_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)SirStWireLY_f };
NETKEY SirStWireRX = { "HERADATA","Str-Wire-SR.X",CF_FLOAT,STWIRER_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)SirStWireRX_f };
NETKEY SirStWireRY = { "HERADATA","Str-Wire-SR.Y",CF_FLOAT,STWIRER_NO,1,RC_INI,T_INIT,ST_INI,
                        SURVVER,(void*)SirStWireRY_f };                        
/**/
#endif
