/*========================= DATA FROM PKTR NET ========================*/
#ifndef PEVER
#define PEVER 100

/*
  PEVER 100: Inaugural version with basic information

*/

#ifndef MSDOS
#define INT_32 unsigned int
#else
#define INT_32 unsigned long
#endif
#define CF_INT_32 CF_LONG


/****** Petra Machine data ** nominal update rate 1 Hz unless otherwise noted ****/

/*      Timestamps categories:

          T0 - Timestamp refers to the time the data were taken.

          T1 - Timestamp refers to the time the data were last 
               requested from the source server.  For most volatile
               parameters, this will still be a good reflection of the
               value at that time.

          T2 - Timestamp refers to the time the data are being 
               requested from the (data) server.  
*/ 

/* below: PETRA State Parameters */
/* Timestamp:  Category  T1 */

#define PETYPE_NO 10
char PETYPE_str[PETYPE_NO];
NETKEY PETYPE={ "PETRADATA","PETYPE",CF_TEXT,PETYPE_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PETYPE_str };
#define STATE_NO 1
char PEESTATE_str[32];
NETKEY PEESTATE={ "PETRADATA","PEESTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEESTATE_str };
char PEPSTATE_str[32];
NETKEY PEPSTATE={ "PETRADATA","PEPSTATE",CF_NAME32,STATE_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEPSTATE_str };
#define PEMODE_NO 32
char PEMODE_str[PEMODE_NO];
NETKEY PEMODE={ "PETRADATA","PEMODE",CF_TEXT,PEMODE_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEMODE_str };

/* below: beam parameters */
/* Timestamp:  Category  T1 */

float PEMAGEN_f[1];                 /* particle momentum - GeV/c (set-value) */

NETKEY PEMAGEN={ "PETRADATA","PEMAGEN",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEMAGEN_f };

                      /* information from Beam Current monitors */
                                    /* Timestamp:  Category  T0 */

float PEDCCur_f[1];                         /*  DC current - mA */ 
float PEDCTau_f[1];                            /* lifetime - hr */
float PEBunCur_f[1];           /* Integrated Bunch Current - mA */
#define PEBUNCH_NO 80
float PEBUNCH_f[PEBUNCH_NO];             /* Bunch Currents - uA */
NETKEY PEDCCur={ "PETRADATA","PEDCCur",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEDCCur_f };
NETKEY PEDCTau={ "PETRADATA","PEDCTau",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEDCTau_f };
NETKEY PEBunCur={ "PETRADATA","PESUCURR",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEBunCur_f };
NETKEY PEBUNCH={ "PETRADATA","PEBUNCH",CF_FLOAT,PEBUNCH_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEBUNCH_f };

float PEEXTUNE_f[1];              /* horz betatron tune for Petra E */
float PEEYTUNE_f[1];              /* vert betatron tune for Petra E */
NETKEY PEEXTUNE={ "PETRADATA","PEEXTUNE",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEEXTUNE_f };
NETKEY PEEYTUNE={ "PETRADATA","PEEYTUNE",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEEYTUNE_f };
float PEPXTUNE_f[1];              /* horz betatron tune for Petra P */
float PEPYTUNE_f[1];              /* vert betatron tune for Petra P */
NETKEY PEPXTUNE={ "PETRADATA","PEPXTUNE",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEPXTUNE_f };
NETKEY PEPYTUNE={ "PETRADATA","PEPYTUNE",CF_FLOAT,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEPYTUNE_f };

/* Below: Electron & Proton Injection Data  */
/* Timestamp:  Category  T1 */

double PEEFREQ_d[1];                   /* PETRA E RF Frequency MHz */
NETKEY PEEFREQ={ "PETRADATA","PEEFREQ",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEEFREQ_d };

double PEPFREQ_d[1];                   /* PETRA P RF Frequency MHz */
NETKEY PEPFREQ={ "PETRADATA","PEPFREQ",CF_DOUBLE,1,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEPFREQ_d };


/* Below: Electron Vacuum data mbar  */
/* Timestamp:  Category  T1 */

#define PEVACN_NO 150
float PEVACNW_f[PEVACN_NO];    /* PETRA Vacuum Pressures for North and West Quadrants */
#define PEVACS_NO 160
float PEVACSO_f[PEVACS_NO];    /* PETRA Vacuum Pressures for South and East Quadrants */
                              /* Timestamp:  Category  T1 */
NETKEY PEVACNW={ "PETRADATA","PENVacPressure",CF_FLOAT,PEVACN_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEVACNW_f };
NETKEY PEVACSO={ "PETRADATA","PESVacPressure",CF_FLOAT,PEVACS_NO,1,RC_INI,T_INIT,ST_INI,PEVER,(void*)PEVACSO_f };

#endif /* PEVER */
