/*$*********************************************************************\

Source File     : pvakdata.h

Description     : contains keyword definitions for pvak server

Authors         : P.Duval, HG Wu : but please direct questions to
                  O.Hensler

Remarks         : 

To connect to the PVAK NextMex server, you have to include the
following line into your 'fecaddr.csv' file:

SUN52A,2,0,0,131.169.148.32,UNIX,PVAK data,Bldg 55,Sun Sparc,O.Hensler

and this one to 'eqpdbase.csv':

143,PVAKDATA,PVAK experimental data,SUN52A,KEYWRD,1,,0

\*************************************************************************/

#ifndef PVAKVER
/* version number of this header file*/
#define PVAKVER 101

#ifdef MSDOS
#  define INT32 long
#else
#  define INT32 int
#endif

/*-----------------------------------------------------------------------*\
  Hera Proton Vacuum Sputter Pumps  (octant-wise until somebody wants
                                     something more specific)

  e.g. call HPGPWR to get the Pressures in WR.  This call returns 100
       values in mBar.
       To see the locations of the Pumps call HPGPWRPo.  This call
       returns 100 values containing the location in meters from the
       Hall.
       To see the status information call HPGPWRSt.

\*-----------------------------------------------------------------------*/
#define MAX_PUMPS_OCTANT 100
float HPGPWL_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPWL={
"PVAKDATA","HPGPWL",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWL_f 
};
INT32 HPGPWLPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPWLPo={
"PVAKDATA","HPGPWLPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWLPo_l 
};
short HPGPWLSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPWLSt={
"PVAKDATA","HPGPWLSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWLSt_s 
};

float HPGPWR_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPWR={
"PVAKDATA","HPGPWR",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWR_f 
};
INT32 HPGPWRPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPWRPo={
"PVAKDATA","HPGPWRPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWRPo_l 
};
short HPGPWRSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPWRSt={
"PVAKDATA","HPGPWRSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPWRSt_s 
};

float HPGPNL_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPNL={
"PVAKDATA","HPGPNL",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNL_f 
};
INT32 HPGPNLPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPNLPo={
"PVAKDATA","HPGPNLPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNLPo_l 
};
short HPGPNLSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPNLSt={
"PVAKDATA","HPGPNLSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNLSt_s 
};

float HPGPNR_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPNR={
"PVAKDATA","HPGPNR",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNR_f 
};
INT32 HPGPNRPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPNRPo={
"PVAKDATA","HPGPNRPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNRPo_l 
};
short HPGPNRSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPNRSt={
"PVAKDATA","HPGPNRSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPNRSt_s 
};

float HPGPOL_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPOL={
"PVAKDATA","HPGPOL",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPOL_f 
};
INT32 HPGPOLPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPOLPo={
"PVAKDATA","HPGPOLPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPOLPo_l 
};
short HPGPOLSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPOLSt={
"PVAKDATA","HPGPOLSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPOLSt_s 
};

float HPGPOR_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPOR={
"PVAKDATA","HPGPOR",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPOR_f 
};
INT32 HPGPORPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPORPo={
"PVAKDATA","HPGPORPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPORPo_l 
};
short HPGPORSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPORSt={
"PVAKDATA","HPGPORSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPORSt_s 
};

float HPGPSL_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPSL={
"PVAKDATA","HPGPSL",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSL_f 
};
INT32 HPGPSLPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPSLPo={
"PVAKDATA","HPGPSLPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSLPo_l 
};
short HPGPSLSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPSLSt={
"PVAKDATA","HPGPSLSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSLSt_s 
};

float HPGPSR_f[MAX_PUMPS_OCTANT];                
NETKEY HPGPSR={
"PVAKDATA","HPGPSR",CF_FLOAT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSR_f 
};
INT32 HPGPSRPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPGPSRPo={
"PVAKDATA","HPGPSRPo",CF_LONG,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSRPo_l 
};
short HPGPSRSt_s[MAX_PUMPS_OCTANT];                
NETKEY HPGPSRSt={
"PVAKDATA","HPGPSRSt",CF_SHORT,MAX_PUMPS_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPGPSRSt_s 
};

/*-----------------------------------------------------------------------*\
  Hera Proton Vacuum Beam Valves  (octant-wise until somebody wants
                                   something more specific)

  e.g. call HPBVWR to get the beam valves in WR.  This call returns 100
       values.  
       To see the locations of the Pumps call HPBVWRPo.  This call
       returns 100 values containing the location in meters from the
       Hall.
       To see the status information call HPBVWRSt.

\*-----------------------------------------------------------------------*/

#define MAX_VALVES_OCTANT 100
float HPBVWL_f[MAX_VALVES_OCTANT];                
NETKEY HPBVWL={
"PVAKDATA","HPBVWL",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWL_f 
};
INT32 HPBVWLPo_l[MAX_PUMPS_OCTANT];                
NETKEY HPBVWLPo={
"PVAKDATA","HPBVWLPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWLPo_l 
};
short HPBVWLSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVWLSt={
"PVAKDATA","HPBVWLSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWLSt_s 
};

float HPBVWR_f[MAX_VALVES_OCTANT];                
NETKEY HPBVWR={
"PVAKDATA","HPBVWR",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWR_f 
};
INT32 HPBVWRPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVWRPo={
"PVAKDATA","HPBVWRPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWRPo_l 
};
short HPBVWRSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVWRSt={
"PVAKDATA","HPBVWRSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVWRSt_s 
};

float HPBVNL_f[MAX_VALVES_OCTANT];                
NETKEY HPBVNL={
"PVAKDATA","HPBVNL",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNL_f 
};
INT32 HPBVNLPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVNLPo={
"PVAKDATA","HPBVNLPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNLPo_l 
};
short HPBVNLSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVNLSt={
"PVAKDATA","HPBVNLSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNLSt_s 
};

float HPBVNR_f[MAX_VALVES_OCTANT];                
NETKEY HPBVNR={
"PVAKDATA","HPBVNR",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNR_f 
};
INT32 HPBVNRPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVNRPo={
"PVAKDATA","HPBVNRPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNRPo_l 
};
short HPBVNRSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVNRSt={
"PVAKDATA","HPBVNRSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVNRSt_s 
};

float HPBVOL_f[MAX_VALVES_OCTANT];                
NETKEY HPBVOL={
"PVAKDATA","HPBVOL",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVOL_f 
};
INT32 HPBVOLPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVOLPo={
"PVAKDATA","HPBVOLPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVOLPo_l 
};
short HPBVOLSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVOLSt={
"PVAKDATA","HPBVOLSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVOLSt_s 
};

float HPBVOR_f[MAX_VALVES_OCTANT];                
NETKEY HPBVOR={
"PVAKDATA","HPBVOR",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVOR_f 
};
INT32 HPBVORPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVORPo={
"PVAKDATA","HPBVORPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVORPo_l 
};
short HPBVORSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVORSt={
"PVAKDATA","HPBVORSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVORSt_s 
};

float HPBVSL_f[MAX_VALVES_OCTANT];                
NETKEY HPBVSL={
"PVAKDATA","HPBVSL",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSL_f 
};
INT32 HPBVSLPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVSLPo={
"PVAKDATA","HPBVSLPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSLPo_l 
};
short HPBVSLSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVSLSt={
"PVAKDATA","HPBVSLSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSLSt_s 
};

float HPBVSR_f[MAX_VALVES_OCTANT];                
NETKEY HPBVSR={
"PVAKDATA","HPBVSR",CF_FLOAT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSR_f 
};
INT32 HPBVSRPo_l[MAX_VALVES_OCTANT];                
NETKEY HPBVSRPo={
"PVAKDATA","HPBVSRPo",CF_LONG,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSRPo_l 
};
short HPBVSRSt_s[MAX_VALVES_OCTANT];                
NETKEY HPBVSRSt={
"PVAKDATA","HPBVSRSt",CF_SHORT,MAX_VALVES_OCTANT,0,RC_INI,T_INIT,ST_INI,PVAKVER,(void*)HPBVSRSt_s 
};

#endif /* PVAKVER */

