/***************************************  zeusdata.h begins here   ************************************************/
#define ZEUSVER 105

/*
  WWW access http://zulu4.desy.de/netmex/

  ZEUSVER 105: add ZC5DATA keyword; 
                   ZBEAMC5 is still supported except values #6 & #7 (are not being measured by C5 atm)
		   ZC5FAST is not supported (are not being measured by C5 atm)
		   ZTOROID,ZYOKECOIL,ZSOLENOID are filling up with data now (06.03.2002)

  ZEUSVER 104: add Motion_Timestamp to ZCALPOS data (31.10.2001) 
  
  ZEUSVER 103: add  ZCALPOS keyword (07.09.2001)

  ZEUSVER 102: add  ZNOTZ keyword (22.02.2001)

  ZEUSVER 101: add  ZC5fast keyword
	       change ZLUMI_NO from 11 to 15
*/

/*      Timestamps categories:

	  T0 - Timestamp refers to the time the data were taken.


	  T1 - Timestamp refers to the time the data were last 
	       requested from the source server.  For most volatile
	       parameters, this will still be a good reflection of the
	       value at that time.

	  T2 - Timestamp refers to the time the data are being 
	       requested from the (data) server.  
		
		e.g. If the dataserver does not explicitly set
		the timestamp for a keyword entry, the timestamp will 
		automatically be set to the time of the call (i.e. now).
*/ 

		/* ZEUS Sinus, gateway test                                                     */

#define ZEUSSIN_NO 10
short ZeusSin_s[ZEUSSIN_NO];
NETKEY ZeusSin={"ZEUSDATA","ZEUSSIN",CF_SHORT,ZEUSSIN_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZeusSin_s};


		/* ZEUS Lumi Counter, rates (Hz): total bremsstrahlung [0], e-p [1], pilot [2]  */ 
		/*                    instantaneous lumi [3], specific lumi [4]                 */
		/*                    integrated lumi since filling [5]                         */
		/*                    integrated lumi during run period [6]                     */
		/*                    vert. beam profile : mean [7] , rms [8]                   */
		/*                    hori. beam profile : mean [9] , rms [10]                  */
		/*                    run_time (?) [11], dead_time (?) [12],                    */
		/*                    gated lumi during the run[13],                            */
		/*                    gated lumi during the fill[14]                            */
		/*     Timestamp:  Category  T1                                                 */

#define ZLUMI_NO 15
double ZLumi_d[ZLUMI_NO];
NETKEY ZLumi={"ZEUSDATA","ZLUMI",CF_DOUBLE,ZLUMI_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZLumi_d};

		/* ZEUS C5 rates (Hz) (1 sec update time)                                       */
		/* proton background rate [0], electron background rate[1]                              */
		/*     Timestamp:  Category  T0                                                 */

#define ZC5FAST_NO 2            
float ZC5fast_f[ZC5FAST_NO];
NETKEY ZC5fast={"ZEUSDATA","ZC5FAST",CF_FLOAT,ZC5FAST_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZC5fast_f};

		/* ZEUS C5 rates (Hz), north beam pipe: proton side [0], electron side [1]      */
		/*                     south beam pipe: proton side [2], electron side [3]      */
		/*                     coincidence:     north [4]      , south [5]              */
		/*                     OR of coincidences [6], AND of coincidences [7]          */
		/*     Timestamp:  Category  T0                                                 */
		
#define ZBEAMC5_NO 8            
float ZBeamC5_f[ZBEAMC5_NO];
NETKEY ZBeamC5={"ZEUSDATA","ZBEAMC5",CF_FLOAT,ZBEAMC5_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZBeamC5_f};


		/* ZEUS Toroid Magnet Current in Amps [0] and % of nominal current [1]          */
		/*     Timestamp:  Category  T0                                                 */

#define ZTOROID_NO 2            
float ZToroid_f[ZTOROID_NO];
NETKEY ZToroid={"ZEUSDATA","ZTOROID",CF_FLOAT,ZTOROID_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZToroid_f};


		/* ZEUS Compensator Magnet Current in Amps [0] and % of nominal current [1]     */
		/*     Timestamp:  Category  T0                                                 */

#define ZCMPNSATOR_NO 2
float ZCmpnsator_f[ZCMPNSATOR_NO];
NETKEY ZCmpnsator={"ZEUSDATA","ZCMPNSATOR",CF_FLOAT,ZCMPNSATOR_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZCmpnsator_f};


		/* ZEUS Yoke Magnet Current in Amps [0] and % of nominal current [1]            */
		/*     Timestamp:  Category  T0                                                 */

#define ZYOKECOIL_NO 2
float ZYokeCoil_f[ZYOKECOIL_NO];
NETKEY ZYokeCoil={"ZEUSDATA","ZYOKECOIL",CF_FLOAT,ZYOKECOIL_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZYokeCoil_f};


		/* ZEUS Solenoid Magnet Current in Amps [0] and % of nominal current [1]        */
		/*     Timestamp:  Category  T0                                                 */

#define ZSOLENOID_NO 2
float ZSolenoid_f[ZSOLENOID_NO];
NETKEY ZSolenoid={"ZEUSDATA","ZSOLENOID",CF_FLOAT,ZSOLENOID_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZSolenoid_f};


		/* ZEUS General First Level Trigger - Sub Trigger Rates in Hz                   */
		/*      Trigger Slot 17 - FNC_neutron          [ 0] current, [ 1] run average   */
		/*      Trigger Slot 21 - C5                   [ 2] current, [ 3] run average   */
		/*      Trigger Slot 19 - C5VETO               [ 4] current, [ 5] run average   */
		/*      Trigger Slot 22 - BCALrc*VWout*IFRbpv  [ 6] current, [ 7] run average   */
		/*      Trigger Slot 28 - VETOwall_IN          [ 8] current, [ 9] run average   */
		/*      Trigger Slot 29 - VETOwall_OR          [10] current, [11] run average   */
		/*      Trigger Slot 18 - LPS_hit              [12] current, [13] run average   */
		/*     Timestamp:  Category  T1                                                 */

#define ZGFLT_NO 14
double ZGflt_d[ZGFLT_NO];
NETKEY ZGflt={"ZEUSDATA","ZGFLT",CF_DOUBLE,ZGFLT_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZGflt_d};


		/* ZEUS data from Dieter Notz:							*/
                /*      F5 				flux g/s				*/
                /*	L1 				0...200 l 				*/
                /*	C19, C24, C28           	valves (0=closed, 1=opened) 		*/
		/*	PA01				current in the coil 0..4000 A		*/
		/*     Timestamp:  Category  T0 						*/

#define ZNOTZ_NO 6		
float ZNotz_f[ZNOTZ_NO];
NETKEY ZNotz={"ZEUSDATA","ZNOTZ",CF_FLOAT,ZNOTZ_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZNotz_f};

                /* ZEUS FCAL and RCAL position data:		                        	*/
                /* 0	FCALN 			FCAL north positions (0..400000) in 1/1000 mm   */
                /* 1	FCALS 			FCAL south positions (0..400000) in 1/1000 mm   */
                /* 2	RCALN			RCAL north positions (0..400000) in 1/1000 mm   */
                /* 3	RCALS			RCAL south positions (0..400000) in 1/1000 mm   */
                /* 4	Positon status		(0: stationary intermediate,
		                                 1: opening,
						 2: closing,
						 3: open,
						 4: closed)					*/
                /* 5  	Inhibited_by_HERA	(0=movement enabled, 1=inhibited)		*/
                /* 6    Motion_Timestamp        (timestamp from R/F CAL movement program, 
		                                 updating when the program is active)           */
                /*     Timestamp:  Category  T1 						*/
#define ZCALPOS_NO 7		
short ZCalpos_l[ZCALPOS_NO];
NETKEY ZCalpos={"ZEUSDATA","ZCALPOS",CF_LONG,ZCALPOS_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZCalpos_l};


                /* ZEUS C5 data:								*/
                /* Rates C5:
                   0 	Chan.1  - single rate (c5 counter above beam pipe, proton side)     
                   1 	Chan.2  - single rate (c5 counter above beam pipe, electron side)        
                   2 	1&2     - coincidence rate (two c5 counters above beam pipe)       
		   3    Chan.3  - single rate (c5 counter below beam pipe, proton side)    
                   4    Chan.4  - single rate (c5 counter below beam pipe, electron side)       
                   5    3&4     - coincidence rate (c5 two counters below beam pipe)        
                   Background rates:
		   6	CTD  1 =         
		   7	CTD  2 =         
		   8	CTD  3 =         
		   9	CTD  4 =         
		   10	FCAL   =         
		   11	RCAL   =        
		   12   *RESERVED*
		   13   *RESERVED*
		   14   *RESERVED*
		   Triger rates:
		   15	Trig   =								*/        
		/*     Timestamp:  Category  T0                                                 */	
#define ZDATAC5_NO 16            
float ZDataC5_f[ZDATAC5_NO];
NETKEY ZDataC5={"ZEUSDATA","ZDATAC5",CF_FLOAT,ZDATAC5_NO,1,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZDataC5_f};

