/***************************************  zeusdata.h begins here   ************************************************/
#define ZEUSVER 110

/*
  WWW access: 
      homepage - 		  http://zulu4.desy.de/netmex/
      the data - 		  http://zulu4.desy.de/netmex/cgi-bin/gw_mex_data.cgi
      latest header (this) file - http://zulu4.desy.de/netmex/cgi-bin/giveheader.cgi?zeusdata
      is it running? 		- http://zulu4.desy.de/netmex/check.htm

  ZEUSVER 110:
               1. Remove ZCMPNSATOR keyword
               2. Remove ZEUSSIN keyword
               3. Remove RUNCREW keyword
               4. Add ZBEAMTEMPSTAT keyword Netmex type INTEGER An array contains 96 values
               5. Add ZBEAMTEMPVALUE keyword Netmex type FLOAT An array contains 96 values
               6. Add ZDATAC5EX keyword Netmex type FLOAT An array contains 16 values
               7. Add ZFDET keyword Netmex type LONG An array contains 6 values
               8. Extend ZLUMI keyword Before it was 15 DOUBLE values Now we want to se 17 DOUBLE values
               9. Add ZRADMONDOSE keyword Netmex type FLOAT An array contains 16 values

  ZEUSVER 109: change all calls to start at array element 0 instead of 1 to avoid out_of_range error
               fix ZCALPOS to allocate an array of UINT32 instead of short (P.D. 30.9.03)


  ZEUSVER 108: add MVD radiation monitor values:
                   ZRADMONRATE
		   ZRADMONSTAT  (30.07.2002)

  ZEUSVER 107: add keywords with Run Control information:
                   ZRUNNUM
		   ZRUNTYPE
		   ZRUNTYPESTR
		   ZTRIGTYPESTR
		   ZRUNCREW
		   ZRCSETUPDEF 	(05.06.2002)

  ZEUSVER 106: add ZFEEDBOX for whole feedbox data (also includes info of ZNOTZ keyword);    
                   ZBEAMC5 keyword removed;
                   ZC5FAST keyword removed;
                   ZDATAC5 number of values downsized to 6 (not-C5 related info removed) (03.05.2002)

  ZEUSVER 105: add ZC5DATA keyword; 
                   ZBEAMC5 is still supported except values #6 & #7 (are not being measured by C5 atm)
		   ZC5FAST is not supported (are not being measured by C5 atm)
		   ZTOROID,ZYOKECOIL,ZSOLENOID are filling up with data now (06.03.2002)

  ZEUSVER 104: add Motion_Timestamp to ZCALPOS data (31.10.2001) 
  
  ZEUSVER 103: add  ZCALPOS keyword (07.09.2001)

  ZEUSVER 102: add  ZNOTZ keyword (22.02.2001)

  ZEUSVER 101: add  ZC5fast keyword
	       change ZLUMI_NO from 11 to 15
*/

/*      Timestamps categories:

	  T0 - Timestamp refers to the time the data were taken.


	  T1 - Timestamp refers to the time the data were last 
	       requested from the source server.  For most volatile
	       parameters, this will still be a good reflection of the
	       value at that time.

	  T2 - Timestamp refers to the time the data are being 
	       requested from the (data) server.  
		
		e.g. If the dataserver does not explicitly set
		the timestamp for a keyword entry, the timestamp will 
		automatically be set to the time of the call (i.e. now).
*/ 


		/* ZEUS Lumi Counter, rates (Hz): total bremsstrahlung [0], e-p [1], pilot [2]  */ 
		/*                    instantaneous lumi [3], specific lumi [4]                 */
		/*                    integrated lumi since filling [5]                         */
		/*                    integrated lumi during run period [6]                     */
		/*                    vert. beam profile : mean [7] , rms [8]                   */
		/*                    hori. beam profile : mean [9] , rms [10]                  */
		/*                    run_time (?) [11], dead_time (?) [12],                    */
		/*                    gated lumi during the run[13],                            */
		/*                    gated lumi during the fill[14]                            */
		/*     Timestamp:  Category  T1                                                 */

#define ZLUMI_NO 17
double ZLumi_d[ZLUMI_NO];
NETKEY ZLumi={"ZEUSDATA","ZLUMI",CF_DOUBLE,ZLUMI_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZLumi_d};

		/* ZEUS Toroid Magnet Current in Amps [0] and % of nominal current [1]          */
		/*     Timestamp:  Category  T0                                                 */

#define ZTOROID_NO 2            
float ZToroid_f[ZTOROID_NO];
NETKEY ZToroid={"ZEUSDATA","ZTOROID",CF_FLOAT,ZTOROID_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZToroid_f};

		/* ZEUS Yoke Magnet Current in Amps [0] and % of nominal current [1]            */
		/*     Timestamp:  Category  T0                                                 */

#define ZYOKECOIL_NO 2
float ZYokeCoil_f[ZYOKECOIL_NO];
NETKEY ZYokeCoil={"ZEUSDATA","ZYOKECOIL",CF_FLOAT,ZYOKECOIL_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZYokeCoil_f};


		/* ZEUS Solenoid Magnet Current in Amps [0] and % of nominal current [1]        */
		/*     Timestamp:  Category  T0                                                 */

#define ZSOLENOID_NO 2
float ZSolenoid_f[ZSOLENOID_NO];
NETKEY ZSolenoid={"ZEUSDATA","ZSOLENOID",CF_FLOAT,ZSOLENOID_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZSolenoid_f};


		/* ZEUS General First Level Trigger - Sub Trigger Rates in Hz                   */
		/*      Trigger Slot 17 - FNC_neutron          [ 0] current, [ 1] run average   */
		/*      Trigger Slot 21 - C5                   [ 2] current, [ 3] run average   */
		/*      Trigger Slot 19 - C5VETO               [ 4] current, [ 5] run average   */
		/*      Trigger Slot 22 - BCALrc*VWout*IFRbpv  [ 6] current, [ 7] run average   */
		/*      Trigger Slot 28 - VETOwall_IN          [ 8] current, [ 9] run average   */
		/*      Trigger Slot 29 - VETOwall_OR          [10] current, [11] run average   */
		/*      Trigger Slot 18 - LPS_hit              [12] current, [13] run average   */
		/*     Timestamp:  Category  T1                                                 */

#define ZGFLT_NO 14
double ZGflt_d[ZGFLT_NO];
NETKEY ZGflt={"ZEUSDATA","ZGFLT",CF_DOUBLE,ZGFLT_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZGflt_d};


		/* ZEUS data from Dieter Notz:							*/
                /*      F5 				flux g/s				*/
                /*	L1 				0...200 l 				*/
                /*	C19, C24, C28           	valves (0=closed, 1=opened) 		*/
		/*	PA01				current in the coil 0..4000 A		*/
		/*     Timestamp:  Category  T0 						*/

#define ZNOTZ_NO 6		
float ZNotz_f[ZNOTZ_NO];
NETKEY ZNotz={"ZEUSDATA","ZNOTZ",CF_FLOAT,ZNOTZ_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZNotz_f};

                /* ZEUS FCAL and RCAL position data:		                        	*/
                /* 0	FCALN 			FCAL north positions (0..400000) in 1/1000 mm   */
                /* 1	FCALS 			FCAL south positions (0..400000) in 1/1000 mm   */
                /* 2	RCALN			RCAL north positions (0..400000) in 1/1000 mm   */
                /* 3	RCALS			RCAL south positions (0..400000) in 1/1000 mm   */
                /* 4	Positon status		(0: stationary intermediate,
		                                 1: opening,
						 2: closing,
						 3: open,
						 4: closed)					*/
                /* 5  	Inhibited_by_HERA	(0=movement enabled, 1=inhibited)		*/
                /* 6    Motion_Timestamp        (timestamp from R/F CAL movement program, 
		                                 updating when the program is active)           */
                /*     Timestamp:  Category  T1 						*/
#define ZCALPOS_NO 7		
UINT32 ZCalpos_l[ZCALPOS_NO];
NETKEY ZCalpos={"ZEUSDATA","ZCALPOS",CF_LONG,ZCALPOS_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZCalpos_l};


                /* ZEUS C5 data:								*/
                /* Rates C5:									
                   0 	Chan.1  - single rate (c5 counter above beam pipe, proton side)     
                   1 	Chan.2  - single rate (c5 counter above beam pipe, electron side)        
                   2 	1&2     - coincidence rate (two c5 counters above beam pipe)       
		   3    Chan.3  - single rate (c5 counter below beam pipe, proton side)    
                   4    Chan.4  - single rate (c5 counter below beam pipe, electron side)       
                   5    3&4     - coincidence rate (c5 two counters below beam pipe)            */
		/*     Timestamp:  Category  T0                                                 */	
#define ZDATAC5_NO 6            
float ZDataC5_f[ZDATAC5_NO];
NETKEY ZDataC5={"ZEUSDATA","ZDATAC5",CF_FLOAT,ZDATAC5_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZDataC5_f};


                /* ZEUS Feedbox data (extended ZNOTZ keyword)
	           - Pressures (P16&P18 E-06 mbar, P17,P19 E-03 mbar, all others mbar):   
		       "P1", "P3", "P4", "P5", "P6", "P7", "P8", "P9", "P10", "P14",
                       "P15", "P16", "P17", "P18", "P19", "P21", "P30";
		   - Fluxes (g/sec):
                       "F1", "F2", "F5", "F6", "F9", "F10", "F12";
                   - Temperatures (K):
                       "T1", "T1C", "T4", "T4C", "T5", "T5C", "T9", "T9C", "T10", "T11",
		       "T16", "T17", "T18", "T20", "T20C", "P30T";
                   - Helium level (l):
                       "L1";
		   - Valves (0=close, 1=open):
		       "VV2", "VV3", "V9", "V17", "V18", "V22", "V27", "V31", "V32", "V55",
		       "V57";
                   - Contignuous valves (0...1):
		       "C4", "C5", "C6", "C7", "C8", "C16", "C19", "C24", "C26", "C28",
		       "C30";
		   - Current in the coil (A):
		       "PA01".
		*/
#define  ZFEEDBOX_NO 64
float ZFeedbox_f[ZFEEDBOX_NO];
NETKEY ZFeedbox={"ZEUSDATA","ZFEEDBOX",CF_FLOAT,ZFEEDBOX_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZFeedbox_f};


long ZRunnum_l[1];
NETKEY ZRunnum={ "ZEUSDATA","ZRUNNUM",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRunnum_l };
long ZRuntype_l[1];
NETKEY ZRuntype={ "ZEUSDATA","ZRUNTYPE",CF_LONG,1,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRuntype_l };
#define ZSTRDATA_NO 256
char   ZRuntypeStr_c[ZSTRDATA_NO];
NETKEY ZRuntypeStr={ "ZEUSDATA","ZRUNTYPESTR",CF_TEXT,ZSTRDATA_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRuntypeStr_c };
char   ZTrigtypeStr_c[ZSTRDATA_NO];   
NETKEY ZTrigtypeStr={ "ZEUSDATA","ZTRIGTYPESTR",CF_TEXT,ZSTRDATA_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZTrigtypeStr_c };
char   ZRCsetupdef_c[ZSTRDATA_NO];     
NETKEY ZRCsetupdef={ "ZEUSDATA","ZRCSETUPDEF",CF_TEXT,ZSTRDATA_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRCsetupdef_c };

                /* ZEUS MVD pedestal subtracted Radmon diode currents (pico Amps) 		*/
                /*     Timestamp:  Category  T0                                                 */
#define  ZRADMONRATE_NO 16
float ZRadmonRate_f[ZRADMONRATE_NO];
NETKEY ZRadmonRate={"ZEUSDATA","ZRADMONRATE",CF_FLOAT,ZRADMONRATE_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRadmonRate_f};
                /* ZEUS MVD Radmon diode statuses (0=ok, 1=bad) 				*/
                /*     Timestamp:  Category  T0                                                 */
#define ZRADMONSTAT_NO 16
short ZRadmonStat_s[ZRADMONSTAT_NO];
NETKEY ZRadmonStat={"ZEUSDATA","ZRADMONSTAT",CF_SHORT,ZRADMONSTAT_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRadmonStat_s};

#define ZBEAMTEMPSTAT_NO 96
short ZBEAMTEMPSTAT_s[ZBEAMTEMPSTAT_NO];
NETKEY ZBEAMTEMPSTAT={"ZEUSDATA","ZBEAMTEMPSTAT",CF_SHORT,ZBEAMTEMPSTAT_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZBEAMTEMPSTAT_s};

#define ZBEAMTEMPVALUE_NO 96
float ZBEAMTEMPVALUE_f[ZBEAMTEMPVALUE_NO];
NETKEY ZBEAMTEMPVALUE={"ZEUSDATA","ZBEAMTEMPVALUE",CF_FLOAT,ZBEAMTEMPVALUE_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZBEAMTEMPVALUE_f};

#define ZDATAC5EX_NO 16
float ZDATAC5EX_f[ZDATAC5EX_NO];
NETKEY ZDATAC5EX={"ZEUSDATA","ZDATAC5EX",CF_FLOAT,ZDATAC5EX_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZDATAC5EX_f};

#define ZFDET_NO 6
long ZFDET_l[ZFDET_NO];
NETKEY ZFDET={"ZEUSDATA","ZFDET",CF_LONG,ZFDET_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZFDET_l};

#define ZRADMONDOSE_NO 16
float ZRADMONDOSE_f[ZRADMONDOSE_NO];
NETKEY ZRADMONDOSE={"ZEUSDATA","ZRADMONDOSE",CF_FLOAT,ZRADMONDOSE_NO,0,RC_INI,T_INIT,ST_INI,ZEUSVER,(void*)ZRADMONDOSE_f};
