#include "prolog.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "errors.h"
#include "rpctypes.h"
#include "servdef.h"
#include "toolkit.h"

#define BLOBSIZE 2000

#ifndef BLOBSIZE
# define BLOBSIZE 8000
#endif
double blob[BLOBSIZE];
PropertyQueryStruct prpinfo;
BYTE *bval = (BYTE *)blob;
short *sval = (short *)blob;
UINT32 *lval = (UINT32 *)blob;
float *fval = (float *)blob;
double *dval = blob;
char *cval = (char *)blob;
NAME16 *n16val = (NAME16 *)blob;
NAME32 *n32val = (NAME32 *)blob;

void tmonitorCallback(int id,int cc)
{
  int i;
  printf("\n");
  if (cc)
  {
    printf("error: %s\n>",RPCLastErrorString);
    return;
  }
  switch (LFMT(prpinfo.prpFormat))
  {
    case CF_BYTE:
      for (i=0; i<prpinfo.prpSize; i++) printf("%d ",bval[i]); break;
    case CF_SHORT:
      for (i=0; i<prpinfo.prpSize; i++) printf("%d ",sval[i]); break;
    case CF_LONG:
      for (i=0; i<prpinfo.prpSize; i++) printf("%d ",lval[i]); break;
    case CF_FLOAT:
      for (i=0; i<prpinfo.prpSize; i++) printf("%g ",fval[i]); break;
    case CF_DOUBLE:
      for (i=0; i<prpinfo.prpSize; i++) printf("%g ",dval[i]); break;
    case CF_TEXT:
      printf("%s",cval); break;
    case CF_NAME16:
      for (i=0; i<prpinfo.prpSize; i++) printf("%16s ",n16val[i].name); break;
    case CF_NAME32:
      for (i=0; i<prpinfo.prpSize; i++) printf("%32s ",n32val[i].name); break;
    default: printf("output format type %d not implemented\n",LFMT(prpinfo.prpFormat));
  }
}
int _tmonitor(void *buf,char *dev,char *prp,int fmt,int len,int tmr)
{
  int id;
  DTYPE dout;

  dout.dFormat = fmt;
  dout.dArrayLength = len;
  dout.dTag[0] = 0;
  dout.data.vptr = buf;
  id = AttachLink(dev,prp,&dout,NULL,CA_READ,tmr,tmonitorCallback,CM_POLL);
  return id < 0 ? -id : 0;
}
int _tget(void *buf,char *dev,char *prp,int fmt,int len,int tmr)
{
  DTYPE dout;

  dout.dFormat = fmt;
  dout.dArrayLength = len;
  dout.dTag[0] = 0;
  dout.data.vptr = buf;
  return ExecLinkEx(dev,prp,&dout,NULL,CA_READ,tmr);
}
int GetPropertyInformation(char *srv,char *prp,PropertyQueryStruct *srvProps)
{
  DTYPE dout,din;
  int cc = 0;
 
  din.dFormat = CF_NAME32;
  din.dArrayLength = 1;
  din.data.vptr = prp;
  dout.dFormat = CF_STRUCT;
  dout.dArrayLength = PROPERTYQUERYSTRUCT_SIZE;
  dout.data.vptr = srvProps;
  if ((cc=ExecLinkEx(srv,"PROPS",&dout,&din,CA_READ,500)) != 0) return cc;
  srvProps->prpSize = ISWAP(srvProps->prpSize);
  return 0;
}
char *fmtstr(int fmt)
{
  static char s[80];
  switch (LFMT(fmt))
  {
    case CF_BYTE: sprintf(s,"BYTE"); break;
    case CF_SHORT: sprintf(s,"short"); break;
    case CF_LONG: sprintf(s,"UINT32"); break;
    case CF_FLOAT: sprintf(s,"float"); break;
    case CF_DOUBLE: sprintf(s,"double"); break;
    case CF_TEXT: sprintf(s,"char"); break;
    case CF_NAME16: sprintf(s,"NAME16"); break;
    case CF_NAME32: sprintf(s,"NAME32"); break;
    default: sprintf(s,"type <%d>",LFMT(fmt)); break;
  }
  return s;
}
int tinfo(char *dev,char *prp)
{
  int i,cc;
  float max,min;
  char egu[80];
 
  if ((cc=GetPropertyInformation(dev,prp,&prpinfo)) != 0) return cc;
  if ((cc=GetDevicePropertyEGU(dev,prp,&max,&min,egu)) != 0) return cc;
  printf("\nDevice : %s\nProperty : %s\nDescription : %s\nSize : %d\nFormat %s\nMaximum : %f\nMinimum : %f\nUnits : %s\n",
         dev,prpinfo.prpName,prpinfo.prpDescription,prpinfo.prpSize,
         fmtstr(prpinfo.prpFormat),max,min,egu);
  return 0;
}
int tservers(char *cnt)
{
  int i,cc,n = 256;
  NAME16 *srvs = (NAME16 *)blob;
  if ((cc=GetDeviceServersEx(cnt,srvs,&n)) != 0) return cc;
  printf("%d servers found :\n",n);
  for (i=0; i<n; i++)
  {
    printf("%16s ",srvs[i].name);
    if ((i+1)%4 == 0) printf("\n");
  }
  return 0;
}
int tprops(char *dev)
{
  int i,cc,n = 256;
  NAME32 *prps = (NAME32 *)blob;
  if ((cc=GetDeviceProperties(dev,prps,&n)) != 0) return cc;
  printf("%d properties found :\n",n);
  for (i=0; i<n; i++)
  {
    printf("%32s ",prps[i].name);
    if ((i+1)%4 == 0) printf("\n");
  }
  return 0;
}
int tdevs(char *dev)
{
  int i,cc,n = 1024;
  NAME16 *devs = (NAME16 *)blob;
  char devname[32];
  if ((cc=GetDeviceNames(dev,devs,&n)) != 0) return cc;
  printf("%d devices found :\n",n);
  for (i=0; i<n; i++)
  {
    strncpy(devname,devs[i].name,16);
    if (devname[15] == '&')
    {
      i++; devname[15] = 0;
      strncat(devname,devs[i].name,16);
    }
    printf("%32s ",devname);
    if ((i+1)%4 == 0) printf("\n");
  }
  return 0;
}
int tdevlist(char *dev)
{
  int i,cc,n = 1024;
  NAME16 *devs = (NAME16 *)blob;
  char devname[32];
  if ((cc=GetDeviceNames(dev,devs,&n)) != 0) return cc;
  printf("DEVICE_NAME,DEVICE_NUMBER\n",n);
  for (i=0; i<n; i++)
  {
    strncpy(devname,devs[i].name,16);
    if (devname[15] == '&')
    {
      i++; devname[15] = 0;
      strncat(devname,devs[i].name,16);
    }
    printf("%s,%d\n",devname,i);
  }
  return 0;
}
/* #define OLDWAY */
int tget(char *dev,char *prp)
{
  int i,cc;
  int fmt,siz;
  char s[65];

# ifdef OLDWAY
  if ((cc=GetPropertyInformation(dev,prp,&prpinfo)) != 0) return cc;
  if ((cc=_tget(blob,dev,prp,LFMT(prpinfo.prpFormat),prpinfo.prpSize,5000)) != 0) return cc;
  fmt = LFMT(prpinfo.prpFormat);
  siz = prpinfo.prpSize;
# else
  if ((cc=_tget(blob,dev,prp,CF_DEFAULT,sizeof(blob),5000)) != 0) return cc;
  fmt = GetCompletionDataType(-1);
  siz = GetCompletionDataSize(-1);
# endif
  switch (fmt)
  {
    case CF_BYTE:
      for (i=0; i<siz; i++) printf("%d ",bval[i]); break;
    case CF_SHORT:
      for (i=0; i<siz; i++) printf("%d ",sval[i]); break;
    case CF_LONG:
      for (i=0; i<siz; i++) printf("%d ",lval[i]); break;
    case CF_FLOAT:
      for (i=0; i<siz; i++) printf("%g ",fval[i]); break;
    case CF_DOUBLE:
      for (i=0; i<siz; i++) printf("%g ",dval[i]); break;
    case CF_TEXT:
      printf("%s",cval); break;
    case CF_NAME16:
      for (i=0; i<siz; i++)
      {
        strncpy(s,n16val[i].name,16);
        printf("%s ",s);
      }
      break;
    case CF_NAME32:
      for (i=0; i<siz; i++)
      {
        strncpy(s,n32val[i].name,32);
        printf("%s ",s);
      }
      break;
    default: printf("output format type %d not implemented\n",fmt);
  }
  return 0;
}
int tmonitor(char *dev,char *prp)
{
  int i,cc;

  if ((cc=GetPropertyInformation(dev,prp,&prpinfo)) != 0) return cc;
  if ((cc=_tmonitor(blob,dev,prp,LFMT(prpinfo.prpFormat),prpinfo.prpSize,1000)) != 0) return cc;
  return 0;
}
time_t parseTimeString(char *ts)
{
  time_t t=time(NULL);
  int i, len;
  char *c,day[16],mon[64],year[16];
  struct tm tmts, *tnow;

  tnow = localtime(&t);
  len = strlen(ts);
  strupr(ts);
  for (i=0; i<len && isdigit(ts[i]); i++);
  if (i == len) t = atoi(ts);
  else if (!strcmp(ts,"NOW")) t = time(NULL);
  else if ((c=strstr(ts,".")) != NULL) // day.month.year format
  {
    *c++ = 0;
    strncpy(day,ts,16);
    strncpy(mon,c,64);
    if ((c=strstr(mon,".")) != NULL)
    {
      *c++ = 0;
      strncpy(year,c,16);
    }
    else
    {
      sprintf(year,"%d",1900+tnow->tm_year);
    }
    memset(&tmts,0,sizeof(struct tm));
    tmts.tm_mon = (atoi(mon)+11)%12;
    tmts.tm_mday = atoi(day);
    tmts.tm_year = atoi(year);
    if (tmts.tm_year > 1900) tmts.tm_year -= 1900;
    t = mktime(&tmts);
  }
  return t;
}
int thistory(char *context,char *keyword,char *devname,char *stoptime,char *depth)
{
  char devsrv[256];
  long stop = time(NULL), start, mf = 1, dp = 0;
  int i, num = 5000, cc;
  FLTINT *fiDataArray = (FLTINT *)blob;
  NAME32I *niDataArray = (NAME32I *)blob;

  if (keyword == NULL) return argument_list_error;
  if (devname == NULL) devname = "#0";
  if (context == NULL) context = "HERA";
  else strupr(context);
  if (!strcmp(context,"HERA")) sprintf(devsrv,"/HERA/HEHISTORY/");
  else if (!strcmp(context,"PETRA")) sprintf(devsrv,"/PETRA/PEHISTORY/");
  else if (!strncmp(context,"DESY",4)) sprintf(devsrv,"/DESY/D3HISTORY/");
  else if (!strncmp(context,"LINAC",5)) sprintf(devsrv,"/DESY/D3HISTORY/");
  else if (!strcmp(context,"DORIS")) sprintf(devsrv,"/DORIS/DOHISTORY/");
  else if (!strcmp(context,"TTF")) sprintf(devsrv,"/TTF/TTFHISTORY/");
  else return argument_list_error;
  strcat(devsrv,keyword);
  strcat(devsrv,"/");
  strcat(devsrv,devname);

  stop = parseTimeString(stoptime);
  strupr(depth);
  if (strstr(depth,"DAY")) mf = 24 * 3600;
  else if (strstr(depth,"HOUR")) mf = 3600;
  else if (strstr(depth,"MINUTE")) mf = 60;
  else if (strstr(depth,"WEEK")) mf = 24 * 3600 * 7;
  dp = mf * atoi(depth);
  start = stop - dp;
  printf("%s/%s, Timestamp\n",keyword,devname);
  if ((cc=GetArchivedDataAsFloat(devsrv,start,stop,fiDataArray,&num)) == 0)
  {
    for (i=0; i<num; i++)
    {
      printf("%g, %s",fiDataArray[i+1].fval,ctime(&fiDataArray[i+1].ival));
    }
    return 0;
  }
  if (cc != illegal_format) return cc;
  if ((cc=GetArchivedDataAsText(devsrv,start,stop,niDataArray,&num)) == 0) 
  {
    for (i=0; i<num; i++)
    {
      printf("%s, %s",niDataArray[i+1].name,ctime(&niDataArray[i+1].ival));
    }
    return 0;
  }
  
  return cc;
}
