#ifndef DATABASE_H_
#define DATABASE_H_

#include "project.def"
#if defined(__cplusplus) && !defined(FORCE_CPP)
extern "C" {
#endif

#define TXT 0
#define INT 1
#define FLT 2
#define HEX 3
#define SHT 4
#define FMT 5
#define ACC 6

typedef struct
{
  char *nam;
  char *def;
  int  pos;
  int  fmt;
  int  siz;
  BYTE *ptr;
  int (*fcn)(char *);
  BYTE *start;
} CSVCOLUMN;

typedef struct CSVDBStruct
{
  CSVCOLUMN *col;
  int ncols;
  int siz;
  void (*ini)(struct CSVDBStruct *);
  int  (*itr)(struct CSVDBStruct *);
  void (*exi)(struct CSVDBStruct *);
  int cnt;
  BYTE *tgt;
  int tgt_siz;
} CSVDB;

typedef struct    /* local */
{
  char UserName[16];
} UserListStruct;

int csvAssignValue(void *ptr,int fmt,int siz,char *str);
int csvReadFile(char *path,char *file,CSVDB *db,void **target);
int csvWriteFile(char *path,char *file,CSVDB *db);
void csvStdInit(struct CSVDBStruct *db);
int  csvStdIterator(struct CSVDBStruct *db);
void csvStdExit(struct CSVDBStruct *db);
int GetDataBaseSize(char *filename);
int locateFecName(char *fecname);
int locateTagName(char *expname,char *context,int *rpcidx);
int asyncLocateFecName(char *fecname);
int asyncLocateTagName(char *expname,char *context);
int GetAddressFromNameFile(char *context,char *expname, char *fecname,int *rpcidx);
int GetAddressFromNameServer(char *context,char *expname, char *fecname,int *rpcidx);
int GetAllocSizesFromNameServer(int *RPCsize,int *FECsize);
int GetAllocSizesFromNameFile(int *RPCsize,int *FECsize);
int appendAddressToCache(char *fecname,char *ip, short portOffset);
int appendServerToCache(char *fecname,char *context,char *expname, char *eqpname);
int namcmp(char *name,char *str,int col,int limit);
int findcol(char *str,char *tag);
char *colptr(int col,char *str);
void BytesToHex(BYTE *b,char *s,int n);
void HextoBytes(char *s,BYTE *b,int nibs);
int GetNameServerAddress(void);
int getFecName(void);
int appendRegisteredUserList(char *user);
int removeRegisteredUser(char *user);
int getRegisteredUsers(UserListStruct **ul,char *fn,int *nr,char *accstr,int *accflg);
void dumpRegisteredUsers(void);
int getRegisteredExports(void);
char *GetExportedName(char *eqpname);
NAME32 *GetEqpModuleNames(char *eqname,int num);
char *GetDeviceName(char *eqm,int devnr);
int GetDeviceNumber(char *eqm,char *dev);
int AssignNumModules(char *eqname,int num);
int GetPropertyId(char *eqm,char *prpName);
int outputConsumerList(void);
int outputContractList(void);
int outputContractStats(int i);
int outputConnectionStats(int k);
int outputServerStats(void);
int outputServerTime(void);
int dumpContractStats(int i);
int dumpConnectionStats(int i);
int dumpServerStats(void);
int dumpServerTime(void);
int dumpVersion(void);

extern FILE *dbgfp;
extern char dbgbuf[256];
extern char tagNameFilter[64];
extern int dbaseloaded;
extern int MaxRPCAllocation;
extern char ControlDBpath[80];
extern char gRPCDataBaseFullName[128];
extern char gFECDataBaseFullName[128];
extern int WRITEaccess;
extern int DENYaccess;
extern int NrOfUsers;
extern int NrOfDeniedUsers;
extern char FecDBpath[80];
extern int FecNameRegistered;
extern int ExportsRegistered;
extern int gotNameServerAddress;
extern UserListStruct *UserList;
extern UserListStruct *DeniedUserList;

#if defined(__cplusplus) && !defined(FORCE_CPP)
}
#endif

#endif /* DATABASE_H_ */
