/* nodal.h - include file for nodal compilation */

#ifndef NODAL_H
#define NODAL_H
#define NODAL
#include <stdio.h>
#include "errors.h"
#include "rpctypes.h"

#define MAX_DEVICES 21
typedef struct
{
	char Name[20];
	FILE *FP;
}DeviceStruct;

#define str_max 80

typedef          short  boolean    ;
typedef unsigned short  cardinal   ;
typedef          int    NodalError ;
typedef          double real;

typedef struct  {
		char *buffer   ;
		short reader   ;
		short writer   ;
		short max      ;
		char  string_buffer[str_max+2] ;
		} nodal_string ;

typedef  struct {
	int F_type;
	char F_name[6];
	char F_args[4];
	NodalError (*F_entry)();
	} FunHead;

/* Gerard specials */
#define AND     &&
#define OR      ||
#define IOR     |=        /* variable = variable | bit pattern */
#define NUL     '\0'
#define NEQ     !=
#define EQU     ==
#define TRUE    1
#define FALSE   0
#define ESCAPE  033
#define TRACE   printf

/* Function type definition */
#define data_module        7
#define rw_real            8
#define wo_real            9
#define ro_real           10
#define free_real         11
#define nodal_call        12
#define defined_function  13
#define defined_call      14
#define defined_string    15
#define free_call         18
#define rw_string         22
#define wo_string         23
#define ro_string         24
#define free_string       26

/* Array type specifiers */
#define ArrIntType         1
#define ArrLongType        2
#define ArrRealType        4


/* Declare Nodal Globals */

extern int far *NGstex;
extern int NGetex;
extern int NGwalim;
extern int far *NGxad;
extern int NGxlin;

extern int far *NGvloc;
extern int NGesymb;
extern int NGvlim;

extern int far *NGsdef;
extern int NGedef;
extern int NGdeflim;

extern int NGdflg;
extern int NGcrflg;
extern int NGslim;
extern int NGchkcnt;
extern int NGloc;
extern int NGtrace;
extern int NGerno;
extern unsigned short NGrunlin;

extern real NGarg[16];
extern nodal_string NGstrarg;
extern unsigned short NGedlin;
extern real NGval;
extern nodal_string NGsval;
extern int NGeximID;
extern int NGremitID;
extern int NGcamqx;
extern int NGdebug;
extern FILE *dbgfp;
extern int FeclogDepth;

/* Nodal Function Prototypes, for checking new work */

extern NodalError       check();
extern NodalError       getl(nodal_string *[]);
extern NodalError       OdevOut(nodal_string *);
extern NodalError       tco(char);
extern NodalError       crlf();
extern NodalError       ecmd(nodal_string  *p);
extern NodalError       gci(nodal_string *, char *);
extern NodalError       s_gci(nodal_string *, char *);
extern NodalError       skip(nodal_string *, char *);
extern NodalError       s_skip(nodal_string *, char *);
extern NodalError       gc(nodal_string *, char *);
extern void             trim(nodal_string *);
extern NodalError       wci(nodal_string *, char);
extern NodalError       fin(nodal_string *, real*);
extern NodalError       nset(nodal_string  *p, char name[6], real value);
extern NodalError       getop(nodal_string *, real *);
extern NodalError       lnum(real, unsigned short *);
extern NodalError       cns(nodal_string *, int);
extern NodalError       nsset(nodal_string *);
extern NodalError       ermes(nodal_string *, NodalError);
extern NodalError       eval(nodal_string *, real *);
extern NodalError       hexad(nodal_string *, long *);
extern NodalError       octal(nodal_string *, short *);
extern NodalError       radix36(nodal_string *, short *);
extern NodalError       fout(short, nodal_string *, real);
extern NodalError       rvar(nodal_string  *, char [6]);
extern NodalError       gentl(int far **element, unsigned short ln, int esize);
extern NodalError       gentd(int size, char name[6], int far **element);
extern NodalError       squeeze(int squize, int far *element, int place);
extern NodalError       gnam(char name[6], int far **element, int *type);
extern NodalError       onus(nodal_string *p, unsigned short ln);
extern boolean          nmck(char n[6]);
extern NodalError       gcond(nodal_string  *p, int *ccode);
extern int              scomp(nodal_string *p1, nodal_string *p2);
extern NodalError       runwa(unsigned short ln);
extern NodalError       get_fname(nodal_string *p, char fname[40]);
extern NodalError       N_load(nodal_string *p);
extern NodalError       N_save(nodal_string *p);
extern NodalError       N_ldef(nodal_string *p);
extern NodalError       N_sdef(nodal_string *p);
extern NodalError       lnls(nodal_string *p, int far *element);
extern NodalError       flin(unsigned short ln, unsigned short *xln, int far **element);
extern NodalError       DoDefinedFunction(nodal_string *p, int far *element, void *value, int flag);
extern NodalError       array(nodal_string *p, int far *element, real *value, int rwflag);
extern NodalError       DoNodalFunction(nodal_string *p, FunHead *header, void *value, int rwflag);
extern NodalError       sysvr(nodal_string *p, FunHead *header, real *value, int rwflag);
extern NodalError       comext(nodal_string *p, char *s);
extern NodalError       check();
extern char             tci(void);
extern NodalError       nfix(real x, int *i);
extern NodalError       nfix32(real x, long *l);
extern NodalError       scopy(nodal_string *s, nodal_string * d);
extern NodalError       sterm(nodal_string  *p, nodal_string  *result, short typeform);
extern NodalError       search_function(char name[6], FunHead **header, int *type);
extern void             NodalDelay(int sec);

int dorpc(void);
void dotask(void);
void doinit(void);
void doexit(void);

#endif /*  NODAL_H */

