/* NODFUNS.C - File of Nodal Functions for the PC */
/* Contains Functions, Function list */
/* routine search_function now in syslib.c */

#include "stdio.h"
#include "stdlib.h"
#include "malloc.h"
#include "ctype.h"
#include <math.h>
#include <time.h>
#include <conio.h>
#include <dos.h>
#include <dir.h>
#include <string.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <sys\timeb.h>
#include <io.h>
#include <signal.h>
#include <fcntl.h>
#include <share.h>
#include "nodal.h"
#include "brdcast.h"
#include "fecdef.h"
#ifdef NGLOBALS
#include "rpctypes.h"
#endif
#ifdef WATCHDOG
#define FECLOG
#define NETWATCH
#endif
#ifndef FECNAME
#define FECNAME "<No FEC name defined>"
#endif

#define FECLOG
#define u16 unsigned short
#define u32 unsigned long

boolean get_next(nodal_string *p,char c);
NodalError crlf(void); // nodal.c
NodalError welds(nodal_string *str, int far *element);
NodalError N_help(nodal_string *);   /* defined after command list */
NodalError plin(nodal_string *p);
NodalError spam(nodal_string  *p);
NodalError conc(nodal_string  *source, nodal_string  *result);
NodalError  N_list(nodal_string *p);
int nsset(nodal_string * str);
NodalError sset(nodal_string *p, char name[6], nodal_string *value);
boolean skip_next(nodal_string *p,char c);
NodalError cond(nodal_string  *p, int *ccond);
void reset_string(nodal_string *p);
short len(nodal_string  *p);
int C_StringCopy(nodal_string *np, char *ap);
NodalError deln(unsigned short ln);
NodalError deld(char name[6]);
NodalError get_array(nodal_string *p, int *d1, int *d2);
NodalError check(void);
NodalError gdef(char name[6], int far **element, int *type);
NodalError search_function(char name[6], FunHead **header, int *type);
NodalError initRPC(void);
NodalError F_sedac(real *value,int rwflag,u16 ln,u16 crate,u16 subadd);
NodalError F_crates(real *value,int rwflag,u16 ln);
NodalError F_addr(real *value,int rwflag,u16 ln,u16 crate);
NodalError F_globals(void);
NodalError F_broadcast(void);
NodalError F_showtime(void);
NodalError F_settime(void);
NodalError F_boot(void);
NodalError F_getConsumerList(void);
NodalError F_getConsumerList(void);
int dumpConsumerList(void);
int dumpRegisteredProperties(char *name);
int dumpConnectionList(void);
NodalError F_getContractList(void);
int dumpContractList(void);
int dumpContractStats(int i);
void dumpIPNets(void);
void dumpIPXNets(void);
int F_synchronize(void);
void quitServer(void);
void closeClients(void);
int findDaylight(void);
int getRegisteredProperties(void);
int getRegisteredUsers(void);
int localFecLog(char *text);
void logAppend(void);
int initSAPWatchDog(void);
void doConsumer(void);
int getRegisteredExports(void);
void checkTasklist(void);
void InitializeEQPModules(void);
void ExitEQPModules(void);
void AssignDotask(void (*tsk)(void));
void AssignExittask(void (*tsk)(void));
int feclog(char *text);
int NetHandler(int errval,int ax,int bp,int si);
int rdate(void);

extern NGodev;
extern int daylite;                         // daylight saving time flag
extern int nConn;
extern unsigned long TCPcount,UDPcount,SPXcount,IPXcount,BRDcount;
extern unsigned long nPCalarms,nArrivals;
extern int NrOfEqpTypes;
extern char fecname[17];
extern int BroadcastServer;

int initTCP(void);
extern int TCPSocket;

int watchdog = 0;
void watchDog(void);
void initWatchDog(void);
void disableWatchDog(void);
void netWatchDog(void);
extern char password[48];
int CommandLevel = 0;
void GetCommand(void) { return; }
#define CMDSIZE 128
char strbuf[CMDSIZE];
int ttyoutput(char *str)
{
  printf(str);
  return 0;
}


/*
check - check for external events
Called on a regular basis from within Nodal to allow event processing
return 0 to continue normally
return -4 if escape typed
*/
NodalError check(void)
{
  NGchkcnt=0;
  checkTasklist();
  dorpc();
  if (nConn) doConsumer();
  netWatchDog();
#ifdef WATCHDOG
  if (watchdog) watchDog();
#endif
  if (kbhit() && (getch()) == 27) return -4;
  else return 0;
}


/* tci - get a character from the input stream */
char tci(void)
{
//	static time_t systime = 0L;
  struct time t;
  static int next_hour = -1;

tciloop:
  checkTasklist();
  dorpc();
  if (nConn) doConsumer();
  netWatchDog();
#ifdef WATCHDOG
  if (watchdog) watchDog();
#endif

  gettime(&t);
  if (next_hour < 0) next_hour = t.ti_hour;
  if (t.ti_hour == next_hour)     // sychronize once per hour
  {
    F_settime();
    gettime(&t);
    next_hour = (next_hour + 1) % 24;
  }

  if (kbhit()) return getch();
  else goto tciloop;
}

extern DeviceStruct DeviceTab[MAX_DEVICES];

NodalError nprintf(char *s)
{
  static nodal_string str;
  crlf();  nsset(&str);
  C_StringCopy(&str,s);
  return OdevOut(&str);
}
/* Nodal LISF function */
NodalError F_lisf()
/*list NODAL programs from current directory*/
{
  struct ffblk ffblk;
  int cc,i,done;
  char Buffer[81];

  if ((cc=nprintf(">local NODAL programs:")) != 0) return cc;
  done = findfirst("*.NOD",&ffblk,0);
  for (i=1,Buffer[0]=0; !done; i++)
  {
     sprintf(&Buffer[strlen(Buffer)],"  %s",ffblk.ff_name);
     if (!(i%5))
     {
       if ((cc=nprintf(Buffer)) != 0) return cc;
       Buffer[0] = 0;
     }
		 done = findnext(&ffblk);
  }
  if (Buffer[0]) if ((cc=nprintf(Buffer)) != 0) return cc;
  return 0;
}
#pragma warn -pia -pro -def -par
/* Nodal DOS function  - Execute DOS command*/
NodalError F_dos(char * command)
{
  NodalError cc;
  if (cc=system(command)) return cc;
  return 0;
}

/* Nodal MEMORY function */
NodalError F_memory()
{
  NodalError cc;
  nodal_string str;
  char Buffer[81];

  crlf();  nsset(&str);
#ifndef MSC
  sprintf(Buffer,">Core Memory Available : %lu", coreleft());
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
#else
  sprintf(Buffer,"Memory Available on Heap:  %u",  _memavl());
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  crlf();  nsset(&str);
  sprintf(Buffer,"Max Heap Contiguous Block: %u", _memmax());
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
#endif
  crlf();  nsset(&str);
  sprintf(Buffer,">Memory Available on Stack: %u", stackavail());
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  return 0;
}

/* InitDeviceTab initializes the device table */
NodalError InitDeviceTab(void)
{
  int i;

  for (i=0;i<21;i++)
  {
    DeviceTab[i].Name[0]=0;DeviceTab[i].FP=NULL;
  }
  strcpy(DeviceTab[1].Name,"STDIN");DeviceTab[1].FP=stdin;
  strcpy(DeviceTab[2].Name,"STDOUT");DeviceTab[2].FP=stdout;
  strcpy(DeviceTab[3].Name,"LPT1");
  strcpy(DeviceTab[4].Name,"LPT2");
  strcpy(DeviceTab[5].Name,"LPT3");
  return 0;
}
NodalError F_open(real *handle,char *func,char *filename)
{
  int empty,found,i,index;
  FILE *fp;
  char type[3],file[80];

  if (!strlen(func) || strlen(func)>1) return argument_list_error;
  type[0]=tolower(func[0]);type[1]='b';type[2]=0;
  i=0;while (filename[i]) file[i++]=toupper(filename[i]);
  file[i]=0;

  i=0;found=0;empty=-1;
  while (i<21)
  {
    if (!strcmp(file,DeviceTab[i].Name)) {found=1;break;}
    if (i>=15 && DeviceTab[i].FP==NULL && empty==-1) empty=i;
    i++;
  }
  if (found)
  {
    index=i;
    if (index>=3 && DeviceTab[index].FP!=NULL)
    if (fclose(DeviceTab[index].FP)) return file_error;
  }
  else
    if (empty!=-1) index=empty;else return resources_exhausted;

  if (index>=3)
  {
    if (index>=15)
    {
      if (strchr(file,(int)'.')==NULL) strcat(file,".TXT");
    }
    if (fp=fopen(file,type),fp==NULL) return no_such_file;
    strcpy(DeviceTab[index].Name,file);DeviceTab[index].FP=fp;
  }
  *handle=(real)index;
  return 0;
}
NodalError F_close(int handle)
{
  int cc,i;

  if (handle==-1)
  {
    for (i=3;i<=5;i++)
      if (DeviceTab[i].FP!=NULL) fprintf(DeviceTab[i].FP,"%c",12);
    cc=0;
    if (fcloseall()==EOF) cc=file_error;
    else
    {
      for (i=3;i<21;i++)
      {
        DeviceTab[i].FP=NULL;
	if (i>=15) DeviceTab[i].Name[0]=0;
      }
      cc=0;
    }
    return cc;
  }
  if (handle<3 || handle>20) return no_such_file;
  if (DeviceTab[handle].FP==NULL) return file_not_open;
  cc=0;
  fprintf(DeviceTab[handle].FP,"%c",12);
  if (fclose(DeviceTab[handle].FP)) cc=file_error;
  if (handle>=15) DeviceTab[handle].Name[0]=0;
  DeviceTab[handle].FP=NULL;
  return cc;
}
NodalError F_odev(real *handle,int rwflag)
{
  int cc,i;
  real h;

  if (rwflag==1) *handle=(real)NGodev;
  else
  {
    i=(int)*handle;
    if (DeviceTab[i].FP==NULL)
    {
      if (strlen(DeviceTab[i].Name)) return file_error;
      if (cc=F_open(&h,"w",DeviceTab[i].Name)) return cc;
    }
    NGodev=i;
  }
  return 0;
}
NodalError F_inbt(real *value,int handle)
{
	int i;

	if (i=fgetc(DeviceTab[handle].FP),i==EOF)
	{
		if (feof(DeviceTab[handle].FP)) return end_of_file;
		else return file_error;
	}
	*value=(real)i;
	return 0;
}
NodalError F_outbt(real *value,int handle)
{
	int i;

	if (i=fputc((int)*value,DeviceTab[handle].FP),i!=(int)*value)
	{
		if (feof(DeviceTab[handle].FP)) return end_of_file;
		else return file_error;
	}
	return 0;
}
NodalError F_inpc(nodal_string *str,int handle)
{
	int i;
	char buf[100];

	if (i=fgetc(DeviceTab[handle].FP),i==EOF)
	{
		if (feof(DeviceTab[handle].FP)) return end_of_file;
		else return file_error;
	}
	sprintf(buf,"%c",i);
	C_StringCopy(str,buf);
	return 0;
}
NodalError F_outc(nodal_string *str,int handle)
{
	int cc;

	if (cc=fwrite(str->buffer,1,str->writer,DeviceTab[handle].FP),cc!=str->writer) return file_error;
	return 0;
}
NodalError F_input(nodal_string *str,int handle)
{
	int cc,x;
	real r;
	char buf[100];

	x=0;while (cc=F_inbt(&r,handle),!cc && r!=13) buf[x++]=(char)r;
	if (cc) return cc;
	if (r==13) F_inbt(&r,handle);
	buf[x]=0;
	C_StringCopy(str,buf);
	return 0;
}
NodalError F_output(nodal_string *str,int handle)
{
	int cc;
	char s[10];

	if (cc=fwrite(str->buffer,1,str->writer,DeviceTab[handle].FP),cc!=str->writer) return file_error;
	sprintf(s,"%c%c",13,10);
	if (cc=fwrite(s,1,2,DeviceTab[handle].FP),cc!=2) return file_error;
	return 0;
}

/* TIME Function to return time in seconds */
NodalError F_time(real *value)
{
  struct timeb ntime;
  ftime(&ntime);
  *value=(real)ntime.time+(real)(ntime.millitm)/1000;
  return 0;
}
NodalError F_showtime(void)
{
  char datebuf[9],timebuf[9],Buffer[81];

  sprintf(Buffer,">current FEC time: %s-%s (UNIX: %ld)\n",
         _strdate(datebuf),_strtime(timebuf),time(NULL));
  return nprintf(Buffer);
}

NodalError F_settime(void)
{
  static time_t lastlogged = 0;
  time_t t;
  int log = 0;
#ifdef NOSYNC
  return 0;
#endif
  t = time(NULL);
//  F_synchronize();
  rdate();
  if (abs((int)(time(NULL) - t)) > 2) log = 1;
  if (t > lastlogged + (3600*24)) log = 1;
  if (NGdebug)
  {
    nprintf(">time synchronized to server");
    F_showtime(); printf(">");
  }
#ifdef FECLOG
  if (log)
  {
    feclog("synchronized");
    lastlogged = t;
  }
#endif
  return 0;
}

#pragma warn -sig

/*
LISD - List contents if defined working area
LISD - list all elements with description
LISD NAME - list text of named defined function
*/
NodalError F_lisd(nodal_string *p)
{
NodalError ccecmd,cc;
char name[6];
int far *element, far *s_NGstex;
int type,npars,s_NGetex;
nodal_string str;
	ccecmd=ecmd(p);
	if (ccecmd) {
		if (!(cc=rvar(p,name))) return cc;
		if (!(cc=gdef(name,&element,&type))) return cc;
		s_NGstex=NGstex; s_NGetex=NGetex;
		npars=*(element+5); if (npars==-1) npars=0;
		NGstex=element+6+(npars<<2);
		NGetex=*element-(NGstex-element);
		nsset(&str);
		if (cc=N_list(&str)) return cc;
		NGstex=s_NGstex; NGetex=s_NGetex;
		}
	else element=NGsdef;
lisdit: if (element>=NGsdef+NGedef) goto lisdex;
	nsset(&str); welds(&str,element);
	if (cc=OdevOut(&str)) return cc;
	if (cc=crlf()) return cc;
	if (cc=check()) return cc;
	if (ccecmd) return ecmd(p);
	element+=*element; goto lisdit;
lisdex: nsset(&str);
	cns(&str,NGedef); C_StringCopy(&str," words used out of ");
	cns(&str,NGdeflim);
	if (cc=OdevOut(&str)) return cc;
	if (cc=crlf()) return cc;
	return 0;
}



/* Nodal SIN function, read only real */
NodalError F_sin(real *value, real *x) {*value=sin(*x); return 0;}


/* Nodal ERROR function */
NodalError F_error(real *value, int rwflag)
{
NodalError nerr;
	if (rwflag==1) {*value=(real)NGerno; return 0;}
	nerr=(NodalError)*value;
	if (nerr<=0) return out_of_range;
	return nerr;
}


/* Nodal ERMES function */
NodalError F_ermes(nodal_string *value, NodalError i)
{
	return ermes(value,i);
}


/* Nodal ARG function */
NodalError F_arg(real *value, int rwflag, int i)
{
	if (i<1 || i>16) return argument_list_error;
	if (rwflag==1) *value=NGarg[i-1]; else NGarg[i-1]=*value;
	return 0;
}

#pragma warn -par

/* Nodal STRARG function */
NodalError F_strarg(nodal_string *value, int rwflag, int i)
{
	if (rwflag==1) {
		NGstrarg.reader=0;
		return scopy(&NGstrarg,value);
		}
	else {  nsset(&NGstrarg);
		return scopy(value,&NGstrarg);
		}
}


/* Nodal AND function, read only int */
NodalError F_and(real *value, int m, int n)
{
	*value=(real)(m & n);
	return 0;
}


/* Nodal IOR function, read only int */
NodalError F_ior(real *value, int m, int n)
{
	*value=(real)(m | n);
	return 0;
}


/* Nodal EOR function, read only real */
NodalError F_eor(real *value, int m, int n)
{
	*value=(real)(m ^ n);
	return 0;
}


/* Nodal NEG function, read only int */
NodalError F_neg(real *value, int m)
{
	*value=(real)(~m);
	return 0;
}


/* Nodal SHIFT function, read only real */
NodalError F_shift(real *value, real *expr, int n)
{
NodalError cc;
long ml;
int sn;
	if (cc=nfix32(*expr,&ml)) return cc;
	sn=abs(n); if (sn>31) return out_of_range;
	if (n>=0) *value=(real)(ml << n);
	if (n<0)
	{
		*value=(real)((ml>>sn) & (0x7FFFFFFFL>>(sn-1)));
	}
	return 0;
}


/* Nodal BIT function, free real */
NodalError F_bit(real *value, int rwflag, nodal_string *p)
{
NodalError cc;
real x;
long la,lm;
int sn, savr, ecptr, sb;
char name[6];
	if (get_next(p,'(') EQU FALSE) return argument_list_error;
	if (cc=eval(p,&x)) return cc;
	if (cc=nfix(x,&sn)) return cc;
	if (sn>31 || sn<0) return out_of_range;
	lm=1; lm<<=sn;
	if (skip_next(p,',') EQU FALSE) return argument_list_error;
	savr=p->reader;
	if (cc=eval(p,&x)) return cc;
	if (cc=nfix32(x,&la)) return cc;
	if (skip_next(p,')') EQU FALSE) return argument_list_error;
	ecptr=p->reader;
	if (rwflag==-1)
	{
			if (cc=nfix(*value,&sb)) return cc;
	    if (sb>1 || sb<0) return out_of_range;
	    if (sb==1) la|=lm;
	    else la=la&~lm;
	    p->reader=savr;
			if (cc=rvar(p,name)) return cc;
			if (cc=nset(p,name,(real)la)) return cc;
	    p->reader=ecptr; return 0;
	}
	else
	{
			la&=lm;
			if (la) *value=1; else *value=0;
	}
	return 0;
}


/* WELDS write a varible descriptor to a string */
NodalError welds(nodal_string *str, int far *element)
{
char c,far *chptr;
int i,nchar,type,Nparam,npar;
int far *elptr;
int rows, cols, sw;
	chptr=(char far *)(element+2);
	for (i=0;i<6;i++) { if ((c=chptr[i])==0) c=' '; wci(str,c);}
	C_StringCopy(str,"  ");
	type=*(element+1);
	switch (type)
		{
		case 2:  C_StringCopy(str,"real variable = ");
			 fout(-1,str,*(real far *)(element+5));
			 break;
		case 3:  C_StringCopy(str,"read only variable = ");
			 fout(-1,str,*(real far *)(element+5));
			 break;
		case 4:  C_StringCopy(str,"array, ");
			 goto welarr;
		case 5:  C_StringCopy(str,"read only array, ");
			 goto welarr;
		case 16: C_StringCopy(str,"string variable = \"");
			 chptr=(char far *)(element+6);
			 nchar=*(element+5);
			 for (i=0;i<nchar;i++) wci(str,*chptr++);
			 wci(str,'"');
			 break;
		case 17: C_StringCopy(str,"string array, ");
			 cns(str,*(element+5));
			 C_StringCopy(str," elements");
			 break;
		case 13: C_StringCopy(str,"DEF-F ");
			 goto weldef;
		case 14: C_StringCopy(str,"DEF-C ");
			 goto weldef;
		case 15: C_StringCopy(str,"DEF-S ");
			 goto weldef;
		case 25: C_StringCopy(str,"reference pointer");
			 break;
		}
	return 0;
welarr: rows=*(element+5)>>4;
	sw=*(element+5) & 0x000F;
	cols=*(element+6)/rows;
	switch (sw)
	{
		case ArrIntType:  C_StringCopy(str,"Integer");
				  break;
		case ArrLongType: C_StringCopy(str,"Long");
					break;
		case ArrRealType: C_StringCopy(str,"Floating");
				  break;
	}
	wci(str,'(');
	cns(str,rows); wci(str,','); cns(str,cols);
	wci(str,')');
	return 0;
weldef:
	if ( (Nparam = *(element+5)) EQU -1 ) return 0;
	if (  Nparam EQU 0 ) {
		if ( type EQU 13 ) C_StringCopy(str," ()") ;
		return 0;
		}
	 wci(str,'(') ; elptr=element+6;
	 for ( npar=0; npar < Nparam; npar++ ) {
		switch( *elptr++ ) {
			case 1  : C_StringCopy(str,"V-"); break ;
			case 2  : C_StringCopy(str,"R-"); break ;
			default : C_StringCopy(str,"S-");
			}
		chptr=(char far *)(elptr);
		for (i=0;i<6;i++) if ((c=chptr[i])!=0) wci(str,c);
		if ( npar < (Nparam - 1) ) {wci(str,','); elptr+=3;}
		}
	 wci(str,')') ;
	 return 0;
}

/*
LISV - List variables in the variable working area
LISV - list all variables with description
LISV NAME - list description of named variable
*/
NodalError F_lisv(nodal_string *p)
{
NodalError ccecmd,cc;
char name[6];
int far *element;
int type;
nodal_string str;
	ccecmd=ecmd(p);
	if (ccecmd) {
		if (cc=rvar(p,name)) return cc;
		if (cc=gnam(name,&element,&type)) return cc;
		}
	else element=NGvloc;
lisvit: if (element>=NGvloc+NGesymb) goto lisvex;
	nsset(&str); welds(&str,element);
	if (cc=OdevOut(&str)) return cc;
	if (cc=crlf()) return cc;
	if (cc=check()) return cc;
	if (ccecmd) return ecmd(p);
	element+=*element; goto lisvit;
lisvex: nsset(&str);
	cns(&str,NGetex); C_StringCopy(&str," words of text out of ");
	cns(&str,NGwalim); C_StringCopy(&str,", ");
	cns(&str,NGesymb);
	C_StringCopy(&str," words of data out of "); cns(&str,NGvlim);
	if (cc=OdevOut(&str)) return cc;
	if (cc=crlf()) return cc;
	return 0;
}

/* Nodal DEBUG function */
NodalError F_debug(real *value, int rwflag)
{
	NodalError cc;
	nodal_string str;

	nsset(&str);
	if (rwflag==1) *value=(real)NGdebug;
	else NGdebug=(int)*value;
	switch (NGdebug) {
		 case 0:  C_StringCopy(&str,"No debug printing"); break;
		 case 1:  C_StringCopy(&str,"Debug level 1"); break;
		 case 2:  C_StringCopy(&str,"Debug level 2"); break;
		 case 3:  C_StringCopy(&str,"Debug level 3"); break;
		 default: return out_of_range;
	}
	if (cc=OdevOut(&str)) return cc;
	return 0;
}
NodalError F_log(real *value, int rwflag)
{
  int cc = 0;

  if (rwflag == -1)         // write it
  {
    if ((int)(*value) != 0) // turn on
    {
      if (dbgfp == NULL)  // was off
      {
        if ((dbgfp=fopen("debug.log","a")) == NULL) cc = file_error;
      }
    }
    else if (dbgfp != stdout)// turn off
    {
      if (fclose(dbgfp)) cc = file_error;
      dbgfp = NULL;
    }
  }
  else                      // read it
  {
    *value = (real)(dbgfp == stdout ? 0 : 1);
  }

  printf("Debug logging is %s\n>",dbgfp == NULL ? "OFF" : "ON");
  printf("Debug level %d\n>",NGdebug);
  return cc;
}
NodalError F_pause(void)
{
  printf("Hit any key to continue ...");
  getch();
  printf("\n");
  return 0;
}
NodalError F_getConsumerList(void)
{
  return dumpConsumerList();
}
NodalError F_getContractList(void)
{
  return dumpContractList();
}
NodalError F_getContractStats(int i)
{
  return dumpContractStats(i);
}
NodalError F_getConnectionList(void)
{
  return dumpConnectionList();
}
NodalError F_packetstats()
{
  NodalError cc;
  nodal_string str;
  char Buffer[81];

  crlf();  nsset(&str);
  sprintf(Buffer,">Total SPX requests   : %lu", SPXcount);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  crlf();  nsset(&str);
  sprintf(Buffer,">Total IPX requests   : %lu", IPXcount);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
#ifdef NGLOBALS
  crlf();  nsset(&str);
  sprintf(Buffer,">Total IPX broadcasts : %lu", BRDcount);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
#endif
  crlf();  nsset(&str);
  sprintf(Buffer,">Total TCP requests   : %lu", TCPcount);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  crlf();  nsset(&str);
  sprintf(Buffer,">Total UDP requests   : %lu", UDPcount);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  if (nConn == 0) return 0;
  crlf();  nsset(&str);
  sprintf(Buffer,">Total Client Arrivals: %lu", nArrivals);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  crlf();  nsset(&str);
  sprintf(Buffer,">Total Client Timeouts: %lu", nPCalarms);
  C_StringCopy(&str,Buffer);
  if (cc=OdevOut(&str)) return cc;
  return 0;
}
#ifdef WATCHDOG
NodalError F_watchdog(real *value, int rwflag)
{
  if (rwflag==1) *value = (real)watchdog;
  else
  {
    if (watchdog = (int)*value ? 1 : 0) initWatchDog();
    else disableWatchDog();
  }

  printf("Watchdog has been %s\n>", watchdog ? "activated" : "deactivated");

  return 0;
}
#endif
#define NumTCPChan	5
/* Nodal Function list */

//#include "emdecls.inc"

#pragma argsused
NodalError D_dummy(real far *v,int r,int n,char *p) { return 0; }
NodalError F_lisr();  /* Defined at end of file */

/* parameter type specifications */
#define RV_par          1    /* real */
#define IV_par          2    /* int */
#define SV_par          3    /* char [] */
#define NM_par          9    /* Nodal Name */

FunHead FunList [] = {
	{ free_call,  {'L','I','S','R',0,0},      {0,0,0,0}, F_lisr },
	{ rw_real,    {'D','E','B','U','G',0},    {0,0,0,0}, F_debug},
	{ rw_real,    {'L','O','G','D','B','G'},  {0,0,0,0}, F_log},
	{ free_call,  {'L','I','S','V',0,0},      {0,0,0,0}, F_lisv },
	{ nodal_call, {'G','L','O','B','A','L'},  {0,0,0,0}, F_globals},
	{ nodal_call, {'B','R','O','A','D',0},    {0,0,0,0}, F_broadcast},
	{ nodal_call, {'S','H','O','T','I','M'},  {0,0,0,0}, F_showtime},
	{ nodal_call, {'S','E','T','T','I','M'},  {0,0,0,0}, F_settime},
	{ nodal_call, {'B','O','O','T',0,0},      {0,0,0,0}, F_boot},
	{ nodal_call, {'L','I','S','F',0,0},      {0,0,0,0}, F_lisf },
	{ nodal_call, {'D','O','S',0,0,0},        {3,0,0,0}, F_dos  },
	{ nodal_call, {'M','E','M','O','R','Y'},  {0,0,0,0}, F_memory},
	{ ro_real,    {'O','P','E','N',0,0},      {3,3,0,0}, F_open},
	{ nodal_call, {'C','L','O','S','E',0},    {2,0,0,0}, F_close},
	{ rw_real,    {'O','D','E','V',0,0},      {0,0,0,0}, F_odev},
	{ ro_real,    {'I','N','B','T',0,0},      {2,0,0,0}, F_inbt},
	{ wo_real,    {'O','U','T','B','T',0},    {2,0,0,0}, F_outbt},
	{ ro_string,  {'I','N','P','C',0,0},      {2,0,0,0}, F_inpc},
	{ wo_string,  {'O','U','T','C',0,0},      {2,0,0,0}, F_outc},
	{ wo_string,  {'O','U','T','P','U','T'},  {2,0,0,0}, F_output},
	{ ro_string,  {'I','N','P','U','T',0},    {2,0,0,0}, F_input},
	{ ro_real,    {'T','I','M','E',0,0},      {0,0,0,0}, F_time},
	{ free_call,  {'L','I','S','D',0,0},      {0,0,0,0}, F_lisd},
	{ ro_real,    {'S','I','N',0,0,0},        {1,0,0,0}, F_sin},
	{ rw_real,    {'E','R','R','O','R',0},    {0,0,0,0}, F_error},
	{ ro_string,  {'E','R','M','E','S',0},    {2,0,0,0}, F_ermes},
	{ rw_real,    {'A','R','G',0,0,0},        {2,0,0,0}, F_arg},
	{ rw_string,  {'S','T','R','A','R','G'},  {0,0,0,0}, F_strarg},
	{ ro_real,    {'A','N','D',0,0,0},        {2,2,0,0}, F_and},
	{ ro_real,    {'I','O','R',0,0,0},        {2,2,0,0}, F_ior},
	{ ro_real,    {'E','O','R',0,0,0},        {2,2,0,0}, F_eor},
	{ ro_real,    {'N','E','G',0,0,0},        {2,0,0,0}, F_neg},
	{ ro_real,    {'S','H','I','F','T',0},    {1,2,0,0}, F_shift},
	{ free_real,  {'B','I','T',0,0,0},        {0,0,0,0}, F_bit},
	{ nodal_call, {'S','T','A','T','S',0},    {0,0,0,0}, F_packetstats},
	{ nodal_call, {'C','O','N','T','R',0},    {0,0,0,0}, F_getContractList},
	{ nodal_call, {'F','C','O','N','T','R'},  {IV_par,0,0,0}, F_getContractStats},
	{ nodal_call, {'C','L','I','E','N',0},    {0,0,0,0}, F_getConsumerList},
	{ nodal_call, {'C','O','N','N','S',0},    {0,0,0,0}, F_getConnectionList},
	{ nodal_call, {'P','A','U','S','E',0},    {0,0,0,0}, F_pause},
#ifdef WATCHDOG
	{ rw_real,    {'W','A','T','C','H','D'},  {0,0,0,0}, F_watchdog},
#endif
#ifdef SEDAC
	{ rw_real, {'S','E','D','A','C',0},    {IV_par,IV_par,IV_par,0}, F_sedac},
	{ rw_real, {'C','R','A','T','E','S'},  {IV_par,0,0,0},           F_crates},
	{ rw_real, {'A','D','D','R',0,0},      {IV_par,IV_par,0,0},      F_addr},
#endif
//#include "emspecs.inc"
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	{ data_module,{'D','U','M','M','Y',0},        {0,0,0,0}, D_dummy },
	};
#define nfuns (sizeof(FunList) / sizeof(FunHead ))
int NumNodalFunctions = nfuns;

#pragma warn -def

/* LISR Nodal list resident functions */
NodalError F_lisr(nodal_string *p)
{
  NodalError ccecmd,cc;
  char c,name[6];
  FunHead * header;
  int i,type,nfun;
  nodal_string str;
  ccecmd=ecmd(p);
  if (ccecmd) {cc=rvar(p,name); if (cc) return cc;} else goto lisrall;
  if (cc=search_function(name,&header,&type))return cc;
  lisrit: nsset(&str);
  for (i=0;i<6;i++) { if ((c=header->F_name[i])==0) c=' '; wci(&str,c);}
  C_StringCopy(&str,"  ");
  switch (type=header->F_type) {
    case data_module: C_StringCopy(&str,"HERA P Equipment Module (IV,SV)"); break;
    case rw_real: C_StringCopy(&str,"R-W Real function"); break;
    case wo_real: C_StringCopy(&str,"W-O Real function"); break;
    case ro_real: C_StringCopy(&str,"R-O Real function"); break;
    case free_real: C_StringCopy(&str,"Free Real function"); break;
    case nodal_call: C_StringCopy(&str,"Call function"); break;
    case free_call: C_StringCopy(&str,"Free Call function"); break;
    case rw_string: C_StringCopy(&str,"R-W String function"); break;
    case wo_string: C_StringCopy(&str,"W-O String function"); break;
    case ro_string: C_StringCopy(&str,"R-O String function"); break;
    case free_string: C_StringCopy(&str,"Free String function"); break;
    default: return wrong_type;
  }
  if (!(type==free_real || type==free_call || type==free_string)) {
    if (!header->F_args[0]) goto lisrout;
    C_StringCopy(&str,"  (");
    for (i=0;;) {
      switch (header->F_args[i]){
        case RV_par: C_StringCopy(&str,"RV"); break;
        case IV_par: C_StringCopy(&str,"IV"); break;
        case SV_par: C_StringCopy(&str,"SV"); break;
        case NM_par: C_StringCopy(&str,"NM");
      }
      if (header->F_args[++i]==0 || i==4) break;
      wci(&str,',');
    }
    wci(&str,')');
  }
lisrout:if (cc=OdevOut(&str)) return cc;
  if (cc=crlf()) return cc;
  if (cc=check()) return cc;
  if (ccecmd) return ecmd(p);
  if (++nfun<nfuns) goto lisrnxt;
  nsset(&str); C_StringCopy(&str,"Total number of functions = ");
  cns(&str,nfuns);
  if (cc=OdevOut(&str)) return cc;
  if (cc=crlf()) return cc;
  return 0;
lisrall:nfun=0;
lisrnxt:header=&FunList[nfun];
  goto lisrit;
}

/* DoNodalFunction - call a Nodal function with parameter list */
/* Equivalent of ASSFN in previous implementations */

#pragma warn -pro
extern int NrOfUsers;

NodalError DoNodalFunction(nodal_string *p, FunHead *header, void *value, int rwflag)
{
  static int arg[4];
  static real real_arg[4];
  static nodal_string string_arg[4];
  static char name[4][6];
  char *nmptr;
  int i;
  NodalError cc;
  char c,ptype;
  nodal_string *sptr;
  real x;
  if (!header->F_args[0]) goto DoNodit;
  if ((cc=gci(p,&c)) && (c!='(')) return argument_list_error;
  i=0; ptype=header->F_args[i];
parloop:switch (ptype) {
    case RV_par: if (cc=eval(p,&real_arg[i])) return cc;
                 arg[i]=(int)&real_arg[i];
		 break;
    case IV_par: if (cc=eval(p,&x)) return cc;
                 if (cc=nfix(x,&arg[i])) return cc;
		 break;
    case SV_par: sptr=&string_arg[i];
                 nsset(sptr);
		 if (cc=conc(p,sptr)) return cc;
		 if (cc=wci(sptr,'\0')) return cc;
		 arg[i]=(int)sptr->buffer;
		 break;
    case NM_par: nmptr=name[i];
		 if (cc=rvar(p,nmptr)) return cc;
		 arg[i]=(int)nmptr;
  }
parnext:if ((++i<4) && (ptype=header->F_args[i]))
  {
    if ((cc=skip(p,&c)) || (c!=',')) return argument_list_error;
    goto parloop;
  }
  if ((cc=gci(p,&c)) || (c!=')')) return argument_list_error;
DoNodit:switch (header->F_type) {
    case nodal_call:
         cc=(*header->F_entry)(arg[0],arg[1],arg[2],arg[3]);
	 return cc;
    case wo_real:
    case ro_real:
    case wo_string:
    case ro_string: cc=(*header->F_entry)(value,arg[0],arg[1],arg[2],arg[3]);
         return cc;
    case rw_real:
    case rw_string: cc=(*header->F_entry)(value,rwflag,arg[0],arg[1],arg[2],arg[3]);
         return cc;
    default: return wrong_type;
  }
}

#pragma warn -pia -pro -def -par

/* sysvr - call an Equipment Module from Nodal */
NodalError sysvr(nodal_string *p, FunHead *header, real *value, int rwflag)
{
real n;
int eqno;
nodal_string str;
char c;
real far *fpdata;
char name[6],Buffer[81];
int far *element;
int cc, flag, type, artyp;

  if ((gci(p,&c)) && (c!='(')) return argument_list_error;
  fpdata=(real far *)value;
  flag=rwflag;
  if (rwflag==0)
  {
    if (cc=rvar(p,name)) return cc;
    if (cc=gnam(name,&element,&type)) return cc;
    if (type!=4) return wrong_type;
    artyp = *(element+5)&0x000F;
    if (artyp!=4) return dimension_error;
    fpdata = (real far *)(element+7);
    if ((gci(p,&c)) && (c!=',')) return argument_list_error;
    flag = *(element+6);
    nsset(&str);
    if (cc=conc(p,&str)) return cc;
    if (cc=gci(&str,&c)) return cc;
    if (c=='W') flag=-flag;
    else if (c!='R') return syntax_error;
    if ((gci(p,&c)) && (c!=',')) return argument_list_error;
  }
  if (cc=eval(p,&n)) return cc;
  if (cc=nfix(n,&eqno)) return cc;
  if (skip(p,&c) || (c!=',')) return argument_list_error;
  nsset(&str);
  if (cc=conc(p,&str)) return cc;
  if (str.writer>9) return illegal_property;
  if (cc=wci(&str,'\0')) return cc;
  if (skip(p,&c) || (c!=')')) return argument_list_error;
  if (!stricmp(str.buffer,"PROPS"))
  {
    sprintf(Buffer,">Registered properties for function %s:",header->F_name);
    if ((cc=nprintf(Buffer)) != 0) return cc;
    *fpdata = dumpRegisteredProperties(header->F_name);
    return 0;
  }
  if (!stricmp(str.buffer,"RDEXPS"))
  {
    cc = getRegisteredExports();
    *fpdata = NrOfEqpTypes;
    return cc;
  }
  if (!stricmp(str.buffer,"RDUSERS"))
  {
    cc = getRegisteredUsers();
    *fpdata = NrOfUsers;
    return cc;
  }
  cc=(*header->F_entry)(fpdata,flag,eqno,str.buffer);
  return cc;
}

NodalError F_globals(void)
{
  char Buffer[81];
	int cc,i,j;

	if (nglobals == 0) return nprintf(">No net globals defined for this FEC");
	for (i=0; i<nglobals; i++)
	{
		sprintf(Buffer,">%s = ",NGlobalList[i]->keyword);
		switch (NGlobalList[i]->format)
		{
			case CF_DOUBLE:
					 for (j=0; j<NGlobalList[i]->size; j++)
						 sprintf(&Buffer[strlen(Buffer)],"%g ",((double far *)NGlobalList[i]->data)[j]);
					 break;
			case CF_INTEGER:
					 for (j=0; j<NGlobalList[i]->size; j++)
						 sprintf(&Buffer[strlen(Buffer)],"%d ",((int far *)NGlobalList[i]->data)[j]);
					 break;
			case CF_LONG:
					 for (j=0; j<NGlobalList[i]->size; j++)
						 sprintf(&Buffer[strlen(Buffer)],"%g ",((long far *)NGlobalList[i]->data)[j]);
					 break;
			case CF_FLOAT:
					 for (j=0; j<NGlobalList[i]->size; j++)
						 sprintf(&Buffer[strlen(Buffer)],"%g ",((float far *)NGlobalList[i]->data)[j]);
					 break;
			case CF_BYTE:
					 for (j=0; j<NGlobalList[i]->size; j++)
						 sprintf(&Buffer[strlen(Buffer)],"%d ",NGlobalList[i]->data[j]);
					 break;
		}
		if ((cc=nprintf(Buffer)) != 0) return cc;
	}
	return 0;
}

NodalError F_broadcast(void)
{
  int i,j;
  char Buffer[81];

  if (nbroadcasts == 0) return nprintf("Not broadcasting from this FEC");
  dumpIPXNets(); dumpIPNets();
  for (i=0; i<nbroadcasts; i++)
  {
    sprintf(Buffer,">%s = ",BrdcstList[i]->keyword);
    switch (BrdcstList[i]->format)
    {
      case CF_DOUBLE:
	   for (j=0; j<BrdcstList[i]->size; j++)
	   sprintf(&Buffer[strlen(Buffer)],"%g ",((double far *)BrdcstList[i]->data)[j]);
	   break;
      case CF_INTEGER:
           for (j=0; j<BrdcstList[i]->size; j++)
	   sprintf(&Buffer[strlen(Buffer)],"%d ",((int far *)BrdcstList[i]->data)[j]);
	   break;
      case CF_LONG:
           for (j=0; j<BrdcstList[i]->size; j++)
	   sprintf(&Buffer[strlen(Buffer)],"%ld ",((long far *)BrdcstList[i]->data)[j]);
	   break;
      case CF_FLOAT:
	   for (j=0; j<BrdcstList[i]->size; j++)
	   sprintf(&Buffer[strlen(Buffer)],"%g ",((float far *)BrdcstList[i]->data)[j]);
	   break;
      case CF_BYTE:
	   for (j=0; j<BrdcstList[i]->size; j++)
	   sprintf(&Buffer[strlen(Buffer)],"%d ",BrdcstList[i]->data[j]);
	   break;
      case CF_CHAR8:
	   for (j=0; j<BrdcstList[i]->size; j++)
	     strncpy(&Buffer[strlen(Buffer)],(char *)&BrdcstList[i]->data[j*8],8);
	   break;
      case CF_CHAR16:
	   for (j=0; j<BrdcstList[i]->size; j++)
	     strncpy(&Buffer[strlen(Buffer)],(char *)&BrdcstList[i]->data[j*16],16);
	   break;
      case CF_CHAR32:
	   for (j=0; j<BrdcstList[i]->size; j++)
	     strncpy(&Buffer[strlen(Buffer)],(char *)&BrdcstList[i]->data[j*32],32);
	   break;
    }
    nprintf(Buffer);
  }
  return 0;
}

#ifdef SEDAC

#ifndef SEDAC_PORTADDR
#define SEDAC_PORTADDR 0x300
#endif

#include "sedac.h"

#ifdef SIMULATE_SEDAC
u16 sedacSimulationBuffer[4];
u16 sedacSimulationCount[4] = {0,0,0,0};
#endif
int nlines;

SEDAC_LINE line[4] = { {SEDAC_PORTADDR,      SEDAC_PORTADDR|0x02,
			SEDAC_PORTADDR|0x04, SEDAC_PORTADDR|0x06},
                       {SEDAC_PORTADDR|0x10, SEDAC_PORTADDR|0x12,
			SEDAC_PORTADDR|0x14, SEDAC_PORTADDR|0x16},
		       {SEDAC_PORTADDR|0x20, SEDAC_PORTADDR|0x22,
			SEDAC_PORTADDR|0x24, SEDAC_PORTADDR|0x26},
		       {SEDAC_PORTADDR|0x30, SEDAC_PORTADDR|0x32,
			SEDAC_PORTADDR|0x34, SEDAC_PORTADDR|0x36} };

void initSedac(void)
{
  int i;

  for (i=0, nlines=0; i<SEDAC && i<4; i++)   // look for up to # SEDAC lines
  {
#ifndef SIMULATE_SEDAC
    outport(line[i].csr, 0x1); delay(5);
    outport(line[i].csr, 0x0);
    outportb(0x20,0x20);
    inport(line[i].igr);
#endif
    nlines++;
  }
  return;
}

// ============  write routine  ===================================

int wsedac(u16 ln, u16 crate, u16 subadd, u16 data)
{
# ifndef SIMULATE_SEDAC
  int address, cnt = 2000, retry = 0, error = 0;
# endif

  if (ln<1 || ln>nlines) return -1;
  ln--;
# ifndef SIMULATE_SEDAC
  address = 0x4000 | (crate << 8) | subadd;
  do
  {
    outport(line[ln].dat,data);
    outport(line[ln].cmd,address);
    while (!(inport(line[ln].csr) & _RBE) && cnt--);
    if (cnt <= 0) { initSedac(); return -1; }
  } while ( (error=inport(line[ln].cmd)) && retry++<1);
  return error;
# else
  sedacSimulationBuffer[ln] = data;
  return 0;
# endif
}

// ================  read routine  ===================================

int rsedac(u16 ln, u16 crate, u16 subadd, u16 far *data)
{
# ifndef SIMULATE_SEDAC
  int address, cnt = 2000, retry = 0, error = 0;
# endif

  if (ln<1 || ln>nlines) return -1;
  ln--;
# ifndef SIMULATE_SEDAC
  address = 0x8000 | (crate << 8) | subadd;
  do
  {
    outport(line[ln].cmd, address);
    while (!(inport(line[ln].csr) & _RBE) && cnt--);
    if (cnt <= 0) { initSedac(); return -1; }
    *data = inport(line[ln].dat);
  } while ( (error=inport(line[ln].cmd)) && retry++<1);
  return error;
# else
  *data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
  return 0;
# endif
}
int rsedacAsync(u16 ln,u16 crate,u16 subadd)
{
# ifndef SIMULATE_SEDAC
  int address;
# endif
  if (ln<1 || ln>nlines) return -1;
  ln--;
# ifndef SIMULATE_SEDAC
  address = 0x8000 | (crate << 8) | subadd;
  outport(line[ln].cmd,address);
# endif
  return 0;
}
int rsedacPoll(u16 ln,u16 *data)
{
# ifndef SIMULATE_SEDAC
  while (!(inport(line[ln].csr) & _RBE));
  *data = inport(line[ln].dat);
  return inport(line[ln].cmd);
# else
  *data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
  return 0;
# endif
}

// ============  Block routines ==========================================

void sedac4(SedacInfo *sedacList, int RWF, int num, int (*fcn)(int))
// reads/writes 4 SEDAC lines Round Robin (with service function if present)
{
  int i, l, cnt;

  switch( RWF )
  {
    case READ:
      for (i=0; i<num; i++)
      {
        for (l=0; l<SEDAC; l++)                          // SEDAC lines out
	{
#         ifndef SIMULATE_SEDAC
          outport(line[l].cmd,0x8000|sedacList[l].csa[i]);   // load CMDreg
#         endif
        }
	if (fcn) fcn(i);                                // service function
	for (l=0; l<SEDAC && (cnt=2000); l++)               // SEDAC line in
	{
#         ifndef SIMULATE_SEDAC
          while (!(inport(line[l].csr) & _RBE) && cnt--);   // poll RBE bit
	  sedacList[l].data[i]  = inport(line[l].dat);         // read data
	  sedacList[l].error[i] = inport(line[l].cmd);       // error table
	  if (cnt <= 0) { sedacList[l].error[i] = -1; initSedac(); }
#         else
          sedacList[l].data[i] = sedacSimulationBuffer[l] +
	                         sedacSimulationCount[l]++;
	  sedacList[l].error[i] = cnt-2000;
#         endif
        }
      }
      if (fcn) fcn(i);                                           // i = num
      return;
    case WRITE:
      for (i=0; i<num; i++)
      {
        for (l=0; l<SEDAC; l++)                          // SEDAC lines out
	{
#         ifndef SIMULATE_SEDAC
          outport(line[l].dat,sedacList[l].data[i]);           // load data
	  outport(line[l].cmd,0x4000|sedacList[l].csa[i]);   // load CMDreg
#         endif
        }
	if (fcn) fcn(i);                                // service function
	for (l=0; l<SEDAC && (cnt=2000); l++)              // SEDAC lines in
	{
#         ifndef SIMULATE_SEDAC
          while (!(inport(line[l].csr) & _RBE) && cnt--);   // poll RBE bit
	  sedacList[l].error[i] = inport(line[l].cmd);
	  if (cnt <= 0) { sedacList[l].error[i] = -1; initSedac(); }
#         else
          sedacSimulationBuffer[l] = sedacList[l].data[i];
	  sedacList[l].error[i] = 0;
#         endif
        }
      }
      if (fcn) fcn(i);                                           // i = num
      return;
  }
  return;
}

int sedac(int ln, u16 csa, int RWF, u16 far *data, int nwords)
{
  int i,cnt;
  u16 addr, cmd, csr, dat;

  if (ln<1 || ln>nlines) return -1;
  ln--;
  cmd = line[ln].cmd; csr = line[ln].csr; dat = line[ln].dat;
  switch( RWF )
  {
    case READ:
      addr = 0x8000 | csa;
      for (i=0; i<nwords && (cnt=2000); i++, data++)
      {
#       ifndef SIMULATE_SEDAC
        outport(cmd,addr);                          // load CMDreg
	while (!(inport(csr) & _RBE) && cnt--);     // poll RBE bit
	*data = inport(dat);                        // read data
	if (cnt <= 0 || inport(cmd)) { initSedac(); return(i+1); }
#       else
        cmd = cmd; csr = csr;  cnt = cnt;           // avoid warnings
	dat = dat, addr = addr;
	*data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
#       endif
      }
      return 0;
      case WRITE:
        addr = 0x4000 | csa;
	for (i=0; i<nwords && (cnt=2000); i++, data++)
	{
#         ifndef SIMULATE_SEDAC
          outport(dat, *data);                        // load data
	  outport(cmd,addr);                          // load CMDreg
	  while (!(inport(csr) & _RBE) && cnt--);     // poll RBE bit
	  if (cnt <= 0 || inport(cmd)) { initSedac(); return(i+1); }
#         else
          sedacSimulationBuffer[ln] = *data;
#         endif
        }
	return 0;
      default:
        return -1;
  }
}

NodalError F_crates(real *value,int rwflag,u16 ln)
{
  char Buffer[81];
  u16 data;
  int cc,i,n;

  if (ln < 1 || ln > SEDAC) return no_such_line;

  switch (rwflag)
  {
    case  1:
      sprintf(Buffer,">line %d has crates :",ln); nprintf(Buffer);
      for (i=0,n=0,Buffer[0]=0; i<32; i++) if (!rsedac(ln,i,252,&data))
      {
        sprintf(&Buffer[strlen(Buffer)],"  %2d",i);
	if ((++n%10) == 0)
	{
	  if ((cc=nprintf(Buffer)) != 0) return cc;
	  Buffer[0] = 0;
	}
      }
      if (Buffer[0]) if ((cc=nprintf(Buffer)) != 0) return cc;
      *value = (double)n;
      return 0;
      default: return illegal_read_write;
  }
}

NodalError F_addr(real *value,int rwflag,u16 ln,u16 crate)
{
  char Buffer[81];
  u16 data;
  int cc,i,n;

  if (ln < 1 || ln > SEDAC) return no_such_line;

  switch (rwflag)
  {
    case  1:
      sprintf(Buffer,"line %d crate %d has subaddresses :",ln,crate);
      nprintf(Buffer);
      for (i=0,n=0,Buffer[0]=0;i<252;i+=16) if (!rsedac(ln,crate,i,&data))
      {
        sprintf(&Buffer[strlen(Buffer)],"  %2d",i);
	if ((++n%10) == 0)
	{
	  if ((cc=nprintf(Buffer)) != 0) return cc;
	  Buffer[0] = 0;
	}
      }
      if (Buffer[0]) if ((cc=nprintf(Buffer)) != 0) return cc;
      *value = (double) n;
      return 0;
    default: return illegal_read_write;
  }
}

NodalError F_sedac(real *value,int rwflag,u16 ln,u16 crate,u16 subadd)
{
	char Buffer[81];
	u16 data;
	int i;

	if (ln < 1 || ln > SEDAC)
	{
		nprintf("line %d not configured");
		sprintf(Buffer,"configured SEDAC lines:",ln); nprintf(Buffer);
		for (i=0,Buffer[0]=0; i<SEDAC; i++)
			sprintf(&Buffer[strlen(Buffer)],"  %d",i+1);
		nprintf(Buffer);
		return out_of_range;
	}

	switch (rwflag)
	{
		case -1: if (wsedac(ln,crate,subadd,(u16)*value)) return sedac_error;
						 else return 0;
		case  1: if (rsedac(ln,crate,subadd,&data)) return sedac_error;
						 else
						 {
							 *value = (double) data;
							 return 0;
						 }
		default: return illegal_read_write;
	}
}

#endif  // SEDAC

#ifdef BORLAND
char *_strdate(char *buf)
{
	struct date d;

	getdate(&d);
	sprintf(buf,"%02d/%02d/%02d",d.da_mon,d.da_day,(d.da_year%100));
	buf[8] = 0;
	return buf;
}
char *_strtime(char *buf)
{
	struct time t;

	gettime(&t);
	sprintf(buf,"%02d:%02d:%02d",t.ti_hour,t.ti_min,t.ti_sec);
	buf[8] = 0;
	return buf;
}
#endif  // BORLAND

// NodalDelay:
void NodalDelay(int sec)
{
  time_t t0 = time(NULL);
  int dly;
  for (dly=sec; dly > 0; dly = t0+sec-time(NULL))
  {
  	dorpc();
	  if (nConn) doConsumer();
    netWatchDog();
  }
  return;
}
/* QUIT command */
N_quit(nodal_string *str)
{
  ExitEQPModules();
#ifdef WATCHDOG
	if (watchdog) disableWatchDog();
#endif
#ifdef FECLOG
  feclog("stopped");
#endif
  closeClients();
  quitServer();
  printf("net kernel removed (returning to DOS)\n>");

  return -6;
}
NodalError initNodal(void)
{
#ifdef PASSFILE
  FILE *fp;
#endif
#ifdef NGLOBALS
  int i;
#endif

password[0] = '\0';
#ifdef PASSFILE
  if ((fp=fopen(PASSFILE,"r")) != NULL)
  {
    fscanf(fp,"%s\n",password);
    strupr(password);
    fclose(fp);
  }
#endif
  fcloseall();
  signal(SIGFPE,SIG_IGN);                    // catch Floating domain errors
  harderr(NetHandler);                          // start error handler
#ifdef FECLOG
  logAppend();
  feclog("started");
#endif
  F_settime();
#ifdef BROADCAST
  BroadcastServer = TRUE;
#endif
  if (initRPC()) nprintf(">RPC server failed to initialize");
  if (initSAPWatchDog()) nprintf(">SAP Watchdog failed to initialize");
#ifdef SEDAC
  initSedac();
#endif
#ifdef WATCHDOG
  watchdogBaseAddress = WATCHDOG;
  initWatchDog();
  watchdog = 1;
#endif

  doinit();                     // required function for NODAL

#ifdef TASK
  AssignDotask(dotask);         // for backward compatibility
#endif
#ifdef EXIT
  AssignExittask(doexit);       // for backward compatibility
#endif

#ifdef NGLOBALS
  for (i=0; i<nglobals; i++)    // get all globals upon start up
  {
    NGlobalList[i]->time = time(NULL);
    if (RPC((char *)NGlobalList[i]->data,GLOBALSRV,1,NGlobalList[i]->keyword,
            NGlobalList[i]->size,NGlobalList[i]->format,
            NGlobalList[i]->size*getFormatSize(NGlobalList[i]->format)))
      NGlobalList[i]->time = NULL;
  }
#endif
  return 0;
}
