#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stddef.h>
#include <conio.h>
#include <dos.h>
#include <alloc.h>
#include <malloc.h>
#include "prolog.h"
#include "rpcdefs.h"
#include "sedac.h"
#include "errors.h"
#include "doscmd.h"

int nSedacLines = 0;
int SedacSimulate = 0;
u16 SedacPortAddress = SEDAC_PORTADDR;

u16 sedacSimulationBuffer[4];
u16 sedacSimulationCount[4] = {0,0,0,0};
SEDAC_LINE line[4];

void initSedac(void)
{
  int i;

  RegisterUserCommand("nlines",NULL,&nSedacLines,NULL,CA_READ);
  RegisterUserCommand("sedac",cmdReadSedac,NULL,NULL,CA_READ);
  RegisterUserCommand("sedac",cmdWriteSedac,NULL,NULL,CA_WRITE);
  RegisterUserCommand("crates",cmdCrates,NULL,NULL,CA_READ);
  RegisterUserCommand("addr",cmdAddr,NULL,NULL,CA_READ);

  if (SedacSimulate) return;
  for (i=0; i<nSedacLines; i++)   // look for up to # SEDAC lines
  {
    printf("Initializing SEDAC line %d at base address %x\n>",i,SedacPortAddress|(i*16));
    line[i].csr = SedacPortAddress|(i*16);
    line[i].dat = SedacPortAddress|(i*16+0x02);
    line[i].cmd = SedacPortAddress|(i*16+0x04);
    line[i].igr = SedacPortAddress|(i*16+0x06);
    outport(line[i].csr, 0x1); delay(5);
    outport(line[i].csr, 0x0);
    outportb(0x20,0x20);
    inport(line[i].igr);
  }
  return;
}

// ============  write routine  ===================================

int wsedac(u16 ln,u16 crate,u16 subadd,u16 data)
{
  int address, cnt = 2000, retry = 0, error = 0;

  if (SedacSimulate)
  {
    sedacSimulationBuffer[ln] = data;
    return 0;
  }

  if (ln<1 || ln>nSedacLines) return -1;
  ln--;
  address = 0x4000 | (crate << 8) | subadd;
  do
  {
    outport(line[ln].dat,data);
    outport(line[ln].cmd,address);
    while (!(inport(line[ln].csr) & _RBE) && cnt--);
    if (cnt <= 0) { initSedac(); return -1; }
  } while ( (error=inport(line[ln].cmd)) != 0 && retry++<1);
  return error;
}

// ================  read routine  ===================================

int rsedac(u16 ln,u16 crate,u16 subadd,u16 far *data)
{
  int address, cnt = 2000, retry = 0, error = 0;

  if (SedacSimulate)
  {
    *data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
    return 0;
  }

  if (ln<1 || ln>nSedacLines) return -1;
  ln--;
  address = 0x8000 | (crate << 8) | subadd;
  do
  {
    outport(line[ln].cmd, address);
    while (!(inport(line[ln].csr) & _RBE) && cnt--);
    if (cnt <= 0) { initSedac(); return -1; }
    *data = inport(line[ln].dat);
  } while ( (error=inport(line[ln].cmd)) != 0 && retry++<1);
  return error;
}
int rsedacAsync(u16 ln,u16 crate,u16 subadd)
{
  int address;

  if (SedacSimulate) return 0;

  if (ln<1 || ln>nSedacLines) return -1;
  ln--;
  address = 0x8000 | (crate << 8) | subadd;
  outport(line[ln].cmd,address);
  return 0;
}
int rsedacPoll(u16 ln,u16 *data)
{
  if (SedacSimulate)
  {
    *data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
    return 0;
  }
  while (!(inport(line[ln].csr) & _RBE));
  *data = inport(line[ln].dat);
  return inport(line[ln].cmd);
}

// ============  Block routines ==========================================

void sedac4(SedacInfo *sedacList, int RWF, int num, int (*fcn)(int))
// reads/writes 4 SEDAC lines Round Robin (with service function if present)
{
  int i, l, cnt;

  switch( RWF )
  {
    case READ:
      for (i=0; i<num; i++)
      {
        for (l=0; !SedacSimulate && l<nSedacLines; l++)   // SEDAC lines out
	{
          outport(line[l].cmd,0x8000|sedacList[l].csa[i]);// load CMDreg
        }
	if (fcn) fcn(i);                                  // service function
	for (l=0; l<nSedacLines && (cnt=2000)!= 0; l++)   // SEDAC line in
	{
          if (SedacSimulate)
          {
            sedacList[l].data[i] = sedacSimulationBuffer[l] +
	                           sedacSimulationCount[l]++;
            sedacList[l].error[i] = cnt-2000;
          }
          else
          {
            while (!(inport(line[l].csr) & _RBE) && cnt--); // poll RBE bit
	    sedacList[l].data[i]  = inport(line[l].dat);    // read data
	    sedacList[l].error[i] = inport(line[l].cmd);    // error table
            if (cnt <= 0) { sedacList[l].error[i] = -1; initSedac(); }
          }
        }
      }
      if (fcn) fcn(i);                                    // i = num
      return;
    case WRITE:
      for (i=0; i<num; i++)
      {
        for (l=0; !SedacSimulate && l<nSedacLines; l++)   // SEDAC lines out
	{
          outport(line[l].dat,sedacList[l].data[i]);      // load data
	  outport(line[l].cmd,0x4000|sedacList[l].csa[i]);// load CMDreg
        }
	if (fcn) fcn(i);                                  // service function
	for (l=0; l<nSedacLines && (cnt=2000)!= 0; l++)   // SEDAC lines in
	{
          if (SedacSimulate)
          {
            sedacSimulationBuffer[l] = sedacList[l].data[i];
	    sedacList[l].error[i] = 0;
          }
          else
          {
            while (!(inport(line[l].csr) & _RBE) && cnt--); // poll RBE bit
	    sedacList[l].error[i] = inport(line[l].cmd);
	    if (cnt <= 0) { sedacList[l].error[i] = -1; initSedac(); }
          }
        }
      }
      if (fcn) fcn(i);                                    // i = num
      return;
  }
  return;
}

int sedac(int ln,u16 csa,int RWF,u16 far *data, int nwords)
{
  int i,cnt;
  u16 addr, cmd, csr, dat;

  if (ln<1 || ln>nSedacLines) return -1;
  ln--;
  cmd = line[ln].cmd; csr = line[ln].csr; dat = line[ln].dat;
  switch( RWF )
  {
    case READ:
      addr = 0x8000 | csa;
      for (i=0; i<nwords && (cnt=2000)!= 0; i++, data++)
      {
        if (SedacSimulate)
        {
          cmd = cmd; csr = csr;  cnt = cnt;           // avoid warnings
	  dat = dat, addr = addr;
          *data = sedacSimulationBuffer[ln] + sedacSimulationCount[ln]++;
        }
        else
        {
          outport(cmd,addr);                          // load CMDreg
	  while (!(inport(csr) & _RBE) && cnt--);     // poll RBE bit
          *data = inport(dat);                        // read data
          if (cnt <= 0 || inport(cmd)) { initSedac(); return(i+1); }
        }
      }
      return 0;
    case WRITE:
      addr = 0x4000 | csa;
      for (i=0; i<nwords && (cnt=2000)!= 0; i++, data++)
      {
        if (SedacSimulate)
        {
          sedacSimulationBuffer[ln] = *data;
        }
        else
        {
          outport(dat, *data);                        // load data
	  outport(cmd,addr);                          // load CMDreg
	  while (!(inport(csr) & _RBE) && cnt--);     // poll RBE bit
	  if (cnt <= 0 || inport(cmd)) { initSedac(); return(i+1); }
        }
      }
      return 0;
    default:
      return -1;
  }
}

int F_crates(int *ncrates,int rwflag,u16 ln)
{
  u16 data;
  int i,n;

  if (ln < 1 || ln > nSedacLines) return no_such_line;

  switch (rwflag)
  {
    case  1:
      printf("line %d has crates :\n>",ln);
      for (i=0,n=0; i<32; i++) if (!rsedac(ln,i,252,&data))
      {
        printf("  %2d",i);
	if ((++n%10) == 0) printf("\n>");
      }
      printf("\n>");
      *ncrates = n;
      return 0;
      default: return illegal_read_write;
  }
}

int F_addr(int *naddr,int rwflag,u16 ln,u16 crate)
{
  u16 data;
  int i,n;

  if (ln < 1 || ln > nSedacLines) return no_such_line;
  switch (rwflag)
  {
    case  1:
      printf("line %d crate %d has subaddresses :\n>",ln,crate);
      for (i=0,n=0;i<252;i+=16) if (!rsedac(ln,crate,i,&data))
      {
        printf("  %2d",i);
	if ((++n%10) == 0) printf("\n>");
      }
      printf("\n>");
      *naddr = n;
      return 0;
    default: return illegal_read_write;
  }
}
int cmdReadSedac(int ln,int cr,int sa,int val)
{
  UINT16 data;
  int cc;
  char str[128];

  if (nSedacLines)
  {
    if ((cc=rsedac(ln,cr,sa,&data)) != 0) sprintf(str,"SEDAC error");
    else sprintf(str,"lin %d cra %d bsa %d : %d (0x%x)\n>",ln,cr,sa,data,data);
    ttyoutput(str);
    return cc;
  }
  return -1;
}
int cmdWriteSedac(int ln,int cr,int sa,int val)
{
  UINT16 data = val;
  int cc=0;
  char str[128];

  if (nSedacLines)
  {
    if ((cc=wsedac(ln,cr,sa,data)) != 0) sprintf(str,"SEDAC error");
    else sprintf(str,"lin %d cra %d bsa %d wrote : %d (0x%x)\n>",ln,cr,sa,data,data);
    ttyoutput(str);
    return cc;
  }
  return -1;
}
int cmdCrates(int ln,int cr,int sa,int access)
{
  UINT16 data;
  int cc,n=0;
  char str[128];

  if (nSedacLines)
  {
    if ((cc=F_crates(&n,1,ln)) != 0) sprintf(str,"line %d : %s",ln,erlst[cc]);
    else sprintf(str,"line %d : %d crates\n>",ln,n);
    ttyoutput(str);
    return n;
  }
}
int cmdAddr(int ln,int cr,int sa,int access)
{
  UINT16 data;
  int cc,n=0;
  char str[128];

  if (nSedacLines)
  {
    if ((cc=F_addr(&n,1,ln,cr)) != 0) sprintf(str,"lin %d cra %d : %s",ln,cr,erlst[cc]);
    else sprintf(str,"lin %d cra %d : %d sub-addresses\n>",ln,cr,n);
    ttyoutput(str);
    return n;
  }
}
