#include "cprolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "errors.h"
#include "rpctypes.h"
#include "database.h"
#include "brdcast.h"
#ifdef MSDOS
# include "doscmd.h"
#endif

#define ccerr(i) { cc = i; goto err; }
extern int NGdebug;
struct GlobalsStruct 
{
  char  keyword[16];         /* broadcast keyword */
  short size;                /* number of elements in broadcast array */
  short format;              /* broadcast data format */
  void  *data;
  int   id;
  struct GlobalsStruct *next;  
} *globals = NULL;
int flagid = 0;
int initGlobalsList(void)
{
  int cc=0,i,fmt,done;
  int key_col,fmt_col,siz_col;
  FILE *fp;
  char str[256],hdr[256],scratch[32];
  struct GlobalsStruct *gs;
  
  if ((fp=fopen("nglobals.csv","r")) == NULL) ccerr(no_such_file);
  for (done=0,i=0; fgets(str,255,fp) != NULL;)
  {
    if (strchr("%;#\n",str[0])) continue; // comment or line feed
    if (!done)
    {
      strcpy(hdr,str);
      if ((key_col=findcol(hdr,"KEYWORD")) < 0) ccerr(no_such_line);
      if ((fmt_col=findcol(hdr,"FORMAT")) < 0) ccerr(no_such_line);
      if ((siz_col=findcol(hdr,"SIZE")) < 0) ccerr(no_such_line);
      done = 1;
      continue;
    }
    if ((gs=(struct GlobalsStruct *)calloc(1,sizeof(struct GlobalsStruct))) == NULL)
      ccerr(out_of_client_memory);
    strncpy(gs->keyword,colptr(key_col,str),16);
    strncpy(scratch,colptr(fmt_col,str),16); strupr(scratch);
    if ((fmt=getFormatType(scratch)) == CF_NULL) fmt = atoi(scratch);
    gs->format = fmt;
    strncpy(scratch,colptr(siz_col,str),16);
    gs->size = atoi(scratch);
    if ((gs->data=calloc(gs->size,fmtsizeof(fmt))) == NULL) ccerr(out_of_client_memory);
    gs->next = globals;
    globals  = gs;
    i++;
  }
  fclose(fp);
err:
  return cc ? -cc : i;
}
void globalsCallback(int id,int cc)
{
  int i;
  struct GlobalsStruct *gs;
  double val;

  if (!NGdebug) return;
  for (gs=globals; gs; gs=gs->next)
  {
    if (gs->id != id) continue;
    printf("%s :",gs->keyword);
    for (i=0; i<gs->size; i++)
    {
      switch (gs->format)
      {
        case CF_BYTE: printf(" %d",((BYTE *)gs->data)[i]); break;
        case CF_SHORT: printf(" %d",((short *)gs->data)[i]); break;
        case CF_LONG: printf(" %d",((long *)gs->data)[i]); break;
        case CF_FLOAT: printf(" %f",((float *)gs->data)[i]); break;
        case CF_DOUBLE: printf(" %f",((double *)gs->data)[i]); break;
        case CF_CHAR8: printf(" %s",((NAME8 *)gs->data)[i].name); break;
        case CF_CHAR16: printf(" %s",((NAME16 *)gs->data)[i].name); break;
        case CF_CHAR32: printf(" %s",((NAME32 *)gs->data)[i].name); break;
      }
    }
    printf("\n>");
  }
  if (id == flagid) printf("\n>");
}
void postSystemInit(void)
{
  int i,n;
  DTYPE dout,din;
  struct GlobalsStruct *gs;

  if ((n=initGlobalsList()) < 0)
  {
    printf("initGlobalsList: %s\n>",erlst[-n]);
    exit(1);
  }

  for (gs=globals; gs; gs=gs->next)
  {
    dout.dFormat = gs->format;
    dout.dArrayLength = gs->size;
    dout.data.vptr = gs->data;
    gs->id = recvNetGlobal(gs->keyword,&dout,globalsCallback);
    if (gs->id < 0)
    {
      printf("NETGLOBAL %s: %s\n>",gs->keyword,erlst[-gs->id]);
    }
    else flagid = gs->id;
  }
}
int main(int argc, char *argv[])
{
  int cc;
  switch (argc)                           /* initial debug level */
  {           /* on UNIX or DOS stations: set debug = 1 (2,or 3) */
    case 2:              /* will dynamically set the debug level */
      NGdebug = atoi(argv[1]);
      break;
    default:
      NGdebug = 0;                              /* default level */
  }
  printf("Server running (debug level %d)\n",NGdebug);

  BroadcastDaemon = TRUE;

  /* call specific initialization: */
  postSystemInit();

  for(;;)
  {
    SystemCycle(TRUE);
  }
  return 0;
}

