#include "cprolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef SVR5
#include <sys/wait.h>
#endif
#include "errors.h"
#include "rpctypes.h"
#include "nextmex.h"
#include "hera.h"
#include "cmfldata.h"
#include "hbtarget.h"
#include "hmagdata.h"

/*
	Please use SystemDelay() in place of sleep() on non SRV5 UNIX machines
	or on VMS machines

	LinkNextMex(keyword,poll,callback,mode) is the suggested API call.

	keyword above is an address of a NEXTMEX KEY (see nextmex.h and heradata.h)
	poll is the requested polling rate in milliseconds
	callback is a "callback" function and must have the prototype:

	void function(char *keyword,int cc);

	mode is the data acquisition mode.
	Please note the possible acquistion modes:

		CM_CANCEL      cancels a data link
		CM_SINGLE      requests a single (asynchronous) transaction
		CM_REFRESH     requests that data be sent only upon change
									 (and at 1 minute intervals - hearbeat)
		CM_POLL        requests that data be sent at the given polling rate
		CM_REGISTER    requests that the link be registered but not activated
									 (normally for internal use only)

*/

extern char *erlst[];
extern int NGdebug;
#define callout(name,cc) {\
  if (cc) {\
     printf("LINK %s error: %s\n>",name,erlst[cc]);\
     return;\
  }\
}
#define callcheck(name,key) (strcmp(name,((NETKEY *)key)->Property) ? 0 : 1)
void showCMFLTrace(char *keyword,int cc)
{
  int i;
  callout(keyword,cc);
/* this is just an example, print this out at your discretion... */
  printf("CMFL bucket %d fwhm %d area %d\n",
	 cmfl.buckt,
	 (unsigned)((cmfl.fwhmh<<16)&0xffff0000)|(cmfl.fwhml&0xffff),
	 (unsigned)((cmfl.areah<<16)&0xffff0000)|(cmfl.areal&0xffff));
  for (i=0; i< 256; i++) printf((i+1)%20 ? "%d " : "%d\n",cmfl.trace[i]);
  printf("\n%d (%d)",(time_t)((cmfl.timeh<<16)&0xffff0000)|(cmfl.timel&0xffff),HPCMFL.TimeStamp);
  printf("\n");
}
void showKeyword(char *keyword,int cc)
{
	int i;
	callout(keyword,cc);
	if (callcheck(keyword,&HPENERGY))
	{
		printf("Energy callback: %f Gev\n",*(float *)HPENERGY.data);
		return;
	}
	if (callcheck(keyword,&HEMAGEN))
	{
		printf("HERA E Energy callback: %f Gev\n",*(float *)HEMAGEN.data);
		return;
	}
	if (callcheck(keyword,&HPColPos))
	{
		printf("Collimator callback:\n");
		for (i=0; i<HPColPos.no_el; i++) printf("%g ",((float *)HPColPos.data)[i]);
		printf("\n");
    return;
  }
  if (callcheck(keyword,&HPbpmX))
  {
    printf("ZEUS X-BPM callback:\n");
    for (i=0; i<HPbpmX.no_el; i++) printf("%g ",((float *)HPbpmX.data)[i]);
    printf("\n");
    return;
  }
  if (callcheck(keyword,&HPbpmY))
  {
    printf("ZEUS Y-BPM callback:\n");
    for (i=0; i<HPbpmY.no_el; i++) printf("%g ",((float *)HPbpmY.data)[i]);
    printf("\n");
    return;
  }
  if (callcheck(keyword,&HEbpmXHE))
  {
    printf("HERMES X-BPM callback:\n");
    for (i=0; i<HEbpmXHE.no_el; i++) printf("%g ",((float *)HEbpmXHE.data)[i]);
    printf("\n");
    return;
  }
  if (callcheck(keyword,&HEbpmYHE))
  {
    printf("HERMES Y-BPM callback:\n");
    for (i=0; i<HEbpmYHE.no_el; i++) printf("%g ",((float *)HEbpmYHE.data)[i]);
    printf("\n");
    return;
  }
  if (callcheck(keyword,&HPRUNNUM))
  {
    printf("HERA P RUN NUMBER: %d\n",*(short *)HPRUNNUM.data);
    return;
  }
  if (callcheck(keyword,&HB_Target))
  {
    printf("HERA B Target:\n");
    for (i=0; i<HB_Target.no_el; i++) printf("%g ",((float *)HB_Target.data)[i]);
    printf("\n");
    return;
  }
  if (callcheck(keyword,&HERATXT1))
  {
    printf("HERA TEXT 1: %s\n",(char *)HERATXT1.data);
    return;
  }
  if (callcheck(keyword,&HERATXT2))
  {
    printf("HERA TEXT 2: %s\n",(char *)HERATXT2.data);
    return;
  }
}
#define PKTR
int main(int argc, char *argv[])
{
  int i,cycle = 0;
  switch (argc)
  {
    case 2:
      NGdebug = atoi(argv[1]);
      break;
    default:
      NGdebug = 0;
  }
  SystemInit(FALSE);
/* from the PKTR SERVER: */        
#ifdef PKTR
	if ((i=LinkNextMex(&HPENERGY,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HPColPos,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HPbpmX,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HPbpmY,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HEMAGEN,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
#endif
/* from the MKI3 SERVER: */        
#ifdef MKI3
	if ((i=LinkNextMex(&HEbpmXHE,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HEbpmYHE,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HPRUNNUM,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HERATXT1,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
	if ((i=LinkNextMex(&HERATXT2,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
#endif
/* from HERA_B server */
#ifdef HERA_B
	if ((i=LinkNextMex(&HB_Target,1000,showKeyword,CM_REFRESH)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
#endif
/* from CMFL server */
#ifdef CMFL
	if ((i=LinkNextMex(&HPCMFL,1000,showCMFLTrace,CM_POLL)) != 0) printf("Link error: %d (%s)\n>",i,erlst[i]);
#endif
  for (;;)
  {
    SystemCycle(TRUE);
    if (NGdebug) printf("loop cycle: %d (5-second intervals)\n",cycle++);
    SystemDelay(5000);
  }

  return 0;
}
