#include "cprolog.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "errors.h"
#include "rpctypes.h"
#include "swaplib.h"  /* for CF_STRUCT example */

/*
   rpccln.c is an example client-side program which can be used to
   test the sinsim.c server program.  Use the export name you used
   for your server as DEVNAME below.
 */

#define DEVNAME "SINE"

/*
   The code section below illustrates how one registers a structure
   to be exported.

   Note the use of the short "reserved" in the structure below.
   Although not necessary, it is strongly recommended that you
   assume structures to be alligned on 4-byte boundaries and fill
   in any gaps with reserved or dummy tags.  This will make life
   much easier in Visual Basic 5.0 (32-bit) where structures are
   by definition alligned on 4-byte boundaries.  If you don't follow
   this suggestion, then you will have to (in Visual basic 5.0) read
   your structure into a byte array and parse it out into your
   desired VB structure.

   Please pay attention to all comments
*/

typedef struct
{
  float a[3];
  long  b[2];
  short c[1];
  short reserved;
  char  d[32];
} Test1Struct;
#define Test1StructSize ((sizeof(float)*3) +\
                         (sizeof(long)*2)  +\
                         (sizeof(short)*1) +\
                         (sizeof(short)*1) +\
                         32)

#define quit(i) { printf("Register struct: out of memory\n"); exit(i); }
void registerStructs(void)
{
  /* this must follow the order of the structure explicitly! */
  if (fillstruct("TEST1",OFFSETIN(Test1Struct,a),3,CF_FLOAT)) quit(1);
  if (fillstruct("TEST1",OFFSETIN(Test1Struct,b),2,CF_LONG)) quit(1);
  if (fillstruct("TEST1",OFFSETIN(Test1Struct,c),1,CF_SHORT)) quit(1);
  if (fillstruct("TEST1",OFFSETIN(Test1Struct,reserved),1,CF_SHORT)) quit(1);
  if (fillstruct("TEST1",OFFSETIN(Test1Struct,d),32,CF_TEXT)) quit(1);
  /* terminate the structure definition like this! */
  if (fillstruct("TEST1",sizeof(Test1Struct),1,CF_NULL)) quit(1);
}

Test1Struct MyTest;

void fillStructs(void)
{
  int i;

  for (i=0; i<3; i++)
  {
    MyTest.a[i] = i+1;
    if (i<2) MyTest.b[i] = i+1;
  }
  MyTest.c[0] = 1;
  strncpy(MyTest.d,"This is a test",32);
}
/*
   End of structure sample routines.  Compare this to
   property STRUCTTEST below.
*/


int testID1 = -1,testID2 = -1,strctID = -1,GroupID = -1;
float *fdat=NULL,*sdat;
float *fdat1 = NULL, *fdat2 = NULL;
Test1Struct *strcdat = NULL;
int NUMSTRUCTS = 2;
void showstruct(int id, int cc)
{
  int i;
  if (cc)
  {
    printf("link %d : %s\n>",id,RPCLastErrorString);
    return;
  }
  printf("structure TEST1\n");
  for (i=0; i<NUMSTRUCTS; i++)
  {
    printf("\ta = %g %g %g\n\tb = %d %d\n\tc = %d\n\td = %s\n\n",
           strcdat[i].a[0],strcdat[i].a[1],strcdat[i].a[2],
           strcdat[i].b[0],strcdat[i].b[1],strcdat[i].c[0],
           strcdat[i].d );
  }
}
#define NUM 300
void showsine(int id,int cc)
{
  int i;
  if (cc)
  {
    printf("link %d : %s\n>",id,RPCLastErrorString);
    return;
  }
  for (i=0; i<NUM; i++) printf("%f ",sdat[i]);
  printf("\n>");
}

#define PATTERN1 212
#define PATTERN2 1

void showlink(int id, int cc)
{
  int i;
  if (cc)
  {
    printf("link %d : %s\n>",id,RPCLastErrorString);
    return;
  }
  if (id == GroupID) printf("group callback ...\n>");
  if (id == testID1 || id == GroupID)
  {
    for (i=0; i<NUM; i++) if (fdat1[i] != PATTERN1+i)
      printf("ECHO failed: %g not equal to %g\n>",fdat1[i],(float)PATTERN1);
  }
  if (id == testID2 || id == GroupID)
  {
    for (i=0; i<NUM; i++) if (fdat2[i] != PATTERN2+i)
      printf("ECHO failed: %g not equal to %g\n>",fdat2[i],(float)PATTERN2);
  }
}
short NGenergyID = -1;
UINT16 hpenergy = -1;
void showenergy(int id,int cc)
{
  static cnt = 0;
  if (cc)
  {
    printf("global %d : comp code %d %s\n>",id,cc,RPCLastErrorString);
    if (cnt++) hpenergy = -1;
    cnt = 0;
    return;
  }
  printf("recv global energy: %d\n",hpenergy);
}
void preSystemInit(void)
{
}
void postSystemInit(void)
{
  static short i;
  static float dat[NUM];
  char devname[32];
  DTYPE dout,din;

  registerStructs();

  dout.dFormat = CF_SHORT;
  dout.dArrayLength = 1;
  dout.data.sptr = &hpenergy;
  NGenergyID = recvNetGlobal("HPENERGY",&dout,showenergy);
  if (NGenergyID < 0)
  {
    printf("NETGLOBAL : %s\n>",erlst[-NGenergyID]);
  }
  sprintf(devname,"%s/#1",DEVNAME);
  dout.dFormat = CF_FLOAT;
  dout.dArrayLength = NUM;
  dout.data.fptr = fdat1;
  for (i=0; i<NUM; i++) dat[i] = PATTERN1+i;
  din.dFormat = CF_FLOAT;
  din.dArrayLength = NUM;
  din.data.fptr = dat;
  testID1 = AttachLink(devname,"ECHO",&dout,&din,CA_READ,1000,showlink,CM_POLL|CM_GROUPED);
  if (testID1 < 0)
  {
    printf("ATTACH : %s\n>",erlst[-testID1]);
  }
  fdat1 = dout.data.fptr;

  dout.dFormat = CF_FLOAT;
  dout.dArrayLength = NUM;
  dout.data.fptr = fdat2;
  for (i=0; i<NUM; i++) dat[i] = PATTERN2+i;
  din.dFormat = CF_FLOAT;
  din.dArrayLength = NUM;
  din.data.fptr = dat;
  testID2 = AttachLink(devname,"ECHO",&dout,&din,CA_READ,1000,showlink,CM_POLL|CM_GROUPED);
  if (testID2 < 0)
  {
    printf("ATTACH : %s\n>",erlst[-testID2]);
  }
  fdat2 = dout.data.fptr;
  GroupID = (int)showlink;

  dout.dFormat = CF_STRUCT;
  dout.dArrayLength = NUMSTRUCTS;
  dout.data.vptr = strcdat;
  strcpy(dout.dTag,"TEST1");
  din.dFormat = CF_NULL;
  din.dArrayLength = 0;
  din.data.vptr = NULL;
  strctID = AttachLink(devname,"STRUCTTEST",&dout,NULL,CA_READ,1000,showstruct,CM_POLL);
  if (strctID < 0)
  {
    printf("ATTACH : %s\n>",erlst[-strctID]);
  }
  strcdat = (Test1Struct *)dout.data.vptr;

}
