package TINE;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;

public class DefaultTInitializerFactory extends TInitializerFactory {
    private static final String ENS="131.169.120.46";
    private static final int SRV_PORT=9003;
    private static final int TCP_PORT=8503;
    private static final String GCAST="238.1.1.0";
    private static final int GCAST_PORT=8004;
    private static final String MCAST="238.1.1.1";
    private static final int MCAST_PORT=9503;

    private class DefaultTInitializer implements TInitializer {
        private String tineHome = null;
        private String addrFileLocation = null;
        private String eqpFileLocation = null;
        private String cshostsFileLocation = null;

        public DefaultTInitializer() {
            addrFileLocation = getTineHome() + "/fecaddr.csv";
            eqpFileLocation = getTineHome() + "/eqpdbase.csv";
            cshostsFileLocation = getTineHome() + "/cshosts.csv";
        }

        private String getTineHome() {
            String thome;
            if (tineHome == null) {
                if ((thome = System.getProperty("tine.home")) != null) {
                    tineHome = new String(thome);
                } else {
                    tineHome = new String(System.getProperty("java.home") + "/tine/database");
                }
            }
            return tineHome;
        }

        /**
         * @see TINE.TInitializer#getAddressResource()
         */
        public BufferedReader getAddressResource() {
            try {
                return new BufferedReader(
                    new InputStreamReader(new FileInputStream(addrFileLocation)));
            } catch (Exception e) {
                System.out.println("Could not open configuration file: " + e);
                return null;
            }
        }

        /**
         * @see TINE.TInitializer#getENSAddress()
         */
        public String getENSAddress() {
            return ENS;
        }

        /**
         * @see TINE.TInitializer#getEquipmentResource()
         */
        public BufferedReader getEquipmentResource() {
            try {
                return new BufferedReader(
                    new InputStreamReader(new FileInputStream(eqpFileLocation)));
            } catch (Exception e) {
                System.out.println("Could not open configuration file: " + e);
                return null;
            }
        }

        /**
         * @see TINE.TInitializer#getHostsResource()
         */
        public BufferedReader getHostsResource() {
            try {
                return new BufferedReader(
                    new InputStreamReader(new FileInputStream(cshostsFileLocation)));
            } catch (Exception e) {
                System.out.println("Could not open configuration file: " + e);
                return null;
            }
        }

        /**
         * @see TINE.TInitializer#getGCastAddress()
         */
        public String getGCastAddress() {
            return GCAST;
        }

        /**
         * @see TINE.TInitializer#getGCastPort()
         */
        public int getGCastPort() {
            return GCAST_PORT;
        }

        /**
         * @see TINE.TInitializer#getMCastAddress()
         */
        public String getMCastAddress() {
            return MCAST;
        }

        /**
         * @see TINE.TInitializer#getMCastPort()
         */
        public int getMCastPort() {
            return MCAST_PORT;
        }

        /**
         * @see TINE.TInitializer#getSrvPort()
         */
        public int getSrvPort() {
            return SRV_PORT;
        }

        /**
         * @see TINE.TInitializer#getTCPPort()
         */
        public int getTCPPort() {
            return TCP_PORT;
        }

    }
    /**
     * @see TINE.TInitializerFactory#getInitializer()
     */
    public TInitializer getInitializer() {
        return new DefaultTInitializer();
    }

}
