//: TCallback.java
package TINE;

/**
 * @brief Interface to a TLink callback class for use in asynchronous links
 * 
 * This class provides an inteface to a TCallback object whose callback 
 * method will be called which the link state of an asynchronous link has 
 * changed, either due to incoming data or non-zero status code.
 *  
 * \b Example:
 * 
 * \include eg_attach1.java
 */
public interface TCallback
{
  /**
   * @brief The method which will be called when new data arrive.
   * 
   * @param LinkIndex is either the index returned when calling the 
   * TLink.attach(short mode,TCallback f) or 
   * TLink.attach(short mode,TCallback f,int pollrate) methods, or the
   * index supplied when calling the 
   * TLink.attach(short mode,TCallback f,int pollrate,int notificationId)
   * method.
   * @param LinkStatus is the current link status code.  The caller should
   * always inspect this value to determine the validity of the data.
   * 
   * @see TINE.TLink#attach
   */
  public void callback(int LinkIndex,int LinkStatus);    
}