package TINE;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Vector;


/**
 * @brief tine Data Type object
 * 
 * All data	transfered client <-> server must be contained in a
 * TDataType object.  Furthermore all constructors of TDataType
 * take	an array of	tine data types	as input argument, i.e. a 
 * single element should be passed as an array of length 1
 */
public final class TDataType implements Cloneable
{
  public static int RPCERR_SIZE = 32;
  private static int dBufferSegSize	= 1472;
  public int	 dCompletionLength;
  protected int	 dArrayLength;
  protected short dFormat;
  protected Vector dVector;
  private String arrayDelimiter = new String("\n");
  private String dTag = new String(new StringBuffer(8));
  private int	 dSizeInBytes;
  private ByteArrayOutputStream	dBuffer;
  private ByteArrayOutputStream[] dBuffers;
  int debugLevel = 0;
  int timestamp;
  int timestampMSEC;
  int numblks;
  int blknum;
  int blkid;
  int blkcnt;
  int mtu;
  int blksin;
  int bytesin;
  NAME8[] hName8;
  NAME16[] hName16;
  NAME16I[]	hName16I;
  NAME16FI[] hName16FI;
  NAME32[] hName32;
  NAME32I[]	hName32I;
  NAME48[] hName48;
  NAME48I[] hName48I;
  NAME64[] hName64;
  NAME64I[] hName64I;
  byte[] hBytes;
  short[] hShorts;
  int[]	hLongs;
  float[] hFloats;
  double[] hDoubles;
  FLTINT[] hFLTINT;
  FLTFLT[] hFLTFLT;
  DBLINT[] hDBLINT;
  DBLDBL[] hDBLDBL;
  LNGINT[] hLNGINT;
  TDS[] hTDS;
  FLTFLTINT[] hFLTFLTINT;
  FLTINTINT[] hFLTINTINT;
  USTRING[] hUSTRING;
  SPECTRUM hSPECTRUM;
  INTINTINTINT[] hINTINTINTINT;
  INTFLTFLTFLT[] hINTFLTFLTFLT;
  INTINTFLTFLT[] hINTINTFLTFLT;
  StringBuffer hStringBuffer;
  public Object clone()
  {
    Object o = null;
    try
    {
      o = super.clone();
    }
    catch (CloneNotSupportedException e)
    {
      System.err.println("TDataType cannot clone");
    }
    return o;
  }
  void initCounters()
  {
	 mtu = (1472-TContract.hdrSizeInBytes-TSubscription.hdrSizeInBytes-Theader.hdrSizeInBytes);
   if (dFormat != TFormat.CF_NULL) mtu -= (mtu % TFormat.formatSizeOf(dFormat));
	 numblks = dSizeInBytes/mtu;
	 if (dSizeInBytes == 0 || (dSizeInBytes % mtu) > 0) numblks += 1;
   blkid = 1;
	 blknum	= 1;
	 blkcnt	= 0;
  }
/**
 * @brief Sets the delimiter used in a string representation of an array
 */
  public void setArrayDelimiter(String delimiter)
  {
    if (arrayDelimiter.compareTo(delimiter) == 0) return;
    arrayDelimiter = new String(delimiter);
  }
/**
 * @brief Get the delimiter used in a string representation of an array
 * 
 * @return the delimiter used in a string representation of an array
 */
  public String getArrayDelimiter()
  {
    return arrayDelimiter;
  }  
/**
 * @brief NULL element
 */
  public TDataType() 
  {
    dFormat = TFormat.CF_NULL;
    dArrayLength = 0;
    dSizeInBytes = 0;
    initCounters();     
  }
/**
 * @brief An unbound DataType object of the specified size and format.
 * 
 * @param dataSize is the array size of the given TINE format.
 * @param dataFormat is the tine data type of the object to create.
 * 
 * @note A call to getData() must be made in order to retrieve the incoming
 * data into the application array parameters.
 */
  public TDataType(int dataSize, short dataFormat) 
  {
    dFormat = dataFormat;
    switch (dFormat)
    {
      case  TFormat.CF_DEFAULT:
   	     dSizeInBytes = dataSize;
         dArrayLength = dataSize;
         blknum	= 1;
         blkcnt	= 0;
        return; // no allocation
      case TFormat.CF_DOUBLE:
         hDoubles = new double[dataSize];
         break;
      case TFormat.CF_FLOAT:
         hFloats = new float[dataSize];
         break;
      case TFormat.CF_SHORT:
         hShorts = new short[dataSize];
         break;
      case TFormat.CF_LONG:
         hLongs = new int[dataSize];
         break;
      case TFormat.CF_NAME16:
         hName16 = new NAME16[dataSize];
         break;
      case TFormat.CF_NAME32:
         hName32 = new NAME32[dataSize];
         break;
      case TFormat.CF_NAME48:
         hName48 = new NAME48[dataSize];
         break;
      case TFormat.CF_NAME64:
         hName64 = new NAME64[dataSize];
         break;
      case TFormat.CF_NAME8:
         hName8 = new NAME8[dataSize];
         break;
      case TFormat.CF_BYTE:
         hBytes = new byte[dataSize];
         break;
      case TFormat.CF_DBLINT:
         hDBLINT = new DBLINT[dataSize];
         break;
      case TFormat.CF_DBLDBL:
         hDBLDBL = new DBLDBL[dataSize];
         break;
      case TFormat.CF_FLTINT:
         hFLTINT = new FLTINT[dataSize];
         break;
      case TFormat.CF_LNGINT:
         hLNGINT = new LNGINT[dataSize];
         break;
      case TFormat.CF_NAME16I:
         hName16I = new NAME16I[dataSize];
         break;
      case TFormat.CF_NAME16FI:
         hName16FI = new NAME16FI[dataSize];
         break;
      case TFormat.CF_NAME32I:
         hName32I = new NAME32I[dataSize];
         break;
      case TFormat.CF_NAME48I:
         hName48I = new NAME48I[dataSize];
         break;
      case TFormat.CF_NAME64I:
         hName64I = new NAME64I[dataSize];
         break;
      case TFormat.CF_SPECTRUM:
         hSPECTRUM = new SPECTRUM();
         break;
      case TFormat.CF_TDS:
         hTDS = new TDS[dataSize];
         break;
      case TFormat.CF_TEXT:
         hText = new char[dataSize];
         break;
      case TFormat.CF_USTRING:
         hUSTRING = new USTRING[dataSize];
         break;
      case TFormat.CF_ADDRESS:
         hINTINTINTINT = new INTINTINTINT[dataSize];
         break;
      case TFormat.CF_POINT:
         hFLTFLT = new FLTFLT[dataSize];
         break;
      case TFormat.CF_FILTER:
         hINTFLTFLTFLT = new INTFLTFLTFLT[dataSize];
         break;
      case TFormat.CF_FLTFLTINT:
         hFLTFLTINT = new FLTFLTINT[dataSize];
         break;
      case TFormat.CF_FLTINTINT:
         hFLTINTINT = new FLTINTINT[dataSize];
         break;
      case TFormat.CF_FWINDOW:
         hINTINTFLTFLT = new INTINTFLTFLT[dataSize];
         break;
    }
    dArrayLength = dataSize;
    dSizeInBytes = dataSize*TFormat.formatSizeOf(dataFormat);
    initCounters();
    dBuffers = new ByteArrayOutputStream[numblks];
    for (int n=0,i=0; n<numblks; n++) 
    {
      dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
    }
  }  
/**
 * @brief Returns the associated data timestamp as traditional UNIX timestamp
 * (seconds since 1.1.1970).
 */
  public int getTimeStamp() { return timestamp;  }  
/**
 * @brief Returns the associated data timestamp as JAVA timestamp
 * (milliseconds since 1.1.1970).
 */
  public long getDataTimeStamp() { return (long)timestamp * 1000 + (long)timestampMSEC; }  
/**
 * @brief Returns the associated data completion length
 */
  public int getLastDataSize() { return dCompletionLength; }  
/**
 * @brief An array of NAME8 objects
 * 
 * @param data is a reference to the NAME8 objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME8[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME8;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME8();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName8 = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME16 objects
 *
 * This data type is frequently used to retrieve name lists.
 *  
 * @param data is a reference to the NAME16 objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME16[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME16;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME16();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName16 =	data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME16I objects
 *
 * This data type is frequently used to retrieve archived string
 * parameters (name + timestamp) or status string + code pairs, etc.
 *  
 * @param data is a reference to the NAME16I objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME16I[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME16I;
      dArrayLength = data.length;
      dSizeInBytes = data.length*20;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME16I();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/20; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName16I = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME16FI objects
 *
 * This data type is frequently used to retrieve triplets of device name
 * float value and status
 *  
 * @param data is a reference to the NAME16FI objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME16FI[] data)
  {
    try
    {
      dFormat = TFormat.CF_NAME16FI;
      dArrayLength = data.length;
      dSizeInBytes = data.length*24;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME16FI();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/24; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName16FI = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of NAME32 objects
 *
 * This data type is frequently used to retrieve name lists
 *  
 * @param data is a reference to the NAME32 objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME32[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME32;
      dArrayLength = data.length;
      dSizeInBytes = data.length*32;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME32();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/32; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName32 =	data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME32I objects
 *
 * This data type is frequently used to retrieve archived string
 * parameters (name + timestamp) or status string + code pairs, etc.
 *  
 * @param data is a reference to the NAME32I objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME32I[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME32I;
      dArrayLength = data.length;
      dSizeInBytes = data.length*36;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME32I();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/36; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName32I = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME48 objects
 *
 * This data type is frequently used to retrieve name lists
 *  
 * @param data is a reference to the NAME48 objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME48[] data)
  {
    try
    {
      dFormat =	TFormat.CF_NAME48;
      dArrayLength = data.length;
      dSizeInBytes = data.length*48;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME48();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/48; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName48 =	data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of NAME48I objects
 *
 * This data type is frequently used to retrieve archived string
 * parameters (name + timestamp) or status string + code pairs, etc.
 *  
 * @param data is a reference to the NAME48I objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME48I[] data)
  {
    try
    {
      dFormat = TFormat.CF_NAME32I;
      dArrayLength = data.length;
      dSizeInBytes = data.length*52;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME48I();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/52; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName48I = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of NAME64 objects
 *
 * This data type is frequently used to retrieve name lists
 *  
 * @param data is a reference to the NAME64 objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME64[] data)
  {
    try
    {
      dFormat = TFormat.CF_NAME64;
      dArrayLength = data.length;
      dSizeInBytes = data.length*64;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME64();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/64; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName64 = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of NAME64I objects
 *
 * This data type is frequently used to retrieve archived string
 * parameters (name + timestamp) or status string + code pairs, etc.
 *  
 * @param data is a reference to the NAME64I objects used to create and
 * bind to the TDataType object
 */
  public TDataType(NAME64I[] data)
  {
    try
    {
      dFormat = TFormat.CF_NAME32I;
      dArrayLength = data.length;
      dSizeInBytes = data.length*68;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new NAME64I();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/68; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hName64I = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of bytes
 *
 * @param data is a reference to the byte array used to create and
 * bind to the TDataType object
 */
  public TDataType(byte[] data)
  {
    try
    {
      dFormat =	TFormat.CF_BYTE;
      dArrayLength = data.length;
      dSizeInBytes = data.length;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu; i++) ds.writeByte(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
    
      hBytes = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of 'structs' with the given tag
 * or untagged structs if tag.length == 0
 *
 * User-defined structures are allowed in tine and not too difficult to
 * use.  It is strongly encouraged to make use of 'tagged' structures
 * whereby the server and client both register the structure contents with
 * the tine engine and apply a tag.  Only then can issues of 'byte-swapping'
 * and structure boundaries be handled correctly.  As java does not have 
 * structures, the associated class is responsible for providing a toStruct()
 * method which returns a byte stream containing the contents of the desired
 * structure. 
 *
 * @param data is a reference to the byte array stream containing the
 * 'struct' used to create and bind to the TDataType object.
 * @param tag is the structure tag associated with the 'struct.'
 */
  public TDataType(byte[] data, String tag)
  {
    dTag = tag;
    try
    {
      dFormat =	TFormat.CF_STRUCT;
      dArrayLength = data.length;
      dSizeInBytes = data.length;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu; i++) ds.writeByte(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hBytes = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of shorts (2-byte integers)
 *
 * This corresponds to tine format CF_SHORT
 *
 * @param data is a reference to the short array used to create and
 * bind to the TDataType object
 */
  public TDataType(short[] data)
  {
    try
    {
      dFormat =	TFormat.CF_SHORT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*2;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/2; i++) ds.writeShort(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,2);
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hShorts =	data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief A single short value 
 *
 * This method is only useful for quick creation of data 'down' objects 
 * (i.e. data sets sent to the server).  As there is no reference to the
 * argument passed, it cannot be used to bind returning data to the object
 * passed as the argument.
 *
 * @param data is a single short value used to create the TDataType object.
 * 
 */
  public TDataType(short data)
  {
    try
    {
      dFormat =	TFormat.CF_SHORT;
      dArrayLength = 1;
      dSizeInBytes = 2;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      dBuffers[0] =	new ByteArrayOutputStream(dBufferSegSize);
      DataOutputStream ds =	new DataOutputStream(dBuffers[0]);
      ds.writeShort(data);
      byte[] b = dBuffers[0].toByteArray();
      Swap.Bytes(b,2);
      dBuffers[0].reset();
      ds.write(b,0,b.length);
      ds.close();
      hShorts =	null;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of ints (4-byte integers)
 *
 * This corresponds to tine format CF_LONG
 *
 * @param data is a reference to the int array used to create and
 * bind to the TDataType object
 */
  public TDataType(int[] data)
  {
    try
    {
      dFormat =	TFormat.CF_LONG;
      dArrayLength = data.length;
      dSizeInBytes = data.length*4;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/4; i++) ds.writeInt(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hLongs = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of floats
 *
 * This corresponds to tine format CF_FLOAT
 *
 * @param data is a reference to the float array used to create and
 * bind to the TDataType object
 */
  public TDataType(float[] data)
  {
    try
    {
      dFormat =	TFormat.CF_FLOAT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*4;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks*3/2];  // NIOS has mtu = 1024
      for (int n=0,i=0;	n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for	(; i<data.length &&	i<(n+1)*mtu/4; i++)	ds.writeFloat(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        dBuffers[n].reset();
        ds.write(b,0,b.length);	  
        ds.close();
      }
      hFloats =	data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of FLTINTs (float + integer pairs)
 *
 * This data type is frequently used to retrieve archived 
 * parameters (float value + timestamp)
 *  
 * @param data is a reference to the FLTINT objects used to create and
 * bind to the TDataType object
 */
  public TDataType(FLTINT[] data)
  {
    try
    {
      dFormat =	TFormat.CF_FLTINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FLTINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hFLTINT =	data;
    }
    catch (NullPointerException	e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of POINTs (float doublet)
 *
 * This data type is frequently used to pass point coordinates (x,y) 
 *  
 *  
 * @param data is a reference to the POINT objects used to create and
 * bind to the TDataType object
 */
  public TDataType(POINT[] data)
  {
    try
    {
      dFormat = TFormat.CF_FLTFLT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new POINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  public TDataType(FLTFLT[] data)
  {
    try
    {
      dFormat = TFormat.CF_FLTFLT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FLTFLT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of LNGINTs (4-byte integer + 4-byte integer pairs)
 *
 * This data type is frequently used to retrieve archived 
 * parameters (integer value + timestamp)
 *  
 * @param data is a reference to the LNGINT objects used to create and
 * bind to the TDataType object
 */
  public TDataType(LNGINT[] data)
  {
    try
    {
      dFormat =	TFormat.CF_LNGINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new LNGINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hLNGINT =	data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of DBLINTs (double + 4-byte integer pairs)
 *
 * This data type is frequently used to retrieve archived 
 * parameters (double value + timestamp)
 *  
 * @param data is a reference to the DBLINT objects used to create and
 * bind to the TDataType object
 * 
 * @note This data type is discouraged as only a few platforms and compilers
 * use 4-byte boundaries by default.  Better is to use DBLDBL.
 */
  public TDataType(DBLINT[] data)
  {
    try
    {
      dFormat =	TFormat.CF_DBLINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*12;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new DBLINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0;	n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/12; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hDBLINT =	data;
    }
    catch (NullPointerException	e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }
  /**
   * @brief An array of DBLDBLs (double + double pairs)
   *
   * This data type is frequently used to retrieve archived 
   * parameters (double value + double timestamp)
   *  
   * @param data is a reference to the DBLDBL objects used to create and
   * bind to the TDataType object
   */
  public TDataType(DBLDBL[] data)
  {
    try
    {
      dFormat = TFormat.CF_DBLDBL;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new DBLDBL();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hDBLDBL = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }  
      
/**
 * @brief An array of TDS objects (integer + float + integers triplets)
 *
 * This data type is frequently used to retrieve archived 
 * parameters (timestamp + float data + int status)
 *  
 * @param data is a reference to the TDS objects used to create and
 * bind to the TDataType object
 */
  public TDataType(TDS[] data)
  {
    try
    {
      dFormat =	TFormat.CF_INTFLTINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*12;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new TDS();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0;	n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for	(; i<data.length &&	i<(n+1)*mtu/12; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hTDS = data;
    }
    catch (NullPointerException	e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of FLTINTINT objects (float + integer doublet)
 *
 * @param data is a reference to the FLTINTINT objects used to create and
 * bind to the TDataType object
 */
  public TDataType(FLTINTINT[] data)
  {
    try
    {
      dFormat = TFormat.CF_FLTINTINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*12;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FLTINTINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/12; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hFLTINTINT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of FLTFLTINT objects (float doublet + integer)
 *
 * @param data is a reference to the FLTFLTINT objects used to create and
 * bind to the TDataType object
 */
  public TDataType(FLTFLTINT[] data)
  {
    try
    {
      dFormat = TFormat.CF_FLTFLTINT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*12;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FLTFLTINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/12; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hFLTFLTINT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief An array of USTRING objects (DOOCS Universal string)
 *
 * This data type corresponds to CF_USTRING 
 * and consists of an integer, float, float, integer header
 * preceeding an 80 character string.
 *  
 * @param data is a reference to the USTRING objects used to create and
 * bind to the TDataType object
 */
  public TDataType(USTRING[] data)
  {
    try
    {
      dFormat =	TFormat.CF_USTRING;
      dArrayLength = data.length;
      dSizeInBytes = data.length*96;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new USTRING();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/96; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hUSTRING = data;
    }
    catch (NullPointerException	e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
/**
 * @brief An array of ADDRESS objects
 *
 * This data type corresponds to CF_ADDRESS 
 * and consists of a 4-integer quartet
 * which can be used to define a hardware address
 *  
 * @param data is a reference to the ADDRESS objects used to create and
 * bind to the TDataType object
 */
  public TDataType(ADDRESS[] data)
  {
    try
    {
      dFormat = TFormat.CF_ADDRESS;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new ADDRESS();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTINTINTINT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  public TDataType(INTINTINTINT[] data)
  {
    try
    {
      dFormat = TFormat.CF_ADDRESS;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new INTINTINTINT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTINTINTINT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  /**
   * @brief An array of FILTER objects
   *
   * This data type corresponds to CF_FILTER 
   * and consists of an integer + float triplet
   * which can be used to define filter parameters
   *  
   * @param data is a reference to the FILTER objects used to create and
   * bind to the TDataType object
   */
  public TDataType(FILTER[] data)
  {
    try
    {
      dFormat = TFormat.CF_FILTER;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FILTER();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTFLTFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  public TDataType(INTFLTFLTFLT[] data)
  {
    try
    {
      dFormat = TFormat.CF_FILTER;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new INTFLTFLTFLT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTFLTFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  /**
   * @brief An array of FWINDOW objects
   *
   * This data type corresponds to CF_FWINDOW 
   * and consists of an integer doublet + a float doublet
   * which can be used to define range criteria
   *  
   * @param data is a reference to the FWINDOW objects used to create and
   * bind to the TDataType object
   */
  public TDataType(FWINDOW[] data)
  {
    try
    {
      dFormat = TFormat.CF_FWINDOW;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new FWINDOW();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTINTFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
  public TDataType(INTINTFLTFLT[] data)
  {
    try
    {
      dFormat = TFormat.CF_INTINTFLTFLT;
      dArrayLength = data.length;
      dSizeInBytes = data.length*16;
      for (int i=0; i<data.length; i++) if (data[i] == null) data[i] = new INTINTFLTFLT();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++) 
      {
        dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds = new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/16; i++) ds.write(data[i].toByteArray());
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hINTINTFLTFLT = data;
    }
    catch (NullPointerException e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }  
  }    
/**
 * @brief A SPECTRUM object (USTRING + float spectrum array)
 *
 * This data type corresponds to CF_SPECTRUM 
 * and consists of an integer, float, float, integer header
 * preceeding an 80 character string, followed by a wave form of
 * up to 4000 float values.
 *  
 * @param data is a reference to the SPECTRUM object used to create and
 * bind to the TDataType object
 */
  public TDataType(SPECTRUM data)
  {
    try
    {
      dFormat =	TFormat.CF_SPECTRUM;
      dArrayLength = data.length + 24;
      dSizeInBytes = data.length*4 + 96;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      byte[] bdata = data.toByteArray();
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<dSizeInBytes && i<(n+1)*mtu; i++) ds.writeByte(bdata[i]);
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);	  
        ds.close();	  
      }
      hSPECTRUM = data;
    }
    catch (NullPointerException	e)
    {
      e.printStackTrace();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }
  private int pushData(double[] data)
  {
  	int fmtsiz = TFormat.formatSizeOf(TFormat.CF_DOUBLE);
    try
    {
      for (int n=0,i=0; n<numblks; n++)	
      {
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/fmtsiz; i++) ds.writeDouble(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,fmtsiz);
        dBuffers[n].reset();
        ds.write(b,0,b.length);	  
        ds.close();
      }
      return 0;
    }
    catch (IOException e)
    {
      e.printStackTrace();
      return -1;
    }	 
  }
  private int initTDataType(short dataFormat, int dataSize)
  {
    int fmtsiz;
    dFormat =	dataFormat;
    if ((fmtsiz=TFormat.formatSizeOf(dFormat)) < 0) return 2;
    if (fmtsiz == 0) return 0;
    dArrayLength = dataSize;
    dSizeInBytes = dArrayLength*fmtsiz;
    initCounters();
    dBuffers = new ByteArrayOutputStream[numblks];
    for (int n=0,i=0; n<numblks; n++)	
    {
	   dBuffers[n] = new ByteArrayOutputStream(dBufferSegSize);
    }
	  return 0;
  }
/**
 * @brief An array of doubles
 *
 * This corresponds to tine format CF_DOUBLE
 *
 * @param data is a reference to the double array used to create and
 * bind to the TDataType object
 */
  public TDataType(double[] data)
  {
    try
    {
      dFormat =	TFormat.CF_DOUBLE;
      dArrayLength = data.length;
      dSizeInBytes = data.length*8;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i<(n+1)*mtu/8; i++) ds.writeDouble(data[i]);
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,8);
        dBuffers[n].reset();
        ds.write(b,0,b.length);	  
        ds.close();
      }
      hDoubles = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }    
 void update(byte[] data,int num, int id)
  {
    if (debugLevel > 1 )
    {
      System.out.println("update: blk " + num + " id	" +	id + "(" + data.length + " bytes)");
    }
    blknum = num;
    // check num vs	numblks, and id	for	integrity
    if (id != blkid)
    {
      blkid	= id;
      blksin = 0;
      bytesin = 0;
    }
    if (data.length == 0) return;
    
    if (dBuffers == null)
    {
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
      }
    }
	
    try
    {
      // possible small mtu server (NIOS) ...
      if (dBuffers[num-1] == null)
        dBuffers[num-1] = new ByteArrayOutputStream(dBufferSegSize);
      dBuffers[num-1].reset(); 
      DataOutputStream ds =	new DataOutputStream(dBuffers[num-1]);
      ds.write(data,0,data.length);
      ds.close();
      blksin++;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }        
  public int getData(NAME8[] data)
  {
    if (dArrayLength > data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/8; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME8(); 
            data[i].toByteArray();
          }
        	for (j=0; j<8; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toName();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME16[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME16(); 
            data[i].toByteArray();
          }
        	for (j=0; j<16; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toName();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME16I[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/20; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME16I(); 
            data[i].toByteArray();
          }
        	for (j=0; j<20; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME16FI[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/24; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME16FI(); 
            data[i].toByteArray();
          }
          for (j=0; j<24; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME32[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/32; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME32(); 
            data[i].toByteArray();
          }
        	for (j=0; j<32; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toName();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME32I[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/36; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME32I(); 
            data[i].toByteArray();
          }
        	for (j=0; j<36; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME48[] data)
  {
    if (dArrayLength < data.length) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/48; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME48(); 
            data[i].toByteArray();
          }
        	for (j=0; j<48; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toName();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME48I[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/52; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME48I(); 
            data[i].toByteArray();
          }
          for (j=0; j<52; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME64[] data)
  {
    if (dArrayLength < data.length) return  -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/64; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME64(); 
            data[i].toByteArray();
          }
          for (j=0; j<64; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toName();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(NAME64I[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/68; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new NAME64I(); 
            data[i].toByteArray();
          }
          for (j=0; j<68; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(byte[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length; i++) data[i] = ds.readByte();
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(short[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,2);
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/2; i++) data[i] = ds.readShort();
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public int putData(short[] data)
  {
    if (dArrayLength != data.length) return -1;
    try
    {
      initCounters();
      int i,n,m,j;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,2);
        dBuffers[n].reset();
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for	(; i<data.length &&	i <	m+b.length/2 ; i++)	ds.writeShort(data[i]);
        ds.close();
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public short getData()
  {
    short data;
    if (dArrayLength != 1) return -1;
    try
    {
    	byte[] b = dBuffers[0].toByteArray();
    	Swap.Bytes(b,2);
    	ByteArrayInputStream bs	= new ByteArrayInputStream(b);
    	DataInputStream	ds = new DataInputStream(bs);
    	data = ds.readShort();
    	ds.close();	bs.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public int getData(int[] data)
  {
    if (dArrayLength < data.length) return -1;
    try
    {
      int i,n,m;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/4; i++)	data[i]	= ds.readInt();
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public int putData(int[] data)
  {
    if (dArrayLength != data.length) return -1;
    try
    {
      initCounters();
      int i,n,m;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        dBuffers[n].reset();
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i < m+b.length/4; i++) ds.writeInt(data[i]);
        ds.close();
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public int getData(float[] data)
  {
    if (dArrayLength <	data.length) return	-1;
    try
    {
      int i,n,m;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/4; i++) data[i]	= ds.readFloat();
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  public int getData(FLTINT[] data)
  {
    if (dArrayLength <	data.length) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/8; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FLTINT(); 
            data[i].toByteArray();
          }
        	for (j=0; j<8; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(FLTFLT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/8; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FLTFLT(); 
            data[i].toByteArray();
          }
          for (j=0; j<8; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(POINT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/8; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new POINT(); 
            data[i].toByteArray();
          }
          for (j=0; j<8; j++) data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(LNGINT[] data)
  {
    if (dArrayLength <	data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/8; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new LNGINT(); 
            data[i].toByteArray();
          }
        	for (j=0; j<8; j++) data[i].hByteArray[j] = ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(DBLINT[] data)
  {
    if (dArrayLength <	data.length) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0;	n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for	(; i<data.length &&	i <	m+b.length/12; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new DBLINT(); 
            data[i].toByteArray();
          }
        	for	(j=0; j<12; j++)	data[i].hByteArray[j] =	ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }
  public int getData(DBLDBL[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/12; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new DBLDBL(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++)  data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }      
  public int getData(TDS[] data)
  {
    if (dArrayLength <	data.length) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/12; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new TDS(); 
            data[i].toByteArray();
          }
        	for	(j=0; j<12; j++)	data[i].hByteArray[j] =	ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(FLTFLTINT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/12; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FLTFLTINT(); 
            data[i].toByteArray();
          }
          for (j=0; j<12; j++)  data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(FLTINTINT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/12; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FLTINTINT(); 
            data[i].toByteArray();
          }
          for (j=0; j<12; j++)  data[i].hByteArray[j] = ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(USTRING[] data)
  {
    if (dArrayLength <	data.length) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/96; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new USTRING(); 
            data[i].toByteArray();
          }
        	for (j=0; j<96; j++) data[i].hByteArray[j] =	ds.readByte();
        	data[i].toStruct();
        }
        ds.close();	bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(ADDRESS[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new ADDRESS(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(INTINTINTINT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new INTINTINTINT(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(FILTER[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FILTER(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(INTFLTFLTFLT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new INTFLTFLTFLT(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(FWINDOW[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new FWINDOW(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(INTINTFLTFLT[] data)
  {
    if (dArrayLength <  data.length) return -1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0,j=0; n<numblks; n++,m=i) 
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs = new ByteArrayInputStream(b);
        DataInputStream ds = new DataInputStream(bs);
        for (; i<data.length && i < m+b.length/16; i++)
        {
          if (data[i] == null) 
          {
            data[i] = new INTINTFLTFLT(); 
            data[i].toByteArray();
          }
          for (j=0; j<16; j++) data[i].hByteArray[j] =  ds.readByte();
          data[i].toStruct();
        }
        ds.close(); bs.close();
      }
      dCompletionLength = i;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int getData(SPECTRUM data)
  {
    if (dArrayLength <	data.length + 24) return	-1;
    try
    {
      int i,n,m,j;
      for (i=0,n=0,m=0; n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for (; i<dArrayLength*4 && i < m+b.length; i++)
        {
        	data.hByteArray[i] = ds.readByte();
        }
        data.toStruct();
        ds.close();	bs.close();
      }
      dCompletionLength = i - 24;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return 0;
  }    
  public int putData(float[] data)
  {
		if (dArrayLength !=	data.length) return	-1;
		try
		{
		  initCounters();
		  for (int i=0,n=0,m=0;	n<numblks; n++,m=i)	
		  {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,4);
        dBuffers[n].reset();
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for	(; i<data.length &&	i <	m+b.length/4 ; i++)	ds.writeFloat(data[i]);
        ds.close();
		  }
		}
		catch (IOException e)
		{
		  e.printStackTrace();
		}	 
		return 0;
  }    
  public int getData(double[] data)
  {
		if (dArrayLength <	data.length) return	-1;
		try
		{
		  int i,n,m;
		  for (i=0,n=0,m=0;	n<numblks; n++,m=i)	
		  {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,8);
        ByteArrayInputStream bs	= new ByteArrayInputStream(b);
        DataInputStream	ds = new DataInputStream(bs);
        for	(; i<data.length &&	i <	m+b.length/8 ; i++)	data[i]	= ds.readDouble();
        ds.close();	bs.close();
		  }
		  dCompletionLength = i;
		}
		catch (IOException e)
		{
		  e.printStackTrace();
		}	 
		return 0;
  }    
  public int putData(double[] data)
  {
    if (dArrayLength != data.length) return	-1;
    try
    {
      initCounters();
      for (int i=0,n=0,m=0;	n<numblks; n++,m=i)	
      {
        byte[] b = dBuffers[n].toByteArray();
        Swap.Bytes(b,8);
        dBuffers[n].reset();
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<data.length && i < m+b.length/8; i++)	ds.writeDouble(data[i]);
        ds.close();
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
    return 0;
  }    
  byte[] getDataBuffer()
  {
    try
    {
      if (dBuffer == null) dBuffer = new ByteArrayOutputStream();
      else dBuffer.reset();
      DataOutputStream dos = new DataOutputStream(dBuffer);
      for (int i=0,n=0,m=0;	n<numblks; n++,m=i)
      {
        dos.write(dBuffers[n].toByteArray());
      }
      dos.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return dBuffer.toByteArray();
  }    
  byte[] getDataBuffer(int n)
  {
    try
    {
      if (dBuffer == null) dBuffer = new ByteArrayOutputStream();
      else dBuffer.reset();
      DataOutputStream dos = new DataOutputStream(dBuffer);
      if (n < numblks)
      {
        dos.write(dBuffers[n].toByteArray());
      }
      dos.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
    return dBuffer.toByteArray();
  }    
  /**
   * @brief Returns the data array length of the associated tine Data Type
   * object
   *
   * @return Returns the data array length of the associated tine Data Type
   * object
   */
  public int getArrayLength() { return dArrayLength; }  
  /**
   * @brief Returns the data type of the associated tine Data Type
   * object
   *
   * @return Returns the data type of the associated tine Data Type
   * object
   */
  public short getFormat() { return dFormat; }  
  String getTag() {	return dTag; }  
  int getSizeInBytes() { return	dSizeInBytes; }  

  public int getData(Object Data)
  {
    int i = 0;

    if      (Data instanceof NAME8[]) getData((NAME8[])Data);
    else if (Data instanceof NAME16[]) getData((NAME16[])Data);
    else if (Data instanceof NAME32[]) getData((NAME32[])Data);
    else if (Data instanceof NAME48[]) getData((NAME48[])Data);
    else if (Data instanceof NAME64[]) getData((NAME64[])Data);
    else if (Data instanceof NAME16I[]) getData((NAME16I[])Data);
    else if (Data instanceof NAME16FI[]) getData((NAME16FI[])Data);
    else if (Data instanceof NAME32I[]) getData((NAME32I[])Data);
    else if (Data instanceof NAME48I[]) getData((NAME48I[])Data);
    else if (Data instanceof NAME64I[]) getData((NAME64I[])Data);
    else if (Data instanceof byte[]) getData((byte[])Data);
    else if (Data instanceof short[]) getData((short[])Data);
    else if (Data instanceof int[]) getData((int[])Data);
    else if (Data instanceof float[]) getData((float[])Data);
    else if (Data instanceof double[]) getData((double[])Data);
    else if (Data instanceof LNGINT[]) getData((LNGINT[])Data);
    else if (Data instanceof FLTINT[]) getData((FLTINT[])Data);
    else if (Data instanceof FLTFLT[]) getData((FLTFLT[])Data);
    else if (Data instanceof POINT[]) getData((POINT[])Data);
    else if (Data instanceof DBLINT[]) getData((DBLINT[])Data);
    else if (Data instanceof DBLDBL[]) getData((DBLDBL[])Data);
    else if (Data instanceof TDS[]) getData((TDS[])Data);
    else if (Data instanceof FLTFLTINT[]) getData((FLTFLTINT[])Data);
    else if (Data instanceof FLTINTINT[]) getData((FLTINTINT[])Data);
    else if (Data instanceof USTRING[]) getData((USTRING[])Data);
    else if (Data instanceof ADDRESS[]) getData((ADDRESS[])Data);
    else if (Data instanceof INTINTINTINT[]) getData((INTINTINTINT[])Data);
    else if (Data instanceof FILTER[]) getData((FILTER[])Data);
    else if (Data instanceof INTFLTFLTFLT[]) getData((INTFLTFLTFLT[])Data);
    else if (Data instanceof FWINDOW[]) getData((FWINDOW[])Data);
    else if (Data instanceof INTINTFLTFLT[]) getData((INTINTFLTFLT[])Data);
    else if (Data instanceof SPECTRUM) getData((SPECTRUM)Data);
    return 0;
  }
  public int dataFill(String fromStr)
  {
    int n = 0;
    switch( dFormat )
    {
      case TFormat.CF_NULL:
        break;
      case TFormat.CF_NAME16FI:
        for (n=0; n<dArrayLength && hName16FI[n] != null; n++) 
          hName16FI[n].name = fromStr;
        break;
      case TFormat.CF_NAME8:
        for (n=0; n<dArrayLength && hName8[n] != null; n++) 
          hName8[n].name = fromStr;
        break;
      case TFormat.CF_NAME16:
        for (n=0; n<dArrayLength && hName16[n] != null; n++) 
          hName16[n].name = fromStr;
        break;
      case TFormat.CF_NAME32:
        for (n=0; n<dArrayLength && hName32[n] != null; n++) 
          hName32[n].name = fromStr;
        break;
      case TFormat.CF_NAME48:
        for (n=0; n<dArrayLength && hName48[n] != null; n++) 
          hName48[n].name = fromStr;
        break;
      case TFormat.CF_NAME64:
        for (n=0; n<dArrayLength && hName64[n] != null; n++) 
          hName64[n].name = fromStr;
        break;
      case TFormat.CF_NAME16I:
        for (n=0; n<dArrayLength && hName16I[n] != null; n++) 
          hName16I[n].name = fromStr;
        break;
      case TFormat.CF_NAME32I:
        for (n=0; n<dArrayLength && hName32I[n] != null; n++) 
          hName32I[n].name = fromStr;
        break;
      case TFormat.CF_NAME48I:
        for (n=0; n<dArrayLength && hName48I[n] != null; n++) 
          hName48I[n].name = fromStr;
        break;
      case TFormat.CF_NAME64I:
        for (n=0; n<dArrayLength && hName64I[n] != null; n++) 
          hName64I[n].name = fromStr;
        break;
      case TFormat.CF_USTRING:
        for (n=0; n<dArrayLength && hUSTRING[n] != null; n++) 
          hUSTRING[n].str = fromStr;
        break;
      case TFormat.CF_TEXT:
        int size = dArrayLength < fromStr.length() ? dArrayLength : fromStr.length();
        char[] s = fromStr.toCharArray();
        for (n=0; n<size && hText != null; n++) 
          hText[n] = s[n];
        break;
    }
    return n;
  }
  public int dataFill(double fromVal)
  {
    int n = 0;
    switch( dFormat )
    {
      case TFormat.CF_NULL:
        break;
      case TFormat.CF_NAME16FI:
        for (n=0; n<dArrayLength && hName16FI[n] != null; n++) 
          hName16FI[n].fval = (float)fromVal;
        break;
      case TFormat.CF_BYTE:
        for (n=0; n<dArrayLength && hBytes != null; n++) 
          hBytes[n] = (byte)fromVal;
        break;
      case TFormat.CF_SHORT:
        for (n=0; n<dArrayLength && hShorts != null; n++) 
          hShorts[n] = (short)fromVal;
        break;
      case TFormat.CF_LONG:
        for (n=0; n<dArrayLength && hLongs != null; n++) 
          hLongs[n] = (int)fromVal;
        break;
      case TFormat.CF_FLOAT:
        for (n=0; n<dArrayLength && hFloats != null; n++) 
          hFloats[n] = (float)fromVal;
        break;
      case TFormat.CF_DOUBLE:
        for (n=0; n<dArrayLength && hDoubles != null; n++) 
          hDoubles[n] = fromVal;
        break;
      case TFormat.CF_LNGINT:
        for (n=0; n<dArrayLength && hLNGINT[n] != null; n++) 
          hLNGINT[n].lval = (int)fromVal;
        break;
      case TFormat.CF_FLTINT:
        for (n=0; n<dArrayLength && hFLTINT[n] != null; n++) 
          hFLTINT[n].fval = (float)fromVal;
        break;
      case TFormat.CF_FLTFLT:
        for (n=0; n<dArrayLength && hFLTFLT[n] != null; n++) 
          hFLTFLT[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_DBLINT:
        for (n=0; n<dArrayLength && hDBLINT[n] != null; n++) 
          hDBLINT[n].dval = fromVal;
        break;
      case TFormat.CF_DBLDBL:
        for (n=0; n<dArrayLength && hDBLDBL[n] != null; n++) 
          hDBLDBL[n].d1val = fromVal;
        break;
      case TFormat.CF_TDS:
        for (n=0; n<dArrayLength && hTDS[n] != null; n++) 
          hTDS[n].dval = (float)fromVal;
        break;
      case TFormat.CF_FLTFLTINT:
        for (n=0; n<dArrayLength && hFLTFLTINT[n] != null; n++) 
          hFLTFLTINT[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_FLTINTINT:
        for (n=0; n<dArrayLength && hFLTINTINT[n] != null; n++) 
          hFLTINTINT[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_ADDRESS:
        for (n=0; n<dArrayLength && hINTINTINTINT[n] != null; n++) 
          hINTINTINTINT[n].i1val = (int)fromVal;
        break;
      case TFormat.CF_FILTER:
        for (n=0; n<dArrayLength && hINTFLTFLTFLT[n] != null; n++) 
          hINTFLTFLTFLT[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_FWINDOW:
        for (n=0; n<dArrayLength && hINTINTFLTFLT[n] != null; n++) 
          hINTINTFLTFLT[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_USTRING:
        for (n=0; n<dArrayLength && hUSTRING[n] != null; n++) 
          hUSTRING[n].f1val = (float)fromVal;
        break;
      case TFormat.CF_SPECTRUM:
        for (n=0; n<dArrayLength && hSPECTRUM != null; n++) 
          hSPECTRUM.d_spect_array[n] = (float)fromVal;
        break;
    }
    return n;
  }
  public int dataCopy(TDataType fromData)
  {
    if (dFormat != fromData.dFormat) return 0;    
    int n = 0;
    int size = dArrayLength < fromData.dArrayLength ? dArrayLength : fromData.dArrayLength;  
    switch( dFormat )
    {
      case TFormat.CF_NULL:
        break;
      case TFormat.CF_NAME8:
        System.arraycopy(fromData.hName8, 0, hName8, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME16:
        System.arraycopy(fromData.hName16, 0, hName16, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME32:
        System.arraycopy(fromData.hName32, 0, hName32, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME48:
        System.arraycopy(fromData.hName48, 0, hName48, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME64:
        System.arraycopy(fromData.hName64, 0, hName64, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME16I:
        System.arraycopy(fromData.hName16I, 0, hName16I, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME16FI:
        System.arraycopy(fromData.hName16FI, 0, hName16FI, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME32I:
        System.arraycopy(fromData.hName32I, 0, hName32I, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME48I:
        System.arraycopy(fromData.hName48I, 0, hName48I, 0, size);
        n = size;
        break;
      case TFormat.CF_NAME64I:
        System.arraycopy(fromData.hName64I, 0, hName64I, 0, size);
        n = size;
        break;
      case TFormat.CF_STRUCT:
      case TFormat.CF_BYTE:
        System.arraycopy(fromData.hBytes, 0, hBytes, 0, size);
        n = size;
        break;
      case TFormat.CF_SHORT:
        System.arraycopy(fromData.hShorts, 0, hShorts, 0, size);
        n = size;
        break;
      case TFormat.CF_LONG:
        System.arraycopy(fromData.hLongs, 0, hLongs, 0, size);
        n = size;
        break;
      case TFormat.CF_FLOAT:
        System.arraycopy(fromData.hFloats, 0, hFloats, 0, size);
        n = size;
        break;
      case TFormat.CF_DOUBLE:
        System.arraycopy(fromData.hDoubles, 0, hDoubles, 0, size);
        n = size;
        break;
      case TFormat.CF_LNGINT:
        System.arraycopy(fromData.hLNGINT, 0, hLNGINT, 0, size);
        n = size;
        break;
      case TFormat.CF_FLTINT:
        System.arraycopy(fromData.hFLTINT, 0, hFLTINT, 0, size);
        n = size;
        break;
      case TFormat.CF_FLTFLT:
        System.arraycopy(fromData.hFLTFLT, 0, hFLTFLT, 0, size);
        n = size;
        break;
      case TFormat.CF_DBLINT:
        System.arraycopy(fromData.hDBLINT, 0, hDBLINT, 0, size);
        n = size;
        break;
      case TFormat.CF_DBLDBL:
        System.arraycopy(fromData.hDBLDBL, 0, hDBLDBL, 0, size);
        n = size;
        break;
      case TFormat.CF_TDS:
        System.arraycopy(fromData.hTDS, 0, hTDS, 0, size);
        n = size;
        break;
      case TFormat.CF_FLTFLTINT:
        System.arraycopy(fromData.hFLTFLTINT, 0, hFLTFLTINT, 0, size);
        n = size;
        break;
      case TFormat.CF_FLTINTINT:
        System.arraycopy(fromData.hFLTINTINT, 0, hFLTINTINT, 0, size);
        n = size;
        break;
      case TFormat.CF_ADDRESS:
        System.arraycopy(fromData.hINTINTINTINT, 0, hINTINTINTINT, 0, size);
        n = size;
        break;
      case TFormat.CF_FILTER:
        System.arraycopy(fromData.hINTFLTFLTFLT, 0, hINTFLTFLTFLT, 0, size);
        n = size;
        break;
      case TFormat.CF_FWINDOW:
        System.arraycopy(fromData.hINTINTFLTFLT, 0, hINTINTFLTFLT, 0, size);
        n = size;
        break;
      case TFormat.CF_USTRING:
        System.arraycopy(fromData.hUSTRING, 0, hUSTRING, 0, size);
        n = size;
        break;
      case TFormat.CF_SPECTRUM:
        System.arraycopy(fromData.hSPECTRUM.d_spect_array, 0, hSPECTRUM.d_spect_array, 0, size);
        n = size;
        break;
    }
    return n;  
  }
  /**
   * @brief Retrieves the returned data as a Vector.
   *
   * This method is useful for obtaining the unbound data
   * when the data type is unkown at link time.  A call to getFormat()
   * can be made to ascertain the tine data type and hence the contents
   * of the Vector.
   * 
   */
  public Vector getVector()
  {
    int i = 0;
    if (dVector == null) dVector = new Vector();
    Vector v = dVector;
    v.clear();
    
    switch( dFormat )
    {
      case TFormat.CF_NULL:
      	break;
      case TFormat.CF_NAME8:
      	NAME8 n8data[] = hName8;
        if (n8data == null)
        {
          n8data = new NAME8[ dArrayLength ];
        	getData( n8data );
        }
      	for (i = 0; i < n8data.length; i++ ) v.addElement(n8data[i]);
      	break;
      case TFormat.CF_NAME16:
      	NAME16 n16data[] = hName16;
        if (n16data == null)
        {
          n16data = new NAME16[ dArrayLength ];
        	getData( n16data );
        }
      	for (i = 0; i < n16data.length; i++ ) v.addElement(n16data[i]);
      	break;
      case TFormat.CF_NAME32:
        NAME32 n32data[] = hName32;
        if (n32data == null)
        {
          n32data = new NAME32[ dArrayLength ];
          getData( n32data );
        }
        for (i = 0; i < n32data.length; i++ ) v.addElement(n32data[i]);
        break;
      case TFormat.CF_NAME48:
        NAME48 n48data[] = hName48;
        if (n48data == null)
        {
          n48data = new NAME48[ dArrayLength ];
          getData( n48data );
        }
        for (i = 0; i < n48data.length; i++ ) v.addElement(n48data[i]);
        break;
      case TFormat.CF_NAME64:
        NAME64 n64data[] = hName64;
        if (n64data == null)
        {
          n64data = new NAME64[ dArrayLength ];
          getData( n64data );
        }
        for (i = 0; i < n64data.length; i++ ) v.addElement(n64data[i]);
        break;
      case TFormat.CF_NAME16I:
        NAME16I n16idata[] = hName16I;
        if (n16idata == null)
        {
          n16idata = new NAME16I[ dArrayLength ];
          getData( n16idata );
        }
        for (i = 0; i < n16idata.length; i++ ) v.addElement(n16idata[i]);
      	break;
      case TFormat.CF_NAME16FI:
        NAME16FI n16fidata[] = hName16FI;
        if (n16fidata == null)
        {
          n16fidata = new NAME16FI[ dArrayLength ];
          getData( n16fidata );
        }
        for (i = 0; i < n16fidata.length; i++ ) v.addElement(n16fidata[i]);
        break;
      case TFormat.CF_NAME32I:
        NAME32I n32idata[] = hName32I;
        if (n32idata == null)
        {
          n32idata = new NAME32I[ dArrayLength ];
          getData( n32idata );
        }
        for (i = 0; i < n32idata.length; i++ ) v.addElement(n32idata[i]);
      	break;
      case TFormat.CF_NAME48I:
        NAME48I n48idata[] = hName48I;
        if (n48idata == null)
        {
          n48idata = new NAME48I[ dArrayLength ];
          getData( n48idata );
        }
        for (i = 0; i < n48idata.length; i++ ) v.addElement(n48idata[i]);
        break;
      case TFormat.CF_NAME64I:
        NAME64I n64idata[] = hName64I;
        if (n64idata == null)
        {
          n64idata = new NAME64I[ dArrayLength ];
          getData( n64idata );
        }
        for (i = 0; i < n64idata.length; i++ ) v.addElement(n64idata[i]);
        break;
      case TFormat.CF_BYTE:
        byte bdata[] = hBytes;
        if (bdata == null)
        {
          bdata = new byte[ dArrayLength ];
          getData( bdata );
        }
        for (i = 0; i < bdata.length; i++ ) v.addElement(new Byte(bdata[i]));
      	break;
      case TFormat.CF_STRUCT:
      	break;
      case TFormat.CF_SHORT:
      	short sdata[] = hShorts;
        if (sdata == null)
        {
          sdata = new short[ dArrayLength ];
        	getData( sdata );
        }
      	for (i = 0; i < sdata.length; i++ ) v.addElement(new Short(sdata[i]));
      	break;
      case TFormat.CF_LONG:
      	int idata[] = hLongs;
        if (idata == null)
        {
          idata = new int[ dArrayLength ];
      	  getData( idata );
        }
      	for (i = 0; i < idata.length; i++ ) v.addElement(new Integer(idata[i]));
      	break;
      case TFormat.CF_FLOAT:
      	float fdata[] = hFloats;
        if (fdata == null)
        {
          fdata = new float[ dArrayLength ];
        	getData( fdata );
        }
      	for (i = 0; i < fdata.length; i++ ) v.addElement(new Float(fdata[i]));
      	break;
      case TFormat.CF_DOUBLE:
      	double ddata[] = new double[ dArrayLength ];
      	getData( ddata );
      	for (i = 0; i < ddata.length; i++ ) v.addElement(new Double(ddata[i]));
      	break;
      case TFormat.CF_LNGINT:
        LNGINT lidata[] = hLNGINT;
        if (lidata == null)
        {
          lidata = new LNGINT[ dArrayLength ];
          getData( lidata );
        }
        for (i = 0; i < lidata.length; i++ ) v.addElement(lidata[i]);
        break;
      case TFormat.CF_FLTINT:
        FLTINT fidata[] = hFLTINT;
        if (fidata == null)
        {
          fidata = new FLTINT[ dArrayLength ];
          getData( fidata );
        }
        for (i = 0; i < fidata.length; i++ ) v.addElement(fidata[i]);
        break;
      case TFormat.CF_FLTFLT:
        FLTFLT ffdata[] = hFLTFLT;
        if (ffdata == null)
        {
          ffdata = new FLTFLT[ dArrayLength ];
          getData( ffdata );
        }
        for (i = 0; i < ffdata.length; i++ ) v.addElement(ffdata[i]);
        break;
      case TFormat.CF_DBLINT:
        DBLINT didata[] = hDBLINT;
        if (didata == null)
        {
          didata = new DBLINT[ dArrayLength ];
          getData( didata );
        }
        for (i = 0; i < didata.length; i++ ) v.addElement(didata[i]);
        break;
      case TFormat.CF_DBLDBL:
        DBLDBL dddata[] = hDBLDBL;
        if (dddata == null)
        {
          dddata = new DBLDBL[ dArrayLength ];
          getData( dddata );
        }
        for (i = 0; i < dddata.length; i++ ) v.addElement(dddata[i]);
        break;
      case TFormat.CF_TDS:
        TDS tdsdata[] = hTDS;
        if (tdsdata == null)
        {
          tdsdata = new TDS[ dArrayLength ];
          getData( tdsdata );
        }
        for (i = 0; i < tdsdata.length; i++ ) v.addElement(tdsdata[i]);
        break;
      case TFormat.CF_FLTFLTINT:
        FLTFLTINT ffidata[] = hFLTFLTINT;
        if (ffidata == null)
        {
          ffidata = new FLTFLTINT[ dArrayLength ];
          getData( ffidata );
        }
        for (i = 0; i < ffidata.length; i++ ) v.addElement(ffidata[i]);
        break;
      case TFormat.CF_FLTINTINT:
        FLTINTINT fiidata[] = hFLTINTINT;
        if (fiidata == null)
        {
          fiidata = new FLTINTINT[ dArrayLength ];
          getData( fiidata );
        }
        for (i = 0; i < fiidata.length; i++ ) v.addElement(fiidata[i]);
        break;
      case TFormat.CF_ADDRESS:
        INTINTINTINT iiiidata[] = hINTINTINTINT;
        if (iiiidata == null)
        {
          iiiidata = new INTINTINTINT[ dArrayLength ];
          getData( iiiidata );
        }
        for (i = 0; i < iiiidata.length; i++ ) v.addElement(iiiidata[i]);
        break;
      case TFormat.CF_FILTER:
        INTFLTFLTFLT ifffdata[] = hINTFLTFLTFLT;
        if (ifffdata == null)
        {
          ifffdata = new INTFLTFLTFLT[ dArrayLength ];
          getData( ifffdata );
        }
        for (i = 0; i < ifffdata.length; i++ ) v.addElement(ifffdata[i]);
        break;
      case TFormat.CF_FWINDOW:
        INTINTFLTFLT iiffdata[] = hINTINTFLTFLT;
        if (iiffdata == null)
        {
          iiffdata = new INTINTFLTFLT[ dArrayLength ];
          getData( iiffdata );
        }
        for (i = 0; i < iiffdata.length; i++ ) v.addElement(iiffdata[i]);
        break;
      case TFormat.CF_USTRING:
        USTRING usdata[] = hUSTRING;
        if (usdata == null)
        {
          usdata = new USTRING[ dArrayLength ];
          getData( usdata );
        }
        for (i = 0; i < usdata.length; i++ ) v.addElement(usdata[i]);
        break;
    }
    dCompletionLength = i;
    return v;
  }  
  /**
   * @brief Outputs the returned data as a String.
   *
   * This method is useful for displaying the unbound data
   * when the data type is unkown at link time.
   * 
   */
  public String toString()
  {
    StringBuffer sb = new StringBuffer(1024);
    
    switch( dFormat )
    {
      case TFormat.CF_NULL:
        return new String("");
      case TFormat.CF_TEXT:
        char t[] = hText;
        if (t == null)
        {
          t = new char[ dArrayLength ];
          getData( t );
        }
        return new String(t);
      case TFormat.CF_USTRING:
        USTRING us[] = hUSTRING;
        if (us == null)
        {
          us = new USTRING[ dArrayLength ];
          getData( us );
        }
        for( int i = 0; i < us.length; i++ )
        {
          sb.append(us[i].ival); sb.append(" ");
          sb.append(us[i].f1val); sb.append(" ");
          sb.append(us[i].f2val); sb.append(" ");
          sb.append(us[i].tm); sb.append(" ");
          sb.append(us[i].str); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_ADDRESS:
        INTINTINTINT iiii[] = hINTINTINTINT;
        if (iiii == null)
        { 
          iiii = new INTINTINTINT[ dArrayLength ];
          getData( iiii );
        }
        for( int i = 0; i < iiii.length; i++ )
        {
          sb.append(iiii[i].i1val); sb.append(" ");
          sb.append(iiii[i].i2val); sb.append(" ");
          sb.append(iiii[i].i3val); sb.append(" ");
          sb.append(iiii[i].i4val); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FILTER:
        INTFLTFLTFLT ifff[] = hINTFLTFLTFLT;
        if (ifff == null)
        {
          ifff = new INTFLTFLTFLT[ dArrayLength ];
          getData( ifff );
        }
        for( int i = 0; i < ifff.length; i++ )
        {
          sb.append(ifff[i].i1val); sb.append(" ");
          sb.append(ifff[i].f1val); sb.append(" ");
          sb.append(ifff[i].f2val); sb.append(" ");
          sb.append(ifff[i].f3val); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FWINDOW:
        INTINTFLTFLT iiff[] = hINTINTFLTFLT;
        if (iiff == null)
        {
          iiff = new INTINTFLTFLT[ dArrayLength ];
          getData( iiff );
        }
        for( int i = 0; i < iiff.length; i++ )
        {
          sb.append(iiff[i].i1val); sb.append(" ");
          sb.append(iiff[i].i2val); sb.append(" ");
          sb.append(iiff[i].f1val); sb.append(" ");
          sb.append(iiff[i].f2val); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_TDS:
        TDS tds[] = hTDS;
        if (tds == null) 
        {
          tds = new TDS[ dArrayLength ];
          getData( tds );
        }
        for( int i = 0; i < tds.length; i++ )
        {
          sb.append(tds[i].tval); sb.append(" ");
          sb.append(tds[i].dval); sb.append(" ");
          sb.append(tds[i].sval); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FLTFLTINT:
        FLTFLTINT ffi[] = hFLTFLTINT;
        if (ffi == null)
        {
          ffi = new FLTFLTINT[ dArrayLength ];
          getData( ffi );
        }
        for( int i = 0; i < ffi.length; i++ )
        {
          sb.append(ffi[i].f1val); sb.append(" ");
          sb.append(ffi[i].f2val); sb.append(" ");
          sb.append(ffi[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FLTINTINT:
        FLTINTINT fii[] = hFLTINTINT;
        if (fii == null)
        {
          fii = new FLTINTINT[ dArrayLength ];
          getData( fii );
        }
        for( int i = 0; i < fii.length; i++ )
        {
          sb.append(fii[i].f1val); sb.append(" ");
          sb.append(fii[i].i1val); sb.append(" ");
          sb.append(fii[i].i2val); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME8:
    		NAME8 n8data[] = hName8;
        if (n8data == null)
        {
          n8data = new NAME8[ dArrayLength ];
          getData( n8data );
        }
    		for( int i = 0; i < n8data.length; i++ )
        {
          sb.append(n8data[i].name); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME16:
        NAME16 n16data[] = hName16;
        if (n16data == null)
        {
      		n16data = new NAME16[ dArrayLength ];
      		getData( n16data );
        }
    		for( int i = 0; i < n16data.length; i++ ) 
        {
          sb.append(n16data[i].name); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME32:
        NAME32 n32data[] = hName32;
        if (n32data == null)
        {
          n32data = new NAME32[ dArrayLength ];
          getData( n32data );
        }
        for( int i = 0; i < n32data.length; i++ ) 
        {
          sb.append(n32data[i].name); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME48:
        NAME48 n48data[] = hName48;
        if (n48data == null)
        {
          n48data = new NAME48[ dArrayLength ];
          getData( n48data );
        }
        getData( n48data );
        for( int i = 0; i < n48data.length; i++ ) 
        {
          sb.append(n48data[i].name); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME64:
        NAME64 n64data[] = hName64;
        if (n64data == null)
        {
          n64data = new NAME64[ dArrayLength ];
          getData( n64data );
        }
        for( int i = 0; i < n64data.length; i++ ) 
        {
          sb.append(n64data[i].name); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME16I:
        NAME16I n16idata[] = hName16I;
        if (n16idata == null)
        {
          n16idata =  new NAME16I[ dArrayLength ];
          getData( n16idata );
        }
        for( int i = 0; i < n16idata.length; i++ )
        { 
          sb.append(n16idata[i].name); sb.append(" ");
          sb.append(n16idata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME16FI:
        NAME16FI n16fidata[] = hName16FI;
        if (n16fidata == null)
        {
          n16fidata = new NAME16FI[ dArrayLength ];
          getData( n16fidata );
        }
        for( int i = 0; i < n16fidata.length; i++ )
        { 
          sb.append(n16fidata[i].name); sb.append(" ");
          sb.append(n16fidata[i].fval); sb.append(" ");
          sb.append(n16fidata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME32I:
        NAME32I n32idata[] = hName32I;
        if (n32idata == null)
        {
          n32idata =  new NAME32I[ dArrayLength ];
          getData( n32idata );
        }
        for( int i = 0; i < n32idata.length; i++ ) 
        { 
          sb.append(n32idata[i].name); sb.append(" ");
          sb.append(n32idata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME48I:
        NAME48I n48idata[] = hName48I;
        if (n48idata == null)
        {
          n48idata = new NAME48I[ dArrayLength ];
          getData( n48idata );
        }
        for( int i = 0; i < n48idata.length; i++ ) 
        { 
          sb.append(n48idata[i].name); sb.append(" ");
          sb.append(n48idata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_NAME64I:
        NAME64I n64idata[] = hName64I;
        if (n64idata == null)
        {
          n64idata = new NAME64I[ dArrayLength ];
          getData( n64idata );
        }
        for( int i = 0; i < n64idata.length; i++ ) 
        { 
          sb.append(n64idata[i].name); sb.append(" ");
          sb.append(n64idata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_BYTE:
        byte bdata[] = hBytes;
        if (bdata == null)
        {
          bdata = new byte[ dArrayLength ];
          getData( bdata );
        }
        for( int i = 0; i < bdata.length; i++ ) 
        {
          sb.append(bdata[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_STRUCT:
        return new String("user defined structure");
      case TFormat.CF_SHORT:
    		short sdata[] = hShorts;
        if (sdata == null)
        {
          sdata = new short[ dArrayLength ];
          getData( sdata );
        }
    		for( int i = 0; i < sdata.length; i++ ) 
        {
          sb.append(sdata[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_LONG:
    		int idata[] = hLongs;
        if (idata == null)
        {
          idata =  new int[ dArrayLength ];
          getData( idata );
        }
    		for( int i = 0; i < idata.length; i++ ) 
        {
          sb.append(idata[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_LNGINT:
        LNGINT lidata[] = hLNGINT;
        if (lidata == null)
        {
          lidata = new LNGINT[ dArrayLength ];
          getData( lidata );
        }
        for( int i = 0; i < lidata.length; i++ )
        { 
          sb.append(lidata[i].lval); sb.append(" ");
          sb.append(lidata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FLOAT:
    		float fdata[] = hFloats;
        if (fdata == null)
        {
          fdata = new float[ dArrayLength ];
          getData( fdata );
        }
    		for( int i = 0; i < fdata.length; i++ ) 
        {
          sb.append(fdata[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FLTINT:
        FLTINT fidata[] = hFLTINT;
        if (fidata== null)
        {
          fidata = new FLTINT[ dArrayLength ];
          getData( fidata );
        }
        for( int i = 0; i < fidata.length; i++ ) 
        { 
          sb.append(fidata[i].fval); sb.append(" ");
          sb.append(fidata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_FLTFLT:
        FLTFLT ffdata[] = hFLTFLT;
        if (ffdata == null)
        {
          ffdata = new FLTFLT[ dArrayLength ];
          getData( ffdata );
        }
        for( int i = 0; i < ffdata.length; i++ ) 
        { 
          sb.append(ffdata[i].f1val); sb.append(" ");
          sb.append(ffdata[i].f2val); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_SPECTRUM:
        SPECTRUM sp = hSPECTRUM;
        if (sp == null)
        {
          sp =  new SPECTRUM();
          getData( sp );
        }
        sb.append(sp.status); sb.append(" ");
        sb.append(sp.s_start); sb.append(" ");
        sb.append(sp.s_inc); sb.append(" ");
        sb.append(sp.tm); sb.append(" ");
        sb.append(sp.comment); sb.append(arrayDelimiter);
        for( int i = 0; i < sp.length; i++ )
        { 
          sb.append(sp.d_spect_array[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_DOUBLE:
    		double ddata[] = hDoubles;
        if (ddata == null)
        {
          ddata = new double[ dArrayLength ];
          getData( ddata );
        }
        for( int i = 0; i < ddata.length; i++ ) 
        { 
          sb.append(ddata[i]); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_DBLINT:
        DBLINT didata[] = hDBLINT;
        if (didata == null)
        {
          didata =  new DBLINT[ dArrayLength ];
          getData( didata );
        }
        for( int i = 0; i < didata.length; i++ ) 
        { 
          sb.append(didata[i].dval); sb.append(" ");
          sb.append(didata[i].ival); sb.append(arrayDelimiter);
        }
        return sb.toString();
      case TFormat.CF_DBLDBL:
        DBLDBL dddata[] = hDBLDBL;
        if (dddata == null)
        {
          dddata =  new DBLDBL[ dArrayLength ];
          getData( dddata );
        }
        for( int i = 0; i < dddata.length; i++ ) 
        { 
          sb.append(dddata[i].d1val); sb.append(" ");
          sb.append(dddata[i].d2val); sb.append(arrayDelimiter);
        }
        return sb.toString();
    }
	  return new String("");
  }  
  char[] hText;
/**
 * @brief An array of chars (to be interpretted as a text string)
 *
 * @param data is a reference to the char[] array used to create and
 * bind to the TDataType object
 */
  public TDataType(char[] data) 
  {
    try
    {
      dFormat =	TFormat.CF_TEXT;
      dArrayLength = data.length;
      dSizeInBytes = data.length;
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0;	n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        byte bdata;
        for	(; i<data.length && i<(n+1)*mtu; i++) 
        {
        	bdata = (byte)data[i];
        	ds.writeByte(bdata);
        }
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hText = data;
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }
/**
 * @brief A text string
 *
 * @param data is a reference to the StringBuffer used to create and
 * bind to the TDataType object
 */
  public TDataType(StringBuffer data) 
  {
    try
    {
      dFormat =	TFormat.CF_TEXT;
      dArrayLength = data.capacity();
      dSizeInBytes = dArrayLength;
      byte[] bdata = data.toString().getBytes();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<bdata.length && i<(n+1)*mtu; i++) ds.writeByte(bdata[i]);
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hStringBuffer = data;
      hText = new char[dArrayLength];
      if (bdata.length > 0) System.arraycopy(bdata,0,hText,0,bdata.length);
      //data.getChars(0,dArrayLength-1,hText,0);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

/**
 * @brief A text string
 *
 * @param data is a reference to the String used to create and
 * bind to the TDataType object
 */
  public TDataType(String data) 
  {
    try
    {
      dFormat =	TFormat.CF_TEXT;
      dArrayLength = data.length();
      dSizeInBytes = dArrayLength;
      byte[] bdata = data.getBytes();
      initCounters();
      dBuffers = new ByteArrayOutputStream[numblks];
      for (int n=0,i=0; n<numblks; n++)	
      {
        dBuffers[n]	= new ByteArrayOutputStream(dBufferSegSize);
        DataOutputStream ds	= new DataOutputStream(dBuffers[n]);
        for (; i<bdata.length && i<(n+1)*mtu; i++) ds.writeByte(bdata[i]);
        byte[] b = dBuffers[n].toByteArray();
        dBuffers[n].reset();
        ds.write(b,0,b.length);
        ds.close();
      }
      hText = new char[dArrayLength];
      if (dArrayLength > 0) data.getChars(0,dArrayLength-1,hText,0);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }
  public int getData(char[] data) 
  {
    if (dArrayLength < data.length) return	-1;
    int i,n,m,j;
    byte bdata[] = new byte[data.length];
    for (i=0,n=0,m=0;	n<numblks; n++,m=i)	
    {
      byte[] b = dBuffers[n].toByteArray();
      if (m + b.length <= data.length)
      { 
        System.arraycopy(b, 0, bdata, m, b.length);
        i += b.length;
      }
    }
    String sdata = new String(bdata);
    sdata.getChars(0, sdata.length(), data, 0);
    if (this.hStringBuffer != null) this.hStringBuffer.insert(0,data);
    dCompletionLength = i;
    return 0;
  }
}