package TINE;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.io.OutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.MulticastSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.Date;

public class TFactory 
{
  private static TFactory instance=null;

  boolean active;
  boolean terminate;	
  Date  date = new Date();
  int   time;
  final TPacket atp;   // general socket for asynchronous communication
  final TPacket stp;   // socket for synchronous communication
  private TPacket gtp; // globals socket
  public TPacket amtp;// asynchronous multicast socket 
  TPublishedData phs;
  TGlobalsData ghs;
  int debugLevel = 0;
  static final int maximumNumberTLinks = 256;
  private int nlinks = 0;
  private TLink tNullLink = new TLink();
  private TLink[] linkTable = new TLink[maximumNumberTLinks];
  private TCallback tcb;
  private TLinkCallback tlcb;
  boolean isInsideCallback = false;
  //static TErrorList tel;
  final TFactoryThread atfThrd;
  final TFactoryThread stfThrd;
  public TFactoryGlobalsThread gtfThrd;
  public TFactoryThread mtfThrd;
  final TFactoryWatchdogThread tfwdThrd = new TFactoryWatchdogThread();
  static final int maximumNumberTBuckets = 10;
  static final int maximumNumberGlobals = 25;
  private TBucket[] bucketTable = new TBucket[maximumNumberTBuckets];
  private TInitializer initializer=TInitializerFactory.getInstance().getInitializer();
  private TFactoryBucketThread getBucketThread(int index)
  { 
    for (int i=0; i<maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null) continue;
      if (bucketTable[i].port != initializer.getTCPPort() + 
                             linkTable[index].modAddress.srvAddress.srvPortOffset) continue;
      if (bucketTable[i].iaddr != linkTable[index].modAddress.srvAddress.srvHost) continue;
      return bucketTable[i].tfBckThrd;
    }
    return null;
  }
  private int putBucketThread(TBucket tb)
  {
    for (int i=0; i<maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null)
      {
        bucketTable[i] = tb;
        return 0;
      }
    }
    return -1;
  }
  private int removeBucketThread(TBucket tb)
  {
    if (tb == null) return -1;
    for (int i=0; i<maximumNumberTLinks; i++)
    {
      if (linkTable[i] == null) continue;
      if (tb.port != initializer.getTCPPort() + 
           linkTable[i].modAddress.srvAddress.srvPortOffset) continue;
      if (tb.iaddr != linkTable[i].modAddress.srvAddress.srvHost) continue;
      linkTable[i].active = false;
      linkTable[i].tb = null;
    }
    for (int i=0; i<maximumNumberTBuckets; i++)
    {
      if (bucketTable[i] == null) continue;
      if (bucketTable[i] != tb) continue;
      bucketTable[i].port = 0;
    }
    try
    {
      tb.sck.shutdownInput();
      tb.sck.shutdownOutput();
      tb.sck.close();
    }
    catch (IOException e)
    {
      if (debugLevel > 1) System.out.println(" IOException : " + e);
    }
    finally
    {
      tb.isDeactivating = false;
    }
    return 0;
  }
  public class TBucket
  {
    int   idx;                  // FEC index
    Socket sck;                 // TCP connection socket
    int   ptr;                  // bucket pointer
    int   siz;                  // bucket target size
    byte[] buf = new byte[1472]; // bucket buffer
    int port;
    InetAddress iaddr;
    boolean active = false;
    boolean isDeactivating = false;
    int number;
    TFactoryBucketThread tfBckThrd;
    OutputStream os;
    InputStream is;
    TBucket(int index)
    {
      if (index < 0 || index >= maximumNumberTLinks) return;
      if (linkTable[index].tb != null) return;
      port = initializer.getTCPPort() + 
             linkTable[index].modAddress.srvAddress.srvPortOffset;
      iaddr = linkTable[index].modAddress.srvAddress.srvHost;
      tfBckThrd = getBucketThread(index);
      if (tfBckThrd != null) 
      {
        sck = tfBckThrd.tb.sck;
        os = tfBckThrd.tb.os;
        is = tfBckThrd.tb.is;
        return;
      }
      try
      {
        sck = new Socket(iaddr,port);
        os = sck.getOutputStream();
        is = sck.getInputStream();
        tfBckThrd = new TFactoryBucketThread(this);
        putBucketThread(this);     
        tfBckThrd.start();
      }
      catch (IOException e)
      {
        System.err.println("Can't create TCP socket");
        e.printStackTrace();
      }
    }
  }
  class TPacket
  {
    DatagramSocket s;
    MulticastSocket m;    
    InetAddress hostAddress;
    byte[] bufOut = new byte[1472];
    DatagramPacket dpOut;
    byte[] bufIn = new byte[1472];
    DatagramPacket dpIn = new DatagramPacket(bufIn, bufIn.length);
    private int counter;
    TPacket()
    {
      try
      {
        s = new DatagramSocket();
        s.setSoTimeout(0);
        s.setReceiveBufferSize(0x10000);
      }
      catch (SocketException e)
      {
        System.err.println("Can't open socket");
        e.printStackTrace();
      }
    }
    TPacket(int port)
    {
      try
      {
        InetAddress grp;
        if (port==0) {
            s = new DatagramSocket();
            s.setSoTimeout(0);
            s.setReceiveBufferSize(0x10000);
        } else if (port== initializer.getGCastPort()) {
            grp = InetAddress.getByName(initializer.getGCastAddress());
            m = new MulticastSocket(port);
//            m.setReuseAddress(true);
            m.joinGroup(grp);
            m.setSoTimeout(0);
        } else if (port== initializer.getMCastPort()) {
            grp = InetAddress.getByName(initializer.getMCastAddress());
            m = new MulticastSocket(port);
//            m.setReuseAddress(true);
            m.joinGroup(grp);
            m.setSoTimeout(0);
            m.setReceiveBufferSize(0x10000);
        } else {
            s = new DatagramSocket(port);
            s.setSoTimeout(0);
            s.setReceiveBufferSize(0x10000);
        }
      }
      catch (SocketException e)
      {
        System.err.println("Can't open socket");
        e.printStackTrace();
      }
      catch (IOException e)
      {
      }
    }
  }
  public class TFactoryWatchdogThread extends Thread
  {
    public synchronized void run()
    {
      TLink eqm;
      long ltime, deltatime, gracePeriod, tcpIdlePeriod;
//      ltime = (date.getTime()/1000)*10;
      ltime = System.currentTimeMillis();
      time = (int)(ltime/1000);
      while (active)
      {
        if (terminate) break;
        try
        {
          sleep(100);
          ltime = System.currentTimeMillis();
          time = (int)(ltime/1000);
          int i;
          for (i=0; i<maximumNumberTLinks; i++)
          {
            if (linkTable[i] != null && linkTable[i] != tNullLink)
            {
              eqm = linkTable[i];
              if (!eqm.active) continue;
              if (eqm.linkStatus == TErrorList.server_redirection)
              {
                short mod = eqm.sub.mode;
                eqm.con = new TContract(eqm);
                eqm.sub = new TSubscription(eqm.con,eqm); // going out ...
                eqm.sub.mode = mod;
                establishLink(eqm);
                eqm.linkStatus = 0;
                continue;
              }
              if (eqm.terminate == true) 
              {
                if (debugLevel > 0) 
                {
                  System.out.println("remove terminated link " +  eqm.linkId + " /" + eqm.cntName + "/" + 
                    eqm.expName + "/" + eqm.devName + " " + eqm.devProperty + 
                    " " + TDataTime.toString(ltime));
                }
                synchronized(eqm) { eqm.notifyAll(); }// wake up the calling thread
                removeTLink(eqm);
              }
              else
              {
                short baseMode = (short)(eqm.sub.mode & 0x00ff);
                tcpIdlePeriod = 3000;
                deltatime = ltime - eqm.sub.linkLastTime;
                if (eqm.devTimeout < 1000) gracePeriod = 100; else gracePeriod = 1000;
                if (deltatime > (long)(eqm.devTimeout + gracePeriod) &&
                    baseMode > TMode.CM_CANCEL && baseMode < TMode.CM_REGISTER)
                {
                  eqm.linkTimeouts++;
                  if (debugLevel > 0) 
                  {
                    System.out.println("link " + eqm.linkId + " link timeout " +  eqm.devTimeout + 
                	       " exceeded : " + ltime + " vs " + eqm.sub.linkLastTime + 
                	       " timeout counter : " + eqm.linkTimeouts + " " + TDataTime.toString(ltime));
                  }
                  eqm.pending = false;
                  eqm.dOutput.timestamp = time;
                  eqm.linkStatus = TErrorList.link_not_open;
                  phs.linkCounter = 0;
                  eqm.needsNotification = true;
                  if (baseMode > TMode.CM_SINGLE) synchronized(eqm) 
                  { 
                    if (eqm.terminate == false) 
                    {
                      if (eqm.tcb != null)
                      {
                         eqm.isInsideCallback = true;
				  	             eqm.tcb.callback(eqm.callbackId,eqm.linkStatus);
                         eqm.isInsideCallback = false; 
				  	          }
                      else if (eqm.tlcb != null)
                      {
                         eqm.isInsideCallback = true;
                         eqm.tlcb.callback(eqm);
                         eqm.isInsideCallback = false; 
                      }
                      eqm.needsNotification = false;
				  	        }
						        eqm.notifyAll(); // wake up the calling thread
                    eqm.sub.linkLastTime = ltime;
                  }
                  if (baseMode > TMode.CM_SINGLE) 
                  { 
                    establishLink(eqm);
                  }
                  else if (baseMode == TMode.CM_SINGLE)
                  {
                    if (eqm.retryOnTimeoutError && eqm.linkTimeouts < 2)
                    {
                      eqm.sub.mode |= TMode.CM_RETRY; 
                      establishLink(eqm);
                    }
                    else 
                    {
                      synchronized(eqm) 
                    	{ 
                        if (eqm.terminate == false) 
                        {
                          if (eqm.tcb != null)
                          {
                            eqm.isInsideCallback = true;
                            eqm.tcb.callback(eqm.callbackId,eqm.linkStatus);
                            eqm.isInsideCallback = false; 
                          }
                          else if (eqm.tlcb != null)
                          {
                             eqm.isInsideCallback = true;
                             eqm.tlcb.callback(eqm);
                             eqm.isInsideCallback = false; 
                          }
                        }
                        if (debugLevel > 0) 
                          System.out.println(" cancel single link " + eqm.linkId + " from watchdog" + " " + TDataTime.toString(ltime));
                        if ((eqm.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
                        {
                          tcpIdlePeriod = 0;
                          eqm.sub.mode = (short)(TMode.CM_CANCEL | TMode.CM_CONNECT);
                        }
                        else 
                        {
                          eqm.sub.mode = TMode.CM_CANCEL;  // turn off the link
                        }
                        cancel(eqm);
                        eqm.notifyAll(); // wake up the calling thread
                      }
                    }
                  }
                }
                if (deltatime > (long)(eqm.devTimeout + tcpIdlePeriod) &&
                    eqm.sub.mode == (TMode.CM_CANCEL | TMode.CM_CONNECT))
                {
                  removeBucketThread(eqm.tb);
                }
                if (eqm.sub.mode == TMode.CM_CANCEL && !eqm.active) eqm.terminate = true; 
                if (debugLevel > 1)
                {  
                    System.out.println("link : " + eqm.expName + 
                    				   "/" + eqm.devName + 
                    				   "/" + eqm.devProperty + 
                    				   " : linkStatus " + eqm.linkStatus + 
                    				   " Link active : " + eqm.active + " terminate : " 
                               + eqm.terminate + " " + TDataTime.toString(ltime));
                }
              }
            }
          }
        }
        catch (InterruptedException e) 
        {
          if (debugLevel > 0)
          {  
            System.out.println(" Watchdog Thread : " + active + " terminate : " + terminate);
          }
       	}
      }
    } 
  }
  private int NotifyDeferredCallbacks()
  {
    TLink eqm;
    for (int i=0; i<maximumNumberTLinks; i++)
    {
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        eqm = linkTable[i];
        if (eqm.needsNotification)
        {
          synchronized(eqm) 
          { 
            if (eqm.terminate == false) 
            {
              if (eqm.tcb != null)
              {
                eqm.isInsideCallback = true;
                eqm.tcb.callback(eqm.callbackId,eqm.linkStatus);
                eqm.isInsideCallback = false;         
              }
              else if (eqm.tlcb != null)
              {
                eqm.isInsideCallback = true;
                eqm.tlcb.callback(eqm);
                eqm.isInsideCallback = false; 
              }
            }
            eqm.needsNotification = false;
          }
          eqm.notifyAll(); // wake up the calling thread
        }
      }
    }
    return 0;
  }
  public int fillinIncomingData(TLink eqm)
  {
    switch (eqm.dOutput.getFormat())
    {
      case TFormat.CF_STRUCT:
      case TFormat.CF_BYTE:
         if (eqm.dOutput.hBytes != null)
           eqm.dOutput.getData(eqm.dOutput.hBytes); 
         break;
      case TFormat.CF_SHORT:
         if (eqm.dOutput.hShorts != null)
           eqm.dOutput.getData(eqm.dOutput.hShorts); 
         break;
      case TFormat.CF_LONG:
         if (eqm.dOutput.hLongs != null)
           eqm.dOutput.getData(eqm.dOutput.hLongs); 
         break;
      case TFormat.CF_LNGINT:
         if (eqm.dOutput.hLNGINT != null)
           eqm.dOutput.getData(eqm.dOutput.hLNGINT); 
         break;
      case TFormat.CF_FLOAT:
         if (eqm.dOutput.hFloats != null)
           eqm.dOutput.getData(eqm.dOutput.hFloats); 
         break;
      case TFormat.CF_FLTINT:
         if (eqm.dOutput.hFLTINT != null)
           eqm.dOutput.getData(eqm.dOutput.hFLTINT); 
         break;
      case TFormat.CF_INTFLTINT:
         if (eqm.dOutput.hTDS != null)
           eqm.dOutput.getData(eqm.dOutput.hTDS); 
         break;
      case TFormat.CF_DOUBLE:
         if (eqm.dOutput.hDoubles != null)
           eqm.dOutput.getData(eqm.dOutput.hDoubles); 
         break;
      case TFormat.CF_DBLINT:
         if (eqm.dOutput.hDBLINT != null)
           eqm.dOutput.getData(eqm.dOutput.hDBLINT); 
         break;
      case TFormat.CF_DBLDBL:
         if (eqm.dOutput.hDBLDBL != null)
           eqm.dOutput.getData(eqm.dOutput.hDBLDBL); 
         break;
      case TFormat.CF_NAME8:
         if (eqm.dOutput.hName8 != null)
           eqm.dOutput.getData(eqm.dOutput.hName8); 
         break;
      case TFormat.CF_NAME16:
         if (eqm.dOutput.hName16 != null)
           eqm.dOutput.getData(eqm.dOutput.hName16); 
         break;
      case TFormat.CF_NAME16I:
         if (eqm.dOutput.hName16I != null)
           eqm.dOutput.getData(eqm.dOutput.hName16I); 
         break;
      case TFormat.CF_NAME32:
         if (eqm.dOutput.hName32 != null)
           eqm.dOutput.getData(eqm.dOutput.hName32); 
         break;
      case TFormat.CF_NAME32I:
         if (eqm.dOutput.hName32I != null)
           eqm.dOutput.getData(eqm.dOutput.hName32I); 
         break;
      case TFormat.CF_USTRING:
         if (eqm.dOutput.hUSTRING != null)
           eqm.dOutput.getData(eqm.dOutput.hUSTRING); 
         break;
      case TFormat.CF_SPECTRUM:
         if (eqm.dOutput.hSPECTRUM != null)
           eqm.dOutput.getData(eqm.dOutput.hSPECTRUM); 
         break;
      case TFormat.CF_TEXT:
         if (eqm.dOutput.hText != null)
           eqm.dOutput.getData(eqm.dOutput.hText); 
         break;
      default: break;
    }
    return 0;
  }
  public int fillinIncomingDataWithErrValue(TLink eqm)
  {
    if (eqm.linkStatus == 0) return 0;
    if (eqm.useErrObject) 
    {
      eqm.dOutput.dataCopy(eqm.dError);
      return 0;
    }
    if (eqm.useErrValue) 
    {
      eqm.dOutput.dataFill(eqm.linkErrValue);
      if (eqm.linkErrString != null) eqm.dOutput.dataFill(eqm.linkErrString);
    } 
    return 0;
  }
  private int InterpretIncomingData(byte[] data, int length, boolean deferCallbacks)
  {
    TLink eqm;
    boolean hasData;
    synchronized (phs)
    {
      phs.prepare(data,length);
    }
    if (debugLevel > 0) System.out.println("recv " + length + " bytes");
    for (int i=0; i<maximumNumberTLinks; i++)
    {
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        eqm = linkTable[i];
        if (!eqm.active) continue;
        if (!eqm.linkStale) continue;
        eqm.linkStale = false;
        if (eqm.linkStatus == TErrorList.server_redirection) continue;
        if (eqm.dOutput.blksin == eqm.dOutput.numblks)
        {
          long ltime = System.currentTimeMillis();
          eqm.sub.linkLastTime = ltime;
          if (eqm.adjustDefaultValues) // adjust contract upon first completion
          {
            int fmtsize = TFormat.formatSizeOf(eqm.dOutput.dFormat);
            int dretsize = eqm.dOutput.bytesin;
            if (eqm.linkStatus != 0) dretsize -= TDataType.RPCERR_SIZE;
            if (dretsize > 0 && fmtsize > 0) eqm.dOutput.dArrayLength = dretsize/fmtsize;
            eqm.adjustDefaultValues = false;
          }
          eqm.pending = false;
          if (eqm.linkStatus == TErrorList.get_subscription_id) continue;
          if (eqm.linkStatus == 0 || 
              (eqm.linkStatus & TErrorList.CE_SENDDATA) == TErrorList.CE_SENDDATA)
            hasData = true;
          else
            hasData = false;

          if (hasData)
          {          
            if (debugLevel > 0)
              System.out.println("call getData for link " + eqm.linkId + 
                                 " format " + TFormat.toString(eqm.dOutput.getFormat()) + 
                                 " " + TDataTime.toString(System.currentTimeMillis()));
            // do something with it (events ?) ...
            fillinIncomingData(eqm);
          }
          else 
          {
            fillinIncomingDataWithErrValue(eqm);
          }
          // reset counter if < 10 !!!!
          if (debugLevel > 0) System.out.println("Thread " + this +  " Link counter: " + eqm.linkCounter + " at " + ltime);
          if (TMode.getBaseMode(eqm.sub.mode) > TMode.CM_SINGLE && eqm.linkCounter < 10 && eqm.pending == false) 
          { 
            establishLink(eqm);
          }
          if (eqm.linkStatus >= 0 && eqm.pending == false) 
          {
            eqm.pending = true;
            if (eqm.dOutput.timestamp == 0)
            {
              eqm.dOutput.timestamp = (int)(ltime/1000);
              eqm.dOutput.timestampMSEC = (int)(ltime % 1000);
            }
            eqm.linkTimeouts = 0;  // reset timeout counter
            // if caller "cancel()s" then terminate is set to true (should be
            // polite and tell the server).
            // if the link automatically terminates (due to SINGLE -> CANCEL, etc.
            // then the TMode is set to CM_CANCEL (don't need to tell the server)
  //          if (eqm.sub.mode == TMode.CM_CANCEL) eqm.terminate = true;
            if (eqm.sub.mode == TMode.CM_CANCEL) eqm.active = false;
            // set notification flag:
            eqm.needsNotification = true;
            if (!deferCallbacks)
            {
              synchronized(eqm) 
              { 
                if (eqm.active == true) 
                {
                  if (eqm.tcb != null)
                  {
                    isInsideCallback = true;
                    eqm.isInsideCallback = true;
                    eqm.tcb.callback(eqm.callbackId,eqm.linkStatus);
                    eqm.isInsideCallback = false;
                    isInsideCallback = false;
                  }
                  else if (eqm.tlcb != null)
                  {
                    isInsideCallback = true;
                    eqm.isInsideCallback = true;
                    eqm.tlcb.callback(eqm);
                    eqm.isInsideCallback = false; 
                    isInsideCallback = false;
                  }
                  eqm.needsNotification = false;
                }
                else
                {
                  eqm.sub.mode = TMode.CM_CANCEL;
                }
                eqm.notifyAll(); // wake up the calling thread
              }
            }
            if ((eqm.devAccess & TAccess.CA_SYNC) != 0 && TMode.getBaseMode(eqm.sub.mode) == TMode.CM_SINGLE)
            {
              if (debugLevel > 0) System.out.println("Thread " + this +  " cancel single link " + eqm.linkId);
              if ((eqm.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
              {
                eqm.sub.mode = (short)(TMode.CM_CANCEL | TMode.CM_CONNECT);
              }
              else
              {
                eqm.sub.mode = TMode.CM_CANCEL;  // turn off the link
                eqm.active = false;
              }
            }
            if (eqm.sub.mode == TMode.CM_SINGLE) eqm.active = false;
          }
        }
      }
    }
    return 0;
  }
  private int InterpretIncomingGlobalsData(byte[] data, int length, boolean deferCallbacks)
  {
    TLink glb;
    synchronized (ghs)
    {
      ghs.prepare(data,length);
    }
    if (debugLevel > 0) System.out.println("glb recv " + length + " bytes");
    for (int i=0; i<maximumNumberTLinks; i++)
    {
      if (linkTable[i] != null && linkTable[i] != tNullLink)
      {
        glb = linkTable[i];
        if (glb.sub.mode == TMode.CM_GLOBAL)
        {
          glb.sub.linkLastTime = System.currentTimeMillis();
          glb.pending = false;
          if (debugLevel > 0)
            System.out.println("call getData for format " + glb.dOutput.getFormat());
          if (glb.linkStatus >= 0 && glb.pending == false) 
          {
            glb.pending = true;
            glb.dOutput.timestamp = time;
            glb.linkTimeouts = 0;  // reset timeout counter
            // if caller "cancel()s" then terminate is set to true (should be
            // polite and tell the server).
            // if the link automatically terminates (due to SINGLE -> CANCEL, etc.
            // then the TMode is set to CM_CANCEL (don't need to tell the server)
            // set notification flag:
            glb.needsNotification = true;
            synchronized(glb) 
            { 
              if (glb.terminate == false) 
              {
                if (glb.tcb != null)
                {
                  isInsideCallback = true;
                  glb.isInsideCallback = true;
                  glb.tcb.callback(glb.callbackId,glb.linkStatus);
                  glb.isInsideCallback = false;
                  isInsideCallback = false;
                }
                else if (glb.tlcb != null)
                {
                  isInsideCallback = true;
                  glb.isInsideCallback = true;
                  glb.tlcb.callback(glb);
                  glb.isInsideCallback = false;
                  isInsideCallback = false;
                }
                glb.needsNotification = false;
              }
              glb.notifyAll(); // wake up the calling thread
            }
          }
        }
      }
    }
    return 0;
  }
  public class TFactoryThread  extends Thread
  {
   	TPacket tp;
   	boolean isWaiting = false;
   	TFactoryThread(TPacket tPacket)
   	{
   	  tp = tPacket;
   	}
   	public synchronized void run()
   	{
   	  if (debugLevel > 0) System.out.println("Factory Thread started ...");
   	  while (active)
   	  {
   		  if (terminate) break;
   	    if (debugLevel > 0) System.out.println("Waiting for data ...");
   		  try
   		  {
          isWaiting = true;
          tp.dpIn.setLength(1472); // reset the receive buffer length
          if (tp.s != null)
          {
            tp.s.receive(tp.dpIn);		  
          }
          else if (tp.m != null)
          {
            tp.m.receive(tp.dpIn);      
          }
          isWaiting = false;
          // got it, now interpret it ...
          InterpretIncomingData(tp.dpIn.getData(),tp.dpIn.getLength(),false);
        }
        catch (InterruptedIOException e) 
        {
          System.out.println(" InterruptedIOException : " + e);
        }
        catch (IOException e)
        {
          if (debugLevel > 1) System.out.println(" IOException : " + e);
        }
      }
    }
  }
  public class TFactoryGlobalsThread  extends Thread
  {
    TPacket tp;
    boolean isWaiting = false;
    TFactoryGlobalsThread(TPacket tPacket)
    {
      tp = tPacket;
    }
    public synchronized void run()
    {
      if (debugLevel > 0) System.out.println("Globals Thread started ...");
      while (active)
      {
        if (terminate) break;
        if (debugLevel > 0) System.out.println("Waiting for globals data ...");
        try
        {
          isWaiting = true;
          tp.dpIn.setLength(1472); // reset the receive buffer length
          tp.m.receive(tp.dpIn);          
          isWaiting = false;
          // got it, now interpret it ...
          InterpretIncomingGlobalsData(tp.dpIn.getData(),tp.dpIn.getLength(),false);
        }
        catch (InterruptedIOException e) 
        {
          System.out.println(" InterruptedIOException : " + e);
        }
        catch (IOException e)
        {
          if (debugLevel > 1) System.out.println(" IOException : " + e);
        }
      }
    }
  }
  public class TFactoryBucketThread  extends Thread
  {
    TBucket tb;
    boolean isWaiting = false;
    TFactoryBucketThread(TBucket tBucket)
    {
      tb = tBucket;
      tb.active = true;
    }
    public synchronized void run()
    {
      byte[] wa = new byte[1472];
      byte[] msgsizb = new byte[2];
      int nread, nleft, n, waptr;
      if (debugLevel > 0) System.out.println("Factory Bucket Thread started ...");
      try
      {
        InputStream is = tb.is;
        while (tb.active)
        {
          if (terminate) break;
          if (debugLevel > 0) System.out.println("Waiting for steam data ...");
          isWaiting = true;
          if ((nread = is.read(wa,0,1472)) == -1) 
          {
            tb.isDeactivating = true;
            break;
          }
          nleft = nread;
          waptr = 0;
          while (nleft > 0)
          {                    
            if (tb.ptr == 0) 
            {
              System.arraycopy(wa,waptr,msgsizb,0,2);
              tb.siz = Swap.Short(new DataInputStream(new ByteArrayInputStream(msgsizb)).readShort());
            }
            n = tb.siz - tb.ptr;
            if (nleft >= n)  // at least enough
            {
              System.arraycopy(wa,waptr,tb.buf,tb.ptr,n);
              InterpretIncomingData(tb.buf,tb.siz,false);
              waptr += n;                 // move the read buffer pointer along
              nleft -= n;                 // decrement what's remaining to be read
              tb.ptr = 0;                 // reset the bucket pointer
            }
            else
            {
              System.arraycopy(wa,waptr,tb.buf,tb.ptr,nleft);
              tb.ptr += nleft;
              nleft = 0;
            }
            //NotifyDeferredCallbacks();
          }                    
        }
        tb.sck.close();
      }
      catch (InterruptedIOException e) 
      {
        System.out.println(" InterruptedIOException : " + e);
      }
      catch (IOException e)
      {
        if (debugLevel > 1) System.out.println(" IOException : " + e);
      }
      finally
      {
        removeBucketThread(tb);
      }
    }
  }
  	
  /**
   * TFactory constructor comment.
   */
  public TFactory() 
  {
    atp = new TPacket();
    stp = new TPacket();
    phs = new TPublishedData(this);
    ghs = new TGlobalsData(this);
    stfThrd = new TFactoryThread(stp);
    atfThrd = new TFactoryThread(atp);
    gtfThrd = null;
    active = true;
    stfThrd.start();
    atfThrd.start();
    tfwdThrd.start();
    String dbglvlStr;
    if ((dbglvlStr = System.getProperty("debug.level")) != null)
    {
      debugLevel = Integer.parseInt(dbglvlStr);
    }
  }
  /**
   * cancel() terminates an active link
   */
  public int cancel(TLink eqm)
  {
    if (eqm.linkId >= 0) linkTable[eqm.linkId] =  null;// return the spot in the table
    if (!eqm.active) return 0;         // already inactive
    if (eqm.sub == null) return 0;     // never allocated 
    if (TMode.getBaseMode(eqm.sub.mode) > TMode.CM_SINGLE)
    {
      if (debugLevel > 0) 
        System.out.println("Thread " + this +  " cancel link " + eqm.linkId + " mode " + TMode.toString(eqm.sub.mode));
      if ((eqm.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
      {
        eqm.sub.mode = TMode.CM_CANCEL | TMode.CM_CONNECT;
      }
      else
      {
        eqm.sub.mode = TMode.CM_CANCEL;  // turn off the link
      }
      establishLink(eqm);
    }
    eqm.active = false;
 // eqm.terminate = true;
    eqm.pending = false;
  //  if (eqm.tb != null) removeBucketThread(eqm.tb);
    return 0;
  }            
  /**
   * Put a new TLink in the link table 
   * Creation date: (8/17/01 5:59:27 PM)
   * @return TINE.TLink
   */
  public TLink createTLink(String devname, String devproperty, TDataType dout, TDataType din, short devaccess) 
  {
    int i;
    for (i=0; i<maximumNumberTLinks && linkTable[i] != null; i++);
    if (i == maximumNumberTLinks) return null;
    linkTable[i] = tNullLink; // place holder for re-entrancy !
    linkTable[i] = new TLink(this,i,devname,devproperty,dout,din,devaccess);
    if (debugLevel > 0) System.out.println("creating TLink " + linkTable[i] + " (" + i + ")");
    return linkTable[i];
  }
  synchronized int establishLink(TLink eqm)
  {
    boolean isSynchronous = ((eqm.devAccess & TAccess.CA_SYNC) != 0);
    if (eqm.linkId < 0) 
    {
      eqm.linkStatus = TErrorList.out_of_client_memory;
      return TErrorList.out_of_client_memory;
    }
    eqm.linkStatus = -1;         // pending
    eqm.active = true;
    if ((eqm.sub.mode & TMode.CM_GLOBAL) == TMode.CM_GLOBAL)
    {
      if (gtfThrd == null)
      {
        gtp = new TPacket(initializer.getGCastPort());
        gtfThrd = new TFactoryGlobalsThread(gtp);
        gtfThrd.start();
      }
      eqm.dOutput.blksin = 0;
      eqm.dOutput.timestamp = time;
      return 0;
    }
    if ((eqm.sub.mode & TMode.CM_MCAST) == TMode.CM_MCAST)
    {
      if (mtfThrd == null)
      {
        amtp = new TPacket(initializer.getMCastPort());
        mtfThrd = new TFactoryThread(amtp);
        mtfThrd.start();
      }
    }
    for (int n=0; n<eqm.dInput.numblks; n++)
    {
      eqm.hdr.prepare((short)(eqm.con.hdrSizeInBytes + eqm.sub.hdrSizeInBytes + eqm.sub.nextDataSizeInBytes));
      eqm.sub.prepare((short)(eqm.sub.mode & 0x0fff));
      if (debugLevel > 0) 
        System.out.println("trying to establish link " + eqm.linkId + 
                           "(" + (n+1) + " of " + eqm.dInput.numblks +
                           " " + eqm.sub.msgSizeInBytes + " bytes)" + 
                           " : " + eqm.expName + " " + eqm.devName + " " + 
                           eqm.devProperty + " " + eqm.devTimeout + " msec");
      try
      {
        TSrvAddress srv = eqm.modAddress.srvAddress;
        ByteArrayOutputStream bs = new ByteArrayOutputStream(1472);
        DataOutputStream ds = new DataOutputStream(bs);
        ds.write(eqm.hdr.dBuffer.toByteArray());
        ds.write(eqm.sub.dBuffer.toByteArray());
        if (srv == null)
        { 
          if (debugLevel > 0) System.out.println("establish link : can't resolve server address");
          return TErrorList.non_existent_fec;
        }
        if (eqm.sub.contract.dBuffer == null) 
        { 
          if (debugLevel > 0) System.out.println("establish link : no address data returned");
          return TErrorList.non_existent_elem;
        }
        ds.write(eqm.sub.contract.dBuffer.toByteArray());
        // package the input data payload ...
        if (eqm.dInput.getArrayLength() > 0) ds.write(eqm.dInput.getDataBuffer(n));
        // send it out ...
        if (isInsideCallback && TMode.getBaseMode(eqm.sub.mode) == TMode.CM_SINGLE) eqm.sub.mode &= ~(TMode.CM_CONNECT);
        if ((eqm.sub.mode & TMode.CM_CONNECT) == TMode.CM_CONNECT)
        {
          if (eqm.tb == null) eqm.tb = new TBucket(eqm.linkId);
          if (eqm.tb.isDeactivating) return -1;
          if (eqm.tb.sck != null)
          {
            OutputStream os = eqm.tb.os;
            os.write(bs.toByteArray());
            os.flush();
          }
        }
        else if (isSynchronous)
        {
          stp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getSrvPort()+srv.srvPortOffset);
  	      stp.s.send(stp.dpOut);
        }
        else
        {
          atp.dpOut = new DatagramPacket(bs.toByteArray(), bs.size(), srv.srvHost, initializer.getSrvPort()+srv.srvPortOffset);
          atp.s.send(atp.dpOut);
        }
        eqm.dOutput.blksin = 0;
        eqm.dOutput.timestamp = time;
  	  }
      catch (IOException e)
      {
        if (debugLevel > 0) System.out.println("establish link : IO Exception");
        if (eqm.tb != null) removeBucketThread(eqm.tb);
        e.printStackTrace();
        return -1;
      }
    }
    if (debugLevel > 0) System.out.println("establish link : success");
    return 0;
  }                                                          
  TLink findTLink(int linkid, int linkstarttime)
  {  
    InetAddress ia;  
    if (linkstarttime == TSrvAddress.BCAST_ID)   /* BCAST or MCAST contract */
    {
      for (int i=0; i<maximumNumberTLinks; i++)        /* cross reference */
      {
        if (linkTable[i] == null || linkTable[i] == tNullLink) continue;
        if (linkTable[i].srvId != linkid) continue;
// what to do here?????
//        ia = linkTable[i].tf.amtp.m.getInetAddress();
//        if (!linkTable[i].modAddress.srvAddress.srvHost.equals(ia)) continue;
        return linkTable[i];
      }
      return null;
    }
    
    if (debugLevel > 0) System.out.println("looking for link " + linkid + 
                        " amoung " + maximumNumberTLinks + " registered links");
    if (linkid < 0 || linkid >= maximumNumberTLinks) return null;
    if (linkTable[linkid] == null || linkTable[linkid].sub == null) return null;
    if (linkTable[linkid].sub.starttime != linkstarttime) return null;
    if (debugLevel > 0) System.out.println("returning " + linkTable[linkid]);
    return linkTable[linkid];
  }          
  TLink findTLink(String keyword)
  {
    keyword = keyword.trim();
    if (debugLevel > 0) System.out.println("looking for link " + keyword + 
                        " amoung " + maximumNumberTLinks + " registered links");
    for (int i=0; i<maximumNumberTLinks; i++)
    {
       if (linkTable[i] == null || linkTable[i] == tNullLink) continue;
       if (linkTable[i].sub.mode != TMode.CM_GLOBAL) 
       {
         if (debugLevel > 0) System.out.println(linkTable[i].devProperty + " is not a global");
         continue;
       }
       if (debugLevel > 0) System.out.println("Compare " + keyword  + " against " + linkTable[i].devProperty);
       if (keyword.compareTo(linkTable[i].devProperty) == 0) // found it
       {
         if (debugLevel > 0) System.out.println("returning " + linkTable[i]);
         return linkTable[i];
       }
    }
    return null;
  }          
  /**
   * Insert the method's description here.
   * Creation date: (8/24/01 3:51:37 PM)
   * @return TINE.TFactory
   */
  public static TFactory getInstance() 
  {
    if (instance==null) instance=new TFactory();
    return instance;
  }
  /**
   * Put a new TLink in the link table 
   * Creation date: (8/17/01 5:59:27 PM)
   * @return TINE.TLink
   */
  public void linkReady(TLink link)
  {
  	linkTable[link.linkId] = link;
  	if (debugLevel > 0) System.out.println("creating TLink " + linkTable[link.linkId] + " (" + link.linkId + ")");
  }
  /**
   * Put a new TLink in the link table 
   * Creation date: (8/17/01 5:59:27 PM)
   * @return TINE.TLink
   */
  public int registerLink(TLink link)
  {
  	int i;
  	for (i=0; i<maximumNumberTLinks && linkTable[i] != null; i++);
  	if (i == maximumNumberTLinks) return -1;
  	linkTable[i] = tNullLink; // place holder for re-entrancy !
  	return i;
  }
  /**
   * Remove a TLink from the link table
   * Creation date: (8/17/01 6:00:09 PM)
   */
  public void removeTLink(TLink eqm) 
  {
  	int i;
  	for (i=0; i<maximumNumberTLinks && linkTable[i] != eqm; i++);
  	if (i == maximumNumberTLinks) return;
  	if (debugLevel > 0) System.out.println("removing link " + linkTable[i]);
    if (eqm.tb == null)
    {
  	  linkTable[i] = null;
    }
    else
    {
      linkTable[i].terminate = false;
    }
  }
  /**
   * Insert the method's description here.
   * Creation date: (8/23/01 10:51:32 AM)
   */
  public void shutdown() 
  { 
  	active = false;  	
  }
}
