package TINE;
/**
 * @brief Interface to a TLink callback class for use in asynchronous links
 * 
 * This class provides an inteface to a TLinkCallback object whose callback 
 * method will be called which the link state of an asynchronous link has 
 * changed, either due to incoming data or non-zero status code.
 *  
 * \b Example:
 * 
 * \include eg_attach4.java
 */

public interface TLinkCallback
{
  /**
   * @brief The method which will be called when new data arrive.
   * 
   * @param link is the TINE TLink object which created the link.  The
   * property values of this object will reflect the current link status
   * and timestamp (among other things) of the associated link.
   * 
   * @see TINE.TLink#attach
   */
    public void callback(TLink link);      
}
