package TINE;
/**
 * @brief tine access modes and modifiers
 */
public class TMode
{
  public static final short CM_CANCEL = 0;
  public static final short CM_SINGLE = 1;
  public static final short CM_REFRESH = 2;
  public static final short CM_POLL = 3;
  public static final short CM_REGISTER = 4;
  public static final short CM_GLOBAL = 5;  
  public static final short CM_BCAST = 0x100;
  public static final short CM_NETWORK = 0x200;
  public static final short CM_MCAST = 0x200;
  public static final short CM_USE_ON_ERROR = 0x400;
  public static final short CM_RETRY = 0x800;
  public static final short CM_GROUPED = 0x1000;
  public static final short CM_CONNECT = 0x2000;
  public static final short CM_WAIT = 0x4000;

  /**
   * @brief The base mode for the specified mode parameter
   * 
   * @param mode is the mode parameter for which the base mode 
   * (CM_SINGLE to CM_GLOBAL) is desired 
   * 
   * @return The base mode for the specified mode parameter
   */
  public static final short getBaseMode(short mode)
  {
    return (short)(mode & 0xff);
  }
  /**
   * @brief The string representation for the specified mode parameter
   * 
   * @param mode is the mode parameter for which the string representation 
   * is desired 
   * 
   * @return The string representation for the specified mode parameter
   */
  public static String toString(short mode)
  {
    String add = "";
    switch (mode & 0xff00)
    {
      case CM_BCAST: add = " + broadcast";
      case CM_NETWORK: add = " + network";
      case CM_USE_ON_ERROR: add = " + use on error";
      case CM_RETRY: add = " + retry on error";
      case CM_GROUPED: add = " + grouped";
      case CM_CONNECT: add = " + connected";
      case CM_WAIT: add = " + wait";
    }
    switch (mode & 0xff)
    {
      case CM_CANCEL: return "cancel" + add;
      case CM_SINGLE: return "single" + add; 
      case CM_REFRESH: return "refresh" + add;
      case CM_POLL: return "poll" + add;
      case CM_REGISTER: return "register" + add;
      case CM_GLOBAL: return "receive" + add;  
      default: return "unknown";     
    }
  }
}