//: TLink.java
package TINE;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
// Producer Header structure (published data)
public class TPublishedData
{
  short msgSizeInBytes;
  short subId;
  short linkCompCode;
  short numblks;
  short blknum;
  short mtu;
  short dArrayLength;
  byte  dFormat;
  byte  linkCounter;
  int   dTimestamp;
  int   dTimestampMSEC;
  int   linkStarttime;

  static final int hdrSizeInBytes = (7*2 + 2*1 + 2*4);
  static ByteArrayInputStream dBuffer;

  void prepare(byte[] data, int length)
  {
    TLink eqm;
    int bytesread = 0, pktsread = 0;
    int datasize, fmtsize, doutsize, gapsize, gapptr;
    short len;
    byte[] d;
    try
    {
      dBuffer = new ByteArrayInputStream(data);
      DataInputStream ds = new DataInputStream(dBuffer);
      // overall message length:
      len = Swap.Short(ds.readShort());
      if (len < length)
      {
        if (tf.debugLevel > 0)
        {
          String msg = "recv: " + length + " bytes : hdr " + len + " " + TDataTime.toString(System.currentTimeMillis());
          System.out.println(msg);
        }
        return;
      }
      bytesread = 2;
      while (bytesread < length)
      {
          msgSizeInBytes = Swap.Short(ds.readShort());
          subId = Swap.Short(ds.readShort());
          linkCompCode = Swap.Short(ds.readShort());
          numblks = Swap.Short(ds.readShort());
          blknum = Swap.Short(ds.readShort());
          mtu = Swap.Short(ds.readShort());
          dTimestampMSEC = Swap.Short(ds.readShort());
          dFormat = ds.readByte();
          linkCounter = ds.readByte();
          dTimestamp = Swap.Long(ds.readInt());
          linkStarttime = Swap.Long(ds.readInt());
          
          if (tf.debugLevel > 0)
            System.out.println("data : blknum " + blknum + " from " + numblks + " <" + linkCompCode + ">");
          bytesread += msgSizeInBytes;
          if ((eqm=tf.findTLink((int)subId,linkStarttime)) == null)
          {
        	// assume the worst and bail out ...
            if (tf.debugLevel > 0) System.out.println("id " + subId + " not found!");
            break;
          }
          eqm.dOutput.numblks = numblks;
          eqm.linkStatus = linkCompCode;
          eqm.linkCounter = linkCounter;
          eqm.linkStale = true;
          datasize = eqm.dOutput.dArrayLength * TFormat.formatSizeOf(dFormat);// size of incoming data in bytes
          doutsize = msgSizeInBytes - hdrSizeInBytes; // size of data + msg in bytes
          gapsize = 0; gapptr = 0;
          pktsread++;
          
          switch (linkCompCode)
          {
            case TErrorList.server_redirection:
              d = new byte[TDataType.RPCERR_SIZE];
              ds.read(d,0,16); // redirected device server name
              eqm.expName = new String(d);
              ds.read(d,0,16); // redirected device property name              
              eqm.devProperty = new String(d);
              break;
            case TErrorList.get_subscription_id:
              eqm.srvId = Swap.Short(ds.readShort());
              break;
            default: 
              if (doutsize < TDataType.RPCERR_SIZE && eqm.dOutput.blksin == 0)
              {
                eqm.dOutput.bytesin = 0;
                continue;
              }
    
              if ((linkCompCode & TErrorList.CE_SENDDATA) != TErrorList.CE_SENDDATA) datasize = 0;
              if ((gapsize=eqm.dOutput.bytesin+doutsize-datasize) > TDataType.RPCERR_SIZE) continue;
              if (gapsize > 0)
              {
                gapptr = 0;
                if (doutsize < TDataType.RPCERR_SIZE)
                {
                  gapsize = doutsize;
                  gapptr = TDataType.RPCERR_SIZE - gapsize;
                }
                doutsize -= gapsize;
                eqm.dOutput.bytesin += gapsize;
              }
              else gapsize = 0;
              if ((linkCompCode & TErrorList.CE_SENDDATA) != TErrorList.CE_SENDDATA)
              {
                eqm.dOutput.blknum = blknum;
                eqm.dOutput.blksin = 1;
                eqm.dOutput.numblks = 1;
                break;
              }
            case 0:
              if (eqm.dOutput.dFormat == TFormat.CF_DEFAULT)
              {
                eqm.dOutput.dFormat = dFormat;
                if ((fmtsize = TFormat.formatSizeOf(eqm.dOutput.dFormat)) < 1) continue; 
                eqm.dOutput.dArrayLength = (doutsize/fmtsize) * eqm.dOutput.numblks;
                eqm.adjustDefaultValues = true;
              }
              d = new byte[msgSizeInBytes-hdrSizeInBytes-gapsize];
              ds.read(d,0,d.length);
              // push this data set into the corresponding TDataType object
              eqm.dOutput.update(d,blknum,linkCounter);
              break;
          }  
          if (gapsize > 0)
          {
            d = new byte[gapsize];
            ds.read(d,0,d.length);
            System.arraycopy(d,0,eqm.linkStatusStringBuffer,gapptr,gapsize);
          }      
          eqm.dOutput.bytesin += doutsize;
          if (eqm.dOutput.blksin == eqm.dOutput.numblks)
          {
            eqm.dOutput.timestamp = dTimestamp;
            eqm.dOutput.timestampMSEC = dTimestampMSEC;
          }
      }
      if (tf.debugLevel > 0) System.out.println(pktsread + " incoming contracts read");
      ds.close();
      dBuffer.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }	 
  }                  
  TFactory tf;
 /**
 * Insert the method's description here.
 * Creation date: (8/21/01 6:33:36 PM)
 */
  public TPublishedData(TFactory tfactory) 
  {
    tf = tfactory;
  }
}