/*
 * Created on May 14, 2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code Template
 */
package TINE;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @brief User-defined structure object
 * 
 * Use TStruct to register user defined structures. 
 * Call the fill() method for each field (or array of fields) and, finally, 
 * call the fill() method with format type CF_NULL when the definition is complete
 */
public final class TStruct
{
  private static Set StructList = new HashSet();
  private static StructStruct[] ss = new StructStruct[100];
  public static int numRegisteredStructs = 0;
  private static final class StructStruct
  {
    String name;             /* structure tag */
    int  rawlen;             /* network data size */
    int  size;               /* local structure size */
    int  num;                /* maximum number of structure sets buffered */
    List structFormatList;   /* structure definition list */
    String toString(byte[] data)
    {
      String addstr = new String("");
      StructFormat sf;
      int len;
      byte[] bstr;
      try
      {
        ByteArrayInputStream dis = new ByteArrayInputStream(data);
        DataInputStream ds = new DataInputStream(dis);
        for (int n=0; n<structFormatList.size(); n++)
        {
          if ((sf = (StructFormat)structFormatList.get(n)) == null) break;
          if ((len = sf.siz) < 1) continue;
            addstr = addstr.concat("["+n+"] ->");        
            switch (sf.fmt)
            {
              case TFormat.CF_BYTE:
                bstr = new byte[len];
                addstr = addstr.concat(" " + ds.readByte());
                for (int i=1; i<len; i++) addstr.concat("," + ds.readByte());
                break;
              case TFormat.CF_TEXT:
                bstr = new byte[len];
                ds.read(bstr, 0, len);
                addstr = addstr.concat(new String(bstr).trim());
                break;
              case TFormat.CF_SHORT:
                addstr = addstr.concat(" " + Swap.Short(ds.readShort()));
                for (int i=1; i<len; i++)
                { 
                  addstr = addstr.concat(","+Swap.Short(ds.readShort()));
                }
                break;
              case TFormat.CF_LONG:
                addstr = addstr.concat(" " + Swap.Long(ds.readInt())); 
                for (int i=1; i<len; i++)
                { 
                  addstr = addstr.concat(","+Swap.Long(ds.readInt()));
                }
                break;
              case TFormat.CF_FLOAT:
                bstr = new byte[4];
                ds.read(bstr,0,4);
                addstr = addstr.concat(" " + Swap.Float(bstr));
                for (int i=1; i<len; i++)
                {
                  ds.read(bstr,0,4);
                  addstr = addstr.concat("," + Swap.Float(bstr));
                }
                break;
              case TFormat.CF_DOUBLE:
                bstr = new byte[8];
                ds.read(bstr,0,8);
                addstr = addstr.concat(" " + Swap.Double(bstr));
                for (int i=1; i<len; i++)
                {
                  ds.read(bstr,0,8);
                  addstr = addstr.concat(","+Swap.Double(bstr));
                }
                break;
            }
            addstr = addstr.concat("\n");
        } 
      }
      catch (IOException e)
      {
        e.printStackTrace();
      }
      return addstr;       
    }
  }
  private static final class StructFormat
  {
    int siz;
    int fmt;
    int off;
    int addr;
  }
  private static StructStruct getStructStruct(String tag)
  {
    int i = 0;
    if (!StructList.isEmpty())
    {
      StructList.toArray(ss);
      for (i=0; i<ss.length; i++) if (ss[i].name.compareTo(tag) == 0) return ss[i];
    }
    if (numRegisteredStructs == 100) return null;
    StructStruct ns = new StructStruct();
    ns.name = new String(tag);
    ns.structFormatList = new LinkedList();
    StructList.add(ns);
    ss[i] = ns;
    numRegisteredStructs++;
    return ns; 
  }
  /**
   * @brief Registers an element of a tagged structure
   * 
   * If a data acquisition needs to make use of user-defined structures,
   * it is strongly suggested to register such structures at both the
   * server and client ends.  This entails passing the structure information
   * at initialization time and giving the structure a 'tag' (i.e. an 8-character
   * name). Structure information is passed by calling the fill() method for
   * each element of the structure in succession. When the entire structure 
   * has been registered the fill() method should be call once more with a
   * a TFormat.CF_NULL data type, which effectively closes the structure
   * registration.  
   * 
   * @param tag is the (up to) eight character structure tag name
   * @param fmt is the TINE Data type appropriate to the structure element
   * @param siz is the array size of the structure element
   * 
   * \b Example:
   * 
   * \include eg_struct1.java
   */
  public static void fill(String tag,short fmt,int siz) 
  { 
    StructStruct s = getStructStruct(tag);
    if (s == null) return;
    if (fmt == TFormat.CF_NULL) // registration finished
    {
      s.num = siz;      
    }
    else
    {
      StructFormat sf = new StructFormat();
      sf.siz = siz;
      sf.fmt = fmt;
      sf.off = s.rawlen; 
      sf.addr = s.rawlen; 
      s.size += siz;
      s.rawlen += siz * TFormat.formatSizeOf(fmt);
      s.structFormatList.add(sf);
    }
  }
  /**
   * @brief Returns the contents of the given tagged structure as a string
   * 
   * @param tag is the (up to) eight character structure tag name
   * @param data is a byte array reference to the tagged structure's data.
   * 
   * @see TINE.TStruct#fill for example
   * 
   */
  public static String toString(String tag, byte[] data)
  {
    StructStruct s = getStructStruct(tag);
    if (s == null) return new String("Structure " + tag + " not registered");
    String ss = new String();
    int ptr = 0;
    byte[] sdata = new byte[s.size];
    for (int i=0; i<s.num; i++)
    {    
      System.arraycopy(data,ptr,sdata,0,s.size);
      ss = ss.concat(s.toString(sdata));
      ptr += s.size;
    }
    return ss;
  }
}

