#include "prolog.h"
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "errors.h"
#include "rpctypes.h"
#include "almsrv.h"
#include "toolkit.h"
#include "sedac.h"
#include "sedacsrv.h"

SedacReadArg SedacRdArgument;
SedacWriteArg SedacWrArgument;
SedacPackReadArg SedacPackRdArgument;
SedacPackReadRet SedacPackRdReturn;
SedacPackWriteArg SedacPackWrArgument;
SedacPackWriteRet SedacPackWrReturn;
SedacPackMixArg SedacPackMxArgument;
SedacPackMixRet SedacPackMxReturn;

int sedeqm(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access){
  int devnr,prpid,i,cc;
  long ldat;
  u16 dat,cra,bsa;

  devnr = GetDeviceNumber("SEDEQM",devName);
  if (devnr < 0) return illegal_equipment_number;

  prpid = GetPropertyId("SEDEQM",devProperty);
 
  switch (prpid)
  {
    case PRP_SEDACREAD:
      if (din == NULL) return argument_list_error;
      switch (din->dFormat)
      {
	case CF_STRUCT:
	  if (din->dArrayLength != sizeof(SedacReadArg)) 
            return  dimension_error;
          memcpy(&SedacRdArgument,din->data.vptr,sizeof(SedacReadArg));      
	  cra = SedacRdArgument.cra;
	  bsa = SedacRdArgument.bsa;
	  break;
	case CF_SHORT:
	  if (din->dArrayLength != 2) return dimension_error;
	  cra = din->data.sptr[0];
	  bsa = din->data.sptr[1];
	  break;
	case CF_LONG:
	  if (din->dArrayLength != 2) return dimension_error;
	  cra = din->data.lptr[0];
	  bsa = din->data.lptr[1];
	  break;
        default:
	  return illegal_format;
      }
      if ((cc=rsedac(devnr,cra,bsa,&dat)) != 0) return sedac_error;
      ldat = dat;
      return putValuesFromLong(dout,&ldat,1);
    case PRP_SEDACWRITE:
      if (din == NULL) return argument_list_error;
      switch (din->dFormat)
      {
	case CF_STRUCT:
          if (din->dArrayLength != sizeof(SedacWriteArg)) return dimension_error;
          memcpy(&SedacWrArgument,din->data.vptr,sizeof(SedacWriteArg));      
          cra = SedacWrArgument.cra;
          bsa = SedacWrArgument.bsa;
          dat = (u16)SedacWrArgument.val;
          break;
        case CF_SHORT:
	  if (din->dArrayLength != 3) return dimension_error;
	  cra = din->data.sptr[0];
	  bsa = din->data.sptr[1];
          dat = din->data.sptr[2];
	  break;
        case CF_LONG:
	  if (din->dArrayLength != 3) return dimension_error;
	  cra = din->data.lptr[0];
	  bsa = din->data.lptr[1];
          dat = din->data.lptr[2];
	  break;
        default:
	  return illegal_format;
      }
      return (wsedac(devnr,cra,bsa,dat) != 0) ? sedac_error : 0;
    case PRP_SEDACPACKRD:
      break;
    case PRP_SEDACPACKWR:
      break;
    case PRP_SEDACPACKMX:
      break;
    default:
        return illegal_property;
  }
}
