/*
   Module: tdaemon.c (daemon server for UNIX machines)
*/
#include "prolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "errors.h"
#include "rpctypes.h"
#include "database.h"
#include "swaplib.h"
#include "almsrv.h"
#include "brdcast.h"
#include "toolkit.h" 
#include "unixcmd.h"

#define ccerr(i) { cc = i; goto err; }
#define PRP_NGLOBALS     1
#define PRP_GLOBALS      2

GlobalListStruct *tgtGlobals = NULL;
int ntgtGlobals = 0;

void rcvglobal(int id,int cc)
{
  int n, i = id - 1;
  GlobalListStruct *t = &tgtGlobals[i];
  short *sdat = (short *)t->data;
  long *ldat = (long *)t->data;
  float *fdat = (float *)t->olddata;
  double *ddat = (double *)t->data;

  if (cc)
  {
    printf("global %d : comp code %d %s\n>",id,cc,RPCLastErrorString);
    return;
  }
  switch (t->format)
  {
    case CF_FLOAT:
       memcpy(t->olddata,t->data,t->size*sizeof(float)); break;
    case CF_SHORT:
      for (n=0; n<t->size; n++) fdat[n] = (float)sdat[n]; break;
    case CF_LONG:
      for (n=0; n<t->size; n++) fdat[n] = (float)ldat[n]; break;
    case CF_DOUBLE:
      for (n=0; n<t->size; n++) fdat[n] = (float)ddat[n]; break;
  }
}

int getGlobalsInfo(char *fn)
{
  FILE *fp = NULL;
  int i=0,cc=0,done=0;
  int key_col,fmt_col,siz_col;
  char str[256],hdr[256],scratch[16];

  if ((ntgtGlobals=GetDataBaseSize(fn)) <= 0) ccerr(-ntgtGlobals);
  if ((tgtGlobals=(GlobalListStruct *)calloc(ntgtGlobals,sizeof(GlobalListStruct))) == NULL)
    ccerr(out_of_local_memory);
  if ((fp=fopen(fn,"r")) == NULL) ccerr(no_such_file);
  while (fgets(str,255,fp) != NULL)
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((key_col=findcol(hdr,"KEYWORD")) < 0) ccerr(no_such_column);
      if ((fmt_col=findcol(hdr,"FORMAT")) < 0) ccerr(no_such_column);
      if ((siz_col=findcol(hdr,"SIZE")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(tgtGlobals[i].keyword,colptr(key_col,str),16);
    strncpy(scratch,colptr(fmt_col,str),16);
    tgtGlobals[i].format = ftoi(scratch);
    strncpy(scratch,colptr(siz_col,str),16);
    tgtGlobals[i].size = atoi(scratch);
    if ((tgtGlobals[i].data=calloc(tgtGlobals[i].size,fmtsizeof(tgtGlobals[i].format))) == NULL)
      ccerr(out_of_local_memory);
    if ((tgtGlobals[i].olddata=calloc(tgtGlobals[i].size,sizeof(float))) == NULL)
      ccerr(out_of_local_memory);
    i++;
  }
err:
  return cc;
}
void dnminit(void)
{
  int i,cc;
  DTYPE dout;

  if (getGlobalsInfo("nglobals.csv")) return;
  for (i=0; i<ntgtGlobals; i++)
  {
    dout.dFormat = tgtGlobals[i].format;
    dout.dArrayLength = tgtGlobals[i].size;
    dout.data.vptr = tgtGlobals[i].data;
    cc = recvNetGlobalEx(tgtGlobals[i].keyword,&dout,rcvglobal,i+1);
    if (cc < 0) printf("NETGLOBAL : %s\n>",erlst[-cc]);
    switch (tgtGlobals[i].format)
    {
      case CF_SHORT:
      case CF_LONG:
      case CF_FLOAT:
      case CF_DOUBLE:
        RegisterUserCommand(tgtGlobals[i].keyword,NULL,NULL,(float *)tgtGlobals[i].olddata,CA_READ);
        break;
    }
  }
}
void dnmbkg(void)
{
}
int dnmeqm(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access)
{
  int devnr,prpid,i,cc;

  devnr = GetDeviceNumber("DNMEQM",devName);
  if (devnr < 0) return illegal_equipment_number;
  prpid = GetPropertyId("DNMEQM",devProperty);

  switch (prpid)
  {
    case PRP_NGLOBALS:
        return putValuesFromLong(dout,(UINT32 *)&ntgtGlobals,1);
    case PRP_GLOBALS:
        if (dout->dFormat != CF_NAME16) return illegal_format;
        for (i=0; i<dout->dArrayLength; i++)
          strncpy(dout->data.cptr+i*16,tgtGlobals[i].keyword,16);
        return 0;
    default:
        return illegal_property;
  }
}
static char srvname[17];
extern int BroadcastDaemon;

void PreSystemInit(void)
{
  char *c,hstname[64],desc[64];
  int cc,len = sizeof(hstname);

  BroadcastDaemon = TRUE;
  StartupDebug = 0;

  if (gethostname(hstname,len) < 0)
  {
    perror("gethostname");
    printf("cannot resolve host name!\n");
    exit(1);
  }
  if ((c=strchr(hstname,'.')) != NULL) *c = 0;
  strncpy(srvname,hstname,14); srvname[14] = 0;
  strupr(srvname); strcat(srvname,"-D");
  sprintf(desc,"[DMN]%s daemon",srvname);
  if ((cc=RegisterFecNameEx(srvname,desc,"UNIX",hstname,"none",hstname,20,"SERVICE")) != 0)
  {
    printf("cannot register server name %s : %s\n",srvname,erlst[cc]);
  }
}
void PostSystemInit(void)
{
  int i,cc;

  RegisterEquipmentModule(srvname,"DNMEQM",0,dnmeqm,dnminit,dnmbkg,1000,NULL);

  if ((cc=RegisterPropertyEx("DNMEQM","NGLOBALS",1,CF_LONG,CA_READ,"number of targetted globals",PRP_NGLOBALS)) < 0)
  {
    printf("Can't register property NGLOBALS : %s\n>",erlst[-cc]);
  }
  if ((cc=RegisterPropertyEx("DNMEQM","GLOBALS",1,CF_NAME16,CA_READ,"targetted globals",PRP_GLOBALS)) < 0)
  {
    printf("Can't register property GLOBALS : %s\n>",erlst[-cc]);
  }
}

