Attribute VB_Name = "PrintForm"
'--------------------------------------------------------------------
'
'   PrintForm.Bas
'
'   Aus: Microsoft Knowledge Base (mit leichten Modifikationen)
'
'
'
'   Exportierte Fnuktionen:
'       prfPrintForm         -       Form mit oder ohne Kopf ausdruchen
'       prfPrintScreen       -       Bildschirm ausdrucken
'       prfPrintActive       -       Aktives Fenster ausdrucken
'       prfSetMargins         -       Randbreiten einstellen
'
'--------------------------------------------------------------------


'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' Visual Basic 4.0 16/32 Capture Routines
'
' This module contains several routines for capturing windows into a
' picture.  All the routines work on both 16 and 32 bit Windows
' platforms.
' The routines also have palette support.
'
' CreateBitmapPicture - Creates a picture object from a bitmap and
' palette
' CaptureWindow - Captures any window given a window handle
' CaptureActiveWindow - Captures the active window on the desktop
' CaptureForm - Captures the entire form
' CaptureClient - Captures the client area of a form
' CaptureScreen - Captures the entire screen
' PrintPictureToFitPage - prints any picture as big as possible on
' the page
'
' NOTES

'    - No error trapping is included in these routines
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'


Option Explicit
Option Base 0

'//
'// Randbreiten
'//
Dim m_left As Integer
Dim m_right As Integer
Dim m_top As Integer
Dim m_btm As Integer
Dim m_orient As Integer

Private Type PALETTEENTRY
    peRed As Byte
    peGreen As Byte
    peBlue As Byte
    peFlags As Byte
End Type
Private Type LOGPALETTE
    palVersion As Integer
    palNumEntries As Integer
    palPalEntry(255) As PALETTEENTRY  ' Enough for 256 colors
End Type
Private Type GUID
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(7) As Byte
End Type
#If Win32 Then
    Private Const RASTERCAPS As Long = 38
    Private Const RC_PALETTE As Long = &H100
    Private Const SIZEPALETTE As Long = 104
Private Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type
Private Declare Function CreateCompatibleDC Lib "GDI32" ( _
                                                         ByVal hdc As Long) As Long
Private Declare Function CreateCompatibleBitmap Lib "GDI32" ( _
                                                             ByVal hdc As Long, ByVal nWidth As Long, _
                                                             ByVal nHeight As Long) As Long
Private Declare Function GetDeviceCaps Lib "GDI32" ( _
                                                    ByVal hdc As Long, ByVal iCapabilitiy As Long) As Long
Private Declare Function GetSystemPaletteEntries Lib "GDI32" ( _
                                                              ByVal hdc As Long, ByVal wStartIndex As Long, _
                                                              ByVal wNumEntries As Long, lpPaletteEntries As PALETTEENTRY) _
        As Long
Private Declare Function CreatePalette Lib "GDI32" ( _
                                                    lpLogPalette As LOGPALETTE) As Long
Private Declare Function SelectObject Lib "GDI32" ( _
                                                   ByVal hdc As Long, ByVal hObject As Long) As Long
Private Declare Function BitBlt Lib "GDI32" ( _
                                             ByVal hDCDest As Long, ByVal XDest As Long, _
                                             ByVal YDest As Long, ByVal nWidth As Long, _
                                             ByVal nHeight As Long, ByVal hDCSrc As Long, _
                                             ByVal XSrc As Long, ByVal YSrc As Long, ByVal dwRop As Long) _
        As Long
Private Declare Function DeleteDC Lib "GDI32" ( _
                                               ByVal hdc As Long) As Long
Private Declare Function GetForegroundWindow Lib "USER32" () _
        As Long
Private Declare Function SelectPalette Lib "GDI32" ( _
                                                    ByVal hdc As Long, ByVal hPalette As Long, _
                                                    ByVal bForceBackground As Long) As Long
Private Declare Function RealizePalette Lib "GDI32" ( _
                                                     ByVal hdc As Long) As Long
Private Declare Function GetWindowDC Lib "USER32" ( _
                                                   ByVal hWnd As Long) As Long
Private Declare Function GetDC Lib "USER32" ( _
                                             ByVal hWnd As Long) As Long
Private Declare Function GetWindowRect Lib "USER32" ( _
                                                     ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function ReleaseDC Lib "USER32" ( _
                                                 ByVal hWnd As Long, ByVal hdc As Long) As Long
Private Declare Function GetDesktopWindow Lib "USER32" () As Long
Private Type PicBmp
    size As Long
    Type As Long
    hBmp As Long
    hPal As Long
    Reserved As Long
End Type
Private Declare Function OleCreatePictureIndirect _
        Lib "olepro32.dll" (PicDesc As PicBmp, RefIID As GUID, _
                            ByVal fPictureOwnsHandle As Long, IPic As IPicture) As Long

#End If
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CreateBitmapPicture
'    - Creates a bitmap type Picture object from a bitmap and palette
'
' hBmp
'    - Handle to a bitmap
'
' hPal
'    - Handle to a Palette
'    - Can be null if the bitmap doesn't use a palette
'
' Returns
'    - Returns a Picture object containing the bitmap
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Public Function CreateBitmapPicture(ByVal hBmp As Long, _
                                    ByVal hPal As Long) As Picture
    
    Dim r As Long
    Dim pic As PicBmp
    ' IPicture requires a reference to "Standard OLE Types"
    Dim IPic As IPicture
    Dim IID_IDispatch As GUID
    ' Fill in with IDispatch Interface ID
With IID_IDispatch
    .Data1 = &H20400
    .Data4(0) = &HC0
    .Data4(7) = &H46
    End With
    ' Fill Pic with necessary parts
With pic
    .size = Len(pic)          ' Length of structure
    .Type = vbPicTypeBitmap   ' Type of Picture (bitmap)
    .hBmp = hBmp              ' Handle to bitmap
    .hPal = hPal              ' Handle to palette (may be null)
    End With
    ' Create Picture object
    r = OleCreatePictureIndirect(pic, IID_IDispatch, 1, IPic)
    ' Return the new Picture object
    Set CreateBitmapPicture = IPic

End Function

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CaptureWindow
'    - Captures any portion of a window
'
' hWndSrc
'    - Handle to the window to be captured
'
' Client
'    - If True CaptureWindow captures from the client area of the
'      window
'    - If False CaptureWindow captures from the entire window
'
' LeftSrc, TopSrc, WidthSrc, HeightSrc
'    - Specify the portion of the window to capture
'    - Dimensions need to be specified in pixels
'
' Returns
'    - Returns a Picture object containing a bitmap of the specified
'      portion of the window that was captured
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
''''''
'
Private Function CaptureWindow(ByVal hWndSrc As Long, _
                               ByVal client As Boolean, ByVal LeftSrc As Long, _
                               ByVal TopSrc As Long, ByVal WidthSrc As Long, _
                               ByVal HeightSrc As Long) As Picture
    Dim hDCMemory As Long
    Dim hBmp As Long
    Dim hBmpPrev As Long
    Dim r As Long
    Dim hDCSrc As Long
    Dim hPal As Long
    Dim hPalPrev As Long
    Dim RasterCapsScrn As Long
    Dim HasPaletteScrn As Long
    Dim PaletteSizeScrn As Long
    Dim LogPal As LOGPALETTE
    
    ' Depending on the value of Client get the proper device context
    If client Then
        hDCSrc = GetDC(hWndSrc) ' Get device context for client area
    Else
        hDCSrc = GetWindowDC(hWndSrc) ' Get device context for entire
        ' window
    End If
    
    ' Create a memory device context for the copy process
    hDCMemory = CreateCompatibleDC(hDCSrc)
    ' Create a bitmap and place it in the memory DC
    hBmp = CreateCompatibleBitmap(hDCSrc, WidthSrc, HeightSrc)
    hBmpPrev = SelectObject(hDCMemory, hBmp)
    
    ' Get screen properties
    RasterCapsScrn = GetDeviceCaps(hDCSrc, RASTERCAPS) ' Raster
    'capabilities
    HasPaletteScrn = RasterCapsScrn And RC_PALETTE       ' Palette
    'support
    PaletteSizeScrn = GetDeviceCaps(hDCSrc, SIZEPALETTE) ' Size of
    ' palette
    ' If the screen has a palette make a copy and realize it
    If HasPaletteScrn And (PaletteSizeScrn = 256) Then
        ' Create a copy of the system palette
        LogPal.palVersion = &H300
        LogPal.palNumEntries = 256
        r = GetSystemPaletteEntries(hDCSrc, 0, 256, _
                                    LogPal.palPalEntry(0))
        hPal = CreatePalette(LogPal)
        ' Select the new palette into the memory DC and realize it
        hPalPrev = SelectPalette(hDCMemory, hPal, 0)
        r = RealizePalette(hDCMemory)
    End If
    
    ' Copy the on-screen image into the memory DC
    r = BitBlt(hDCMemory, 0, 0, WidthSrc, HeightSrc, hDCSrc, _
               LeftSrc, TopSrc, vbSrcCopy)
    ' Remove the new copy of the  on-screen image
    hBmp = SelectObject(hDCMemory, hBmpPrev)
    ' If the screen has a palette get back the palette that was
    ' selected in previously
    If HasPaletteScrn And (PaletteSizeScrn = 256) Then
        hPal = SelectPalette(hDCMemory, hPalPrev, 0)
    End If
    ' Release the device context resources back to the system
    r = DeleteDC(hDCMemory)
    r = ReleaseDC(hWndSrc, hDCSrc)
    ' Call CreateBitmapPicture to create a picture object from the
    ' bitmap and palette handles.  Then return the resulting picture
    ' object.
    Set CaptureWindow = CreateBitmapPicture(hBmp, hPal)
End Function
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CaptureScreen
'    - Captures the entire screen
'
' Returns
'    - Returns a Picture object containing a bitmap of the screen
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Private Function CaptureScreen() As Picture
    #If Win32 Then
        Dim hWndScreen As Long
    #ElseIf Win16 Then
        Dim hWndScreen As Integer
    #End If
    ' Get a handle to the desktop window
    hWndScreen = GetDesktopWindow()
    ' Call CaptureWindow to capture the entire desktop give the handle
    ' and return the resulting Picture object
    Set CaptureScreen = CaptureWindow(hWndScreen, False, 0, 0, _
                                      Screen.Width \ Screen.TwipsPerPixelX, _
                                      Screen.Height \ Screen.TwipsPerPixelY)
End Function
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CaptureForm
'    - Captures an entire form including title bar and border
'
' frmSrc
'    - The Form object to capture
'
' Returns
'    - Returns a Picture object containing a bitmap of the entire
'      form
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Private Function CaptureForm(frmSrc As Form) As Picture
    ' Call CaptureWindow to capture the entire form given it's window
    ' handle and then return the resulting Picture object
    Set CaptureForm = CaptureWindow(frmSrc.hWnd, False, 0, 0, _
                                    frmSrc.ScaleX(frmSrc.Width, vbTwips, vbPixels), _
                                    frmSrc.ScaleY(frmSrc.Height, vbTwips, vbPixels))
End Function
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CaptureClient
'    - Captures the client area of a form
'
' frmSrc
'    - The Form object to capture
'
' Returns
'    - Returns a Picture object containing a bitmap of the form's
' client area
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Public Function CaptureClient(frmSrc As Form) As Picture
    ' Call CaptureWindow to capture the client area of the form given
    ' it's window handle and return the resulting Picture object
    Set CaptureClient = CaptureWindow(frmSrc.hWnd, True, 0, 0, _
                                      frmSrc.ScaleX(frmSrc.ScaleWidth, frmSrc.ScaleMode, vbPixels), _
                                      frmSrc.ScaleY(frmSrc.ScaleHeight, frmSrc.ScaleMode, vbPixels))
End Function
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' CaptureActiveWindow
'    - Captures the currently active window on the screen
'
' Returns
'    - Returns a Picture object containing a bitmap of the active
'      window
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Private Function CaptureActiveWindow() As Picture
    
    Dim hWndActive As Long
    Dim r As Long

    Dim RectActive As RECT
    ' Get a handle to the active/foreground window
    hWndActive = GetForegroundWindow()
    ' Get the dimensions of the window
    r = GetWindowRect(hWndActive, RectActive)
    ' Call CaptureWindow to capture the active window given it's
    ' handle and return the Resulting Picture object
    Set CaptureActiveWindow = CaptureWindow(hWndActive, False, 0, 0, _
                                            RectActive.Right - RectActive.Left, _
                                            RectActive.Bottom - RectActive.Top)

End Function
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
' PrintPictureToFitPage
'    - Prints a Picture object as big as possible
'
' Prn
'    - Destination Printer object
'
' Pic
'    - Source Picture object
'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
'
Private Sub PrintPictureToFitPage(prn As Printer, pic As Picture)
    
    Const vbHiMetric As Integer = 8
    Dim PicRatio As Double
    Dim PrnWidth As Double
    Dim PrnHeight As Double
    Dim PrnRatio As Double
    Dim PrnPicWidth As Double
    Dim PrnPicHeight As Double
    
    Dim PrnX As Double
    Dim PrnY As Double
    Dim buffer As Double
    
    ' Determine if picture should be printed in landscape or portrait
    ' and set the orientation
    If m_orient = 0 Then
        If pic.Height >= pic.Width Then
            prn.Orientation = vbPRORPortrait   ' Taller than wide
        Else
            prn.Orientation = vbPRORLandscape  ' Wider than tall
        End If
    Else
        prn.Orientation = m_orient
    End If
    ' Calculate device independent Width to Height ratio for picture
    PicRatio = pic.Width / pic.Height
    ' Calculate the dimentions of the printable area in HiMetric
    PrnWidth = prn.ScaleX(prn.ScaleWidth, prn.ScaleMode, vbHiMetric) '* 1.2
    PrnHeight = prn.ScaleY(prn.ScaleHeight, prn.ScaleMode, vbHiMetric)
    

m_right = 10
    
    If (m_left + m_right > 0) Then
        PrnWidth = PrnWidth * (1# - (CDbl(m_left + m_right) * 0.01))
    End If
    
    If (m_top + m_btm > 0) Then
        PrnHeight = PrnHeight * (1# - (CDbl(m_top + m_btm) * 0.01))
    End If
        
    ' Calculate device independent Width to Height ratio for printer
    PrnRatio = PrnWidth / PrnHeight
    ' Scale the output to the printable area
    If PicRatio >= PrnRatio Then
        ' Scale picture to fit full width of printable area
        PrnPicWidth = prn.ScaleX(PrnWidth, vbHiMetric, prn.ScaleMode)
        PrnPicHeight = prn.ScaleY(PrnWidth / PicRatio, vbHiMetric, _
                                  prn.ScaleMode)
    Else
        ' Scale picture to fit full height of printable area
        PrnPicHeight = prn.ScaleY(PrnHeight, vbHiMetric, prn.ScaleMode)
        PrnPicWidth = prn.ScaleX(PrnHeight * PicRatio, vbHiMetric, _
                                 prn.ScaleMode)
    End If
    
    PrnX = CDbl(m_left) * 0.01 * prn.ScaleWidth
    PrnY = CDbl(m_top) * 0.01 * prn.ScaleHeight
    
    ' Print the picture using the PaintPicture method
    prn.PaintPicture pic, PrnX, PrnY, PrnPicWidth, PrnPicHeight
    
    prn.EndDoc
End Sub
'--------------------------------------------------------------------



'
'   prfPrintForm
'
'   Form ausdrucken
'
'   Parameter:
'       frm as form        -    auszudruckende Form
'       client as boolean  -    Nur Client-Area (true) oder
'                               ganze Form (false)
Public Sub prfPrintForm(frm As Form, client As Boolean)

    Dim pic As Picture
    
    If client Then
        Set pic = CaptureClient(frm)
    Else
        Set pic = CaptureForm(frm)
    End If
    
    If Not (pic Is Nothing) Then
        PrintPictureToFitPage Printer, pic
    End If
    
End Sub

'
'   prfPrintScreen
'
'   Bildschirm ausdrucken
'
Public Sub prfPrintScreen()

    Dim pic As Picture
    
    Set pic = CaptureScreen()
    
    If Not (pic Is Nothing) Then
        PrintPictureToFitPage Printer, pic
    End If

End Sub
'
'   prfPrintActive
'
'   Aktives Fenster ausdrucken
'
Public Sub prfPrintActive()

    Dim pic As Picture

    Set pic = CaptureActiveWindow()
    
    If Not (pic Is Nothing) Then
        PrintPictureToFitPage Printer, pic
    End If
    
End Sub


'
'   prfSetMargins
'
'   Randbreiten in % setzen
'
'   Parameter:
'
'       mleft, mright, mtop, mbtm  - Randbreiten links, rechts, oben, unten in %
'
'
'   Limits:
'       0 <= any_margin <= 30%
'
Public Sub prfSetMargins(mleft As Integer, mright As Integer, mtop As Integer, mbtm As Integer)


    m_left = mleft
    m_right = mright
    m_top = mtop
    m_btm = mbtm
    
    Call forceRange(m_left, 0, 30)
    Call forceRange(m_right, 0, 30)
    Call forceRange(m_top, 0, 30)
    Call forceRange(m_btm, 0, 30)
                    
End Sub


Private Sub forceRange(X As Integer, imin As Integer, imax As Integer)

    If (X < imin) Then
        X = imin
    ElseIf (X > imax) Then
        X = imax
    End If

End Sub


'
'   Seitenausrichtung einstellen
'
'   Parameter
'
'       orient as integer   - Seitenausruichtung: vbPRORLandscape
'                             oder vbPRORPortrait oder 0 (default).
'                             Bei 0 automatische Bestimmung nach
'                             Seitenverhltnis des Fensters.
Public Sub prfSetOrientation(Orient As Integer)

    If Orient = vbPRORLandscape Or Orient = vbPRORPortrait Then
        m_orient = Orient
    Else
        m_orient = 0
    End If

End Sub
