// =============================================================
// ==
// ==   Dataserver routine: herastatebase.c
// ==
// =============================================================
#include "prolog.h"
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>
#include <ctype.h>
#include <signal.h>
#include <dirent.h>
#include "fecerr.h"
#include "rpctypes.h"
#include "database.h"
#include "almsrv.h"
#include "errors.h"
#include "archive.h"
#include "toolkit.h"

extern FEC *fec;

// important initialization:
int iHPenergy;
double HPCur = -1,HPTau = -1,HECur = -1,HETau = -1;
float fminus = -1;
float *HPenergy = &fminus,*HEenergy = &fminus;
NAME32 heratxt[2];

// state variables:
extern int ramping, running, beam, lumi;
extern u16 Pramping, Eramping, Pbeam, Ebeam;
extern u16 Prunning, Erunning;
extern int DclEramping, DclPramping, DclLumi;
extern int DclEInj, DclPInj; 

void updateStateInfo(int i, int j,int cc)
{
  static int lastPEnergy;
  static float lastEEnergy;
  static int PEpanic = 0, EEpanic = 0,PIpanic = 0; // monitor curcial parameters  static int STpanic = 0;
  static int STpanic = 0;
  float *fptr;
  int k;
  
  if (!strncmp(fec[i].fecdata[j].param,"HEbpm",5) || !strncmp(fec[i].fecdata[j].param,"HPbpm",5))
  {
    fptr = (float *)(fec[i].data+fec[i].fecdata[j].offset);
    for (k=0; k<fec[i].fecdata[j].size; k++)
    {
      if (*fptr > 1e11) *fptr = 0;
      fptr++;
    }
    goto fixstate;
  }
  
  if (!strcmp(fec[i].fecdata[j].param,"HPMAGEN"))
  {
    HPenergy = (float *)(fec[i].data+fec[i].fecdata[j].offset);
    if (cc)            // there's something wrong
    {
      if (++PEpanic > 3)
      {
        *HPenergy = -1;
        SetAlarm("FILER",0,hpenergy_undefined,NULL);
      }
    }
    else               // everything's okay
    {
      if (abs(*HPenergy - lastPEnergy) > 1)
      {
        Pramping = (int)(*HPenergy - lastPEnergy);
        lastPEnergy = *HPenergy;
      }
      else Pramping = 0;
      PEpanic = 0;
      ClearAlarm("FILER",0);            
    }
    iHPenergy = (int)*HPenergy;
    Prunning =  *HPenergy > 2 ? 1 : 0;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HEMAGEN"))
  {
    HEenergy = (float *)(fec[i].data+fec[i].fecdata[j].offset);
    if (cc)          // there's something wrong
    {
      if (++EEpanic > 3)
      {
        *HEenergy = -1;
        SetAlarm("FILER",1,heenergy_undefined,NULL);
      }
    }
    else             // everything's okay
    {
      if (abs(*HEenergy - lastEEnergy) > 1.0)            
      {
        Eramping = (int)(*HEenergy - lastEEnergy);
        lastEEnergy = *HEenergy;
      }
      else Eramping = 0;
      EEpanic = 0;
      ClearAlarm("FILER",1);
    }
    Erunning =  (*HEenergy > 1) ? 1 : 0;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HPDCCur"))
  {
    HPCur = (double)(*(float *)(fec[i].data+fec[i].fecdata[j].offset));
    if (cc && ++PIpanic < 6) goto fixstate;
    if (cc) HPCur = -1.0;
    Pbeam = HPCur > 0.1 ? 1 : 0;
    if (HPCur != -1 && HPCur < 0) HPCur = 0;
    PIpanic = 0; 
    if (HPCur == -1) SetAlarm("FILER",2,hpcurrent_undefined,NULL);
    else ClearAlarm("FILER",2);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HPDCLoss"))
  {
    HPTau = (double)(*(float *)(fec[i].data+fec[i].fecdata[j].offset));
    if (cc && ++PIpanic < 6) goto fixstate;
    HPTau = cc ? -1 : fabs(HPTau);
    if (HPTau < 0.0001) HPTau = 0.0001;
    HPTau = 1/HPTau;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HEDCCur"))
  {
    HECur = (double)(*(float *)(fec[i].data+fec[i].fecdata[j].offset));
    if (cc && ++PIpanic < 6) goto fixstate;
    if (cc) HECur = -1;
    Ebeam = HECur > 0.1 ? 1 : 0; 
    if (HECur != -1 && HECur < 0) HECur = 0;
    if (HECur == -1)SetAlarm("FILER",3,hecurrent_undefined,NULL);
    else ClearAlarm("FILER",3);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HEDCLoss"))
  {
    HETau = (double)(*(float *)(fec[i].data+fec[i].fecdata[j].offset));
    if (cc && ++PIpanic < 6) goto fixstate;
    HETau = cc ? -1 : fabs(HETau);
    if (HETau < 0.0001) HETau = 0.0001;
    HETau = 1/HETau;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HESTATE"))
  {
    if (cc && ++STpanic < 6) goto fixstate;
    if (cc) { DclEInj = DclEramping = DclLumi = 0; goto fixstate; }
    STpanic = 0;
    DclEramping = !strcmp("Ramp",fec[i].data+fec[i].fecdata[j].offset);
    DclEInj = !strcmp("Injection Ready",fec[i].data+fec[i].fecdata[j].offset);      DclLumi = !strcmp("Lumi Run",fec[i].data+fec[i].fecdata[j].offset);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HPSTATE"))
  {
    if (cc && ++STpanic < 6) goto fixstate;
    if (cc) { DclPInj = DclPramping = DclLumi = 0; goto fixstate; }
    STpanic = 0;
    DclPramping = !strcmp("Ramp",fec[i].data+fec[i].fecdata[j].offset);
    DclPInj = !strcmp("Injection Ready",fec[i].data+fec[i].fecdata[j].offset);      DclLumi = !strcmp("Lumi Run",fec[i].data+fec[i].fecdata[j].offset);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"HERATEXT"))
  {
    strncpy(heratxt[0].name,fec[i].data+fec[i].fecdata[j].offset,32);
    strncpy(heratxt[1].name,fec[i].data+fec[i].fecdata[j].offset+32,32);
    goto fixstate;
  }

fixstate:
  lumi = DclLumi;
  running = Erunning | Prunning;
  beam = Ebeam | Pbeam;
  ramping = DclEramping | DclPramping; 
}

int overloadEqpmod(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access)
{
  int cc, i, j, n, len;
  char prop[32], tmp[65], *c;
  short stateArray[7];
  short dsize,EqNo = 0,fmt;                                                     

  if (devName[0] == '#') EqNo = atoi(&devName[1]);
  if (EqNo < 0) return illegal_equipment_number;
  if (EqNo > 0) EqNo = EqNo - 1;                  // count from 0
  fmt = dout->dFormat;
  dsize = dout->dArrayLength;              

  if (!strcmp(devProperty,"HERATXT1"))
  {
    switch (dout->dFormat)
    {
      case CF_NAME32:
        if (dsize < 1) return dimension_error;
        strncpy(dout->data.cptr,heratxt[0].name,32);
        return 0;
      case CF_TEXT:
        if (dsize < 1) return dimension_error;
        strncpy(dout->data.cptr,heratxt[0].name,dsize);
        return 0;
      default: return illegal_format;
    }
    return 0;
  } 
  if (!strcmp(devProperty,"HERATXT2"))
  {
    switch (dout->dFormat)
    {
      case CF_NAME32:
        if (dsize < 1) return dimension_error;
        strncpy(dout->data.cptr,heratxt[1].name,32);
        return 0;
      case CF_TEXT:
        if (dsize < 1) return dimension_error;
        strncpy(dout->data.cptr,heratxt[1].name,dsize);
        return 0;
      default: return illegal_format;
    }
    return 0;
  }
  if (!strcmp(devProperty,"HERATXT"))
  {
    switch (dout->dFormat)
    {
      case CF_NAME16:
      case CF_NAME32:
      case CF_NAME48:
        if (dsize != 1) return dimension_error;
        len = fmtsizeof(dout->dFormat);
        break;
      case CF_TEXT:
        if (dsize < 1) return dimension_error;
        len = dsize;
        break;
      default: return illegal_format;
    }
    strncpy(tmp,heratxt[0].name,32); tmp[32] = 0;
    strncpy(&tmp[strlen(tmp)],heratxt[1].name,32); tmp[64] = 0;
    n = strlen(tmp);
    for (i=0,c=tmp; i < n && (*c == ' ' || *c < 65); i++) c++;
    for (i=n-1; i > 0 && tmp[i] == ' '; i--) tmp[i] = 0; 
    strncpy(dout->data.cptr,c,len);
    return 0;
  } 
  if (!strncmp(devProperty,"HEMAIN",6) && (c=strchr(devProperty,'Q')) != NULL)
  {
    *c = 0; strcat(devProperty,".NAM");
    return illegal_property; // force call further down stream
  } 
  if (!strncmp(devProperty,"HPMAIN",6) && (c=strchr(devProperty,'Q')) != NULL)
  {
    *c = 0; strcat(devProperty,".NAM");
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmXH1"))
  {
    strncpy(devProperty,"HEOrbitX",32);
    sprintf(devName,"#%d",93);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmYH1"))
  {
    strncpy(devProperty,"HEOrbitY",32);
    sprintf(devName,"#%d",93);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmXB"))
  {
    strncpy(devProperty,"HEOrbitX",32);
    sprintf(devName,"#%d",44);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmYB"))
  {
    strncpy(devProperty,"HEOrbitY",32);
    sprintf(devName,"#%d",44);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmXHE"))
  {
    strncpy(devProperty,"HEOrbitX",32);
    sprintf(devName,"#%d",176);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmYHE"))
  {
    strncpy(devProperty,"HEOrbitY",32);
    sprintf(devName,"#%d",176);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmXZU"))
  {
    strncpy(devProperty,"HEOrbitX",32);
    sprintf(devName,"#%d",245);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HEbpmYZU"))
  {
    strncpy(devProperty,"HEOrbitY",32);
    sprintf(devName,"#%d",245);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HPbpmX"))
  {
    strncpy(devProperty,"HPOrbitX",32);
    sprintf(devName,"#%d",26);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HPbpmY"))
  {
    strncpy(devProperty,"HPOrbitY",32);
    sprintf(devName,"#%d",26);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"H1bpmX"))
  {
    strncpy(devProperty,"HPOrbitX",32);
    sprintf(devName,"#%d",95);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"H1bpmY"))
  {
    strncpy(devProperty,"HPOrbitY",32);
    sprintf(devName,"#%d",95);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HBbpmX") || !strcmp(devProperty,"KOLbpmX"))
  {
    strncpy(devProperty,"HPOrbitX",32);
    sprintf(devName,"#%d",132);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HBbpmY") || !strcmp(devProperty,"KOLbpmY"))
  {
    strncpy(devProperty,"HPOrbitY",32);
    sprintf(devName,"#%d",132);
    return illegal_property; // force call further down stream
  } 
  if (!strcmp(devProperty,"HPENERGY"))
  {
    if (dsize != 1) return illegal_read_write;
    return putValuesFromLong(dout,(UINT32 *)&iHPenergy,1);
    return 0;
  }  
  if (!stricmp(devProperty,"HECur") || !strcmp(devProperty,"HESUCURR") ||
      !strcmp(devProperty,"HECurMKI"))
  {
    if (dsize != 1) return illegal_read_write;
    if (fmt == CF_FLOAT) dout->data.fptr[0] = HECur;
    else if (fmt == CF_DOUBLE) dout->data.dptr[0] = HECur;
    else return illegal_format;
    return 0;
  }
  if (!stricmp(devProperty,"HETau") || !strcmp(devProperty,"HESULIFE") ||
      !strcmp(devProperty,"HEtauMKI") || !strcmp(devProperty,"HEBuntau"))
  {
    if (dsize != 1) return illegal_read_write;
    if (fmt == CF_FLOAT) dout->data.fptr[0] = HETau;
    else if (fmt == CF_DOUBLE) dout->data.dptr[0] = HETau;
    else return illegal_format;
    return 0;
  } 
  if (!stricmp(devProperty,"HPCur") || !strcmp(devProperty,"HPSUCURR") ||
      !strcmp(devProperty,"HPCurMKI"))
  {
    if (dsize != 1) return illegal_read_write;
    if (fmt == CF_FLOAT) dout->data.fptr[0] = HPCur;
    else if (fmt == CF_DOUBLE) dout->data.dptr[0] = HPCur;
    else return illegal_format;
    return 0;
  }
  if (!stricmp(devProperty,"HPTau") || !strcmp(devProperty,"HPSULIFE") ||
      !strcmp(devProperty,"HPtauMKI") || !strcmp(devProperty,"HPBuntau"))
  {
    if (dsize != 1) return illegal_read_write;
    if (fmt == CF_FLOAT) dout->data.fptr[0] = HPTau;
    else if (fmt == CF_DOUBLE) dout->data.dptr[0] = HPTau;
    else return illegal_format;
    return 0;
  }  
  if (!strcmp(devProperty,"HEinjSt") || !strcmp(devProperty,"HPinjSt"))
  {
    return 0;
  }
  return illegal_property;
}
extern void cbackBrdsrv(int id,int cc);
int GlobalsRegistrationEx(void)
{
  DTYPE din;   
  // for historical reasons:
  din.dFormat = CF_SHORT;
  din.dArrayLength = 1;
  din.data.vptr = &iHPenergy;
  sendNetGlobal("HPENERGY",&din,cbackBrdsrv,1,1,0);
  din.dFormat = CF_DOUBLE;
  din.dArrayLength = 1;
  din.data.vptr = &HPCur;
  sendNetGlobal("HPCur",&din,cbackBrdsrv,1,1,0);
  din.data.vptr = &HECur;
  sendNetGlobal("HECur",&din,cbackBrdsrv,1,1,0);
  din.data.vptr = &HPTau;
  sendNetGlobal("HPtau",&din,cbackBrdsrv,1,1,0);
  din.data.vptr = &HETau;
  sendNetGlobal("HEtau",&din,cbackBrdsrv,1,1,0);  
}

