// =============================================================
// ==
// ==   Dataserver routine: herastatebase.c
// ==
// =============================================================
#include "prolog.h"
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <malloc.h>
#include <ctype.h>
#include <signal.h>
#include <dirent.h>
#include "fecerr.h"
#include "rpctypes.h"
#include "database.h"
#include "almsrv.h"
#include "errors.h"
#include "archive.h"
#include "toolkit.h"

extern FEC *fec;

// important initialization:
float PPCur = -1,PPTau = -1,PECur = -1,PETau = -1;
float PPenergy = -1,PEenergy = -1;
char Ptype[16] = "undefined";
char Pmode[32] = "undefined";

// state variables:
extern int ramping, running, beam, lumi;
extern int Pramping, Eramping, Pbeam, Ebeam;
extern int Prunning, Erunning;
extern int DclEramping, DclPramping, DclLumi;
extern int DclEInj, DclPInj; 
int eundulating = 0;

void updateStateInfo(int i, int j,int cc)
{
  static int lastPEnergy;
  static float lastEEnergy;
  static int Ppanic = 0, Ipanic = 0; // monitor curcial parameters  static int STpanic = 0;
  static int STpanic = 0, Mpanic = 0;
  float *fptr;
  int k;
  
  if (!strcmp(fec[i].fecdata[j].param,"PEMAGEN"))
  {
    if (!stricmp(Ptype,"Protonen"))
    {
      PPenergy = *(float *)(fec[i].data+fec[i].fecdata[j].offset);
      PEenergy = 0;
    }
    else if (!stricmp(Ptype,"Positronen") || !stricmp(Ptype,"Elektronen"))
    {
      PEenergy = *(float *)(fec[i].data+fec[i].fecdata[j].offset);
      PPenergy = 0;
    }
    else
    {
      PEenergy = 0;
      PPenergy = 0;
    }
    if (cc)            // there's something wrong
    {
      if (++Ppanic > 3)
      {
        if (PPenergy > 0) PPenergy = -1;
        if (PEenergy > 0) PEenergy = -1;
        SetAlarm("FILER",0,penergy_undefined,NULL);
      }
    }
    else               // everything's okay
    {
      if (abs(PPenergy - lastPEnergy) > 1)
      {
        Pramping = (int)(PPenergy - lastPEnergy);
        lastPEnergy = PPenergy;
      }
      else Pramping = 0;
      if (abs(PEenergy - lastEEnergy) > 1)
      {
        Eramping = (int)(PEenergy - lastEEnergy);
        lastEEnergy = PEenergy;
      }
      else Eramping = 0;
      Ppanic = 0;
      ClearAlarm("FILER",0);            
    }
    Prunning =  PPenergy > 2 ? 1 : 0;
    Erunning =  PEenergy > 2 ? 1 : 0;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"PEDCCur"))
  {
    if (!stricmp(Ptype,"Protonen"))
    {
      PPCur = *(float *)(fec[i].data+fec[i].fecdata[j].offset);
      PECur = 0;
    }
    else if (!stricmp(Ptype,"Positronen") || !stricmp(Ptype,"Elektronen"))
    {
      PECur = *(float *)(fec[i].data+fec[i].fecdata[j].offset);
      PPCur = 0;
    }
    else
    {
      PPCur = 0;
      PECur = 0;
    }
    if (cc && ++Ipanic < 6) goto fixstate;
    if (cc) 
    {
      if (PPCur > 0) PPCur = -1.0;
      if (PECur > 0) PECur = -1.0;
      SetAlarm("FILER",2,pcurrent_undefined,NULL);
    }
    else ClearAlarm("FILER",2);
    Pbeam = PPCur > 0.1 ? 1 : 0;
    Ebeam = PECur > 0.1 ? 1 : 0;
    Ipanic = 0; 
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"PETYPE"))
  {
    if (cc && ++STpanic < 6) goto fixstate;
    if (cc) { strcpy(Ptype,"undefined"); goto fixstate; }
    STpanic = 0;
    strncpy(Ptype,fec[i].data+fec[i].fecdata[j].offset,10);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"PEMODE"))
  {
    if (cc && ++Mpanic < 6) goto fixstate;
    Mpanic = 0;
    strncpy(Pmode,fec[i].data+fec[i].fecdata[j].offset,32);
    if (strstr(Pmode,"Undulator") != NULL) eundulating = 1;
    else eundulating = 0;
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"PEESTATE"))
  {
    if (cc && ++STpanic < 6) goto fixstate;
    if (cc) { DclEInj = DclEramping = 0; goto fixstate; }
    STpanic = 0;
    DclEramping = !strcmp("Ramp",fec[i].data+fec[i].fecdata[j].offset);
    DclEInj = !strcmp("Injection Ready",fec[i].data+fec[i].fecdata[j].offset);
    goto fixstate;
  }
  if (!strcmp(fec[i].fecdata[j].param,"PEPSTATE"))
  {
    if (cc && ++STpanic < 6) goto fixstate;
    if (cc) { DclPInj = DclPramping = 0; goto fixstate; }
    STpanic = 0;
    DclPramping = !strcmp("Ramp",fec[i].data+fec[i].fecdata[j].offset);
    DclPInj = !strcmp("Injection Ready",fec[i].data+fec[i].fecdata[j].offset);
    goto fixstate;
  }

fixstate:
  running = Erunning | Prunning;
  beam = Ebeam | Pbeam;
  ramping = Eramping | Pramping; 
}

int overloadEqpmod(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access)
{
  int cc, i, j, n;
  char prop[32], *c;
  short stateArray[7];
  short dsize,EqNo = 0,fmt;                                                     

  if (devName[0] == '#') EqNo = atoi(&devName[1]);
  if (EqNo < 0) return illegal_equipment_number;
  if (EqNo > 0) EqNo = EqNo - 1;                  // count from 0
  fmt = dout->dFormat;
  dsize = dout->dArrayLength;

  if (!strcmp(devProperty,"PEPENERGY"))
  {
    if (dsize != 1) return illegal_read_write;
    return putValuesFromFloat(dout,&PPenergy,1);
    return 0;
  }
  if (!strcmp(devProperty,"PEEENERGY"))
  {
    if (dsize != 1) return illegal_read_write;
    return putValuesFromFloat(dout,&PEenergy,1);
    return 0;
  }
  if (!strcmp(devProperty,"PEPDCCur"))
  {
    if (dsize != 1) return illegal_read_write;
    return putValuesFromFloat(dout,&PPCur,1);
    return 0;
  }
  if (!strcmp(devProperty,"PEEDCCur"))
  {
    if (dsize != 1) return illegal_read_write;
    return putValuesFromFloat(dout,&PECur,1);
    return 0;
  }
  return illegal_property;
}
extern void cbackBrdsrv(int id,int cc);
int GlobalsRegistrationEx(void)
{
  DTYPE din;   
  // for historical reasons:
  din.dFormat = CF_FLOAT;
  din.dArrayLength = 1;
  din.data.vptr = &PPenergy;
  sendNetGlobal("PEPENERGY",&din,cbackBrdsrv,1,1,0);
  din.dFormat = CF_FLOAT;
  din.dArrayLength = 1;
  din.data.vptr = &PEenergy;
  sendNetGlobal("PEEENERGY",&din,cbackBrdsrv,1,1,0);
  din.dFormat = CF_FLOAT;
  din.dArrayLength = 1;
  din.data.vptr = &PPCur;
  sendNetGlobal("PEPCur",&din,cbackBrdsrv,1,1,0);
  din.data.vptr = &PECur;
  sendNetGlobal("PEECur",&din,cbackBrdsrv,1,1,0);
  return 0;
}

