Attribute VB_Name = "SystemTools"
Option Explicit

Public Const MAX_PROC_INFO = 200

Public Const MAX_CONSOLE_MODES = 10
Const MAX_WINDOWS = 10
Const MAX_CAPTION_LEN = 32
Const MAX_EXE_NAME = 20
Const MAX_ARG_LEN = 64


Public Const MAX_PROCS_PRO_CONSMODE = 15

Public Type T_WINDOWDETAILS
    handle As Long
    g As T_WINDOWPLACEMENT
    reqTop As Long
    reqLeft As Long
    reqBottom As Long
    reqRight As Long
    reqStatus As Integer
    parentWindowHandle As Long
    caption As String * MAX_CAPTION_LEN
    '''reqCaption As String * MAX_CAPTION_LEN
    childWindowIndex(MAX_WINDOWS - 1) As Integer
    nChildWindows As Integer
End Type

Public Type T_PROCESSDETAILS
    procEntry As T_PROCESSENTRY32
    path As String * MAX_PATH
    exeName As String * MAX_EXE_NAME
    arguments As String * MAX_ARG_LEN
    visibleWindows(MAX_WINDOWS) As T_WINDOWDETAILS
    nVisibleWindows As Integer
    parentExeName As String * MAX_EXE_NAME
    parentPath As String * MAX_PATH
    running As Integer  '0 - defined, not running, 1 - defined, running, 2 - not defined, running
    zombieCnt As Integer
End Type

Public Type T_REMOTESCREENSHOT
    bitMapInfo As T_BITMAPINFO
    screenDump() As Byte
End Type

Public Type T_SCREENINFO
    xRes As Long
    yRes As Long
    monitorHandle As Long
    monitorHDC As Long
End Type

Public Type T_SYSTEMINFO
    nMonitors As Integer
    scrInfo(3) As T_SCREENINFO  ' up to four monitors
    UserName As String * 20
    osVersion As String * 10
    machineName As String * 16
    ip As String * 16
End Type

Public Type T_STARTPROC
    pathName(MAX_PATH) As Byte
    exeNname(MAX_EXE_NAME) As Byte
    args(MAX_ARG_LEN) As Byte
    workingDirectory(MAX_PATH) As Byte
End Type

Public Type T_STOPPROC
    procIdx As Integer              ' there could be more processes with the same exe name
    exeName As String * MAX_PATH    ' exe name is needed, since process index could be changed and be pointing to another exe
End Type

Public Type T_CONSOLE_MODE
    Name As NAME16I
    nProcs As Integer
    procs(MAX_PROCS_PRO_CONSMODE - 1) As T_PROCESSDETAILS
    selected As Boolean
End Type


Public Const MAX_DAEMONS = 100

Public DaemonName(MAX_DAEMONS) As String
Public Ndaemons As Integer

Public NprocessDetails(MAX_DAEMONS) As Integer
Public RunningProcesses(MAX_PROC_INFO, MAX_DAEMONS) As T_PROCESSDETAILS
Public ConsoleModes(MAX_CONSOLE_MODES - 1, MAX_DAEMONS) As T_CONSOLE_MODE
Public TestWinDet As T_WINDOWDETAILS

Public Function GetExeName(processId As Long) As String
    Dim hProcess As Long, hExe As Long
    Dim lret As Long, cbNeeded As Long
    
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, _
                            0&, processId)


    If hProcess <> 0 Then
   
      ' Now get the handle of the first module
      ' in this process, since first module is EXE
      hExe = 0
      lret = EnumProcessModules(hProcess, hExe, 4&, cbNeeded)
      
      If hExe <> 0 Then
         ' Get full path name
         GetExeName = String$(MAX_PATH, 0)
         lret = GetModuleFileNameEx(hProcess, hExe, GetExeName, Len(GetExeName))
         GetExeName = Trim0(GetExeName)
      End If   ' EXE <> 0
   End If      ' hProcess <> 0
   
   ' Close handle
   lret = CloseHandle(hProcess)


End Function

Public Sub fillPIDsTable(pidTable() As Long)
    Dim nProc As Long, needed As Long
  
    Dim lret As Long
     
    nProc = 25  'set to initial, reasonable value
    Do
        ReDim pidTable(nProc - 1)
   
        lret = EnumProcesses(pidTable(0), nProc * 4, needed)
        If lret = 0 Then
            ReDim pidTable(0)
            Exit Sub
        End If
        ' Compare needed bytes with array size in bytes.
        ' If less, then we got them all.
        If needed < nProc * 4 Then
            nProc = needed / 4
            ReDim Preserve pidTable(nProc - 1)
            Exit Sub
        End If
        nProc = nProc * 2
    Loop
End Sub


Public Function Trim0(sName As String) As String
' Keep left portion of string sName up to first 0. Useful with Win API null terminated strings.
    Dim x As Integer
    x = InStr(sName, Chr(0))
    If x > 0 Then Trim0 = Left(sName, x - 1) Else Trim0 = sName
End Function


Public Function GetPathFromName(p As String) As String
    Dim pos As Integer, lastFound As Integer
    
    pos = 1
    Do
        pos = InStr(pos, p, "\")
        If pos > 0 Then lastFound = pos
        pos = pos + 1
    Loop While pos <> 1
    
    If lastFound > 0 Then GetPathFromName = Left(p, lastFound)
    
End Function

Public Function GetExeFromName(p As String) As String
    Dim pos As Integer, lastFound As Integer
    
    pos = 1
    Do
        pos = InStr(pos, p, "\")
        If pos > 0 Then lastFound = pos
        pos = pos + 1
    Loop While pos <> 1
    
    If lastFound > 0 Then GetExeFromName = Right(p, Len(p) - lastFound)
    
End Function


Public Sub GetIndexes(idxkey As String, dmnIdx As Integer, modeIdx As Integer, procIdx As Integer, windowIdx As Integer)
    Dim pos As Integer
    Dim l As Integer
    Dim idx() As String
    
   
    windowIdx = -1
    modeIdx = -1
    procIdx = -1
    dmnIdx = -1
    
    l = Len(idxkey) - 1
    If (l < 1) Then Exit Sub
    
    idxkey = Right(idxkey, l)
    idx = Split(idxkey, ",")
    
    Select Case UBound(idx)
        Case 0
           dmnIdx = Val(idx(0))
        Case 1
            dmnIdx = Val(idx(0))
            modeIdx = Val(idx(1))
        Case 2
            dmnIdx = Val(idx(0))
            modeIdx = Val(idx(1))
            procIdx = Val(idx(2))
        Case 3
            dmnIdx = Val(idx(0))
            modeIdx = Val(idx(1))
            procIdx = Val(idx(2))
            windowIdx = Val(idx(3))
    End Select
    
''''''    pos = InStr(1, idxkey, ",")
''''''    If pos = 0 Then
''''''        dmnIdx = Val(idxkey)
''''''    Else
''''''        dmnIdx = Val(Left(idxkey, pos))
''''''        idxkey = Right(idxkey, l - pos)
''''''        l = Len(idxkey)
''''''        pos = InStr(1, idxkey, ",")
''''''        If pos = 0 Then
''''''            procIdx = Val(idxkey)
''''''        Else
''''''            procIdx = Val(Left(idxkey, pos))
''''''            windowIdx = Val(Right(idxkey, l - pos))
''''''        End If
''''''    End If
''''''
End Sub

Public Sub GetScreenResolution(hResolution As Integer, vResolution As Integer)
    
    hResolution = Screen.Width / Screen.TwipsPerPixelX
    vResolution = Screen.Height / Screen.TwipsPerPixelY
    
End Sub

Public Sub putlog(t As String)
    FORM_consoleDeamon.LOG.LOG (t)
End Sub

Public Sub getScreenShotDims(w As Long, h As Long)
    Dim Result As Long, desktopHwnd As Long
    '''''', desktopHdc As Long
    Dim desktop As T_RECT
    
    'find size of desktop in pixels
    desktopHwnd = GetDesktopWindow()
    '''''desktopHdc = GetDC(desktopHwnd)
    Result = GetWindowRect(desktopHwnd, desktop)
    
    w = (Abs(desktop.Right - desktop.Left + 1) / 4) * 4 + 4
    w = w * 3  ' because of 24 bit  colors  in use?
    h = Abs(desktop.Bottom - desktop.Top) + 1
    ''''Call ReleaseDC(desktopHwnd, desktopHdc)
    
End Sub

Public Sub getScreenShot(screenDumpBuff() As Byte, bi24BitInfo As T_BITMAPINFO)
    Dim Result As Long, desktopHwnd As Long, desktopHdc As Long
    Dim desktop As T_RECT
    Dim w As Long, h As Long
    Dim mDC As Long, mBitmap As Long
    Dim errorCode As Long
    
    'find size of desktop in pixels
    desktopHwnd = GetDesktopWindow()
   
    Result = GetWindowRect(desktopHwnd, desktop)
    
    
    Call getScreenShotDims(w, h)
    ReDim screenDumpBuff(w, h) As Byte
    
    'Bitmap-Header
    With bi24BitInfo.bmiHeader
        .biBitCount = 24
        .biCompression = BI_RGB
        .biPlanes = 1
        .biSize = Len(bi24BitInfo.bmiHeader)
        .biWidth = Abs(desktop.Right - desktop.Left) + 1
        .biHeight = Abs(desktop.Bottom - desktop.Top) + 1
    End With
     
    
    desktopHdc = GetDC(desktopHwnd)
    
    'create compatible memory device context
    mDC = CreateCompatibleDC(desktopHdc)
    Call ReleaseDC(desktopHwnd, desktopHdc)

    'create device independent bitmap
    mBitmap = CreateDIBSection(mDC, bi24BitInfo, _
              DIB_RGB_COLORS, ByVal 0&, ByVal 0&, ByVal 0&)

    Call SelectObject(mDC, mBitmap)



    'copy contents of the desktop window to the device independend bitmap
    desktopHdc = GetDC(desktopHwnd)
    Call BitBlt(mDC, 0, 0, bi24BitInfo.bmiHeader.biWidth, _
                bi24BitInfo.bmiHeader.biHeight, desktopHdc, 0, _
                0, vbSrcCopy)
    

    'copy device independent bitmap to the screen dump array
    Result = GetDIBits(desktopHdc, mBitmap, 0, _
                   bi24BitInfo.bmiHeader.biHeight, _
                   screenDumpBuff(0, 0), bi24BitInfo, DIB_RGB_COLORS)
    
    errorCode = GetLastError()
    
    Call ReleaseDC(desktopHwnd, desktopHdc)
    Call DeleteObject(mBitmap)
    Call DeleteDC(mDC)
    
End Sub

Public Sub setName16(dest As NAME16I, src As String)

    With dest
        .iVal = Len(src)
        Call String2Bytes(.nam(0), src, .iVal)
    End With
End Sub

Public Function getName16String(src As NAME16I) As String
    
    getName16String = BSTR(src.nam)
    
End Function


