Attribute VB_Name = "SystemTools"
Option Explicit

Public Const EDIT_MODE = 1
Public Const OPERATIONAL_MODE = 2

Public Const SYSTEM_WINDOWS_PATH = "C:\WINDOWS\"
Public Const SYSTEM_WINDOWS_EXPLORER = "EXPLORER.EXE"

Public Const MAX_PROC_INFO = 200
Public Const MAX_PROCS_PRO_CONSMODE = 15

Public Const ALL_KNOWN_PROCESSES_LIST = "\\mkinthera1\hera\projects\service\vb\ConsoleDaemon\bin\HeraKnownProcesses.csv"
Public Const MAX_KNOWN_PROCESSES = 250
Public Const MAX_ZOMBIE_STATE_CNT = 3

Const MAX_WINDOWS = 10
Const MAX_CAPTION_LEN = 32 ''64
Const MAX_EXE_NAME = 20
Public Const MAX_ARG_LEN = 64
Public Const MAX_CONSOLE_MODES = 10

Public Type T_KNOWN_PROCESS
    exeName As String
    zombieStateCnt As Integer
End Type

Public Type T_KNOWN_PROCESSES
    nProcs As Integer
    procs(MAX_KNOWN_PROCESSES) As T_KNOWN_PROCESS
End Type

Public Type T_DAEMON_CONFIG
    restartProcesses As Boolean
    placeWindowsAfterApplicationStart As Integer ' in seconds
    keepWindowsInGivenPosition As Boolean
    currentMode As Integer
    checkProcessesPeriod As Long
    scriptLocation As String
End Type

Public Type T_WINDOWDETAILS
    handle As Long
    g As T_WINDOWPLACEMENT
    reqTop As Long
    reqLeft As Long
    reqBottom As Long
    reqRight As Long
    reqStatus As Integer
    parentWindowHandle As Long
    caption As String * MAX_CAPTION_LEN
    ''''reqCaption As String * MAX_CAPTION_LEN
    childWindowIndex(MAX_WINDOWS - 1) As Integer
    nChildWindows As Integer
End Type

Public Type T_PROCESSDETAILS
    procEntry As T_PROCESSENTRY32
    path As String * MAX_PATH
    exeName As String * MAX_EXE_NAME
    arguments As String * MAX_ARG_LEN
    visibleWindows(MAX_WINDOWS) As T_WINDOWDETAILS
    nVisibleWindows As Integer
    parentExeName As String * MAX_EXE_NAME
    parentPath As String * MAX_PATH
    launcherScript As String * MAX_PATH
    running As Integer  '0 - defined, not running, 1 - defined, running, 2 - not defined, running
    zombieCnt As Integer
End Type

Public Type T_CONSOLE_MODE
    name As NAME16I
    nProcs As Integer
    procs(MAX_PROCS_PRO_CONSMODE - 1) As T_PROCESSDETAILS
    selected As Boolean
End Type


Public Type T_REMOTESCREENSHOT
    bitMapInfo As T_BITMAPINFO
    screenDump() As Byte
End Type

Public Type T_SCREENINFO
    xRes As Long
    yRes As Long
    monitorHandle As Long
    monitorHDC As Long
End Type

Public Type T_SYSTEMINFO
    nMonitors As Integer
    scrInfo(3) As T_SCREENINFO  ' up to four monitors
    UserName As String * 20
    osVersion As String * 10
    machineName As String * 16
    ip As String * 16
End Type

Public Type T_STARTPROC
    pathName(MAX_PATH) As Byte
    exeNname(MAX_EXE_NAME) As Byte
    args(MAX_ARG_LEN) As Byte
    workingDirectory(MAX_PATH) As Byte
End Type

Public Type T_STOPPROC
    procIdx As Integer              ' there could be more processes with the same exe name
    exeName As String * MAX_PATH    ' exe name is needed, since process index could be changed and be pointing to another exe
End Type

Public NRunningProcesses As Integer
Public RunningProcesses(MAX_PROC_INFO - 1) As T_PROCESSDETAILS

Public TestWinDet As T_WINDOWDETAILS
Public SystemInfo As T_SYSTEMINFO
Public ConsoleModes(MAX_CONSOLE_MODES - 1) As T_CONSOLE_MODE
Public ConsoleModesName16(MAX_CONSOLE_MODES - 1) As NAME16I

Public CurrentConsoleMode As String
Public CurrentConsoleModeIdx As Integer

Public MyCompName As String

Public SystemTray As T_NOTIFYICONDATA
Public DaemonConfig As T_DAEMON_CONFIG
Public AllKnownProcesses As T_KNOWN_PROCESSES
Public FormProcessDetailsShown As Boolean

Dim ModeTmp As T_CONSOLE_MODE

Public Declare Function getRemoteProcessCommandLine Lib "remproc.dll" (ByVal pid As Long, ByVal s As String, ByVal bufferSize As Long) As Long


Public Function GetExeName(processId As Long) As String
    Dim hProcess As Long, hExe As Long
    Dim lret As Long, cbNeeded As Long
    
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, _
                            0&, processId)


    If hProcess <> 0 Then
   
      ' Now get the handle of the first module
      ' in this process, since first module is EXE
      hExe = 0
      lret = EnumProcessModules(hProcess, hExe, 4&, cbNeeded)
      
      If hExe <> 0 Then
         ' Get full path name
         GetExeName = String$(MAX_PATH, 0)
         lret = GetModuleFileNameEx(hProcess, hExe, GetExeName, Len(GetExeName))
         GetExeName = Trim0(GetExeName)
      End If   ' EXE <> 0
   End If      ' hProcess <> 0
   
   ' Close handle
   lret = CloseHandle(hProcess)


End Function

Public Sub fillPIDsTable(pidTable() As Long)
    Dim nProc As Long, needed As Long
  
    Dim lret As Long
     
    nProc = 25  'set to initial, reasonable value
    Do
        ReDim pidTable(nProc - 1)
   
        lret = EnumProcesses(pidTable(0), nProc * 4, needed)
        If lret = 0 Then
            ReDim pidTable(0)
            Exit Sub
        End If
        ' Compare needed bytes with array size in bytes.
        ' If less, then we got them all.
        If needed < nProc * 4 Then
            nProc = needed / 4
            ReDim Preserve pidTable(nProc - 1)
            Exit Sub
        End If
        nProc = nProc * 2
    Loop
End Sub

Public Function Trim0(sName As String) As String
' Keep left portion of string sName up to first 0. Useful with Win API null terminated strings.
    Dim x As Integer
    x = InStr(sName, Chr(0))
    If x > 0 Then Trim0 = Left(sName, x - 1) Else Trim0 = sName
End Function

Public Sub GetWindowInfo(ByVal hWindow As Long)
    Dim parentWindow As Long
    Dim windowPID As Long
    Dim windowStyle As Long
    Dim windowCaption As String
    Dim captionLength As Long
    Dim result As Long
    Dim i As Integer
    
    
    'check window style
    windowStyle = GetWindowLong(hWindow, GWL_STYLE)
    windowStyle = windowStyle And (WS_VISIBLE Or WS_BORDER)
    If (windowStyle = (WS_VISIBLE Or WS_BORDER)) Then   ' more windows attributes?
       
      result = GetWindowThreadProcessId(hWindow, windowPID)
      
      'find a aplication, which has the same pid
      For i = 0 To NRunningProcesses - 1
        With RunningProcesses(i)
            If windowPID = .procEntry.th32ProcessID Then
                .visibleWindows(.nVisibleWindows).handle = hWindow
                captionLength = GetWindowTextLength(hWindow) + 1
                windowCaption = String(captionLength, " ")
                result = GetWindowText(hWindow, windowCaption, captionLength)
                If result <> 0 Then
                    .visibleWindows(.nVisibleWindows).caption = windowCaption
                Else
                    .visibleWindows(.nVisibleWindows).caption = "no caption??"
                End If
                .visibleWindows(.nVisibleWindows).parentWindowHandle = GetParent(hWindow)
                Call GetWindowPlacement(hWindow, .visibleWindows(.nVisibleWindows).g)
                .nVisibleWindows = .nVisibleWindows + 1
                Exit Sub
            End If
        End With
      Next
    End If
End Sub

Public Sub linkWindowList()
    Dim i As Integer
    Dim j As Integer
    Dim k As Integer
    Dim parentHandle As Long
    
    For i = 0 To NRunningProcesses - 1
        With RunningProcesses(i)
            For j = 0 To .nVisibleWindows - 1
                parentHandle = .visibleWindows(j).parentWindowHandle
                If parentHandle <> 0 Then
                    For k = 0 To .nVisibleWindows - 1
                        If .visibleWindows(k).handle = parentHandle Then
                            .visibleWindows(k).childWindowIndex(.visibleWindows(k).nChildWindows) = j
                            .visibleWindows(k).nChildWindows = .visibleWindows(k).nChildWindows + 1
                            Exit For
                        End If
                    Next
                End If
            Next
        End With
    Next
    
End Sub

Public Function GetExeDetails(procDetails() As T_PROCESSDETAILS) As Integer
    Dim hSnapShot As Long, result As Long
    Dim tmpstr As String
    Dim cnt As Integer

    hSnapShot = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0&)
    If hSnapShot = 0 Then Exit Function

    cnt = 0
    procDetails(cnt).procEntry.dwSize = Len(procDetails(cnt).procEntry)
    result = ProcessFirst(hSnapShot, procDetails(cnt).procEntry)
  
    If result <> 0 Then
        RunningProcesses(cnt).path = GetPathFromName(GetExeName(procDetails(cnt).procEntry.th32ProcessID))
        RunningProcesses(cnt).arguments = getProcessCommandLine(procDetails(cnt).procEntry.th32ProcessID)
        tmpstr = GetExeName(procDetails(cnt).procEntry.th32ParentProcessID)
        RunningProcesses(cnt).parentPath = GetPathFromName(tmpstr)
        RunningProcesses(cnt).parentExeName = GetExeFromName(tmpstr)
    End If
    
    Do While result <> 0
        'tmpStr = procDetails(cnt).procEntry.szExeFile
        'tmpStr = Left(tmpStr, InStr(tmpStr, Chr(0)) - 1)
        cnt = cnt + 1
        procDetails(cnt).procEntry.dwSize = Len(procDetails(cnt).procEntry)
        result = ProcessNext(hSnapShot, procDetails(cnt).procEntry)
        If result <> 0 Then
         RunningProcesses(cnt).exeName = GetExeFromName(GetExeName(procDetails(cnt).procEntry.th32ProcessID))
            RunningProcesses(cnt).path = GetPathFromName(GetExeName(procDetails(cnt).procEntry.th32ProcessID))
            RunningProcesses(cnt).arguments = getProcessCommandLine(procDetails(cnt).procEntry.th32ProcessID)
            tmpstr = GetExeName(procDetails(cnt).procEntry.th32ParentProcessID)
            RunningProcesses(cnt).parentPath = GetPathFromName(tmpstr)
            RunningProcesses(cnt).parentExeName = GetExeFromName(tmpstr)
        End If
    Loop
    Call CloseHandle(hSnapShot)
    
    GetExeDetails = cnt
End Function


Public Function GetPathFromName(p As String) As String
    Dim pos As Integer, lastFound As Integer
    
    pos = 1
    Do
        pos = InStr(pos, p, "\")
        If pos > 0 Then lastFound = pos
        pos = pos + 1
    Loop While pos <> 1
    
    If lastFound > 0 Then GetPathFromName = Left(p, lastFound)
    
End Function

Public Function GetExeFromName(p As String) As String
    Dim pos As Integer, lastFound As Integer
    
    pos = 1
    Do
        pos = InStr(pos, p, "\")
        If pos > 0 Then lastFound = pos
        pos = pos + 1
    Loop While pos <> 1
    
    If lastFound > 0 Then
        GetExeFromName = Right(p, Len(p) - lastFound)
    Else
        GetExeFromName = p
    End If
    
End Function


Public Sub GetIndexes(idxkey As String, procIdx As Integer, windowIdx As Integer)
    Dim pos As Integer
    Dim l As Integer
    
    windowIdx = -1
    procIdx = -1
    
    l = Len(idxkey) - 1
    If (l < 1) Then Exit Sub
    
    idxkey = Right(idxkey, l)
    pos = InStr(1, idxkey, ",")
    If pos = 0 Then
        procIdx = Val(idxkey)
    Else
        procIdx = Val(Left(idxkey, pos))
        windowIdx = Val(Right(idxkey, l - pos))
    End If
    
End Sub

Public Sub GetScreenResolution(hResolution As Integer, vResolution As Integer)
    
    hResolution = Screen.Width / Screen.TwipsPerPixelX
    vResolution = Screen.Height / Screen.TwipsPerPixelY
    
End Sub

Public Sub putlog(t As String)
    FORM_log.LOG.LOG (t)
End Sub

Public Sub SetWindowPosition(modeIdx As Integer, pIdx As Integer, wIdx As Integer, wp As T_WINDOWPLACEMENT)
    Dim h As Long
    
    h = ConsoleModes(modeIdx).procs(pIdx).visibleWindows(wIdx).handle
    FORM_consoleDaemon.TIMER_checkProcesses.Enabled = False
    Call SetWindowPlacement(h, wp)
    saveCurrentConfig
    FORM_consoleDaemon.TIMER_checkProcesses.Enabled = True
End Sub

Public Sub getScreenShotDims(idx As Integer, w As Long, h As Long)
    Dim result As Long, desktopHwnd As Long
    '''''', desktopHdc As Long
    Dim desktop As T_RECT
    
    'find size of desktop in pixels
    '''desktopHwnd = GetDesktopWindow()
   ''desktopHwnd = SystemInfo.scrInfo(idx).monitorHandle
    ''''desktopHdc = GetDC(desktopHwnd)
   '' result = GetWindowRect(desktopHwnd, desktop)
    Call getDesktopMaximumDims(w, h)
    ''w = (Abs(desktop.Right - desktop.Left + 1) / 4) * 4 + 4
    w = ((w + 1) / 4) * 4 + 4
    w = w * 3  ' because of 24 bit  colors  in use?
    
    '''h = Abs(desktop.Bottom - desktop.Top)
    ''''Call ReleaseDC(desktopHwnd, desktopHdc)
    
End Sub

Public Sub getScreenShot(screenIdx As Integer, screenDumpBuff() As Byte)
    Dim result As Long, desktopHwnd As Long, desktopHdc As Long
    ''Dim desktop As T_RECT
    Dim bi24BitInfo As T_BITMAPINFO
    Dim w As Long, h As Long
    Dim desktopWidth As Long, desktopHeight As Long
    Dim mDC As Long, mBitmap As Long
    Dim errorCode As Long
    
    'find size of desktop in pixels
    desktopHwnd = GetDesktopWindow()
   
   
    ''result = GetWindowRect(desktopHwnd, desktop)
        
    Call getScreenShotDims(screenIdx, w, h)
    ReDim screenDumpBuff(w, h + 1) As Byte ' first column will contain headre info
    
    Call getDesktopMaximumDims(desktopWidth, desktopHeight)
    'Bitmap-Header
    With bi24BitInfo.bmiHeader
        .biBitCount = 24
        .biCompression = BI_RGB
        .biPlanes = 1
        .biSize = Len(bi24BitInfo.bmiHeader)
        '.biWidth = Abs(desktop.Right - desktop.Left) + 1
        '.biHeight = Abs(desktop.Bottom - desktop.Top) + 1
        .biWidth = desktopWidth
        .biHeight = desktopHeight
    End With
     
    
    desktopHdc = GetDC(desktopHwnd)
    
    'create compatible memory device context
    mDC = CreateCompatibleDC(desktopHdc)
    Call ReleaseDC(desktopHwnd, desktopHdc)

    'create device independent bitmap
    mBitmap = CreateDIBSection(mDC, bi24BitInfo, _
              DIB_RGB_COLORS, ByVal 0&, ByVal 0&, ByVal 0&)

    Call SelectObject(mDC, mBitmap)



    'copy contents of the desktop window to the device independend bitmap
    desktopHdc = GetDC(desktopHwnd)
    Call BitBlt(mDC, 0, 0, bi24BitInfo.bmiHeader.biWidth, _
                bi24BitInfo.bmiHeader.biHeight, desktopHdc, 0, _
                0, vbSrcCopy)
    

    'copy header info to the first row
     Call MEMCPY(screenDumpBuff(0, 0), bi24BitInfo.bmiHeader.biSize, LenB(bi24BitInfo))
    'copy device independent bitmap to the screen dump array
     result = GetDIBits(desktopHdc, mBitmap, 0, _
                   bi24BitInfo.bmiHeader.biHeight, _
                   screenDumpBuff(0, 1), bi24BitInfo, DIB_RGB_COLORS)
    
    errorCode = GetLastError()
    
    Call ReleaseDC(desktopHwnd, desktopHdc)
    Call DeleteObject(mBitmap)
    Call DeleteDC(mDC)
    
End Sub

Public Function GetMonitorCount() As Long
  'returns the number of display
  'monitors on the desktop in a
  'multi-monitor system.
   GetMonitorCount = GetSystemMetrics(SM_CMONITORS)
End Function

Public Function MonitorEnumProc(ByVal hMonitor As Long, ByVal hdcMonitor As Long, _
                                ByVal rcMonitor As Long, dwData As Long) As Long

   Dim rc As T_RECT
   
  'copy the rectangle data from
  'rcMonitor into a RECT type
   Call CopyMemory(rc, ByVal rcMonitor, LenB(rc))
   
  With SystemInfo
    .scrInfo(.nMonitors).xRes = Abs(rc.Right - rc.Left)
    .scrInfo(.nMonitors).yRes = Abs(rc.Bottom - rc.Top)
    .scrInfo(.nMonitors).monitorHandle = hMonitor
    .scrInfo(.nMonitors).monitorHDC = hdcMonitor
    .nMonitors = .nMonitors + 1
  End With
   
   
  'return 1 to continue processing
  'monitor information, or return
  '0 to stop. Returning 1 obtains
  'info for all physical and invisible
  'pseudo-monitors associated with
  'mirroring drivers
   MonitorEnumProc = 1

End Function

Public Sub fillMonitorsInfo()
    Dim i As Long
    Dim displayDevice As T_DISPLAYDEVICE
    SystemInfo.nMonitors = 0
    Call EnumDisplayMonitors(0&, 0&, AddressOf MonitorEnumProc, 0)
 
    '''displayDevice.cb = LenB(displayDevice)
    ''For i = 0 To 10000
    ''    If EnumDisplayDevices(0&, i, displayDevice, 0) = 0 Then
    ''        MsgBox "N displays = " + str(i)
    ''        Exit Sub
    ''    End If
    ''Next
    
End Sub

Public Function isMultiMonitorSystem() As Boolean
  'Returns True if a multi-monitor system
   isMultiMonitorSystem = GetSystemMetrics(SM_CMONITORS) > 1
End Function

Public Sub getDesktopMaximumDims(w As Long, h As Long)
    
   If isMultiMonitorSystem() Then
      w = GetSystemMetrics(SM_CXVIRTUALSCREEN)
      h = GetSystemMetrics(SM_CYVIRTUALSCREEN)
   Else
      w = Screen.Width / Screen.TwipsPerPixelX
      h = Screen.Height / Screen.TwipsPerPixelY
   End If

End Sub

Public Sub fillSystemInfo()
    Dim hostname As String, ip As String
    
    
    Call GetNetInfo(ip, hostname)
    
    fillMonitorsInfo
    
    With SystemInfo
        .ip = ip
        .machineName = hostname
        If .nMonitors <> GetMonitorCount() Then
            putlog ("WARNING: number of monitors doesn't match")
        End If
    End With
    
End Sub

Public Sub getProcessAndWindowInfo()
    Dim i As Integer, j As Integer
    
    For i = 0 To UBound(RunningProcesses)
        With RunningProcesses(i)
            .nVisibleWindows = 0
            
            For j = 0 To UBound(.visibleWindows)
                .visibleWindows(j).handle = 0
                .visibleWindows(j).caption = ""
                .visibleWindows(j).parentWindowHandle = 0
                .visibleWindows(j).nChildWindows = 0
            Next
        End With
    Next
    NRunningProcesses = GetExeDetails(RunningProcesses)
    EnumWindows
End Sub

Private Sub EnumWindows()
    Dim hWindow As Long
  
    'Desktop is also a window
    hWindow = GetDesktopWindow
    Call GetWindowInfo(hWindow)
    
    hWindow = GetWindow(FORM_consoleDaemon.hWnd, GW_HWNDFIRST)
    
    Do  'walk through all windows, until hWindow = 0
      Call GetWindowInfo(hWindow)
      hWindow = GetWindow(hWindow, GW_HWNDNEXT)
    Loop Until hWindow = 0
    
    linkWindowList
End Sub

Public Function startProcess(scriptPath As String, scriptArguments As String, pathName As String, exeName As String, args As String, workingPath As String) As Boolean
     Dim pInfo As T_PROCESS_INFORMATION
     Dim sInfo As T_STARTUP_INFO
     Dim sNull As String
     Dim err As Long
     Dim tmpPath As String, tmpargs As String
     Dim wp As String, exn As String
     
     startProcess = False
     
     exn = Trim(scriptPath)
     If exn <> "" Then exn = exn + " " + Trim(scriptArguments) + " "
     wp = Trim0(workingPath)
     exn = exn + Trim(Trim0(pathName)) + Trim(Trim0(exeName))
    
     tmpargs = Trim(Trim0(args))
     If tmpargs <> "" Then
        exn = exn + " " + tmpargs
     End If
     
     If wp <> "" Then
        tmpPath = CurDir
        ChDir (wp)
        If CurDir <> wp Then
            putlog ("startProcess: can not change path")
            ChDir (tmpPath)
            Exit Function
        End If
     End If
     sInfo.cb = Len(sInfo)
     putlog ("Creating process: " + exn)
     err = CreateProcess(sNull, exn, ByVal 0&, ByVal 0&, 1&, NORMAL_PRIORITY_CLASS, ByVal 0&, sNull, sInfo, pInfo)
     startProcess = (err > 0)
     If tmpPath <> "" Then ChDir (tmpPath)
End Function

Public Function stopProcess(procDetail As T_PROCESSDETAILS)
    Dim i As Integer
    Dim pHnd As Long
    
    With procDetail
        putlog ("stopping process: " + procDetail.procEntry.szExeFile + " # of windows: " + str(.nVisibleWindows))
        For i = 0 To .nVisibleWindows - 1
            Call killWindow(procDetail, i)
        Next
        
        pHnd = OpenProcess(PROCESS_ALL_ACCESS, 0&, .procEntry.th32ProcessID)
        If (pHnd < 1) Then
            putlog ("stopProcess: no such process")
            Exit Function
        End If
        If TerminateProcess(pHnd, 0) = 0 Then
            putlog ("TerminateProcess() failed. Error code = " + str(GetLastError()))
        Else
            putlog ("TerminateProcess() succeded.")
        End If
        CloseHandle (pHnd)
    End With
End Function

Private Sub killWindow(procDetail As T_PROCESSDETAILS, windowIdx As Integer)
    Dim i As Integer
    Dim hWnd As Long
    Dim result As Long
    
    With procDetail.visibleWindows(windowIdx)
        For i = 0 To .nChildWindows - 1
            Call killWindow(procDetail, .childWindowIndex(i))
        Next
    
        hWnd = .handle
        result = SendMessageTimeout(hWnd, WM_CLOSE, 0&, 0&, SMTO_ABORTIFHUNG Or SMTO_BLOCK, 1000, result)
        If result > 0 Then
            putlog ("request to close window: " + .caption + " OK")
        Else
            putlog ("request to close window: " + .caption + " FAILED. Error = " + str(GetLastError))
        End If
    End With
End Sub


Public Sub loadAllKnownProcesses()
    Dim h As Integer
    Dim tmp As Integer
    Dim l As String
    Dim words() As String
    
    On Error GoTo err_handler
    
    putlog ("reading list of known processes.")
    
    h = -1
    tmp = FreeFile
    Open ALL_KNOWN_PROCESSES_LIST For Input As tmp
    h = tmp
    With AllKnownProcesses
        While Not EOF(h)
             Line Input #h, l
             l = Trim(l)
             If l <> "" Then
                If .nProcs >= MAX_KNOWN_PROCESSES - 1 Then
                    putlog ("WARNING: maximum number of known processes reached")
                    Close #h
                    Exit Sub
                End If
                If Left(l, 1) <> "#" Then
                    words = Split(l)
                    .procs(.nProcs).exeName = Trim(words(0))
                    .procs(.nProcs).zombieStateCnt = 0
                    .nProcs = .nProcs + 1
                End If
             End If
        Wend
    End With
    
    putlog ("Known processes list read. " + str(AllKnownProcesses.nProcs) + " items found")
    Exit Sub
    
err_handler:
    putlog ("reading known processes list: " + err.Description)
    If h > -1 Then Close #h
End Sub


Private Function getProcessCommandLine(pid As Long) As String
    Dim s As String * 300
    Dim retCode As Long
    
  
    retCode = getRemoteProcessCommandLine(pid, s, 200)
    If retCode <> 0 Then Exit Function
    
    getProcessCommandLine = s
    
End Function


Public Sub setName16(dest As NAME16I, src As String)

    With dest
        .iVal = Len(src)
        Call String2Bytes(.nam(0), src, .iVal)
    End With
End Sub

Public Function getName16String(src As NAME16I) As String
    
    getName16String = BSTR(src.nam)
    
End Function


Public Sub old___saveCurrentConfig()
    Dim i As Integer
  
    getProcessAndWindowInfo
    
    With ConsoleModes(CurrentConsoleModeIdx)
        .nProcs = 0
        For i = 0 To NRunningProcesses - 1
            If RunningProcesses(i).nVisibleWindows > 0 Then
                If RunningProcesses(i).procEntry.th32ProcessID <> GetCurrentProcessId Then
                    If ((UCase(Trim(RunningProcesses(i).path)) <> SYSTEM_WINDOWS_PATH) Or _
                        (UCase(Trim0(RunningProcesses(i).procEntry.szExeFile)) <> SYSTEM_WINDOWS_EXPLORER)) Then
                        .procs(.nProcs) = RunningProcesses(i)
                        .procs(.nProcs).running = 1
                        .procs(.nProcs).path = Trim(Trim0(.procs(.nProcs).path))
                        .procs(.nProcs).exeName = Trim(Trim0(.procs(.nProcs).procEntry.szExeFile))
                        .nProcs = .nProcs + 1
                    End If
                End If
            End If
        Next
    End With
    saveXMLConfig
End Sub

Public Sub saveCurrentConfig()
    Dim i As Integer
    Dim idx As Integer
    
    getProcessAndWindowInfo
        
    With ConsoleModes(CurrentConsoleModeIdx)
        ModeTmp.nProcs = .nProcs
        For i = 0 To .nProcs - 1
            ModeTmp.procs(i).arguments = .procs(i).arguments
            ModeTmp.procs(i).exeName = .procs(i).exeName
            ModeTmp.procs(i).path = .procs(i).path
            ModeTmp.procs(i).launcherScript = .procs(i).launcherScript
        Next
        
        .nProcs = 0
        For i = 0 To NRunningProcesses - 1
            If RunningProcesses(i).nVisibleWindows > 0 Then
                If RunningProcesses(i).procEntry.th32ProcessID <> GetCurrentProcessId Then
                    If ((UCase(Trim(RunningProcesses(i).path)) <> SYSTEM_WINDOWS_PATH) Or _
                        (UCase(Trim0(RunningProcesses(i).procEntry.szExeFile)) <> SYSTEM_WINDOWS_EXPLORER)) Then
                         idx = processBelongsToMode(RunningProcesses(i), ModeTmp)
                         .procs(.nProcs) = RunningProcesses(i)
                         If idx > -1 Then .procs(.nProcs).launcherScript = ModeTmp.procs(idx).launcherScript
                        .procs(.nProcs).running = 1
                        .procs(.nProcs).path = Trim(Trim0(.procs(.nProcs).path))
                        .procs(.nProcs).exeName = Trim(Trim0(.procs(.nProcs).procEntry.szExeFile))
                        .nProcs = .nProcs + 1
                    End If
                End If
            End If
        Next
    End With
    
    
    saveXMLConfig
End Sub

Public Function processBelongsToMode(p As T_PROCESSDETAILS, cMode As T_CONSOLE_MODE) As Integer
    Dim i As Integer
    
    
    With cMode
        For i = 0 To .nProcs - 1
            If (UCase(Trim(Trim0(.procs(i).exeName))) = UCase(Trim(Trim0(p.exeName))) _
                And UCase(Trim(Trim0((.procs(i).arguments))) = UCase(Trim(Trim0(p.arguments))))) Then
                processBelongsToMode = i
                Exit Function
            End If
        Next
    End With
    
    processBelongsToMode = -1
    
End Function
