Attribute VB_Name = "TreeVievUtils"

Option Explicit
'
' Copyright  1997-1999 Brad Martinez, http://www.mvps.org
'
' Code was written in and formatted for 8pt MS San Serif

' User-defined as the maximum treeview item text length.
' If an items text exceeds this value when calling GetTVItemText,
' there will be problems...
Public Const MAX_ITEM = 256

Public Const EM_SETTABSTOPS = &HCB

'Declare Function IsWindowUnicode Lib "user32" (ByVal hwnd As Long) As Long

' Determines whether a buffer *probably* contains a form of Unicode text.
' Is unsupported on Win95/98
Declare Function IsTextUnicode Lib "advapi32" _
                            (lpBuffer As Any, _
                            ByVal cb As Long, _
                            lpi As Long) As Long
Public Const IS_TEXT_UNICODE_UNICODE_MASK = &HF

' What IsTextUnicode returns on 95/98...
' "This function is only valid in Win32 mode."
Public Const ERROR_CALL_NOT_IMPLEMENTED = 120

Declare Sub MoveMemory Lib "kernel32" Alias "RtlMoveMemory" (pDest As Any, pSource As Any, ByVal dwLength As Long)
Declare Function LocalSize Lib "kernel32" (ByVal hMem As Long) As Long

Declare Function CharNextA Lib "user32" (lpsz As Any) As Long
Declare Function lstrlenA Lib "kernel32" (lpString As Any) As Long
Declare Function lstrcpyA Lib "kernel32" (lpString1 As Any, lpString2 As Any) As Long
Declare Function lstrcmpA Lib "kernel32" (lpString1 As Any, lpString2 As Any) As Long
Declare Function lstrlenW Lib "kernel32" (lpString As Any) As Long

' Converts a Unicode string to a ANSI string.
' Specify -1 for cchWideChar and 0 for cchMultiByte to return the ANSI string char count.
Declare Function WideCharToMultiByte Lib "kernel32" _
                            (ByVal CodePage As Long, _
                            ByVal dwFlags As Long, _
                            lpWideCharStr As Any, _
                            ByVal cchWideChar As Long, _
                            lpMultiByteStr As Any, _
                            ByVal cchMultiByte As Long, _
                            ByVal lpDefaultChar As String, _
                            ByVal lpUsedDefaultChar As Long) As Long
' CodePage
Public Const CP_ACP = 0        ' ANSI code page
Public Const CP_OEMCP = 1   ' OEM code page

Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
                            (ByVal hwnd As Long, _
                            ByVal wMsg As Long, _
                            wParam As Any, _
                            lParam As Any) As Long   ' <---

' ===========================================================================
' what's in treeview hItems... i.e
'     MoveMemory ti, ByVal hItem, Len(ti)

' very undocumented, but has been tested and is consistant for real treeviews on the following:
' (testing included all versions of the Comctl32.ocx amd Mscomctl.ocx TreeViews as well)
'
'  Platform        IE version    Comctl32.dll version       TREEITEM bytes
  ' Win95          IE2 (?)         4.00.950                        32 bytes
  ' Win95          IE3.02         4.70                               32 bytes
  ' NT4 SP3      IE3.02         4.70                               32 bytes
  
  ' Win95          IE4.01         4.72.3110.1                   36 bytes
  ' NT4 SP5      IE4.0           4.72.3612.1702             36 bytes
  ' Win98          IE4.72         4.72.3612.1702             36 bytes
  
  ' NT5 b2         IE5.00         5.00.0707.3000             36 bytes
  ' W2K(b3)       IE5.00         5.81 (5.00.2116.100)     36 bytes
  
  ' Win95          IE5.??         5.80 (5.00.2314.1000)   36 bytes
  ' NT4 SP5      IE5.??         5.80 (5.00.2314.1000)   36 bytes
  ' W2K(rc1)      IE5.??         5.81 (5.00.2919.800)     36 bytes
'
' Set in the Project Properties dialog, Make tab.
'#Const WIN32_IE = &H500

Public Type TREEITEM   ' ti
  hitemParent As Long             ' handle of the item's parent, can be 0 (root's parent is a dummy
                                                ' item, whose only valid member is the hitemChild, the root itself)
  hitemNextSibling As Long      ' handle of the item's next sibling, can be 0
  hitemChild As Long                ' handle of the item's first child, can be 0
  lpszText As Long                   ' pointer to the item's text, allocated in DWORD chunks, can be
                                                ' LPSTR_TEXTCALLBACK, is Unicode on NT.
  wState As Integer                  ' item's state flags, 1st byte is state, low nibble of 2nd byte is
                                                ' overlay image index, high nibble of 2nd byte is state image index
  wImage As Integer                 ' normal icon imagelist index, can be I_IMAGECALLBACK (&HFFFF)
  wSelectedImage As Integer   ' selected icon imagelist index, can be I_IMAGECALLBACK (&HFFFF)
  cxItem As Integer                   ' item's label width in pixels (TVM_GETITEMRECT/TRUE)
  wIndex As Integer                  ' visible item's 0-based physical tree position, is -1 if in collapsed parent, can change
  bLevel As Byte                       ' item's 0-based hierarchical level in the tree
  bChildren As Byte                   ' TVITEM.cChildren, can't be I_CHILDRENCALLBACK (?)
  lParam As Long                      ' TVITEM.lParam, usually points to something internally allocated
#If (WIN32_IE >= &H400) Then
  dwID As Long                        ' always 1
#End If
End Type   ' / 36 bytes (WIN32_IE < &H400 = 32 bytes)

' Microsoft, in its complete freedom to break the undocumented, has done just that.
' Begining with Comctl32.dll v5.xx, the TREEITEM structure above has been changed,
' not by much, but just enough to break the apps of folks who decided to use it. The
' lParam  member has been moved up, from below the bChildren member, to below the
' lpszText member. The dwID (was dwIDBit) member's value has also been changed.
'''#If (WIN32_IE >= &H500) Then
Public Type TREEITEM5   ' ti5
  hitemParent As Long             ' handle of the item's parent, can be 0 (root's parent is a dummy
                                                ' item, whose only valid member is the hitemChild, the root itself)
  hitemNextSibling As Long      ' handle of the item's next sibling, can be 0
  hitemChild As Long                ' handle of the item's first child, can be 0
  lpszText As Long                   ' pointer to the item's Unicode text, allocated in DWORD chunks,
                                                ' can be LPSTR_TEXTCALLBACK.
  lParam As Long                      ' TVITEM.lParam, usually points to something internally allocated
  wState As Integer                  ' item's state flags, 1st byte is state, low nibble of 2nd byte is
                                                ' overlay image index, high nibble of 2nd byte is state image index
  wImage As Integer                 ' normal icon imagelist index, can be I_IMAGECALLBACK (&HFFFF)
  wSelectedImage As Integer   ' selected icon imagelist index, can be I_IMAGECALLBACK (&HFFFF)
  cxItem As Integer                   ' item's label width in pixels (TVM_GETITEMRECT/TRUE)
  wIndex As Integer                  ' visible item's 0-based physical tree position, is -1 if in collapsed parent, can change
  bLevel As Byte                       ' item's 0-based hierarchical level in the tree
  bChildren As Byte                   ' TVITEM.cChildren, can't be I_CHILDRENCALLBACK (?)
  dwID As Long                        ' always &HABCD0001 ?
End Type   ' / 36 bytes
''''#End If

' ===========================================================================
' Comctl32.dll version definitions

Public Type DLLVERSIONINFO
  cbSize As Long
  dwMajorVersion As Long
  dwMinorVersion As Long
  dwBuildNumber As Long
  dwPlatformId As Long
End Type

Declare Function DllGetVersion_Comctl32 Lib "comctl32" Alias "DllGetVersion" (pdvi As DLLVERSIONINFO) As Long

' ===========================================================================
' treeview definitions

' Callback constants

' TV/LVITEM.pszText
Public Const LPSTR_TEXTCALLBACK = (-1)

' TVITEM.iImage/iSelectedImage, LVITEM.iImage
Public Const I_IMAGECALLBACK = (-1)

' TVITEM.cChildren
Public Const I_CHILDRENCALLBACK = (-1&)

Public Type TVITEM   ' was TV_ITEM
  mask As TVITEM_mask
  hItem As Long
  state As TVITEM_state
  stateMask As TVITEM_state
  pszText As Long       ' pointer
  cchTextMax As Long
  iImage As Long
  iSelectedImage As Long
  cChildren As Long
  lParam As Long
End Type

Public Enum TVITEM_mask
  TVIF_TEXT = &H1
  TVIF_IMAGE = &H2
  TVIF_PARAM = &H4
  TVIF_STATE = &H8
  TVIF_HANDLE = &H10
  TVIF_SELECTEDIMAGE = &H20
  TVIF_CHILDREN = &H40

#If (WIN32_IE >= &H400) Then
  TVIF_INTEGRAL = &H80
#End If
  
  TVIF_DI_SETITEM = &H1000   ' Notification
End Enum
  
Public Enum TVITEM_state
  TVIS_FOCUSED = &H1   ' no more than one item
  TVIS_SELECTED = &H2   ' highlight, more than one?!
  TVIS_CUT = &H4
  TVIS_DROPHILITED = &H8
  TVIS_BOLD = &H10
  TVIS_EXPANDED = &H20
  TVIS_EXPANDEDONCE = &H40
  
#If (WIN32_IE >= &H300) Then
  TVIS_EXPANDPARTIAL = &H80
#End If
  
  TVIS_OVERLAYMASK = &HF00
  TVIS_STATEIMAGEMASK = &HF000
  TVIS_USERMASK = &HF000
End Enum

' messages
Public Const TV_FIRST = &H1100
Public Const TVM_GETNEXTITEM = (TV_FIRST + 10)
Public Const TVM_GETITEM = (TV_FIRST + 12)
Public Const TVM_SETITEM = (TV_FIRST + 13)

Public Enum TVM_GETNEXTITEM_wParam
  TVGN_ROOT = &H0
  TVGN_NEXT = &H1
  TVGN_PREVIOUS = &H2
  TVGN_PARENT = &H3
  TVGN_CHILD = &H4
  TVGN_FIRSTVISIBLE = &H5
  TVGN_NEXTVISIBLE = &H6
  TVGN_PREVIOUSVISIBLE = &H7
  TVGN_DROPHILITE = &H8
  TVGN_CARET = &H9
#If (WIN32_IE >= &H400) Then
  TVGN_LASTVISIBLE = &HA
#End If
End Enum
'

' Returns an ANSI string from a Long pointer to an ANSI or Unicode string.

Public Function GetStrFromPtr(lpsz As Long) As String
  Dim cb As Long
  Dim sRtn As String
  Dim dwRtnFlags As Long
  Dim fIsUnicode As Boolean
  
  ' lstrlenW, which is implemented on 95/98, returns 1/2 an
  ' ANSI string's char count
  cb = lstrlenW(lpsz) * 2
  If cb Then
    sRtn = String$(cb, 0)
  
    dwRtnFlags = IS_TEXT_UNICODE_UNICODE_MASK
    fIsUnicode = IsTextUnicode(ByVal lpsz, cb, dwRtnFlags)
    
    ' If IsTextUnicode err'd (probably 95/98), and the string is
    ' more than 1 char long, see if its 2nd char is a vbNullChar...
    If Err.LastDllError And (cb > 1) Then
      fIsUnicode = (lstrcmpA(ByVal CharNextA(ByVal lpsz), ByVal vbNullChar) = 0)
    End If
    
    If fIsUnicode Then
      Call WideCharToMultiByte(CP_ACP, 0, ByVal lpsz, cb, ByVal sRtn, cb, 0, 0)
    Else
      Call lstrcpyA(ByVal sRtn, ByVal lpsz)
    End If
    
    ' We need this since we did some heavy buffer padding...
    GetStrFromPtr = GetStrFromBufferA(sRtn)
  End If
  
End Function

' Returns an ANSI string from a Long pointer to an ANSI string.

Public Function GetStrFromPtrA(lpszA As Long) As String
  Dim sRtn As String
  sRtn = String$(lstrlenA(ByVal lpszA), 0)
  Call lstrcpyA(ByVal sRtn, ByVal lpszA)
  GetStrFromPtrA = sRtn
End Function

' Returns an ANSI string from a Long pointer to a Unicode string.

Public Function GetStrFromPtrW(lpszW As Long) As String
  Dim sRtn As String
  sRtn = String$(lstrlenW(ByVal lpszW) * 2, 0)   ' 2 bytes/char
'  sRtn = String$(WideCharToMultiByte(CP_ACP, 0, ByVal lpszW, -1, 0, 0, 0, 0), 0)
  Call WideCharToMultiByte(CP_ACP, 0, ByVal lpszW, -1, ByVal sRtn, Len(sRtn), 0, 0)
  GetStrFromPtrW = GetStrFromBufferA(sRtn)
End Function

' Returns the string before first null char encountered (if any) from an ANSI string.

Public Function GetStrFromBufferA(sz As String) As String
  If InStr(sz, vbNullChar) Then
    GetStrFromBufferA = Left$(sz, InStr(sz, vbNullChar) - 1)
  Else
    ' If sz had no null char, the Left$ function
    ' above would return a zero length string ("").
    GetStrFromBufferA = sz
  End If
End Function

' ===========================================================================
' treeview calls

' Returns the lParam of the specified treeview item.

Public Function GetTVItemlParam(HwndTV As Long, hItem As Long) As Long
  Dim tvi As TVITEM
  tvi.mask = TVIF_PARAM
  tvi.hItem = hItem
  If TreeView_GetItem(HwndTV, tvi) Then
    GetTVItemlParam = tvi.lParam
  End If
End Function

' Returns the text of the specified treeview item if successful,
' returns an empty string otherwise.

'   hwndTV      - treeview's window handle
'   hItem          - item's handle whose text is to be to returned
'   cbItem        - length of the specified item's text.

Public Function GetTVItemText(HwndTV As Long, _
                                                  hItem As Long, _
                                                  Optional cbItem As Long = MAX_ITEM) As String
  Dim tvi As TVITEM
  
  ' Initialize the struct to retrieve the item's text.
  tvi.mask = TVIF_TEXT
  tvi.hItem = hItem
  tvi.pszText = StrPtr(String$(cbItem, 0))
  tvi.cchTextMax = cbItem
  
  If TreeView_GetItem(HwndTV, tvi) Then
    GetTVItemText = GetStrFromPtrA(tvi.pszText)
  End If

End Function

' Returns the specified treeview item's TVITEM.cChildren value

'   hwndTV      - treeview's window handle
'   hItem          - handle of the item to evaluate

Public Function GetTVItemChildren(HwndTV As Long, _
                                                      hItem As Long) As Long
  Dim tvi As TVITEM

  tvi.mask = TVIF_CHILDREN
  tvi.hItem = hItem
  Call TreeView_GetItem(HwndTV, tvi)
  GetTVItemChildren = tvi.cChildren
  
End Function

' ===========================================================================
' Treeview macros defined in Commctrl.h

' Retrieves some or all of a tree-view item's attributes.
' Returns TRUE if successful or FALSE otherwise.

Public Function TreeView_GetItem(hwnd As Long, pitem As TVITEM) As Boolean
  TreeView_GetItem = SendMessage(hwnd, TVM_GETITEM, 0, pitem)
End Function

' Sets some or all of a tree-view item's attributes.
' Old docs say returns zero if successful or - 1 otherwise.
' New docs say returns TRUE if successful, or FALSE otherwise

Public Function TreeView_SetItem(hwnd As Long, pitem As TVITEM) As Boolean
  TreeView_SetItem = SendMessage(hwnd, TVM_SETITEM, 0, pitem)
End Function

' TreeView_GetNextItem

' Retrieves the tree-view item that bears the specified relationship to a specified item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetNextItem(hwnd As Long, hItem As Long, flag As Long) As Long
  TreeView_GetNextItem = SendMessage(hwnd, TVM_GETNEXTITEM, ByVal flag, ByVal hItem)
End Function

' Retrieves the first child item. The hitem parameter must be NULL.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetChild(hwnd As Long, hItem As Long) As Long
  TreeView_GetChild = TreeView_GetNextItem(hwnd, hItem, TVGN_CHILD)
End Function

' Retrieves the next sibling item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetNextSibling(hwnd As Long, hItem As Long) As Long
  TreeView_GetNextSibling = TreeView_GetNextItem(hwnd, hItem, TVGN_NEXT)
End Function

' Retrieves the previous sibling item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetPrevSibling(hwnd As Long, hItem As Long) As Long
  TreeView_GetPrevSibling = TreeView_GetNextItem(hwnd, hItem, TVGN_PREVIOUS)
End Function

' Retrieves the parent of the specified item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetParent(hwnd As Long, hItem As Long) As Long
  TreeView_GetParent = TreeView_GetNextItem(hwnd, hItem, TVGN_PARENT)
End Function

' Retrieves the first visible item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetFirstVisible(hwnd As Long) As Long
  TreeView_GetFirstVisible = TreeView_GetNextItem(hwnd, 0, TVGN_FIRSTVISIBLE)
End Function

' Retrieves the next visible item that follows the specified item. The specified item must be visible.
' Use the TVM_GETITEMRECT message to determine whether an item is visible.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetNextVisible(hwnd As Long, hItem As Long) As Long
  TreeView_GetNextVisible = TreeView_GetNextItem(hwnd, hItem, TVGN_NEXTVISIBLE)
End Function

' Retrieves the first visible item that precedes the specified item. The specified item must be visible.
' Use the TVM_GETITEMRECT message to determine whether an item is visible.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetPrevVisible(hwnd As Long, hItem As Long) As Long
  TreeView_GetPrevVisible = TreeView_GetNextItem(hwnd, hItem, TVGN_PREVIOUSVISIBLE)
End Function

' Retrieves the currently selected item.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetSelection(hwnd As Long) As Long
  TreeView_GetSelection = TreeView_GetNextItem(hwnd, 0, TVGN_CARET)
End Function

' Retrieves the item that is the target of a drag-and-drop operation.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetDropHilight(hwnd As Long) As Long
  TreeView_GetDropHilight = TreeView_GetNextItem(hwnd, 0, TVGN_DROPHILITE)
End Function

' Retrieves the topmost or very first item of the tree-view control.
' Returns the handle to the item if successful or 0 otherwise.

Public Function TreeView_GetRoot(hwnd As Long) As Long
  TreeView_GetRoot = TreeView_GetNextItem(hwnd, 0, TVGN_ROOT)
End Function


