#include "epics2tine.h"

#ifdef VXWORKS
#include "taskLib.h"
#include "moduleLib.h"
#include "loadLib.h"
#endif	/* VXWORKS */

#define ccerr(i) { cc = i; goto err; }

#define DZK(c)   if(verboseTINE) {c}
long verboseTINE = 0;

extern int NGdebug;
extern void haltServer(void);
extern int max_rcvbuf, max_sndbuf;

/************************************************************************
 *									*
 *	EPICS Specific Part						*
 *									*
 ************************************************************************/
int	CA_OK  	       = 0;
int	ERRORnoDevice  = 1;
int	ERRORnoChannel = 2;
int	ERRORnoValue   = 3;

long tineGetNumRecords()
{
DBENTRY     dbentry;
DBENTRY     *pdbentry=&dbentry;
long        status = 0;
int         nrecords = 0;
int         total = 0;

    dbInitEntry(pdbbase,pdbentry);
    status = dbFirstRecordType(pdbentry);
    if(status) printf("No record description\n");

    while(!status) {
	nrecords = dbGetNRecords(pdbentry);
	total += nrecords;
	status = dbNextRecordType(pdbentry);
    }
    dbFinishEntry(pdbentry);
    printf("Total records (tineGetNumRecords): %d\n",total);
    return(total);
}

long tineGetRecords(NAME32 *pNames)
{
DBENTRY     dbentry;
DBENTRY     *pdbentry=&dbentry;
long        status = 0;
short       c = 0;

    dbInitEntry(pdbbase,pdbentry);
    status = dbFirstRecordType(pdbentry);
    if(status) printf("No record description\n");

    while(!status) {
        status = dbFirstRecord(pdbentry);
        while(!status) {
            strncpy(pNames[c].name, dbGetRecordName(pdbentry), 32);
            status = dbNextRecord(pdbentry);
            c++;
        }
        status = dbNextRecordType(pdbentry);
    }
    dbFinishEntry(pdbentry);

    printf("Total records (tineGetRecords): %d\n", c);
    return (c);
}

long tineGetNumProperties(char *pname)
{
    return(2);
}

long tineGetProperties(char *pname, Tproperty *pprop)
{
DBADDR    	addr;
long            status = 0;

    DZK(printf("add to the generic list of props >>> %s:\n",pname);)

    status = dbNameToAddr(pname, &addr);
    if(status) {
	printf("error no record with this name <%s>\n",pname);
        errMessage(status," dbNameToAddr failed");
        return(0);
    }

    strncpy(pprop[0].name,"DESC",16);
    strncpy(pprop[0].desc,"Description",32);
    pprop[0].fmt = getTINEformat(DBF_STRING);
    pprop[0].elem = 40;

    strncpy(pprop[1].name,"VAL",16);
    strncpy(pprop[1].desc,"Description",32);
    pprop[1].fmt = getTINEformat(DBF_DOUBLE);
    pprop[1].elem = 1;

    status = tineFillProp(pname, &addr);
    if(status <= 0) return(0);

    return(2);
}

static int tineFillProp(char *pname,DBADDR *paddr)
{
int		co;

    DZK(printf(">>> tineFillProp\n");)
    co = 2;

    /* Only Static List support for the beginning
     * Dynamic works fine, but don't need more then VAL & DESC 
     * for the beginning
     */
    return (co);
}

int tineGetFieldValue(char *pname, char *pprop, void *ptr, int tineFmt, int *pCount)
{
long 		buffer[2000];
long 		*pbuffer=&buffer[0];
DBADDR 		addr;
long		status = 0;
long		options,nr;
int		getUnixTime = FALSE;
char		locName[40] = "";
int		i;

    DZK(printf(">> tineGetFieldValue(%s, %s, %x, %d, %d)\n",pname,pprop,(int)ptr,tineFmt,*pCount);)

    sprintf(locName, "%s.%s", pname, pprop);

    status=dbNameToAddr(&locName[0],&addr);
    if(status) {
	DZK(errMessage(status," dbNameToAddr failed");
	    printf(" dbNameToAddr failed >>> no record with name %s\n", locName);)
	return(ERRORnoChannel);
    }

    if( tineFmt == CF_LONG && addr.precord->dtyp == 2 ) getUnixTime = TRUE;
    if( getUnixTime ) {
	sprintf(locName, "%s.%s", pname, "SVAL");
        
    	DZK(printf("for Event Time String locName = %s\n",locName);)
	status=dbNameToAddr(&locName[0],&addr);
	if(status) {
	    errMessage(status," dbNameToAddr failed");
            printf(" dbNameToAddr failed >>> no record with name %s\n", locName);
            return(ERRORnoChannel);
   	}
    } 

    nr=MIN(addr.no_elements,((sizeof(buffer))/addr.field_size));
    options=0;
    if(addr.dbr_field_type==DBR_ENUM) {
	status=dbGetField(&addr,DBR_STRING,pbuffer,&options,&nr,NULL);
    } else {
	status=dbGetField(&addr,addr.dbr_field_type,pbuffer,&options,&nr,NULL);
    }
    
    if( status != 0 ) {
	printf(" dbGetField failed with status = %d\n", (int)status);
	return(ERRORnoValue);
    }

    *pCount = *pCount + nr;
    DZK(printf("\t%s >>> addr.dbr_field_type = %d\n", locName, addr.dbr_field_type );
	printf("buffer = %s\n", (char *)buffer);)

    if(nr > 1) { 
      	for(i = 0; i < nr; i++) {
    	    switch (addr.dbr_field_type) {
    	    case DBR_ENUM:
    	    case DBR_STRING:
    	    case DBR_CHAR:
    	    case DBR_UCHAR:
	        switch (tineFmt) {
	        case CF_SHORT: 	*(short *)ptr  = atoi((char *)buffer); ptr = (char *)ptr + sizeof(short); status = 0; break;
	        case CF_LONG: 	*(long *)ptr   = atoi((char *)buffer); ptr = (char *)ptr + sizeof(long); status = 0; break;
	        case CF_FLOAT: 	*(float *)ptr  = atof((char *)buffer); ptr = (char *)ptr + sizeof(float); status = 0; break;
	        case CF_DOUBLE: *(double *)ptr = atof((char *)buffer); ptr = (char *)ptr + sizeof(double); status = 0; break;
	        case CF_NAME16: strncpy(ptr, (char *)buffer, 16); ptr = (char *)ptr + 16; status = 0; break;
     		case CF_NAME32: strncpy(ptr, (char *)buffer, 32); ptr = (char *)ptr + 32; status = 0; break;
        	case CF_TEXT: 	strncpy(ptr, (char *)buffer, 40); ptr = (char *)ptr + 40; status = 0; break;
        	default: 	status = illegal_format;
	    }
	    pbuffer += addr.field_size;	
	    break;
        case DBR_SHORT:
        case DBR_USHORT:
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr  = *(short *)pbuffer; ptr = (char *)ptr + sizeof(short); status = 0; break; 
	    case CF_LONG:   *(long *)ptr   = *(short *)pbuffer; ptr = (char *)ptr + sizeof(long); status = 0; break;
	    case CF_FLOAT:  *(float *)ptr  = *(short *)pbuffer; ptr = (char *)ptr + sizeof(float); status = 0; break;  
	    case CF_DOUBLE: *(double *)ptr = *(short *)pbuffer; ptr = (char *)ptr + sizeof(double); status = 0; break;
	    case CF_NAME16: strncpy(ptr, (char *)buffer, 16); ptr = (char *)ptr + 16; status = 0; break;
	    case CF_NAME32: strncpy(ptr, (char *)buffer, 32); ptr = (char *)ptr + 32; status = 0; break;
	    case CF_TEXT:   strncpy(ptr, (char *)buffer, 40); ptr = (char *)ptr + 40; status = 0; break;
	    default:	    status = illegal_format;
	    }
	    pbuffer += addr.field_size;
	    break;
        case DBR_LONG:
        case DBR_ULONG:
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr  = *pbuffer; ptr = (char *)ptr + sizeof(short); status = 0; break;
	    case CF_LONG:   *(long *)ptr   = *pbuffer; ptr = (char *)ptr + sizeof(long); status = 0; break;
	    case CF_FLOAT:  *(float *)ptr  = *pbuffer; ptr = (char *)ptr + sizeof(float); status = 0; break;
	    case CF_DOUBLE: *(double *)ptr = *pbuffer; ptr = (char *)ptr + sizeof(double); status = 0; break;
	    case CF_NAME16: strncpy(ptr, (char *)buffer, 16); ptr = (char *)ptr + 16; status = 0; break;
	    case CF_NAME32: strncpy(ptr, (char *)buffer, 32); ptr = (char *)ptr + 32; status = 0; break;
	    case CF_TEXT:   strncpy(ptr, (char *)buffer, 40); ptr = (char *)ptr + 40; status = 0; break;
	    default:	    status = illegal_format;
	    }
	    pbuffer += addr.field_size;
	    break;
        case DBR_FLOAT:
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr  = *(float *)pbuffer; ptr = (char *)ptr + sizeof(short); status = 0; break;
	    case CF_LONG:   *(long *)ptr   = *(float *)pbuffer; ptr = (char *)ptr + sizeof(long); status = 0; break;
	    case CF_FLOAT:  *(float *)ptr  = *(float *)pbuffer; ptr = (char *)ptr + sizeof(float); status = 0; break;
	    case CF_DOUBLE: *(double *)ptr = *(float *)pbuffer; ptr = (char *)ptr + sizeof(double); status = 0; break;
	    case CF_NAME16: cvtFloatToString(*(float *)pbuffer, ptr, NULL); ptr = (char *)ptr + 16; status = 0; break;
	    case CF_NAME32: cvtFloatToString(*(float *)pbuffer, ptr, NULL); ptr = (char *)ptr + 32; status = 0; break;
	    case CF_TEXT:   cvtFloatToString(*(float *)pbuffer, ptr, NULL); ptr = (char *)ptr + 40; status = 0; break;
	    default:	    status = illegal_format;
	    }
	    pbuffer += addr.field_size;
	    break;
        case DBR_DOUBLE:
	    switch (tineFmt) {
            case CF_SHORT:  *(short *)ptr  = *(double *)pbuffer; ptr = (char *)ptr + sizeof(short); status = 0; break;
            case CF_LONG:   *(long *)ptr   = *(double *)pbuffer; ptr = (char *)ptr + sizeof(long); status = 0; break;
            case CF_FLOAT:  *(float *)ptr  = *(double *)pbuffer; ptr = (char *)ptr + sizeof(float); status = 0; break;
            case CF_DOUBLE: *(double *)ptr = *(double *)pbuffer; ptr = (char *)ptr + sizeof(double); status = 0; break;
	    case CF_NAME16: cvtDoubleToString(*(double *)pbuffer, ptr, NULL); ptr = (char *)ptr + 16; status = 0; break;
	    case CF_NAME32: cvtDoubleToString(*(double *)pbuffer, ptr, NULL); ptr = (char *)ptr + 32; status = 0; break;
	    case CF_TEXT:   cvtDoubleToString(*(double *)pbuffer, ptr, NULL); ptr = (char *)ptr + 40; status = 0; break;
    	    default:        status = illegal_format;
            }
	    pbuffer += addr.field_size;
	    break;
        case DBR_PUT_ACKT:
        case DBR_PUT_ACKS:
        case DBR_NOACCESS:
        default: 		
	    DZK(printf("default\n");)
	    status = ERRORnoValue;
        }
        }
    } else {
        switch (addr.dbr_field_type) {
        case DBR_ENUM:
        case DBR_STRING:
        case DBR_CHAR:
        case DBR_UCHAR:
	    DZK(printf("DBR_STRING %s \n", (char *)buffer);)
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr  = atoi((char *)buffer); status = 0; break;
	    case CF_LONG:   *(long *)ptr   = atoi((char *)buffer); status = 0; break;
	    case CF_FLOAT:  *(float *)ptr  = atof((char *)buffer); status = 0; break; 
	    case CF_DOUBLE: *(double *)ptr = atof((char *)buffer); status = 0; break;
	    case CF_NAME16: strncpy(ptr, (char *)buffer, 16); status = 0; break;
	    case CF_NAME32: strncpy(ptr, (char *)buffer, 32); status = 0; break;
	    case CF_TEXT:   strncpy(ptr, (char *)buffer, 40); status = 0; break;
	    default: 	    status = illegal_format;
	    }
	    break;
        case DBR_SHORT:
        case DBR_USHORT:
	    DZK(printf("DBR_SHORT %d\n", *(short *)pbuffer);)
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr  = *(short *)pbuffer; status = 0; break; 
	    case CF_LONG:   *(long *)ptr   = *(short *)pbuffer; status = 0; break;
	    case CF_FLOAT:  *(float *)ptr  = *(short *)pbuffer; status = 0; break;  
	    case CF_DOUBLE: *(double *)ptr = *(short *)pbuffer; status = 0; break;
	    case CF_NAME16: strncpy(ptr, (char *)buffer, 16); status = 0; break;
	    case CF_NAME32: strncpy(ptr, (char *)buffer, 32); status = 0; break;
	    case CF_TEXT:   strncpy(ptr, (char *)buffer, 40); status = 0; break;
	    default:	    status = illegal_format;
	    }
	    break;
        case DBR_LONG:
        case DBR_ULONG:
	    DZK(printf("DBR_LONG %d\n", *(int *)pbuffer);)
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr = *pbuffer; status = 0; break;
	    case CF_LONG:   *(long *)ptr = *pbuffer; status = 0; break;
	    case CF_FLOAT:  *(float *)ptr = *pbuffer; status = 0; break;
	    case CF_DOUBLE: *(double *)ptr = *pbuffer; status = 0; break;
	    case CF_NAME16: strncpy(ptr, (char *)buffer, 16); status = 0; break;
	    case CF_NAME32: strncpy(ptr, (char *)buffer, 32); status = 0; break;
	    case CF_TEXT:   strncpy(ptr, (char *)buffer, 40); status = 0; break;
	    default:	    status = illegal_format;
	    }
	    break;
        case DBR_FLOAT:
	    DZK(printf("DBR_FLOAT %f\n", *(float *)pbuffer);)
	    switch (tineFmt) {
	    case CF_SHORT:  *(short *)ptr = *(float *)pbuffer; status = 0; break;
	    case CF_LONG:   *(long *)ptr = *(float *)pbuffer; status = 0; break;
	    case CF_FLOAT:  *(float *)ptr = *(float *)pbuffer; status = 0; break;
	    case CF_DOUBLE: *(double *)ptr = *(float *)pbuffer; status = 0; break;
	    case CF_NAME16:	
	    case CF_NAME32:
	    case CF_TEXT:   cvtFloatToString(*(float *)pbuffer, ptr, NULL); status = 0; break;
	    default:	    status = illegal_format;
	    }
	    break;
        case DBR_DOUBLE:
	    DZK(printf("DBR_DOUBLE %f\n", *(double *)pbuffer);)
	    switch (tineFmt) {
            case CF_SHORT:  *(short *)ptr = *(double *)pbuffer; status = 0; break;
            case CF_LONG:   *(long *)ptr = *(double *)pbuffer; status = 0; break;
            case CF_FLOAT:  *(float *)ptr = *(double *)pbuffer; status = 0; break;
            case CF_DOUBLE: *(double *)ptr = *(double *)pbuffer; status = 0; break;
	    case CF_NAME16:
	    case CF_NAME32:
	    case CF_TEXT:   cvtDoubleToString(*(double *)pbuffer, ptr, NULL); status = 0; break;
    	    default:        status = illegal_format;
            }
	    break;
        case DBR_PUT_ACKT:
        case DBR_PUT_ACKS:
        case DBR_NOACCESS:
        default: 		
	    DZK(printf("default\n");)
	    status = ERRORnoValue;
        }
    }

    return(status);
}

long tinePutFieldValue(char *pname,char *pvalue)
{
/* declare buffer long just to ensure correct alignment */
DBADDR addr;
long	  status;

    /* make sure value was passed*/
    if(pvalue==0) {
	errMessage(0L,"No value was specified");
	return(1);
    }
    status=dbNameToAddr(pname,&addr);
    if(status) {
         errMessage(status,"dbNameToAddr error");
         return(status);
    }
    /* For enumerated types must allow for ENUM rather than string*/
    /* If entire field is digits then use DBR_ENUM else DBR_STRING*/
    if((addr.dbr_field_type==DBR_ENUM) && (*pvalue!=0)
    &&  (strspn(pvalue,"0123456789")==strlen(pvalue))) {
	    short value;

	    sscanf(pvalue,"%hu",&value);
	    status=dbPutField(&addr,DBR_ENUM,&value,1L);
    } else {
	status=dbPutField(&addr,DBR_STRING,pvalue,1L);
    }
    if(status) {
         errMessage(status,"dbPutField error");
         return(status);
    }
    /* status=(long) tineGetFieldValue(pname, "VAL"); */
    return(status);
}

int tineGetAlarmLimits(char *recName, ALARM_WATCH_THRESHOLDS *pThr)
{
int status = 0;

  status = tineGetFieldValue(recName,"HIHI",&(pThr->hi),CF_FLOAT,1);
  if(status) return(FALSE);
  tineGetFieldValue(recName,"HIGH",&(pThr->hiwarn),CF_FLOAT,1);
  if(status) return(FALSE);
  tineGetFieldValue(recName,"LOLO",&(pThr->lo),CF_FLOAT,1);
  if(status) return(FALSE);
  tineGetFieldValue(recName,"LOW",&(pThr->lowarn),CF_FLOAT,1);
  if(status) return(FALSE);

  return(TRUE);
}

int tineGetRecordFromFile(void)
{
int c;
FILE *fp = NULL;
char str[32];
static char RecordFilename[32];
	
	if (strlen(RecordFilename) == 0) {
		sprintf(RecordFilename,"%s","freeRecord.list");
		printf("file to open = %s\n", RecordFilename);
	}
	if ((fp=fopen(RecordFilename,"r")) == NULL) {
		printf("no such file\n");
		return -1;
	}

	c = 0;
	while (fscanf(fp,"%s",str) != EOF) {
		if (strchr("#",str[0])) continue;
		c++;
	}
	if (feof(fp)) fclose(fp);
	return c;
}

int getPropsNotInclude(void)
{
int c;
FILE *fp = NULL;
char str[8];
static char PropsFilename[32];
	
	if (strlen(PropsFilename) == 0) {
		sprintf(PropsFilename,"%s","propsNotIncl.list");
		printf("file to open = %s\n", PropsFilename);
	}
	if ((fp=fopen(PropsFilename,"r")) == NULL) {
		printf("no such file\n");
		return -1;
	}

	c = 0;
	while (fgets(str,7,fp) != NULL) {
		if (strchr("#",str[0])) continue;      /* comment line(s) */
		printf("%d\t%s\n",c,str);
		c++;
	}

	if (feof(fp)) fclose(fp);
	return 1;
}

/****************************************************************
 *								*
 *	TINE Specific Part					*
 *								*
 ****************************************************************/
int readStructureFile(char *filename, NAME32 **cmpPrps, NAME32 **cmpDevPrps, NAME32 **cmpDevs, NAME16 **cmpAliasDevs, NAME32 **cmpDesc, int *numCmpProperties)
{
int cc=0,i = 0,n = 0,siz = 0,done = 0,count = 0;
char str[256],hdr[256];
int prp_col=0,dev_col=0,alias_col=0, desc_col=0;
FILE *fp = NULL;
NAME32 *devprps = NULL;
NAME32 *prps = NULL;
NAME32 *devs = NULL;
NAME16 *aliasdevs = NULL;
NAME32 *desc = NULL;
int openCount;

	do {
		siz=GetDataBaseSize(filename);
		count++;
		printf("reading composite structure file <%d> %s\n",count,filename);
	} while (siz <= 0 && count <= 5 );
	if (siz <=0 && count == 6) ccerr(no_such_file);

	*numCmpProperties = 0;
	openCount = 0;
	while(fp == NULL && openCount <= 3) {
		if ((fp=fopen(filename,"r")) != NULL) {
			if ((devprps=(NAME32 *)calloc(siz,sizeof(NAME32))) == NULL) ccerr(out_of_local_memory);
			if ((prps=(NAME32 *)calloc(siz,sizeof(NAME32))) == NULL) ccerr(out_of_local_memory);
			if ((devs=(NAME32 *)calloc(siz,sizeof(NAME32))) == NULL) ccerr(out_of_local_memory);
			if ((aliasdevs=(NAME16 *)calloc(siz,sizeof(NAME16))) == NULL) ccerr(out_of_local_memory);
			if ((desc=(NAME32 *)calloc(siz,sizeof(NAME32))) == NULL) ccerr(out_of_local_memory);
			for (done=0; fgets(str,255,fp) != NULL;) {
			if (strlen(str) == 0) continue;          /* carriage return */
			if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
			if (!done) {
				strcpy(hdr,str);
				if ((dev_col=findcol(hdr,"DEVICE")) < 0) ccerr(no_such_column);
				if ((prp_col=findcol(hdr,"PROPERTY")) < 0) ccerr(no_such_column);
				if ((alias_col=findcol(hdr,"ALIAS")) < 0) ccerr(no_such_column);
				desc_col=findcol(hdr,"DESCRIPTION");
				done = 1;
				continue;
			}
			strncpy(devprps[i].name,colptr(prp_col,str),32);
			strncpy(devs[i].name,colptr(dev_col,str),32);
			strncpy(aliasdevs[i].name,colptr(alias_col,str),16);
			if(desc_col >= 0) strncpy(desc[i].name,colptr(desc_col,str),32);
			else strcpy(desc[i].name,"");
			for (n=0; n<i && strncmp(devprps[n].name,devprps[i].name,32); n++);
				if (n == i) {
					strncpy(prps[*numCmpProperties].name,devprps[i].name,32);
					*numCmpProperties = *numCmpProperties + 1;
				}
				i++;
			}
			*cmpDevPrps = devprps; *cmpDevs = devs;
			*cmpPrps = prps; *cmpAliasDevs = aliasdevs;
			*cmpDesc = desc;
			return i;
		}
		openCount++;
	}
	err:
	printf("error read struct file\n");
	if (cc) printf("read structure file : %s\n",erlst[cc]);
	return 0;
}

int readCompositeFile(char *cmpFileName)
{
int i,cc = 0,done = 0;
char str[256],hdr[256],scratch[32];
int name_col=0,fmt_col=0,file_col=0,desc_col=0,devs_col=0;
FILE *fp = NULL;
int compCount=0;

	DZK(printf("reading composite file %s\n",cmpFileName);)
	while(ncomposites <= 0 && compCount <= 3) {
		if ((ncomposites=GetDataBaseSize(cmpFileName)) <= 0) ccerr(no_such_file);
		compCount++;
	}

	compCount=0;
	while(fp == NULL && compCount <= 3) {
		if ((fp=fopen(cmpFileName,"r")) != NULL) {
			if ((composites=(COMPOSITE *)calloc(ncomposites,sizeof(COMPOSITE))) == NULL) ccerr(out_of_local_memory);
			for (done=0,i=0; fgets(str,255,fp) != NULL;) {
				if (strlen(str) == 0) continue;          /* carriage return */
				if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
				if (!done) {
					strcpy(hdr,str);
					if ((devs_col=findcol(hdr,"DEVICENAME")) < 0) ccerr(no_such_column);
					if ((name_col=findcol(hdr,"PROPERTYNAME")) < 0) ccerr(no_such_column);
					if ((fmt_col=findcol(hdr,"FORMAT")) < 0) ccerr(no_such_column);
					if ((file_col=findcol(hdr,"STRUCTUREFILE")) < 0) ccerr(no_such_column);
					desc_col=findcol(hdr,"DESCRIPTION");
					done = 1;
					continue;
				}
				strncpy(composites[i].devName,colptr(devs_col,str),32);
				strncpy(composites[i].devProperty,colptr(name_col,str),32);
				strncpy(scratch,colptr(fmt_col,str),32);
				composites[i].prpFormat = ftoi(scratch);
				strncpy(scratch,colptr(file_col,str),32);
				composites[i].prpLength = readStructureFile(scratch, &composites[i].cmpPrps, &composites[i].cmpDevPrps, &composites[i].cmpDevs, &composites[i].cmpAliasDevs, &composites[i].cmpDesc, &composites[i].numCmpPrps);
				if(desc_col >= 0) strncpy(composites[i].prpDesc,colptr(desc_col,str),32);
				else strcpy(composites[i].prpDesc,"");
				i++;
			}
			if (i < ncomposites) ncomposites = i;
			if(debug >= 1) {
				printf("Composites\t\tdevProperty\n");
				for(i=0;i<ncomposites;i++) {
					printf("%s\t\t", (char *)composites[i].cmpDevs);
					printf("%s\n", composites[i].devProperty);
				}
			}

			return 0;
		}
		compCount++;
	}
	cc = no_such_file;
	err:
	printf("ERROR >>>Composite file : %s\n",erlst[cc]);
	return cc;

}

short getTINEformat(int epicsFmt)
{
	switch (epicsFmt) {
  	case DBF_STRING:	return CF_TEXT;
  	case DBF_CHAR:
  	case DBF_UCHAR:		return CF_BYTE; 
  	case DBF_SHORT:	
  	case DBF_USHORT:	return CF_SHORT;
  	case DBF_LONG:
  	case DBF_ULONG:		return CF_LONG;
  	case DBF_FLOAT:		return CF_FLOAT;
  	case DBF_DOUBLE:	return CF_DOUBLE;
  	case DBF_ENUM:		return CF_LONG;
  	case DBF_MENU:		return CF_NAME16;
  	case DBF_DEVICE:	return CF_TEXT;
  	case DBF_INLINK:	return CF_NAME32;
  	case DBF_OUTLINK:	return CF_NAME32;
  	case DBF_FWDLINK:	return CF_NAME32;
  	case DBF_NOACCESS:	return CF_NULL;
  	default: 		return CF_DOUBLE;
  	}
}

int fillDeviceNamesAndProperties(char *fec,char *server,NAME32 **devnames,int *numDevices,Tproperty **prp,int *numProperties)
{
int           	cc=0,i;
Tproperty	*pT = 0;
NAME32		*pD;

  	/* fill in fec and server */
  	/* gethostname(fec, 16); */
  	strcpy(server, fec);

  	*numDevices = tineGetNumRecords();
  	if(debug) printf("numDevices : %d\n",*numDevices);

	if(*numDevices <= 0) return no_such_file;
  	pD=calloc(*numDevices,sizeof(NAME32));
	cc = tineGetRecords(pD);
	if(cc == *numDevices) cc = 0;
	else cc = 13;
  	DZK(for (i=0; i<*numDevices; i++) {
		if(i<10) printf("%s\t", pD[i].name);
	}
	printf("\nTotal of Records: %d\n", *numDevices);)

  	*devnames = pD;
  	if(debug) printf("properties of %s:\n",server);

  	/* find numProperties */
  	/* for the moment static properties */
	*numProperties = tineGetNumProperties(pD[0].name);
  	DZK(if(debug) printf("numProperties : %d\n",*numProperties);)
	
  	if(*numProperties > 0) {
    		DZK(if(debug) printf("*numProperties= %d\tsizeof(Tproperty) = %d\n",*numProperties,(int)sizeof(Tproperty));)

    		if((pT=(Tproperty *)calloc(*numProperties,sizeof(Tproperty))) == NULL) ccerr(out_of_local_memory);
    		DZK(if(debug) printf("pT = %x\n",(int)pT);)
    
		cc = tineGetProperties(pD[0].name, pT);
		if(cc == *numProperties) cc = 0;
		else {
			cc = 13;
			printf("ERROR: wrong number of properties\n");
			return(cc);
		}
    		DZK(for(i = 0; i < *numProperties; i++) {
			printf("%d : %d %s %s %d\n",i,pT[i].elem,pT[i].name,pT[i].desc,pT[i].fmt);
    		})
  	}

  	*prp = pT;

err:
  	return(cc);
}

int gtweqm(char *devName,char *devProperty,DTYPE *dout, DTYPE *din,short access)
{
int nr,len=0,cc=0;
int prpId;
char caName[32];	
int i,status;
int cmpId = -1;
char epicsProperty[32];
char *c,tmpPrpName[32];
void *ptr;
int *pCount = 0;
UINT32 bufsiz;
char strbuf[40];
char *pStrBuf = &strbuf[0];


  DZK(printf("gtweqm params: devName = %s\tdevProperty = %s\n",devName,devProperty);)

  if ((prpId=GetPropertyId("GTWEQM",devProperty)) < 0) return illegal_property;
  nr = GetDeviceNumber("GTWEQM",devName);
  DZK(printf("nr = %d\n",nr);)
  if (prpId >= numProperties) {
    if (prpId > numProperties + ncomposites*3) ccerr(illegal_property);
    cmpId = prpId - numProperties;
  } else strncpy(epicsProperty,devProperty,32);
  DZK(printf("prpId = %d\tcmpId = %d\n",prpId,cmpId);)

  if (access & CA_WRITE) if (din->dArrayLength == 0) return 0;
  if (dout->dFormat == CF_TEXT && dout->dArrayLength < 40) return dimension_error;

  /* zu verbessern */
  strncpy(tmpPrpName,devProperty,16);
  if ((c=strstr(tmpPrpName,".NAM")) != NULL) *c = 0;
  if ((c=strstr(tmpPrpName,".DESC")) != NULL) *c = 0;
  for(i=0;i<ncomposites;i++) {
    if (!strncmp(composites[i].devName,devName,32) &&
        !strncmp(composites[i].devProperty,tmpPrpName,16)) break;
  }

  if (cmpId > -1 && i == ncomposites) return illegal_property;

  if (access & CA_WRITE) {
    if (din->dFormat != CF_TEXT) return illegal_format;
    strncpy(caName, devName,27); caName[27] = '\0';
    strcat(caName,".");
    strcat(caName,devProperty);
    printf("name = %s\tval = %s\n", caName, din->data.cptr);
    bufsiz = sizeof(strbuf);
    cc = getValuesAsString(din,pStrBuf,&bufsiz);
    if(cc) return(cc);
    tinePutFieldValue(caName, pStrBuf);
  }
  /* cmpId = i; */
  DZK(printf("cmpId = %d\tncomposites = %d\n",cmpId,ncomposites);)
  if ((len=dout->dArrayLength) > 0) {
     if (cmpId >= 0 && cmpId < ncomposites) {
        if (dout->dFormat == CF_TEXT) return illegal_format;
        DZK(printf("got cmpId [%d] >> devName = %s\n", cmpId, composites[cmpId].devName);)

/* ADDED P.D. */
          if (strstr(devProperty,".NAM") != NULL)
          {
            for (i=0; i<composites[cmpId].prpLength && i<dout->dArrayLength; i++) {
              memcpy(((NAME32 *)dout->data.vptr)[i].name, composites[cmpId].cmpAliasDevs[i].name, 32);
            }
            return 0;
          }
          if (strstr(devProperty,".DESC") != NULL)
          {
            for (i=0; i<composites[cmpId].prpLength && i<dout->dArrayLength; i++) {
              memcpy(((NAME32 *)dout->data.vptr)[i].name, composites[cmpId].cmpDesc[i].name, 32);
            }
            return 0;
          }

	if (cmpId == ncomposites) return illegal_property;
	DZK(printf("dout->dArrayLength: %d\n", (int)dout->dArrayLength);)

	for (i=0; i<composites[cmpId].prpLength && i<dout->dArrayLength; i++) {
	   strncpy(epicsProperty,composites[cmpId].cmpDevPrps[i].name,32);
           strncpy(caName,composites[cmpId].cmpDevs[i].name,32);
           /* request for channel names */

	   if ((dout->dFormat == CF_NAME32) && (strcmp(composites[cmpId].cmpDevPrps[i].name, "NAME") == 0)) {
	     memcpy(((NAME32 *)dout->data.vptr)[i].name, caName, 32);
	     continue;	
           }

 	   ptr = dout->data.bptr + i*fmtsizeof(dout->dFormat);

	   status = tineGetFieldValue(&caName[0], epicsProperty, ptr, dout->dFormat, pCount);
	   if(status) return 512;
        }
      } else {
	sprintf(caName, "%s", devName);
 	status = tineGetFieldValue(&caName[0], devProperty, dout->data.vptr, dout->dFormat, pCount);
	if(status) return 512;
      }
  }

err:
  if (cc) printf("read structure file : %s\n",erlst[cc]);

  *pCount = 0;
  return 0;
}

int E2T_SystemOnly()
{
int cc;

    if ((cc=SystemInit(TRUE)) != 0) { 
	printf("init error: %s\n",erlst[cc]); 
	exit(1);
    }
    return(0);
}

int E2T()
{
  int status, i,cc;
  int numDevices;
  char ipName[64],tmp[64],devServer[64];
  NAME32 *devnames = NULL;
  Tproperty *prp = NULL;
  ALARM_WATCH_THRESHOLDS *pThr = NULL;
  
  /* fill in fecName */
  gethostname(ipName, 16);

  /* find out if there are any composite properties ... */
  sprintf(tmp,"%s-composites.csv",ipName);
  readCompositeFile(tmp);

  max_rcvbuf = 0;
  max_sndbuf = 0;
  if ((cc=SystemInit(TRUE)) != 0) { printf("init error: %s\n",erlst[cc]); exit(1); }

  /* find out how many and the names of the devices and properties ... */
  status = fillDeviceNamesAndProperties(fecname,devServer,&devnames,&numDevices,&prp,&numProperties);
  if(status) {
	printf("fillDeviceNamesAndProperties : <%d> %s\n",status,erlst[status]);
	return(-1);
  }

  /* register server name dynamically ... */
  /* RegisterFecName(fecName,"EPICS To TINE Translator","VXW","Bldg 52","??","P.Duval Z.Kakucs",0); */

  /* initialize the TINE server ... */
  longDeviceNames = TRUE;
  /* register the equipment module ... */
  if(ncomposites <= 0) {
     printf("No composites\n");
     ncomposites = 0;
  }
  printf("%s %s %d devices & %d composites\n",fecname,devServer,numDevices,ncomposites);
  RegisterEquipmentModule(devServer,"GTWEQM",numDevices+ncomposites,gtweqm,NULL,NULL,0,NULL);

  /* register all known properties ... */
  for (i=0; i<numProperties; i++) RegisterPropertyEx("GTWEQM",prp[i].name,prp[i].elem,prp[i].fmt,CA_READ,prp[i].desc,i);

  DZK(printf("composites summe: %d\n", ncomposites);)
  for (i=0; i<ncomposites; i++) {
    RegisterPropertyEx("GTWEQM",composites[i].devProperty,composites[i].prpLength, composites[i].prpFormat,CA_READ,composites[i].prpDesc,numProperties+i);
    sprintf(tmp,"%s.NAM",composites[i].devProperty);
    RegisterPropertyEx("GTWEQM",tmp,composites[i].prpLength, CF_NAME32, CA_READ,composites[i].prpDesc,numProperties+ncomposites+i);
    sprintf(tmp,"%s.DESC",composites[i].devProperty);
    RegisterPropertyEx("GTWEQM",tmp,composites[i].prpLength, CF_NAME32, CA_READ,composites[i].prpDesc,numProperties+2*ncomposites+i);

  }

  /* register all known devices ... */
  for (i=0; i<numDevices; i++) {
    if(strlen(devnames[i].name) < 5) printf("dev null: for <%d> <%s>\n", i,devnames[i].name);
    RegisterDeviceName("GTWEQM",devnames[i].name,i);
/*    if(tineGetAlarmLimits(devnames[i].name, pThr)) appendAlarmWatchTable("GTWEQM", "VAL", devnames[i].name, 1, CF_DOUBLE, 12, pThr); */
  /* appendAlarmWatchTable(char *eqm,char *prp,char *dev,int siz,int fmt,int sev,ALARM_WATCH_THRESHOLDS *thr) */
  }

  for (i=0; i<ncomposites; i++) {
	DZK(printf("composites[%d].devName = %s\n", i, composites[i].devName);)
	RegisterDeviceName("GTWEQM",composites[i].devName,i+numDevices);
  }

  /* run the server ...   */
  for(;;) SystemCycle(TRUE);

  return 0;
}

void loadE2TModules()
{
STATUS status;
MODULE_ID modId;
char pathName[MAX_FILENAME_LENGTH];
char *pPathName = &pathName[0];
char pathFrom[MAX_FILENAME_LENGTH];
/* char *pPathFrom = &pathFrom[0]; */
char archName[8];
char *pArchName = &archName[0];

  ioDefPathGet(pPathName);
  printf("default Path = %s\n", pathName);
  printf("Type-in the architecture (e.g.: mv162 mv167 mv177 pc486 ...):\n");
  gets(pArchName);
  sprintf(pathFrom, "%s/EPICS/%s", pathName, pArchName);
  printf("path = %s\n", pathFrom);

  status = chdir(pathFrom);
  if(status != 0) return;

  modId = loadModule((int)"rpcLib", LOAD_ALL_SYMBOLS);
  if((int)modId == 0) printf("can't load rpcLib !!!\n");
  else printf("rpcLib module id: %x\n", (int)modId);

  modId = loadModule((int)"epics2tine.o", LOAD_ALL_SYMBOLS);
  if((int)modId == 0) printf("can't load epics2tine.o !!!\n");
  else printf("epics2tine.o module id: %x\n", (int)modId);
}

void removeE2TModules()
{
MODULE_ID modToDel;

  do {
    modToDel = moduleFindByName("rpcLib");
    if(modToDel != 0) {
      printf("delete module modToDel = %x\n", (int)modToDel);
      moduleDelete(modToDel);
    }
  } while(modToDel != 0);

  do {
    modToDel = moduleFindByName("epics2tine.o");
    if(modToDel != 0) {
      printf("delete module modToDel = %x\n", (int)modToDel);
      moduleDelete(modToDel);
    }
  } while(modToDel != 0);

  do {
    modToDel = moduleFindByName("e2tLib");
    if(modToDel != 0) {
      printf("delete module modToDel = %x\n", (int)modToDel);
      moduleDelete(modToDel);
    }
  } while(modToDel != 0);

  printf("All modules for TINE are deleted\n");
}

void startE2T_SystemOnly()
{
int tid;
MODULE_ID modToDel;

  modToDel = moduleFindByName("rpcLib");
  if(modToDel != 0) {
    modToDel = moduleFindByName("epics2tine.o");
    if(modToDel != 0) {
      tid = taskNameToId("E2T_SystemOnly");
      if(tid == ERROR) taskSpawn("E2T_SystemOnly", 100, VX_FP_TASK, 10000, (FUNCPTR)E2T_SystemOnly,0,0,0,0,0,0,0,0,0,0);
      else printf("Task tid = %d is running, stop or restart possible only !!!\n", tid);
    } else printf("Module <epics2tine.o> failed, load it !!!\n");
  } else printf("Module <rpcLib> failed, load it !!!\n");
}

void startE2T()
{
int tid;
MODULE_ID modToDel;
MODULE_ID modE2TObj;
MODULE_ID modE2TLib;

  modToDel = moduleFindByName("rpcLib");
  if(modToDel != 0) {
    modE2TObj = moduleFindByName("epics2tine.o");
    modE2TLib = moduleFindByName("e2tLib");
    if(modE2TObj != 0 || modE2TLib != 0) {
      tid = taskNameToId("E2T");
      if(tid == ERROR) taskSpawn("E2T", 190, VX_FP_TASK, 10000, (FUNCPTR)E2T,0,0,0,0,0,0,0,0,0,0);
      else printf("Task tid = %d is running, stop or restart possible only !!!\n", tid);
    } else printf("Module <epics2tine.o> or <e2tLib> failed, load it !!!\n");
  } else printf("Module <rpcLib> failed, load it !!!\n");
}

void stopE2T(int art)
{
int tid, tidHS;
char *pString = "";
int status;
int delTask = FALSE;

  tid = taskNameToId("E2T");
  printf("task E2T has tid = %x\n", tid);

  status = taskStatusString(tid, pString);
  if(status == 0) {
    printf("task E2T %d has status = %s\n", status, pString);
    if((strcmp(pString, "SUSPEND") != 0) ||
       (strcmp(pString, "DELAY+S") != 0) ||
       (strcmp(pString, "PEND+S") != 0) ||
       (strcmp(pString, "PEND+S+T") != 0) ||
       (strcmp(pString, "PEND+S+T+I") != 0) ||
       (strcmp(pString, "DEAD") != 0)) delTask = TRUE;

    if(delTask && art) {
      printf("Close all Sockets, please wait ...\n");
      taskSpawn("haltE2T", 101, VX_FP_TASK, 1000, (FUNCPTR)haltServer,0,0,0,0,0,0,0,0,0,0);
      taskDelay(300);
      tidHS = taskNameToId("haltE2T");
      taskSuspend(tidHS);
      taskDelete(tidHS);
      taskDelete(tid);
      removeE2TModules();
    } else printf("keeping task !!! \n");
  }
}

int restartE2T()
{
int tid, tidHS, i;
char *pString = "";
int status;
char customInp[8];
char *pCustomInp = &customInp[0];

  printf("**************************\n");
  printf("Are you sure ? yes/no\n");
  printf("**************************\n\n");
  gets(pCustomInp);
  printf("returned = %s\n", customInp);
  if(strcmp(customInp, "yes") != 0) {
    printf("Cancelled...\n");
    return(-1);
  }

  for(i=0;i<ncomposites;i++) {
    free(composites[i].cmpPrps);
    free(composites[i].cmpDevPrps);
    free(composites[i].cmpDevs);
    free(composites[i].cmpAliasDevs);
    free(composites[i].cmpDesc);
  }
  free(composites);
  ncomposites = 0;

  tid = taskNameToId("E2T");
  printf("task E2T has tid = %x\n", tid);

  status = taskStatusString(tid, pString);
  if(status == 0) {
    printf("task E2T %d has status = %s\n", status, pString);
    if(strcmp(pString, "SUSPEND") != 0) {
      taskSuspend(tid);
      printf("task suspended\n");
    } 
    printf("Close all Sockets, please wait ...\n");
    taskSpawn("haltE2T", 101, VX_FP_TASK, 1000, (FUNCPTR)haltServer,0,0,0,0,0,0,0,0,0,0);
    taskDelay(600);
    tidHS = taskNameToId("haltE2T");
    taskSuspend(tidHS);
    taskDelete(tidHS);
    taskDelete(tid);
    taskDelay(300);
    printf("******************************************************\n");
    printf("Reload objects before starting the server !!!\n");
    printf("******************************************************\n\n");
    printf("returned = %s\n", customInp);
    /* gets(pCustomInp);
    if(strcmp(customInp, "yes") == 0) { */
      removeE2TModules();
      /* load first the modules */
      printf("Now load the startup file startup.tine, then startE2T.\n");
      /* loadE2TModules(); */
    /* }  */
  }
  return(0);
}
