/*
   This following code demonstrates the EVENT Server - ARCHIVE - Server
   communication API.  This is only an example.  Feel free to modify
   anything you see.  This is also a c-wrapper for the CPP API interface.
   Thus you can convert this to legitimate CPP code if you are so
   inclined.
*/
#include "cprolog.h"
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <string.h>
#include "errors.h"
#include "rpctypes.h"
#include "servdef.h"
#include "arceqm.h"
#include "arctypes.h"

// This is an example archive server routine to get local archive data
// according to input arguments.  This needs to registered in "setGetArchiveDataFunction()"

float MyFakeData[1024];
#define srcptr(fmt) (\
  fmt == CF_SHORT ? (BYTE *)&sdat :\
  fmt == CF_LONG  ? (BYTE *)&ldat :\
  fmt == CF_FLOAT ? (BYTE *)&fdat :\
  fmt == CF_DOUBLE ? (BYTE *)&ddat: NULL\
)
#define srcval(fmt,i) \
  switch (fmt) {\
    case CF_SHORT: sdat = (short)MyFakeData[i%1024]; break;\
    case CF_LONG:  ldat = (UINT32)MyFakeData[i%1024]; break;\
    case CF_FLOAT: fdat = (float)MyFakeData[i%1024]; break;\
    case CF_DOUBLE: ddat = (double)MyFakeData[i%1024]; break;\
  }
int GetLocalArchiveData(BYTE *data,ulong eventNumber,time_t starttime,time_t stoptime,DEVSPEC *devSpecifier)
{
  int i,dsize,fmtsize;
  static int cnt = 0;
  BYTE *sptr;
  float fdat;
  short sdat;
  UINT32 ldat;
  double ddat;

  printf("I've got to get the data for:\n\t event number %d\n",eventNumber);
  printf("\t startime %d stoptime %d\n",starttime,stoptime);
  printf("\t device server %s Property %s device %s\n",
         devSpecifier->deviceServerName,devSpecifier->deviceProperty,devSpecifier->deviceName);
  printf("\t requested data format: %d\n",devSpecifier->dataFormat);
  printf("\t available client buffer space %d\n",devSpecifier->dataDimension);

  fmtsize = getFormatSize(devSpecifier->dataFormat);
  dsize = devSpecifier->dataDimension*fmtsize;
  sptr = srcptr(devSpecifier->dataFormat);

  // put the data into memory area pointed to by data
  // data is guaranteed to point to an area at least dsize bytes big

  // I hope it is clear that this is just an example
  // You should catch your own device server names
  // and your own device properties
  // and your own device names here ...

  if (!strcmp(devSpecifier->deviceServerName,"SERVER1"))
  {
    // also find the data that correspond to devSpecifier->deviceProperty
    // and devSpecifier->deviceName ...
    for (i=0; i<devSpecifier->dataDimension && i<1024; i++)
    {
      srcval(devSpecifier->dataFormat,i+cnt);
      memcpy(&data[i*fmtsize],sptr,fmtsize);
    }
    cnt++;
    return 0;
  }
  // etc ...

  return non_existent_elem;
}

// This is an example trigger callback function (needs to be registered)

int respondToTrigger(ulong eventNumber,ulong eventCode)
{
  int cc = 0;
  printf("I need to see if I want to react to event %d with code %d\n",
          eventNumber,eventCode);

  // call RecordEvent for the Device servers that have something to
  // record for this event.
  // These are "collected" and sent out at the same time ...

  if ((cc=RecordEvent(eventNumber,eventCode,"DEVICESERVER1")) != 0) return cc;
  if ((cc=RecordEvent(eventNumber,eventCode,"DEVICESERVER2")) != 0) return cc;

  // ...

  return 0;
}

// Example SetEvent callback function (needs to be registered : see doinit())
void checkSetEvent(ulong eventnumber, int cc)
{
  printf("SetEvent %d : %s\n>",eventnumber,cc ? erlst[cc] : "Success");
}

// Example RecordEvent callback function

void checkRecordEvent(int cc)
{
  printf("RecordEvent: %s\n>",cc ? erlst[cc] : "Success");
}

int GetLocalArchiveInfo(AIB *aib,ulong eventNumber,char *deviceServer)
{
  return 0;
}
int GetLocalPropertyInfo(PIB *pib,ulong eventNumber,char *deviceServer,char *devProperty)
{
  return 0;
}
int GetNumLocalArchivedEvents(ulong starttime,ulong stoptime,ulong eventNumber)
{
  return 0;
}
int GetLocalArchivedEvents(ulong starttime,ulong stoptime,AES *aes,ulong num)
{
  return 0;
}
int myLocalnumDeviceServers(void)
{
 return 300;
}
int myLocalDeviceServers(NAME16 *names,unsigned long start,unsigned long stop,int id,int num)
{
  // react to id = device Nr.
  
  printf("start time : %d, stop time %d\n",start,stop);
  strcpy(names[0].name,"DEVICESERVER1");
  num = 1;
  return 0;
}
int myLocalnumDevices(void)
{
 return 3000;
}
int myLocalDevices(NAME16 *names,char *devSrvName,int id,int num)
{
  // react to id = device Nr.
  
  printf("dev Server Name: %s\n",devSrvName);
  strcpy(names[0].name,"DEVICE1");
  num = 1;
  return 0;
}
int myArchivedEvents(char *krykDevice,AES *aes,ulong num)
{
  printf("dev Name: %s\n",krykDevice);
  aes[0].eventNumber = 365374624;
  aes[1].eventNumber = 8746331;
  num = 2;

  return 0;
}

// this init routine does the callback function registration
// and calls initArchiveServer()

void doinit(void)
{
  int i;
  initArchiveServer();
  setTriggerCallback(respondToTrigger);
  setRecordEventCallback(checkRecordEvent);
  setSetEventCallback(checkSetEvent);
  setGetArchiveDataFunction(GetLocalArchiveData);
  setQueryArchiveInfoFunction(GetLocalArchiveInfo);
  setQueryPropertyInfoFunction(GetLocalPropertyInfo);
  setQueryNumLocalArchivedEventsFunction(GetNumLocalArchivedEvents);
  setQueryLocalArchivedEventsFunction(GetLocalArchivedEvents);
  setQueryNumLocalDeviceServersFunction(myLocalnumDeviceServers);
  setQueryLocalDeviceServerNamesFunction(myLocalDeviceServers);
  setQueryNumLocalDevicesFunction(myLocalnumDevices);
  setQueryLocalDeviceNamesFunction(myLocalDevices);
  setQueryDeviceArchivedEventsFunction(myArchivedEvents);
  printf("Archive Server running ...\n>");

  // fill in a data buffer to use in this example:
  for (i=0; i<1024; i++) MyFakeData[i] = i;

}

// This is just a way of exercisizing the various test routines
// many of the calls below will be made from a console application
// and not an archive server.  The archive server will make use of
// SetGlobalEvent

extern int CommandLevel;
char *commandText[] = {
  "No command","SetGlobalEvent","GetNumberOfEvents",
  "GetEvents","GetNumberOfArchiveServers","GetArchiveServers",
  "GetNumberOfDeviceServers","GetDeviceServers", "GetRemoteArchiveData" };

int numCommandLevels = sizeof(commandText)/sizeof(commandText[0]);

#define ASRVBUFFERSIZE 10
#define DSRVBUFFERSIZE 50
#define AESBUFFERSIZE 20
#define rpcerror(i) {\
  printf("%s : %s\n>",commandText[CommandLevel],erlst[i]);\
}
#define commandFinished {\
  for (i=0; i < numCommandLevels; i++)\
    printf("Set CommandLevel = %d: %s\n",i,commandText[i]);\
  CommandLevel = 0;  break;\
}

// This is a sample routine which launches an empty request for
// data from archive server MARCHSRV.  It's just to give you an
// idea

int queryRemoteData(void)
{
  float data[100];
  DEVSPEC devspec;
  strcpy(devspec.deviceServerName,"");
  strcpy(devspec.deviceProperty,"");
  strcpy(devspec.deviceName,"");
  devspec.dataDimension = 100;
  devspec.dataFormat = CF_FLOAT;
  return GetRemoteArchiveData("MARCHSRV",(BYTE *)data,0,0,0,&devspec);
}
void queryCommandLevel(void)
{
  time_t interim=0,t;
  int i,cc,num;
  AES aes[AESBUFFERSIZE];
  NAME16 srv[ASRVBUFFERSIZE],dev[DSRVBUFFERSIZE];

  switch (CommandLevel)
  {
    case 0: break;
    case 1: if ((cc=SetGlobalEvent(1234,"This is a test")) != 0) rpcerror(cc);
            commandFinished;
    case 2: if ((cc=GetNumberOfEvents(0,time(NULL))) >= 0)
              printf("There are %d recorded events\n>",cc);
            else  rpcerror(-cc);
            commandFinished;
    case 3: num = AESBUFFERSIZE;
            if ((cc=GetEvents(0,time(NULL),0,aes,&num)) == 0)
            {
              if (num > AESBUFFERSIZE)
              {
                printf("More events found in the specified time range ");
                printf("than can be buffered.\nIncrease buffer size or ");
                printf("partition the time range !\n");
              }
              printf("\n Found %d events\n",num);
              for (i=0; i<AESBUFFERSIZE && i<num; i++)
                printf(" %d %d %s\n",
                       aes[i].eventNumber,
                       aes[i].eventCode,
                       aes[i].comment[0]);
            }
            else rpcerror(cc);
            commandFinished;
    case 4: if ((cc=GetNumberOfArchiveServers()) >= 0)
              printf("There are %d Archive Servers\n>",cc);
            else rpcerror(-cc);
            commandFinished;
    case 5: num = ASRVBUFFERSIZE;
            if ((cc=GetArchiveServers(-1,srv[0].name,&num)) == 0)
            {
              printf("archive servers:\n");
              for (i=0; i<num; i++)
                if (strlen(srv[i].name))printf("%s\n",srv[i].name);
            }
            else rpcerror(cc);
            commandFinished;
    case 6: if ((cc=GetNumberOfDeviceServers()) >= 0)
              printf("There are %d Device Servers\n>",cc);
            else rpcerror(-cc);
            commandFinished;
    case 7: num = DSRVBUFFERSIZE;
            if ((cc=GetDeviceServers(-1,dev[0].name,&num)) == 0)
            {
              printf("device servers:\n");
              for (i=0; i<num; i++)
                if (strlen(dev[i].name)) printf("%s\n",dev[i].name);
            }
            else rpcerror(cc);
            commandFinished;
    case 8: queryRemoteData();
            commandFinished;
   default:
            commandFinished;
  }
  return;
}
int main(int argc, char *argv[])
{
  int cc;

  switch (argc)                           /* initial debug level */
  {                  /* on UNIX stations: set debug = 1 (2,or 3) */
    case 2:              /* will dynamically set the debug level */
      NGdebug = atoi(argv[1]);
      break;
    default:
      NGdebug = 0;                              /* default level */
  }
  if ((cc=SystemInit(FALSE)) != 0) { printf("init error: %s\n",erlst[cc]); exit(1); }
  printf("Server running (debug level %d)\n",NGdebug);

  doinit();

  for(;;)
  {
    while (SystemCycle(FALSE));
    if (CommandLevel) queryCommandLevel();
  }
  return 0;
}
