#include "prolog.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "servdef.h"
#include "database.h"
#include "errors.h"

/* 

  VB Server Wizard :

  DONE ->> EGU info -> range checks for CA_WRITE
  DONE ->> if !CA_WRITE -> random simulation data in background
  DONE ->> alarms in background ?
  does device file exist ? -> read device information
  links.csv -> start any client links, globals (HW links ?)
  DONE ->> optional RegisterPropertyEx() routine
  DONE ->> fecid.csv -> optional RegisterFecEx() routine

  if "." shows up in property name -> replace with "_" in variables !!!!!
*/
FILE *dbgfp = NULL;
long *tnull = 0;
char dbgbuf[256];
int NGdebug = 0;
unsigned long PASCAL FAR inet_addr (const char FAR * cp) { return 0; }
int putProcessFunction(FILE *fp,char *procname, int nsteps);

int feclog(char *txt) { return 0; }
char ControlDBpath[] = "";
char tagNameFilter[64];
char *gettype(int fmt, char *fstr);
#define EXPORT_FILE_BAK "exports.bak"
#define MAKEFILE_NAME "fec.mak"


typedef struct ExportPropertyListEx
{
  char  prpName[32];       /* property name */
  long  prpId;             /* property id */
  UINT32 prpSize;          /* maximum supported array size */
  UINT32 prpSizeIn;        /* maximum supported input array size */
  short prpFormat;         /* native format */
  short prpFormatIn;       /* native input format */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  char  prpDescription[32];/* short description of property */
  PropertyEGUStruct egu;   /* EGU information for display */
  short numProcessSteps; 
  struct ExportPropertyListEx *next;
} ExportPropertyListStructEx;

char targetPath[64];

short ftoi(char *fmt)
{
  strupr(fmt);
  return getFormatType(fmt);
}
short mtoi(char *mod)
{
  short accessMode = CA_NULL;

  if (strstr(strupr(mod),"READ")) accessMode |= CA_READ;
  if (strstr(mod,"WRITE"))        accessMode |= CA_WRITE;
  if (strstr(mod,"ASYNC"))        accessMode |= CA_ASYNC;

  return accessMode;
}
char *strudr(char *s)
{
  char *c;
  for (c=s; c && *c; c++) if (*c == '.' || *c == '-') *c = '_';
  return s;
}

void ParsePropertyExDescription(char *str,ExportPropertyListStructEx *prp)
{
  char *c;

  if (str == NULL) return;
  if ((c=strchr(str,'[')) != NULL)
  {
    if ((c=strtok(++c,"]")) != NULL)               /* 1st token: egu */
    {
      str = strtok(NULL,"]");         /* 2nd token: description part */
      if (strchr(c,':'))
      {
        prp->egu.min = strtod(c,&c);
        prp->egu.max = strtod(++c,&c);
        strncpy(prp->egu.units,++c,16);
        prp->egu.graph = 0;
      }
    }
  }
  if (str != NULL) strncpy(prp->prpDescription,str,32);
}

typedef struct EQPstruct
{
  char tagname[16];
  char lclname[8];
  int numdevs;
  int numprps;
  ExportPropertyListStructEx *prp;
  struct EQPstruct *next;
} EQP;

EQP *eqplist = NULL;
int putExportInformation(void)
{
  int cc = 0;
  FILE *fp=NULL;
  ExportPropertyListStructEx *prp;
  EQP *el;
  char desc[128],mode[128];
  char type[32],cftype[32];
  char fullFileName[128],fullBackupName[128];

  if (strlen(targetPath) > 0) 
  {
	 sprintf(fullFileName,"%s\\%s",targetPath,EXPORT_FILE);	 
	 sprintf(fullBackupName,"%s\\%s",targetPath,EXPORT_FILE_BAK);	 
  }
  else
  {
	 sprintf(fullFileName,"%s",EXPORT_FILE);	 
	 sprintf(fullBackupName,"%s",EXPORT_FILE_BAK);	 
  }
  cc = rename(fullFileName,fullBackupName);  /* back it up ! */
  if (cc != 0)
  {
	perror("rename");
	cc = 0;
  }

  if ((fp=fopen(fullFileName,"w")) == NULL) ccerr(no_such_file);
  fprintf(fp,"EXPORT_NAME,LOCAL_NAME,NUM_MODULES,PROPERTY,PROPERTY_SIZE,PROPERTY_ID,FORMAT,ACCESS,DESCRIPTION\n");
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    for (prp=el->prp; prp != NULL; prp = prp->next)
	{
      strcpy(mode,"CA_READ");
	  if (prp->prpAccessMode&CA_WRITE) strcat(mode,"|CA_WRITE");
   	  strcpy(type,gettype(prp->prpFormat,cftype));
	  if (prp->prpFormat == CF_STRUCT) strcpy(type,"struct");
	  fprintf(fp,"%s,%s,%d,",el->tagname,el->lclname,el->numdevs);
      fprintf(fp,"%s,%d,%d,%s,%s,",prp->prpName,prp->prpSize,prp->prpId,type,mode);
	  if (prp->egu.min == 0 && prp->egu.max == 0 && strlen(prp->egu.units) == 0)
	    sprintf(desc,"%s",prp->prpDescription);
	  else
	    sprintf(desc,"[%g:%g %s]%s",prp->egu.min,prp->egu.max,prp->egu.units,prp->prpDescription);
      fprintf(fp,"%s\n",desc);
	}
  }
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
char fecname[17] = "";
char gDeviceContext[17] = "TEST";
RPCFecExStruct RPCFecEx;
BYTE PortOffset = 0;

void getBestGuessFecInfo(void)
{
  char *c,tmp[64];
  if (((c=getenv("HOSTNAME")) == NULL) &&
      ((c=getenv("HOST")) == NULL) &&
      ((c=getenv("COMPUTERNAME")) == NULL))
  {
	  c = "TESTSERVER";
  }
  strncpy(tmp,c,12); tmp[12] = 0;
  sprintf(&tmp[strlen(tmp)],".%d",PortOffset);
  strncpy(fecname,tmp,16);
  if (((c=getenv("USER")) == NULL) &&
      ((c=getenv("USERNAME")) == NULL) &&
      ((c=getenv("LOGINNAME")) == NULL))
  {
	  c = "ME";
  }
  strcpy(RPCFecEx.os,OS_TAG);
  sprintf(RPCFecEx.desc,"%s's Test FEC",c);
  strcpy(RPCFecEx.hdw,"none");
  sprintf(RPCFecEx.loc,"%s's office",c);
  strncpy(RPCFecEx.resp,c,16);
}
int getFecInformation(void)
{
  FILE *fp = NULL;
  int cc=0,done=0;
  int name_col,desc_col,loc_col,hdw_col,resp_col,poff_col,con_col;
  char str[256],hdr[256],fecfilename[128],scratch[16];

  if (strlen(targetPath) > 0) 
  {
	 sprintf(fecfilename,"%s\\%s",targetPath,FECID_FILE);	 
  }
  else
  {
	 sprintf(fecfilename,"%s",FECID_FILE);	 
  }
  sprintf(fecfilename,"%s",FECID_FILE);
  if ((fp=fopen(fecfilename,"r")) == NULL) ccerr(no_such_file);
  while (fgets(str,255,fp) != NULL)
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((name_col=findcol(hdr,"FEC_NAME")) < 0) ccerr(no_such_column);
      poff_col = findcol(hdr,"PORT_OFFSET");
      con_col = findcol(hdr,"CONTEXT");
      if ((desc_col=findcol(hdr,"DESCRIPTION")) < 0) ccerr(no_such_column);
      if ((loc_col=findcol(hdr,"LOCATION")) < 0) ccerr(no_such_column);
      if ((hdw_col=findcol(hdr,"HARDWARE")) < 0) ccerr(no_such_column);
      if ((resp_col=findcol(hdr,"RESPONSIBLE")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(fecname,colptr(name_col,str),16);
    strncpy(RPCFecEx.os,OS_TAG,12);
    if (poff_col >= 0)
    {
      strncpy(scratch,colptr(poff_col,str),16);
      PortOffset = atoi(scratch);
    }
    if (con_col >= 0)
    {
      strncpy(gDeviceContext,colptr(con_col,str),16);
      strupr(gDeviceContext);
    }
    strncpy(RPCFecEx.desc,colptr(desc_col,str),64);
    strncpy(RPCFecEx.loc,colptr(loc_col,str),32);
    strncpy(RPCFecEx.hdw,colptr(hdw_col,str),48);
    strncpy(RPCFecEx.resp,colptr(resp_col,str),32);
    break;
  }
err:
  if (fp) fclose(fp);
  if (cc)
  {
	getBestGuessFecInfo();
  }
  return cc;
}

int dumpExportsFiles = 0;
int getExportInformation(void)
{
  FILE *fp;
  int cc=0,done=0;
  int export_name_col,local_name_col,property_col,property_size_col,
      num_modules_col,format_col,access_col,description_col,
      property_id_col,insize_col,informat_col,nsteps_col;
  char str[256],hdr[256],scratch[16],fullFileName[128];
  ExportPropertyListStructEx *prp;
  EQP *el;
  long prpId;

  if (strlen(targetPath) > 0) 
  {
	 sprintf(fullFileName,"%s\\%s",targetPath,EXPORT_FILE);	 
  }
  else
  {
	 sprintf(fullFileName,"%s",EXPORT_FILE);	 
  }
  if ((fp=fopen(fullFileName,"r")) == NULL) ccerr(no_such_file);
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;    /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((export_name_col=findcol(hdr,"EXPORT_NAME")) < 0) ccerr(no_such_column);
      if ((local_name_col=findcol(hdr,"LOCAL_NAME")) < 0) ccerr(no_such_column);
      if ((num_modules_col=findcol(hdr,"NUM_MODULES")) < 0) ccerr(no_such_column);
      if ((property_col=findcol(hdr,"PROPERTY")) < 0) ccerr(no_such_column);
      if ((property_size_col=findcol(hdr,"PROPERTY_SIZE")) < 0) ccerr(no_such_column);
      if ((property_id_col=findcol(hdr,"PROPERTY_ID")) < 0) prpId = -1;
      if ((format_col=findcol(hdr,"FORMAT")) < 0) ccerr(no_such_column);
      if ((access_col=findcol(hdr,"ACCESS")) < 0) ccerr(no_such_column);
      if ((description_col=findcol(hdr,"DESCRIPTION")) < 0) ccerr(no_such_column);
      if ((informat_col=findcol(hdr,"INFORMAT")) < 0) informat_col = -1;
      if ((insize_col=findcol(hdr,"PROPERTY_INSIZE")) < 0) insize_col = -1;
      if ((nsteps_col=findcol(hdr,"NUM_STEPS")) < 0) nsteps_col = -1;
      done = 1;                              /* read header info */
      continue;
    }
	strncpy(scratch,colptr(export_name_col,str),16);
    for (el=eqplist; el != NULL; el=el->next) if (!strncmp(scratch,el->tagname,16)) break;
    if (el == NULL)               /* new eqp type */
    {
      el = (EQP *)calloc(1,sizeof(EQP));
      if (el == NULL) ccerr(out_of_server_memory);
      strncpy(el->tagname,scratch,16);
      strncpy(el->lclname,colptr(local_name_col,str),8);
      strncpy(scratch,colptr(num_modules_col,str),16);
      el->numdevs = atoi(scratch);
      if (el->numdevs == 0) el->numdevs = 1;
      el->numprps = 0;
      el->next = eqplist;
      eqplist = el;
    }	
    if ((prp=(ExportPropertyListStructEx *)calloc(1,sizeof(ExportPropertyListStructEx))) == NULL) ccerr(out_of_server_memory);
    strncpy(prp->prpName,colptr(property_col,str),32);
    strncpy(scratch,colptr(property_size_col,str),16);
    prp->prpSize = atoi(scratch);
	if (insize_col != -1)
	{
      strncpy(scratch,colptr(insize_col,str),16);
      prp->prpSizeIn = atoi(scratch);
	}
	else prp->prpSizeIn = prp->prpSize;
    strncpy(scratch,colptr(format_col,str),16);
    prp->prpFormat = ftoi(scratch);
	if (prp->prpFormat == CF_NULL) prp->prpSize = 0;
	if (informat_col != -1)
	{
      strncpy(scratch,colptr(informat_col,str),16);
      prp->prpFormatIn = ftoi(scratch);
	}
	else prp->prpFormatIn = prp->prpFormat;
	if (prp->prpFormatIn == CF_NULL) prp->prpSizeIn = 0;
    strncpy(scratch,colptr(access_col,str),16);
    prp->prpAccessMode = mtoi(scratch);
    ParsePropertyExDescription(colptr(description_col,str),prp);
    if (property_id_col >= 0)
    {
      strncpy(scratch,colptr(property_id_col,str),16);
      prp->prpId = atoi(scratch);
    }
    else 
	{
	  prp->prpId = el->numprps + 1;
	  dumpExportsFiles = -1;
	}
    if (nsteps_col >= 0)
    {
      strncpy(scratch,colptr(nsteps_col,str),16);
      prp->numProcessSteps = atoi(scratch);
    }
    else 
	{
	  prp->numProcessSteps = 0;
	}
	prp->next = el->prp;
	el->prp = prp;
    el->numprps++;
  }
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
char *gettype(int fmt, char *fstr)
{
  static char str[32], fmtstr[32];
  switch(fmt)
  {
    default:
	case CF_NULL: strcpy(str,"Byte"); strcpy(fmtstr,"CF_NULL"); break;
	case CF_STRUCT: strcpy(str,"Byte"); strcpy(fmtstr,"CF_STRUCT"); break;
    case CF_BYTE: strcpy(str,"Byte"); strcpy(fmtstr,"CF_BYTE"); break;
	case CF_SHORT: strcpy(str,"Integer"); strcpy(fmtstr,"CF_SHORT"); break;
	case CF_LONG: strcpy(str,"Long"); strcpy(fmtstr,"CF_LONG"); break;
	case CF_FLOAT: strcpy(str,"Single"); strcpy(fmtstr,"CF_FLOAT"); break;
	case CF_DOUBLE: strcpy(str,"Double"); strcpy(fmtstr,"CF_DOUBLE"); break;
	case CF_TEXT: strcpy(str,"String"); strcpy(fmtstr,"CF_TEXT"); break;
	case CF_NAME8: strcpy(str,"NAME8"); strcpy(fmtstr,"CF_NAME8"); break;
	case CF_NAME8I: strcpy(str,"NAME8I"); strcpy(fmtstr,"CF_NAME8I"); break;
	case CF_NAME16: strcpy(str,"NAME16"); strcpy(fmtstr,"CF_NAME16"); break;
	case CF_NAME16I: strcpy(str,"NAME16I"); strcpy(fmtstr,"CF_NAME16I"); break;
	case CF_NAME32: strcpy(str,"NAME32"); strcpy(fmtstr,"CF_NAME32"); break;
	case CF_NAME32I: strcpy(str,"NAME32I"); strcpy(fmtstr,"CF_NAME32I"); break;
	case CF_NAME48: strcpy(str,"NAME48"); strcpy(fmtstr,"CF_NAME48"); break;
	case CF_NAME48I: strcpy(str,"NAME48I"); strcpy(fmtstr,"CF_NAME48I"); break;
	case CF_NAME16FI: strcpy(str,"NAME16FI"); strcpy(fmtstr,"CF_NAME16FI"); break;
	case CF_FLTINT: strcpy(str,"FLTINT"); strcpy(fmtstr,"CF_FLTINT"); break;
	case CF_LNGINT: strcpy(str,"INTINT"); strcpy(fmtstr,"CF_LNGINT"); break;
	case CF_DBLINT: strcpy(str,"DBLINT"); strcpy(fmtstr,"CF_DBLINT"); break;
	case CF_SHTINT: strcpy(str,"SHTINT"); strcpy(fmtstr,"CF_SHTINT"); break;
	case CF_TDS: strcpy(str,"INTFLTINT"); strcpy(fmtstr,"CF_TDS"); break;
	case CF_USTRING: strcpy(str,"DUSTRING"); strcpy(fmtstr,"CF_USTRING"); break;
	case CF_XYS: strcpy(str,"FLTFLTINT"); strcpy(fmtstr,"CF_XYS"); break;
	case CF_FLTINTINT: strcpy(str,"FLTINTINT"); strcpy(fmtstr,"CF_FLTINTINT"); break;
	case CF_FILTER: strcpy(str,"DFILTER"); strcpy(fmtstr,"CF_FILTER"); break;
	case CF_SPECTRUM: strcpy(str,"DSPECTRUM"); strcpy(fmtstr,"CF_SPECTRUM"); break;
	case CF_ADDRESS: strcpy(str,"DADDRESS"); strcpy(fmtstr,"CF_ADDRESS"); break;
	case CF_WINDOW: strcpy(str,"DWINDOW"); strcpy(fmtstr,"CF_WINDOW"); break;
	case CF_POINT: strcpy(str,"DPOINT"); strcpy(fmtstr,"CF_POINT"); break;
  }
  if (fstr != NULL) strcpy(fstr,fmtstr);
  return str;
}
int isComplexProperty(ExportPropertyListStructEx *prp)
{
   if (prp->prpSizeIn == 0 || prp->prpSize == 0) return 0;
   if (prp->prpFormat == prp->prpFormatIn &&
	   prp->prpSize == prp->prpSizeIn) return 0;
   return -1;
}
int writeThisHdrFile(FILE *fp,EQP *el)
{
  char tag[64],lcltag[64],bigtag[64];
  char type[32],typeIn[32];
  char idleName[128],levelName[128],stepsName[128],procname[32],*c;
  ExportPropertyListStructEx *prp;
  int cc=0;
  double mx,mn;
  time_t t = time(NULL);

  sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag);
  tag[0] = toupper(tag[0]);
  sprintf(bigtag,"%s",tag); strupr(bigtag);

  fprintf(fp,"Attribute VB_Name = \"%sModule\"\n",tag);
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' %sModule.bas    REVISION HISTORY:\n",tag);
  fprintf(fp,"' Generated by SRVWIZARD on %s",ctime(&t));
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");

  fprintf(fp,"\n''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' Global variables and constants\n");
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"Private Const %sLocalName$ = \"%s\"\n",tag,bigtag);
  fprintf(fp,"Private Const %sNumDevices",tag); fputc('%',fp);
  fprintf(fp," = %d\n",el->numdevs);
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
	sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
	fprintf(fp,"Global Const PRP_%s_SIZE = %d\n",bigtag,prp->prpSize);
	if (isComplexProperty(prp))
	{
	  fprintf(fp,"Global Const PRP_%s_INSIZE = %d\n",bigtag,prp->prpSizeIn);
	}
	if (prp->prpFormat < CF_STRUCT && prp->prpFormat != CF_TEXT)
	{
	  mx = prp->egu.max;
	  mn = prp->egu.min;
	  if (mx == 0 && mn == 0) mx = 100;
      fprintf(fp,"Global Const PRP_%s_UPRLIMIT = %g\n",bigtag,mx);
      fprintf(fp,"Global Const PRP_%s_LWRLIMIT = %g\n",bigtag,mn);
	}
  }
  fprintf(fp,"\n");
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
	strcpy(type,gettype(prp->prpFormat,NULL));
	sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
	sprintf(lcltag,"%s",prp->prpName); strlwr(lcltag); strudr(lcltag);
	lcltag[0] = toupper(lcltag[0]);
	if (prp->prpFormat == CF_STRUCT)
	{
      fprintf(fp,"\n' TODO: change the following datatype to your structure :\n");
	}
    if (prp->numProcessSteps > 0)
	{
		strncpy(procname,prp->prpName,32);
		if ((c=strstr(procname,".START")) == NULL) continue;
		*c = 0;
        sprintf(idleName,"g%sIdleCondition",procname);
        sprintf(levelName,"g%sProcessLevel",procname);
        sprintf(stepsName,"g%sNumProcessSteps",procname);
		fprintf(fp,"Global %s As Boolean\n",idleName);
		fprintf(fp,"Global %s As Integer\n",levelName);
		fprintf(fp,"Global Const %s = %d\n",stepsName,prp->numProcessSteps);
	}
	else if (prp->prpSize == 1)
	{
	  if (prp->prpFormat == CF_TEXT)
        fprintf(fp,"Global prp%s As String * 1\n",lcltag);
	  else
        fprintf(fp,"Global prp%s As %s\n",lcltag,type);
	}
	else if (prp->prpSize > 1)
	{
	  if (prp->prpFormat == CF_TEXT)
        fprintf(fp,"Global prp%s As String * PRP_%s_SIZE\n",lcltag,bigtag);
	  else
        fprintf(fp,"Global prp%sBuffer(PRP_%s_SIZE - 1) As %s\n",lcltag,bigtag,type);
	}
	if (isComplexProperty(prp))
	{
	    strcpy(typeIn,gettype(prp->prpFormatIn,NULL));
		if (prp->prpSizeIn == 1)
		{
		  if (prp->prpFormatIn == CF_TEXT)
			fprintf(fp,"Global prp%sIn As String * 1\n",lcltag);
		  else
			fprintf(fp,"Global prp%sIn As %s\n",lcltag,typeIn);
		}
		else if (prp->prpSizeIn > 1)
		{
		  if (prp->prpFormatIn == CF_TEXT)
			fprintf(fp,"Global prp%sIn As String * PRP_%s_INSIZE\n",lcltag,bigtag);
		  else
			fprintf(fp,"Global prp%sInBuffer(PRP_%s_INSIZE - 1) As %s\n",lcltag,bigtag,typeIn);
		}
	}
  }
  return cc;
}
void putThisBkgInfo(FILE *fp,EQP *el)
{
  char tag[64],bigtag[64],lcltag[64],type[32];
  ExportPropertyListStructEx *prp;

  fprintf(fp,"\n");
  sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag);
  tag[0] = toupper(tag[0]);
  sprintf(bigtag,"%s",tag); strupr(bigtag);

  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' %sSimulateData is a useful routine for test servers\n",tag);
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"Function %sSimulateData(offset As Double, range As Double) As Double\n",tag);
  fprintf(fp,"%sSimulateData = offset + range * Rnd\n",tag);
  fprintf(fp,"End Function\n\n");  

  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' include %sBackgroundFunction in a Timer to perform background IO, etc.\n",tag);
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"Function %sBackgroundFunction(srvControl As Srv) As Integer\n",tag);
  fprintf(fp,"Dim i As Integer\n");
  fprintf(fp,"' TODO: put your IO or other background activity here\n\n");
  fprintf(fp,"' Clear alarms at start of IO task (see if they come back)\n");
  fprintf(fp,"  srvControl.EqpClearAlarm -1\n\n");
  fprintf(fp,"' TODO: replace %sSimulateData() with your own data acquisition\n",tag);
  fprintf(fp,"'       and manipulation routines\n");
  
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
	if (prp->prpAccessMode & CA_WRITE) continue;
	strcpy(type,gettype(prp->prpFormat,NULL));
	sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
	sprintf(lcltag,"%s",prp->prpName); strlwr(lcltag); strudr(lcltag);
	lcltag[0] = toupper(lcltag[0]);
    switch( prp->prpFormat)
	{
      case CF_SHORT:
	  case CF_LONG:
	  case CF_FLOAT:
	  case CF_DOUBLE:
	  case CF_BYTE:
	    if (prp->prpSize == 1)
		{
          fprintf(fp,"  prp%s = %sSimulateData(PRP_%s_LWRLIMIT, PRP_%s_UPRLIMIT - PRP_%s_LWRLIMIT)\n",
			  lcltag,tag,bigtag,bigtag,bigtag);
		}
  	    else if (prp->prpSize > 1)
		{
          fprintf(fp,"  For i = 0 To PRP_%s_SIZE - 1\n",bigtag);
          fprintf(fp,"     prp%sBuffer(i) = %sSimulateData(PRP_%s_LWRLIMIT, PRP_%s_UPRLIMIT - PRP_%s_LWRLIMIT)\n",
			  lcltag,tag,bigtag,bigtag,bigtag);
          fprintf(fp,"  Next\n");
 		}
        break;
	  default:
		break;
	}
  }
  fprintf(fp,"' make use of SetAlarm() as needed :\n\n");
  fprintf(fp,"' srvControl.EqpSetAlarm devNr, almCode, CADDR(almData(0)), 0\n\n");
  fprintf(fp,"' set error codes as needed:\n");
  fprintf(fp,"  %sBackgroundFunction = 0\n",tag);
  fprintf(fp,"End Function\n\n");
}
void putRegisterPropertyInfo(FILE *fp, EQP *el)
{
  int cc = 0;
  ExportPropertyListStructEx *prp;
  char desc[128],mode[128];
  char tag[64],lcltag[64],bigtag[64],type[32],cftype[32],cftypeIn[32];

  sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag);
  sprintf(lcltag,"%s_TAG",tag); strupr(lcltag); 
  tag[0] = toupper(tag[0]);
  
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' include %sInitFunction in Form_load (for instance) to initialize the server\n",tag);
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"Function %sInitFunction(srvControl As Srv, expName As String) As Integer\n",tag);
  fprintf(fp,"Dim rc As Integer, id As Integer\n\n");

  fprintf(fp,"' FECNAME: either read from file 'fecid.csv' or user API :\n");
  fprintf(fp,"'rc = RegisterFecName(fecname$, desc$, loc$, hdw$, resp$, portoffset)\n\n");

  fprintf(fp,"' DEVICE SERVER: device server parameters\n");
  fprintf(fp,"srvControl.EqpNumberModules = %sNumDevices",tag); fputc('%',fp); fprintf(fp,"\n");
  fprintf(fp,"srvControl.EqpName = %sLocalName$\n",tag);
  fprintf(fp,"srvControl.ExportName = expName\n");
  fprintf(fp,"'now enable the server\n");
  fprintf(fp,"srvControl.Enabled = True\n");
  fprintf(fp,"rc = srvControl.EqpStatus\n");
  fprintf(fp,"If rc Then GoTo Exit%sInitFunction\n",tag);
  fprintf(fp,"'Property Registration:\n");
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
      sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
   	  strcpy(type,gettype(prp->prpFormat,cftype));
	  if (prp->egu.min == 0 && prp->egu.max == 0 && strlen(prp->egu.units) == 0)
	    sprintf(desc,"%s",prp->prpDescription);
	  else
	    sprintf(desc,"[%g:%g %s]%s",prp->egu.min,prp->egu.max,prp->egu.units,prp->prpDescription);
	  strcpy(mode,"CA_READ");
	  if (prp->prpAccessMode&CA_WRITE) strcat(mode," + CA_WRITE");
      if (isComplexProperty(prp))
	  {
   	    strcpy(type,gettype(prp->prpFormatIn,cftypeIn));
        fprintf(fp,"id = srvControl.EqpRegisterPropertyEx(\"%s\", PRP_%s_INSIZE, %s, \"\", PRP_%s_SIZE, %s, \"\", %s, \"%s\")\n",
		  prp->prpName,bigtag,cftypeIn,bigtag,cftype,mode,desc);
	  }
	  else if (prp->prpSize == 0 && prp->prpSizeIn == 0)
	  {
        fprintf(fp,"id = srvControl.EqpRegisterPropertyEx(\"%s\", 0, CF_NULL, \"\", 0, CF_NULL, \"\", %s, \"%s\")\n",
		  prp->prpName,mode,desc);
	  }
	  else if (prp->prpSize > 0 && prp->prpSizeIn == 0)
	  {
        fprintf(fp,"id = srvControl.EqpRegisterPropertyEx(\"%s\", 0, CF_NULL, \"\", PRP_%s_SIZE, %s, \"\", %s, \"%s\")\n",
		  prp->prpName,bigtag,cftype,mode,desc);
	  }
	  else if (prp->prpSize == 0 && prp->prpSizeIn > 0)
	  {
        fprintf(fp,"id = srvControl.EqpRegisterPropertyEx(\"%s\", PRP_%s_SIZE, %s, \"\", 0, CF_NULL, \"\", %s, \"%s\")\n",
		  prp->prpName,bigtag,cftype,mode,desc);
	  }
      else
	  {
        fprintf(fp,"id = srvControl.EqpRegisterPropertyEx(\"%s\", PRP_%s_SIZE, %s, \"\", PRP_%s_SIZE, %s, \"\", %s, \"%s\")\n",
		  prp->prpName,bigtag,cftype,bigtag,cftype,mode,desc);
	  }
      fprintf(fp,"If id < 0 Then GoTo Exit%sInitFunction\n",tag);
  }
  fprintf(fp,"'Device Name Registration:\n");
  fprintf(fp,"' TODO: fill in the devices name which make sense for your server\n");
  fprintf(fp,"For i = 0 To %sNumDevices",tag); fputc('%',fp); fprintf(fp," - 1\n");
  fprintf(fp,"  rc = srvControl.EqpRegisterModule(\"DEVICE\" + Trim$(str$(i)), i)\n");
  fprintf(fp,"  If rc Then GoTo Exit%sInitFunction\n",tag);
  fprintf(fp,"Next\n");
  fprintf(fp,"' Everything should be okay, check for deeper problems:\n");
  fprintf(fp,"%sInitFunction = SRVSTATUS\n",tag);
  fprintf(fp,"'\n");
  fprintf(fp,"Exit%sInitFunction:\n",tag);
  fprintf(fp,"If rc Then\n");
  fprintf(fp,"  %sInitFunction = rc\n",tag);
  fprintf(fp,"ElseIf id < 0 Then\n");
  fprintf(fp,"  %sInitFunction = -id\n",tag);
  fprintf(fp,"End If\n");
  fprintf(fp,"End Function\n");
  return;
}
void putThisEqmInfo(FILE *fp,EQP *el)
{
  char tag[64],bigtag[64],type[32],cftype[32],procname[32],*c;
  ExportPropertyListStructEx *prp;

  sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);

  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"' include %sEqpFcn in the EqpFcn event of the Srv control\n",tag);
  fprintf(fp,"''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''\n");
  fprintf(fp,"Sub %sEqpFcn(srvControl As Srv, ByVal devName As String, ByVal devProperty As String, ByVal outArrayLen As Long, ByVal inArrayLen As Long, ByVal devAccess As Integer)\n",tag);
  fprintf(fp,"Dim i As Integer\n");
  fprintf(fp,"Dim devNr As Integer\n");
  /* local variables for input data */
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
	if (prp->prpSizeIn == 0) continue;
    sprintf(tag,"%s",prp->prpName); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
	sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
   	strcpy(type,gettype(prp->prpFormat,cftype));
    if (prp->prpSizeIn > 1) 
	{
	  if (prp->prpFormat == CF_TEXT)
	  {
        if (isComplexProperty(prp))
          fprintf(fp,"Dim lcl%s As String * PRP_%s_INSIZE\n",tag,bigtag);
		else
          fprintf(fp,"Dim lcl%s As String * PRP_%s_SIZE\n",tag,bigtag);
	  }
	  else
	  {
        if (isComplexProperty(prp))
          fprintf(fp,"Dim lcl%sBuffer(PRP_%s_INSIZE - 1) As %s\n",tag,bigtag,type);
		else
          fprintf(fp,"Dim lcl%sBuffer(PRP_%s_SIZE - 1) As %s\n",tag,bigtag,type);
	  }
	}
	else
    {
	  if (prp->prpFormat == CF_TEXT)
	  {
         fprintf(fp,"Dim lcl%s As String * 1\n",tag);
	  }
	  else
	  {
        fprintf(fp,"Dim lcl%s As %s\n",tag,type);
	  }
	}
  }

  if (el->numdevs > 1)
  {
	fprintf(fp,"' If your properties make use of the device number associated with the device name:\n");
	fprintf(fp,"devNr = srvControl.EqpGetModuleNumber(devName)\n");
	fprintf(fp,"If devNr < 0 Then\n");
	fprintf(fp,"    srvControl.EqpSetCompletion illegal_equipment_number, \"\"\n");
	fprintf(fp,"    Exit Sub\n");
	fprintf(fp,"End If\n");
  }
  fprintf(fp,"' check which property was asked for:\n");
  fprintf(fp,"Select Case devProperty\n");
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
	strncpy(procname,prp->prpName,32);
    sprintf(tag,"%s",prp->prpName); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
	sprintf(bigtag,"%s",prp->prpName); strupr(bigtag); strudr(bigtag);
    fprintf(fp,"  Case \"%s\"\n",prp->prpName);
	
    strcpy(type,gettype(prp->prpFormat,cftype));
	if (prp->prpAccessMode & CA_WRITE)
	{
	  fprintf(fp,"    If (devAccess And CA_WRITE) Then\n");
	  if (prp->numProcessSteps > 0 && (c=strstr(procname,".PAUSE")) != NULL)
	  {
	    *c = 0;
        fprintf(fp,"      g%sIdleCondition = True\n",procname);
	  }
	  else if (prp->numProcessSteps > 0 && (c=strstr(procname,".RESUME")) != NULL)
	  {
	    *c = 0;
        fprintf(fp,"      g%sIdleCondition = False\n",procname);
	  }
	  else if (prp->numProcessSteps > 0 && (c=strstr(procname,".STOP")) != NULL)
	  {
	    *c = 0;
		fprintf(fp,"      g%sProcessLevel = 0\n",procname);
		fprintf(fp,"      g%sIdleCondition = False\n",procname);
	  }
	  else if (prp->prpSizeIn > 0)
	  {
		  fprintf(fp,"      If inArrayLen = 0 Then\n");
		  fprintf(fp,"        srvControl.EqpSetCompletion illegal_read_write, \"\"\n");
		  fprintf(fp,"        Exit Sub\n");
		  fprintf(fp,"      End If\n");
	  }
	}
	else
	{
	  fprintf(fp,"    If (devAccess And CA_WRITE) Then\n");
	  fprintf(fp,"      srvControl.EqpSetCompletion illegal_read_write, \"\"\n");
	  fprintf(fp,"      Exit Sub\n");
	  fprintf(fp,"    End If\n");
	  if (prp->prpSizeIn > 0) fprintf(fp,"    If inArrayLen > 0 Then\n");
    }
	if (prp->prpSizeIn > 0)
	{
	  fprintf(fp,"      ' Get the incoming data:\n");
	  if (prp->prpSizeIn > 1) 
	  {
	    if (prp->prpFormatIn != CF_TEXT)
		{
 	      fprintf(fp,"      srvControl.EqpRecvData lcl%sBuffer\n",tag);
		  fprintf(fp,"      ' Do range checking:\n");
		  fprintf(fp,"      For i = 0 To inArrayLen - 1\n");
		  fprintf(fp,"        If lcl%sBuffer(i) > PRP_%s_UPRLIMIT Then\n",tag,bigtag);
		  fprintf(fp,"          srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"          Exit Sub\n");
		  fprintf(fp,"        End If\n");
		  fprintf(fp,"        If lcl%sBuffer(i) < PRP_%s_LWRLIMIT Then\n",tag,bigtag);
		  fprintf(fp,"          srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"          Exit Sub\n");
		  fprintf(fp,"        End If\n");
		  fprintf(fp,"      Next\n");
		  fprintf(fp,"      ' Copy incoming data into global data buffer\n");
          if (isComplexProperty(prp))
		    fprintf(fp,"      Call MEMCPY(prp%sInBuffer(0), lcl%sBuffer(0), inArrayLen * Len(lcl%sBuffer(0)))\n",
  			     tag,tag,tag);
		  else
		    fprintf(fp,"      Call MEMCPY(prp%sBuffer(0), lcl%sBuffer(0), inArrayLen * Len(lcl%sBuffer(0)))\n",
  			     tag,tag,tag);
		}
		else
		{
 	      fprintf(fp,"      srvControl.EqpRecvData lcl%s\n",tag);
		  fprintf(fp,"      ' Copy incoming data into global data buffer\n");
          if (isComplexProperty(prp))
		    fprintf(fp,"      prp%sIn =  lcl%s\n",tag,tag);
		  else
		    fprintf(fp,"      prp%s =  lcl%s\n",tag,tag);
		}
	  }
	  else
	  {
	    fprintf(fp,"      srvControl.EqpRecvData lcl%s\n",tag);
		if (prp->numProcessSteps > 0 && (c=strstr(procname,".START")) != NULL)
		{
		  *c = 0;
		  fprintf(fp,"      ' Do range checking:\n");
		  fprintf(fp,"      If g%sProcessLevel > g%sNumProcessSteps Then\n",procname,procname);
		  fprintf(fp,"        srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"        Exit Sub\n");
		  fprintf(fp,"      End If\n");
		  fprintf(fp,"      If g%sProcessLevel < 0 Then\n",procname);
		  fprintf(fp,"        srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"        Exit Sub\n");
		  fprintf(fp,"      End If\n");
		  fprintf(fp,"      ' Copy incoming data into global variable\n");
		  fprintf(fp,"      g%sProcessLevel = lcl%s\n",procname,tag);
		  fprintf(fp,"      g%sIdleCondition = False\n",procname);
		}
	    else if (prp->prpFormatIn != CF_TEXT)
		{
		  fprintf(fp,"      ' Do range checking:\n");
		  fprintf(fp,"      If lcl%s > PRP_%s_UPRLIMIT Then\n",tag,bigtag);
		  fprintf(fp,"        srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"        Exit Sub\n");
		  fprintf(fp,"      End If\n");
		  fprintf(fp,"      If lcl%s < PRP_%s_LWRLIMIT Then\n",tag,bigtag);
		  fprintf(fp,"        srvControl.EqpSetCompletion out_of_range, \"\"\n");
		  fprintf(fp,"        Exit Sub\n");
		  fprintf(fp,"      End If\n");
		  fprintf(fp,"      ' Copy incoming data into global data buffer\n");
		  fprintf(fp,"      prp%s =  lcl%s\n",tag,tag);
		}
		else
		{
		  fprintf(fp,"      ' Copy incoming data into global data buffer\n");
		  fprintf(fp,"      prp%s =  lcl%s\n",tag,tag);
		}
	  }
	}
	if (prp->prpSizeIn > 0 || prp->prpAccessMode & CA_WRITE) fprintf(fp,"    End If\n");

    if (prp->prpSize == 0) continue;
	strncpy(procname,prp->prpName,32);
	if (prp->numProcessSteps > 0 && (c=strstr(procname,".START")) != NULL)
	{
	  *c = 0;
      fprintf(fp,"    If outArrayLen > 0 Then\n");
      fprintf(fp,"      srvControl.EqpSendData g%sProcessLevel\n",procname);
	}
	else if (prp->numProcessSteps > 0 && (c=strstr(procname,".STATUS")) != NULL)
	{
	  *c = 0;
      fprintf(fp,"    If outArrayLen > 0 Then\n");
      fprintf(fp,"      srvControl.EqpSendData g%sProcessLevel\n",procname);
	}
    else if (prp->prpSize > 1 && prp->prpFormat != CF_TEXT) 
	{
	  if ((int)prp->prpSize == el->numdevs)
	  {
        fprintf(fp,"    If (outArrayLen > 0) And (outArrayLen <= (PRP_%s_SIZE - devNr)) Then\n",bigtag);
        fprintf(fp,"      srvControl.EqpSendData prp%sBuffer(devNr)\n",tag);
        fprintf(fp,"    Else\n");
        fprintf(fp,"      srvControl.EqpSetCompletion dimension_error, \"\"\n");
        fprintf(fp,"      Exit Sub\n");
	  }
	  else
	  {
        fprintf(fp,"    If outArrayLen > 0 Then\n");
        fprintf(fp,"      srvControl.EqpSendData prp%sBuffer\n",tag);
	  }
	}
	else
	{
      fprintf(fp,"    If outArrayLen > 0 Then\n");
      fprintf(fp,"      srvControl.EqpSendData prp%s\n",tag);
	}
    fprintf(fp,"    End If\n");
  }
  fprintf(fp,"  Case Else\n");
  fprintf(fp,"    srvControl.EqpSetCompletion illegal_property, \"\"\n");
  fprintf(fp,"End Select\n");
  fprintf(fp,"' If it's made it this far, it's a success:\n");
  fprintf(fp,"srvControl.EqpSetCompletion 0, \"\"\n");
  fprintf(fp,"End Sub\n");
}
int putEQPTemplate(EQP *el)
{
  FILE *fp = NULL;
  char fname[64],tag[64],fullFileName[128],procname[32],*c;
  int cc=0;
  time_t t = time(NULL);
  ExportPropertyListStructEx *prp;

  sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
  sprintf(fname,"%s.bas",tag);
  if (strlen(targetPath) > 0) 
  {
	 sprintf(fullFileName,"%s\\%s",targetPath,fname);	 
  }
  else
  {
	 sprintf(fullFileName,"%s",fname);
  }
  if ((fp=fopen(fullFileName,"w")) == NULL) ccerr(file_error);
  if ((cc=writeThisHdrFile(fp,el)) != 0) ccerr(cc);
  putThisBkgInfo(fp,el);
  for (prp=el->prp; prp != NULL; prp = prp->next)
  {
    if (prp->numProcessSteps > 0)
	{
	  strncpy(procname,prp->prpName,32);
	  if ((c=strstr(procname,".START")) == NULL) continue;
	  *c = 0;
      putProcessFunction(fp,procname,prp->numProcessSteps);
 	}
  }
  putRegisterPropertyInfo(fp,el);
  putThisEqmInfo(fp,el);
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
char *getSystemDir(void)
{
  char *p = getenv("SystemRoot");
  if (p == NULL) return "C:\\WINNT\\System32"; // best guess
  return p;
}
char *getVBCommonDir(void)
{
  FILE *fp = NULL;
  int cc=0,done=0;
  int dir_col;
  static char str[256],hdr[256],scratch[16];

  if ((fp=fopen("wizard.csv","r")) == NULL) ccerr(no_such_file);
  while (fgets(str,255,fp) != NULL)
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((dir_col=findcol(hdr,"COMMON_DIR")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(str,colptr(dir_col,str),256);
    break;
  }
err:
  if (fp) fclose(fp);
  if (cc)
  {
    strncpy(str,"\\\\mkinthera1\\hera\\Projects\\Service\\VB\\FORMS",256);
  }
  return str;

}
int putProjectFile(void)
{
  FILE *fp = NULL;
  char fname[64],tag[64],namtag[64],bigtag[64],fullFileName[128],*c;
  int cc=0;
  EQP *el;
  time_t t = time(NULL);

  sprintf(namtag,"%s",eqplist->lclname); strlwr(namtag); strudr(namtag); namtag[0] = toupper(namtag[0]);
  sprintf(fname,"%s.vbp",namtag);
  strcpy(bigtag,namtag); strupr(bigtag);
  if (strlen(targetPath) > 0) 
  {
	 sprintf(fullFileName,"%s\\%s",targetPath,fname);	 
  }
  else
  {
	 sprintf(fullFileName,"%s",fname);
  }
  if ((fp=fopen(fullFileName,"w")) == NULL) ccerr(file_error);
  fprintf(fp,"Type=Exe\n");
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
    strncpy(bigtag,tag,64); strupr(bigtag);
    fprintf(fp,"Module=%s; %s.bas\n",tag,tag);
  }
  c = getSystemDir();
  fprintf(fp,"Reference=*\\G{00020430-0000-0000-C000-000000000046}#2.0#0#%s\\StdOle2.Tlb#OLE Automation\n",c);
  fprintf(fp,"Form=%s.frm\n",namtag);
  fprintf(fp,"Object={4727FED0-D8EC-11D0-B104-0000E80D9CC1}#1.0#0; srv.ocx\n");
  c = getVBCommonDir();
  fprintf(fp,"Module=tine; %s\\tine.bas\n",c);
  fprintf(fp,"Module=Utils; %s\\Utils.bas\n",c);
  fprintf(fp,"Startup=\"%s\"\n",namtag);
  fprintf(fp,"HelpFile=\"\"\n");
  fprintf(fp,"Command32=\"\"\n");
  fprintf(fp,"Name=\"%s\"\n",namtag);
  fprintf(fp,"HelpContextID=\"0\"\n");
  fprintf(fp,"CompatibleMode=\"0\"\n");
  fprintf(fp,"MajorVer=1\n");
  fprintf(fp,"MinorVer=0\n");
  fprintf(fp,"RevisionVer=0\n");
  fprintf(fp,"AutoIncrementVer=0\n");
  fprintf(fp,"ServerSupportFiles=0\n");
  fprintf(fp,"VersionCompanyName=\"DESY\"\n");
  fprintf(fp,"CompilationType=0\n");
  fprintf(fp,"OptimizationType=0\n");
  fprintf(fp,"FavorPentiumPro(tm)=0\n");
  fprintf(fp,"CodeViewDebugInfo=0\n");
  fprintf(fp,"NoAliasing=0\n");
  fprintf(fp,"BoundsCheck=0\n");
  fprintf(fp,"OverflowCheck=0\n");
  fprintf(fp,"FlPointCheck=0\n");
  fprintf(fp,"FDIVCheck=0\n");
  fprintf(fp,"UnroundedFP=0\n");
  fprintf(fp,"StartMode=0\n");
  fprintf(fp,"Unattended=0\n");
  fprintf(fp,"Retained=0\n");
  fprintf(fp,"ThreadPerObject=0\n");
  fprintf(fp,"MaxNumberOfThreads=1\n");

err:
  if (fp != NULL) fclose(fp);
  return cc;
}
int putProcessFunction(FILE *fp,char *procname, int nsteps)
{
  char fcnName[128],idleName[128],exitName[128],levelName[128],stepsName[128];
  int i;

  sprintf(exitName,"%sProcessFunctionExit",procname);
  sprintf(idleName,"g%sIdleCondition",procname);
  sprintf(levelName,"g%sProcessLevel",procname);
  sprintf(fcnName,"%sProcessFunction",procname);
  sprintf(stepsName,"g%sNumProcessSteps",procname);
  fprintf(fp,"Function %s() As Integer\n",fcnName);
  fprintf(fp,"  Static duration As Integer\n");
  fprintf(fp,"  Static start\n");
  fprintf(fp,"  Dim interim\n\n");
  fprintf(fp,"  interim = timer\n\n");
  fprintf(fp,"\n  If %s Then GoTo %s\n",idleName,exitName);
  fprintf(fp,"' TODO: fill in the activities to be performed at each step\n");
  fprintf(fp,"'       The following example code simply waits for 3 seconds at\n");
  fprintf(fp,"'       each process level.  You may want to start IO readouts\n");
  fprintf(fp,"'       at one level and poll for a ready flag in another, etc.\n");

  fprintf(fp,"  If %s  > 0 And duration = 0 Then\n",levelName);
  fprintf(fp,"    start = interim\n");
  fprintf(fp,"    duration = 3\n");
  fprintf(fp,"  End If\n");

  fprintf(fp,"  Select Case %s\n",levelName);
  for (i=1; i<=nsteps; i++)
  {
    fprintf(fp,"    Case %d:\n",i);
    fprintf(fp,"      If interim < start + duration Then GoTo %s\n",exitName);
    fprintf(fp,"      duration = 0\n");
  }
  fprintf(fp,"    Case Else:\n");
  fprintf(fp,"       %s = 0\n",levelName);
  fprintf(fp,"  End Select\n");
  fprintf(fp,"  If %s > 0 Then %s = (%s + 1) Mod (%s + 1)\n",levelName,levelName,levelName,stepsName);
  fprintf(fp,"%s:\n",exitName);
  fprintf(fp,"  %s = 0\n",fcnName);
  fprintf(fp,"End Function\n");
  return 0;
}
int putForm1(void)
{
  FILE *fp = NULL;
  char fname[64],namtag[64],tag[64],bigtag[64],fullFileName[128],procname[32],*c;
  int cc=0;
  EQP *el;
  time_t t = time(NULL);
  ExportPropertyListStructEx *prp;

  sprintf(namtag,"%s",eqplist->lclname); strlwr(namtag); strudr(namtag); namtag[0] = toupper(namtag[0]);
  sprintf(fname,"%s.frm",namtag);
  strcpy(bigtag,namtag); strupr(bigtag);
  if (strlen(targetPath) > 0) 
  {
	 sprintf(fullFileName,"%s\\%s",targetPath,fname);	 
  }
  else
  {
	 sprintf(fullFileName,"%s",fname);
  }
  if ((fp=fopen(fullFileName,"w")) == NULL) ccerr(file_error);
  fprintf(fp,"VERSION 5.00\n");
  fprintf(fp,"Object = \"{4727FED0-D8EC-11D0-B104-0000E80D9CC1}#1.0#0\"; \"srv.ocx\"\n");
  fprintf(fp,"Begin VB.Form %s\n",namtag); 
  fprintf(fp,"   Caption         =   \"%s\"\n",namtag);
  fprintf(fp,"   ClientHeight    =   960\n");
  fprintf(fp,"   ClientLeft      =   60\n");
  fprintf(fp,"   ClientTop       =   345\n");
  fprintf(fp,"   ClientWidth     =   2085\n");
  fprintf(fp,"   LinkTopic       =   \"%s\"\n",namtag);
  fprintf(fp,"   ScaleHeight     =   960\n");
  fprintf(fp,"   ScaleWidth      =   2085\n");
  fprintf(fp,"   StartUpPosition =   3  'Windows Default\n");
  // for each eqp module:
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
    strcpy(bigtag,tag); strupr(bigtag);
    fprintf(fp,"   Begin SRVLib.Srv Srv%s \n",tag);
    fprintf(fp,"      Left            =   150\n");
    fprintf(fp,"      Top             =   510\n");
    fprintf(fp,"      _Version        =   65536\n");
    fprintf(fp,"      _ExtentX        =   714\n");
    fprintf(fp,"      _ExtentY        =   714\n");
    fprintf(fp,"      _StockProps     =   64\n");
    fprintf(fp,"   End\n");
  }
  // only one timer
  fprintf(fp,"   Begin VB.Timer IOTimer \n");
  fprintf(fp,"      Interval        =   1000\n");
  fprintf(fp,"      Left            =   630\n");
  fprintf(fp,"      Top             =   510\n");
  fprintf(fp,"   End\n");
  fprintf(fp,"   Begin VB.Label StatusLabel \n");
  fprintf(fp,"      BorderStyle     =   1  'Fixed Single\n");
  fprintf(fp,"      Height          =   435\n");
  fprintf(fp,"      Left            =   150\n");
  fprintf(fp,"      TabIndex        =   0\n");
  fprintf(fp,"      Top             =   30\n");
  fprintf(fp,"      Width           =   1815\n");
  fprintf(fp,"   End\n");
  fprintf(fp,"End\n");

  fprintf(fp,"Attribute VB_Name = \"%s\"\n",namtag);
  fprintf(fp,"Attribute VB_GlobalNameSpace = False\n");
  fprintf(fp,"Attribute VB_Creatable = False\n");
  fprintf(fp,"Attribute VB_PredeclaredId = True\n");
  fprintf(fp,"Attribute VB_Exposed = False\n");
  // only one form_load
  fprintf(fp,"Private Sub Form_Load()\n");
  fprintf(fp,"Dim cc As Integer\n");
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
    strcpy(bigtag,tag); strupr(bigtag);
    fprintf(fp,"cc = %sInitFunction(Srv%s, \"%s\")\n",tag,tag,el->tagname);
    fprintf(fp,"If cc Then goto SrvInitError\n");
  }
  fprintf(fp,"SrvInitError:\n");
  fprintf(fp,"If cc Then\n");
  fprintf(fp,"  StatusLabel.BackColor = &HFF  ' RED\n");
  fprintf(fp,"  StatusLabel.Caption = RPCERROR(cc)\n");
  fprintf(fp,"Else\n");
  fprintf(fp,"  StatusLabel.BackColor = &HFF00& ' GREEN\n");
  fprintf(fp,"  StatusLabel.Caption = \"Server running\"\n");
  fprintf(fp,"End If\n");
  fprintf(fp,"End Sub\n");
  fprintf(fp,"\n");
  // for each eqp module
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
    strcpy(bigtag,tag); strupr(bigtag);
    fprintf(fp,"Private Sub Srv%s_EqpFcn(ByVal devName As String, ByVal devProperty As String, ByVal outArrayLen As Long, ByVal inArrayLen As Long, ByVal devAccess As Integer)\n",tag);
    fprintf(fp,"%sEqpFcn Srv%s, devName, devProperty, outArrayLen, inArrayLen, devAccess\n",tag,tag);
    fprintf(fp,"End Sub\n");
    fprintf(fp,"\n");
  }
  // only one timer
  fprintf(fp,"Private Sub IOTimer_Timer()\n");
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
    sprintf(tag,"%s",el->lclname); strlwr(tag); strudr(tag); tag[0] = toupper(tag[0]);
    strcpy(bigtag,tag); strupr(bigtag);
    fprintf(fp,"%sBackgroundFunction Srv%s\n",tag,tag);
    for (prp=el->prp; prp != NULL; prp = prp->next)
	{
	  if (prp->numProcessSteps > 0)
	  {
		 strncpy(procname,prp->prpName,32);
		 if ((c=strstr(procname,".START")) == NULL) continue;
		 *c = 0;
         fprintf(fp,"%sProcessFunction\n",procname);
	  }
	}
  }
  fprintf(fp,"End Sub\n");
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
int main(int argc, char *argv[])
{
  EQP *el;
  int cc,len;

  switch (argc)
  {           
    case 2:
	  strncpy(targetPath,argv[1],64); targetPath[63] = 0;
	  if ((len=strlen(targetPath)) > 0)
	  {
	    if (targetPath[len-1] == '\\') targetPath[len-1] = 0;
	  }
      break;
    default:
	  targetPath[0] = 0;
  }


  getFecInformation();

  if ((cc=getExportInformation()) != 0)
  {
	printf("error : %s\n",erlst[cc]);
	exit(1);
  }

  if ((cc=putProjectFile()) != 0)  
  {
	printf("Project file error : %s\n",erlst[cc]);
    exit(1);
  }
  if ((cc=putForm1()) != 0)
  {
	printf("putForm file error : %s\n",erlst[cc]);
	exit(1);
  }
  for (el=eqplist; el != NULL; el=el->next)  // go through the list
  {
     if ((cc=putEQPTemplate(el)) != 0)  
	 {
		printf("%s file error : %s\n",el->lclname,erlst[cc]);
		exit(1);
	 }
  }
}
