Attribute VB_Name = "OpticsHdrs"
Option Explicit

Global Const MAIN_VERSION% = 400
Global Const ORBCOR_VERSION% = 400
Global Const BUMP_VERSION% = 100

Global Const HORIZONTAL% = 0
Global Const VERTICAL% = 1
Global Const HOR% = 0
Global Const VER% = 1

' Array Dimensions for HERA-e/p (sufficient for all DESY Accelerators):
' Array Dimension = #Elements - 1 for VB
' Cor/Mon Dimensions from 280 to 300 for Hera Upgrade: 25, plus
' define constants explicitly as ints (were they variants?) 25.06.01 S. Herb

Global Const CORMONDIM% = 300 - 1        ' max of mondim and cordim
Global Const MONDIM% = 300 - 1           ' Size for monitor arrays
Global Const CORDIM% = 300 - 1           ' Size for correction coil arrays
Global Const LATDIM% = 3600 - 1          ' Size for Optic Lattice arrays
Global Const ELEDIM% = 256 - 1           ' Size for Optic Element Type arrays
Global Const MAXSPEZBUMPS% = 128 - 1     ' 128 Spezialbeulen / Maschine u. Jahr
Global Const MAXORBITSRPC% = 10 - 1      ' max. 10 Orbits pro RPC Aufruf
Global Const MAXORBITS% = 20 - 1         ' max. 20 Orbits fr Gesamtliste
Global Const MAXOPTICS% = 20 - 1         ' 20 Optics / year

Global Const NRMSvsPhase% = 2500 - 1         ' points in phase to calculate rms values

Global Const NO_YEAR$ = "1234"
Global Const NO_MACHINE$ = "12345678"
Global Const ErrorLeft$ = ">>>>> "
Global Const ErrorRight$ = " <<<<<"

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' Globals

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' Alle RPC Aufrufe
Global OpticServer As String              ' DNS name of server
Global OpticServerTag As String * 2       ' Tag = Eqp (max. 6 char) + ServerTag (max. 2 char)
Global OpticRPCEqpF As String             ' Equipment Function (Global for use in RPC Error Messages)
Global OpticRPCProp As String             ' Property           (Global for use in RPC Error Messages)
Global OpticRC As Integer                 ' return code from server
Global OpticServerErrText As String * 32  ' Errors > 600 from HP Server: from header of RPC Call
Global OpticReturnMsgShort As String      ' Message string (short version)
Global OpticReturnMsgLong As String       ' Message string (long version - with "short" message, properties, eqp functions....)

Global OpticRWFlag As Integer             ' read write flag: schreiben < 0; lesen > 0
Global NGDebug As Integer                 ' Debug Flag for Server, 0 = no Debugging
Global HPDebug As Integer                 ' Debug Flag for Server, 0 = no Debugging

' fr Zeitmessungen
Global RPCStart1, RPCEnd1 As Date         ' inner RPC
Global RPCStart2, RPCEnd2 As Date         ' outer RPC

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' Datenstrukturen fr Datenaustausch mit HP via RPC

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' --------------------------------------------------------------------------------------------------------------------------
' Alle RPC Aufrufe benutzen den gleichen RPC Header Typen
' --------------------------------------------------------------------------------------------------------------------------

Type RpcHdrType
' Information from client: determines optics:
    Machine(7) As Byte              ' 8 must be set by client:
                                    '   "herap", "herae", "petrap", "petrae", "doris"
                                    '   "pr-weg", "el-weg", "p-weg", "e-weg", "r-weg"
    Year(3) As Byte                 ' 4 must be set by client, e.g. "1996" is recommended
    
    Text1(59) As Byte               '60
    Text2(59) As Byte               '60
    DateTime(31) As Byte            '32
    fileName(79) As Byte            '80
    Optic(15) As Byte               '16  must be set by client
    OpticFlag(15) As Byte           '16  server: optic status is "THEORY", "VALID", "INVALID" or "UNKNOWN"
    Dummy4(3) As Byte               ' 4
    ErrorString(31) As Byte         '32
                                   '---
                                   '312
    
    OpticEnergy As Single          ' 4

' Control Information from Client:
    Arr_mon As Integer              ' 2   monitor array dimension
    Arr_cor As Integer              ' 2   corrector array dimension
    Arr_il As Integer               ' 2   lattice array dimension
    Arr_jl As Integer               ' 2   element type array dimension
    Hdr_Size As Integer             ' 2   Headergre in Bytes: LEN(rpc_hdr)
    Spez_Hdr_Size As Integer        ' 2   SpezHeadergre in Bytes: LEN(spezhdr), e.g. orbcor_header
    Arr_Size As Integer             ' 2   Arraygre in Bytes: LEN(arr_xxx)

' Information from Server:
    MonNum(1) As Integer            ' 4   # x,y monitors
    CorNum(1) As Integer            ' 4   # x,y correctors
    LatNum As Integer               ' 2   # lattice elements
    EleNum As Integer               ' 2   # element types
    OpticNum As Integer             ' 2   0 < OpticNum < MaxOptics
    
' Misc
    SpezNum As Integer              ' 2   0 < SpezNum < MaxSpezBumps
    Counter As Integer              ' 2   e.g. # of OrbitNames transfered
                                    '32

    QTheory(1) As Single            ' 8
    MomComp As Single               ' 4
    Circumference As Double         ' 8 [m]
                                    '24

' Control Information from Client:
    NGDebug As Integer              ' 2   set debug level on server (Communicationlevel)
    HPDebug As Integer              ' 2   set debug level on server (Applikationsoftware)
    Index As Integer                ' 2   e.g. Length of Orbitlist
    Version As Integer              ' 2   must be set to MAIN_VERSION%
                                    ' 8
                                    '---
                                    '376 total
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "OPTICLIST"
' --------------------------------------------------------------------------------------------------------------------------

Type RpcOpticListType
    OpticName(15) As Byte
    Text1(59) As Byte
    Text2(59) As Byte
End Type

Type RpcHdrOpticListType
    Hdr As RpcHdrType
    Optics(MaxOptics) As RpcOpticListType
End Type
    
' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS",
' Properties "POSITIONS", "DRIFT", "BETA", "ALPHA", "PHI", "DISP", "DISPPRIME", "TYPEPOINTER"
' --------------------------------------------------------------------------------------------------------------------------
Type RpcHdrLatticeDouble
    Hdr As RpcHdrType
    dval(LatDim%) As Double
End Type
    
Type RpcHdrLatticeSingle
    Hdr As RpcHdrType
    sval(LatDim%) As Single
End Type
    
Type RpcHdrLatticeInteger
    Hdr As RpcHdrType
    ival(LatDim%)   As Integer
End Type

Type RpcHdrLattice8Bytes
    Hdr As RpcHdrType
    B8(7, LatDim%) As Byte
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "ELEMENTLIST" (5.8.97)
' --------------------------------------------------------------------------------------------------------------------------
Type RpcElementType
    TypeName(7) As Byte     ' 8
    Length As Double        ' 8
    Strength As Double      ' 8 Quads: k-Value, Dipols: 1/RHO-X/Z
    HX As Double            ' 8 1/RHO-X
    HZ As Double            ' 8 1/RHO-Z
    Angle As Double         ' 8 Tilt
    PetType As Integer      ' 2
    Dummy6(5) As Byte       ' 6
End Type                    '56 ok

Type RpcHdrElementListType
    Hdr As RpcHdrType
    List(EleDim) As RpcElementType
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "LATTICE" (18.8.97)
' --------------------------------------------------------------------------------------------------------------------------
Type RpcHdrLatticeElementType
    Hdr As RpcHdrType
    Position(7) As Byte     ' 8
    Drift As Double         ' 8
    Beta(1) As Single       ' 8
    Alpha(1) As Single      ' 8
    Phi(1) As Single        ' 8
    Disp(1) As Single       ' 8
    DispP(1) As Single      ' 8
    Pointer As Integer      ' 2 Pointer to ElementType
    Dummy6(5) As Byte       ' 6
End Type                    '64 ok

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "CMSTRUCT" (15.8.97)
' --------------------------------------------------------------------------------------------------------------------------
Type RpcCMStructType
    CMName(7) As Byte
    Beta As Single
    Phi As Single
    Disp As Single
    KMax As Single                  ' nur benutzt fr Korrekturspulen, nicht fr Monitore
End Type

Type RpcHdrCMStructType
    Hdr As RpcHdrType
    Mon(MonDim, 1) As RpcCMStructType
    Cor(CorDim, 1) As RpcCMStructType
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "RCMOPS" (18.8.97), Arrays compatible to HP Server 3.00
' --------------------------------------------------------------------------------------------------------------------------
Type RpcHdrRcmops
     Hdr As RpcHdrType
     MonPhi(MonDim%, 1) As Single
     MonBeta(MonDim%, 1) As Single
     MonDisp(MonDim%, 1) As Single
     CorPhi(CorDim%, 1) As Single
     CorBeta(CorDim%, 1) As Single
     CorDisp(CorDim%, 1) As Single
     CorKmax(CorDim%, 1) As Single
     MonName(7, MonDim%, 1) As Byte
     CorName(7, CorDim%, 1) As Byte
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function ORBCOR Special (2.) Header
' --------------------------------------------------------------------------------------------------------------------------
Type RpcOrbcorHdrType
' Orbit Correction:
    QMeas(1) As Single              ' 8
    OrbitEnergy As Single           ' 4
    IBunch As Single                ' 4
        
' EFFCOR called "Micado" (R. Brinkmann), "EFFKCK", "LUMICOR" and "SVDCOR":
    Neff(1) As Integer              ' 4  # x,y for most effective corrector 0..10 per plane is recommended
    KMaxScaled As Byte              ' 1 Y/N N=default; "Y" means KMax=KMax*Energy/MaxEnergy(Machine)
                                    '   "N" is recommended for all Lepton Optics, is default
    KMaxIgnore As Byte              ' 1 Y/N N=default; "Y" means KMax will be ignored in EFFCOR using LIMFLG
    KicksOrCurrents As Byte         ' 1 "Y" server uses kick-array
    Overwrite As Byte               ' 1 if "Y" overwriting of files allowed for Write Orbit
    MonitorMode(15) As Byte         '16
                                    '--
                                    '24
    
' RORBIT WORBIT POLFIT
    OrbitName(31) As Byte          ' client: kann gesetzt sein, kann "" sein, kann auch z.B. "ORBCOR" sein,
                                    ' damit nicht zu viele Orbits abgespeichert werden, aber POLFIT
                                    ' anschlieend laufen kann.
' POLCOR / POLFIT (E. Gianfelice):
                                    '   POLCOR arbeitet NUR mit dem Maschinenorbit als Input u. Output
                                    '   POLFIT arbeitet NUR mit abgespeicherten Orbits
                                    '
    ReferenceOrbit(31) As Byte      ' client: mu nicht gesetzt sein
    MachineFile(31) As Byte         ' Vorsicht: hat im Optic Header 80 Bytes
    UserName(31) As Byte
    
    Comment(127) As Byte
    dummy(252) As Byte
    
    CorrectEnergy As Byte           ' 1 if "Y" do not subtract disp. before corr., SVD
    ReturnDifference As Byte         ' 1 if "Y" return Inew-Iold and Kick_new-Kick_old
    FirstTurn As Byte               ' 1 if "Y" do orbitcorrection assuming first turn or transportline
    
    FitQuality As Double            ' server
    

' POLCOR und POLFIT:
    PolVal8(7) As Double            '64 POLCOR: input from client => pchb.inc variable ah
                                    '           nderungen der Harmonischen, d.h. viele sind 0
                                    '   POLFIT: output from server <= pchb.inc variable ah
                                    '           Mit Referenzorbit = nderungen der Harmonischen
                                    '           Ohne Referenzorbit = Absolut Werte der Harmonischen
    PolStr8(7, 7) As Byte           '64 server e.g. "-1  real", " 0  imag" index harmonic (indh)
                                   '---
                                   '128
    
' Totsch = 3BUMP called "Totschlger" (F. Willeke):
    Totsch(7, 2) As Byte            '24  initially for FW 3-bump information
                                    '--
                                    '32
' OrbScan (F. Brinker)
    MaxAblage(1) As Single
    CorNow(1) As Integer
    CorNext(1) As Integer
    MaxOrbits(1) As Integer
    Pattern As Integer
    
    OrbcorVersion As Integer
    
    ParaText(7, 7) As Byte
    Parameter(7) As Double
End Type                            ' total 496 bytes:  must be mult of 8 for compatibility!
                     
' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function ORBCOR - all Properties except "ORBLIST"
' --------------------------------------------------------------------------------------------------------------------------
Type RpcHdrsOrbcorType
    Hdr As RpcHdrType
    Hdr2 As RpcOrbcorHdrType
    Orbit(MonDim, 1) As Single
    Kicks(CorDim, 1) As Single
    Currents(CorDim, 1) As Single
    MonFlags(MonDim, 1) As Integer
    CorFlags(CorDim, 1) As Integer
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function ORBCOR - Property "ORBLIST"
' --------------------------------------------------------------------------------------------------------------------------
Type RpcOrbListType
    OrbName(31) As Byte
    OrbDatum(31) As Byte
    OrbOptic(15) As Byte
End Type

Type RpcHdrOrbListType
    Hdr As RpcHdrType
    List(MaxOrbitsRPC) As RpcOrbListType
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function BCOEFF
' --------------------------------------------------------------------------------------------------------------------------
Type RpcBumpHeaderType
  BumpType(7) As Byte       '  8 client: "3", "4", "F"ixed ...
  ErrorString(31) As Byte   ' 32 server
  BumpOptic(15) As Byte     ' 16 server: optic for which bump was generated
  Reserve As Single         '  4
  BumpEnergy As Single      '  4 server: energy for saved bump
  dEm As Single             '  4 server/client: untere Grenze = E*(1+dEp)
  dEp As Single             '  4 server/client: obere Grenze = E*(1+dEp)
  CurrentE As Single        '  4 client: bunch current
  CurrentP As Single        '  4 client: bunch current p if ep ww
  QxMeas As Single          '  4 client , if 0 use mops values
  QzMeas As Single          '  4

                            '    k(soll) given by server
  Calibration  As Single    '  4 server: strength = calib.*k(ist)/k(soll)
  Precision As Single       '  4 client: allowed deviation from linearity
  StrengthMin As Single     '  4 server: linear range of bump
  StrengthMax As Single     '  4 server: "
  Ablage As Single          '  4 client: in mm (fr jeden Typ, kann 0 sein)
  Winkel As Single          '  4 client: in mrad (nur fr Type="4")
                            '    Ablage UND Winkel knnen ungleich 0 sein)
  EnergyShift As Single     '  4 server
  Reserve2 As Single        '  4
                            '    = 15*8

  text(63) As Byte          ' 64
  Unit(15) As Byte          ' 16 server: units (mm, mrad, kHz ...)
  BumpName(31) As Byte      ' 32 client: name of bump (special bump)
  
  Calculate As Byte         '  1 client/server: calculate bump
                            '                   with actual optic ?
                            '                   "I" means calculate currents
  Scalable As Byte          '  1 server: are bump coeff. scalable
                            '            with energy?
                            '            "Y" or "N"
                            
                            '    3/4er Bumps
                            
  HV As Byte                '  1 client: "H" or "V" horizontal or vertical
  Auto As Byte              '  1 client: "Y" or "N",
                            '           "Y" =>Server bestimmt Korrektoren
  EpWw As Byte              '  1 client: "Y" or "N" (ep Wechselwirkung
                            '                bercksichtigen, nur HERA)
  UseIndex As Byte          '  1 client: ="Y" => use PoI_Name, else use Index
  Flag1 As Byte             '  1
  Flag2 As Byte             '  1
  PoIname(15) As Byte       ' 16 client/server: ( see useIndex )
                            '     Point of Interest = ele_name + pos
                            '     e.g. PETRA "QB53N123NOL  123"
                            '          PETRA "QC54N234 NL  234"
                            '          HERA  "QE20W111NL   123"
                            '     If Position Is omitted Then First
                            '     occurence of name is choosen
                            ' = 9*8

  CoilsNum As Integer       ' 2 server: # of circuits in bump
  PoI_MIndex As Integer     ' 2 client/server: mops Index of POI
    
  DataLength As Integer     ' 2 client
  BumpVersion As Integer    ' 2 client
                            '---------
                          ' 264 bytes

' Auswirkung einer Beule ist :
' eingestellte Koeff. / server Koeff. * strengthB * unit
End Type

Type RpcBumpDataType
  CoilName(15) As Byte   '  /* 16 client or set by BCS                         */
  Strength As Single     '  /*  4 normalised strength ( k, 1/r, kick ... )     */
  Current As Single      '  /*  4 absolut current calculated from dk2dI server */
  dK As Single           '  /*  4 from BCS, to dk2dI                           */
  dI As Single           '  /*  4 from dk2dI  server                           */
  dKmax As Single        '  /*  4 from BCS and dk2dI                           */
  dKmin As Single        '  /*  4 from BCS and dk2dI                           */
  CoilMopsIndex As Integer  '  /*  2 Index in MopsFile                            */
  Coil_CorMIndex As Integer '  /*  2    , vgl. CF_CHAR16FI                        */
  kIMinMax As Integer       '  /*  2  1 -> dkmin,dkmax; 2 -> dImin,dImax         */
  ErrorFlag As Integer      '  /*  2                                              */
                         ' /* --- */
                         ' /* 48  */
End Type

' to Server RPC Header + RPC Bump Header + e.g. 4 * RPC Bump Data

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' Datenstrukturen fr Applikationsprogramme

' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
' XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

' Fr die speicherplatzintensiven Lattice Arrays sind keine Typen definiert
' Es kann wegen der 64K Grenze keine Optikstruktur definiert werden,
' die das Lattice enthlt

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS"
' --------------------------------------------------------------------------------------------------------------------------
Type OpticHeaderType
    Machine As String * 8
    Year As String * 4
    name As String * 16
    Flag As String * 16             ' "THEORY", "VALID", "INVALID", "UNKNOWN"
    MopsText1 As String * 60
    MopsText2 As String * 60
    TableText1 As String * 60
    TableText2 As String * 60
    DateTime As String * 32
    fileName As String * 80
    MonNum(1) As Integer            ' # x,y monitors
    CorNum(1) As Integer            ' # x,y correctors
    LatNum As Integer               ' # lattice elements
    EleNum As Integer               ' # element types
    QTheory(1) As Single
    MomComp As Single
    Circumference As Double         ' [m]
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "OPTICLIST"
' --------------------------------------------------------------------------------------------------------------------------
Type OpticListType
    Machine As String * 8
    Year As String * 4
    NumOptics As Integer
    Optic(MaxOptics) As String * 16
    Description1(MaxOptics) As String * 60
    Description2(MaxOptics) As String * 60
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "LATTICE" (18.8.97)
' --------------------------------------------------------------------------------------------------------------------------

' Fr einzelne Lattice Elemente
Type LatticeElementType
    Position As String * 8  '16
    Drift As Double         ' 8
    Beta(1) As Single       ' 4
    Alpha(1) As Single      ' 4
    Phi(1) As Single        ' 4
    Disp(1) As Single       ' 4
    DispP(1) As Single      ' 4
    Pointer As Integer      ' 2 Pointer to ElementType
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "ELEMENTLIST" (5.8.97)
' --------------------------------------------------------------------------------------------------------------------------
Type ElementType
    TypeName As String * 8  '16
    Length As Double        ' 8
    Strength As Double      ' 8 Quads: k-Value, Dipols: 1/RHO-X/Z
    HX As Double            ' 8 1/RHO-X
    HZ As Double            ' 8 1/RHO-Z
    Angle As Double         ' 8 Tilt
    PetType As Integer      ' 2
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "OPTICS", Property "CMSTRUCT" (15.8.97)
' --------------------------------------------------------------------------------------------------------------------------
Type MonitorType
    name As String * 8
    Beta As Single
    Phi As Single
    Disp As Single
End Type

Type CorrectorType
    name As String * 8
    Beta As Single
    Phi As Single
    Disp As Single
    KMax As Single
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "ORBCOR"
' --------------------------------------------------------------------------------------------------------------------------
Type OrbitCorrectionType
    Machine As String * 8
    Year As String * 4
    user As String * 32             ' who saves orbits?
    Neff(1) As Integer
    KMaxScaled As String * 1        ' "Y" : KMax = KMax * MaxEnergy / Energy
    KMaxIgnore As String * 1        ' "Y" : KMax ignored
    KicksOrCurrents As String * 1   ' "K" = Kicks: Currents will be calculated by Server
                                    ' "C" = Currents: Kicks will be calculated by Server
    Overwrite As String * 1         ' "Y" : overwrite orbits allowed
    
    ' "POLCOR" (Gianfelice)
    FitQuality As Double
    ReferenceOrbit As String * 32
        
    ' "3BUMP" (Willeke)
    Totsch(2) As String * 8
    
    ' "ORBSCAN" (Brinker)
    MaxAblage(1) As Single
    CorNow(1) As Integer
    CorNext(1) As Integer
    MaxOrbits(1) As Integer
    Pattern As Integer
    
    PolFitOK As Boolean
    PolFit2OK As Boolean
    PolCorOK As Boolean
End Type

' --------------------------------------------------------------------------------------------------------------------------

Type OrbitType
    Loaded As Boolean               ' successfully loaded from server
    OpticLoaded As Boolean          ' successfully loaded from server
    OpticFound As Boolean           ' Optic of Orbit in Opticlist?
    name As String * 64 '* 32
    Machine As String * 8
    Year As String * 4
    Text1 As String * 60
    Text2 As String * 60
    DateTime As String * 32
    MachineFile As String * 32      ' from Controlsystem
    UserName As String * 32
    Optic As String * 16
    OpticFlag As String * 16
    QTheo(1) As Single              ' Theoretical Tunes for Plots/Phaseadvance
    QMeas(1) As Single              ' Measured Tunes for Info only
    Energy As Single                ' [Gev/c]
    Polarisation As Single          ' [%]
    IBunchE As Single               ' Leptons
    IBunchP As Single               ' Protons
    MonitorMode As String * 16
    Circumference As Single         ' Meters (for x-max)
    
    MonNum(1) As Integer
    CorNum(1) As Integer

    OrbRMS(1) As Single
    OrbMean(1) As Single
    OrbMinMax(1) As Single          ' z.B. -9.5

    CorRMS(1) As Single             ' Kicks
    CorMean(1) As Single
    CorMinMax(1) As Single
    
    ' Diese Arrays knnen "verdreht" sein, damit in Flugrichtung geplottet wird:

    EnergyKMax As Single
    KMax(CorDim, 1) As Single
    
    Orbit(MonDim, 1) As Single
    MonName(MonDim, 1) As String * 8
    MonFlags(MonDim, 1) As Integer
    MonPhi(MonDim, 1) As Single
    MonMeters(MonDim, 1) As Single
    
    Kicks(CorDim, 1) As Single
    Currents(CorDim, 1) As Single
    CorName(CorDim, 1) As String * 8
    CorFlags(CorDim, 1) As Integer
    CorPhi(CorDim, 1) As Single
    CorMeters(CorDim, 1) As Single
    
    ' Zusatzarrays
    ParaText(7) As String * 8
    Parameter(7) As Double
    
    ' "POLCOR" + "POLFIT"
    PolVal8(7) As Double
    PolStr8(7) As String * 8

    FirstTurn As Boolean
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function "ORBCOR", Property "ORBLIST"
' --------------------------------------------------------------------------------------------------------------------------
Type OrbitListType
    OrbNum As Integer       ' # of orbits
    OrbSel As Integer       ' # of selected orbit
    OrbType As Integer      ' iorb, icor, iref, also 0 .. 2
    OrbName(MaxOrbits) As String * 32
    OrbDatum(MaxOrbits) As String * 32
    OrbOptic(MaxOrbits) As String * 16
    OrbitMask As String * 32
    DateMask As String * 32
    OpticMask As String * 16
    Filter As Boolean
End Type

' --------------------------------------------------------------------------------------------------------------------------
' Equipment Function ORBCOR - Property "FINDKICK"
' --------------------------------------------------------------------------------------------------------------------------
Type PhaseRmsType
    Phase(NRMSvsPhase%) As Single
    RMS(NRMSvsPhase%) As Single
End Type

Type RpcFindKickType
    Hdr As RpcHdrType
    Hdr2 As RpcOrbcorHdrType
    Orbit(MonDim, 1) As Single
    Kicks(CorDim, 1) As Single
    Currents(CorDim, 1) As Single   ' F.Brinker
    MonFlags(MonDim, 1) As Integer
    CorFlags(CorDim, 1) As Integer
    
    PhaseRms As PhaseRmsType
End Type

