/* database.c */
#include "cprolog.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "database.h"
#include "servdef.h"
#include "srvip.h"
#include "rpctypes.h"
#include "errors.h"

static BYTE zero[6] = {0,0,0,0,0,0};
char ControlDBpath[80];
char gRPCDataBaseFullName[128];
char gFECDataBaseFullName[128];
int RPCNumRecsRead=0,RPCNumFecsRead=0,RPCNumEqpsRead=0;
int RPCallocNum = 0,FECallocNum=0;
int NameServerLoaded = 0;
extern RPCFecStruct *RPCFec, *NameServer;
extern int numNameServers;
int curNameServer = 0;
extern int isNameServerRequest;
RPCDataStruct *RPCData = NULL, NameServerData;
ConnTableEntry connNameServer;
int gUseLoopback = FALSE;

int fixFecProtocol(RPCFecStruct *fec)
{
  int defaultProtocol = 0;
# if defined(UNIX) && !defined(NO_TINE_PIPE)
  int myip = getmyipaddr();
# endif
# if defined(IPXNW) || defined(IPXBSD)
  int TransportTime = 0;
  if (memcmp(fec->Network,zero,4))
  {
    defaultProtocol = IPX;
#   ifdef IPXNW
    if (IPXGetLocalTarget(fec->Network,
        &fec->ImmediateAddress,
        &TransportTime)) return ipx_socket_error;
#   endif
  }
# endif
  if (memcmp(fec->IPh_addr,zero,4)) defaultProtocol = UDP;
# if defined(UNIX) && !defined(NO_TINE_PIPE)
  if (!gUseLoopback && (!memcmp(&myip,fec->IPh_addr,4) || !strncmp(fec->IP,LOOPBACKADDR,16)))
  {
    memset(fec->Network,0,4);
    defaultProtocol = PIPE;
  }
# endif
  fec->Prot = defaultProtocol;
  if (defaultProtocol)
  {
    if (NGdebug)
      dbglog("Communicating with %s via %s\n>",fec->Name,getProtocolAsString(defaultProtocol));
    return 0;
  }
  return illegal_protocol;
}
int locateTagName(char *expname,char *context,int *rpcidx)
{
  char fecname[FEC_NAME_SIZE];
  int i,cc,idx;
  if ((i=GetAddressFromNameServer(context,expname,fecname,&idx)) < 0 &&
      (i=GetAddressFromNameFile(context,expname,fecname,&idx)) < 0) return i;
  if ((cc=fixFecProtocol(&RPCFec[i])) != 0) return -cc;
  if (rpcidx != NULL) *rpcidx = idx;
  return i;
}
int locateFecName(char *fecname)
{
  int i,cc;
  if ((i=GetAddressFromNameServer(NULL,NULL,fecname,NULL)) < 0 &&
      (i=GetAddressFromNameFile(NULL,NULL,fecname,NULL)) < 0) return i;
  if ((cc=fixFecProtocol(&RPCFec[i])) != 0) return -cc;
  return i;
}

#define ToggleNameServer(txt) {\
  curNameServer = (curNameServer+1) % numNameServers;\
  memcpy(RPCFec,&NameServer[curNameServer],sizeof(RPCFecStruct));\
  strncpy(RPCData[0].FecName,NameServer[curNameServer].Name,FEC_NAME_SIZE);\
  feclog(txt);\
}

int fecNameId = -1;
char fecNameTarget[FEC_NAME_SIZE+4];
char tagNameTarget[ELEM_NAME_SIZE+4];
BYTE fecAddressBuf[sizeof(RPCDataStruct)+sizeof(RPCFecStruct)];
void fillinFecName(int id,int cc)
{
  int i;
  static int ens_to = 0;
  fecNameId = -1;             /* release semaphore */
  if (NGdebug > 1) dbglog("asyncLocateFec callback %d <%d>",id,cc);
  for (i=0; i<RPCNumFecsRead && strncmp(fecNameTarget,RPCFec[i].Name,FEC_NAME_SIZE); i++);
  if (i == RPCNumFecsRead)
  {
    if (RPCNumFecsRead < FECallocNum) RPCNumFecsRead++;
    else return;
  }
  if (cc == 0)
  {
    ens_to = 0;
    prepFecaddr(&RPCFec[i],fecAddressBuf);
    fixFecProtocol(&RPCFec[i]);
    feclog("assigned %s",fecNameTarget);
  }
  else
  {
    if (ens_to++ > 5) ToggleNameServer("ENS: toggle name server");
  }
  return;
}
void fillinTagName(int id,int cc)
{
  int i,j,k;
  static int ens_to = 0;
  char preCallExp[20],preCallEqm[8];
  fecNameId = -1;             /* release semaphore */
  if (NGdebug > 1) dbglog("asyncLocateTag callback %d <%d>",id,cc);
  for (j=0; j<RPCNumRecsRead && strncmp(tagNameTarget,RPCData[j].ExportName,ELEM_NAME_SIZE); j++);
  if (j == RPCNumRecsRead)
  {
    if (RPCNumRecsRead < RPCallocNum) RPCNumRecsRead++;
    else return;
  }
  if (cc == 0)
  {
    ens_to = 0;
    strncpy(preCallExp,RPCData[j].ExportName,16); preCallExp[16] = 0;
    strncpy(preCallEqm,RPCData[j].EqName,6); preCallEqm[6] = 0;
    prepEqpaddr(&RPCData[j],fecAddressBuf);
    if (strncmp(preCallExp,RPCData[j].ExportName,16) || strncmp(preCallEqm,RPCData[j].EqName,8))
    {
      feclog("ENS: name change in assignment %s %s vs %s %s\n",
        preCallExp,preCallEqm,RPCData[j].ExportName,RPCData[j].EqName);
    }
    for (i=0; i<RPCNumFecsRead && strcmp(RPCData[j].FecName,RPCFec[i].Name); i++);
    if (i == RPCNumFecsRead)
    {
      if (RPCNumFecsRead < FECallocNum) RPCNumFecsRead++;
      else return;
    }
    /* avoid allignment problems on some machines: */
    memcpy(fecAddressBuf,&fecAddressBuf[RPCDATASTRUCT_SIZE],RPCFECSTRUCT_SIZE);
    prepFecaddr(&RPCFec[i],fecAddressBuf);
    fixFecProtocol(&RPCFec[i]);
    for (k=0; k<nConnectionTableEntries; k++)
    {
      if (connTable[k]->RPCindex == j && !strncmp(connTable[k]->name,RPCData[j].ExportName,16))
      {
        connTable[k]->FECindex = i;  /* The device server got assigned to a new FEC ! */
      }
    }
    feclog("ENS: %s [FEC %s] added",tagNameTarget,RPCFec[i].Name);
  }
  else
  {
    if (ens_to++ > 5) ToggleNameServer("ENS: toggle name server");
  }
  return;
}
int asyncLocateFecName(char *fecname)
{
  int cc;
  DTYPE dout;
  char scratch[64];

  if (!NameServerLoaded || fecNameId != -1) return -not_ready;

  if (NGdebug > 1) dbglog("Locate FEC %s",fecname);
  sprintf(scratch,"%s/#1",CSHOSTS);
  strncpy(fecNameTarget,fecname,FEC_NAME_SIZE);
  dout.dFormat = CF_STRUCT;
  dout.dArrayLength = RPCFECSTRUCT_SIZE;
  memset(dout.dTag,0,8);
  dout.data.vptr = fecAddressBuf;
  if ((fecNameId=AttachLink(scratch,fecNameTarget,&dout,NULL,
       CA_READ,500,fillinFecName,CM_SINGLE)) < 0)
  {
    cc = -fecNameId;
    fecNameId = -1;
    return cc;
  }
  return fecNameId;
}
int asyncLocateTagName(char *expname,char *context)
{
  int cc;
  DTYPE dout;
  char scratch[64],cntxt[32] = "";

  if (!NameServerLoaded || fecNameId != -1) return -not_ready;

  if (NGdebug > 1) dbglog("Locate export %s",expname);
  strncpy(tagNameTarget,expname,ELEM_NAME_SIZE);

  if (context != NULL) strncpy(cntxt,context,EQP_CONTEXT_SIZE);
  if (strlen(cntxt) == 0) sprintf(cntxt,"#2");
  sprintf(scratch,"%s/%s",CSHOSTS,cntxt);
  dout.dFormat = CF_STRUCT;
  dout.dArrayLength = RPCFECSTRUCT_SIZE+RPCDATASTRUCT_SIZE;
  memset(dout.dTag,0,8);
  dout.data.vptr = fecAddressBuf;

  if ((fecNameId=AttachLink(scratch,expname,&dout,NULL,
       CA_READ,500,fillinTagName,CM_SINGLE)) < 0)
  {
    cc = -fecNameId;
    fecNameId = -1;
    return cc;
  }
  return fecNameId;
}
#ifndef MSDOS
int appendAddressToCache(char *fecname,char *ip, short portOffset)
{
  int i,cc = 0;
  DWORD haddr;

  initClient();
  for (i=0; i<RPCNumFecsRead && strcmp(fecname,RPCFec[i].Name); i++);
  if (i < RPCNumFecsRead && RPCFec[i].Prot) ccerr(0);
  strncpy(RPCFec[i].Name,fecname,FEC_NAME_SIZE);
  strncpy(RPCFec[i].IP,ip,16);
  haddr = inet_addr(RPCFec[i].IP);
  memcpy(RPCFec[i].IPh_addr,&haddr,4);
  RPCFec[i].PortOffset = (BYTE)portOffset;
  RPCFec[i].Port[0] = portOffset&0xff;
  RPCFec[i].Port[1] = (portOffset>>8)&0xff;
  fixFecProtocol(&RPCFec[i]);
  RPCNumFecsRead++;
err:
  return cc;
}
int appendServerToCache(char *fecname,char *context,char *expname, char *eqpname)
{
  int i,cc = 0;

  if (fecname == NULL || strlen(fecname) == 0) ccerr(argument_list_error);
  if (expname == NULL || strlen(expname) == 0) ccerr(argument_list_error);
  if (eqpname == NULL || strlen(eqpname) == 0) ccerr(argument_list_error);
  initClient();
  for (i=0; i<RPCNumRecsRead; i++)
  {
    if (context != NULL && strlen(context) &&
        strncmp(context,RPCData[i].Context,EQP_CONTEXT_SIZE)) continue;
    if (!strncmp(expname,RPCData[i].ExportName,ELEM_NAME_SIZE)) break;
  }
  if (i < RPCNumRecsRead) ccerr(0);
  strncpy(RPCData[i].FecName,fecname,FEC_NAME_SIZE);
  if (context != NULL) strncpy(RPCData[i].Context,context,EQP_CONTEXT_SIZE);
  strncpy(RPCData[i].ExportName,expname,ELEM_NAME_SIZE);
  strncpy(RPCData[i].EqName,eqpname,EQP_NAME_SIZE);
  RPCNumRecsRead++;
err:
  return cc;
}
#endif
int GetAddressFromNameServer(char *context,char *expname,char *fecname,int *rpcidx)
{
  int retry = 1;
  char scratch[64];
  int n,i=0,j,cc=0,isNew=FALSE;
  DTYPE dout;
  char cntxt[32] = "";

  if (rpcidx != NULL) *rpcidx = -1;
  if (!NameServerLoaded) return -not_ready;
  for (n=0; n<numNameServers; n++)
  {
retryRequest:
    if (expname == NULL)                  /* resolve fecname only */
    {
      if (fecname == NULL || strlen(fecname) == 0) ccerr(non_existent_fec);
      for (i=0; i<RPCNumFecsRead && strcmp(fecname,RPCFec[i].Name); i++);
      if (i < RPCNumFecsRead && RPCFec[i].Prot) ccerr(0);
      sprintf(scratch,"%s/#1",CSHOSTS);
      dout.dFormat = CF_STRUCT;
      dout.dArrayLength = RPCFECSTRUCT_SIZE;
      memset(dout.dTag,0,8);
      dout.data.vptr = fecAddressBuf;
      isNameServerRequest = TRUE;
      cc = tExecLink(0,(char *)fecAddressBuf,CSHOSTS,"#1",expname,(short)(RPCFECSTRUCT_SIZE),CF_STRUCT,500);
      isNameServerRequest = FALSE;
      if (cc) ccerr(cc);
      if (i == RPCNumFecsRead)
      {
        if (RPCNumFecsRead < FECallocNum) 
        {
          RPCNumFecsRead++;
          isNew = TRUE;
        }
        else ccerr(buffer_too_small);
      }
      prepFecaddr(&RPCFec[i],fecAddressBuf);
      BytesToHex(RPCFec[i].Node,scratch,12);
      if (isNew) feclog("ENS: FEC %s [%s %s] added",RPCFec[i].Name,scratch,RPCFec[i].IP);
    }
    else                               /* resolve both tagname and fecname */
    {
      fecname[0] = 0;
      if (strlen(expname) == 0) ccerr(non_existent_elem);
      for (j=0; j<RPCNumRecsRead; j++)
      {
        if (context != NULL && strlen(context) &&
            strncmp(context,RPCData[j].Context,EQP_CONTEXT_SIZE)) continue;
        if (!strncmp(expname,RPCData[j].ExportName,ELEM_NAME_SIZE)) break;
      }
      if (j < RPCNumRecsRead)
      {
        for (i=0; i<RPCNumFecsRead && strncmp(RPCData[j].FecName,RPCFec[i].Name,FEC_NAME_SIZE); i++);
        if (rpcidx != NULL) *rpcidx = j;
        if (i < RPCNumFecsRead && RPCFec[i].Prot) ccerr(0);
      }
      if (context != NULL) strncpy(cntxt,context,EQP_CONTEXT_SIZE);
      if (strlen(cntxt) == 0) sprintf(cntxt,"#2");
      sprintf(scratch,"%s/%s",CSHOSTS,cntxt);
      dout.dFormat = CF_STRUCT;
      dout.dArrayLength = RPCFECSTRUCT_SIZE+RPCDATASTRUCT_SIZE;
      memset(dout.dTag,0,8);
      dout.data.vptr = fecAddressBuf;
      isNameServerRequest = TRUE;
      cc = tExecLink(0,(char *)fecAddressBuf,CSHOSTS,cntxt,expname,(short)(RPCFECSTRUCT_SIZE+RPCDATASTRUCT_SIZE),CF_STRUCT,500);
      isNameServerRequest = FALSE;
      if (cc) ccerr(cc);
      if (j == RPCNumRecsRead)
      {
        if (RPCNumRecsRead < RPCallocNum) 
        {
          RPCNumRecsRead++;
          isNew = TRUE;
        }
        else ccerr(buffer_too_small);
      }
      prepEqpaddr(&RPCData[j],fecAddressBuf);
      if (rpcidx != NULL) *rpcidx = j;
      for (i=0; i<RPCNumFecsRead && strncmp(RPCData[j].FecName,RPCFec[i].Name,FEC_NAME_SIZE); i++);
      if (i == RPCNumFecsRead)
      {
        if (RPCNumFecsRead < FECallocNum) 
        {
          RPCNumFecsRead++;
          isNew = TRUE;
        }
        else ccerr(buffer_too_small);
      }
      /* avoid alignment problems on some machines: */
      memcpy(fecAddressBuf,&fecAddressBuf[RPCDATASTRUCT_SIZE],RPCFECSTRUCT_SIZE);
      if (i > 0) prepFecaddr(&RPCFec[i],fecAddressBuf);
      strncpy(fecname,RPCFec[i].Name,FEC_NAME_SIZE);
      if (isNew) feclog("ENS: %s [FEC %s, EQP %s] added",RPCData[j].ExportName,RPCData[j].FecName,RPCData[j].EqName);
    }
err:
    if ((cc == link_not_open || cc == connection_timeout) && retry-- > 0)
      goto retryRequest;
    if (cc == 0) break;                      /* finished */
    feclog("ENS: %s %s : %s",
	    expname != NULL ? expname : "",fecname != NULL ? fecname : "",erlst[cc]);
    ToggleNameServer("ENS: toggle name server");
  }
  return cc ? -cc : i;                      /* return FEC index */
}

int GetAddressFromNameFile(char *context,char *expname,char *fecname,int *rpcidx)
{
  FILE *fp = NULL;
  DWORD haddr;
  int i = -1,cc = 0,done = 0,col=-1;
  char *c,str[256],hdr[256],scratch[16];
  UINT16 poff;

  if (rpcidx != NULL) *rpcidx = -1;
  if (fecname == NULL) ccerr(non_existent_fec);
  if (expname == NULL) goto acquireFecName;     /* only interested in FEC address*/
  if (strlen(expname) == 0) ccerr(non_existent_elem);

  if ((fp=fopen(gRPCDataBaseFullName,"rt")) == NULL) ccerr(no_such_file);
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((col=findcol(hdr,"NAME")) < 0) ccerr(no_such_column);
      done = 1;
      continue;
    }
    if (!namcmp(expname,str,col,ELEM_NAME_SIZE))            /* found entry in file */
    {
      for (i=0; i<RPCNumRecsRead && strncmp(expname,RPCData[i].ExportName,ELEM_NAME_SIZE); i++);
      if (i == RPCNumRecsRead)
      {
        if (RPCNumRecsRead < RPCallocNum) RPCNumRecsRead++;
        else ccerr(buffer_too_small);
      }
      strncpy(RPCData[i].ExportName,colptr(col,str),ELEM_NAME_SIZE);
      if ((col=findcol(hdr,"FECNAME")) < 0) ccerr(no_such_column);
      strncpy(RPCData[i].FecName,colptr(col,str),FEC_NAME_SIZE);
      strncpy(fecname,RPCData[i].FecName,FEC_NAME_SIZE);
      if ((col=findcol(hdr,"EQPMODULE")) < 0) ccerr(no_such_column);
      strncpy(RPCData[i].EqName,colptr(col,str),EQP_NAME_SIZE);
      if (context != NULL && strlen(context) > 0 &&
          (col=findcol(hdr,"CONTEXT")) >= 0)
      {
        c = colptr(col,str);
        if (strncmp(context,c,EQP_CONTEXT_SIZE)) continue;
        strncpy(RPCData[i].Context,c,EQP_CONTEXT_SIZE);
      }
      else RPCData[i].Context[0] = 0;
      feclog("EFL: %s [FEC %s, EQP %s] added",
	      RPCData[i].ExportName,RPCData[i].FecName,RPCData[i].EqName);
      if (rpcidx != NULL) *rpcidx = i;
      break;
    }
  }
  fclose(fp); fp = NULL;

acquireFecName:
  if (strlen(fecname) == 0) ccerr(non_existent_fec);
  if ((fp=fopen(gFECDataBaseFullName,"r")) == NULL) ccerr(no_such_file);
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((col=findcol(hdr,"FECNAME")) < 0) ccerr(no_such_column);
      done = 1;
      continue;
    }
    if (!namcmp(fecname,str,col,FEC_NAME_SIZE))           /* found entry in file */
    {
      for (i=0; i<RPCNumFecsRead && strncmp(fecname,RPCFec[i].Name,FEC_NAME_SIZE); i++);
      if (i == RPCNumFecsRead)
      {
        if (RPCNumFecsRead < FECallocNum) RPCNumFecsRead++;
        else ccerr(buffer_too_small);
      }
      strncpy(RPCFec[i].Name,colptr(col,str),FEC_NAME_SIZE);
      if ((col=findcol(hdr,"FECNETWORK")) < 0) ccerr(no_such_column);
      strncpy(scratch,colptr(col,str),16);
      HextoBytes(scratch,RPCFec[i].Network,8);
      if ((col=findcol(hdr,"FECNODE")) < 0) ccerr(no_such_column);
      strncpy(scratch,colptr(col,str),16);
      HextoBytes(scratch,RPCFec[i].Node,12);
      if ((col=findcol(hdr,"IPADDR")) < 0) ccerr(no_such_column);
      strncpy(RPCFec[i].IP,colptr(col,str),16);
      haddr = inet_addr(RPCFec[i].IP);
      memcpy(RPCFec[i].IPh_addr,&haddr,4);
      feclog("EFL: FEC %s [%s %s] added",RPCFec[i].Name,scratch,RPCFec[i].IP);
      if ((col=findcol(hdr,"PORTOFFSET")) >= 0)
      {
        strncpy(scratch,colptr(col,str),16);
        poff = atoi(scratch);
        RPCFec[i].Port[0] = poff&0xff;
        RPCFec[i].Port[1] = (poff>>8)&0xff;
        RPCFec[i].PortOffset = (BYTE)poff;
      }
      else RPCFec[i].PortOffset = 0;
      break;
    }
  }
  if (i < 0) cc = non_existent;
err:
  if (fp) fclose(fp);
  if (cc)
  {
    if (expname == NULL) expname = "(null)";
    if (fecname == NULL) fecname = "(null)";
    feclog("EFL: %s %s - %s",expname,fecname,erlst[cc]);
  }
  return cc ? -cc : i;
}
int dbaseloaded = 0;
int MaxRPCAllocation = -1;
int initFecTable(void)
{
  char *ptr;
  int i,cc = 0;

  if (dbaseloaded) return 0;

  ControlDBpath[0] = 0;
  if ((ptr=getenv("TINE.HOME")) != NULL ||
      (ptr=getenv("CONTROLDB")) != NULL)
  {
    strncpy(ControlDBpath,ptr,80);
#   ifdef FS_DELIMITER
    if (((i=strlen(ControlDBpath)) > 0) &&
        (ControlDBpath[i-1] != FS_DELIMITER))  /* no delimiter ! */
    {
      ControlDBpath[i] = FS_DELIMITER; ControlDBpath[i+1] = 0;
    }
#   endif
  }
  sprintf(gRPCDataBaseFullName,"%s%s",ControlDBpath,EQPDB_FILE);
  sprintf(gFECDataBaseFullName,"%s%s",ControlDBpath,FECDB_FILE);

  if (GetAllocSizesFromNameServer(&RPCallocNum,&FECallocNum) &&
      GetAllocSizesFromNameFile(&RPCallocNum,&FECallocNum))
    ccerr(database_not_loaded);

  if (!RPCallocNum || !FECallocNum) ccerr(un_allocated);
  if (MaxRPCAllocation > 0) RPCallocNum = FECallocNum = MaxRPCAllocation;
  feclog("Reserved %d client connection link entries",RPCallocNum);
  feclog("Reserved %d client connection addr entries",FECallocNum);

  if (!(RPCData=(RPCDataStruct *)SystemCalloc(RPCallocNum,sizeof(RPCDataStruct))))
    ccerr(out_of_client_memory);

  if (!(RPCFec=(RPCFecStruct *)SystemCalloc(FECallocNum,sizeof(RPCFecStruct))))
    ccerr(out_of_client_memory);

  if (NameServerLoaded)  /* put NameServer into the tables for future calls*/
  {
    memcpy(RPCData,&NameServerData,sizeof(RPCDataStruct));
    memcpy(RPCFec,&NameServer[0],sizeof(RPCFecStruct));
    connTable[0] = &connNameServer;
  }
  dbaseloaded = 1;
err:
  if (cc) nConnectionTableEntries = 0;
  return cc;
}
int GetAllocSizesFromNameServer(int *RPCsize,int *FECsize)
{
  int cc=0;

  if (!NameServerLoaded)
  {
    if ((cc=GetNameServerAddress()) != 0) ccerr(cc);
    strncpy(NameServerData.FecName,NameServer[0].Name,FEC_NAME_SIZE);
    strncpy(NameServerData.ExportName,CSHOSTS,ELEM_NAME_SIZE);
    strncpy(NameServerData.EqName,CSHOSTS,6);
    RPCFec = &NameServer[0]; RPCData = &NameServerData;
    RPCNumRecsRead = RPCNumFecsRead = 1;  /* Hardwire first call */
    strncpy(connNameServer.name,CSHOSTS,ELEM_NAME_SIZE);
    strncpy(connNameServer.deviceName,"#0",EQP_DEVICE_SIZE);
    strncpy(connNameServer.prop,"SIZES",EQP_PROP_SIZE);
    connNameServer.EqNo = 0;
    connNameServer.formatOut = CF_SHORT;
    connNameServer.formatIn = CF_NULL;
    connNameServer.RPCindex = 0;
    connNameServer.sizeOut = 2;
    connNameServer.sizeIn = 0;
    connNameServer.RPCprotocol = 5;
    connNameServer.mode = REGISTER;
    connNameServer.blknumIn = 1;
    connNameServer.numblksIn = 1;
    connNameServer.nextDataPtr = 0;
    nConnectionTableEntries = 1;
    NameServerLoaded = 1;
  }
  *RPCsize =  *FECsize = MaxNumConnections;
err:
  return cc;
}

int GetAllocSizesFromNameFile(int *RPCsize,int *FECsize)
{
  int size;
  char str[256];

  sprintf(str,"Reading %s : ",gRPCDataBaseFullName);
  if ((size=GetDataBaseSize(gRPCDataBaseFullName))<0)
  {
    sprintf(&str[strlen(str)],"%s",erlst[-size]);
    return -size;
  }
  sprintf(&str[strlen(str)],"%d lines",size);
  feclog(str);
  if (size == 0) return un_allocated;
  *RPCsize = size;

  sprintf(str,"Reading %s : ",gFECDataBaseFullName);
  if ((size=GetDataBaseSize(gFECDataBaseFullName))<0)
  {
    sprintf(&str[strlen(str)],"%s",erlst[-size]);
    return -size;
  }
  sprintf(&str[strlen(str)],"%d lines",size);
  feclog(str);
  if (size == 0) return un_allocated;
  *FECsize = size;

  return 0;
}
