/** \file dblib.c
 *
 *      general database routines:
 *
 */
#ifndef NO_DOXYGEN
#include "prolog.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "servdef.h"
#include "database.h"
#include "errors.h"

#define LOINT(l)       ((int)(l))
#define HIINT(l)       ((int)(((long)(l) >> 16) & 0xFFFF))

unsigned int HexToDec(char *str);
void BytesToHex(BYTE *b,char *s,int n)
{
  int v;

  while (n--)
  {
    *s++ = (v = (*b&0xf0)>>4) < 10 ? '0' + v : 'A' + v - 10;
    *s++ = (v = *b++&0xf) < 10 ? '0' + v : 'A' + v - 10;
  }
  *s = 0;
  strrev(s);
  return;
}
void HextoBytes(char *s,BYTE *b,int nibs)
{
  char buf[20];
  int l,i;

  if ((l=strlen(s)) < nibs)
  {
    strrev(s); strncat(s,"000000000000",nibs-l);
    strrev(s);
  }
  for (i=0; i<nibs/2; i++)
  {
    strncpy(buf,&s[i*2],2); buf[2] = 0;
    b[i] = (unsigned char)strtol(buf,NULL,16);
  }
}
unsigned int HexToDec(char *str)
{
  int i,d;
  unsigned int off,value;

  off=1;value=0;
  for (i=strlen(str)-1;i>=0;i--)
  {
    if (str[i]>='A') d=str[i]-'A'+10;else d=str[i]-'0';
    value=value+off*d;
    off=off*16;
  }
  return value;
}
char *strtrm(char *s)
{
  char str[516],*c=str;
  unsigned int len;
  int done0=0,done1=0;
  if (s == NULL || (len=strlen(s)) == 0 || len > 512) return s;
  if (s[len-1] == '\n') s[len-1] = 0;
  strncpy(c,s,512); c[512] = 0;
  for (len=strlen(c); len > 0 && len < 512; len=strlen(c))
  {
    if (c[len-1] == ' ' || c[len-1] == '\t' || c[len-1] == '\n') c[len-1] = 0; else done1 = TRUE;
    if (c[0] == ' ' || c[0] == '\t') c++; else done0 = TRUE;
    if (done0 && done1) break;
  }
  if (strlen(s) != len) 
  {
    if (len > 0) 
    {
      strncpy(s,c,len);
      s[len] = 0;
    }
    else s[0] = 0;
  }
  return s;
}
int GetDataBaseSize(char *filename)
{
  char str[256];
  int line = -1;
  FILE *fp;

  if ((fp=fopen(filename,"r")) == NULL) return -no_such_file;
  while (fgets(str,255,fp) != NULL)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#",str[0])) continue;      /* comment line(s) */
    line++;
  }
  fclose(fp);
  return line;
}
int csvNameServerFillPortOffset(char *po);
int csvNameServerFillHAddr(char *ip);
RPCFecStruct *RPCFec = NULL,*NameServer = NULL;
int numNameServers = 0;
CSVCOLUMN csvNameServerColumns[] =
{
  { "FECNAME","",0,CF_TEXT,FEC_NAME_SIZE,(BYTE *)OFFSETIN(RPCFecStruct,Name),NULL,0 },
  { "FECNETWORK","0",0,CF_BYTE,8,(BYTE *)OFFSETIN(RPCFecStruct,Network),NULL,0 },
  { "FECNODE","0",0,CF_BYTE,12,(BYTE *)OFFSETIN(RPCFecStruct,Node),NULL,0 },
  { "PORT_OFFSET","0",0,CF_NULL,1,NULL,csvNameServerFillPortOffset,0 },
  { "IPADDR","",0,CF_TEXT,16,(BYTE *)OFFSETIN(RPCFecStruct,IP),csvNameServerFillHAddr,0 },
};
CSVDB csvNameServerDb =
{
  csvNameServerColumns,
  sizeof(csvNameServerColumns)/sizeof(csvNameServerColumns[0]),
  0,
  csvStdInit,
  csvStdIterator,
  csvStdExit,
  0,
  NULL,
  sizeof(RPCFecStruct)
};
int csvNameServerFillHAddr(char *ip)
{
  char str1[32],str2[32];
  DWORD haddr = inet_addr(ip);
  memcpy(NameServer[csvNameServerDb.cnt].IPh_addr,&haddr,4);
  if (NGdebug)
  {
    BytesToHex(NameServer[csvNameServerDb.cnt].Network,str1,4);
    BytesToHex(NameServer[csvNameServerDb.cnt].Node,str2,6);
    dbglog("Using Name Server %s [%s %s %s]",
            NameServer[csvNameServerDb.cnt].Name,
            str1,str2,
            NameServer[csvNameServerDb.cnt].IP);
  }
  return 0;
}
int csvNameServerFillPortOffset(char *po)
{
  int poff = atoi(po);
  NameServer[csvNameServerDb.cnt].Port[0] = poff&0xff;
  NameServer[csvNameServerDb.cnt].Port[1] = (poff>>8)&0xff;
  NameServer[csvNameServerDb.cnt].PortOffset = (BYTE)poff;
  return 0;
}
int gotNameServerAddress = 0;
int GetNameServerAddress(void)
{
  int cc;
  short tmpPortOffset;
  char *ptr;

  if ((ptr=getenv("TINE.ENS")) != NULL || (ptr=getenv("CONTROLENS")) != NULL) 
    SetNameServerAddress(ptr);
  if (gotNameServerAddress) return 0;
  ControlDBpath[0] = 0;
  if ((ptr=getenv("TINE.HOME")) != NULL || (ptr=getenv("CONTROLDB")) != NULL) 
    strcpy(ControlDBpath,ptr);
  feclog("TINE HOME : [%s]",ControlDBpath);
  numNameServers = 0;
  csvNameServerColumns[3].ptr = (BYTE *)&tmpPortOffset;
  if ((cc=csvReadFile(ControlDBpath,CSHOSTS_FILE,&csvNameServerDb,(void **)&NameServer)) == 0)
  {
    gotNameServerAddress = 1;
    numNameServers = csvNameServerDb.siz;
  }
  else
  {
    feclog("name server address unresolved : %s",erlst[cc]);
  }
  return cc;
}
int findcol(char *str,char *tag)
{
  char *c;
  int len,i;

  for (i=0,c=str; strlen(c); i++,c++)
  {
    while (*c == '\"') { c = strchr(++c,'\"'); if (c++ == NULL) return -1; }
    while (*c == ' ') c++;
    strupr(c);
    len = strcspn(c,",\n"); while (c[len-1] == ' ') len--;
    if (!strncmp(c,tag,MAX(len,(int)strlen(tag)))) return i;
    if ((c=strpbrk(c,",\n")) == NULL) return -1;
  }
  return -1;
}
char *colptr(int col,char *str)
{
  static char buf[128];
  char *c;
  int i,sslen;

  for (i=0,c=str; i < col && strlen(c); i++,c++)
  {
    while (*c == '\"')
    {
      c = strchr(++c,'\"');
      if (c++ == NULL)
      {
        sprintf(buf,"%s : missing quote in column %d\n",str,col);
        feclog(buf); printf(buf);
      }
    }
    if ((c=strpbrk(c,",")) == NULL)
    {
      sprintf(buf,"%s : missing column %d\n",str,col);
      feclog(buf); printf(buf);
    }
  }
  while (*c == ' ') c++;
  if (*c == '\"') sslen = strcspn(++c,"\"");
  else sslen = strcspn(c,",\n");
  if (sslen > 126)
  {
    sprintf(buf,"sub string \"%s\" too long (%d vs 126)\n",c,sslen);
    feclog(buf); printf(buf);
  }
  while (sslen > 0 && c[sslen-1] == ' ') sslen--;
  strncpy(buf,c,i=MIN((int)sizeof(buf)-1,sslen));
  buf[i] = 0;
  return buf;
}
int namcmp(char *name,char *str,int col,int limit)
{
  char *c = colptr(col,str);
  int n = MAX(limit,(int)strcspn(c,",\n"));
  return strncmp(name,c,n);
}
void csvStdInit(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return;
  db->cnt = 0;
  if (db->tgt != NULL) SystemFree(db->tgt);
  db->tgt = (BYTE *)SystemCalloc(db->siz,db->tgt_siz);
  for (i=0; i<db->ncols; i++) 
  {
    db->col[i].start = db->col[i].ptr;
    db->col[i].ptr = db->tgt + (UINT32)db->col[i].start;
  }
}
int csvStdIterator(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return -1;  /* don't iterate */
  for (i=0; i<db->ncols; i++) if (db->col[i].fmt != CF_NULL) db->col[i].ptr += db->tgt_siz;
  db->cnt++;
  return 0;
}
void csvStdExit(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return;
  for (i=0; i<db->ncols; i++) db->col[i].ptr = db->col[i].start;
}
int csvAssignValue(void *ptr,int fmt,int siz,char *str)
{
  if (ptr == NULL) return 0;         /* intentionally not assigned */
  if (str == NULL) return argument_list_error;
  if (fmt != CF_TEXT && fmt != CF_NULL && strlen(str) == 0) return argument_list_error;
  switch (fmt)
  {
    case CF_NULL: break;
    case CF_TEXT: if (siz == 0) *(char *)ptr = 0; else strncpy((char *)ptr,str,siz); break;
    case CF_BYTE: 
      if (siz > 1) HextoBytes(str,(BYTE *)ptr,siz); 
      else *(BYTE *)ptr = (BYTE)atoi(str);
      break;
    case CF_SHORT: *(short *)ptr = (short)strtol(str,NULL,0); break;
    case CF_LONG:  *(long *)ptr = strtol(str,NULL,0); break;
    case CF_FLOAT: *(float *)ptr = (float)strtod(str,NULL); break;
    case CF_DOUBLE: *(double *)ptr = strtod(str,NULL); break;
    default: return illegal_format;
  }
  return 0;
}
int csvReadFile(char *path,char *file,CSVDB *db,void **target)
{
  FILE *fp=NULL;
  int i,cc=0,done=0;
  char *c,str[256],hdr[256],scratch[80],fn[128];

  sprintf(fn,"%s%s",path,file);
  /* find database size */
  if ((db->siz=GetDataBaseSize(fn)) < 0) ccerr(-db->siz);
  if (target != NULL) db->tgt = (BYTE *)*target;
  /* call init routine if there */
  if (db->ini != NULL) db->ini(db);
  if (target != NULL) *target = db->tgt;
  /* read through */
  if ((fp=fopen(fn,"r")) == NULL) ccerr(no_such_file);
  while (fgets(str,255,fp) != NULL)
  {
    strtrm(str);
    if (strlen(str) == 0) continue;               /* carraige return */
    if (strchr("%;#\n",str[0]) != NULL) continue; /* comment line */
    if (!done)
    {
      strncpy(hdr,str,255);
      for (i=0; i<db->ncols; i++) db->col[i].pos = findcol(hdr,db->col[i].nam);
      done = TRUE;
      continue;
    }
    for (i=0; i<db->ncols; i++)
    {
      if (db->col[i].pos < 0) 
      {
        c = db->col[i].def;
      }
      else
      {
        strncpy(scratch,colptr(db->col[i].pos,str),64); scratch[64] = 0;
        c = scratch;
      }
      /* assign value to structure location */
      cc = csvAssignValue(db->col[i].ptr,db->col[i].fmt,db->col[i].siz,c);
      if (cc)
      {
        sprintf(str,"column %s entry %d : %s\n",db->col[i].nam,i,erlst[cc]);
        feclog(str); if (StartupDebug) printf("%s\n>",str);
        ccerr(cc);
      }
      /* call action function if there */
      if (db->col[i].fcn != NULL) if (db->col[i].fcn(c) != 0) break;    
    }
    /* call iterator if there */
    if (db->itr != NULL) if (db->itr(db) != 0) break;
  }
  /* call end routine if there */
  if (db->exi != NULL) db->exi(db);
err:
  if (fp) fclose(fp);
  return cc;
}
int csvGetValue(void *ptr,int fmt,int siz,char *str)
{
  if (ptr == NULL) return 0;         /* intentionally not assigned */
  if (str == NULL) return argument_list_error;
  switch (fmt)
  {
    case CF_TEXT: strncpy(str,(char *)ptr,siz); break;
    case CF_BYTE: BytesToHex((BYTE *)ptr,str,siz); break;
    case CF_SHORT: sprintf(str,"%d",*(short *)ptr); break;
    case CF_LONG:  sprintf(str,"%ld",*(long *)ptr); break;
    case CF_FLOAT: sprintf(str,"%g",*(float *)ptr); break;
    case CF_DOUBLE: sprintf(str,"%g",*(double *)ptr); break;
    default: return illegal_format;
  }
  return 0;
}
int csvWriteFile(char *path,char *file,CSVDB *db)
{
  FILE *fp=NULL;
  int i,j,cc=0;
  char str[256],scratch[64],fn[128];

  sprintf(fn,"%s%s",path,file);
  if ((fp=fopen(fn,"w")) == NULL) ccerr(file_error);
  for (i=0,str[0]=0; i<db->ncols; i++)
  {
    strcat(str,db->col[i].nam);
    if (i<db->ncols-1) strcat(str,","); 
    if (db->tgt_siz > 0)  /* return pointers to start position */
    {
      db->col[i].ptr = db->tgt + (UINT32)db->col[i].start;
    }
  }
  strcat(str,"\n");
  fwrite(str,strlen(str),1,fp);
  for (j=0; j<db->siz; j++)
  {
    str[0] = 0;
    for (i=0; i<db->ncols; i++)
    {
      csvGetValue(db->col[i].ptr,db->col[i].fmt,db->col[i].siz,scratch);
      strcat(str,scratch);
      if (i<db->ncols-1) strcat(str,","); 
    }
    strcat(str,"\n");
    fwrite(str,strlen(str),1,fp);
    /* call iterator if there */
    if (db->itr != NULL) if (db->itr(db) != 0) break;
  }
  /* call end routine if there */
  if (db->exi != NULL) db->exi(db);
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
#endif /* NO_DOXYGEN */
