/** \file server.c
 * \brief TINE server engine plus API routines
 *
 */
#include "prolog.h"
#include "cprolog.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "servdef.h"
#include "threader.h"
#include "brdcast.h"
#include "errors.h"
#include "database.h"
#include "almsrv.h"
#include "history.h"
#include "srvip.h"
#include "srvipx.h"
#if defined(UNIX) && !defined(NOREMOTE)
# define UNIX_TTY
# include "unixcmd.h"
#endif

int BroadcastServer = UDP_BROADCAST;
int nbroadcasts = 0, nglobals = 0;
int longDeviceNames = FALSE;
int gBurstLimit = BURST_LIMIT;
int gCycleDelay = CYCLE_DELAY;
int SystemPresetMemory = TRUE;

int RPCprotocol;
#ifndef srvWorkArea
BYTE *srvWorkArea;
unsigned long srvWorkAreaSize = WORK_SIZE;
#else
BYTE *WorkArea;
unsigned long WorkAreaSize = WORK_SIZE;
#endif
BYTE *tmpWorkArea = NULL;
unsigned long tmpWorkAreaSize = WORK_SIZE;

int getRegisteredExports(void);
void SurveyAlarms(struct timeval *tv);
void doBroadcast(struct timeval *thisTime);
void DataDump(char *txt,char *data,int siz,short fmt,short dir);
int datacmp(BYTE *d1,BYTE *d2,int siz,int fmt,double t);
void CloseLink(int i);
void CleanupStrandedDatagrams(void);
void _SystemEngine(void);

int gEqpFcnBusySemaphore = 0;
ContractListStruct *ContractListItem = NULL;
char *SRVDataBuffer;          /* global Buffer pointers for data exchange */
long SysPoll = DEFAULT_POLLRATE;
unsigned short MinPollingRate = MIN_POLLRATE;
unsigned short MaxPollingRate = DEFAULT_POLLRATE;
unsigned long MaxRPCTransportSize = MAX_TRANSPORTSIZE;
int gBrdcstBufferSize = BRDCST_BUFFER_SIZE; 
int AveBusyTime = 0;
struct timeval thisCycleTime;
int RequireAcknowledgments = 1; /* global flag to force acknowledge in REFRESH mode */
short NrEQPBckTsks = 0;
int nExpiredContracts = 0;

AccessLockStruct currentAccessLock;
GlobalListStruct **BrdcstList = NULL;
IPXAddress IPXFecAddress;
int ExportsRegistered = 0;
int FecNameRegistered = 0;
RPCFecExStruct RPCFecEx;

int SystemRunning = 0;
int gRetardSingleContractRemoval = TRUE;
long findpoll(long p1,long p2)
{
  long p3;

  if (MinPollingRate > MaxPollingRate) MinPollingRate = MaxPollingRate;
  if (p1 < MinPollingRate) return p2 <= MinPollingRate ? MinPollingRate : p2;
  if (p2 < MinPollingRate || p1 == p2) return p1;
  if (p2 < p1) while ((p3 = p1%p2) >= MinPollingRate) { p1 = p2; p2=p3; }
  else if (p1 < p2) while ((p3 = p2%p1) >= MinPollingRate) { p2 = p1; p1=p3; }
  return p1 > p2 ? p2 : p1;
}
int SetBurstLimit(int npackets)
{
  if (npackets < 1) npackets = 1;
  gBurstLimit = npackets;
  return gBurstLimit;
}
int SetCycleDelay(int msecs)
{
  if (msecs < 0) msecs = 0;
  gCycleDelay = msecs;
  return gCycleDelay;
}
char RPCLastErrorString[RPCERR_SIZE];
ExportListStruct *ExportList = NULL;
int NrOfEqpTypes = 0;

void InitializeEQPModules(void)
{
  ExportListStruct *el;

  for (el=ExportList; el != NULL; el=el->next)
    if (el->eqpIni) el->eqpIni();
  return;
}
void devNamesInitialization(void)
{
  ExportListStruct *el;
  ExportPropertyListStruct *prp;
  int plen, idx;
  char prpName[64];

  for (el=ExportList; el != NULL; el=el->next)
  {
    if (el->hasPropertyDeviceNames)
    {
      for (idx=0; idx<PROPERTY_HASH_SIZE; idx++)
        for (prp=el->EqpProperty[idx]; prp; prp=prp->next)
      {
        if ((plen=strlen(prp->prpName)) < 5) continue;
        if (strcmp(&prp->prpName[plen-4],".NAM")) continue;
        strncpy(prpName,prp->prpName,plen-4); prpName[plen-4] = 0;
        GetDeviceListFromProperty(el->EqpName,prpName);
      }
    }
  }
  acquireDeviceNamesFromProperties = FALSE;
}
int LastMinuteInitialization(void)
{
  ExportListStruct *el;
  ExportPropertyListStruct *prp;
  int idx;

  InitializeEQPModules();

  if (StartupDebug)
  {
    printf("Exporting the following properties:\n>");
    for (el=ExportList; el != NULL; el=el->next)
      for (idx=0; idx<PROPERTY_HASH_SIZE+1; idx++)
        for (prp=el->EqpProperty[idx]; prp; prp=prp->next)
    {
      printf("%s (%s):  %s (%s) -> %s <-\n>",el->tagName,
             el->EqpName,prp->prpName,
             itom(prp->prpAccessMode),prp->prpDescription);
    }
  }
  SystemRunning = 1;
  return 1;  /* init done */
}
void ExitEQPModules(void)
{
  ExportListStruct *el;

  for (el=ExportList; el != NULL; el=el->next)
  {
# ifdef MULTITHREADED
    if (el->hEqpBkg != 0) threadExit(el->hEqpBkg);
    if (hEqpFcnThread != 0) threadExit(hEqpFcnThread);
# endif
    if (el->eqpExi) el->eqpExi();  /* call exit routines */
  }
  return;
}
void AssignDotask(void (*tsk)(void)) /* for backward compatibility of NODAL FECs */
{
  if (ExportList == NULL) return;
  ExportList->eqpBkg  = tsk;
  ExportList->rateBkg = 0;
}
void AssignExittask(void (*tsk)(void)) /* for backward compatibility of NODAL FECs */
{
  if (ExportList == NULL) return;
  ExportList->eqpExi  = tsk;
}
void RegisterEQPtasks(char *name,void (*ini)(void),void (*tsk)(void),int rate,void (*exi)(void))
{
  ExportListStruct *el;

  if ((el=getExportListItem(name)) == NULL) return;

  if ((el->eqpBkg=tsk) != NULL) NrEQPBckTsks++;
  el->rateBkg = rate;
  if (ServerCycleMode != POLLING && rate < MaxPollingRate)
  {
    SysPoll = rate;
    MaxPollingRate = rate;
    if (rate < MinPollingRate) MinPollingRate = rate;
  }
  el->eqpIni = ini;
  el->eqpExi = exi;

  if (StartupDebug)
    printf("%12s task registration: %s %s %s\n>",name,
            ini ? "INIT" : "----",
            tsk ? "BCKG" : "----",
            exi ? "EXIT" : "----");

  return;
}
void checkTasklist(void)
{
  long msec;
  struct timeval tv;
  ExportListStruct *el;

  gettimeofday(&tv,(struct timezone *)NULL);

  for (el=ExportList; el != NULL; el=el->next) 
  {
    if (el->eqpBkg)
    {
      if ((msec=MSECS(tv,el->lastBkg)) < el->rateBkg) continue;
      if (msec == 0 && el->rateBkg != 0) continue;
      if (el->bkgbusy) continue;
      el->bkgbusy = 1;
      el->lastBkg = tv;
      el->eqpBkg();
      el->bkgbusy = 0;
      if (el->next != NULL) _SystemEngine();
    }
  }
  return;
}
extern char tRedirectedServer[64];
int WRITEaccessFromAllNets = 0; /* leave this undocumented! */
short IsIllegalProperty(char *eqpname,char *prop,char *dev,CLN *cln,short access,ExportListStruct **listEntry)
{
  int k,plen=0,ps;
  short prpType,cc=0,rc;
  ExportPropertyListStruct *prp=NULL;
  ExportListStruct *el;
  char *c,Property[32];

  strncpy(Property,prop,32);
  for (k=0; k<NrStockModifiers; k++)
    if ((c=strstr(Property,StockModifier[k].modName)) != NULL)
    {
      *c = 0;
      break;
    }

  prpType = EMPTY_PROP;
  for (el=ExportList; el != NULL; el=el->next) if (!strncmp(eqpname,el->EqpName,EQP_NAME_SIZE)) break;
  if (el == NULL)
  {
    for (el=ExportList; el != NULL; el=el->next) if (!strncmp(eqpname,el->EqpAlias,EQP_NAME_SIZE)) break;
  }
  if (el == NULL) ccerr(non_existent_elem);
  if (el->EqpPropertyQueryFcn == NULL && el->XEqpPropertyQueryFcn == NULL) /* use default property list */
  {
    if ((rc=lookupPropertyName(el,prop,dev)) == -server_redirection) ccerr(-rc);
    if (rc < 0 && (rc=lookupPropertyName(el,Property,dev)) < 0) /* failed first test */
    {
      if (rc == -server_redirection) ccerr(-rc);
      /* check for wild cards */
      for (prp=el->EqpProperty[PROPERTY_HASH_SIZE]; prp; prp=prp->next)
      {
        for (ps = 0; ps < (int)strlen(prp->prpName) && prp->prpName[ps] == '*'; ps++);
        plen = (ps < (int)strlen(prp->prpName)) ? strcspn(&prp->prpName[ps],"*") : 0;
        if (plen && !strncmp(&Property[ps],&prp->prpName[ps],plen)) break;
      }
      if (prp == NULL) /* Maybe it's a query */
      {
        if ((prpType=IsStockProperty(Property)) != STOCK_PROP) ccerr(illegal_property);
      }
      else prpType = WILDCARD_PROP;
    }
    else /*  is registered normally */
    {
      prpType = TARGET_PROP;
    }
  }
  else  /* uses a property query function, but check a few things up front ... */
  {
    if (lookupPropertyName(el,prop,dev) == -server_redirection) ccerr(server_redirection);
    if (strncmp(prop,Property,32) && lookupPropertyName(el,Property,dev) == -server_redirection) 
      ccerr(server_redirection);
    if (lookupRedirectionNameStub != NULL) 
    {
      if (lookupRedirectionNameStub(el->EqpName,prop,dev) == server_redirection) ccerr(server_redirection);
      if (strncmp(prop,Property,32) && 
          lookupRedirectionNameStub(el->EqpName,Property,dev) == server_redirection) ccerr(server_redirection);
    }
    prpType = IsStockProperty(Property);
  }
  if ((access&CA_WRITE) && strcmp("MESSAGE",Property)) /* not a MESSAGE */
  {
    for (k=0;!WRITEaccess && k<NrOfUsers;k++)
      if (!strnicmp(UserList[k].UserName,cln->userName,16)) break;
    if (!WRITEaccess && k == NrOfUsers) 
    {
      feclog("COMMAND %s called by %s : user refused access",Property,cln->userName);
      ccerr(access_denied);
    }
    if (!WRITEaccessFromAllNets &&
        !IsMemberControlNets(cln)) 
    {
      inet_ntoa_b(cln->IPaddress.sin_addr,clientaddr);
      feclog("COMMAND %s called by %s from addr %s: network refused access",
        Property,cln->userName,clientaddr);
      ccerr(access_denied);
    }
    if (strcmp("ACCESSLOCK",Property) && !IsMemberLockSet(cln))
    {
      inet_ntoa_b(cln->IPaddress.sin_addr,clientaddr);
      feclog("COMMAND %s called by %s from addr %s: access lock refused access",
        Property,cln->userName,clientaddr);
      ccerr(access_denied);
    }
    if (putCommandsInFeclog) feclog("COMMAND %s called by %s",Property,cln->userName);
  }
err:
  *listEntry = el;
  return cc;
}
int renegotiateContract(ContractListStruct *cl,int cc)
{
  CONSUMER *c;
  UINT32 doutsize = 0,fmtoutsize;

  if ((fmtoutsize=fmtsizeof(cl->contract.EqpFormatOut)) != 0)
  {
    doutsize = cl->dataSizeOut*fmtoutsize;
    if (cc) doutsize += RPCERR_SIZE;
    if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
  }
  for (c=cl->consumer; c != NULL; c=c->next)
  {
    c->status.BytesRemaining = (UINT32)doutsize;
    c->status.BytesTotal = (UINT32)doutsize;
    c->status.nextDataSize = c->status.BytesRemaining;
    if (c->status.BytesRemaining > c->status.mtu) c->status.nextDataSize = c->status.mtu;
    c->status.numblks = (UINT16)(doutsize/c->status.mtu + (doutsize%c->status.mtu ? 1 : 0));
    if (doutsize == 0) c->status.numblks = 1;
  }
  return 0;
}
int doContract(ContractListStruct *cl)
{
  CONTRACT *con = &cl->contract;
  ExportListStruct *el = cl->exp;
  BYTE *dataIn = cl->DataIn;
  BYTE *dataOut = cl->DataBuf;
  int i,fmtsiz,cc=0,rw=1;
  UINT32 doutlen,doutsize,dp;
  struct structStruct *s;
  char txtbuf[128];
  double dts = gServerDataTimeStamp; /* gServerDataTimeStamp is set prior to call */
  static DTYPE din,dout;
  static char sDevName[36];

  doutlen = con->EqpSizeOut; fmtsiz = fmtsizeof(con->EqpFormatOut);
  doutsize = doutlen*fmtsiz; if (LFMT(con->EqpFormatOut) == CF_SPECTRUM) doutsize += 96;

  if (NGdebug > 3)
  {
    strncpy(txtbuf,con->EqpName,EQP_NAME_SIZE); txtbuf[EQP_NAME_SIZE] = 0; strcat(txtbuf," ");
    strncat(txtbuf,con->EqpProperty,EQP_PROP_SIZE); strcat(txtbuf," ");
    strncat(txtbuf,con->EqpDeviceName,EQP_DEVICE_SIZE);
    if (con->EqpSizeIn != 0)
      DataDump(txtbuf,(char *)dataIn,(int)con->EqpSizeIn,(short)LFMT(con->EqpFormatIn),(short)RTOL);
  }

  if (cl->expired)
  {
    if (NGdebug > 1) dbglog("%s %s %s -> returning expired data",
      cl->contract.EqpName,cl->contract.EqpDeviceName,cl->contract.EqpProperty);
    con->EqpSizeOut = cl->dataSizeOut;
    ccerr(cl->returnCode);
  }

 /* check stock properties: */
  if ((i=GetStockPropertyId(con->EqpProperty)) >= 0)
  {
    dts = putDataTimeStamp(gDataTimeStampOffset,startuptime,0);
    setDataTimeStamp(dts);
    if (doutsize) memset(dataOut,0,(size_t)doutsize);
    ccerr((*StockProperty[i].fcn)(con,(char *)dataIn,(char *)dataOut));
  }
  /* check stock modifiers: (allow overloads) */
  if (GetPropertyId(con->EqpName,con->EqpProperty) < 0)
  {
    for (i=0; i<NrStockModifiers; i++)
      if (strstr(con->EqpProperty,StockModifier[i].modName))
      {
        if (doutsize) memset(dataOut,0,(size_t)doutsize);
        if ((cc=StockModifier[i].fcn(con,(char *)dataIn,(char *)dataOut)) == 0 ||
             cc == server_redirection) ccerr(cc);
        if (el->EqpPropertyQueryFcn == NULL && el->XEqpPropertyQueryFcn == NULL) ccerr(cc);
      }
  }

  if (el == NULL || (el->EqpFcn == NULL && el->NdlFcn == NULL)) return non_existent;

  if (cl->compStatus == not_signalled && gEqpFcnBusySemaphore) 
    return gEqpFcnBusySemaphore > 1 ? cl->compStatus = operation_busy : not_ready;

  strncpy(sDevName,con->EqpDeviceName,16); sDevName[16] = 0;
  if (strlen(sDevName) == 16 && sDevName[15] == '&') /* long device name requested */
  {
    sDevName[15] = 0;
    strncat(sDevName,&con->EqpProperty[17],15);
    sDevName[30] = 0;
  }

  /* call equipment function: */
  if (el->EqpFcn != NULL)      /* Non-Nodal EQM function Prototype */
  {
    dout.dTimeStamp = dts;
    dout.dArrayLength = con->EqpSizeOut;
    dout.dFormat = LFMT(con->EqpFormatOut);
    dout.data.vptr = dataOut;
    strncpy(dout.dTag,con->strTagOut,8);
    if (dout.dFormat == CF_STRUCT)
    {
      AssignStructDataPtr(dout.data.vptr,dataOut,NONE);
      dout.dArrayLength = i;
    }
    if (SystemPresetMemory && doutsize && cl->compStatus == not_signalled) memset(dataOut,0,(size_t)doutsize);
    din.dArrayLength = con->EqpSizeIn;
    din.dFormat = LFMT(con->EqpFormatIn);
    din.data.vptr = dataIn;
    strncpy(din.dTag,con->strTagIn,8);
    if (din.dFormat == CF_STRUCT)
    {
      AssignStructDataPtr(din.data.vptr,dataIn,RTOL);
      din.dArrayLength = i;
    }
    cl->lngDeviceName = sDevName;
    cl->dout = &dout;
    cl->din = &din;

#   ifdef MULTITHREADED
    if (useMultiThreadedEquipmentFunctions)
    {
      if (cl->compStatus == not_signalled) 
      {
        if (NGdebug > 2) thread_dbglog("release mutex");
        ContractListItem = cl;
        gEqpFcnBusySemaphore = 1;
        ReleaseMutex(hEqpThreadMutex);
      }
      cc = WaitForMutex(hEqpThreadMutex,gSystemTick);
      if (cc != 0)
      {
        if (NGdebug > 2) thread_dbglog("timed out waiting for mutex");
        gEqpFcnBusySemaphore++;
        cc = not_ready; 
      } 
      else
      {
        if (NGdebug > 2) thread_dbglog("got mutex");
        gEqpFcnBusySemaphore = 0;
        cc = cl->compStatus == not_signalled ? not_ready : cl->compStatus;
      }
    }
    else
#   endif
    {
      gEqpFcnBusySemaphore = 1;
      cc = (*el->EqpFcn)(sDevName,con->EqpProperty,&dout,&din,con->EqpAccess);
      gEqpFcnBusySemaphore = 0;
    }
    if (cc != 0 && !(cc & CE_SENDDATA)) ccerr(cc);
    if (dout.dTimeStamp != dts) gServerDataTimeStamp = dout.dTimeStamp;
    if (dout.dFormat == CF_STRUCT)
    {
      AssignStructDataPtr(dataOut,dataOut,LTOR);
    }
    if (dout.dArrayLength < con->EqpSizeOut)
    {
      if (!SystemPresetMemory) /* zero out the remainder for later data comparisons */
      {
        dp = dout.dArrayLength*fmtsiz;
        memset(&dataOut[(int)dp],0,(size_t)(con->EqpSizeOut-dout.dArrayLength)*fmtsiz);
      }
      con->EqpSizeOut = dout.dArrayLength;
    }
  }
  else if (el->NdlFcn != NULL) /* Nodal EQM function Prototype */
  {
    if (con->EqpSizeIn != 0 &&
        con->EqpSizeIn*fmtsizeof(con->EqpFormatIn) <= con->EqpSizeOut*fmtsizeof(con->EqpFormatOut))
    {
      if (dataOut != dataIn) memcpy(dataOut,dataIn,(size_t)con->EqpSizeIn*fmtsizeof(con->EqpFormatIn));
      if (PrepFormattedData(dataOut,con->EqpSizeIn,CF_DOUBLE,LFMT(con->EqpFormatIn))) return illegal_format;
      rw = -1;
    }
    i = sDevName[0] == '#' ? atoi(&sDevName[1]) : 0;
    if ((cc=(*el->NdlFcn)((double *)dataOut,
                          (short)(con->EqpSizeOut*rw),
                          (short)i,
#   ifndef NODAL
                          (short)LFMT(con->EqpFormatOut),
#   endif
                          con->EqpProperty)) != 0) ccerr(cc);
    if (PrepFormattedData(dataOut,con->EqpSizeOut,LFMT(con->EqpFormatOut),CF_DOUBLE)) return illegal_format;
  }
err:
  if (con->EqpAccess&CA_FIRST) con->EqpAccess &= ~(CA_FIRST);
  if (cc < 0) cc = ambiguous;
  if (NGdebug > 3 && (cc == 0 || cc & CE_SENDDATA))
    DataDump(txtbuf,(char *)dataOut,(int)con->EqpSizeOut,(short)LFMT(con->EqpFormatOut),(short)LTOR);

  cl->dataSizeOut = con->EqpSizeOut;    /* record possible new data output length */
  if (doutlen != con->EqpSizeOut)
  {
    con->EqpSizeOut = doutlen;   /* don't disrupt the contract itself */
    cc |= CE_RENEGOTIATE;
  }
  return cc;
}
#ifdef INCLUDE_LEGACY_CODE
unsigned short execRequest(CLN *cln)
{
  HeaderStruct tmpH,*h = &tmpH;
  short cc,sval,i,len,doffset=0,fmtsize=0,dsize=0;
  short eqpidx=-1,eqpsize=0;
  unsigned short msgsize=2;
  long worksize=0, ts;
  UINT32 datasize;
  char eqpmodname[8];
  BYTE *Values=NULL;
  BYTE *tWorkArea;
  EqpInfoStruct *EqpInfo=NULL;
//  CONTRACT contract;
  ExportListStruct *el;
  ContractListStruct cl;
  short access = CA_SYNC;

  if (SRVDataBuffer == NULL) ccerr(out_of_server_memory);
  PrepIncomingHeader(h,SRVDataBuffer); eqpsize = abs(h->EqpSize);

  for (i=0,len=0; i<abs(h->EqpSize); i++)
  {
    PrepIncomingInfo(EqpInfo,&SRVDataBuffer[HEADERSTRUCT_SIZE+EQPINFOSTRUCT_SIZE*i]);
    worksize += 3*sizeof(short) + EqpInfo->DataSize*fmtsizeof(EqpInfo->Format) + 8;
    if (EqpInfo->DataSize > len) len = EqpInfo->DataSize;
  }
  if (worksize > (long)srvWorkAreaSize)
  {
   if ((tWorkArea=(BYTE *)realloc((char *)srvWorkArea,(size_t)worksize)) == NULL) ccerr(out_of_server_memory);
   srvWorkArea = tWorkArea;
   srvWorkAreaSize = worksize;
  }
  if ((Values=(BYTE *)SystemCalloc(len,sizeof(double))) == NULL) ccerr(out_of_server_memory);

  memset(eqpmodname,0,8);
  if (h->EqpSize < 0) access |= CA_WRITE;
  doffset = HEADERSTRUCT_SIZE+EQPINFOSTRUCT_SIZE*abs(h->EqpSize);
  RPCprotocol = h->Protocol;

  strncpy(cln->userName,h->UserName,16);
  if (NGdebug) sprintf(dbgbuf,"%s : ",cln->userName);

  for (eqpidx=0; eqpidx<eqpsize; eqpidx++)
  {
    cc=0;
    EqpInfo=(EqpInfoStruct *)&SRVDataBuffer[HEADERSTRUCT_SIZE+EQPINFOSTRUCT_SIZE*eqpidx];
    strncpy(cl.contract.EqpName,EqpInfo->EqpModName,EQP_NAME_SIZE);
    strncpy(cl.contract.EqpProperty,EqpInfo->Property,EQP_PROP_SIZE);
    cl.contract.EqpSizeIn = access & CA_WRITE ? EqpInfo->DataSize : 0;
    cl.contract.EqpSizeOut = EqpInfo->DataSize;
    cl.contract.EqpAccess = access & CA_WRITE ? CA_WRITE|CA_READ : CA_READ;
    cl.contract.EqpFormatIn = BFMT(access & CA_WRITE ? EqpInfo->Format : CF_NULL);
    cl.contract.EqpFormatOut = (BYTE)EqpInfo->Format;
    cl.contract.strTagIn[0] = 0;
    cl.contract.strTagOut[0] = 0;
    sprintf(cl.contract.EqpDeviceName,"#%d",EqpInfo->EqpModNo);

    fmtsize = fmtsizeof(EqpInfo->Format);
    dsize = EqpInfo->DataSize*fmtsize;

    if ((cc=IsIllegalProperty(cl.contract.EqpName,
            cl.contract.EqpProperty,cl.contract.EqpDeviceName,
            cln,access,&el)) != 0) ccerr(cc);

    if (access&CA_WRITE)                                   /* WRITE command */
    {
      PrepStreamData(Values,&SRVDataBuffer[doffset],-EqpInfo->DataSize,EqpInfo->Format);
      doffset += dsize;
    }
    if (strcmp(eqpmodname,EqpInfo->EqpModName)) strncpy(eqpmodname,EqpInfo->EqpModName,6);
    gDataTimeStamp = makeDataTimeStamp();
    ts = RPCtimestamp;

    datasize = EqpInfo->DataSize;

    cl.dataSizeOut = datasize;
    cl.exp = el;
    cl.DataOut = Values;
    cl.DataIn = Values;
    cl.DataBuf = srvWorkArea;
    if ((cc=doContract(&cl)) != 0) goto err;

err:
    PrepWord(cc,cc);
    memcpy(&srvWorkArea[msgsize],&cc,2); msgsize += 2;
    if (!cc)
    {
      PrepWord(sval,EqpInfo->Format); memcpy(&srvWorkArea[msgsize],&sval,2); msgsize += 2;
      PrepWord(sval,dsize); memcpy(&srvWorkArea[msgsize],&sval,2); msgsize += 2;
      PrepStreamData(&srvWorkArea[msgsize],Values,EqpInfo->DataSize,EqpInfo->Format);
      msgsize += EqpInfo->DataSize*fmtsize;

      if (RPCprotocol > 1)
      {
        PrepDWord(ts,ts);
        memcpy(&srvWorkArea[msgsize],&ts,4);
        msgsize += 8;
      }
    }
    if (NGdebug) sprintf(&dbgbuf[strlen(dbgbuf)],"%s %s <%d> ",cl.contract.EqpName,cl.contract.EqpProperty,cc);
  }
  if (NGdebug) dbglog(dbgbuf);
  if (Values != NULL) SystemFree((char *)Values);
  PrepWord(*(unsigned short *)srvWorkArea,msgsize);
  return msgsize;
}
#endif
int initBroadcastList(void)
{
  int cc=0,i,n=1,fmtsize,dsize;
  int BrdcstPacketSize = 0;
  static int initialized = 0;

  if (initialized) return 0; else initialized = 1;
  BrdcstPacketSize = 0;
  if (StartupDebug) printf("Broadcasting the following parameters :\n");
  for (i = 0, BrdcstPacketSize = sizeof(HeaderStruct);
       i < nbroadcasts && BrdcstPacketSize < gBrdcstBufferSize; i++)
  {
    if (StartupDebug) printf("%s ",BrdcstList[i]->keyword);
    BrdcstPacketSize += sizeof(BrdcstData);
    fmtsize = fmtsizeof(BrdcstList[i]->format);
    if (!fmtsize) ccerr(illegal_format);
    if ((dsize=BrdcstList[i]->size*fmtsize) >
         (int)(gBrdcstBufferSize - sizeof(HeaderStruct) - sizeof(BrdcstData)))
    {
      if (StartupDebug) printf("Broadcast %s exceeds %d bytes\n>",
	      BrdcstList[i]->keyword,gBrdcstBufferSize);
      ccerr(buffer_too_small);
    }
    BrdcstPacketSize += dsize;
    if (BrdcstPacketSize > gBrdcstBufferSize)
    {
      n++;
      BrdcstPacketSize = sizeof(HeaderStruct);
    }
  }
  if (StartupDebug) printf("\n>Broadcast packets required : %d\n>",n);
err:
  if (cc)
  {
    if (StartupDebug) printf("\n>initBroadcastList : %s\n>",erlst[cc]);
    nbroadcasts = 0;
  }
  return 0;
}

int initRPC(void)
{
  int cc;
  BYTE *ver = SystemVersion();

  feclog("VERSION : %d.%02d.%04d",ver[0],ver[1],ver[2]*256 + ver[3]);
  feclog("OS      : %s",OS_TAG);
  if (srvWorkAreaSize >= WORK_SIZE && 
      MaxRPCTransportSize > MAX_TRANSPORTSIZE &&
      MaxRPCTransportSize > srvWorkAreaSize) srvWorkAreaSize = MaxRPCTransportSize;
  tmpWorkAreaSize = srvWorkAreaSize;
# ifdef MULTITHREADED
  srvWorkAreaSize = MIN_WORK_SIZE;
  feclog("Work Area: %ld",srvWorkAreaSize);
  feclog("Temp Size: %ld",tmpWorkAreaSize);
  feclog("FEC: is running multi-threaded");
# else
  if (srvWorkAreaSize <= WORK_SIZE && 
      MaxRPCTransportSize > srvWorkAreaSize) MaxRPCTransportSize = srvWorkAreaSize;
  feclog("Work Area: %ld",srvWorkAreaSize);
  feclog("Temp Size: %ld",tmpWorkAreaSize);
  feclog("Max Transport Size: %ld",MaxRPCTransportSize);
# ifdef INTEL
  tmpWorkArea = srvWorkArea;
# endif
# endif
  feclog("FECDB : [%s]",FecDBpath);
  startuptime = time(NULL);
  nconsumer = ncontract = 0;
  if ((srvWorkArea = (BYTE *)SystemCalloc(1,(size_t)srvWorkAreaSize)) == NULL)
  {
   printf("Work Space %ld bytes: insufficient memory\n>",(long)srvWorkAreaSize);
   return out_of_server_memory;
  }
  if (tmpWorkArea == NULL && (tmpWorkArea=(BYTE *)SystemCalloc(1,tmpWorkAreaSize)) == NULL)
  {
    printf("Temp Work Space: insufficient memory\n>");
    return out_of_server_memory;
  }
  if (MaxNumClients <= 0) return 0; /* no server functionality required */
  if ((ContractList=(ContractListStruct **)SystemCalloc(MaxNumContracts,sizeof(ContractListStruct *))) == NULL)
  {
   printf("Contract List: insufficient memory\n>");
   return out_of_server_memory;
  }
  if ((Consumers=(CLN **)SystemCalloc(MaxNumClients,sizeof(CLN *))) == NULL)
  {
   printf("Client List: insufficient memory\n>");
   return out_of_server_memory;
  }
  if (BroadcastServer)
  {
    if ((BrdcstList=(GlobalListStruct **)SystemCalloc(MaxNumGlobals,sizeof(GlobalListStruct *))) == NULL)
    {
      printf("Broadcast List: insufficient memory\n>");
      return out_of_server_memory;
    }
  }
  GetAliasTable();
  memset(&IPXFecAddress,0,sizeof(IPXAddress));
  cc = getRegisteredUsers(&UserList,USERS_FILE,&NrOfUsers,"WRITE",&WRITEaccess);
  cc = getRegisteredUsers(&DeniedUserList,DENYUSER_FILE,&NrOfDeniedUsers,"DENIED",&DENYaccess);
  cc = getRegisteredExports();
  cc = getFecName();
  initServers(0);
  initAlarmServer();
  makeStockPropXRefTable();
  memset(&currentAccessLock,0,sizeof(currentAccessLock));
  SysPoll = MaxPollingRate;
  if (cdiSystemInitHook != 0) cdiSystemInitHook(); /* if CDI is in use ... */
  return 0;
}
int ServerExitCondition = 0;
void quitServer(void)                         /* allow a clean exit */
{
  ServerExitCondition = 1;
  return;
}
UINT32 gBurstLimitReachedCount = 0;
UINT32 gTimeServerCycle = 0;
UINT32 gTimeServerIdleLastCycle = 0;
int gCycleCounts = 0, gMaxCycleCounts = 1;
int dorpc(void)
{
  static struct timeval lastCycleTime;
  static int lmi_done = 0;
  static int busy = 0;
  static int CycleCounts = 0;
  int i;

  if (MaxNumClients <= 0) return 0; /* no server functionality required */
  if (ServerExitCondition) { if (ServerExitCondition++ > 3) haltServer(); }
  if (!lmi_done) 
  {
    gettimeofday(&thisCycleTime,(struct timezone *)NULL);/* Cycle time */
    lmi_done = LastMinuteInitialization();
    lastCycleTime = thisCycleTime;
    if (useGlobalSynchronization) SystemStartGlobalSynchronization();
  }
  if (busy) return not_ready; else busy = 1;
  gettimeofday(&thisCycleTime,(struct timezone *)NULL);/* Cycle time */
  gTimeServerCycle += MSECS(thisCycleTime,lastCycleTime);
  lastCycleTime = thisCycleTime;
  AcceptRequest(&thisCycleTime);/* Accept Consumer subscriptions and renewals   */
  gTimeServerIdleLastCycle += MSECS(thisCycleTime,lastCycleTime);
  ProduceData(&thisCycleTime);  /* Is it time to service consumers?   */
  for (i=0; StaleData && i<gBurstLimit; i++)
  {
    DeliverData(&thisCycleTime);
  }
  if (StaleData)
  {
    if (gCycleDelay) millisleep(gCycleDelay);
    gBurstLimitReachedCount++;
  }
  if (nbroadcasts) doBroadcast(&thisCycleTime);
  if (acquireDeviceNamesFromProperties) devNamesInitialization();
  Cleanup(&thisCycleTime);
  SurveyAlarms(&thisCycleTime);
  if (!ExportsRegistered) sndRegisteredExports(&thisCycleTime);
  if (currentAccessLock.lockType) checkAccessLocks((time_t)thisCycleTime.tv_sec);
  gTimeServerCycle += MSECS(thisCycleTime,lastCycleTime);
  lastCycleTime = thisCycleTime;
  CycleCounts++;
  if (gTimeServerCycle >= 1000)
  {
    if (CycleCounts > gMaxCycleCounts) gMaxCycleCounts = CycleCounts;
    gCycleCounts = CycleCounts;
    if (ServerCycleMode != POLLING)
    {
      AveBusyTime = (int)((gTimeServerCycle-gTimeServerIdleLastCycle)*100/gTimeServerCycle);
    }
    else
    {
      AveBusyTime = (gMaxCycleCounts-CycleCounts)*100/gMaxCycleCounts;
    }
    gTimeServerCycle = gTimeServerIdleLastCycle = 0;
    CycleCounts = 0;
  }
  return busy = 0;
}

/* =================================================
//
//    Producer-consumer definitions and routines:
//
// ================================================= */

int MaxNumContracts = NCONTRACT;
int MaxNumClients   = NCONSUMER;
ContractListStruct **ContractList;
CLN **Consumers;                /* registered consumer list */
int nconsumer = 0, ncontract = 0;

int StaleData = 0;

char *SubType[] = {"CANCEL","SINGLE","REFRESH","POLL"};
int RemoveConsumerFromList(CONSUMER *c,int j)          /* from contract j */
{
  CONSUMER *p=ContractList[j]->consumer;
  int wasPending = FALSE;

  if (j < 0 || j>=ncontract || c == NULL) return -2;

  if (NGdebug > 2) dbglog("remove consumer %s from contract %d",c->client->userName,j);
  if (c->status.blknum < c->status.numblks) wasPending = TRUE;
  c->client->count = MAX(c->client->count-1,0);
  StaleData++;                                  /*  Force RemoveConsumer() */
  if (p == c) ContractList[j]->consumer = c->next;
  else
  {
    while (p->next != c) p=p->next;
    p->next = c->next;
  }
  SystemFree((char *)c);
  ContractList[j]->nconsumer = MAX(ContractList[j]->nconsumer-1,0);
  if (ContractList[j]->nconsumer == 0) return 0;
  if (wasPending) ContractList[j]->pending--;
  p = ContractList[j]->consumer;
  ContractList[j]->PollingRate = p->status.PollingRate;
  for (p=p->next; p != NULL; p=p->next)
   ContractList[j]->PollingRate =
     findpoll(ContractList[j]->PollingRate,p->status.PollingRate);
  return 0;
}
int RemoveConsumer(int i)  /* return memory to heap */
{
  if (i < 0 || i >= nconsumer) return -2;
  if (Consumers[i] == NULL) return -3;
  if (NGdebug > 2) dbglog("remove consumer %s",Consumers[i]->userName);
# if defined(UNIX) && !defined(NO_TINE_PIPE)
  if (Consumers[i]->transport == PIPE && Consumers[i]->sck > 0) 
  {
    close(Consumers[i]->sck);
    if (nipcconsumers > 0) nipcconsumers--;
  }
# endif  
  SystemFree((char *)Consumers[i]);
  for (;i<nconsumer-1; i++) Consumers[i] = Consumers[i+1];
  Consumers[--nconsumer] = NULL;
  return 0;
}
int RemoveContract(int j)  /* return memory to heap, decrement ncontract */
{
  CONSUMER *c=ContractList[j]->consumer,*p;

  if (j < 0 || j>=ncontract) return argument_list_error;
  if (ContractList[j]->compStatus == not_posted) return ContractList[j]->compStatus;
  if (NGdebug > 2) dbglog("remove contract %d",j);
  for (p=c; p != NULL; c=p)                /* decrement consumer count: */
  {
    c->client->count = MAX(c->client->count-1,0);
    StaleData++;                            /*  Force RemoveConsumer()   */
    p = c->next;
    SystemFree((char *)c);
  }
  if (ContractList[j]->expired && nExpiredContracts > 0) nExpiredContracts--;
  if (ContractList[j]->DataIn) SystemFree((char *)ContractList[j]->DataIn);
  if (ContractList[j]->DataOut) SystemFree((char *)ContractList[j]->DataOut);
  SystemFree((char *)ContractList[j]);
  for (;j<ncontract-1; j++) ContractList[j] = ContractList[j+1];
  ContractList[--ncontract] = NULL;
  for (j=0, SysPoll = MaxPollingRate; j<ncontract; j++)
    SysPoll = findpoll(SysPoll,ContractList[j]->PollingRate);
  return 0;
}
UINT32 gContractMisses = 0;
UINT32 gContractDelays = 0;
UINT32 gClientMisses = 0;
UINT32 gClientReconnects = 0;
UINT32 gClientRetries = 0;
UINT32 gSystemTick = SYSTEM_TICK;
UINT32 gSynchronizeContracts = (UINT32)CONTRACT_SYNCHRONIZE;
CONSUMER *EQPconsumerlist;                      /* global reference  */
int EQPnconsumers;
CONTRACT *EQPcontract;
int ProduceData(struct timeval *tv)             /* Production system */
{
  static int busy = 0;
  ContractListStruct *cl;       /* improve readability by assignment */
  CONSUMER *c;
  short cc,rc,i,dirty=0,Stale,fmtoutsize;
  UINT32 doutsize;
  long delta_t;
  double dts;
  int isScheduled, msec;
  struct timeval tv_stamp;                

  if (busy) return not_ready; else busy = TRUE;
  for (i=0; i<ncontract; i++)
  {
    if ((cl=ContractList[i])->nconsumer == 0 && !cl->expired) 
    {                                                   /* no clients, not marked as expired */
      if (cl->contract.EqpAccess & CA_LAST) doContract(cl);            /* going out of scope */
      if (RemoveContract(i) == 0) i--;                /* decrements ncontract => decrement i */
      continue;
    }
    cc = Stale = isScheduled = 0;
    if ((fmtoutsize=fmtsizeof(cl->contract.EqpFormatOut)) != 0)
    {
      doutsize = cl->contract.EqpSizeOut*fmtoutsize;
      if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
    }
    else doutsize = 0;
    if ((delta_t=MSECS(*tv,cl->lasttime)) < 0) cl->lasttime = *tv; /* system clock set back ! */
    if (delta_t <= cl->PollingRate - (long)gSystemTick) continue;
    if (cl->expired && cl->nconsumer == 0)
    {
      if (delta_t > 3*cl->PollingRate && 
          RemoveContract(i) == 0) i--;                  /* decrements ncontract => decrement i */
      continue;
    }
    if (delta_t < MAXALLOWED_POLLRATE && delta_t >= 2*cl->PollingRate)
    {
      if (NGdebug > 1)
      {
        dbglog("[%d] (%s) %s %s missed last schedule (%ld vs %ld)",i,
          cl->contract.EqpName,
          cl->contract.EqpDeviceName,
          cl->contract.EqpProperty,delta_t,2*cl->PollingRate);
      }
      cl->misses++;
      gContractMisses++;
    }
    if (cl->pending > 0) 
    {
      if (NGdebug > 2)
      {
        dbglog("[%d] (%s) %s %s still pending",i,
          cl->contract.EqpName,
          cl->contract.EqpDeviceName,
          cl->contract.EqpProperty);
      }
      cl->delays++;
      gContractDelays++;
      continue;           /* still sending out the last round */
    }
    if (cl->starttime.tv_sec == 0 && cl->starttime.tv_usec == 0) cl->starttime = *tv;
    tv_stamp = *tv; 
    msec = (int)(tv->tv_usec/1000);
    /* Synchronize on 1000 msec boundaries if possible */
    if (!(cl->PollingRate%1000) && 
        gSynchronizeContracts   && 
        msec < (1000 - (int)gSystemTick)) tv_stamp.tv_usec = 0;
    dts = putDataTimeStamp(gDataTimeStampOffset,tv->tv_sec,msec);
    setDataTimeStamp(dts);
    EQPnconsumers = cl->nconsumer;
    EQPconsumerlist = cl->consumer;
    EQPcontract = &cl->contract;
    if (cl->nconsumer == 1 && cl->consumer->status.counter == 1)
      cl->contract.EqpAccess |= CA_LAST; /* dangling client going out of scope */
    switch (cc=doContract(cl))
    {
      case 0: break;
      case not_ready:
        if (!gEqpFcnBusySemaphore ||   /* call returned not_ready or still not_signalled */
            (MSECS(*tv,cl->starttime) < cl->PollingRate + 500 && gEqpFcnBusySemaphore))
        {
          for (c=cl->consumer; c != NULL; c=c->next) c->status.Stale = 0;
          cl->pending = 0;
          continue;      /* don't send out */
        }
      case server_redirection:
        strncpy(cl->compString,tRedirectedServer,64);
      default: 
        if (cc & 0xf000) /* CE_RENEGOTIATE or CE_SENDDATA */
        {
          if ((rc=renegotiateContract(cl,(cc & 0xfff))) != 0) cc = rc; else cc &= ~CE_RENEGOTIATE;
          if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
        }
        else goto err;
    }

    if (doutsize && !cl->expired && memcmp(cl->DataOut,cl->DataBuf,(size_t)doutsize))
    {
      memcpy(cl->DataOut,cl->DataBuf,(size_t)doutsize);
      Stale = dirty = 1;                       /* Data have changed: */
    }                                          /* the cached data are dirty */
    else
    {
      if (MSECS(*tv,cl->lastmarked) > (HEARTBEAT-1)*1000L) Stale = 1;
      dirty = 0;                               /* Cached data have not changed */
    }

err:if (cc) Stale = 1;
    if (Stale) cl->lastmarked = tv_stamp;      /* Last marked as Stale */
    if (cl->lasttime.tv_sec == PRP_SCHEDULE_SIGNAL) isScheduled = TRUE;
    if (cc == not_ready)
    {
      cc = operation_busy;
    }
    else
    {
      cl->starttime.tv_sec = cl->starttime.tv_usec = 0;
      cl->lasttime = tv_stamp;                   /* Last polled */
      cl->compStatus = not_signalled;
    }
    cl->dtimestamp = (long)gServerDataTimeStamp;
    cl->dtimestampMSEC = (int)((gServerDataTimeStamp - (long)gServerDataTimeStamp) * 1000); 
    cl->returnCode = cc;

    for (c=cl->consumer; c != NULL; c=c->next)
    {
      if (c->status.counter == 0)              /* scheduled for removal */
      {
        StaleData++;                           /* Ensure pass thru deliverData */
        continue; 
      }
      c->status.statusCode = cc;
      if (isScheduled) c->status.lasttime.tv_sec = c->status.lasttime.tv_usec = PRP_SCHEDULE_SIGNAL;
      if (cc) c->status.Stale = 1;             /* Stale because of error code */
      if (c->status.mode == REFRESH)
      {
        if (Stale)                             /* Stale because of data change */
        {                                      /* or system heartbeat          */
          c->status.Stale = 1;
          if (dirty && RequireAcknowledgments)
            c->status.counter = ACK_REQUEST;   /* Force acknowledgment */
        }
      }
      else                                     /* Honor polling rate   */
      {
        if (MSECS(*tv,c->status.lasttime) > c->status.PollingRate - (long)gSystemTick)
          c->status.Stale = 1;                 /* Stale because of polling rate */
      }
      if (c->status.Stale)                     /* Consumer for this contract */
      {                                        /* is to be refreshed */
        cl->pending++;
        c->status.lasttime = tv_stamp;
        StaleData++;                           /* Ensure pass thru deliverData */
      }
      if (c->status.counter == ACK_REQUEST)    /* Get serious until an */
      {                                        /* acknowledgment comes */
        c->status.mode = ACK_PENDING;          /* Reset by renewal */
        c->status.PollingRate = cl->PollingRate;
      }
    }
  }
  gettimeofday(tv,(struct timezone *)NULL);
  busy = FALSE;
  return 0;
}
#define maxDeadTime(m,p) ((m) == CM_REFRESH ? HEARTBEAT + (p) : 2 * (p))
int DeliverData(struct timeval *tv)            /* Delivery system */
{
  static int busy = 0;
  ProducerHeaderStruct4 tmpP,*Pheader = &tmpP;
  ProducerHeaderStruct tmpPEx,*PheaderEx = &tmpPEx;
  CONSUMER *c;
  ContractListStruct *cl;       /* improve readability by assignment */
  short i,k,n,dsize,ssize,fsize,passed,snddata,basemode,actionTaken=FALSE;
  long gapsize, gapptr;
  UINT16 msgsize,totalsize=0,packetmtu;
  UINT32 dptr,delta_t;
  short stepsize,fmt;
  BYTE *buf = srvWorkArea;

  if (busy) return not_ready; else busy = TRUE;
  for (k=0; k<nconsumer; k++)
  {
    if (Consumers[k] == NULL) continue;
    if (Consumers[k]->count == 0)
    {
      RemoveConsumer(k--);
      continue;
    }
    passed = 1; totalsize = sizeof(totalsize);
    for (i=0,n=0; i<ncontract && n<Consumers[k]->count; i++)
    {
      cl = ContractList[i];
      for (c=cl->consumer; c != NULL; c=c->next) if (c->client == Consumers[k]) break;
      if (c == NULL) continue;
      if (Consumers[k]->transport == TCP && Consumers[k]->sck == 0)
        c->status.counter = 0;    /*  TCP connection error -> remove it */
      n++; snddata = TRUE; gapptr = gapsize = 0;
      basemode = c->status.mode;
      if (c->status.Stale)
      {
        actionTaken = TRUE;
        stepsize = c->client->protocol == 4 ? PRODUCER4_SIZE : PRODUCER_SIZE;
        packetmtu = sizeof(UINT16) + stepsize + c->status.mtu;
        snddata = (c->status.statusCode == 0) ? TRUE : 
                  (c->status.statusCode & CE_SENDDATA) ? TRUE : FALSE;
        if (snddata)
        {
          dsize = (short)c->status.nextDataSize;
          dptr  = c->status.nextDataPtr;
        }
        else
        {
          gapsize = (c->client->protocol == 4 ? 0 : RPCERR_SIZE);
          dsize = (short)gapsize;
          dptr  = 0;
        }
        if ((msgsize=dsize+stepsize) > packetmtu) { c->status.counter = 0; continue; }
        if (msgsize % 2 != 0) msgsize++;  /* make sure of even boundaries */
        PrepPCHeader(&buf[totalsize],c,cl->contract,cl->dtimestamp,cl->dtimestampMSEC);
        if (snddata && (fsize=fmtsizeof(cl->contract.EqpFormatOut)) != 0)
        {
          if ((c->status.statusCode & 0xfff))
          {
            gapsize = (long)(c->status.nextDataPtr + c->status.nextDataSize -
                             cl->contract.EqpSizeOut*fmtsizeof(cl->contract.EqpFormatOut));
            if (gapsize > RPCERR_SIZE) gapsize = RPCERR_SIZE; /* shouldn't be possible */
            if (gapsize > dsize)                              /* err string remainder  */
            {
              gapsize = dsize;
              gapptr  = RPCERR_SIZE - gapsize;
            }
            if (gapsize > 0) dsize -= (short)gapsize; else gapsize = 0;
          }
          if (dsize > 0)
          {
            memcpy(&buf[totalsize+stepsize],&cl->DataOut[(int)dptr],(size_t)dsize);
            ssize = dsize / fsize; fmt = cl->contract.EqpFormatOut;
            if (c->status.blknum > 1 &&        /* long CF_SPECTRUM contract */
                LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM)
            fmt = BFMT(CF_FLOAT);            /* remaing data is CF_FLOAT */
            PrepSubData(&buf[totalsize+stepsize],ssize,fmt,LTOR);
          }
        }
        if ((totalsize + msgsize) > packetmtu) { passed = 0; goto bok; }
        totalsize += msgsize;

        if (NGdebug > 2 || (NGdebug && c->status.mode <= SINGLE))
        {
          dbglog("[%d]: %s %s %s <%d>(%d b %d of %d) to %s (+%d)",
                  i,cl->contract.EqpName,
                  cl->contract.EqpDeviceName,
                  cl->contract.EqpProperty,c->status.statusCode,msgsize,
                  c->status.blknum,c->status.numblks,
                  c->client->userName,c->status.counter);
        }
        if (gapsize > 0)                      /* insert err string */
        {
          if (c->status.statusCode == server_redirection) 
          {
            memcpy(RPCLastErrorString,cl->compString,RPCERR_SIZE);
            c->status.counter = 1;
          }
          else if ((c->status.statusCode & 0xfff) < numErr) 
            SetRPCCompletion(erlst[c->status.statusCode & 0xff]);
          memcpy(&buf[totalsize-(int)gapsize],&RPCLastErrorString[(int)gapptr],(size_t)gapsize);
        }
        if (snddata == 0 || c->status.numblks == c->status.blknum)
        {                                     /* datablock finished and/or error code */
          c->status.Stale = 0;
          if (c->status.counter > 0 ) c->status.counter--;
          if (c->status.mode == CM_CANCEL) c->status.counter = 0;
          c->status.blknum = 1;
          c->status.nextDataPtr = 0;
          c->status.BytesRemaining = c->status.BytesTotal;
          ReturnToServerCycle = 0;
          cl->pending--;
          if ((delta_t=MSECS(*tv,c->status.lasttime)) > (UINT32)maxDeadTime(c->status.mode,c->status.PollingRate))
          {
            if (NGdebug > 1)
            {
              dbglog("[%d]: %s %s %s -> returning late data (%d > %d ms)",
                  i,cl->contract.EqpName,
                  cl->contract.EqpDeviceName,
                  cl->contract.EqpProperty,(int)delta_t,c->status.PollingRate);
            }
            c->status.misses++;
            gClientMisses++;
          }
        }
        else
        {
          ReturnToServerCycle = c->status.numblks - c->status.blknum;
          c->status.blknum++;
          c->status.nextDataPtr += c->status.nextDataSize;
          c->status.BytesRemaining = c->status.BytesTotal - c->status.nextDataPtr;
          StaleData++;
        }
        c->status.nextDataSize = MIN(c->status.mtu,c->status.BytesRemaining);
      }
      if (c->status.counter == 0) 
      {
        actionTaken = TRUE;
        RemoveConsumerFromList(c,i); 
        n--;
      }
      if (!cl->nconsumer) 
      {
        actionTaken = TRUE;
        if (gRetardSingleContractRemoval && basemode == CM_SINGLE) 
        {
          cl->expired = TRUE;
          cl->contract.EqpAccess &= ~(CA_FIRST|CA_LAST);
          nExpiredContracts++;
        }
        else 
        {
          if (RemoveContract(i) == 0) i--; /* decrements ncontract => decrement i */
        }
      }
    }

bok:if (totalsize > sizeof(totalsize) && Consumers[k])
    {
      actionTaken = TRUE;
      PrepWord(*(short *)srvWorkArea,totalsize);
      SendToPeer(Consumers[k],srvWorkArea);
    }
    if (!passed) k--;
  }
  gettimeofday(tv,(struct timezone *)NULL);
  busy = FALSE;
  if (!actionTaken) StaleData = 0;
  return StaleData = --StaleData > 0 ? StaleData : 0;
}
ContractProgress cp[MAXLONGGRAMS];
void CleanupStrandedDatagrams(void)
{
  int i, n;

  for (i=0; i<MAXLONGGRAMS; i++)
    if (cp[i].busy && thisCycleTime.tv_sec > cp[i].blktime + 5)
  {
    SystemFree((char *)cp[i].data);
    memset(&cp[i],0,sizeof(ContractProgress));
    if (NGdebug > 1) dbglog("stale long datagram %d",i);
  }
  for (i=0, n=0; i<ncontract; i++)
  {
    if (ContractList[i]->compStatus == not_signalled && gEqpFcnBusySemaphore) n++;
  }
  if (n == ncontract) /* this is a lock-out and should never happen !*/
  {
    gEqpFcnBusySemaphore = 0;
  }
}
int GetDefaultFormatAndSize(CONTRACT *con)
{
  int i, num, idx, cc = illegal_property;
  ExportListStruct *el;
  ExportPropertyListStruct *prp;
  char *eqm, *eqpprp;
  BYTE *fmt;
  UINT32 *siz;
  PropertyQueryStruct *pqs;
  XPropertyQueryStruct *xpqs;

  if (con == NULL) ccerr(code_failure);

  eqm = con->EqpName; eqpprp = con->EqpProperty;
  fmt = &con->EqpFormatOut; siz = &con->EqpSizeOut;

  for (el=ExportList; el != NULL; el=el->next) if (!strncmp(eqm,el->EqpName,EQP_NAME_SIZE)) break;
  if (el == NULL) ccerr(non_existent_elem);

  if (el->XEqpPropertyQueryFcn != NULL)        /* registered ext query fcn */
  {
    num = el->XEqpPropertyQueryFcn(con->EqpDeviceName,&xpqs);
    for (i=0; i<num; i++) if (!strncmp(xpqs->prpName,eqpprp,EQP_PROP_SIZE))
    {
      *fmt = BFMT(xpqs->prpFormat);
      *siz = xpqs->prpSize;
      ccerr(0);
    }
  }
  else if (el->EqpPropertyQueryFcn != NULL)         /* registered query fcn */
  {
    num = el->EqpPropertyQueryFcn(con->EqpDeviceName,&pqs);
    for (i=0; i<num; i++) if (!strncmp(pqs->prpName,eqpprp,EQP_PROP_SIZE))
    {
      *fmt = BFMT(pqs->prpFormat);
      *siz = pqs->prpSize;
      ccerr(0);
    }
  }
  else                                                    /* handle it here */
  {  
    idx = (int)(ElfHash((unsigned char *)eqpprp) % PROPERTY_HASH_SIZE);
    for (prp=el->EqpProperty[idx]; prp; prp=prp->next) if (!strncmp(prp->prpName,eqpprp,EQP_PROP_SIZE))
    {
      *fmt = BFMT(prp->prpFormat);
      *siz = prp->prpSize;
      ccerr(0);
    }
  }
  if ((i=GetStockPropertyId(eqpprp)) >= 0)
  {
    *fmt = BFMT(StockProperty[i].prpFormat);
    *siz = StockProperty[i].prpSize;
    ccerr(0);
  }
err:
  return cc;
}
int SendMessageToCaller(CLN *cln,struct subscription *s,int cc,BYTE *data,int len)
{
  BYTE dbuf[HEADERSTRUCT_SIZE+PRODUCER_SIZE+RPCERR_SIZE], *buf;
  CONSUMER tmpC,*c = &tmpC;
  HeaderStruct tmpH,*h = &tmpH;
  ProducerHeaderStruct4 tmpP,*Pheader = &tmpP;
  ProducerHeaderStruct tmpPEx,*PheaderEx = &tmpPEx;
  UINT16 msgsize,prot = 5;
  double dts;

  buf = srvWorkArea;
  PrepIncomingHeader(h,buf);
  switch (cc)
  {
    case get_subscription_id:
     if (h->Protocol !=  5) ccerr(illegal_protocol);
     break;
    case server_redirection:
      len = RPCERR_SIZE; /* = 2 * NAME16 */
      data = (BYTE *)tRedirectedServer;
      break;
    default:
      if (cc > 0 && cc < numErr) SetRPCCompletion(erlst[cc]);
      break;
  }
  if (NGdebug) 
  {
    switch (cc)
    {
      case illegal_property:
      case illegal_equipment_name:
      case non_existent_elem:
      case server_redirection:
      case access_denied:
        dbglog("MSG: (%s) %s -> %s",s->contract.EqpName,s->contract.EqpProperty,erlst[cc]);
        break;
      default: 
        dbglog("MSG: -> %s",erlst[cc]);
        break;
    }
  }
  (c = &tmpC)->status.starttime = s->starttime;
  c->status.statusCode = cc;
  c->status.id = s->id;
  c->status.numblks = 1;
  c->client = cln;
  c->client->protocol = prot;
  dts = putDataTimeStamp(gDataTimeStampOffset,thisCycleTime.tv_sec,(int)(thisCycleTime.tv_usec/1000));
  msgsize = prot == 4 ? PRODUCER4_SIZE : PRODUCER_SIZE+RPCERR_SIZE;
  buf = dbuf;
  PrepWord(*(short *)buf,msgsize+sizeof(msgsize));
  PrepPCHeader(&buf[sizeof(msgsize)],c,s->contract,(long)dts,(int)((dts - (long)dts) * 1000));
  if (data != NULL && len > 0)
  {
    memcpy(&buf[sizeof(msgsize)+PRODUCER_SIZE],data,len);
  }
  else if (cc)
  {
    memcpy(&buf[sizeof(msgsize)+PRODUCER_SIZE],RPCLastErrorString,RPCERR_SIZE);
  }
  SendToPeer(cln,buf);
err:
  return cc;
}
int getNextFreeContractID(void)
{
  static UINT16 cid = 0;
  int i;
  for (i=0,cid++; i<ncontract; i++) if (ContractList[i]->contractID == cid) cid++;
  return cid;
}
UINT32 gSingleLinkCount = 0;
#define replaceContractAliasNames(cn) { if (nAliasTableEntries > 0) {\
  if ((i=getAliasTableIndex((cn)->EqpProperty)) >= 0) strncpy((cn)->EqpProperty,gAliasTable[i].name,EQP_PROP_SIZE);\
  if ((i=getAliasTableIndex((cn)->EqpDeviceName)) >= 0) strncpy((cn)->EqpDeviceName,gAliasTable[i].name,EQP_DEVICE_SIZE);\
  }\
}
#define ccout(o) { cc = o; goto out; }
int getConsumerRequest(CLN *cln,BYTE *buf,int packetsize)
{
  BYTE *dbuf;
  HeaderStruct tmpH,*h = &tmpH;
  CONSUMER tmpC,*c = &tmpC;
  short i,j = -1,subid,fmtinsize,fmtoutsize,prot,cc=0,isTrueMCast=0,renewalMultiplier;
  UINT16 totalsize,msgsize,datasize,bytesread=HEADERSTRUCT_SIZE;
  static long dinsize,doutsize,defsize,dataPtr;
  struct subscription4 tmpReg,*reg = &tmpReg;
  struct subscription tmpRegEx,*regEx = &tmpRegEx;
  static UINT16 datamtu;

  msgsize = 0; /* avoid compiler warning */
  PrepIncomingHeader(h,buf); totalsize = h->TotalSize;

  if (totalsize <= 0) ccout(out_of_range);
  if ((prot=h->Protocol) < 4 || prot > 5) ccout(illegal_protocol);
  if (NGdebug) sprintf(&dbgbuf[strlen(dbgbuf)],"%s -> ",h->UserName);
  strncpy(cln->userName,h->UserName,15);                  /* always reset   */

  PrepSubHeader(prot,&buf[bytesread]);
  replaceContractAliasNames(&regEx->contract);

  while (bytesread < totalsize && bytesread < packetsize) /* unpack request */
  {
    if (NGdebug && regEx->numblks < 4) /* output to screen can influence io buffering */
    {
      sprintf(&dbgbuf[strlen(dbgbuf)],"%s %s %s %ld msec (%d of %d)[%s]",
              regEx->contract.EqpName,regEx->contract.EqpDeviceName,
              regEx->contract.EqpProperty,regEx->PollingRate,
              regEx->blknum,regEx->numblks,SubType[(regEx->mode)&0x03]);
      dbglog(dbgbuf);
      sprintf(dbgbuf,"\t\t ->");
    }
    if (NrOfDeniedUsers > 0 && IsDeniedUser(h->UserName)) ccerr(access_denied);
    if ((fmtoutsize=fmtsizeof(regEx->contract.EqpFormatOut)) != 0)
    {
      if ((doutsize=regEx->contract.EqpSizeOut*fmtoutsize) <= 0) ccerr(illegal_data_size);
      if (LFMT(regEx->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96; /* CF_SPECTRUM HEADER */
      if (doutsize > (long)MaxRPCTransportSize) ccerr(invalid_transport_size);
    }
    else doutsize = 0;

    if ((fmtinsize=fmtsizeof(regEx->contract.EqpFormatIn)) != 0) /* data input */
    {
      if (regEx->blknum > regEx->numblks) ccerr(link_error);
      dinsize = regEx->contract.EqpSizeIn*fmtinsize;
      dbuf = &buf[bytesread + (prot == 4 ? SUBSCRIPTION4_SIZE : SUBSCRIPTION_SIZE)];
      dataPtr = ((long)(regEx->blknum-1))*regEx->mtu;
      datasize = MIN((UINT16)(dinsize-dataPtr),regEx->mtu);
      if (dinsize > (long)MaxRPCTransportSize) ccerr(invalid_transport_size);
      if (dinsize > regEx->mtu) /* incoming long datagram */
      {
        for (i=0; i<MAXLONGGRAMS; i++)
          if (cp[i].haddr == (long)cln->IPaddress.sin_addr.s_addr &&
              cp[i].SubscriptionID == regEx->id &&
              cp[i].SubsStarttime  == regEx->starttime) break;
        if (i == MAXLONGGRAMS) for (i=0; cp[i].busy && i<MAXLONGGRAMS; i++);
        if (i == MAXLONGGRAMS) ccerr(resources_exhausted);
        if (++(cp[i].blksin) <= regEx->numblks)
        {
          if (cp[i].blksin == 1) cp[i].blkid = regEx->blkid;
          else if (cp[i].blkid < regEx->blkid ||
                   cp[i].blkid > regEx->blkid + 500)
          {
            if (NGdebug > 1)
              dbglog("re-sync long datagram id %d to %d",cp[i].blkid,regEx->blkid);
            cp[i].blksin = 1;
            cp[i].blkid = regEx->blkid;
            cp[i].blktime = time(NULL);
            cp[i].SubsStarttime = regEx->starttime;
          }
          if (!cp[i].busy)
          {
            if (NGdebug > 1) dbglog("new long datagram (%d) sub id %d %ld bytes",i,regEx->id,dinsize);
            cp[i].busy = 1;
            cp[i].haddr = cln->IPaddress.sin_addr.s_addr;
            cp[i].SubscriptionID = regEx->id;
            cp[i].blktime = time(NULL);
            cp[i].SubsStarttime = regEx->starttime;
            if ((cp[i].data=(BYTE *)SystemCalloc(1,(size_t)dinsize)) == NULL) ccerr(out_of_server_memory);
          }
          memcpy(&cp[i].data[(int)dataPtr],dbuf,(size_t)datasize);
          ReturnToServerCycle = regEx->numblks - cp[i].blksin;
          if (cp[i].blksin < regEx->numblks) goto bow;
          cp[i].busy = 2;
          dbuf = cp[i].data;
          if (NGdebug > 1) dbglog("in-coming long datagram (%d blocks) complete",regEx->numblks);
        }
        else ReturnToServerCycle = 0;
      }
      PrepSubData(dbuf,regEx->contract.EqpSizeIn,regEx->contract.EqpFormatIn,RTOL);
    }
    else { dinsize = 0; dbuf = NULL; }

    if (regEx->PollingRate < MinPollingRate) regEx->PollingRate = MinPollingRate;
    if (regEx->PollingRate > 1000) regEx->PollingRate = (regEx->PollingRate/1000)*1000;

    if (LFMT(regEx->contract.EqpFormatOut) == CF_DEFAULT)
    {                                      /* caller wants default settings */
      if ((cc=GetDefaultFormatAndSize(&regEx->contract)) != 0) ccerr(cc);
      if ((fmtoutsize=fmtsizeof(regEx->contract.EqpFormatOut)) != 0)
      {
        if ((defsize=regEx->contract.EqpSizeOut*fmtoutsize) <= 0) ccerr(illegal_data_size);
        if (LFMT(regEx->contract.EqpFormatOut) == CF_SPECTRUM) defsize += 96; /* CF_SPECTRUM HEADER */
        if (defsize > (long)MaxRPCTransportSize) ccerr(invalid_transport_size);
        if (defsize > doutsize)           /* caller can't buffer default */
        {
          regEx->contract.EqpSizeOut = doutsize/fmtoutsize;
          defsize = regEx->contract.EqpSizeOut*fmtoutsize;
        }
      }
      else defsize = 0;
      doutsize = defsize;                  /* for allocation purposes */
    }

    for (j=0; j<ncontract; j++)            /* is contract in list ? */
      if (!memcmp(&ContractList[j]->contract,&regEx->contract,sizeof(CONTRACT)))
      {
        if (!dbuf && !ContractList[j]->DataIn) break;          /* no data input */
        if (dbuf && ContractList[j]->DataIn) /* input buffer has been allocated */
        {
          if (!memcmp(ContractList[j]->DataIn,dbuf,(size_t)dinsize)) break;
        }
      }

    if (j == ncontract)                    /* no, add to list  */
    {
      if (j >= MaxNumContracts) ccerr(resources_exhausted);
      if ((ContractList[j]=(ContractListStruct *)SystemCalloc(1,sizeof(ContractListStruct))) == NULL)
        ccerr(out_of_server_memory);
      memcpy(&ContractList[j]->contract,&regEx->contract,sizeof(CONTRACT));
      ContractList[j]->compStatus = not_signalled;
      ContractList[j]->returnCode = not_posted;
      ncontract++;

      if ((cc=IsIllegalProperty(ContractList[j]->contract.EqpName,
                                ContractList[j]->contract.EqpProperty,
                                ContractList[j]->contract.EqpDeviceName,
                                cln,
                                ContractList[j]->contract.EqpAccess,
                                &ContractList[j]->exp)) != 0) ccerr(cc);

      if (dinsize && (ContractList[j]->DataIn = (BYTE *)SystemCalloc(1,(size_t)dinsize)) == NULL)
      {
        SystemFree((char *)ContractList[j]);
        ccerr(out_of_server_memory);
      }
      if (doutsize && (ContractList[j]->DataOut = (BYTE *)SystemCalloc(nDataOutBuffers,(size_t)doutsize)) == NULL)
      {
        if (dinsize) SystemFree((char *)ContractList[j]->DataIn);
        SystemFree((char *)ContractList[j]);
        ccerr(out_of_server_memory);
      }
      ContractList[j]->DataBuf = nDataOutBuffers > 1 ? &ContractList[j]->DataOut[doutsize] : srvWorkArea;
      if (dbuf) memcpy(ContractList[j]->DataIn,dbuf,(size_t)dinsize);
      ContractList[j]->contract.EqpAccess |= CA_FIRST; /* turn on FIRST bit */
      ContractList[j]->contractID = getNextFreeContractID();
    }
    /* is a new link attaching to an expired contract ? */
    if (ContractList[j]->expired && !(regEx->mode & CM_RETRY)) 
    {
      ContractList[j]->expired = 0;
      if (nExpiredContracts > 0) nExpiredContracts--;
    }

    /* is broadcast or multicast request ? */
    if (regEx->mode & (CM_BCAST|CM_MCAST))
    {
      isTrueMCast = regEx->mode & CM_MCAST;
      regEx->mode &= 0x0ff; /* strip off */
      if (regEx->mode == CM_CANCEL) ccerr(0); /* don't do anything */
      if (IsMemberBcastNets(cln) || isTrueMCast)
      {
        /* send subscription ID to caller */
        subid = ISWAP(ContractList[j]->contractID);
        SendMessageToCaller(cln,regEx,get_subscription_id,(BYTE *)&subid,2);
        /* add network to client list and replace cln */
        strncpy(cln->userName,"NETWORK",16);
        if (cln->IPaddress.sin_addr.s_addr != 0)
        {
          cln->IPaddress.sin_port = htons((UINT16)gtMCastPort);
          if (isTrueMCast) cln->IPaddress.sin_addr.s_addr = tMCastAddr.s_addr;
          else             cln->IPaddress.sin_addr.s_addr |= 0xff000000L;
          cln = LocateConsumerInList(&cln->IPaddress,NULL,NULL);
        }
        else if (*(long *)cln->IPXaddress.network != 0)
        {
          memset(&cln->IPXaddress.network[10],MCAST_SOCKET,2);
          memset(cln->IPXaddress.node,0xff,6);
          cln = LocateConsumerInList(NULL,&cln->IPXaddress,cln->node);
        }
        regEx->id = ContractList[j]->contractID;
        regEx->starttime = BCAST_ID;
      }
    }

    /* is client in contractList?    */
    for (c=ContractList[j]->consumer; c!=NULL && c->client!=cln; c=c->next);

    if (c == NULL)  /* no, add to list */
    {
      if ((c=(CONSUMER*)SystemCalloc(1,sizeof(CONSUMER))) == NULL) ccerr(out_of_server_memory);
      c->next = ContractList[j]->consumer;
      ContractList[j]->consumer = c;
      c->client = cln;
      ContractList[j]->nconsumer++;
      c->client->count++;
      c->client->protocol = prot;
      c->status.statusCode = ContractList[j]->returnCode;
      c->status.counter = 1;
    }
    c->status.PollingRate = regEx->PollingRate;
    c->status.id = regEx->id;
    c->status.starttime = regEx->starttime;
    c->status.blknum = 1;
    c->status.nextDataPtr = 0;
    c->status.BytesRemaining = doutsize;
    c->status.BytesTotal = doutsize;
    datamtu = PMTU(*(short *)c->client->IPXaddress.socket);
    if (fmtoutsize > 0) datamtu -= datamtu%fmtoutsize;
    c->status.mtu = datamtu;
    c->status.nextDataSize = c->status.BytesRemaining;
    if (c->status.BytesRemaining > c->status.mtu) c->status.nextDataSize = c->status.mtu;
    c->status.numblks = doutsize/((long)c->status.mtu) + (doutsize%((long)c->status.mtu) ? 1 : 0);
    if (doutsize == 0) c->status.numblks = 1;
    renewalMultiplier = 1;
    if (regEx->PollingRate < 500) renewalMultiplier = 3;
    if (c->status.mode != ACK_PENDING)         /* Force Contract execution */
    {
      if ((c->status.counter == 0 && regEx->starttime != BCAST_ID) || regEx->mode == CM_REFRESH)
      {
        c->status.Stale = 1;                     
        c->status.lasttime = thisCycleTime;
      }
      if (regEx->mode != CM_CANCEL && regEx->mode != CM_REFRESH &&
          c->status.counter > ACK_REQUEST*renewalMultiplier &&
          c->status.counter < RENEWAL*renewalMultiplier)
      {
        if (NGdebug > 1)
        {
          dbglog("reconnecting to %s %s %s from %s (+%d)",
            regEx->contract.EqpName,regEx->contract.EqpDeviceName,
            regEx->contract.EqpProperty,h->UserName,c->status.counter);
        }
        c->status.reconnects++;
        gClientReconnects++;
      }
      StaleData++;
    }
    if (regEx->mode & CM_RETRY)
    {
      if (NGdebug > 1)
      {
        dbglog("retrying %s %s %s by %s",
          regEx->contract.EqpName,regEx->contract.EqpDeviceName,
          regEx->contract.EqpProperty,h->UserName);
      }
      c->status.Stale = 1;                     
      StaleData++;   /* if expired, DeliverData() will send the last payload again */
      c->status.isRetry = TRUE;
      gClientRetries++;
      regEx->mode = BASEMODE(regEx->mode);
    }
    if (ContractList[j]->nconsumer == 0)       /* just turned off, turn it back on */
    {
      ContractList[j]->nconsumer++;
      ContractList[j]->contract.EqpAccess &= ~(CA_LAST);
    }
    switch (c->status.mode = regEx->mode)
    {
      case CM_SINGLE :
        gSingleLinkCount++;
        c->status.counter = 1;
        if (!(ContractList[j]->expired && c->status.isRetry))
          ContractList[j]->lasttime.tv_sec = PRP_RETRY_SIGNAL;  /* a RETRY marked as expired should not be called */
        else
          ContractList[j]->lasttime.tv_sec = time(NULL);
        if (ContractList[j]->PollingRate == 0) ContractList[j]->PollingRate = c->status.PollingRate;
        if (ContractList[j]->nconsumer == 1)
        {
          ContractList[j]->contract.EqpAccess |= CA_LAST; /* turn on LAST bit */
        }
        break;
      case CM_POLL   :
      case CM_REFRESH:
        c->status.counter = RENEWAL * renewalMultiplier;
        ContractList[j]->PollingRate = findpoll(regEx->PollingRate,ContractList[j]->PollingRate);
        break;
      default     :
      case CM_CANCEL :
        if (!strcmp(c->client->userName,"NETWORK")) break;
        if (ContractList[j]->nconsumer == 1)
        {
          if (c->status.counter)                            /* was active */
            ContractList[j]->contract.EqpAccess |= CA_LAST; /* turn on LAST bit */
          ContractList[j]->lasttime.tv_sec = PRP_CANCEL_SIGNAL;
          ContractList[j]->pending = 0;
        }
        c->status.counter = 0;
        break;
    }
    if (NGdebug > 2)
      dbglog("link counter set to %d for %s (mode %d)",c->status.counter,c->client->userName,c->status.mode);
err:if (cc)
    {
      SendMessageToCaller(cln,regEx,cc,NULL,0);
      if (j >= 0 && j < MaxNumContracts) RemoveContract(j);
      cc = 0;
    }
    else
    {
      SysPoll = findpoll(SysPoll,ContractList[j]->PollingRate);
    }
    for (i=0; i<MAXLONGGRAMS; i++) if (cp[i].busy == 2)
    {
      SystemFree((char *)cp[i].data);
      memset(&cp[i],0,sizeof(ContractProgress));
      if (NGdebug > 1) dbglog("discarding long datagram %d",i);
    }
bow:if ((bytesread += regEx->msgsize) < totalsize)
    {
      if (regEx->msgsize % 2 != 0) ccerr(not_properly);
      PrepSubHeader(prot,&buf[bytesread]);
    }
  }                                                           /* continue unpacking  */
out:
  if (cc) SendMessageToCaller(cln,regEx,cc,NULL,0);
  return cc;
}
void doBroadcast(struct timeval *thisTime)
{
  BYTE *buf = srvWorkArea;
  int cc = 0,i,n,setsize,BrdcstPacketSize,datasize;
  char *strptr;
  BrdcstData tmpBD,*brdcstdata = &tmpBD;
  HeaderStruct tmpH,*brdcstHeader = &tmpH;
  int delta_t;
  time_t interim = thisTime->tv_sec;
  static int done = FALSE;

  if (!done)
  {
    if (initBroadcastList()) return;
    done = TRUE;
  }
  BrdcstPacketSize = HEADERSTRUCT_SIZE;
  for (i=0, n=0; i<nbroadcasts; i++)
  {
    if (BrdcstList[i]->tv.tv_sec > 0 &&                         /* not a scheduled broadcast */
        (BrdcstList[i]->minPeriod <= 0 || BrdcstList[i]->maxPeriod <= 0)) continue;
    delta_t = (int)MSECS(*thisTime,BrdcstList[i]->tv);
    if (delta_t < BrdcstList[i]->minPeriod) continue; /* too soon */
    if (delta_t < BrdcstList[i]->maxPeriod) /* within heartbeat */
    {
      if (!datacmp(BrdcstList[i]->refdata,
            BrdcstList[i]->data,
            BrdcstList[i]->size,
            BrdcstList[i]->format,
            BrdcstList[i]->tolerance)) continue;
    }
    datasize = BrdcstList[i]->size*fmtsizeof(BrdcstList[i]->format);
    setsize = BRDCSTDATA_HEADER_SIZE + datasize;
    if (BrdcstPacketSize + setsize > gBrdcstBufferSize)
    {                                          /* send what's there  */
      AssignOutgoingHeader(brdcstHeader,buf);  /* fill in the header */
      brdcstHeader->EqpSize = n;               /* number of keywords */
      strncpy(brdcstHeader->UserName,gUserName,16);
      brdcstHeader->Protocol = 3;              /* broadcast protocol */
      brdcstHeader->TotalSize = BrdcstPacketSize;
      PrepOutgoingHeader(brdcstHeader,buf);
      cc = SendBroadcast(buf,BrdcstPacketSize);
      BrdcstPacketSize = HEADERSTRUCT_SIZE;
      n = 0;
    }
    AssignOutgoingBrdcstData(brdcstdata,&buf[BrdcstPacketSize]);
    strncpy(brdcstdata->name,BrdcstList[i]->keyword,16);
    brdcstdata->size = BrdcstList[i]->size;
    brdcstdata->format = BrdcstList[i]->format;
    brdcstdata->time = BrdcstList[i]->time = interim;
    BrdcstList[i]->tv = *thisTime;
    PrepOutgoingBrdcstData(brdcstdata,&buf[BrdcstPacketSize]);
    BrdcstPacketSize += BRDCSTDATA_HEADER_SIZE;
    if (BrdcstList[i]->data)
    {
      memcpy(BrdcstList[i]->refdata,BrdcstList[i]->data,datasize);
      PrepOutgoingData(&buf[BrdcstPacketSize],BrdcstList[i]->data,datasize,BrdcstList[i]->format);
    }
    if (NGdebug > 1)
    {
      sprintf(dbgbuf,"GCAST: %s ",BrdcstList[i]->keyword);
      strptr = &dbgbuf[strlen(dbgbuf)];
      if (cc) sprintf(strptr,"(ERR: %s)",erlst[cc]);
      dbglog(dbgbuf);
      if (cc == 0 && NGdebug > 2)
      {
        DataDump("GCAST: ",(char *)BrdcstList[i]->data,
                  BrdcstList[i]->size,BrdcstList[i]->format,LTOR);
      }
    }
    if (BrdcstList[i]->fcn) (BrdcstList[i]->fcn)(i,cc);
    BrdcstPacketSize += datasize; n++;
  }
  if (n > 0)                                 /* something left     */
  {
    AssignOutgoingHeader(brdcstHeader,buf);  /* fill in the header */
    brdcstHeader->EqpSize = n;               /* number of keywords */
    strncpy(brdcstHeader->UserName,gUserName,16);
    brdcstHeader->Protocol = 3;              /* broadcast protocol */
    brdcstHeader->TotalSize = BrdcstPacketSize;
    PrepOutgoingHeader(brdcstHeader,buf);
    cc = SendBroadcast(buf,BrdcstPacketSize);
  }
  return;
}

/* trivial RPC call: */
int tRPC(int fecindex,char *data,char *name,short EqNo,char *prop,
	   short size,short format,short timout)
{
  char dev[16];
  sprintf(dev,"#%d",EqNo);
  return tExecLink(fecindex,data,name,dev,prop,size,format,timout);
}
int tExecLink(int fecindex,char *data,char *name,char *dev,char *prop,
	   short size,short format,short timout)
{
  int fmtsize,retry = 2;
  short cc=0,EqNo,dsize,sstepsize,pstepsize;
  static BYTE lclWorkArea[IPX_BUFFER_SIZE];
  static short NameServerProtocol = 5;
  static UINT16 tid = 0;        /* trivial subscription id */
  HeaderStruct hTmp,*h = &hTmp;
  struct subscription4 sTmp, *s = &sTmp;
  struct subscription  sTmpEx, *sEx = &sTmpEx;
  ProducerHeaderStruct4 pTmp, *Pheader = &pTmp;
  ProducerHeaderStruct pTmpEx, *PheaderEx = &pTmpEx;
  ConnTableEntry c;

  if (NGdebug) dbglog("trivial RPC: %s %s %s",name,prop,dev);
  format = format ? format : CF_DOUBLE;
  fmtsize = fmtsizeof(format);
  if (!fmtsize || strlen(name) > 16 || strlen(prop) > 32) return string_too_long;
  if ((dsize = abs(size)*fmtsize) < 1) return out_of_range;
  if ((HEADERSTRUCT_SIZE+SUBSCRIPTION4_SIZE+dsize) > IPX_BUFFER_SIZE) return use_stream_transport;
  if (timout < 1) timout = 1;

  memset(&c,0,sizeof(ConnTableEntry));
  c.mode = SINGLE; c.starttime = time(NULL);
  strncpy(c.name,name,16); strncpy(c.prop,prop,32);
  strncpy(c.deviceName,dev,16); EqNo = dev[0] == '#' ? atoi(&dev[1]) : 0;
  c.EqNo = EqNo; c.FECindex = fecindex;

  if (size < 0)
  {
    c.sizeIn = -size; c.formatIn = format;
    c.sizeOut = -size; c.formatOut = format;
    c.access = CA_WRITE|CA_READ;
  }
  else
  {
    c.sizeIn = 0; c.formatIn = CF_NULL;
    c.sizeOut = size; c.formatOut = format;
    c.access = CA_READ;
  }

  c.datamtu = IPX_BUFFER_SIZE-SUBSCRIPTION_SIZE-HEADERSTRUCT_SIZE;
  c.blkidIn = 0;
  c.blknumIn = 1;
  c.numblksIn = 1;
  if (NameServerProtocol == 5)
  {
    sstepsize = SUBSCRIPTION_SIZE;
    pstepsize = PRODUCER_SIZE;
  }
  else
  {
    sstepsize = SUBSCRIPTION4_SIZE;
    pstepsize = PRODUCER4_SIZE;
  }
  AssignOutgoingHeader(h,lclWorkArea);

tRETRY:
  c.RPCprotocol = NameServerProtocol;
  AssignOutgoingSubscription(c.RPCprotocol,tid,&c,name,&lclWorkArea[HEADERSTRUCT_SIZE]);
  if (size < 0)
  {
    memcpy(&lclWorkArea[HEADERSTRUCT_SIZE+sstepsize],data,dsize);
    s->msgsize = sstepsize + dsize;
  }
  else s->msgsize = sstepsize;
  PrepOutgoingSubscription(c.RPCprotocol,&lclWorkArea[HEADERSTRUCT_SIZE]);

  h->TotalSize = HEADERSTRUCT_SIZE + s->msgsize;
  memcpy(h->UserName,gUserName,16);
  h->Protocol = c.RPCprotocol;
  h->EqpSize = 1;
  PrepOutgoingHeader(h,lclWorkArea);

  switch (cc = tCall(fecindex,lclWorkArea,timout))
  {
    case 0: break;
    case link_not_open:
      if (--retry) goto tRETRY;
      ccerr(cc);
    case illegal_protocol:
      if (NameServerProtocol == 5)
      {
        NameServerProtocol = 4;
        sstepsize = SUBSCRIPTION4_SIZE;
        pstepsize = PRODUCER4_SIZE;
        goto tRETRY;
      }
      ccerr(cc);
    default:
      ccerr(cc);
  }

  PrepIncomingSubscription(NameServerProtocol,&lclWorkArea[2]);
  if (PheaderEx->SubscriptionID != tid) ccerr(connection_timeout);
  if ((cc=PheaderEx->CompletionCode) == 0)
  {
    if (dsize != PheaderEx->msgsize-pstepsize)
    {
      ccerr(illegal_format);
    }
    else memcpy(data,&lclWorkArea[2+pstepsize],dsize);
  }
err:
  tid = (tid + 1) % 10000;
  if (cc && NGdebug)
  {
    dbglog("trivial RPC error: <%d> %s",cc, cc > 0 && cc < numErr ? erlst[cc] : "unknown");
  }
  return cc;
}
