#ifndef _RPCSRV_H_
#define _RPCSRV_H_

#include "rpcdefs.h"

#define MAXALLOWED_POLLRATE 65535L
#define MSECS(a,b) ((a).tv_sec-(b).tv_sec > MAXALLOWED_POLLRATE ? MAXALLOWED_POLLRATE :\
		    ((a).tv_sec-(b).tv_sec)*1000+((a).tv_usec-(b).tv_usec)/1000)
#define MAX_ENTRIES      300 // connection table entries (* 70 Kbytes)
#define FEC_TIMEOUT      1   // seconds
#define SERVER_TIMEOUT   45  //clock ticks
#define SYSTEM_DELAY     100 // msecs
#define INTSWAP(w)       ((w<<8)|(w>>8))
#define hNULL            0
#define MAXHOTLINKS     400
#define MAXCOMMONLINKS   20
#define _POLLRATE      1000
#define RPC_RETRY_COUNT   3
#define RPC_TIMEOUT       6

#define FEC_NAME_SIZE    20
#define ELEM_NAME_SIZE   15
#define PROPERTY_SIZE    32
#define EQNAME_SIZE	      6

#define PREFIX 6
#define RPCERR_SIZE 32
extern char RPCLastErrorString[PREFIX+RPCERR_SIZE];
#define SetErrorString(s,rc,cl) {\
  memset(s,0,PREFIX+RPCERR_SIZE);\
  strcpy(s, (cl) ? "[RMT] " : "[LCL] ");\
  if (rc > 0 && rc < numErr) memcpy(&(s)[PREFIX],erlst[rc],RPCERR_SIZE-1);\
  else strcpy(&(s)[PREFIX],"error undefined");\
}

#define ccerr(e) {cc = e; goto err; }
#define DeReference(h)  GlobalLock(h); GlobalUnlock(h);
#define OFFSETIN(struc,field)	((unsigned short)&(((struc *)0L)->field))

typedef struct
{
  char name[8];
} NAME8;

typedef struct
{
  char name[16];
} NAME16;

typedef struct
{
  char name[32];
} NAME32;

typedef struct
{
  char name[48];
} NAME48;

typedef struct
{
  char comment[80];
  time_t tm;
  float s_start;
  float s_inc;
  UINT32 status;
  float d_spect_array[1024];
} DSPECTRUM;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[8];
} DBLK;

struct structStruct
{
  char name[12];
  short rawlen;
  short size;
  short num;
  HANDLE hNext;
};
extern struct structStruct far *structList;
struct structStruct far *getStructFromTag(char far *tag);
struct structStruct far *findstruct(char far *tag);

// =============================================================
//
// WINDOWS environment definitions:
//
// =============================================================
#define RPCMSG_RECEIVE  (WM_USER+1)
#define WM_RCVPKT       (WM_USER)
#define WM_RCVPKT_IPX   (WM_USER+1)
#define WM_RCVPKT_UDP   (WM_USER+2)
#define WM_RCVPKT_GLB   (WM_USER+3)
#define WM_CREATELINK   (WM_USER+4)
#define WM_SYNCRPC      (WM_USER+5)
#define WM_REMOVELINK   (WM_USER+6)
#define WM_SETRPCDATA   (WM_USER+7)
#define WM_GETRPCDATA   (WM_USER+8)
#define WM_FLUSHRPC     (WM_USER+9)
#define WM_LOCKLINK     (WM_USER+10)
#define WM_GETLINKID    (WM_USER+11)
#define WM_RPCDEBUG     (WM_USER+12)
#define WM_INITAPPLICATION (WM_USER+13)
#define WM_RCVPKT_UDPBCST (WM_USER+14)
#define WM_RCVPKT_UDPGLB (WM_USER+15)

// =============================================================
//
// Subscription oriented Producer-consumer definitions:
//
// =============================================================
#define NCONSUMER     20
#define NCONTRACT     50
#define RENEWAL       60
#define HEARTBEAT     60
#define REMINDER      5
#define ACK_REQUEST   (2*REMINDER)
#define REPLY_PENDING 3
#define SUB_PENDING   0x3fff
#define ACK_PENDING   0x7fff
#define CANCEL	      CM_CANCEL
#define SINGLE        CM_SINGLE
#define REFRESH       CM_REFRESH
#define POLL	      CM_POLL
#define REGISTER      CM_REGISTER

#define BASEMODE(m) ((m) & 0xff)

extern char *SubType[];
extern int nconsumer;
extern int ncontract;

typedef struct
{
  char EqpName[8],
	 Property[12];
  short EqpSize,
	 EqpFormat,
	 EqpNumber;
  UINT16 PollRate,  // find a place for these two
	 PollMode;
} CONTRACT4;
#define CONTRACT4_SIZE (8+12+5*2)

typedef struct             /* net: Protocol 5 (in) */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
} CONTRACT;
#define CONTRACT_SIZE (32+16+8+4*2+4*1+2*8)

typedef struct             /* local */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpTagName[16];
  char   EqpContext[16];
  char   FecName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
  BYTE   far *DataIn;
  UINT32 PollRate;
  UINT32 PollMode;
} LCLCONTRACT;

typedef struct
{
  char               userName[16];
  IPXAddress         IPXaddress;  // if 0 then IP transfer is assumed
  BYTE               node[6];     // IPX routing node (Immediate Address)
  struct sockaddr_in IPaddress;
  int                count;       // number of registrations
} CLIENT;

typedef struct
{
  UINT16 msgsize,
         SubscriptionID,         // supplied by the consumer upon registration
	 CompletionCode,
	 counter;                // tell consumer how much is left
  time_t timestamp,              // RPCtimestamp
     	 starttime;              // supplied by the consumer upon registration
} ProducerHeaderStruct4;
#define PRODUCER4_SIZE (4*2+2*4)

typedef struct                  /* net: Protocol 5 (out) */
{
  UINT16 msgsize;               /* message size in bytes */
  UINT16 SubscriptionID;        /* supplied by the consumer upon registration */
  short  CompletionCode;        /* completion or interim status */
  UINT16 numblks;               /* total number of message blocks */
  UINT16 blknum;                /* block number of this message */
  UINT16 mtu;                   /* maximum transport unit */
  UINT16 EqpSize;               /* Eqp Size of returned data */
  BYTE   EqpFormat;             /* Eqp Format of returned data */
  BYTE   counter;               /* tell consumer how much is left */
  time_t timestamp;             /* RPCtimestamp */
  time_t starttime;             /* supplied by the consumer upon registration */
} ProducerHeaderStruct;
#define PRODUCER_SIZE (7*2+2*1+2*4)

struct subscription4
{
  UINT16 msgsize;
  UINT16 PollingRate,
		 id,
		 mode;
  time_t   starttime;
  CONTRACT4 contract;
};
typedef struct subscription4 SUBSCRIPTION4;
#define SUBSCRIPTION4_SIZE (4*2+1*4+CONTRACT4_SIZE)

struct subscription         /* net: Protocol 5 (in) */
{
  UINT16    msgsize;
  UINT16    mtu;            /* maximum transport unit for incoming data */
  UINT8     numblks;        /* total number of message blocks           */
  UINT8     blknum;         /* block number of this message             */
  UINT16    id;
  short     mode;
  short     blkid;          /* (incremented) block id */
  UINT32    PollingRate;
  time_t    starttime;
  CONTRACT contract;
};
typedef struct subscription SUBSCRIPTION;
#define SUBSCRIPTION_SIZE (6*2+2*4+CONTRACT_SIZE)

extern HANDLE hContractList[NCONTRACT];  // ContractListStruct
extern HANDLE hConsumers[NCONSUMER];  // registered consumer list (CLIENT)

extern int PCSocket;              // IPX Producer/Consumer send socket
extern int ProducerSocket;        // UDP Producer Socket
extern int ConsumerSocket;        // UDP Consumer Socket
extern int BrdcstSocket;          // UDP Broadcast Socket
extern int GlobalSocket;          // UDP Global listening Socket
extern int MCastConsumerSocket;
extern int ListenForMCasts;

extern struct sockaddr_in prod_name;
#define SendToPC(c,d,s)	\
 ((cc=SendIPXData(c,d)) == illegal_protocol ? SendUDPData(c,d,s) : cc )

// =================================================================

typedef struct ELEMDATA
{
  HANDLE hNext;
  int hDataIn;        // shared memory handle (data down)
  int hDataOut;       // shared memory handle (data up)
  char ElemName[16];
  char EqName[EQNAME_SIZE+1];
  char Property[32];
  char DeviceName[16];
  char Context[16];
  char FecName[16];
  short EqNo;
  short formatIn;       /* eqp data format down */
  short formatOut;      /* eqp data format up */
  char   strTagIn[8];   // structure tag Input
  char   strTagOut[8];  // structure tag Output
  UINT32 sizeIn;        /* data array size down */
  UINT32 sizeOut;       /* data array size up */
  UINT32 lastDataSize;  // last completed data size
  short access;         /* data access */
  short CompCode;       // completion code
  short CompLoc;        // completion location (remote = 1, local =0)
  UINT32 PollRate;
  short FecIndex;
  short links[MAXCOMMONLINKS]; // make this a link list when we have the time
  BOOL Acquire;
  short timeoutCount;   /* used by alarmCheck as retry mechanism */
  short lockCount;      /* used by sync subscriptions */
  short ID;             /* used by subscriptions */
  UINT32 heartbeat;     /* used by subscriptions */
  WORD mode;            /* used by subscriptions */
  WORD PrimeMode;       /* used as internal record */
  short  RPCprotocol;   /* RPC header type */
  short  protocol;      /* transport protocol (IPX,SPX,UDP,TCP) */
  time_t time;          /* last polling request */
  time_t starttime;     /* entry into table (used by subscriptions) */
  time_t RPCtimestamp;  /* arriving RPC timestamp */
  short RPCversion[2];
  int  counter;         /* current subscription counter */
  UINT8 numblks;        /* progress counters for incoming data */
  UINT8 blksin;         /* progress counters for incoming data */
  UINT16 datamtu;       /* outgoing data minimum transport unit */
  UINT16 nextDataSize;  /* next outgoing data size in bytes */
  UINT32 nextDataPtr;   /* next outgoing data array pointer */
  UINT32 BytesRemaining;/* number of bytes data remaining */
  UINT32 Bytesin;       /* progress counters for incoming data  */
  UINT32 BytesTotal;    /* total number of bytes data */
  UINT16 blkidIn;       /* block Id of data to send */
  UINT8 numblksIn;      /* num blks input data to send */
  UINT8 blknumIn;       /* this blk num of input data */
  char RPCLastErrorString[PREFIX+RPCERR_SIZE];
  UINT16 srvID;         /* server subscription ID (applicable in MCAST) */
} ElemDataStruct;

//#define rpcDATAIN(e) ((BYTE *)((BYTE *)(e) + (e)->hDataIn))
//#define rpcDATAOUT(e) ((BYTE *)((BYTE *)(e) + (e)->hDataOut))

typedef struct
{
  HWND hVBWnd;  	  // calling program to receive WM_PAINT (paint method)
  HWND hVBXWnd;           // calling VBX to receive VB_RECEIVE (VBX method)
  HANDLE hElem;           // Handle to Element structure
  short dataFlg,          // present status
        active;           // 0: free, 1: in use, -1: on hold
  short  mode,            // requested mode for this link
         cnt;             // cleanup counter
} linkType;

typedef struct
{
  char FecName[FEC_NAME_SIZE+1];
  char FullName[ELEM_NAME_SIZE+1];
  char EqName[EQNAME_SIZE+1];
  char  Context[16];
  short alarmSystem;
} RPCDataStruct;
#define RPCDATASTRUCT_SIZE (FEC_NAME_SIZE+ELEM_NAME_SIZE+EQNAME_SIZE+3+16+2)

typedef struct
{
  char Name[FEC_NAME_SIZE+1];
  BYTE Network[4];
  BYTE Node[6];
  BYTE ImmediateAddress[6];
  char IP[16];
  BYTE PortOffset;
  DWORD iaddr;
  BYTE Prot;
  BYTE MaxRPCProt;
} RPCFecStruct;
#define RPCFECSTRUCT_SIZE (FEC_NAME_SIZE+1+16+21+2)

typedef struct
{
  char keyword[16];
  int  size;
  char formatString[16];
  int  format;
  long time;
  long brdcstTime;
  int heartbeat;
  HANDLE hData;
}BrdcstListStruct;

typedef struct
{
  UINT16 TotalSize;
  char UserName[16];
  short Protocol;
  short EqpSize;
}HeaderStruct;
#define HEADERSTRUCT_SIZE 22

typedef struct
{
  char ElemName[16];
  char EqpModName[8];
  short EqpModNo;
  char Property[12];
  char Cycle[22];         // cycle specification for CERN PPM elements
  short Format;
  short DataSize;
}EqpInfoStruct;

typedef struct
{
  char  prpName[16];
  char  prpDescription[32];
  unsigned short prpSize;
  BYTE prpFormat;
  BYTE prpAccess;
} PropertyQueryStruct;
#define PROPERTYQUERYSTRUCT_SIZE (16+32+2+1+1)

typedef struct
{
  char FecName[FEC_NAME_SIZE];
  time_t localtime;
  time_t starttime;
  long   systemPollingRate;
  short  numBkgTsks;
  short  numTotalContracts;
  short  numTotalClients;
  short  numTargetContracts;
  short  numTargetClients;
  short  numConnections;
  UINT32 numConnectionTimeouts;
  UINT32 numConnectionArrivals;
  UINT32 numUDPpkts;
  UINT32 numTCPpkts;
  UINT32 numIPXpkts;
  UINT32 numSPXpkts;
} ActivityQueryStruct;
#define ACTIVITYQUERYSTRUCT_SIZE (FEC_NAME_SIZE+4*3+2*6+4*6)

typedef struct
{
  char desc[64];
  BYTE dataFmt;
  BYTE dataSiz;
  BYTE data[64];
  time_t timeEntered;
  UINT32 codeNow;
  UINT32 codePrv;
  UINT32 codeNxt;
} ServerStateStruct;
#define SERVERSTATESTRUCT_SIZE (64+2+64+4*4)

typedef struct
{
	char name[16];       // broadcast element name  (16 bytes)
	short  size;           // number of elements broadcast under "name" (2 bytes)
	short  format;         // data format (as per RPC definitions)  (2 bytes)
	long time;           // timestamp of broadcast data (4 bytes)
} BrdcstData;


#define IPX			    0x01
#define SPX         0x02
#define TCP         0x04
#define UDP         0x08
#define GLB         0x10
#define UDPBCST     0x20
#define IPXBCST     0x40

#define BCAST_ID 255

#define WORK_BUFFER_SIZE 0x1000
// IPX/SPX specific:
#define RPC_SOCKET      0x45 // listening SPX socket
#define BRDCST_SOCKET   0x67 // listening broadcast socket
#define REG_SOCKET      0x55 // listening registration socket
#define MCAST_SOCKET    0x85 // listening mcast socket
#define NUM_RCV_ECB     2              // associated incoming contracts
#define NUM_GLB_ECB     2              // incoming broadcasts
#define NUM_REG_ECB     2              // incoming registration requests
#define NUM_CLN_ECB     3              // client RPC ecbs
#define NUM_CNS_ECB     3              // consumer ecbs
#define NUM_MCST_ECB    2              // incoming mcasts
#define IPX_DATA_SIZE   512
#define IPX_SHORTBUFFER_SIZE (IPX_DATA_SIZE + sizeof(ProducerHeaderStruct4) + sizeof(short))
//#define IPX_BUFFER_SIZE 1470
#define IPX_BUFFER_SIZE 1466 // for Client32 for DOS

// TCP/UDP specific:
#define NumTCPChan	 5

#define	UDP_PORT	     9003
#define	TCP_PORT	     8503
#define TCP_LEGACYPORT 8003
#define GCAST_PORT     8004
#define MCAST_PORT     9503

#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif

#define UDP_BUFFER_SIZE 1472

#define WORK_SIZE  (WORK_BUFFER_SIZE*sizeof(double))

// ECB allocated memory:
typedef struct ipxecb
{
  ECB       ecb;
  IPXHeader hdr;
  BYTE      buf[IPX_BUFFER_SIZE];
} IPXECB;
typedef struct spxecb
{
  ECB       ecb;
  SPXHeader hdr;
  BYTE      buf[IPX_DATA_SIZE];
} SPXECB;
extern SPXECB far *rcvECB[NUM_RCV_ECB];
extern SPXECB far *conECB;
extern SPXECB far *sndECB;
extern IPXECB far *cnsECB[NUM_CNS_ECB];
extern IPXECB far *glbECB[NUM_GLB_ECB];
extern IPXECB far *PCECB;
extern IPXECB far *asyECB;
extern IPXECB far *pnpECB;
extern IPXECB far *mcstECB[NUM_MCST_ECB];
extern DWORD IPXTaskID,SPXTaskID;
#define SPXIPXMemPoolSize ((NUM_RCV_ECB+2)*sizeof(SPXECB)+\
                           (NUM_CNS_ECB+NUM_GLB_ECB+NUM_MCST_ECB+3)*sizeof(IPXECB))
extern HANDLE hSPXIPXMemPool;
extern BYTE far *SPXIPXMemPool;
extern WORD SPXsocket;
extern WORD CNSsocket,ASYsocket;

#define initSPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
	(e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(SPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_DATA_SIZE; }

#define initIPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
    (e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(IPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_BUFFER_SIZE; }


#define AssignOutgoingSubscription(p,buf) {\
 if (p == 4) {\
   s = (struct subscription4 far *)buf;\
   s->msgsize = SUBSCRIPTION4_SIZE + dsize;\
   s->PollingRate = 0;\
   s->id = 0;\
   s->mode = SINGLE;\
   s->starttime = time(NULL);\
   _fstrncpy(s->contract.EqpName,name,8);\
   _fstrncpy(s->contract.Property,prop,12);\
   s->contract.EqpSize = sizeOut;\
   if (access & CA_WRITE) s->contract.EqpSize *= -1;\
   s->contract.EqpFormat = formatOut;\
   s->contract.EqpNumber = EqNo;\
 } else {\
   sEx = (struct subscription far *)buf;\
   sEx->msgsize = SUBSCRIPTION_SIZE + dsize;\
   sEx->PollingRate = 0;\
   sEx->id = 0;\
   sEx->blknum = 1;\
   sEx->numblks = 1;\
   sEx->blkid = 0;\
   sEx->mtu = IPX_BUFFER_SIZE-SUBSCRIPTION_SIZE-HEADERSTRUCT_SIZE;\
   sEx->mode = SINGLE;\
   sEx->starttime = time(NULL);\
   _fstrncpy(sEx->contract.EqpName,name,8);\
   _fstrncpy(sEx->contract.EqpProperty,prop,32);\
   _fstrncpy(sEx->contract.EqpDeviceName,devname,16);\
   sEx->contract.EqpSizeOut = sizeOut;\
   sEx->contract.EqpSizeIn = sizeIn;\
   sEx->contract.EqpAccess = access;\
   sEx->contract.EqpFormatOut = formatOut;\
   sEx->contract.EqpFormatIn = formatIn;\
   _fstrncpy(sEx->contract.strTagIn,"",8);\
   _fstrncpy(sEx->contract.strTagOut,"",8);\
 }\
}
#define PrepIncomingSubscription(p,dat) {\
 if (p == 4) {\
   Pheader = (ProducerHeaderStruct4 far *)dat;\
   PheaderEx->msgsize = Pheader->msgsize;\
   PheaderEx->SubscriptionID = Pheader->SubscriptionID;\
   PheaderEx->numblks = 1;\
   PheaderEx->blknum  = 1;\
   PheaderEx->mtu = UDP_BUFFER_SIZE;\
   PheaderEx->EqpSize = sizeOut;\
   PheaderEx->CompletionCode = Pheader->CompletionCode;\
   PheaderEx->EqpFormat = formatOut;\
   PheaderEx->counter = Pheader->counter;\
   PheaderEx->timestamp = Pheader->timestamp;\
   PheaderEx->starttime = Pheader->starttime;\
 }\
 else PheaderEx = (ProducerHeaderStruct far *)dat;\
}
// ==========================================================
// ======            externals:                     =========
// ==========================================================
extern HANDLE hDLL,hDLLWnd;
extern NWCONN_HANDLE hConn, hControl;
extern int RPCNumRecsRead, RPCNumFecsRead;
extern int TimerPeriod;
extern HCURSOR WaitCurs;
extern char far *erlst[];
extern int numErr;
extern BrdcstListStruct far *BrdcstList;
extern int nglobals, TCPloaded, IPXloaded;
extern RPCFecStruct  far *RPCFec;
extern BYTE far *WorkArea;
extern unsigned long WorkAreaSize;
extern BYTE far *Values;
extern HANDLE hWorkArea,hValues;
extern RPCDataStruct far *RPCData;
extern RPCFecStruct  far *RPCFec;
extern HANDLE hRPCData, hRPCFec;
extern HANDLE hBrdcstList;
extern DataBaseElemsStruct RPCDataElems[];
extern BYTE rpc_socket[2];
extern BYTE reg_socket[2];
extern const BYTE brdcst_socket[2];

// ==========================================================
// ======            prototypes:                    =========
// ==========================================================

int CreateElem(int ID);
HANDLE CreateClient(int id,HANDLE hElemPointer,long Timer);

void RemoveElem(HANDLE);
HANDLE GetElemPointer(LCLCONTRACT far *c);
void SendData (HWND, HWND, char far *, char far * ,double far *, int,BOOL ,BOOL ,BOOL ,int,int);
void ValueToStr(double far *Values,char far *ValDesc,int Datasize,int RowNum,int ColNum,char far *EqName,char far *Property,BOOL Format);
int StrToValue(double far *Value,char *Command,char far *EqName);
int RPCSync(char far *Name,int EqNo, char far *Property,int Size,void far *Data,int Datasize,int Format);
short syncAsyncRPC(char far *name,short EqNo,char far *prop,
                   short size,void far *data,short datasize,short format,UINT16 timout);
void InvalidReadSyntax(HWND hWnd);
int FPostMessage(HWND hWnd,WORD wm,WORD wParam,LONG lParam);
int AttachToControlServer(void);
int DetachFromControlServer(void);
int addFecToList(char far *name);
void trim(char far *str);
void DoRPC(void);
void RemoveRPC(int ID);
void GetIPXData(int num,IPXECB far *e[]);
void GetUDPData(int sck);
void CheckConnections(void);
void CheckSubscriptions(void);
void Cleanup(void);
void doGlobalUpdate(BYTE far *buf);
int listenUDP(int far *s,unsigned short port);
void PrepRequest(int size,HANDLE hElemStart);
int SendRequest(int size,HANDLE hElemStart);
WORD connectSPX(HANDLE hElem);
int connectTCP(HANDLE hElem);
void SPXSendData(WORD ID);
void TCPSendData(int socket);
int getSPXStream(WORD ID);
int getTCPStream(int socket);
int decodeData(int cc,int size,HANDLE hElemStart);
void NotifyClients(ElemDataStruct far *e);
short consumeData(BYTE far *buf);
int PrepSubBuffer(BYTE far *data);
int SendSubBuffer(int FECindex,BYTE far *data);
int SendIPXData(CLIENT far *cln,BYTE far *data);
int SendUDPData(CLIENT far *cln,BYTE far *data,int UDPsocket);
short Subscribe(HWND hWnd,HWND hVBX,char far *name,short EqNo,char far *prop,
                long timer,short size,short format,short mode);
int initApplication(void);
int AllocateAndAssignMemory(void);
int initSPX(void);
int initTCP(void);
int initDataBase(void);
void initNetGlobals(void);
void GetGlobalIPX(void);
void GetGlobalUDP(void);
void FreeApplication(void);

int tExecLink(int fecindex,char far *name,char far *dev,char far *prop,
	   short size,char far *data,short format,short timout);
int FAR PASCAL _export RPC(char far *name, int EqNo, char far *prop, int size, void far *data, int datasize, int format);
int FAR PASCAL _export RPCEx(char far *name, int EqNo, char far *prop, int size, void far *data, short datasize, int format,long timeout);
int FAR PASCAL _export HOTRPC(HWND hVBWnd, char far *name, int EqNo, char far *prop, long timer, int size, int format);
int FAR PASCAL _export CLOSERPC(HWND hVBWnd, int ID);
int FAR PASCAL _export GETRPC(int ID, void far *buf);
int FAR PASCAL _export GETGLOBAL(char far *keyword, void far *data, int size, int format);
int FAR PASCAL _export PUTRPC (int i, void far *VBbuf);
void FAR PASCAL IncomingIPX(void);
void FAR PASCAL IncomingIPXMCast(void);
void FAR PASCAL IncomingGlobalIPX(void);
int FAR PASCAL IncomingUDP(unsigned long i, int err);
int FAR PASCAL _export FILLSTRUCT (char far *tag, short addr, short size, short fmt);

#endif //_RPCSRV_H_
