#ifndef _ALMSRV_H_
#define _ALMSRV_H_
#include <time.h>
#ifndef BYTE
#  define BYTE unsigned char
#endif

#define CASHOST_FILE "cashosts.csv"
#define EXPORT_FILE "exports.csv"

#define MAXALARMS 100   // put this in project.def later
#define DEFAULT_ALARM_WINDOW 3
#define DEFAULT_ALARM_HEARTBEAT 1200
#define DEFAULT_INSTANT_WINDOW 4
#define DEFAULT_TERMINATION_WINDOW 30

#define almNEWALARM     0x01
#define almHEARTBEAT    0x02
#define almOSCILLATION  0x04
#define almDATACHANGE   0x08
#define almINSTANT      0x10
#define almTERMINATE    0x40
#define almTEST         0x80

typedef struct AlarmMsgStruct
{
  char tag[16];
  char device[16];
  char alarmTag[16];
  long alarmCode;
  time_t timestamp;
  unsigned long alarmMask;
  BYTE alarmData[6];
  BYTE alarmDataFormat;
  BYTE alarmDataArraySize;
  BYTE severity;
  BYTE descriptor;
  UINT16 alarmSystem;
} AMS;                    /* AMS sent out upon request */
#define AMS_SIZE  72

typedef struct ADStag    /* Alarm Definition Structure */
{
   char alarmTag[16];
   long alarmCode;
   unsigned long alarmMask;
   short severity;
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   float timeRes;
   short hitsChannel;
   short hitsModule;
   char alarmText[40];
   char deviceText[38];
   UINT16 alarmSystem;
   char dataText[40];
   char url[40];
   HANDLE hNext;
   short reserved;
} ADS;
#define ADS_SIZE  200

typedef struct AMRBtag
{
  long almStartTime;
  long  almStopTime;
  long almMinSeverity;
  long reservered;
} AMRB;                   /* Alarm Message Request Block */
#define AMRB_SIZE  16

typedef struct ALARMstruct/* local alarm list          */
{
  time_t timestamp;       /* our old friend                                */
  long   alarmCode;       /* standardized return code (implies severity)   */
  int    removalBlocked;
  BYTE   alarmData[6];    /* alarm specific data (database interpreted)    */
  BYTE   alarmStatus;     /* corresponds to the descriptor                 */
  BYTE   clrCount;        /* record of clear requests                      */
  HANDLE hNext;
} ALARM;

extern HANDLE hAlmList;
extern ALARM far *almList;

#define idx_too_high      0
#define idx_warn_too_high 1
#define idx_too_low       2
#define idx_warn_too_low  3

#define ALMARM_WATCH_FILE "almwatch.csv"
typedef struct AlarmWatchStruct
{
  char eqm[8];
  char dev[32];
  char prp[32];
  int  siz;
  int  fmt;
  int  sev;
  int  cnt;
  float hi;               /* high threshold should be < max */
  float lo;               /* low threshold should be > min */
  float hiwarn;           /* high warn threshold */
  float lowarn;           /* low warn threshold */
  int  interimStatus;
  int  interimCounter;
  HANDLE hData;           /* allocated data buffer */
} ALARM_WATCH;

extern HANDLE hAlmWatchList;
extern ALARM_WATCH far *almWatchList;
extern int nAlmWatchList;
extern ADS far almWatchTableADS[];

void SurveyAlarms(void);
int __export __loadds FAR PASCAL SetAlarm(char far *tag,short devNr,long almCode,BYTE far *almData);
int __export __loadds FAR PASCAL SetAlarmEx(char far *tag,short devNr,long almCode,BYTE far *almData,BYTE almflags);
int getAlarms(char far *tag,AMS far *ams,short num,short minSeverity);
int getAlarmsEx(char far *eqpname,AMS far *ams,short num,short minSeverity,time_t starttime,time_t stoptime,short startnum);
int numAlarms(char far *tag,time_t far *almTimeStamp,short far *almSeverity);
int numAlarmsEx(char far *eqpname,time_t far *almTimeStamp,short far *almSeverity,short *NrAtAlmTimeStamp,short far *NrAtAlmSeverity);
void __export __loadds FAR PASCAL ClearAlarm(char far *tag,short devNr);
void __export __loadds FAR PASCAL RemoveAlarm(char far *name,short devNr,long code);
short findSeverity(char far *eqm, long cc);
int fillHRTstamp(BYTE far *HRTstamp);
int getAlarmInfoTable(char far *eqname);
int getAlarmDefs(char far *eqm,ADS far *rads,int num);
int numAlarmDefs(char far *eqm);
HANDLE getAlarmTable(char far *eqm,long code);
int isCASInUserList(void);

extern char far *tagList[];
extern int numStoredAlarms;
extern int AlarmWindow;
extern int AlarmHeartbeat;
extern int NTAGS;
extern int NSEVERITYCODE;
extern char gCASName[17];
extern int gRequireReadFromCAS;

#endif /* _ALMSRV_H_ */
