#ifndef _HISTORY_H_
#define _HISTORY_H_

#ifndef MAX_HISTORY_RECORDS
# define MAX_HISTORY_RECORDS 20
#endif

typedef struct
{
  CONTRACT c;
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthShort;             /* for short term storage */
  int  depthLong;               /* for long term storage */
  int  dataPtr;
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
  int  recordIndex;            /* local index identifier */
  long  recordLength;
  time_t lastArchiveTime;
  char ArchiveFileName[16]; /* current working file name */
  HANDLE hTimestamp;
  HANDLE hData;
//  time_t *timestamp;                  /* timestamp array */
//  BYTE *data;
} HistoryRecord;

typedef struct
{
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthShort;             /* for short term storage */
  int  depthLong;               /* for long term storage */
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
} HistorySpecification;

#define HISTORY_FILENAME "history.csv"

#define HISTORY_BUFFER_SIZE 0x1000
#define STS_DEADBAND 2

extern char ArchiveLocation[128];
extern HistoryRecord far *historyRecords;
extern int nHistoryRecords;
extern int maxHistoryRecords;
extern HANDLE hHistoryRecords;

int __export __loadds FAR PASCAL AppendHistoryInformation(
  char far *eqp,char far *prp,char far *dev,
  int len,int fmt,int idx,HistorySpecification far *hspec);
int LoadHistoryInformation(char far *fname);
void historyInit(void);
void historyCycle(void);
long loadLTSData(int i,int index,time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 void far *datbuf,int datfmt,int bufsize,int sample);
long getLTSData(char far *eqp,char far *prp, char far *dev,int index,
                 time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 BYTE far *data,int fmt,int arraysize,int sample,int recsreadFlag);
long getSTSData(char far *eqp,char far *prp, char far *dev,int index,
                 time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 BYTE far *data,int fmt,int arraysize,int sample,int recsreadFlag);
int HISTQueryFunction(CONTRACT far *con,char far *dataIn,char far *data);
void assignNumberOfPoints(BYTE far *data,int fmt,int siz,int far *npoints,int far *nfound,int recsreadFlag);
int isInRange(BYTE far *data,int fmt,int number,float lwrLimit,float uprLimit);
long getStartTimeFromDepth(char far *eqp,char far *prp, int depth);

#endif /* _HISTORY_H_ */
