#ifndef _WINSRV_H_
#define _WINSRV_H_

#include <nwipxspx.h>
#include <winsock.h>
#include "rpcdefs.h"
#include "almsrv.h"
#include "database.h"

#define RPCERR_SIZE 32
extern char RPCLastErrorString[RPCERR_SIZE];

#define BUFLEN 1280
extern char txtbuffer[BUFLEN+26]; // make sure the last ctime() fits

#define BURST_LIMIT 10
extern int gBurstLimit;
#define MIN_POLLRATE        1000
#define DEFAULT_POLLRATE    30000
#define MAXALLOWED_POLLRATE 65535L
#define MSECS(a,b) ((a).tv_sec-(b).tv_sec > MAXALLOWED_POLLRATE ? MAXALLOWED_POLLRATE :\
		   ((a).tv_sec-(b).tv_sec)*1000+((a).tv_usec-(b).tv_usec)/1000)
#define FEC_NAME_SIZE     16
#define EXPORT_NAME_SIZE  16
#define EQP_NAME_SIZE      6
#define EQP_PROP_SIZE     32
#define EQP_DEVICE_SIZE   16
#define BYTE        unsigned char
#define MAX_ENTRIES      100 // connection table entries (* 70 Kbytes)
#define FEC_TIMEOUT      1   // seconds
#define SERVER_TIMEOUT   45  //clock ticks
#define SYSTEM_DELAY     20 // msecs
#define INTSWAP(w)     ((w<<8)|(w>>8))
#define hNULL             0
#define MAX_EQP_FCN       20

#define CONNECT_MSG    (WM_USER+1)
#define RECEIVE_MSG    (WM_USER+2)
#define ACTIVATE_MSG   (WM_USER+3)

#define WM_CONNECT_UDP     (WM_USER+1)
#define WM_CONNECT_TCP     (WM_USER+2)
#define WM_CONNECT_IPX     (WM_USER+3)
#define WM_CONNECT_SPX     (WM_USER+4)
#define WM_INITAPPLICATION (WM_USER+5)
#define WM_TCALL           (WM_USER+6)
#define WM_CREATESTRUCT    (WM_USER+7)
#define WM_READ_UDP        (WM_USER+8)
#define WM_READ_TCP        (WM_USER+9)

#define ccerr(e) {cc = e; goto err; }
#define DeReference(h)  GlobalLock(h); GlobalUnlock(h);
#define OFFSETIN(struc,field)	((unsigned short)&(((struc *)0L)->field))

#define IPX	    0x01
#define SPX     0x02
#define TCP     0x04
#define UDP     0x08

// IPX/SPX specific:
#define RPC_SOCKET      0x45 // listening SPX socket
#define BRDCST_SOCKET   0x67 // listening broadcast socket
#define REG_SOCKET      0x55 // listening registration socket
#define MCAST_SOCKET    0x85 // listening mcast socket
#define NUM_LSN_ECB     2              // allowed simultaneous connections
#define NUM_RCV_ECB    (NUM_LSN_ECB*2) // associated incoming contracts
#define NUM_GLB_ECB     2              // incoming broadcasts
#define NUM_REG_ECB     2              // incoming registration requests
#define NUM_CLN_ECB     3              // client RPC ecbs
#define NUM_CON_ECB     5              // consumer ecbs
#define NUM_MCST_ECB    2              // incoming mcasts
#define IPX_DATA_SIZE   512
#define IPX_BUFFER_SIZE 1466 // the stupid 32-bit client for DOS can't
                             // handle 1470 like everybody else

// TCP/UDP specific:
#define NumTCPChan	  5
#ifndef UDP_PORT
# define	UDP_PORT         9003
#endif
# define TCP_LEGACYPORT    8003
#ifndef TCP_PORT
# define TCP_PORT          8503
#endif
#ifndef GCAST_PORT
# define GCAST_PORT        8004
#endif
#ifndef MCAST_PORT
# define MCAST_PORT        9503
#endif

#define UDP_BUFFER_SIZE 1472
#define WORK_BUFFER_SIZE 0x1000
#define BCAST_ID 255
#define PRP_SCHEDULE_SIGNAL 0
#define PRP_RETRY_SIGNAL 1
#define PRP_CANCEL_SIGNAL 2

#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif

#define EMPTY_PROP    0
#define TARGET_PROP   1
#define WILDCARD_PROP 2
#define STOCK_PROP    3

extern HANDLE hWinSrv;
extern HANDLE hDLLWnd;
extern char fecname[17];
extern char gDeviceContext[17];
extern time_t startuptime;
extern long SysPoll;
extern unsigned short MinPollingRate;
extern unsigned short MaxPollingRate;

extern short EqpFormats[];
extern int TCPParent,TCPindex,TCPloaded;
extern int TCPerrcnt,TCPcumcnt;
extern struct fd_set TCPrset,rwset,exset;
extern struct timeval TCPto,fec_to;
extern struct sockaddr addr[NumTCPChan];
extern int TCPsocket[NumTCPChan],TCPrc[NumTCPChan],
           TCPclose_rc[NumTCPChan],TCPconnection[NumTCPChan];
extern int addrsize;
extern int nUDPsets;

extern FARPROC lpfnListenConnectTCP,lpfnListenConnectSPX;
extern FARPROC lpfnListenRequestUDP,lpfnListenRequestIPX;
extern FARPROC lpfnWinSrvTimer;

#define LEGACY_PROTOCOL 2
#define STD_PROTOCOL    5
#define HST_PROTOCOL   10
#define ALM_PROTOCOL   15

#define PROTALM  0x4000
#define PROTHIST 0x2000
#define PROT1ID  0x1000
#define PROT5ID  0x0000

#define MAKEPROTID(i,p) ((p) < LEGACY_PROTOCOL ? ((i)|PROT1ID) :\
                         (p) < HST_PROTOCOL    ? ((i)|PROT5ID) :\
                         (p) == HST_PROTOCOL   ? ((i)|PROTHIST): ((i)|PROTALM))

#define MAKERPCID(i)  ((i)&0xfff)

typedef struct
{
  char name[8];
} NAME8;

typedef struct
{
  char name[16];
} NAME16;

typedef struct
{
  char name[16];
  UINT32 ival;
} NAME16I;

typedef struct
{
  char name[32];
} NAME32;

typedef struct
{
  char name[32];
  UINT32 ival;
} NAME32I;

typedef struct
{
  char name[48];
} NAME48;

typedef struct
{
  char name[48];
  UINT32 ival;
} NAME48I;

typedef struct
{
  char name[64];
} NAME64;

typedef struct
{
  char name[64];
  UINT32 ival;
} NAME64I;

typedef struct
{
  char   name[16];
  float  fval;
  UINT32 ival;
} NAME16FI;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
} INTINT;

typedef struct
{
  UINT32 ival;
  double dval;
} INTDBL;

typedef struct
{
  UINT32 lval;
  double dval;
} LNGDBL;

typedef struct
{
  float  fval;
  UINT32 ival;
} FLTINT;

typedef struct
{
  float  fval;
  double dval;
} FLTDBL;

typedef struct
{
  double dval;
  UINT32 ival;
} DBLINT;

typedef struct
{
  double d1val;
  double d2val;
} DBLDBL;

typedef struct
{
  float  fval;
  UINT32 i1val;
  UINT32 i2val;
} FLTINTINT;

typedef struct
{
  UINT32 i1val;
  float  fval;
  UINT32 i2val;
} INTFLTINT;

typedef struct
{
  time_t t1;
  time_t t2;
  UINT32 i1val;
  UINT32 i2val;
} DWINDOW;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
  float  f1val;
  float  f2val;
} FWINDOW;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  time_t tm;
  char   str[80];
} DUSTRING;

typedef struct
{
  float  xval;
  float  yval;
  UINT32 ival;
} XYS;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  float  f3val;
} DFILTER;

typedef struct
{
  char   comment[80];
  time_t tm;
  float  s_start;
  float  s_inc;
  UINT32 status;
  float  d_spect_array[1024];
} DSPECTRUM;

typedef union
{
  void   far *vptr;
  char   far *cptr;
  UINT8  far *bptr;
  UINT16 far *sptr;
  UINT32 far *lptr;
  float  far *fptr;
  double far *dptr;
} DUNION;

typedef union
{
  void   huge *vptr;
  char   huge *cptr;
  UINT8  huge *bptr;
  UINT16 huge *sptr;
  UINT32 huge *lptr;
  float  huge *fptr;
  double huge *dptr;
} HDUNION;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[16];
  DUNION data;
} DTYPE;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[16];
  HDUNION data;
} HDTYPE;

typedef struct PropertyEGU
{
  char units[16];
  double min;
  double max;
  BYTE graph;
  BYTE reserved[3];
} PropertyEGUStruct;

typedef struct ExportPropertyListTag
{
  char  prpName[32];       /* property name */
  char  prpAlias[32];      /* property alias name */
  short prpId;             /* property id */
  UINT16 prpSizeIn;        /* maximum supported array size */
  UINT16 prpSizeOut;       /* maximum supported array size */
  short prpFormatIn;       /* native format */
  short prpFormatOut;      /* native format */
  short prpArrayType;      /* property array type if array */
  short numRows;           /* num rows if array */
  short rowSize;           /* row size if array */
  char  strTagIn[8]; 	     /* structure tag */
  char  strTagOut[8];      /* structure tag */
  int   structSizeIn;      /* input structure size */
  int   structSizeOut;     /* output structure size */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  HANDLE hPrpRedirection;  /* re-director if not hNULL */
  char  prpDescription[64];/* short description of property */
  PropertyEGUStruct egu;   /* EGU information for display */
  PropertyEGUStruct xegu;  /* x-axis EGU information for spectrum display */
  HANDLE hNext;
//struct ExportPropertyListTag *next;
} ExportPropertyListStruct;

#define PROPERTY_HASH_SIZE 211
#define PROPERTY_HASH_INDEX(i) ((i) & 0xff)
#define MAKE_PROPERTY_ID(i,indx) (((i) << 8) | (indx))

#define MODULE_HASH_SIZE 211
#define MODULE_HASH_PAYOFF 25 /* don't hash if fewer modules than this */

typedef struct HashXRef
{
  int index;
  HANDLE hNext;
} HashXRefStruct;

#define STOCKPROP_HASH_SIZE 103

extern HANDLE hhStockPropXRefTable;
extern HANDLE far *hStockPropXRefTable;

#ifndef ALIAS_HASH_SIZE
# define ALIAS_HASH_SIZE 103
#endif
#define ALIAS_HASH_PAYOFF (ALIAS_HASH_SIZE/10)
typedef struct
{
  char name[32];
  char alias[32];
} ALIAS_TABLE;
extern ALIAS_TABLE far *gAliasTable;
extern HANDLE hAliasTable;
extern int nAliasTableEntries;
//extern HashXRefStruct **AliasXRefTable;
extern HANDLE hhAliasXRefTable;
extern HANDLE far *hAliasXRefTable;
int makeAliasXRefTable(void);
int GetAliasTable(void);
//int getTableIndex(char *str,HashXRefStruct **xrf,int hshsiz,char *tbl,int tblsiz,int off,int reclen);
int getTableIndex(char far *str,HANDLE hhXrf,int hshsiz,char far *tbl,int tblsiz,int off,int reclen);
#define getAliasTableIndex(str) \
   getTableIndex(str,hhAliasXRefTable,ALIAS_HASH_SIZE,(char far *)gAliasTable,nAliasTableEntries,OFFSETIN(ALIAS_TABLE,alias),sizeof(ALIAS_TABLE))
#define getAliasTableIndexFromName(str) \
   getTableIndex(str,NULL,0,(char far *)gAliasTable,nAliasTableEntries,OFFSETIN(ALIAS_TABLE,name),sizeof(ALIAS_TABLE))
#define GetStockPropertyId(str) \
  getTableIndex(str,hhStockPropXRefTable,STOCKPROP_HASH_SIZE,(char far *)StockProperty,NrStockProperties,OFFSETIN(StockPropertyStructEx,prpName),sizeof(StockPropertyStructEx))

typedef struct ExportListTag
{
  char   Context[16];       /* device context */
  char   tagName[16];       /* exported equipment name tag */
  char   EqpName[8];        /* local equipment name tag */
  char   EqpAlias[8];       /* local equipment name tag alias */
  int    exportId;          /* local ID */
  HANDLE hEqpProperty[PROPERTY_HASH_SIZE+1];	/* pointer to chained equipment property hash list */
  HANDLE hEqpModuleName;    /* pointer to device name list */
  HANDLE hEqpModuleXRef[MODULE_HASH_SIZE]; /* ModuleName cross-reference */
  int    hasPropertyDeviceNames;
  short  EqpNumProperties;  /* total number of properties supported */
  short  EqpNumModules;     /* total number of devices supported */
  short  registered;        /* registered with name server */
  HANDLE hEqpFcn;
//ADS    *adsTable;         /* alarm definition structure table */
  HANDLE hAdsTable;
  short  alarmSystem;       /* alarm system flag */
//ALARM  **almLst;          /* associated alarm list */
  HANDLE hhAlmLst;
  short  nalarms;           /* number of active alarms */
  time_t almMRT;            /* alarm Most Recent Timestamp */
  time_t almMRCT;           /* alarm Most Recent Complete Timestamp */
  short  almHSV;            /* alarm highest Severity */
  short  nAlmMRT;           /* number of alarms at MRT */
  short  nAlmMRCT;          /* number of alarms at MRCT */
  short  nAlmHSV;           /* number of alarms at HSV */
  short  nclients;          /* number of active clients */
  short  ncontracts;        /* number of active contracts */
  HANDLE hNext;
//struct ExportListTag *next;
}ExportListStruct;

extern ExportPropertyListStruct far *ExportPropertyList;
extern ExportListStruct far *ExportList;
extern int NrOfEqpTypes;

typedef struct
{
  char  prpName[32];
  char  prpDescription[32];
  UINT16 prpSize;
  BYTE prpFormat;
  BYTE prpAccess;
} PropertyQueryStruct;
#define PROPERTYQUERYSTRUCT_SIZE (32+32+2+1+1)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
} PropertyQueryStructEx;
#define PROPERTYQUERYSTRUCTEX_SIZE (32+64+32+8*2+16+4*5+2*2+4)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
  char  rngUnits[16];
  float rngMinValue;
  float rngMaxValue;
  UINT16 numRows;
  UINT16 rowSize;
  UINT16 prpArrayType;
  UINT16 reserved[3];
} XPropertyQueryStruct;
#define XPROPERTYQUERYSTRUCT_SIZE (PROPERTYQUERYSTRUCTEX_SIZE + 16+2*4+6*2)

typedef struct
{
  char FecName[FEC_NAME_SIZE];
  char reserved[4];
  time_t localtime;
  time_t starttime;
  long   systemPollingRate;
  short  numBkgTsks;
  short  numTotalContracts;
  short  numTotalClients;
  short  numTargetContracts;
  short  numTargetClients;
  short  numConnections;
  UINT32 numConnectionTimeouts;
  UINT32 numConnectionArrivals;
  UINT32 numUDPpkts;
  UINT32 numTCPpkts;
  UINT32 numIPXpkts;
  UINT32 numSPXpkts;
} ActivityQueryStruct;
#define ACTIVITYQUERYSTRUCT_SIZE (FEC_NAME_SIZE+4+4*3+2*6+4*6)

typedef struct
{
  char desc[64];
  BYTE dataFmt;
  BYTE dataSiz;
  BYTE data[64];
  time_t timeEntered;
  UINT32 codeNow;
  UINT32 codePrv;
  UINT32 codeNxt;
} ServerStateStruct;
#define SERVERSTATESTRUCT_SIZE (64+2+64+4*4)

typedef struct
{
  UINT32 AveBusyTime;
  UINT32 CycleCounts;
  UINT32 MaxCycleCounts;
  UINT32 SingleLinkCount;
  UINT32 ClientMisses;
  UINT32 ClientReconnects;
  UINT32 ClientRetries;
  UINT32 ContractMisses;
  UINT32 ContractDelays;
  UINT32 BurstLimitReachedCount;
  UINT32 DataTimeStampOffset;
} ServerStatsStruct;
#define SERVERSTATSSTRUCT_SIZE (11*4)

// common definitions:
#define WORK_SIZE  (WORK_BUFFER_SIZE*sizeof(double))
#define BRDCST_BUFFER_SIZE IPX_BUFFER_SIZE

typedef struct
{
  int TotalSize;
  char UserName[16];
  int Protocol;
  int EqpSize;
} HeaderStruct;
#define HEADERSTRUCT_SIZE  22

typedef struct
{
  char ElemName[16];
  char EqpModName[8];
  int EqpModNo;
  char Property[12];
  char Cycle[22];
  int Format;
  int DataSize;
} EqpInfoStruct;

typedef struct
{
  char UserName[16];
}UserListStruct;

typedef struct
{
  char name[16];       // broadcast element name  (16 bytes)
  int  size;           // number of elements broadcast under "name" (2 bytes)
  int  format;         // data format (as per RPC definitions)  (2 bytes)
  long time;           // timestamp of broadcast data (4 bytes)
} BrdcstData;
#define BRDCSTDATA_HEADER_SIZE (16 + 2 + 2 + 4)

typedef struct
{
  char EqpName[8];      // Registered Equipment function name
  char Property[12];    // property
  int  EqpSize;         // maximum supported array size
  int  EqpFormat;       // lowest common denominator format
  int  EqpType;         // 1 = Read, -1 = Write/Read
  char Description[50]; // short description of property
}PropertyListStruct;

typedef struct
{
  char  FecName[FEC_NAME_SIZE];
  char  reserved[5];
  char  FullName[EXPORT_NAME_SIZE];
  char  EqName[EQP_NAME_SIZE+1];
  char  Context[16];
  short alarmSystem;
} RPCDataStruct;
#define RPCDATASTRUCT_SIZE (FEC_NAME_SIZE+5+EXPORT_NAME_SIZE+EQP_NAME_SIZE+1+16+2)

typedef struct
{
  char Name[FEC_NAME_SIZE];
  BYTE reserved[3];
  BYTE Port[2];
  BYTE Network[4];
  BYTE Node[6];
  BYTE ImmediateAddress[6];
  char IP[16];
  BYTE PortOffset;
  char IPh_addr[4];
  short  Prot;
}RPCFecStruct;
#define RPCFECSTRUCT_SIZE (FEC_NAME_SIZE+3+2+16+21+2)

typedef struct
{
  char OS[12];
  char desc[64];
  char loc[32];
  char hdw[32];
  char sub[16];
  char resp[32];
} RPCFecExStruct;

extern RPCFecExStruct RPCFecEx;
extern char gDeviceContext[17];

typedef struct
{
  BYTE net[4];  /* IPX Network Address */
  BYTE mac[6];  /* IPX Node (or MAC) Address */
  BYTE node[6]; /* IPX Immediate Address */
}
IPXNetListStruct;


#undef NONE
#define NONE   -1
#define RTOL    0  /* Swap: Remote to Local */
#define LTOR    1  /* Swap: Local to Remote  */

struct structFormat
{
  int   siz;
  int   fmt;
  int   off;
  int  addr;
  HANDLE hNxt; // struct structFormat *nxt;
};

struct structStruct
{
  char name[12];
  short rawlen;
  short size;
  short num;
  HANDLE hS; // struct structFormat *s;  /* structure definition */
//  struct structStruct *next;
  HANDLE hNext;
};

extern struct structStruct far *structList;
struct structStruct far *getStructFromTag(char far *tag);
struct structStruct far *findstruct(char far *tag);
int addtostruct(HANDLE far *hTarget,int addr,int size, int fmt);
int queryStructFormat(char far *tag,INTINT far *sfq,int far *sfqsiz);

typedef struct
{
  char keyword[16];          /* broadcast keyword */
  short size;                /* number of elements in broadcast array */
  short format;              /* broadcast data format */
  time_t time;               /* timestamp of broadcast data */
  HANDLE hData;              /* data pointer */
  HANDLE hOlddata;           /* for comparison */
  int  minPeriod,            /* minimum period between broadcasts */
       maxPeriod;            /* maximum period between broadcasts */
  double tolerance;          /* allowed deviation of data (data units) */
  HANDLE hNext;
} GlobalListStruct;

extern HANDLE hBrdcstList;
extern GlobalListStruct far *BrdcstList;


// =============================================================
//
// Subscription oriented Producer-consumer definitions:
//
// =============================================================

#define NCONSUMER     100
#define NCONTRACT     100
#define RENEWAL       60
#define HEARTBEAT     60
#define REMINDER      5
#define ACK_REQUEST   (2*REMINDER)
#define REPLY_PENDING 3
#define SUB_PENDING   0x3fff
#define ACK_PENDING   0x7fff
#define CANCEL        CM_CANCEL
#define SINGLE        CM_SINGLE
#define REFRESH       CM_REFRESH
#define POLL          CM_POLL
#define REGISTER      CM_REGISTER
#define GROUPED       CM_GROUPED

extern char far *SubType[];
extern int nconsumer;
extern int ncontract;

typedef struct
{
  char  EqpName[8],
        Property[12];
  int   EqpSize,
	EqpFormat,
	EqpNumber;
  HANDLE hVBWnd;
  int   EqpNativeFormat;
} CONTRACT4;
#define CONTRACT4_SIZE 30

typedef struct             /* net: Protocol 5 (in) */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
} CONTRACT;
#define CONTRACT_SIZE (32+16+8+4*2+4*1+2*8)

typedef struct
{
  CONTRACT contract;
  long pollrate;
  long nconsumers;
} ContractQueryStruct;
#define CONTRACT_QUERY_SIZE (CONTRACT_SIZE + 4 + 4)

typedef struct
{
  short NativeFormatIn;
  short NativeFormatOut;
  UINT16 MaxSizeIn;
  UINT16 MaxSizeOut;
  short MaxBytesIn;
  short MaxBytesOut;
} CONTRACTeX;

typedef struct
{
  char               userName[16];
  IPXAddress         IPXaddress;  // if 0 then IP transfer is assumed
  BYTE               node[6];     // IPX routing node (Immediate Address)
  struct sockaddr_in IPaddress;
  short              count;       // number of registrations
  short              protocol;    /* target client protocol */
  short              transport;   /* transport protocol */
  int                sck;         /* communication socket */
} CLN;
#define CLIENT_SIZE (16 + 12 + 6 + 16 + 2 + 2 + 2 + 2)

typedef struct
{
  char               userName[16];
  IPXAddress         IPXaddress;  // if 0 then IP transfer is assumed
  BYTE               node[6];     // IPX routing node (Immediate Address)
  struct sockaddr_in IPaddress;
  short              count;       // number of registrations
  short              protocol;    /* target client protocol */
} CLNQS;

#define LOCK_UNLOCKED 0
#define LOCK_PREEMPTIVE 1
#define LOCK_PERSISTENT 2
#define LOCK_ABORT 3
typedef struct
{
  CLN cln;
  int lockType;
  int lockDuration;
  UINT32 lockToken;
  time_t lockStart;
} AccessLockStruct;

typedef struct                /* local */
{
  short  id;                  /* contract ID as seen by consumer */
  short  mode;                /* SINGLE,REFRESH,POLL,CANCEL */
  long   PollingRate;         /* requested polling rate */
  struct timeval lasttime;    /* time of most recent update */
  time_t starttime;           /* start of initial subscription */
  BYTE   counter;             /* counts remaining in subscription(s) */
  BYTE   Stale;               /* 1 = send new data to consumer */
  UINT16 numblks;             /* total number of message blocks */
  UINT16 blknum;              /* block number of this message */
  UINT32 nextDataSize;        /* next outgoing data size in bytes */
  UINT32 nextDataPtr;         /* next outgoing data array pointer */
  UINT32 BytesRemaining;      /* number of bytes data remaining */
  UINT32 BytesTotal;          /* total number of bytes data */
  UINT16 mtu;                 /* maximal transport unit */
  short  statusCode;          /* interim status */
  UINT32 misses;              /* data return after expiration */
  UINT32 reconnects;          /* client unexpectedly re-establishes link */
  int    isRetry;             /* the contract is being resubmitted by the client */
} ClientSTATUS;

typedef struct
{
  CLNQS cln;
} ClientQueryStruct;
#define CLIENT_QUERY_SIZE (16 + 12 + 6 + 16 + 2 + 2)

#define MAXLONGGRAMS 5
typedef struct
{
  long   haddr;
  UINT16 SubscriptionID;      /* supplied by the consumer upon registration */
  UINT16 blksin;              /* # blocks arrived from long datagram */
  UINT16 blkid;               /* block id of long datagram */
  UINT8  busy,
         reserved;
//  UINT8  *data;
  HANDLE hData;
  time_t blktime;
  time_t SubsStarttime;
} ContractProgress;
extern ContractProgress cp[MAXLONGGRAMS];

typedef struct ConStruct
{
  HANDLE hClient;
  ClientSTATUS status;
  HANDLE hNext;
} CONSUMER;

typedef struct                  /* local */
{
  HANDLE hConsumer;             /* assciated consumer list */
  int      nconsumer;           /* number of consumers for this contract */
  CONTRACT contract;            /* see above */
  CONTRACTeX  prp;              /* Contract property information */
  long     PollingRate;         /* in mseconds, smallest number wins ... */
  struct   timeval lasttime;    /* most recent data polled */
  struct   timeval lastmarked;  /* last time the data were marked STALE */
  time_t   dtimestamp;          /* data timestamp */
  int      dtimestampMSEC;      /* data timestamp (msec fraction) */
  UINT32   dataSizeOut;
  HANDLE hDataOut;              /* data to send  */
  HANDLE hDataIn;               /* data received */
  HANDLE hExp;                  /* associated export element */
  UINT16 contractID;
  int      pending;             /* pending flag for DeliverData() vs ProduceData() */
  int      expired;             /* expired flag for contract removal */
  UINT32   delays;              /* delayed scheduling due to pending */
  UINT32   misses;              /* missed scheduling (i.e. > 2x polling rate) */
} ContractListStruct;

typedef struct
{
  UINT16 msgsize;
  short SubscriptionID,         // supplied by the consumer upon registration
        CompletionCode,
        counter;                // tell consumer how much is left
  time_t timestamp,           // RPCtimestamp
	starttime;           // supplied by the consumer upon registration
} ProducerHeaderStruct4;
#define PRODUCER4_SIZE 16

typedef struct                  /* net: Protocol 5 (out) */
{
  UINT16 msgsize;               /* message size in bytes */
  UINT16 SubscriptionID;        /* supplied by the consumer upon registration */
  short  CompletionCode;        /* completion or interim status */
  UINT16 numblks;               /* total number of message blocks */
  UINT16 blknum;                /* block number of this message */
  UINT16 mtu;                   /* maximum transport unit */
  UINT16 timestampMSEC;         /* msec fraction of data timestamp */
  BYTE   EqpFormat;             /* Eqp Format of returned data */
  BYTE   counter;               /* tell consumer how much is left */
  time_t timestamp;             /* RPCtimestamp */
  time_t ClnStarttime;          /* supplied by the consumer upon registration */
} ProducerHeaderStruct;
#define PRODUCER_SIZE (7*2+2*1+2*4)
#define PMTU(s)  (((s) ? IPX_BUFFER_SIZE : UDP_BUFFER_SIZE) - PRODUCER_SIZE - sizeof(UINT16))

struct subscription4
{
   UINT16 msgsize;
   short PollingRate,
         id,
         mode;
   time_t   starttime;
   CONTRACT4 contract;
};
typedef struct subscription4 SUBSCRIPTION4;
#define SUBSCRIPTION4_SIZE (12+CONTRACT4_SIZE)

struct subscription         /* net: Protocol 5 (in) */
{
  UINT16    msgsize;
  UINT16    mtu;            /* maximum transport unit for incoming data */
  UINT8     numblks;        /* total number of message blocks           */
  UINT8     blknum;         /* block number of this message             */
  UINT16    id;
  short     mode;
  short     blkid;          /* (incremented) block id */
  UINT32    PollingRate;
  time_t    starttime;
  CONTRACT contract;
};
typedef struct subscription SUBSCRIPTION;
#define SUBSCRIPTION_SIZE (6*2+2*4+CONTRACT_SIZE)

typedef struct
{
  char prpName[32];
  UINT16 prpSize;          /* maximum supported array size */
  short prpFormat;         /* native format */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  char  prpDescription[32];/* short description of property */
  int  (far *fcn)(CONTRACT far *,char far *,char far *);
} StockPropertyStruct;

typedef struct
{
  char prpName[32];
  UINT32 prpSize;          /* maximum supported array size (output) */
  UINT32 prpSizeIn;        /* maximum supported array size (input)  */
  short prpFormat;         /* format (output) */
  short prpFormatIn;       /* format (input) */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  short prpArrayType;      /* output array type */
  char  prpStructTag[8];   /* output data structure tag */
  char  prpDescription[64];/* short description of property */
  int  (far *fcn)(CONTRACT far *,char far *,char far *);
} StockPropertyStructEx;

typedef struct
{
  char modName[16];
  int  (far *fcn)(CONTRACT far *,char far *,char far *);
} StockModifierStruct;
extern int NrStockModifiers;
extern StockModifierStruct StockModifier[];

extern HANDLE hContractList[NCONTRACT];  // ContractListStruct
extern HANDLE hConsumers[NCONSUMER];  // registered consumer list (CLN)

extern int StaleData;
extern int PCSocket;              // IPX Producer/Consumer send socket
extern int ProducerSocket;        // UDP Producer Socket
extern int ConsumerSocket;        // UDP Consumer Socket
extern int BrdcstSocket;          // UDP Broadcast Socket
extern int GlobalSocket;          // UDP Global listening Socket
extern struct sockaddr_in prod_name;
#define SendToPeer(c,d)	\
 ((cc=SendIPXData(c,d)) == ipx_socket_error ? SendIPData(c,d) : cc )
void doConsumer(void);

typedef struct
{
  char   dTag[8];
  short  dFormat;
  UINT16 dSize;
} LDS;                     /* Long Data Structure */
extern LDS ldsXRef[];
extern int nLdsXRef;

typedef struct tcpbckt
{
  int idx;                  // FEC index
  int sck;                  // TCP connection socket
  int ptr;                  // bucket pointer
  int siz;                  // bucket target size
  BYTE buf[UDP_BUFFER_SIZE];// bucket buffer
//  struct tcpbckt *nxt;      // this is a linked list
  HANDLE hNxt;
} TCPBCKT;
extern HANDLE hBcktList;
extern TCPBCKT far *bcktList;
TCPBCKT far *findBucket(int sck);
TCPBCKT far *getBucket(int sck);
void freeBucket(int sck);
void __export __loadds FAR PASCAL CloseConnectionSocket(int sck);
int __export __loadds FAR PASCAL AssignConnectionSocket(void);

// =========================================================================

extern AccessLockStruct currentAccessLock;
extern BYTE far *WorkArea;
extern unsigned long WorkAreaSize;
extern BYTE far *Values;
extern HANDLE hWorkArea,hValues;
extern IPXAddress FecAddress;

extern int RPCNumRecsRead,RPCNumFecsRead,RPCNumEqpsRead,FECallocNum;
extern unsigned long TCPcount,UDPcount,SPXcount,IPXcount,BRDcount;

// ==================================================================

extern HANDLE hUserList,hDeniedUserList;
extern UserListStruct far *UserList;
extern UserListStruct far *DeniedUserList;
extern int WRITEaccess,NrOfUsers;
extern int NrOfDeniedUsers;
extern int openToAllUsers;
extern int accessToAllUsers;
extern HANDLE hPropertyList;
extern PropertyListStruct far *PropertyList;
extern int NrOfProperties;

extern int numEqpFcn;
extern HCURSOR WaitCurs;
extern DWORD IPXTaskID;
extern WORD destConID;

extern WORD IPXsocket;
extern const BYTE rpc_socket[2];
extern const BYTE brdcst_socket[2];
extern const BYTE reg_socket[2];
// ECB allocated memory:
typedef struct ipxecb
{
  ECB       ecb;
  IPXHeader hdr;
  BYTE      buf[IPX_BUFFER_SIZE];
} IPXECB;
typedef struct spxecb
{
  ECB       ecb;
  SPXHeader hdr;
  BYTE      buf[IPX_DATA_SIZE];
} SPXECB;
extern SPXECB far *rcvECB[NUM_RCV_ECB];
extern SPXECB far *conECB[NUM_LSN_ECB];
extern SPXECB far *sndECB;
extern IPXECB far *regECB[NUM_REG_ECB];
extern IPXECB far *brdcstECB;
extern IPXECB far *PCECB;
extern IPXECB far *pnpECB;
#define SPXIPXMemPoolSize ((NUM_RCV_ECB+NUM_LSN_ECB+1)*sizeof(SPXECB)+\
                           (NUM_REG_ECB+3)*sizeof(IPXECB))
extern HANDLE hSPXIPXMemPool;
extern BYTE far *SPXIPXMemPool;

#define initSPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
    (e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(SPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_DATA_SIZE; }

#define initIPXECB(e,sock) {\
    (e)->ecb.ESRAddress=(void (far *)())NULL;\
    (e)->ecb.fragmentCount=2;\
    (e)->ecb.socketNumber=(sock);\
    (e)->ecb.fragmentDescriptor[0].address=&(e)->hdr;\
    (e)->ecb.fragmentDescriptor[0].size=sizeof(IPXHeader);\
    (e)->ecb.fragmentDescriptor[1].address=(e)->buf;\
    (e)->ecb.fragmentDescriptor[1].size=IPX_BUFFER_SIZE; }


extern WORD REGsocket,PnPsocket;
extern HANDLE hExportList;
extern int ExportsRegistered;
extern int eqpsize;
extern BYTE far *DataBuffer;
extern int RPCprotocol,TotalMsgSize;
extern int msgcnt,msgsize;
extern double gDataTimeStamp;
extern double gDataTimeStampOffset;
#define RPCtimestamp ((long)gDataTimeStamp)
#define RPCtimestampMSEC (int)((gDataTimeStamp - (long)gDataTimeStamp) * 1000)
extern char clientName[17],clientaddr[30],msgbuf[256];
extern char far *erlst[];
extern HANDLE hEQPconsumerlist;
extern int interimStatus;
extern struct timeval lastCycleTime;
extern time_t appmktime;
extern char appmkversion[64];

extern struct sockaddr_in tGCastAddrSck;
extern struct sockaddr_in tMCastAddrSck;
extern struct in_addr tMCastAddr;
extern struct in_addr tGCastAddr;
extern char tRedirectedServer[64];

extern int AveBusyTime;
extern UINT32 gContractMisses;
extern UINT32 gContractDelays;
extern UINT32 gClientMisses;
extern UINT32 gClientReconnects;
extern UINT32 gClientRetries;
extern UINT32 gBurstLimitReachedCount;
extern UINT32 gSingleLinkCount;
extern int gCycleCounts, gMaxCycleCounts;
extern int max_rcvbuf;
extern int gRetardSingleContractRemoval;
extern struct timeval gThisCycleTime;
extern UINT32 gTimeServerCycle;
extern UINT32 gTimeServerIdleLastCycle;
extern int CycleCounts;
extern int SystemPresetMemory;
extern int gtUDPPort;
extern int gtTCPPort;
extern int gtGCastPort;
extern int gtMCastPort;
extern int gtUCPClnPort;
extern char far *gtMCastAddr;
extern char far *gtGCastAddr;
extern int gtMCastTTL;
extern int nExpiredContracts;

#define NLINES 100
extern int nofeclog;
extern int FeclogDepth;
int feclog(char far *text);
void output(char far *txt);
#define dbllog(txt) { output(txt); feclog(txt); }

/* trivial RPC call: */
int tUDPCall(int fecindex,BYTE far *buf,int timout);
int tIPXCall(int fecindex,BYTE far *buf,int timout);
#define tCall(idx,buf,to) ( TCPloaded ? tUDPCall(idx,buf,to) : tIPXCall(idx,buf,to) )
int tExecLink(int fecindex,char far *name,char far *dev,char far *prop,
	   short size,char far *data,short format,short timout);
int tRPC(int fecindex,char far *data,char far *name,short EqNo,char far *prop,
	   short size,short format,short timout);

char far *getCaller(void);
int makeStockPropXRefTable(void);
//short GetStockPropertyId(char far *Property);
short IsStockProperty(char far *Property);
void FreeApplication(WORD iMessage);
int InitApplication(void);
int AllocateAndAssignMemory(void);
int createStructMemory(char far *tag);
int initSPX(void);
int initTCP(void);
int GetSPXStream(WORD ID,CLN far *cln);
int GetTCPStream(int socket,CLN far *cln);
int execRequest(CLN far *cln);
void SPXSendData(WORD ID);
void TCPSendData(int socket);
void CloseSPXStream(WORD ID);
void CloseTCPStream(int socket);
int fulfillTCPRequest(int s);
int fulfillSPXRequest(void);
int doContract(CONTRACT far *con,HANDLE hEl,BYTE far *dataIn,
               BYTE far *dataOut,int id,int prot);
int SendIPXData(HANDLE hCln,BYTE far *data);
int SendUDPData(HANDLE hCln,BYTE far *data,int UDPsocket);
int SendIPData(HANDLE hCln,BYTE far *data);
HANDLE SearchFunction(char far *name);
int ProduceData(void);
int DeliverData(void);
int listenUDP(int far *s,unsigned short port);
int getConsumerRequest(HANDLE hCln,BYTE far *buf,int packetsize);
HANDLE getConsumerIPXAddress(void);
HANDLE getConsumerIPAddress(int sck, int prot);
int RemoveConsumer(int i);
int RemoveContract(int j);
int renegotiateContract(ContractListStruct far *cl,int cc);
unsigned long ElfHash(const unsigned char far *key);
HANDLE findTagIDHandle(char far *name);
ExportListStruct far *findTagID(char far *name);
int propsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int npropsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int ndeviceQuery(CONTRACT far *con,char far *dataIn,char far *data);
int deviceQuery(CONTRACT far *con,char far *dataIn,char far *data);
int alarmsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int alarmsQueryEx(CONTRACT far *con,char far *dataIn,char far *data);
int nalarmsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int alarmDefsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int nalarmDefsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int msgDisplay(CONTRACT far *con,char far *data,char far *dataOut);
int activityQuery(CONTRACT far *con,char far *dataIn,char far *data);
int contractsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int clientsQuery(CONTRACT far *con,char far *dataIn,char far *data);
int setDebugLevel(CONTRACT far *con,char far *dataIn,char far *data);
int getDebugBuffer(CONTRACT far *con,char far *dataIn,char far *data);
int getNumContracts(char far *eqname);
int getNumConsumers(char far *eqname);
void FAR PASCAL ListenConnectSPX(void);
void FAR PASCAL ListenRequestIPX(void);
int FAR PASCAL ListenConnectTCP(unsigned long i, int err);
int FAR PASCAL ListenRequestUDP(unsigned long i, int err);
int appendRegisteredUserList(char far *user);
int GetPropertyOutputFormat(char far *eqm,char far *prpName);
int removeRegisteredUser(char far *user);
struct timezone { int dummy; };
struct timeval far *gettimeofday(struct timeval far *t,struct timezone far *tz);
int EGUQueryFunction(CONTRACT far *con,char far *dataIn,char far *data);
int IsMemberLockSet(CLN far *cln);
void checkAccessLocks(time_t t);
int lockAccess(CONTRACT far *con,char far *dataIn,char far *data);
short IsIllegalProperty(CONTRACT far *con,CLN far *cln,short far *EqpNativeFormatIn,short far *EqpNativeFormatOut,
                        UINT16 far *MaxSizeIn,UINT16 far *MaxSizeOut,HANDLE far *hListEntry);
int readFecLog(char far *textbuf,int sizeInBytes);
int readFecLogFile(char far *fname,char far *textbuf,int sizeInBytes);
void applyFecLogPath(char far *path,int pathlen);
int scanFecLogFiles(NAME64 far *namebuf,int far *numfiles);
int readSelfTest(char far *eqm,char far *textbuf,int sizeInBytes);
double putDataTimeStamp(double toffset, time_t tsec, int tmsec);
double  __export __loadds FAR PASCAL makeDataTimeStamp(void);
void __export __loadds FAR PASCAL setDataTimeStamp(double ts);
struct timeval far *getDataTimeStampAsTimeval(void);
double __export __loadds FAR PASCAL getDataTimeStamp(void);
int __export __loadds FAR PASCAL GetDeviceNumber(char far *eqm,char far *dev);
void __export __loadds FAR PASCAL AcceptTCPStream(void);
void __export __loadds FAR PASCAL GetIPXData(void);
void __export __loadds FAR PASCAL GetIPData(int sck,int prot);
void __export __loadds FAR PASCAL AcceptSPXStream(void);
long __export __loadds FAR PASCAL GETSRVSTRUCTSIZE (char far *tag);
int __export __loadds FAR PASCAL DebugServer(int level);
int __export __loadds FAR PASCAL GetPropertyId(char far *eqm,char far *prpName);
int __export __loadds FAR PASCAL DebugLog(int level);
void __export __loadds FAR PASCAL GetCallerInfo(char far *ClientName,char far *ClientAddress);
long  __export __loadds FAR PASCAL SetWorkAreaSize(long waSize);

#endif // _WINSRV_H_
