/** \file eqpdll.c
 * \brief TINE Client DLL documentation 
 *
 */
#include "project.def"
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <dos.h>
#include <signal.h>
#include <wsipx.h>
#include <wsnwlink.h>
#include <sys/timeb.h>
#include "errors.h"
#include "database.h"
#include "rpcsrv.h"
#ifndef USE_SHARED_MEMORY
# include "rpcalloc.h"
#endif
#include "mmfshare.h"

HANDLE hDLL;

#ifdef USE_SHARED_MEMORY
# include "mmfshare.h"
int numglobals = -1;
#else
# include "rpcalloc.h"
int usageCount = 0;
int nglobals = MAXGLOBALS;
int numglobals = 0;
HANDLE hDLLWnd;
linkType gLinkBuffer[MAXHOTLINKS], *link = gLinkBuffer;
int SyncID;
int debug = 0;
int debugTargetError = 0;
LCLCONTRACT gLclContractBuffer, *lclContract = &gLclContractBuffer;
#endif

HANDLE hWatchdogWnd;
int thisUsage = 0;
int thisUsageMsec = 0;
int ControlNet = 0;  // Kluge this for now
int EngineShuttingDown = 0;
char szDebugFile[82];
char szDebugFileBak[82];
char lpszSubSemaphore[128];

int nUDPsets;
int TimeLeft = 0;
BYTE  *WorkArea;
unsigned long WorkAreaSize = WORK_SIZE;
int WorkAreaSemaphore = 0;

int ConsumerSocket = 0,SyncSocket,GlobalSocket;
SOCKET IPXConsumerSocket = 0;
struct sockaddr_in prod_name;
SOCKADDR_IPX ipx_prod_name;
SOCKET IPXMCastConsumerSocket = 0;
const BYTE mcast_socket[2] = {MCAST_SOCKET,MCAST_SOCKET};

GlobalListStruct *GlobalList;

int MCastConsumerSocket = 0;
int ListenForMCasts = TRUE;
int ListenForGlobals = TRUE;
int MCastPreference = FALSE;

extern BYTE *msgbuf;
extern int rcvBufferSpace;
extern int gRPCSyncNotify;

//extern int debug;
double gDataTimeStamp = 0;
double gDataTimeStampOffset = 0;
double putDataTimeStamp(double toffset, time_t tsec, int tmsec)
{
  double ts;
  ts = (double)tsec;
  ts += (double)tmsec/1000.0;
  ts += toffset;
  return ts;
}
double makeDataTimeStamp(void)
{
  double ts;
  struct timeb tb;
  ftime(&tb);
  ts = (double)tb.time;
  ts += (double)(tb.millitm)/1000.0;
  return ts;
}
int IPXloaded = 0;
int initialized = 0, DBinitialized = 0;
int TimerID=1;
char username[17];
char myipaddr[17];
extern int RPCNumFecsKeep, RPCNumRecsKeep;
__declspec(dllexport) time_t RPCtimestamp;

int loadGlobalsDBase(void);
int initIPXSPX(void);
int listenIPX(SOCKET *s,unsigned short port);
int getNextLinkID(short id);
int checkNet(char *myipaddr);
int GetDatagram(int sck,int prot);
int OpenRPCSemaphore(char *lpszSemaphore);

extern void soperror(char  *txt);
// WSA cover routines :
int loaded(void)
{
	static WSADATA wsaData;
	static int WSAinitialized = -1;
  static WORD wVersionRequested;
  char *ptr;

  if((ptr=getenv("OS")) != NULL && !stricmp(ptr,"Windows_NT"))
  {
    wVersionRequested = MAKEWORD( 2, 0 );
  }
  else
  {
    wVersionRequested = MAKEWORD( 1, 1 );
  }

  if (WSAinitialized == -1) WSAinitialized = WSAStartup(wVersionRequested,&wsaData) ? 0 : 1;
	if (WSAinitialized ==  0 && debug) soperror("WSAStartup");
	return WSAinitialized;
}
int GetErrno(void) { return WSAGetLastError();}
int soioctl(int s,long cmd,char  *mode)
{
	return ioctlsocket(s,cmd,(u_long  *)mode);
}
int soclose(int s) { return closesocket(s); }
int TCPGetResource(void)
{
	int cc;
	static WSADATA wsaData;
	if ((cc=WSAStartup(0x0101,&wsaData)) != 0 && debug) soperror("WSAStartup");
	return cc;
}
int TCPFreeResource(void)
{
	int cc;
	if ((cc=WSACleanup()) != 0 && debug) soperror("WSACleanup");
	return cc;
}
DWORD getmyipaddr(void)
{
  static char hname[30];
  static LPHOSTENT hent;
  static SOCKADDR_IN stlclAddr;
  static SOCKADDR_IN strmtAddr;
  static done = 0;
  static DWORD myip = 0;
  SOCKET hSock;
  int nRet=sizeof(SOCKADDR);

  if (done) return myip;
  stlclAddr.sin_addr.s_addr = INADDR_ANY;
  if (gethostname(hname,30) != SOCKET_ERROR)
  {
    if ((hent=gethostbyname((LPSTR)hname)) != NULL)
    {
      stlclAddr.sin_addr.s_addr = *((u_long FAR *)(hent->h_addr));
    }
  }
  if (stlclAddr.sin_addr.s_addr == INADDR_ANY)
  {
    if ((hSock=socket(AF_INET, SOCK_DGRAM, 0)) != INVALID_SOCKET)
    {
      strmtAddr.sin_family = AF_INET;
      strmtAddr.sin_port = htons(IPPORT_ECHO);
      strmtAddr.sin_addr.s_addr = inet_addr("128.127.50.1");
      if (connect(hSock,(LPSOCKADDR)&strmtAddr,sizeof(SOCKADDR)) != SOCKET_ERROR)
      {
        getsockname(hSock,(LPSOCKADDR)&stlclAddr,&nRet);
      }
      closesocket(hSock);
    }
  }
  myip = stlclAddr.sin_addr.s_addr;
  done = TRUE;
  return stlclAddr.sin_addr.s_addr;
}
#define MAX_DEBUG_TIME 300
int gMaxDebugTime = MAX_DEBUG_TIME;
void CheckDebugLevel(void)  // don't let this stay on forever ... 
{
  static time_t lastDebugStart = 0;
  static int lastDebugLevel = 0;
  if (debugTargetError > 0) return; // looking for an error condition
  if (debug != lastDebugLevel)
  {
    lastDebugLevel = debug;
    if (debug) lastDebugStart = time(NULL);
  }
  if (debug > 0 && debug < 5)
  {
    if (time(NULL) > lastDebugStart + gMaxDebugTime) 
    {
      debug = 0;
      gMaxDebugTime = MAX_DEBUG_TIME;
    }
  }
}
FILE *fpDebugFile = NULL;
#define BUFLEN 2304
char txtbuffer[BUFLEN+26]; // make sure the last ctime() fits
void output(char *txt)
{
  static int initialized = 0;
  static HANDLE hDC;
  static RECT Rect;
  static linesout = 0;
  int buflen=0,len;
  time_t t;
  char *c;

  if ((len=strlen(txt)) == 0) return;
  while ((buflen=strlen(txtbuffer)) + len > BUFLEN || linesout > 30) 
  {
    if ((c=strchr(txtbuffer,'\n')) == NULL) return;
    strcpy(txtbuffer,++c);
    linesout--;
  }
  strcat(txtbuffer,txt);
  if (txtbuffer[(len=strlen(txtbuffer)-1)] != '\n') return;
  linesout++; time(&t); 
  sprintf(&txtbuffer[len]," %s",ctime(&t));
  if (debug > 2) OutputToDebugFile("output",txt);
  if (!IsWindowVisible(hDLLWnd)) return;
  if (!initialized)
  {
    initialized = 1;
    hDC = GetDC(hDLLWnd);
    GetClientRect(hDLLWnd, (LPRECT) &Rect);
    FillRect(hDC,&Rect,GetStockObject(WHITE_BRUSH));
  }
  FillRect(hDC,&Rect,GetStockObject(WHITE_BRUSH));
  DrawText(hDC,txtbuffer,-1,&Rect,DT_LEFT);
}
struct structStruct *structList = NULL;
struct structStruct *getStructFromTag(char *tag)
{
  struct structStruct *s;

  if (tag == NULL || strlen(tag) == 0) return NULL;
  for (s=structList; s && strncmp(s->name,tag,8); s=s->next);
  if (s && (s->rawlen == 0 || s->size == 0)) return NULL;
  return s;
}
struct structStruct *findstruct(char *tag)
{
  struct structStruct *s;

  for (s=structList; s; s=s->next) if (!strncmp(s->name,tag,8)) return s;
  if ((s=(struct structStruct *)calloc(1,sizeof(struct structStruct))) != NULL)
  {
    strncpy(s->name,tag,8);
    s->rawlen = 0;
    s->next = structList;
    structList = s;
  }
  return s;
}

char txtbuf[128];
#define soerr(s)	{strcat(txtbuf,s); goto err;}
__declspec(dllexport) LRESULT CALLBACK EQPDLLWndProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
  int siz,sck,sckevn,sckerr,cc=0;
  static int TimerSemaphore = 0, SyncSemaphore = 0;
  HANDLE hSyncMutex;
  char lpszSyncMutex[64];
  char shmTag[64];
  shmDataGram *dg;
  HANDLE hDG;

  switch (iMessage)
  {
    case WM_RPCDEBUG:
      if (debug > 2) fpDebugFile=fopen(szDebugFile,"w");
      else
      {
        if (fpDebugFile) fclose(fpDebugFile);
        fpDebugFile = NULL;
      }
    case WM_LOCKLINK:    // lock the Mutex in this thread
#     ifdef USE_SHARED_MEMORY
      sprintf(lpszSyncMutex,"SyncRPC:%d",wParam);
#     else
      sprintf(lpszSyncMutex,"SyncRPC:%d:%d:%d",thisUsage,thisUsageMsec,wParam);
#     endif
      hSyncMutex=(HANDLE)OpenRPCSemaphore(lpszSyncMutex);
      link[wParam].hCtrlWnd = hSyncMutex;
      if (debug)
      {
        sprintf(txtbuf,"created semaphore: %s\n",lpszSyncMutex);
        output(txtbuf);
      }
      return 0;
    case WM_GETLINKID: 
      return getNextLinkID((short)wParam);
    case WM_REMOVELINK:  // used with sychronous calls
      RemoveRPC(wParam);
      if (wParam == 0) SyncSemaphore = 0; // release semaphore
      break;
    case WM_CREATELINK:  // put entry into elem table
      if ((cc=CreateElem(wParam)) != 0)
       return cc;
      break;
    case WM_READ_LCL:
      sprintf(shmTag,"_shmTData:%d",hDLLWnd); // me
      siz = wParam;
      dg = (shmDataGram *)shmGetDataGram(shmTag,siz,&hDG);
      consumeData(dg->payload);
      dg->dirty = 0;
      shmFreeDataGram(dg,hDG);
      break;
    case WM_RCVPKT_UDPBCST:
      sck = wParam;
      while (GetDatagram(sck,UDPBCST) > 0);
      break;
    case WM_RCVPKT_IPXBCST:
      sck = wParam;
      while (GetDatagram(sck,IPXBCST) > 0);
      break;
    case WM_RCVPKT_GLB:
      sck = wParam;
      while (GetDatagram(sck,GLB) > 0);
      break;
    case WM_RCVPKT:
      sck = wParam;
      GetDatagram(sck,UDP);
      GetDatagram(sck,IPX);
      GetDatagram(sck,GLB);
      break;
    case WM_RCVPKT_TCP:
      sck = wParam;
      sckevn = WSAGETSELECTEVENT(lParam);
      sckerr = WSAGETSELECTERROR(lParam);
      if (sckerr != 0 || sckevn == FD_CLOSE) CloseConnectionSocket(sck);
      else while (GetDatagram(sck,TCP) > 0);
      break;
    case WM_RCVPKT_UDP:
      sck = wParam;
      while (GetDatagram(sck,UDP) > 0);
      break;
    case WM_RCVPKT_IPX:     
      sck = wParam;
      while (GetDatagram(sck,IPX) > 0);
      break;
    case WM_TIMER:       // Stream-type hot-links and alarm checks
      if (TimerSemaphore) break;
      TimerSemaphore = 1;
      DoRPC();
      CheckConnections();
      CheckSubscriptions();
      Cleanup();
      CheckDebugLevel();
      TimerSemaphore = 0;
      break;
    case WM_DESTROY:
      KillTimer(hWnd,1);
    default:
      return DefWindowProc (hWnd, iMessage, wParam, lParam);
  }
  return 0;
}
char  lpszEqpWN[64];
int CreateDLLWindow(void)
{
	static WNDCLASS wndclass;
  
# ifdef USE_SHARED_MEMORY
  sprintf(lpszEqpWN,"EQPDLL Window");
# else
  sprintf(lpszEqpWN,"EQPDLL Window %d:%d",thisUsage,thisUsageMsec);
# endif
	wndclass.style         = CS_GLOBALCLASS|CS_SAVEBITS;  // essential style!!
	wndclass.lpfnWndProc   = (WNDPROC) EQPDLLWndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hDLL;
	wndclass.hIcon         = LoadIcon (hNULL, IDI_APPLICATION);
	wndclass.hCursor       = LoadCursor (hNULL, IDC_ARROW);
	wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	wndclass.lpszMenuName  = hNULL;
	wndclass.lpszClassName = lpszEqpWN;

	if (!RegisterClass(&wndclass)) return -1;

	if (!(hDLLWnd = CreateWindow (
							lpszEqpWN, lpszEqpWN,         // lpClassName, lpWindowName
							WS_BORDER|WS_CAPTION,         // dwStyle
							CW_USEDEFAULT, CW_USEDEFAULT, // X, Y
							CW_USEDEFAULT, CW_USEDEFAULT, // nWidth, nHeight
							hNULL, hNULL,                 // hWndParent, hMenu
							hDLL,                         // hInstance
							NULL                          // createstruct
		 ) ) ) return -1;

	return 0;
}
#define SYSTEM_MESSAGE "Control Software has become unstable on this machine\nPlease login again !"
__declspec(dllexport) LRESULT CALLBACK EQPDLLWatchdogProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
  static MessageDelivered = 0;
  int i,cc;
  switch (iMessage)
  {
    case WM_TIMER:
      if (IsWindow(hDLLWnd) || MessageDelivered) break;
      MessageDelivered = 1;
      MessageBox(hNULL,SYSTEM_MESSAGE,"",IDOK);
      break;
    case WM_PAINT:
      i = wParam; cc = lParam;
      if (link[i].dataOut != NULL) cc = GETRPCDATAEx((short)i,link[i].dataOut,&RPCtimestamp,NULL);
      link[i].callback((int)link[i].hCtrlWnd,cc);
      break;
    case WM_DESTROY:
      KillTimer(hWnd,1);
    default:
      return DefWindowProc (hWnd, iMessage, wParam, lParam);
  }
  return 0;
}
char lpszWatchDog[] = "EQPDLL Watchdog";
int RegisterWatchdogWindow(HANDLE hInst)
{
	static WNDCLASS wndclass;

	wndclass.style         = CS_GLOBALCLASS|CS_SAVEBITS;  // essential style!!
	wndclass.lpfnWndProc   = (WNDPROC) EQPDLLWatchdogProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInst;
	wndclass.hIcon         = LoadIcon (hNULL, IDI_APPLICATION);
	wndclass.hCursor       = LoadCursor (hNULL, IDC_ARROW);
	wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
	wndclass.lpszMenuName  = hNULL;
	wndclass.lpszClassName = lpszWatchDog;

	if (!RegisterClass(&wndclass)) return -1;
	return 0;
}
int CreateWatchdogWindow(HANDLE hInst)
{
	if (RegisterWatchdogWindow(hInst)) return -1;
	if (!(hWatchdogWnd = CreateWindow (
							lpszWatchDog, lpszWatchDog,   // lpClassName, lpWindowName
							WS_BORDER|WS_CAPTION,         // dwStyle
							CW_USEDEFAULT, CW_USEDEFAULT, // X, Y
							CW_USEDEFAULT, CW_USEDEFAULT, // nWidth, nHeight
							hNULL, hNULL,                 // hWndParent, hMenu
							hInst,                        // hInstance
							NULL                          // createstruct
		 ) ) ) return -1;

	if (!SetTimer(hWatchdogWnd,1,1000,(TIMERPROC)NULL))
      OutputToDebugFile("CreateWatchdogWindow","Can't set Watchdog Timer!");
	return 0;
}
void GetDebugFilePath(void)
{
  char *ptr;
  if      ((ptr=getenv("RPCDEBUGFILE")) != NULL) strncpy(szDebugFile,ptr,80);
  else if ((ptr=getenv("TEMP")) != NULL) 
  {
    strncpy(szDebugFile,ptr,68); 
    strncpy(szDebugFileBak,ptr,68); 
    strcat(szDebugFile,"\\tinecln.dbg");
    strcat(szDebugFileBak,"\\tinecln.bak");
  }
  else 
  {
    strcpy(szDebugFile,"C:\\tinecln.dbg");
    strcpy(szDebugFileBak,"C:\\tinecln.bak");
  }
}
void GetUserFromEnvironment(void)
{
	char *ptr;
	if      ((ptr=getenv("CONTROLUSER")) != NULL) strncpy(username,ptr,16);
	else if ((ptr=getenv("USERNAME")) != NULL) strncpy(username,ptr,16);
	else strcpy(username,"WINNT");
	strupr(username);
}
void GetBufferSpaceFromEnvironment(void)
{
	char *ptr, scratch[256];

  if ((ptr=getenv("RPCBUFFERSPACE")) != NULL) 
  {
    strncpy(scratch,ptr,256); strlwr(scratch);
    if (!strncmp(scratch,"0x",2))
      rcvBufferSpace = strtol(scratch,NULL,16);
    else
      rcvBufferSpace = strtol(scratch,NULL,10);
  }
  return;
}
int  PASCAL LibMain(HANDLE hInstance,WORD wDataSegment,WORD wHeapSize,LPSTR lpszCmdLine )
{
	HCURSOR OldCurs;
	unsigned int i;
	hDLL = hInstance;
	signal(SIGFPE,SIG_IGN);  // ignore domain errors if they occur

	if (CreateDLLWindow()) return 0;
	WaitCurs = LoadCursor(NULL,IDC_WAIT);

	for (i=1; i<MAXHOTLINKS; i++) link[i].active=0;

	OldCurs=SetCursor(WaitCurs);
	if ((i=initApplication()) != 0)
	{
		strcpy(txtbuf,"init err: "); strcat(txtbuf,erlst[i]);
		OutputToDebugFile("LibMain",txtbuf);
		MessageBox(GetFocus(),txtbuf,"",IDOK);
	}
	SetCursor(OldCurs);
	return 1;
}
int WEP (int bSystemExit)
{
	FreeApplication();
	return 0;
}
int GetMulticastPreference(void)
{
  char *ptr = NULL;
  if ((ptr=getenv("MULTICAST_PREFERENCE")) != NULL)
  {
    if (!stricmp(ptr,"TRUE") || !stricmp(ptr,"YES")) return TRUE;
  }
  return FALSE;
}
BOOL WINAPI DllMain(HINSTANCE hInstDll,DWORD fdwReason,LPVOID lpvReserved)
{
  unsigned int counter = 0;
  struct _timeb tb;
  _ftime(&tb);
  switch (fdwReason)
  {
    case DLL_PROCESS_ATTACH:  // DLL being loaded
      counter++;
      GetBufferSpaceFromEnvironment();
      GetUserFromEnvironment();
      GetDebugFilePath();
      MCastPreference = GetMulticastPreference();
#     ifdef USE_SHARED_MEMORY
      ListenForMCasts = TRUE;
      ListenForGlobals = TRUE;
      sprintf(lpszSubSemaphore,"RPCSubscriptionSemaphore");
      if (mmfInit())
      {
        OutputToDebugFile("DllMain","Can't initialize shared memory");
        MessageBox(hNULL, "Can't initialize shared memory","",IDOK);
        return 0;
      }
      if ((thisUsage=usageCount) == 1) LibMain(hInstDll,0,0,0);
      else if (CreateWatchdogWindow(hInstDll))
      {
        OutputToDebugFile("DllMain","Can't create watchdog window");
      }
#     else
      ListenForMCasts = FALSE;
      ListenForGlobals = FALSE;
      thisUsage = tb.time;
      thisUsageMsec = tb.millitm;
      sprintf(lpszSubSemaphore,"RPCSubscriptionSemaphore:%d:%d",thisUsage,thisUsageMsec);
      LibMain(hInstDll,0,0,0);
#     endif
      break;
    case DLL_THREAD_ATTACH:
      break;
    case DLL_THREAD_DETACH:
      break;
    case DLL_PROCESS_DETACH:  // DLL being unloaded
#     ifdef USE_SHARED_MEMORY
      if (hWatchdogWnd) 
      {
        DestroyWindow(hWatchdogWnd);
        UnregisterClass(lpszWatchDog,hDLL);
      }
      usageCount--;
      if (!usageCount) WEP (0);
#     else
      WEP (0);
#     endif
      break;
  }
  return (TRUE);
}
HANDLE hSubSemaphore;
int setDebugFromEnvironment(void)
{
  char *ptr;
  int dbglevel = 0;
  if  ((ptr=getenv("TINE_DEBUG_LEVEL")) != NULL) 
  {
    dbglevel = atoi(ptr);
  }
  if  ((ptr=getenv("TINE_DEBUG_TARGET_ERROR")) != NULL) 
  {
    debugTargetError = atoi(ptr);
  }
  return dbglevel;
}
extern int breakingFactor;
int setBreakingFactor(void)
{
  char *ptr;
  int factor = 10;
  if  ((ptr=getenv("TINE_BREAKING_FACTOR")) != NULL) 
  {
    factor = atoi(ptr);
  }
  return factor;
}
int initApplication(void)
{
  int cc = 0;
  if (initialized) return 0;
  debug = setDebugFromEnvironment();
  breakingFactor = setBreakingFactor();
  if ((cc=AllocateAndAssignMemory()) != 0) return cc;
  if ((hSubSemaphore=CreateSemaphore(NULL,1,1,lpszSubSemaphore)) == NULL) return semaphore_error;
  if ((cc=initTCP()) != 0) OutputToDebugFile("initApplication","TCP not loaded!");
  if (initIPXSPX() != 0) OutputToDebugFile("initApplication","IPX not loaded!");
  if (!(DBinitialized=initDataBase() ? 0 : 1)) OutputToDebugFile("initApplication","Database not loaded!");
  initialized = 1;
  if (ListenForGlobals)          // set at startup
  {
    ListenForGlobals = FALSE;    // let ListenForNetGlobals() set this flag ...
    ListenForNetGlobals();
  }
  if (ListenForMCasts)
  {
    ListenForMCasts = FALSE;
    ListenForMulticasts();
  }
  initNetGlobals();
  RPCNumFecsKeep = RPCNumFecsRead;
  RPCNumRecsKeep = RPCNumRecsRead;
  return cc;
}
void FreeApplication()
{
  time_t t0 = time(NULL), t = t0;

  if (!initialized) return;
  initialized=0;
  EngineShuttingDown = TRUE;

  RemoveRPC(-1);

  if (IPXloaded)
  {
    soclose(IPXConsumerSocket);
    if (ListenForMCasts) soclose(IPXMCastConsumerSocket);
  }

  if (TCPloaded)
  {
    soclose(ConsumerSocket);
    soclose(SyncSocket);
    if (ListenForGlobals) soclose(GlobalSocket);
    if (ListenForMCasts) soclose(MCastConsumerSocket);
    while (!WSACleanup() && t < t0 + 5) t = time(NULL);
  }

  free(WorkArea);
  free(RPCData); free(RPCFec);
  free(GlobalList);

  DestroyWindow (hDLLWnd);
  UnregisterClass(lpszEqpWN,hDLLWnd);

# ifdef USE_SHARED_MEMORY
  CloseHandle(hSharedKeys);
# endif
  CloseHandle(hSubSemaphore);
  if (fpDebugFile) fclose(fpDebugFile);
}

// ==================================================
//       DLL initialization routines
// ==================================================
int RegisterSystemStructs(void)
{
  FILLSTRUCT("XPQS",XPROPERTYQUERYSTRUCT_SIZE, 100, CF_NULL);
  return 0;
}
int AllocateAndAssignMemory(void)
{
  int cc=0,placeholder = 0;
  char txt[128];

  if (!(WorkArea=calloc(1,WorkAreaSize))) ccerr(out_of_client_memory);

  if (!SetTimer(hDLLWnd,1,SYSTEM_DELAY,(TIMERPROC)NULL))
  {
    OutputToDebugFile("AllocateAndAssignMemory","Can't set system Timer!");
    ccerr(resources_exhausted);
  }
  RegisterSystemStructs();
err:
  if (cc)
  {
    sprintf(txt,"%s",erlst[cc]);
    OutputToDebugFile("AllocateAndAssignMemory",txt);
  }
  return cc;
}
#define outerr(txt) { strcpy(buf,txt); goto err; }
char *GLBEQM = GLOBALSRC;
int AcquireNetGlobal(int i)
{
  int id,cc = 0,retry=2;
  ElemDataStruct  *e;
  HANDLE hContract=0;
  time_t t = time(NULL);
  static time_t lastCalled = 0;
  static int lastCompletion = 0;

  id = i + 1; // this should be systematically true !!!
  if (strlen(GlobalList[i].keyword) == 0)
  {
    link[id].active = -1; // placeholder
    return 0;
  }
  if (link[id].active > 0) return 0; // done
  if ((id=Subscribe(hDLLWnd,hNULL,GLBEQM,1,GlobalList[i].keyword,
                    1000,GlobalList[i].size,GlobalList[i].format,REGISTER)) < 1)
  {
    sprintf(txtbuf,"Cannot subscribe to %s : %s (list at %d)",
            GlobalList[i].keyword,erlst[-id],GlobalList);
    OutputToDebugFile("AcquireNetGlobal",txtbuf);
    ccerr(-id);
  }
  if (id != i + 1)
  {
    sprintf(txtbuf,"link table alignment problem for %s : %d vs %d",
            GlobalList[i].keyword,id,GlobalList[i].ID);
    OutputToDebugFile("AcquireNetGlobal",txtbuf);
    ccerr(-software_error);
  }
  GlobalList[i].ID = id;
  if (!(e=ContractAlloc(link[id].linkTag,link[id].linkSizeIn,link[id].linkSizeOut,&hContract))) return un_allocated;
  GlobalList[i].Data = rpcDATAOUT(e);
  GlobalList[i].cTimeStamp = makeDataTimeStamp();

  while ((lastCompletion == 0 || lastCalled != t) && retry--)
  {
    if ((cc=tExecLink(link[id].Elem->FecIndex,GLBNAME,"#0",GlobalList[i].keyword,
                      GlobalList[i].size,GlobalList[i].Data,GlobalList[i].format,200)) == 0) break;
    if (cc == illegal_property || cc == illegal_format)
    {
      cc = 0;  // the desginated globals server doesn't know about it
      break;
    }
  }

  if (ControlNet)  // expect broadcasts
  {
    e->heartbeat = GlobalList[i].heartbeat + 2;
  }
  else e->mode = REFRESH;
  if (hContract) ContractFree(e,hContract);
err:
  lastCompletion = cc;
  lastCalled = t;
  return cc;
}
void initNetGlobals(void)
{
  int i,err = 0;
  DWORD in;

# ifdef USE_SHARED_MEMORY
  if ((nglobals=loadGlobalsDBase()) < 0) nglobals = 0;
# else
  if (!(GlobalList=calloc(nglobals,sizeof(GlobalListStruct)))) nglobals = 0;
# endif

  if ((in = getmyipaddr()) != -1)
  {
    strcpy(myipaddr,inet_ntoa(*(struct in_addr *)&in));
    ControlNet = checkNet(myipaddr);
    // when we go to multicast, this is no longer necessary !
  }
  for (i=0; i<nglobals; i++) err += AcquireNetGlobal(i) ? 1 : 0;
  if (!ControlNet) nglobals -= err;
  return;
}
int initIPXSPX(void)
{
  strcpy(txtbuf,"IPX: ");
  if (listenIPX(&IPXConsumerSocket,0)) return -1;
  if (WSAAsyncSelect(IPXConsumerSocket,hDLLWnd,WM_RCVPKT_IPX,FD_READ)) soerr("select");
  IPXloaded = 1;
  return 0;
err: soperror(txtbuf);
  return -1;
}
int initTCP(void)
{
  strcpy(txtbuf,"TCP: ");
  if (!loaded()) soerr("loaded");
  TCPloaded = 1;
  msgbuf = (BYTE *)&WorkArea[sizeof(short)];
  if (MCastPreference)
  {
    ListenForMCasts = FALSE;  ListenForMulticasts();
  }
  if (listenUDP(&ConsumerSocket,0,rcvBufferSpace)) return -1;
  if (WSAAsyncSelect(ConsumerSocket,hDLLWnd,WM_RCVPKT_UDP,FD_READ)) soerr("select");
  if (listenUDP(&SyncSocket,0,0)) return -1;

  return 0;
err: soperror(txtbuf);
  return -1;
}
int listenIPX(SOCKET *s,unsigned short port)
{
  SOCKADDR_IPX addr;
  int PacketType = 0, reuseFlg = FALSE;
  static int optval = 1;
  char Local_Network_Number[4]  = {0x00, 0x00, 0x00, 0x00};
  char Local_Node_Number[6]     = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
  static u_long mode = 1;

  strcpy(txtbuf,"IPX: ");
  if ((*s=socket(AF_NS,SOCK_DGRAM,NSPROTO_IPX)) == INVALID_SOCKET) soerr("socket");
  if (port == *(unsigned short *)mcast_socket) reuseFlg = TRUE;
  if (reuseFlg)
  {
    optval = 1;
    if (setsockopt(*s,SOL_SOCKET,SO_REUSEADDR,(char *)&optval,sizeof(optval))) soerr("REUSEADDR");
  }
  if (port)
  {
    addr.sa_family = AF_NS;
    memcpy(&addr.sa_netnum, Local_Network_Number, 4);
    memcpy(&addr.sa_nodenum, Local_Node_Number, 6);
    memcpy(&addr.sa_socket, &port, 2);
    if (bind(*s,(struct sockaddr *)&addr,sizeof(addr)) ==  SOCKET_ERROR)
	  soerr("bind");
  }
  if (soioctl(*s,FIONBIO,(char  *)&mode)) soerr("NONBLOCKING");
//  if (setsockopt(*s,SOL_SOCKET,IPX_PTYPE,(const char *)&PacketType,4) == SOCKET_ERROR)
//	soerr("setsockopt");

  return 0;
err: soperror(txtbuf);
  soclose(*s);
  return -1;
}
struct in_addr tMCastAddr;
struct in_addr tGCastAddr;
int rcvBufferSpace = 0xffff;
int assignBufferSpaceToSocket(int sck, int bufferSpace)
{
  int cc = 0, err;
  static int optval = 1, optlen;
  optval = bufferSpace; optlen = sizeof(optval);
  cc = setsockopt(sck,SOL_SOCKET,SO_RCVBUF,(char *)&optval,sizeof(optval));
  cc = getsockopt(sck,SOL_SOCKET,SO_RCVBUF,(char *)&optval,&optlen);
  if (cc) err = WSAGetLastError();
  if (optval != bufferSpace) 
  {
    sprintf(txtbuf,"reserved only %d from %d rcv bufs",bufferSpace,optval);
    output(txtbuf);
  }
  else
  {
    rcvBufferSpace = bufferSpace;
  }
  return cc;
}
__declspec(dllexport) short PASCAL RPCAssignBufferSpace(int bufferSpace)
{
  int cc = 0;
  if (MCastConsumerSocket != 0) cc = assignBufferSpaceToSocket(MCastConsumerSocket,bufferSpace);
  if (ConsumerSocket != 0) cc = assignBufferSpaceToSocket(ConsumerSocket,bufferSpace);
  rcvBufferSpace = bufferSpace;
  return cc;
}
__declspec(dllexport) short PASCAL RPCSynchronousNotification(int syncNotify)
{
  int cc = 0;
  gRPCSyncNotify = syncNotify;
  return cc;
}
int listenUDP(int *s,unsigned short port,unsigned int rcvbufs)
{
  static struct sockaddr_in sck_name;
  static int optval = 1, optlen = 2;
  static u_long mode = 1;
  int cc;
  unsigned short udpport;
  static unsigned short nextFreePort = UDP_CLN_PORT;
  static int mCastDone = FALSE;
  int reuseFlg = FALSE;
  long ip;
  struct ip_mreq mreq;

  if (!mCastDone) /* always fill this in at initialization time */
  {
    nextFreePort = gtUCPClnPort;
    mCastDone = TRUE;
    ip = inet_addr(gtMCastAddr);
    memcpy(&tMCastAddr,&ip,sizeof(struct in_addr));  /* producer-subscriber */
    ip = inet_addr(gtGCastAddr);
    memcpy(&tGCastAddr,&ip,sizeof(struct in_addr));  /* producer-consumer   */
  }
  strcpy(txtbuf,"UDP: ");
  if ((*s=socket(PF_INET,SOCK_DGRAM,0)) == INVALID_SOCKET) soerr("socket");
  if (port == gtMCastPort || port == gtGCastPort) reuseFlg = TRUE;
  if (reuseFlg)
  {
    optval = 1;
    if (setsockopt(*s,SOL_SOCKET,SO_REUSEADDR,(char *)&optval,sizeof(optval))) soerr("REUSEADDR");
  }
  memset(&sck_name,0,sizeof(sck_name));
  sck_name.sin_addr.s_addr = 0;
  sck_name.sin_family = AF_INET;
  udpport = port ? port : nextFreePort;
  if (udpport == 0) goto portbound;
bindport:
  sck_name.sin_port = htons(udpport);
  if (bind(*s,(struct sockaddr  *)&sck_name,sizeof(sck_name)))
  {
    if (port == 0 && udpport++ < gtUCPClnPort + UDP_CLN_RANGE) goto bindport;
    soerr("bind");
  }
  if (port == 0) nextFreePort = udpport + 1;
portbound:
  if (rcvbufs > 0)
  {
    optval = rcvbufs; optlen = sizeof(optval);
    cc = setsockopt(*s,SOL_SOCKET,SO_RCVBUF,(char *)&optval,sizeof(optval));
    cc = getsockopt(*s,SOL_SOCKET,SO_RCVBUF,(char *)&optval,&optlen);
    if ((unsigned int)optval != rcvbufs) 
    {
      sprintf(txtbuf,"reserved only %d from %d rcv bufs",rcvbufs,optval);
      output(txtbuf);
    }
  }
  if (soioctl(*s,FIONBIO,(char  *)&mode)) soerr("NONBLOCKING");
  if (port == gtMCastPort) /* join multicast group */
  {
    mreq.imr_multiaddr.s_addr = tMCastAddr.s_addr;
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);
    cc = setsockopt(*s,IPPROTO_IP,IP_ADD_MEMBERSHIP,(void *)&mreq,sizeof(mreq));
    sprintf(&txtbuf[strlen(txtbuf)],"joining multicast group : %s",cc ? "failed" : "success");
    if (cc) soerr("sockopt");
  }
  else if (port == gtGCastPort) /* join multicast group */
  {
    mreq.imr_multiaddr.s_addr = tGCastAddr.s_addr;
    mreq.imr_interface.s_addr = htonl(INADDR_ANY);
    cc = setsockopt(*s,IPPROTO_IP,IP_ADD_MEMBERSHIP,(void *)&mreq,sizeof(mreq));
    sprintf(&txtbuf[strlen(txtbuf)],"joining globals multicast group : %s",cc ? "failed" : "success");
    if (cc) soerr("sockopt");
  }
  return 0;
err: 
  soperror(txtbuf); soclose(*s);
  return -1;
}

int AppendGlobalsList(char *keyword, short size, short format, short heartbeat)
{
  int i,cc;

  if (numglobals < 0) return -non_existent;
  for (i=0; i<numglobals; i++) if (!strncmp(GlobalList[i].keyword,keyword,16)) break;
  if (i == numglobals)
  {
    if (numglobals >= nglobals) return -resources_exhausted;
    numglobals++; 
  }
  strncpy(GlobalList[i].keyword,keyword,16);
  GlobalList[i].format = format;
  GlobalList[i].size = size;
  GlobalList[i].heartbeat = heartbeat;
  if ((cc=AcquireNetGlobal(i)) != 0) return -cc;
  return i;
}

// ====================================================
//       DLL export routines:
// ====================================================
int findDaylight(void)
{
  int dl;
  time_t t = time(NULL);
  struct tm *tm = localtime(&t);

  if      (tm->tm_mon<2 || tm->tm_mon>9)       dl = 0;  /* winter */
  else if (tm->tm_mon>2 && tm->tm_mon<9)       dl = 1;  /* summer */
  else if (tm->tm_mon==2)                               /* March */
  { if      (tm->tm_mday < 25)                  dl = 0;
    else if (tm->tm_wday > tm->tm_mday-25)      dl = 0;
    else if (tm->tm_wday == 0 && tm->tm_hour<2) dl = 0; /* Sunday */
    else dl = 1;
  }
  else                                                  /* October */
  { if      (tm->tm_mday < 25)                  dl = 1;
    else if (tm->tm_wday > tm->tm_mday-25)      dl = 1;
    else if (tm->tm_wday == 0 && tm->tm_hour<2) dl = 1; /* Sunday */
    else dl = 0;
  }
  return dl;
}
__declspec(dllexport) void PASCAL getDataTimeString(double ts,char *tsstr,int useLongStringFormat)
{
  time_t t = (time_t)ts;
  struct tm *tms = localtime(&t);
  UINT32 ms = (UINT32)((ts-t+0.0005) * 1000.0);
  int isdst = tms->tm_isdst > 0 ? 1 : 0;

  if (tms->tm_isdst < 0) isdst = findDaylight();
  if (useLongStringFormat)
  {
    strncpy(tsstr,ctime(&t),24);
    strncpy(&tsstr[28],&tsstr[20],4); tsstr[32] = 0;
    sprintf(&tsstr[19],".%03d",ms);
    sprintf(&tsstr[23]," %s",tzname[isdst]);
    tsstr[27] = ' ';
  }
  else
  {
    sprintf(tsstr,"%02d.%02d.%02d %02d:%02d:%02d.%03ld %s",
      tms->tm_mday,tms->tm_mon+1,tms->tm_year%100,
      tms->tm_hour,tms->tm_min,tms->tm_sec,
      ms,tzname[isdst]);
  }
  return;
}
__declspec(dllexport) short PASCAL ListenForNetGlobals(void)
{
  int cc = 0;

  if (ListenForGlobals) ccerr(0);
  if (TCPloaded) 
  {
    if (listenUDP(&GlobalSocket,(unsigned short)gtGCastPort,0)) soerr("UDP GCAST socket");
    if (WSAAsyncSelect(GlobalSocket,hDLLWnd,WM_RCVPKT_GLB,FD_READ))	soerr("select");
  }
  ListenForGlobals = TRUE;
err:
  if (cc) ListenForGlobals = FALSE;
  return cc;
}

__declspec(dllexport) short PASCAL ListenForMulticasts(void)
{
  int cc = 0;

  if (ListenForMCasts) ccerr(0);
  ListenForMCasts = FALSE;
  if (TCPloaded) 
  {
    if (listenUDP(&MCastConsumerSocket,(unsigned short)gtMCastPort,rcvBufferSpace)) ccerr(tcp_socket_error);
    if (WSAAsyncSelect(MCastConsumerSocket,hDLLWnd,WM_RCVPKT_UDPBCST,FD_READ)) ccerr(tcp_socket_error);
  }
  if (IPXloaded) 
  {
    if (listenIPX(&IPXMCastConsumerSocket,*(unsigned short *)mcast_socket)) ccerr(ipx_socket_error);
    if (WSAAsyncSelect(IPXMCastConsumerSocket,hDLLWnd,WM_RCVPKT_IPXBCST,FD_READ)) ccerr(ipx_socket_error);
  }
err:
  if (cc == 0) ListenForMCasts = TRUE;
  return cc;
}
void flushFecRpcCache(void);
__declspec(dllexport) void PASCAL FLUSHCACHE(void)
{
  flushFecRpcCache();
}

__declspec(dllexport) time_t RPCtimestamp;

__declspec(dllexport) long PASCAL RPCGETLASTTIME(void)
{
  return RPCtimestamp;
}
__declspec(dllexport) short PASCAL GETUSER(char  *name)
{
	strcpy(name,username);
	return 0;
}

__declspec(dllexport) short PASCAL LOADDLL(void) { return RPCNumRecsRead; }
__declspec(dllexport) short PASCAL GETEQPUSAGE(void) { return usageCount; }
__declspec(dllexport) short PASCAL EQPDEINIT(void)
{
	FreeApplication();
	return 0;
}

__declspec(dllexport) short PASCAL RPC(char *name, short EqNo, char *prop,
					short size, void *data, short datasize, short format)
{
	return syncAsyncRPC(name,EqNo,prop,size,data,datasize,format,3000);
}
__declspec(dllexport) short PASCAL RPCEx(char *name, short EqNo, char *prop,
					short size, void *data, short datasize, short format,long timeout)
{
	return syncAsyncRPC(name,EqNo,prop,size,data,datasize,format,(unsigned short)timeout);
}
__declspec(dllexport) short PASCAL S_RPC(char  *name, short EqNo, char  *prop,
					short size, void  *data, short datasize, short format)
{
	return syncAsyncRPC(name,EqNo,prop,size,data,datasize,format,RPC_TIMEOUT);
}
__declspec(dllexport) short PASCAL A_RPC (HWND hWnd, char  *name, short EqNo, char  *prop,
					short size, short format)
{
	short id = Subscribe(hWnd,hNULL,name,EqNo,prop,0L,size,format,SINGLE);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}
__declspec(dllexport) short PASCAL SA_RPC (char  *name, short EqNo, char  *prop,
					short size, void  *data, short datasize, short format)
{
	return syncAsyncRPC(name,EqNo,prop,size,data,datasize,format,3000);
}

__declspec(dllexport) short PASCAL SUBSCRIBE (HWND hWnd, char  *name, short EqNo,
						 char  *prop, long timer, short size, short format,short mode)
{
	short id = Subscribe(hWnd,hNULL,name,EqNo,prop,timer,size,format,mode);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}

__declspec(dllexport) short LINKDATA (HWND hWnd, char  *name, short EqNo,
						 char  *prop, long timer, short size, short format,short mode)
{
	short id = Subscribe(hNULL,hWnd,name,EqNo,prop,timer,size,format,mode);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}

__declspec(dllexport) short PASCAL T_ATTACHLINK(char *devName,char *devprop,
				DTYPE *dout,DTYPE *din,short access,int pollrate,
        void (*callback)(int,int),int mode)
{
  return (short)AttachLink(devName,devprop,dout,din,access,pollrate,callback,mode);
}
__declspec(dllexport) short PASCAL HOTRPC (HWND hWnd, char  *name, short EqNo,
						 char  *prop, long timer, short size, short format)
{
	short id = Subscribe(hWnd,hNULL,name,EqNo,prop,timer,size,format,POLL);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}
__declspec(dllexport) short OPENLINK (HWND hWnd,char *devname,char *devprop,
						    DBLK *dout,DBLK *din,void *datain,
							short access,long timer,short mode)
{
	short id = OpenLink(hWnd,devname,devprop,dout,din,datain,access,timer,mode);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}
__declspec(dllexport) short PASCAL T_OPENLINK (HWND hWnd,char *devname,char *devprop,
						    DBLK *dout,DBLK *din,void *datain,
							short access,long timer,short mode)
{
	short id = OpenLink(hWnd,devname,devprop,dout,din,datain,access,timer,mode);
	if (id < 0) 
	{
	  SetErrorString(RPCLastErrorString,-id,0);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	return id;
}
__declspec(dllexport) short PASCAL EXECLINK (char *devname,char *devprop,
						    DBLK *dout,void *dataout,DBLK *din,void *datain,short access)
{
	return dllExecLink(devname,devprop,dout,dataout,din,datain,access);
}
__declspec(dllexport) short PASCAL EXECLINKEx (char *devname,char *devprop,
						    DBLK *dout,void *dataout,DBLK *din,void *datain,short access,long timeout)
{
	return dllExecLinkEx(devname,devprop,dout,dataout,din,datain,access,timeout);
}
__declspec(dllexport) short PASCAL T_EXECLINK (char *devname,char *devprop,
						    DBLK *dout,void *dataout,DBLK *din,void *datain,short access,long timeout)
{
	return dllExecLinkEx(devname,devprop,dout,dataout,din,datain,access,timeout);
}
__declspec(dllexport) short PASCAL RPCGETLASTERR(char *errstr)
{
	if (errstr == NULL) return argument_list_error;
	if (IsBadWritePtr(errstr,PREFIX+RPCERR_SIZE)) return buffer_too_small;
	strncpy(errstr,RPCLastErrorString,PREFIX+RPCERR_SIZE);
	return 0;
}
__declspec(dllexport) short PASCAL GetRPCLastError(short cc, char  *errstr)
{
	if (errstr == NULL) return argument_list_error;
	if (IsBadWritePtr(errstr,2*(PREFIX+RPCERR_SIZE))) return buffer_too_small;
	if (cc >= numErr)
	{
		strcpy(errstr,"Error Not Defined");
		return out_of_range;
	}
  if (cc < 0)
  {
	  strncpy(errstr,RPCLastErrorSource,PREFIX+RPCERR_SIZE);
	  strncpy(&errstr[strlen(errstr)],RPCLastErrorString,PREFIX+RPCERR_SIZE);
	  strncpy(errstr,erlst[cc],32);
  }
  else
  {
    strncpy(errstr,erlst[cc],32);
  }
	return 0;
}
__declspec(dllexport) short PASCAL RPCGETLASTERROR(char *errstr)
{
	if (errstr == NULL) return argument_list_error;
	if (IsBadWritePtr(errstr,2*(PREFIX+RPCERR_SIZE))) return buffer_too_small;
	strncpy(errstr,RPCLastErrorSource,PREFIX+RPCERR_SIZE);
	strncpy(&errstr[strlen(errstr)],RPCLastErrorString,PREFIX+RPCERR_SIZE);
	return 0;
}
__declspec(dllexport) short PASCAL RPCGETLASTSOURCE(char *srcstr)
{
	if (srcstr == NULL) return argument_list_error;
	if (IsBadWritePtr(srcstr,(PREFIX+RPCERR_SIZE))) return buffer_too_small;
	strncpy(srcstr,RPCLastErrorSource,PREFIX+RPCERR_SIZE);
	return 0;
}
__declspec(dllexport) short GETGLOBALID(char *keyword)
{
  int i;
  char keytag[20];
  if (strlen(keyword) > 17) return -non_existent;
  sprintf(keytag,":%s:",keyword);
  for (i=1; i<=nglobals; i++) if (strstr(link[i].linkTag,keytag)) return i;
  return -non_existent;
}
__declspec(dllexport) short PASCAL GETGLOBALEx(char *keyword, void *buf, short size, short format,double *stale)
{
  int cc,i;
  ElemDataStruct *e;
  HANDLE hContract=0;
  char keytag[20];
  void *src;
  double ts, dts;

  if (!ListenForGlobals) 
  {
    if ((cc=ListenForNetGlobals()) != 0) return cc;
    ListenForGlobals = TRUE;
  }
  if (strlen(keyword) > 17) return non_existent;
  sprintf(keytag,":%s:",keyword);
  if (buf == NULL || IsBadWritePtr(buf,size*getFormatSize(format)) ) return buffer_too_small;

  for (i=1; i<=nglobals; i++) if (strstr(link[i].linkTag,keytag))
  {
    if (!(e=ContractAlloc(link[i].linkTag,link[i].linkSizeIn,link[i].linkSizeOut,&hContract))) return un_allocated;
    src = rpcDATAOUT(e); //  pointer to chunck
    if (stale != NULL)
    {
      ts = makeDataTimeStamp();
      dts = putDataTimeStamp(0,e->RPCtimestamp,e->RPCtimestampMSEC);
      *stale = ts - dts;
    }
    if (link[i].dataFlg <= 0)
    {
      cc = reformatData(buf,format,size,src,e->formatOut,(unsigned short)e->sizeOut);
    }
    else cc = link[i].dataFlg;
    if (hContract) ContractFree(e,hContract);
    return cc;
  }
# ifdef USE_SHARED_MEMORY
  cc = non_existent;
# else
  if ((i=AppendGlobalsList(keyword,size,format,1)) < 0) return -i;
  cc = reformatData(buf,format,size,GlobalList[i].Data,GlobalList[i].format,GlobalList[i].size);
  if (stale != NULL) *stale = 0;
# endif
  return cc;
}
__declspec(dllexport) short PASCAL GETGLOBAL(char *keyword, void *buf, short size, short format)
{
  return GETGLOBALEx(keyword,buf,size,format,NULL);
}
__declspec(dllexport) int GetNetGlobalEx(char *keyword, void *buf, short size, short format, double *stale)
{
  return GETGLOBALEx(keyword,buf,size,format,stale);
}
__declspec(dllexport) int GetNetGlobal(char *keyword, void *buf, short size, short format)
{
  return GETGLOBAL(keyword,buf,size,format);
}
__declspec(dllexport) short PASCAL PUTRPC (short i, void  *buf)
{
  ElemDataStruct  *e;
  HANDLE hContract=0;
  int cc=0,dsize;

  if (buf == NULL) return illegal_delimiter;
  if (i > MAXHOTLINKS || i < 0) return argument_list_error;
  if (!link[i].active) return ask_error;
  if (!(e=ContractAlloc(link[i].linkTag,link[i].linkSizeIn,link[i].linkSizeOut,&hContract))) return un_allocated;
  if (buf == NULL || 
      IsBadReadPtr(buf,dsize=e->sizeIn*getFormatSize(e->formatIn)) ) 
    ccerr(buffer_too_small);
  memcpy(rpcDATAIN(e),buf,dsize);
err:
  if (hContract) ContractFree(e,hContract);
  return cc;
}
__declspec(dllexport) short PutRPCErrorValue (short i, void *errValue)
{
  if (i > MAXHOTLINKS || i < 0) return argument_list_error;
  if (!link[i].active) return ask_error;
  memcpy(link[i].errValue,errValue,64);
  link[i].useErrValue = TRUE;
	return 0;
}
__declspec(dllexport) short PASCAL T_PUTRPCERRORVALUE(short i, void *errValue)
{
  return PutRPCErrorValue (i,errValue);
}
__declspec(dllexport) short CHANGEMODE (short i, short mode)
{
  ElemDataStruct  *e;
  HANDLE hContract=0;

  if (i > MAXHOTLINKS || i < 0) return argument_list_error;
  if (!link[i].active) return ask_error;
  if (!(e=ContractAlloc(link[i].linkTag,link[i].linkSizeIn,link[i].linkSizeOut,&hContract))) return un_allocated;
  link[i].mode = mode;
  e->mode = mode;
// 	if (mode == SINGLE) e->lockCount++;
  if (hContract) ContractFree(e,hContract);
	return 0;
}
__declspec(dllexport) short PASCAL GETRPCDATAEx(short i, void *buf,time_t *timestamp,long *datasize)
{
  short cc;
  double ts;
  cc = GetDataAndTime(i,buf,&ts,datasize,NULL);
  *timestamp = (long)ts;
  return cc;
}
__declspec(dllexport) short PASCAL GetRPCDataAndTime(short i, void *buf,double *dtimestamp,long *datasize)
{
  return GetDataAndTime(i,buf,dtimestamp,datasize,NULL);
}
__declspec(dllexport) short PASCAL GetDataAndTime(short i, void *buf,double *dtimestamp,long *datasize,int *status)
{
	short flg,rc,cc=0;
	ElemDataStruct *e = NULL;
	HANDLE hContract=0;
  int dsize,sts = 0;

	if (i > MAXHOTLINKS || i < 1) ccerr(argument_list_error);
	if (!link[i].active) ccerr(ask_error);
	if (!(e=ContractAlloc(link[i].linkTag,link[i].linkSizeIn,link[i].linkSizeOut,&hContract))) ccerr(un_allocated);
	if (e->mode <= SINGLE && e->lockCount <= 0) ccerr(device_not_connected);
  if (dtimestamp == NULL) ccerr(argument_list_error);
  *dtimestamp = putDataTimeStamp(0,e->RPCtimestamp,e->RPCtimestampMSEC);
	if (e->CompCode) 
	{
	  strncpy(RPCLastErrorString,e->RPCLastErrorString,PREFIX+RPCERR_SIZE);
    strncpy(RPCLastErrorSource,e->FecName,PREFIX+RPCERR_SIZE);
	}
	dsize = e->sizeOut*getFormatSize(e->formatOut);
	if (e->formatOut == CF_SPECTRUM) dsize += 96;
  if (buf != NULL && IsBadWritePtr(buf,dsize)) ccerr(buffer_too_small);
  if ((flg=link[i].dataFlg) > 0 && (flg & CE_SENDDATA)) flg = -1; // return code with data
  if (flg > 0 && link[i].useErrValue) flg = 0; // filled with error value
	switch (flg)
	{
	  case -not_posted:
      if (buf && dsize) memset(buf,0,dsize);
      rc = -1;
      break;
	  case 0:
      rc = (e->CompCode & 0x0fff);   // no new data -> use current error code
	  case -1:
      if (status != NULL && link[i].supressDuplicates) sts = !_datcmp(buf,rpcDATAOUT(e),dsize,e->formatOut,link[i].dtf);
      if (buf && dsize) 
      {
        e->isBeingAccessed = -1;
        memcpy(buf,rpcDATAOUT(e),dsize); // success
        e->isBeingAccessed = 0;
      }
      flg = 0; // read or not read -> let this be success
      if (datasize != NULL) *datasize = (long)e->lastDataSize;
	  default:
      if (e->mode <= SINGLE) e->lockCount--;
      link[i].dataFlg = 0;   // mark link as read
      rc = e->CompCode;
      break;
	}
err:
  if (cc) 
	{
	  rc = cc;
	  sprintf(RPCLastErrorString,"[LCL] %s",cc < numErr ? erlst[cc] : RPCLastErrorString);
    strncpy(RPCLastErrorSource,"[INTERNAL]",PREFIX+RPCERR_SIZE);
	}
	if (e != NULL && hContract) ContractFree(e,hContract);
  if (status != NULL) *status = sts;
	return rc;
}
__declspec(dllexport) short PASCAL GETRPCDATA(short i, void *buf,time_t *timestamp)
{
  return GETRPCDATAEx(i,buf,timestamp,NULL);
}
__declspec(dllexport) short PASCAL GETRPC (short i, void  *buf)
{
  time_t scrap;
  return GETRPCDATA(i,buf,&scrap);
}
__declspec(dllexport) short PASCAL CLOSERPC (HWND hWnd, short i)
{
  short rc = 0;
  char str[128];

  if (i > MAXHOTLINKS || i < 1) return argument_list_error;
  if (i <= nglobals) return out_of_range;
  if (!link[i].active) return ask_error;
  SendMessage(hDLLWnd,WM_REMOVELINK,(WORD)i,0);
  sprintf(str,"CloseRPC %d",i);
  messageLog(str,0);
  return rc;
}
__declspec(dllexport) short CLOSELINK (HWND hWnd, short i)
{
  short rc = 0;
  char str[128];

  if (i > MAXHOTLINKS || i < 1) return argument_list_error;
  if (i <= nglobals) return out_of_range;
  if (!link[i].active) return ask_error;
  if (link[i].mode != SINGLE) SendMessage(hDLLWnd,WM_REMOVELINK,(WORD)i,0);
  sprintf(str,"CloseLink %d",i);
  messageLog(str,0);
  return rc;
}
__declspec(dllexport) short PASCAL RPCERR(short err, char  *errstr)
{
	if (err < 0 || err >= numErr)
	{
		strcpy(errstr,"Error Not Defined");
		return -1;
	}
	strncpy(errstr,erlst[err],32);
	return 0;
}
__declspec(dllexport) short SetUserName(char *uname)
{
  if (uname != NULL) strncpy(username,uname,16);
	return 0;
}
__declspec(dllexport) short PASCAL RPCDBG(short dbg)
{
  if (dbg >= 0) debug = dbg;
  if (debug < 3) ShowWindow(hDLLWnd,debug ? SW_SHOW : SW_HIDE);
//	SendMessage(hDLLWnd,WM_RPCDEBUG,(WORD)debug,0);
	return debug;
}
__declspec(dllexport) short PASCAL FILLSTRUCT (char *tag, short addr, short size, short fmt)
{
  struct structStruct *s;
  if ((s=findstruct(tag)) == NULL) return out_of_local_memory;
  if (fmt == CF_NULL)
  {
    s->num = size;
    s->size = addr;
    s->rawlen = s->size;
    return 0;
  }
  return illegal_format;
}
__declspec(dllexport) long PASCAL GETSTRUCTSIZE (char *tag)
{
  struct structStruct *s=getStructFromTag(tag);
  return s == NULL ? 0 : (long) s->size;
}

__declspec(dllexport) short GetContexts(NAME16 *contexts,int *bufsiz)
{
  int n,cc;
  DBLK dout;

  if (bufsiz == NULL || (n = *bufsiz) == 0) ccerr(argument_list_error);
  dout.dArrayLength = n;
  dout.dFormat = CF_NAME16;
  memset(dout.dTag,0,8);
  cc = dllExecLinkEx(CSHOSTS,"CONTEXTS",&dout,contexts,NULL,NULL,CA_READ,900);
  if (cc == 0) *bufsiz = dout.dArrayLength;
err:
  return cc;
}
__declspec(dllexport) short GetDeviceServers(NAME16 *context,NAME16 *servers,int *bufsiz)
{
  int n,cc;
  DBLK dout;
  char dn[64];

  if (bufsiz == NULL || (n = *bufsiz) == 0) ccerr(argument_list_error);
  dout.dArrayLength = n;
  dout.dFormat = CF_NAME16;
  memset(dout.dTag,0,8);
  if (context != NULL && strlen(context->name) > 0) sprintf(dn,"%s/%s",CSHOSTS,context->name);
  else sprintf(dn,"%s",CSHOSTS);
  cc = dllExecLinkEx(dn,"TAGS",&dout,servers,NULL,NULL,CA_READ,900);
  if (cc == 0) *bufsiz = dout.dArrayLength;
err:
  return cc;
}
__declspec(dllexport) short PASCAL GetPropertyDeviceNames(NAME16 *context,NAME16 *server,NAME16 *names,NAME32 *property,int *bufsiz)
{
  int n,cc;
  DBLK dout;
  char dn[64],prp[33];

  if (bufsiz == NULL || (n = *bufsiz) == 0) ccerr(argument_list_error);
  if (strlen(property->name) > 28) ccerr(string_too_long);
  if (context != NULL && strlen(context->name) > 0) sprintf(dn,"/%s/%s/#0",context->name,server->name);
  else sprintf(dn,"%s/#0",server->name);
  sprintf(prp,"%s.NAM",property->name);
  memset(dout.dTag,0,8);
  dout.dArrayLength = n;
  dout.dFormat = CF_NAME16;
  cc = dllExecLink(dn,prp,&dout,names,NULL,NULL,CA_READ);
  if (cc & CE_SENDDATA) cc = 0; // has query function !
  if (cc == 0) *bufsiz = dout.dArrayLength;
err:
  return cc;
}
__declspec(dllexport) short GetDeviceNames(NAME16 *context,NAME16 *server,NAME16 *names,NAME32 *property,int *bufsiz)
{
  int n,cc;
  short ndevs;
  DBLK dout;
  char dn[64];

  if (bufsiz == NULL || (n = *bufsiz) == 0) ccerr(argument_list_error);
  if (context != NULL && strlen(context->name) > 0) sprintf(dn,"/%s/%s/#0",context->name,server->name);
  else sprintf(dn,"%s/#0",server->name);
  memset(dout.dTag,0,8);

  dout.dArrayLength = 1;
  dout.dFormat = CF_SHORT;
  if ((cc=dllExecLinkEx(dn,"NDEVICES",&dout,&ndevs,NULL,NULL,CA_READ,200)) != 0) ccerr(cc);
  if (ndevs > n) ndevs = n;
  
  dout.dArrayLength = ndevs;
  dout.dFormat = CF_NAME16;
  cc = dllExecLink(dn,"DEVICES",&dout,names,NULL,NULL,CA_READ);
  if (cc & CE_SENDDATA) cc = 0; // has query function !
  if (cc == 0) *bufsiz = dout.dArrayLength;
err:
  return cc;
}
__declspec(dllexport) short PASCAL GetDevicePropertyInformation(
    char *fullDeviceName,
    char *property,
    PropertyQueryStructEx *pqsx,
    int *bufsiz
  )
{
  int cc;
  short nprps;
  NAME32 prp;
  DBLK dout,din;

  if (bufsiz == NULL || (nprps=*bufsiz) < 1) return argument_list_error;
  if (property == NULL || strlen(property) == 0) return argument_list_error;

  strncpy(prp.name,property,32);
  din.dArrayLength = 1;
  din.dFormat = CF_NAME32;
  memset(din.dTag,0,8);

  dout.dArrayLength = nprps*PROPERTYQUERYSTRUCTEX_SIZE;
  dout.dFormat = CF_STRUCT;
  memset(dout.dTag,0,8);

  if ((cc=dllExecLink(fullDeviceName,"PROPS",&dout,pqsx,&din,&prp,CA_READ)) == 0)
  {
    *bufsiz = dout.dArrayLength / PROPERTYQUERYSTRUCTEX_SIZE;
  }
  return cc;
}
__declspec(dllexport) short GetDeviceProperties(NAME16 *context,NAME16 *server,NAME16 *name,NAME32 *properties,int *bufsiz)
{
  int i,n,cc;
  short nprps;
  DBLK dout;
  char dn[64],dev[17];
  PropertyQueryStruct *pqs;

  if (bufsiz == NULL || (n = *bufsiz) == 0) ccerr(argument_list_error);
  memset(dout.dTag,0,8);
  if (name == NULL || strlen(name->name) == 0) strcpy(dev,"#0");
  else strncpy(dev,name->name,16);
  if (context != NULL && strlen(context->name) > 0) sprintf(dn,"/%s/%s/%s",context->name,server->name,dev);
  else sprintf(dn,"%s/%s",server->name,dev);

  dout.dArrayLength = 1;
  dout.dFormat = CF_SHORT;
  if ((cc=dllExecLinkEx(dn,"NPROPS",&dout,&nprps,NULL,NULL,CA_READ,900)) != 0) ccerr(cc);
  if (nprps > n) nprps = n;

  dout.dArrayLength = nprps;
  dout.dFormat = CF_NAME32;
  cc = dllExecLink(dn,"PROPS",&dout,properties,NULL,NULL,CA_READ);
  if (cc & CE_SENDDATA) cc = 0; // has query function !

  if (cc == 0) *bufsiz = dout.dArrayLength;
  if (cc == illegal_format) // try again for the legacy guys ...
  {
    dout.dArrayLength = nprps*PROPERTYQUERYSTRUCT_SIZE;
    dout.dFormat = CF_STRUCT;
    memset(dout.dTag,0,8);
    if ((pqs=(PropertyQueryStruct *)_alloca(nprps*PROPERTYQUERYSTRUCT_SIZE)) == NULL) ccerr(out_of_local_memory);
    if ((cc=dllExecLink(dn,"PROPS",&dout,pqs,NULL,NULL,CA_READ)) == 0)
    {
      n = dout.dArrayLength / PROPERTYQUERYSTRUCT_SIZE;
      for (i=0; i<n; i++) strncpy(properties[i].name,pqs[i].prpName,32);
      *bufsiz = n;
    }
  }
err:
  return cc;
}
__declspec(dllexport) short GetDeviceEGU(NAME16 *context,NAME16 *server,NAME16 *name,NAME32 *property,float *min,float *max,char *egu)
{
  int cc=0;
  DBLK dout;
  char dn[64],dp[32],dev[17];
  DUSTRING ustr;

  memset(dout.dTag,0,8);
  if (name == NULL || strlen(name->name) == 0) strcpy(dev,"#0");
  else strncpy(dev,name->name,16);
  if (context != NULL && strlen(context->name) > 0) sprintf(dn,"/%s/%s/%s",context->name,server->name,dev);
  else sprintf(dn,"%s/%s",server->name,dev);
  if (strlen(property->name) > 28) ccerr(illegal_property);
  sprintf(dp,"%s.EGU",property->name);
  dout.dArrayLength = 1;
  dout.dFormat = CF_USTRING;
  if ((cc=dllExecLinkEx(dn,dp,&dout,&ustr,NULL,NULL,CA_READ,900)) != 0) ccerr(cc);
  *min = ustr.f1val;
  *max = ustr.f2val;
  strncpy(egu,ustr.str,80);
err:
  return cc;
}

__declspec(dllexport) short PASCAL T_GETCONTEXTS(NAME16 *contexts,int *bufsiz)
{
	return GetContexts(contexts,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETCONTEXTSFromFile(NAME16 *contexts,int *bufsiz)
{
	return GetContextsFromFile(contexts,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETDEVICESERVERS(NAME16 *context,NAME16 *servers,int *bufsiz)
{
	return GetDeviceServers(context,servers,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETDEVICESERVERSFromFile(NAME16 *context,NAME16 *servers,int *bufsiz)
{
	return GetDeviceServersFromFile(context,servers,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETDEVICENAMES(NAME16 *context,NAME16 *server,NAME16 *names,NAME32 *property,int *bufsiz)
{
	return GetDeviceNames(context,server,names,property,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETDEVICEPROPERTIES(NAME16 *context,NAME16 *server,NAME16 *name,NAME32 *properties,int *bufsiz)
{
	return GetDeviceProperties(context,server,name,properties,bufsiz);
}
__declspec(dllexport) short PASCAL T_GETDEVICEEGU(NAME16 *context,NAME16 *server,NAME16 *name,NAME32 *property,float *min,float *max,char *egu)
{
	return GetDeviceEGU(context,server,name,property,min,max,egu);
}
__declspec(dllexport) double PASCAL T_GETDATATIMESTAMP(void)
{
  return gDataTimeStamp;
}
