/** \file rpcsrv.h
 * \brief TINE Client DLL documentation 
 *
 */
#ifndef _RPCSRV_H_
#define _RPCSRV_H_

#include "rpcdefs.h"

#ifndef MAXGLOBALS
# define MAXGLOBALS 25
#endif

#define MAX_ENTRIES      500 // connection table entries (* 70 bytes)
#define FEC_TIMEOUT      1   // seconds
#define SERVER_TIMEOUT   45  //clock ticks
#define SYSTEM_DELAY     100 // msecs
#define INTSWAP(w)       ((w<<8)|(w>>8))
#define hNULL            0
#define MAXHOTLINKS     500
#define MAXCOMMONLINKS   20
#define _POLLRATE      1000
#define RPC_RETRY_COUNT   3
#define RPC_TIMEOUT       6

#define FEC_NAME_SIZE    16
#define ELEM_NAME_SIZE   16
#define PROPERTY_SIZE    32
#define EQNAME_SIZE	      6
#define DEVICE_SIZE      16
#define CONTEXT_SIZE     16

#define DEFAULT_TCP_CONNECTION_TIMEOUT 200
#define MAXALLOWED_POLLRATE 65535L
#define MSECS(a,b) ((a).tv_sec-(b).tv_sec > MAXALLOWED_POLLRATE ? MAXALLOWED_POLLRATE :\
		    ((a).tv_sec-(b).tv_sec)*1000+((a).tv_usec-(b).tv_usec)/1000)

#define PREFIX 6
#define RPCERR_SIZE 32
extern char RPCLastErrorString[PREFIX+RPCERR_SIZE];
extern char RPCLastErrorSource[PREFIX+RPCERR_SIZE];
#define SetErrorString(s,rc,cl) {\
  memset(s,0,PREFIX+RPCERR_SIZE);\
  strcpy(s, (cl) ? "[RMT] " : "[LCL] ");\
  if (rc > 0 && rc < numErr) memcpy(&(s)[PREFIX],erlst[rc],RPCERR_SIZE-1);\
  else strcpy(&(s)[PREFIX],"error undefined");\
}

#define ccerr(e) {cc = e; goto err; }
#define DeReference(h)  GlobalLock(h); GlobalUnlock(h);
#define OFFSETIN(struc,field)	((unsigned short)&(((struc *)0L)->field))

typedef struct
{
  char name[8];
} NAME8;

typedef struct
{
  char name[16];
} NAME16;

typedef struct
{
  char name[32];
} NAME32;

typedef struct
{
  char comment[80];
  time_t tm;
  float s_start;
  float s_inc;
  UINT32 status;
  float d_spect_array[1024];
} DSPECTRUM;

typedef union
{
  void   *vptr;
  char   *cptr;
  UINT8  *bptr;
  UINT16 *sptr;
  UINT32 *lptr;
  float  *fptr;
  double *dptr;
} DUNION;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[16];
  DUNION data;
} DTYPE;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[8];
} DBLK;

typedef struct
{
  long   ival;
  float f1val;
  float f2val;
  time_t   ts;
  char str[80];
} DUSTRING;

struct structStruct
{
  char name[12];
  short rawlen;
  short size;
  short num;
  struct structStruct *next;
};
extern struct structStruct *structList;
struct structStruct *getStructFromTag(char *tag);
struct structStruct *findstruct(char *tag);

// =============================================================
//
// WINDOWS environment definitions:
//
// =============================================================
#define RPCMSG_RECEIVE  (WM_USER+1)
#define WM_RCVPKT       (WM_USER)
#define WM_RCVPKT_IPX   (WM_USER+1)
#define WM_RCVPKT_UDP   (WM_USER+2)
#define WM_RCVPKT_GLB   (WM_USER+3)
#define WM_CREATELINK   (WM_USER+4)
#define WM_SYNCRPC      (WM_USER+5)
#define WM_REMOVELINK   (WM_USER+6)
#define WM_SETRPCDATA   (WM_USER+7)
#define WM_GETRPCDATA   (WM_USER+8)
#define WM_FLUSHRPC     (WM_USER+9)
#define WM_LOCKLINK     (WM_USER+10)
#define WM_GETLINKID    (WM_USER+11)
#define WM_RPCDEBUG     (WM_USER+12)
#define WM_RCVPKT_UDPBCST (WM_USER+13)
#define WM_RCVPKT_IPXBCST (WM_USER+14)
#define WM_RCVPKT_TCP   (WM_USER+15)
#define WM_RPCCALLBACK  (WM_USER+16)
#define WM_READ_LCL        (WM_USER+100)
// =============================================================
//
// Subscription oriented Producer-consumer definitions:
//
// =============================================================

#define NCONSUMER     20
#define NCONTRACT     50
#define RENEWAL       60
#define HEARTBEAT     60
#define REMINDER      5
#define ACK_REQUEST   (2*REMINDER)
#define REPLY_PENDING 3
#define SUB_PENDING   0x3fff
#define ACK_PENDING   0x7fff
#define CANCEL	      CM_CANCEL
#define SINGLE        CM_SINGLE
#define REFRESH       CM_REFRESH
#define POLL	      CM_POLL
#define REGISTER      CM_REGISTER
extern char *SubType[];
extern int nconsumer;
extern int ncontract;

#define MAXMODE(m1,m2) (\
  m1 == REGISTER ? m2 :\
  m2 == REGISTER ? m1 :\
  m1 > m2 ? m1 : m2\
)

typedef struct
{
  char EqpName[8],
  Property[12];
  short EqpSize,
  EqpFormat,
  EqpNumber;
  UINT16 PollRate,  // find a place for these two
  PollMode;
} CONTRACT4;
#define CONTRACT4_SIZE (8+12+5*2)

typedef struct             /* net: Protocol 5 (in) */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
} CONTRACT;
#define CONTRACT_SIZE (32+16+8+4*2+4*1+2*8)

typedef struct             /* local */
{
  char   EqpProperty[32];
  char   EqpDeviceName[32];
  char   EqpTagName[16];
  char   EqpContext[16];
  char   FecName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
  BYTE   *DataIn;  
  UINT32 PollRate;
  UINT32 PollMode;
} LCLCONTRACT;

typedef struct
{
  BYTE network[4],
       node[6],
       socket[2];
}
IPXAddress;

typedef struct
{
  char               userName[16];
  SOCKADDR_IPX       IPXaddress;  // if 0 then IP transfer is assumed
  struct sockaddr_in IPaddress;
  short               count;      // number of registrations
  short              transport;   // transport protocol
  int                sck;         // communication socket
} CLIENT;

typedef struct
{
  UINT16 msgsize,
  SubscriptionID,         // supplied by the consumer upon registration
	CompletionCode,
  counter;                // tell consumer how much is left
  time_t timestamp,             // RPCtimestamp
  starttime;             // supplied by the consumer upon registration
} ProducerHeaderStruct4;
#define PRODUCER4_SIZE (4*2+2*4)

typedef struct                  /* net: Protocol 5 (out) */
{
  UINT16 msgsize;               /* message size in bytes */
  UINT16 SubscriptionID;        /* supplied by the consumer upon registration */
  short  CompletionCode;        /* completion or interim status */
  UINT16 numblks;               /* total number of message blocks */
  UINT16 blknum;                /* block number of this message */
  UINT16 mtu;                   /* maximum transport unit */
  UINT16 timestampMSEC;         /* data timestamp (msec fraction) */
  BYTE   EqpFormat;             /* Eqp Format of returned data */
  BYTE   counter;               /* tell consumer how much is left */
  time_t timestamp;             /* data timestamp */
  time_t starttime;             /* supplied by the consumer upon registration */
} ProducerHeaderStruct;
#define PRODUCER_SIZE (7*2+2*1+2*4)

struct subscription4
{
  UINT16 msgsize;
  UINT16 PollingRate,
		 id,
		 mode;
  time_t   starttime;
  CONTRACT4 contract;
};
typedef struct subscription4 SUBSCRIPTION4;
#define SUBSCRIPTION4_SIZE (4*2+1*4+CONTRACT4_SIZE)

struct subscription         /* net: Protocol 5 (in) */
{
  UINT16    msgsize;
  UINT16    mtu;            /* maximum transport unit for incoming data */
  UINT8     numblks;        /* total number of message blocks           */
  UINT8     blknum;         /* block number of this message             */
  UINT16    id;
  short     mode;
  short     blkid;          /* (incremented) block id */
  UINT32    PollingRate;
  time_t    starttime;
  CONTRACT contract;
};
typedef struct subscription SUBSCRIPTION;
#define SUBSCRIPTION_SIZE (6*2+2*4+CONTRACT_SIZE)

extern HANDLE hContractList[NCONTRACT];  // ContractListStruct
extern HANDLE hConsumers[NCONSUMER];  // registered consumer list (CLIENT)

extern int PCSocket;              // IPX Producer/Consumer send socket
extern int ProducerSocket;        // UDP Producer Socket
extern int ConsumerSocket;        // UDP Consumer Socket
extern int GlobalSocket;          // UDP Global listening Socket
extern int MCastConsumerSocket;
extern int ListenForMCasts;
extern char *GLBEQM;
extern int numglobals;

extern struct sockaddr_in prod_name;
extern SOCKADDR_IPX ipx_prod_name;
#define SendToPC(c,d,s)	\
 ((cc=sendDatagram(c,d,s)) == illegal_protocol ? SendUDPData(c,d,s) : cc )

// =================================================================
typedef struct
{
  float absolute;
  float percent;
} DataToleranceFactor;

typedef struct RedirectedLinkStruct
{
  char srcContext[16];
  char srcServerName[16];
  char srcProperty[32];
  char srcDeviceName[32];
  char dstServerName[16];
  char dstProperty[32];
  struct RedirectedLinkStruct *prv;
  struct RedirectedLinkStruct *nxt;
} REDIRECTED_LINK;
extern REDIRECTED_LINK *RedirectedLinkList;

typedef struct ELEMDATA
{
  struct ELEMDATA *Next;
  int hDataIn;        // shared memory handle (data down)
  int hDataOut;       // shared memory handle (data up)
  char ElemName[16];
  char EqName[EQNAME_SIZE+1];
  char Property[32];
  char DeviceName[32];
  char Context[16];
  char FecName[16];
  short EqNo;
  short formatIn;       /* eqp data format down */
  short formatOut;      /* eqp data format up */
  char   strTagIn[8];   // structure tag Input
  char   strTagOut[8];  // structure tag Output
  UINT32 sizeIn;        /* data array size down */
  UINT32 sizeOut;       /* data array size up */
  short access;         /* data access */
  short CompCode;       // completion code
  short LastCode;       // last completion code
  short isInitializing; // flag
  short isDiscardable;  // throw it away ?
  short CompLoc;        // completion location (remote = 1, local =0)
  UINT32 PollRate;
  short FecIndex;
  short links[MAXCOMMONLINKS]; // make this a link list when we have the time
  BOOL Acquire;
  short timeoutCount;   /* used by alarmCheck as retry mechanism */
  short lockCount;      /* used by sync subscriptions */
  short ID;             /* used by subscriptions */
  UINT32 heartbeat;     /* used by subscriptions */
  WORD mode;            /* used by subscriptions */
  WORD PrimeMode;       /* used as internal record */
  short  RPCprotocol;   /* RPC header type */
  short  protocol;      /* transport protocol (IPX,SPX,UDP,TCP) */
  int  TCPsocket;       /* if there's a connection */
  time_t time;          /* last polling request */
  time_t starttime;     /* entry into table (used by subscriptions) */
  time_t RPCtimestamp;  /* arriving data timestamp */
  UINT16 RPCtimestampMSEC; /* arriving data timestamp (msec fraction) */
  short RPCversion[2];
  int  counter;         /* current subscription counter */
  UINT16 numblks;        /* progress counters for incoming data */
  UINT16 blksin;         /* progress counters for incoming data */
  UINT16 datamtu;       /* outgoing data minimum transport unit */
  UINT16 nextDataSize;  /* next outgoing data size in bytes */
  UINT32 nextDataPtr;   /* next outgoing data array pointer */
  UINT32 BytesRemaining;/* number of bytes data remaining */
  UINT32 BytesTotal;    /* total number of bytes data */
  UINT32 Bytesin;       /* progress counters for incoming data  */
  UINT16 blkidIn;       /* block Id of data to send */
  UINT16 numblksIn;      /* num blks input data to send */
  UINT16 blknumIn;       /* this blk num of input data */
  char RPCLastErrorString[PREFIX+RPCERR_SIZE];
  BYTE *outBuf;         /* data collection buffer if necessary (multiple-packets) */
  UINT16 isBeingAccessed;/* data block being accessed via GETRPCDATA */
  UINT16 srvID;         /* server subscription ID (applicable in MCAST) */
  UINT32 lastDataSize;  /* last data size returned from server */
  REDIRECTED_LINK *redirectedLink;   
} ElemDataStruct;

extern ElemDataStruct *ElemList;

#define rpcDATAIN(e) ((BYTE *)((BYTE *)(e) + (e)->hDataIn))
#define rpcDATAOUT(e) ((BYTE *)((BYTE *)(e) + (e)->hDataOut))

typedef struct
{
  char ElemName[16];
  char EqName[8];
  char Property[32];
  char DeviceName[16];
  short formatIn;       /* eqp data format down */
  short formatOut;      /* eqp data format up */
//  char  strTagIn[8];   // structure tag Input
//  char  strTagOut[8];  // structure tag Output
  char DeviceContext[16];
  UINT32 sizeIn;        /* data array size down */
  UINT32 sizeOut;       /* data array size up */
  short access;         /* data access */
  short CompCode;       // completion code
  UINT16 PollRate;
  short FecIndex;
  short links[MAXCOMMONLINKS]; // make this a link list when we have the time
  short timeoutCount;   /* used by alarmCheck as retry mechanism */
  short lockCount;      /* used by sync subscriptions */
  short ID;             /* used by subscriptions */
  WORD heartbeat;       /* used by subscriptions */
  WORD mode;            /* used by subscriptions */
  WORD PrimeMode;       /* used as internal record */
  short  RPCprotocol;   /* RPC header type */
  short  protocol;      /* transport protocol (IPX,SPX,UDP,TCP) */
  time_t time;          /* last polling request */
  time_t starttime;     /* entry into table (used by subscriptions) */
  time_t RPCtimestamp;  /* arriving RPC timestamp */
  int  counter;         /* current subscription counter */
} ElemQueryStruct;

typedef struct
{
  HWND hWnd;     	        // calling program to receive WM_PAINT (paint method)
  HWND hCtrlWnd;          // calling Ctrl to receive event notification
  void (*callback)(int,int); // callback routine (direct interface)
  BYTE *dataOut;          // pointer to dataout buffer for direct interface
  DataToleranceFactor *dtf;// pointer to data tolerance factor if present
  ElemDataStruct *Elem;   // Pointer to Element structure
  char  linkTag[128];     // MMF identifier
  int   linkSizeIn;       // MMF identifier
  int   linkSizeOut;      // MMF identifier
  HANDLE hContract;       // MMF handle
  short dataFlg,          // present status
        active;           // 0: free, 1: in use, -1: on hold
  short mode,             // requested mode for this link
		    cnt;              // cleanup counter
  BYTE  errValue[64];     // buffer for assigned error value
  short useErrValue,      // flag 
        supressDuplicates;
} linkType;

typedef struct
{
  char  FecName[FEC_NAME_SIZE];
  char  reserved[5];
  char  FullName[ELEM_NAME_SIZE];
  char  EqName[EQNAME_SIZE+1];
  char  Context[16];
  short alarmSystem;
} RPCDataStruct;
#define RPCDATASTRUCT_SIZE (FEC_NAME_SIZE+5+ELEM_NAME_SIZE+EQNAME_SIZE+1+16+2)

typedef struct
{
  char Name[FEC_NAME_SIZE];
  BYTE reserved[3];
  BYTE Port[2];
  BYTE Network[4];
  BYTE Node[6];
  BYTE ImmediateAddress[6];
  char IP[16];
  BYTE PortOffset;
  DWORD iaddr;
  BYTE Prot;
  BYTE MaxRPCProt;
} RPCFecStruct;
#define RPCFECSTRUCT_SIZE (FEC_NAME_SIZE+3+2+16+21+2)

typedef struct
{
  char keyword[16];
  short size;
  short format;
  short ID;
  short heartbeat;
  double cTimeStamp;
  double dTimeStamp;
  BYTE *Data;
} GlobalListStruct;

typedef struct
{
  UINT16 TotalSize;
  char UserName[16];
  short Protocol;
  short EqpSize;
}HeaderStruct;
#define HEADERSTRUCT_SIZE 22

typedef struct
{
  char ElemName[16];
  char EqpModName[8];
  short EqpModNo;
  char Property[12];
  char Cycle[22];         // cycle specification for CERN PPM elements
  short Format;
  short DataSize;
}EqpInfoStruct;

typedef struct
{
  char  prpName[32];
  char  prpDescription[32];
  unsigned short prpSize;
  BYTE prpFormat;
  BYTE prpAccess;
} PropertyQueryStruct;
#define PROPERTYQUERYSTRUCT_SIZE (32+32+2+1+1)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
} PropertyQueryStructEx;
#define PROPERTYQUERYSTRUCTEX_SIZE (32+64+32+8*2+16+4*5+2*2+4)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
  char  rngUnits[16];
  float rngMinValue;
  float rngMaxValue;
  UINT16 numRows;
  UINT16 rowSize;
  UINT16 prpArrayType;
  UINT16 reserved[3];
} XPropertyQueryStruct;
#define XPROPERTYQUERYSTRUCT_SIZE (PROPERTYQUERYSTRUCTEX_SIZE + 16+2*4+6*2)

typedef struct
{
  char FecName[FEC_NAME_SIZE];
  char reserved[4];
  time_t localtime;
  time_t starttime;
  long   systemPollingRate;
  short  numBkgTsks;
  short  numTotalContracts;
  short  numTotalClients;
  short  numTargetContracts;
  short  numTargetClients;
  short  numConnections;
  UINT32 numConnectionTimeouts;
  UINT32 numConnectionArrivals;
  UINT32 numUDPpkts;
  UINT32 numTCPpkts;
  UINT32 numIPXpkts;
  UINT32 numSPXpkts;
} ActivityQueryStruct;
#define ACTIVITYQUERYSTRUCT_SIZE (FEC_NAME_SIZE+4+4*3+2*6+4*6)

typedef struct
{
  char desc[64];
  BYTE dataFmt;
  BYTE dataSiz;
  BYTE data[64];
  time_t timeEntered;
  UINT32 codeNow;
  UINT32 codePrv;
  UINT32 codeNxt;
} ServerStateStruct;
#define SERVERSTATESTRUCT_SIZE (64+2+64+4*4)

typedef struct
{
  char name[16];       // broadcast element name  (16 bytes)
  short  size;           // number of elements broadcast under "name" (2 bytes)
  short  format;         // data format (as per RPC definitions)  (2 bytes)
  long time;           // timestamp of broadcast data (4 bytes)
} BrdcstData;

#define IPX			    0x01
#define SPX         0x02
#define TCP         0x04
#define UDP         0x08
#define GLB         0x10
#define UDPBCST			0x20
#define IPXBCST			0x40
#define LCL         0x80

#define BCAST_ID 255

#define WORK_BUFFER_SIZE 0x1000
// IPX/SPX specific:
#define RPC_SOCKET      0x45 // listening SPX socket
#define BRDCST_SOCKET   0x67 // listening broadcast socket
#define REG_SOCKET      0x55 // listening registration socket
#define MCAST_SOCKET    0x85 // listening mcast socket
#define NUM_RCV_ECB     3              // associated incoming contracts
#define NUM_GLB_ECB     2              // incoming broadcasts
#define NUM_REG_ECB     2              // incoming registration requests
#define NUM_CLN_ECB     3              // client RPC ecbs
#define NUM_CNS_ECB     5              // consumer ecbs
#define IPX_DATA_SIZE   512
#define IPX_SHORTBUFFER_SIZE (IPX_DATA_SIZE + sizeof(ProducerHeaderStruct4) + sizeof(short))
//#define IPX_BUFFER_SIZE 1470
#define IPX_BUFFER_SIZE 1466 // for Client32 for DOS

// TCP/UDP specific:
#define NumTCPChan	      5
#ifndef UDP_PORT
# define UDP_PORT	      9003
#endif
#ifndef TCP_PORT
# define TCP_PORT	      8503
#endif
# define TCP_LEGACYPORT 8003
#ifndef GCAST_PORT
# define GCAST_PORT     8004
#endif
#ifndef MCAST_PORT
# define MCAST_PORT     9503
#endif
#ifndef UDP_CLN_PORT
# define UDP_CLN_PORT   8050
#endif
#ifndef UDP_CLN_RANGE
# define UDP_CLN_RANGE 400
#endif

#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif
#ifndef MCAST_TTL
# define MCAST_TTL 16
#endif

#define UDP_BUFFER_SIZE 1472
#define WORK_SIZE  (WORK_BUFFER_SIZE*sizeof(double))

#define MakeContractTag(tag,con,id,szin,szout) {\
	 sprintf((tag),"%d:%.16s:%.32s:%.16s:%d:%d:%d:%s:%d:%s:%d",id,(con)->EqpTagName,(con)->EqpProperty,(con)->EqpDeviceName,\
	    (con)->EqpSizeIn,(con)->EqpSizeOut,(con)->EqpFormatIn,(con)->strTagIn,(con)->EqpFormatOut,(con)->strTagOut,(con)->EqpAccess);\
	 szin = fmtsizeof((con)->EqpFormatIn)*(con)->EqpSizeIn;\
	 szout = fmtsizeof((con)->EqpFormatOut)*(con)->EqpSizeOut;\
 }

// ==========================================================
// ======            externals:                     =========
// ==========================================================
extern HANDLE hDLL,hDLLWnd;
extern int RPCNumRecsRead, RPCNumFecsRead;
extern int TimerPeriod;
extern HCURSOR WaitCurs;
extern char *erlst[];
extern int numErr;
extern GlobalListStruct *GlobalList;
extern TCPloaded;
extern RPCFecStruct *RPCFec;
extern BYTE  *WorkArea;
extern unsigned long WorkAreaSize;
extern int WorkAreaSemaphore;
extern BYTE  *Values;
extern RPCDataStruct  *RPCData;
extern RPCFecStruct   *RPCFec;
extern HANDLE hRPCData, hRPCFec;
extern HANDLE hGlobalList;
extern DataBaseElemsStruct RPCDataElems[];
extern BYTE rpc_socket[2];
extern BYTE reg_socket[2];
extern const BYTE brdcst_socket[2];
extern int lastAssignedLinkID;
extern double gDataTimeStamp;

extern int gtUDPPort;
extern int gtTCPPort;
extern int gtGCastPort;
extern int gtMCastPort;
extern int gtUCPClnPort;
extern char *gtMCastAddr;
extern char *gtGCastAddr;
extern int gtMCastTTL;

// ==========================================================
// ======            prototypes:                    =========
// ==========================================================

void output(char *txt);
int CreateElem(int linkID);
HANDLE CreateClient(int id,HANDLE hElemPointer,long Timer);
void RemoveElem(ElemDataStruct *Elem);
ElemDataStruct *GetElemPointer(LCLCONTRACT *c);
void SendData (HWND, HWND, char  *, char  * ,double  *, int,BOOL ,BOOL ,BOOL ,int,int);
void ValueToStr(double  *Values,char  *ValDesc,int Datasize,int RowNum,int ColNum,char  *EqName,char  *Property,BOOL Format);
int StrToValue(double  *Value,char *Command,char  *EqName);
int RPCSync(char  *Name,int EqNo, char  *Property,int Size,void  *Data,int Datasize,int Format);
short syncAsyncRPC(char *name,short EqNo,char *prop,short size,void *data,short datasize,short format,unsigned short timout);
void InvalidReadSyntax(HWND hWnd);
int FPostMessage(HWND hWnd,WORD wm,WORD wParam,LONG lParam);
int AttachToControlServer(void);
int DetachFromControlServer(void);
int addFecToList(char  *name);
void trim(char  *str);
void DoRPC(void);
void RemoveRPC(int ID);
void GetUDPData(void);
void GetUDPBrdcstData(void);
void CheckConnections(void);
void CheckSubscriptions(void);
void Cleanup(void);
void doGlobalUpdate(BYTE  *buf);
int listenUDP(int  *s,unsigned short port,unsigned int rcvbufs);
void PrepRequest(int size,ElemDataStruct *ElemStart);
int SendRequest(int size,ElemDataStruct *ElemStart);
int connectTCP(int port,ElemDataStruct *Elem);
char *ipxnet_ntoa(BYTE *node,int len);
void SPXSendData(WORD ID);
void TCPSendData(int socket);
ElemDataStruct *CloseConnectionSocket(int sck);
int getTCPStream(int socket);
int decodeData(int cc,int size,ElemDataStruct *ElemStart);
void NotifyClients(ElemDataStruct  *e);
short consumeData(BYTE  *buf);
int PrepSubBuffer(BYTE  *data);
int SendSubBuffer(int FECindex,int sck,int transport,BYTE *data);
#ifdef NOVELL
int getSPXStream(WORD ID);
int SendIPXData(CLIENT  *cln,BYTE  *data);
int connectSPX(ElemDataStruct *Elem);
void GetIPXData(void);
#endif
void OutputToDebugFile(char *routine, char *txt);
int SendUDPData(CLIENT  *cln,BYTE  *data,int UDPsocket);
short Subscribe(HWND hWnd,HWND hVBX,char *name,short EqNo,char *prop,long timer,short size,short format,short mode);
short OpenLink(HWND hWnd,char *devName,char *devprop,DBLK *dout,DBLK *din,void *datain,short access,long timer,short mode);
short dllExecLink(char *devname,char *devprop,DBLK *dout,void *dataout,DBLK *din,void *datain,short access);
short dllExecLinkEx(char *devname,char *devprop,DBLK *dout,void *dataout,DBLK *din,void *datain,short access,long timeout);
int initApplication(void);
int AllocateAndAssignMemory(void);
int initSPX(void);
int initTCP(void);
int initDataBase(void);
void initNetGlobals(void);
void GetGlobalIPX(void);
void FreeApplication(void);
short getRPCData (short i, void *data);
short setRPCData (short i, void *data);
short tRPC(int fecindex, char *name,short EqNo,char *prop,short size,void *data,short format,short timout);
extern int tExecLinkReturnedDataSize;
int tExecLink(int fecindex,char *name,char *dev,char *prop,
	   short size,char *data,short format,short timout);
int messageLog(char *txt,int status);
UINT16 GetPortOffset(RPCFecStruct *f);
struct timeval *gettimeofday(struct timeval *t,struct timezone *tz);
double putDataTimeStamp(double toffset, time_t tsec, int tmsec);
DWORD getmyipaddr(void);
double makeDataTimeStamp(void);
double putDataTimeStamp(double toffset, time_t tsec, int tmsec);
int _datcmp(char *d1,char *d2,int dsz,short fmt,DataToleranceFactor *dtf);

__declspec(dllexport) short PASCAL RPC(char *name, short EqNo, char *prop, short size, void *data, short datasize, short format);
__declspec(dllexport) short PASCAL RPCEx(char *name, short EqNo, char *prop, short size, void *data, short datasize, short format,long timeout);
__declspec(dllexport) short PASCAL HOTRPC(HWND hWnd, char *name, short EqNo, char *prop, long timer, short size, short format);
__declspec(dllexport) short PASCAL CLOSERPC(HWND hWnd, short ID);
__declspec(dllexport) short PASCAL GETRPC(short ID, void *buf);
__declspec(dllexport) short PASCAL GETGLOBAL(char *keyword, void *data, short size, short format);
__declspec(dllexport) short PASCAL PUTRPC (short i, void  *VBbuf);

/** 
 * \brief Use with \b Visual \b Basic \b 3.0 or \b 6.0. Retrieves data associated with the given link index into the calling process.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b GETRPCDATA Lib "tinecln32.dll" (ByVal linkId As Integer, databuf As Any, timestamp As Long) As Integer
 * \n Declare Function \b GETRPCSTRDATA Lib "tinecln32.dll" Alias "GETRPCDATA" (ByVal linkId As Integer, ByVal sdatabuf As String, timestamp As Long) As Integer
 *
 * \param linkId is the link identifier for which the data is desired
 * \param databuf is a reference to the data buffer into which the link data are to be copied.  This
 *        buffer must be pre-alloced with enough room to store the data.
 * \param timestamp is the data timestamp as UTC (seconds since Jan 1, 1970)
 *
 * It is only necessary to call this routine directly when using the 'Simple TINE Client ActiveX 
 * interface.
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \see GETRPCDATAEx(), GetRPCDataAndTime()
 *
 * \b Example:
 *
 * \include eg_GETRPCDATA.txt
 */
__declspec(dllexport) short PASCAL GETRPCDATA(short linkId, void *databuf,time_t *timestamp);

/** 
 * \brief Use with \b Visual \b Basic \b 3.0 or \b 6.0. Retrieves data associated with the given link index into the calling process.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b GETRPCDATAEx Lib "tinecln32.dll" (ByVal linkId As Integer, databuf As Any, timestamp As Long, datasize As Long) As Integer
 * \n Declare Function \b GETRPCSTRDATAEx Lib "tinecln32.dll" Alias "GETRPCDATA" (ByVal linkId As Integer, ByVal sdatabuf As String, timestamp As Long, datasize As Long) As Integer
 *
 * \param linkId is the link identifier for which the data is desired
 * \param databuf is a reference to the data buffer into which the link data are to be copied.  This
 *        buffer must be pre-alloced with enough room to store the data.
 * \param timestamp is the data timestamp as UTC (seconds since Jan 1, 1970)
 * \param datasize contains the data size (as array dimension) of the data set returned from
 *        the server. It is assumed that the size of the data buffer can hold the data returned.
 *        The data returned will never be greater than the size demanded by the initial
 *        link parameters as given for example in AttachLink().
 *
 * It is only necessary to call this routine directly when using the 'Simple TINE Client ActiveX 
 * interface.  This call differs from GETRPCDATA() in that it returns in the fourth parameter
 * the size of the data set actually returned from the server (possibly less than what was asked for.).
 * This call differs from GetRPCDataAndTime(), in that only the 'seconds' part of the data timestamp
 * is returned in the third parameter.
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \see GETRPCDATA(), GetRPCDataAndTime()
 */
__declspec(dllexport) short PASCAL GETRPCDATAEx(short linkId, void *databuf,time_t *timestamp,long *datasize);

/** 
 * \brief Use with \b Visual \b Basic \b 6.0. Retrieves data associated with the given link 
 * index into the calling process
 *
 * \b Visual \b Basic :
 * \n Declare Function \b GetRPCDataAndTime Lib "tinecln32.dll" (ByVal linkId As Integer, databuf As Any, timestamp As Long, datasize As Long) As Integer
 * \n Declare Function \b GetRPCStrDataAndTime Lib "tinecln32.dll" Alias "GetRPCDataAndTime" (ByVal linkId As Integer, ByVal sdatabuf As String, timestamp As Long, datasize As Long) As Integer
 *
 * \param linkId is the link identifier for which the data is desired
 * \param databuf is a reference to the data buffer into which the link data are to be copied.  This
 *        buffer must be pre-alloced with enough room to store the data.
 * \param dtimestamp is the TINE data timestamp as UTC + fractional seconds.
 * \param datasize contains the data size (as array dimension) of the data set returned from
 *        the server. It is assumed that the size of the data buffer can hold the data returned.
 *        The data returned will never be greater than the size demanded by the initial
 *        link parameters as given for example in AttachLink().
 *
 * It is only necessary to call this routine directly when using the 'Simple TINE Client ActiveX 
 * interface.  This call differs from GETRPCDATA() in that it returns the full TINE timestamp
 * (UTC + millisecons as fractional part) as well as the size of the data set actually returned
 * from the server (possibly less than what was asked for.).
 *
 * \note This call is unavailable in Visual Basic 3.0 (use GETRPCDATAEx()).
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \see GETRPCDATA(), GETRPCDATAEx()
 */
__declspec(dllexport) short PASCAL GetRPCDataAndTime(short linkId, void *databuf,double *dtimestamp,long *datasize);
__declspec(dllexport) short PASCAL GetDataAndTime(short i, void *buf,double *dtimestamp,long *datasize,int *status);
void IncomingIPX(void);
void IncomingGlobalIPX(void);
int  IncomingUDP(unsigned long i, int err);
__declspec(dllexport) short GetDeviceServersFromFile(NAME16 *context,NAME16 *servers,int *bufsiz);
__declspec(dllexport) short GetContextsFromFile(NAME16 *contexts,int *bufsiz);
__declspec(dllexport) short PASCAL ListenForMulticasts(void);
__declspec(dllexport) short PASCAL ListenForNetGlobals(void);

/**
 * \brief Non-Pascal version (Win32 Only). Establishes an asynchronous link with the given calling parameters.
 * 
 * Asynchronous data exchange. Calls in the AttachLink() family attempt to establish the requested data link
 * with the given calling parameters and complete immediately.  If the address of the requested 
 * server can be resolved and the calling parameters are otherwise okay, the call will complete
 * successfully and return a valid (positive) link handle.  This does not mean that the 
 * requested server will respond to the request.  The data and status information from the server
 * will be returned asynchronously inside the callback routine passed in the call.
 * In Visual Basic 3.0, there is no equivalent to this call.
 * In the Win32 world, use T_ATTACHLINK() with Visual Basic.
 * Use AttachLink() with interfaces which do not require the Pascal calling convention (e.g. VC++,
 * LabView, MatLab, etc.).
 *
 * \param devName is the full device name (/\<Context\>/\<Server\>/\<Device\>) of the device to contact.
 * For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module,
 * an internal call to ExecLocalLink() is made.  For instance: "/LOCAL/BPMEQM/WL167" sets the context to 
 * "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only 
 * be successful when the client and server are running together in the same process). 
 * \param devProperty is the device property requested, for example "ORBIT.X"
 * \param dout is a pointer to the output data set, that is, the data set to be returned from the server 
 * to the client.  The structure element dout->data should be a pointer to a pre-allocated data buffer,
 * which is to receive the returned data.  If dout is a NULL pointer, it is assumed that no data are to 
 * be returned.
 * \param din is a pointer to the input data set, that is, the data set to be sent to the server from the 
 * client.  If din is a NULL pointer, it is assumed that no data are to be sent to the server
 * \param access is the data access mode.  This can be any combination of access codes ORed together (CA_READ,
 * CA_WRITE, CA_CONNECT, etc.).
 * \param pollrate is the polling interval in milliseconds at which the server is requested to
 * acquire data. This value also signals a timeout if there is no notification within this amount
 * of time plus the system heartbeat (1000 msec for POLLing links).  One can use AttachLink()
 * for asynchronous single READ and WRITE requests by specifying a pollmode of CM_SINGLE.
 * \param callback is a reference to the a callback function which is to be called by the TINE
 * engine when either incoming data have arrived or an error condition has arised.  This should
 * be a pointer to a funtion with prototype \b void \b callback(\b int linkId, \b int statusCode)
 * \param pollmode is the TINE poll mode, i.e. one of CM_CANCEL, CM_SINGLE, CM_REFRESH, CM_POLL,
 * which can be ORed with one of CM_BCAST, CM_NETWORK, CM_MCAST, CM_USE_ON_ERROR, CM_RETRY,
 * CM_GROUPED, CM_CONNECT, CM_WAIT, or CM_SYNCNOTIFY. (see the discussion of Data Access Modes).
 *
 * \return A positive link handle ID if successful, otherwise the negative of a TINE completion 
 * code which can be interpreted by a call to GetRPCLastError().  If the TINE engine can resolve
 * the address of the requested server and all of the calling parameters are otherwise
 * correct the call should complete successfully.  Possible errors might be 'non existent element'
 * or 'non existent fec', etc.
 *
 * \note This form of the call cannot be used with Visual Basic. Use T_ATTACHLINK() instead.
 *
 * \see T_ATTACHLINK(), DTYPE, EXECLINK(), EXECLINKEx()
 */
__declspec(dllexport) int AttachLink(char *devName,char *devProperty,
				DTYPE *dout,DTYPE *din,short access,int pollrate,
        void (*callback)(int,int),int pollmode);
/**
 * \brief Use with \b Visual \b Basic \b 6.0. Establishes an asynchronous link with the given calling parameters.
 * 
 * Asynchronous data exchange. Calls in the AttachLink() family attempt to establish the requested data link
 * with the given calling parameters and complete immediately.  If the address of the requested 
 * server can be resolved and the calling parameters are otherwise okay, the call will complete
 * successfully and return a valid (positive) link handle.  This does not mean that the 
 * requested server will respond to the request.  The data and status information from the server
 * will be returned asynchronously inside the callback routine passed in the call.
 * In Visual Basic 3.0, there is no equivalent to this call.
 * In Visual Basic 6.0, you must declare the callback routine as a Subroutine 
 * (not a Function) inside a .BAS module and make use of the AddressOf routine to pass the
 * address of the this subroutine to T_ATTACHLINK().  This works in Visual Basic only to the
 * extent that no debug break points are set inside the callback routine.
 *
 * \param devName is the full device name (/\<Context\>/\<Server\>/\<Device\>) of the device to contact.
 * For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module,
 * an internal call to ExecLocalLink() is made.  For instance: "/LOCAL/BPMEQM/WL167" sets the context to 
 * "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only 
 * be successful when the client and server are running together in the same process). 
 * \param devProperty is the device property requested, for example "ORBIT.X"
 * \param dout is a pointer to the output data set, that is, the data set to be returned from the server 
 * to the client.  The structure element dout->data should be a pointer to a pre-allocated data buffer,
 * which is to receive the returned data.  If dout is a NULL pointer, it is assumed that no data are to 
 * be returned.
 * \param din is a pointer to the input data set, that is, the data set to be sent to the server from the 
 * client.  If din is a NULL pointer, it is assumed that no data are to be sent to the server
 * \param access is the data access mode.  This can be any combination of access codes ORed together (CA_READ,
 * CA_WRITE, CA_CONNECT, etc.).
 * \param pollrate is the polling interval in milliseconds at which the server is requested to
 * acquire data. This value also signals a timeout if there is no notification within this amount
 * of time plus the system heartbeat (1000 msec for POLLing links).  One can use AttachLink()
 * for asynchronous single READ and WRITE requests by specifying a pollmode of CM_SINGLE.
 * \param callback is a reference to the a callback function which is to be called by the TINE
 * engine when either incoming data have arrived or an error condition has arised.  This should
 * be a pointer to a funtion with prototype \b Sub \b sinCallback(\b ByVal id \b As \b Long, \b ByVal cc \b As \b Long)
 * \param pollmode is the TINE poll mode, i.e. one of CM_CANCEL, CM_SINGLE, CM_REFRESH, CM_POLL,
 * which can be ORed with one of CM_BCAST, CM_NETWORK, CM_MCAST, CM_USE_ON_ERROR, CM_RETRY,
 * CM_GROUPED, CM_CONNECT, CM_WAIT, or CM_SYNCNOTIFY. (see the discussion of Data Access Modes).
 *
 * \return A positive link handle ID if successful, otherwise the negative of a TINE completion 
 * code which can be interpreted by a call to GetRPCLastError().  If the TINE engine can resolve
 * the address of the requested server and all of the calling parameters are otherwise
 * correct the call should complete successfully.  Possible errors might be 'non existent element'
 * or 'non existent fec', etc.
 *
 * \note Unless you have a good reason to use this routine, it is better to make use of the
 * ACOP Receive Event for asynchronous data links.
 *
 * \see AttachLink(), DTYPE, EXECLINK(), EXECLINKEx()
 *
 * \b Example:
 * 
 * \include eg_TATTACHLINK.txt
 */
__declspec(dllexport) short PASCAL T_ATTACHLINK(char *devName,char *devprop,
				DTYPE *dout,DTYPE *din,short access,int pollrate,
        void (*callback)(int,int),int mode);

__declspec(dllexport) short OPENLINK (HWND hWnd,char *devname,char *devprop,
						    DBLK *dout,DBLK *din,void *datain,
							short access,long timer,short mode);

__declspec(dllexport) short PASCAL RPCERR(short err, char *errstr);

/** 
 * \brief Retrieves the TINE status string for the given completion code for the most
 * recent data link if cc < 0.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b GetRPCLastError Lib "tinecln32.dll" (ByVal cc As Integer, ByVal ErrString As String) As Integer
 *
 * \param cc is the TINE completion code (i.e. status code) for which the string representation 
 * is desired.
 * \param errString is a (pre-allocated!) string buffer into which the most recent completion
 * status will be copied.  This must point to a string buffer with enough room to hold the 
 * completion status (at least 48 characters).
 *
 * It is only necessary to call this routine directly when using the 'Simple TINE Client ActiveX 
 * interface.  If this the routine is called passing a negative number as the first
 * parameter, then
 * this routine should be called immediately following any synchronous link or 
 * asynchronous Receive event.  Used in this manner, the call is clearly not thread-safe, but
 * as Visual Basic builds single-threaded applications, this will
 * most likely give reasonable results. 
 * As a synchronous call will return the completion code directly as a return code, you can
 * safely use this call by passing the returned completion code as the first parameter.
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \b Example:
 *
 * \include eg_GETRPCDATA.txt
 */
__declspec(dllexport) short PASCAL GetRPCLastError(short cc, char *ErrString);

__declspec(dllexport) short PASCAL RPCGETLASTERROR(char *errString);
__declspec(dllexport) short CLOSELINK (HWND hWnd, short i);
__declspec(dllexport) int GetNetGlobal(char *keyword, void *buf, short size, short format);

/**
 * \brief Use with \b Visual \b Basic \b 3.0 or \b 6.0. Executes a synchronous link. 
 * 
 * Synchronous data exchange. Calls in the ExecLink() family do not complete until the data transfer has completed or 
 * a timeout has ensued. EXECLINK() differs from EXECLINKEx() in that the default timeout 
 * of 1000 msec is assumed.
 * Use ExecLink() with interfaces which do not require the Pascal calling convention.
 *
 * \param devName is the full device name (/\<Context\>/\<Server\>/\<Device\>) of the device to contact.
 * For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module,
 * an internal call to ExecLocalLink() is made.  For instance: "/LOCAL/BPMEQM/WL167" sets the context to 
 * "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only 
 * be successful when the client and server are running together in the same process). 
 * \param devProperty is the device property requested, for example "ORBIT.X"
 * \param dout is a reference to the output data block, that is, the data set to be returned from the server 
 * to the client.  This parameter must be present and should specifiy a format data type of
 * CF_NULL and data size of 0 if no return data are requested.
 * \param dataout is a reference to the array into which the server's output data should be copied.
 * This must be consistent with the data block parameters passed in the dout parameter,
 * otherwise the call will fail.
 * \param din is a pointer to the input data block, that is, the data set to be sent to the server from the 
 * client.  This parameter is required and should contain a format data type of CF_NULL and
 * data size of 0 if no data are to be sent to the server.
 * \param datain is a reference to the array from which the input data should be copied prior to
 * sending to the server.  If a 0 is passed as this parameter, no data are submitted to the
 * server.  This must be consistent with the data block parameters passed in the din parameter,
 * otherwise the call will fail.
 * \param access is the data access mode.  This can be any set of access codes ORed together (CA_READ,
 * CA_WRITE, CA_CONNECT, etc.)
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \see EXECLINKEx(), DBLK  
 *
 * \b Example:
 *
 * \include eg_EXECLINK.txt
 */
__declspec(dllexport) short PASCAL EXECLINK (char *devName,char *devProperty,DBLK *dout,void *dataout,DBLK *din,void *datain,short access);

/**
 * \brief Use with \b Visual \b Basic \b 3.0 or \b 6.0. Extended call. Executes a synchronous link with timeout parameter.
 * 
 * Synchronous data exchange. Calls in the ExecLink() family do not complete until the data transfer has completed or 
 * a timeout has ensued. EXECLINK() differs from EXECLINKEx() in that the default timeout 
 * of 1000 msec is assumed.
 * Use ExecLinkEx() with interfaces which do not require the Pascal calling convention.
 *
 * \param devName is the full device name (/\<Context\>/\<Server\>/\<Device\>) of the device to contact.
 * For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module,
 * an internal call to ExecLocalLink() is made.  For instance: "/LOCAL/BPMEQM/WL167" sets the context to 
 * "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only 
 * be successful when the client and server are running together in the same process). 
 * \param devProperty is the device property requested, for example "ORBIT.X"
 * \param dout is a reference to the output data block, that is, the data set to be returned from the server 
 * to the client.  This parameter must be present and should specifiy a format data type of
 * CF_NULL and data size of 0 if no return data are requested.
 * \param dataout is a reference to the array into which the server's output data should be copied.
 * This must be consistent with the data block parameters passed in the dout parameter,
 * otherwise the call will fail.
 * \param din is a pointer to the input data block, that is, the data set to be sent to the server from the 
 * client.  This parameter is required and should contain a format data type of CF_NULL and
 * data size of 0 if no data are to be sent to the server.
 * \param datain is a reference to the array from which the input data should be copied prior to
 * sending to the server.  If a 0 is passed as this parameter, no data are submitted to the
 * server.  This must be consistent with the data block parameters passed in the din parameter,
 * otherwise the call will fail.
 * \param access is the data access mode.  This can be any set of access codes ORed together (CA_READ,
 * CA_WRITE, CA_CONNECT, etc.)
 * \param timeout the timeout period in milliseconds for the call to complete.  When it is known a priori 
 * that a call takes a long time to complete than you can set this value accordingly
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \see EXECLINK(), DBLK, ExecLinkEx()
 *
 * \b Alias: T_EXECLINK()
 *
 * \b Example:
 *
 * \include eg_EXECLINKEx.txt
 */
__declspec(dllexport) short PASCAL EXECLINKEx (char *devName,char *devProperty,DBLK *dout,void *dataout,DBLK *din,void *datain,short access,long timeout);

__declspec(dllexport) short PASCAL T_EXECLINK (char *devName,char *devProperty,DBLK *dout,void *dataout,DBLK *din,void *datain,short access,long timeout);

/**
 * \brief Non-Pascal version (Win32 Only). Executes a synchronous link with timeout parameter.
 * 
 * Synchronous data exchange. Calls in the ExecLink() family do not complete until the data transfer has completed or 
 * a timeout has ensued. In the Win32 world, use EXECLINK() or T_EXECLINK() with Visual Basic.
 * Use ExecLinkEx() with interfaces which do not require the Pascal calling convention (e.g. VC++,
 * LabView, MatLab, etc.). 
 *
 * \param devName is the full device name (/\<Context\>/\<Server\>/\<Device\>) of the device to contact.
 * For example: "/HERA/BPM/WL167". Note that, if devName references a locally registered equipment module,
 * an internal call to ExecLocalLink() is made.  For instance: "/LOCAL/BPMEQM/WL167" sets the context to 
 * "LOCAL", and "BPMEQM" must be a locally registered equipment module. In such a case, the call can only 
 * be successful when the client and server are running together in the same process). 
 * \param devProperty is the device property requested, for example "ORBIT.X"
 * \param dout is a pointer to the output data set, that is, the data set to be returned from the server 
 * to the client.  The structure element dout->data should be a pointer to a pre-allocated data buffer,
 * which is to receive the returned data.  If dout is a NULL pointer, it is assumed that no data are to 
 * be returned.
 * \param din is a pointer to the input data set, that is, the data set to be sent to the server from the 
 * client.  If din is a NULL pointer, it is assumed that no data are to be sent to the server
 * \param access is the data access mode.  This can be any combination of access codes ORed together (CA_READ,
 * CA_WRITE, CA_CONNECT, etc.).
 * \param timeout the timeout period in milliseconds for the call to complete.  When it is known a priori 
 * that a call takes a long time to complete than you can set this value accordingly
 *
 * \return 0 if successful, otherwise a TINE completion code which can be interpreted by a call to 
 * GetRPCLastError().
 *
 * \note This form of the call cannot be used with Visual Basic.  ExecLinkEx() has the same prototype
 * as the console API call, namely it makes use of the DTYPE object (which is cumbersome to use in
 * Visual Basic) instead of the DBLK object + data reference used in EXECLINK().
 *
 * \see EXECLINK(), EXECLINKEx(), DTYPE
 */
__declspec(dllexport) short ExecLinkEx(char *devName,char *devProperty,DTYPE *dout,DTYPE *din,short access,long pollrate);
__declspec(dllexport) short PASCAL GetDevicePropertyInformation(char *fullDeviceName,char *property,PropertyQueryStructEx *pqsx,int *bufsiz);
__declspec(dllexport) short PASCAL FILLSTRUCT (char *tag, short addr, short size, short fmt);
__declspec(dllexport) short PASCAL SetNotificationTolerance(int i, float toleranceAbsolute, float tolerancePercent);

#endif // _RPCSRV_H_

