#ifndef _ALMSRV_H_
#define _ALMSRV_H_
#include <time.h>
#ifndef BYTE
#  define BYTE unsigned char
#endif

#define CASHOST_FILE "cashosts.csv"
#define EXPORT_FILE "exports.csv"

#define MAXALARMS 100   // put this in project.def later
#define DEFAULT_ALARM_WINDOW 3
#define DEFAULT_ALARM_HEARTBEAT 1200
#define DEFAULT_INSTANT_WINDOW 4
#define DEFAULT_TERMINATION_WINDOW 30

#define almNEWALARM     0x01
#define almHEARTBEAT    0x02
#define almOSCILLATION  0x04
#define almDATACHANGE   0x08
#define almINSTANT      0x10
#define almTERMINATE    0x40
#define almTEST         0x80

typedef struct AlarmMsgStruct
{
  char tag[16];
  char device[16];
  char alarmTag[16];
  long alarmCode;
  time_t timestamp;
  long alarmMask;
  BYTE alarmData[6];
  BYTE alarmDataFormat;
  BYTE alarmDataArraySize;
  BYTE severity;
  BYTE descriptor;
  UINT16 alarmSystem;
} AMS;                    /* AMS sent out upon request */
#define AMS_SIZE 72

typedef struct ADStag    /* Alarm Definition Structure */
{
   char alarmTag[16];
   long alarmCode;
   long alarmMask;
   short severity;
   BYTE alarmDataFormat;
   BYTE alarmDataArraySize;
   float timeRes;
   short hitsChannel;
   short hitsModule;
   char alarmText[40];
   char deviceText[38];
   UINT16 alarmSystem;
   char dataText[40];
   char url[40];
   struct ADStag *next;
} ADS;
#define ADS_SIZE 200

typedef struct AMRBtag
{
  long almStartTime;
  long almStopTime;
  long almMinSeverity;
  long reserved;
} AMRB;
#define AMRB_SIZE 16

typedef struct ALARMstruct/* local alarm list          */
{
  time_t timestamp;       /* our old friend                                */
  long  alarmCode;        /* standardized return code (implies severity)   */
  int    removalBlocked;  /* flag to keep alarm in table for CAS readout   */
  BYTE   alarmData[6];    /* alarm specific data (database interpreted)    */
  BYTE   alarmStatus;     /* corresponds to the descriptor                 */
  BYTE   clrCount;        /* record of clear requests                      */
  struct ALARMstruct *next;
} ALARM;

#define ALMARM_WATCH_FILE "almwatch.csv"
typedef struct AlarmWatchStruct
{
  char eqm[8];
  char dev[32];
  char prp[32];
  int  siz;
  int  fmt;
  int  atyp;              /* array type */
  int  asys;              /* alarm system id */
  int  sev;
  int  cnt;
  float hi;               /* high threshold should be < max */
  float lo;               /* low threshold should be > min */
  float hiwarn;           /* high warn threshold */
  float lowarn;           /* low warn threshold */
  int  interimStatus;
  int  interimCounter;
  BYTE *data;             /* allocated data buffer */
  struct AlarmWatchStruct *nxt;
} ALARM_WATCH;

extern ALARM_WATCH *almWatchList;
extern int nAlmWatchList;

typedef struct 
{
  float hi;
  float hiwarn;
  float lo;
  float lowarn;
} ALARM_WATCH_THRESHOLDS;

extern ALARM *almList;
extern short EqpFormats[];
extern int gRequireReadFromCAS;

void SurveyAlarms(void);
__declspec(dllexport) short SetAlarm(char *tag,short devNr,long almCode,BYTE *almData);
__declspec(dllexport) short SetAlarmEx(char *name,short devNr,long almCode,BYTE *almData,BYTE flags);
short getAlarms(char *tag,AMS *ams,short num,short minSeverity);
short getAlarmsEx(char *eqpname,AMS *ams,short num,short minSeverity,time_t starttime,time_t stoptime,short startnum);
short numAlarms(char *tag,time_t *almTimeStamp,short *almSeverity);
short numAlarmsEx(char *eqpname,time_t *almTimeStamp,short *almSeverity,short *NrAtAlmTimeStamp,short *NrAtAlmSeverity);
__declspec(dllexport) void ClearAlarm(char *tag,short devNr);
__declspec(dllexport) void RemoveAlarm(char *name,short devNr,long code);
short findSeverity(char *name,long cc);
void getAlarmInfo(void);
int getAlarmDefs(char *eqm,ADS *rads,int num);
int numAlarmDefs(char *eqm);
ADS *getAlarmTable(char *eqm,long code);
int getAlarmInfoTable(char *eqname);
__declspec(dllexport) short PASCAL SetMaximumAlarms(short MaxAlarms);
__declspec(dllexport) short PASCAL appendAlarmInfoTable(char *eqm,ADS *ads);
int isCASInUserList(void);

extern char *tagList[];
extern int numStoredAlarms;
extern int AlarmWindow;
extern int AlarmHeartbeat;
extern int NTAGS;
extern int NSEVERITYCODE;

#endif /* _ALMSRV_H_ */
