/** \file database.c
 * \brief TINE Server DLL documentation 
 *
 */
#include "project.def"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include <windows.h>
#include <winsock.h>
#include "database.h"
#include "winsrv.h"
#include "errors.h"
#include "almsrv.h"

#define LOINT(l)       ((short)(l))
#define HIINT(l)       ((short)(((long)(l) >> 16) & 0xFFFF))

unsigned long ElfHash(const unsigned char *key);
void output(char *txt);

static char strn[100];
char ControlDBpath[80];
char NameServerPath[80];
char FecDBpath[80];
char gFeclogPath[80];
char FecPath[128];
int usersFecnameColumnExists = FALSE;
char *strtrm(char *s)
{
  char str[516],*c=str;
  unsigned int len;
  int done0=0,done1=0;
  if (s == NULL || (len=strlen(s)) == 0 || len > 512) return s;
  if (s[len-1] == '\n') s[len-1] = 0;
  strncpy(c,s,512); c[512] = 0;
  for (len=strlen(c); len > 0 && len < 512; len=strlen(c))
  {
    if (c[len-1] == ' ' || c[len-1] == '\t' || c[len-1] == '\n') c[len-1] = 0; else done1 = TRUE;
    if (c[0] == ' ' || c[0] == '\t') c++; else done0 = TRUE;
    if (done0 && done1) break;
  }
  if (strlen(s) != len) 
  {
    if (len > 0) 
    {
      strncpy(s,c,len);
      s[len] = 0;
    }
    else s[0] = 0;
  }
  return s;
}
void HextoBytes(char *s,BYTE *b,short nibs)
{
  char buf[20];
  short l,i;

  if ((l=strlen(s)) < nibs)
  {
    strrev(s); strncat(s,"000000000000",nibs-l);
    strrev(s);
  }
  for (i=0; i<nibs/2; i++)
  {
    strncpy(buf,&s[i*2],2); buf[2] = 0;
    b[i] = (unsigned char)strtol(buf,NULL,16);
  }
}
void BytesToHex(BYTE *b,char *s,int n)
{
  int v;

  while (n--)
  {
    *s++ = (v = (*b&0xf0)>>4) < 10 ? '0' + v : 'A' + v - 10;
    *s++ = (v = *b++&0xf) < 10 ? '0' + v : 'A' + v - 10;
  }
  *s = 0;
  strrev(s);
  return;
}

short GetDataBaseSize(char *filename)
{
  char str[256];
  int line = -1;
  FILE far *fp;

  if ((fp=fopen(filename,"r")) == NULL) return -no_such_file;
  while (fgets(str,255,fp) != NULL)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#",str[0])) continue;      /* comment line(s) */
    line++;
  }
  fclose(fp);
  return line;
}
char *fmtStr[] = { "double","short","byte","long","char","float","pocalblk","struct","char8",
                   "char16","char16fi","bit","hitts","char32","fltint","intint","lngint",
				   "dblint","shtint","intfltint","char48","char8i","char16i","char32i",
				   "char48i","ustring","xys","fltfltint","filter","intfltfltflt","spectrum" };
int nFmtStr = sizeof(fmtStr)/sizeof(char);

short MoveAndReformat(char *dst, short dstFmt, char *src, short srcFmt, unsigned long num)
{
  unsigned long i;
  unsigned long sizd,sizs;
  char s[80],*fss,*fsd;

  if (num == 0) return 0; // nothing to do ...
  sizd = getFormatSize(dstFmt);
  sizs = getFormatSize(srcFmt);
  if (debug > 1) 
  {
    fss =  BFMT(srcFmt) < 28 ? fmtStr[BFMT(srcFmt)] : "null";
    fsd =  BFMT(dstFmt) < 28 ? fmtStr[BFMT(dstFmt)] : "null";
    sprintf(s,"Move %d  %d byte - %s  to %d byte - %s\n",num,sizs,fss,sizd,fsd);
    output(s);
  }
  if (sizd <= 0 || sizs <= 0) return illegal_format;

  if (dstFmt == srcFmt)
  {
    memcpy(dst,src,num*sizd);
    return 0;
  }

  switch (dstFmt)
  {
    case CF_TEXT:
    case CF_POCALBLK:
    case CF_STRUCT:
      if (srcFmt != dstFmt) return illegal_format;
      memcpy(dst,src,num*sizd);
      return 0;
    case CF_BYTE:
      switch (srcFmt)
      {
        case CF_INTEGER:
          for (i=0; i<num; i++) dst[i] = (char)((short *)src)[i];
          return 0;
        case CF_LONG:
          for (i=0; i<num; i++) dst[i] = (char)((long *)src)[i];
          return 0;
        case CF_FLOAT:
          for (i=0; i<num; i++) dst[i] = (char)((float *)src)[i];
          return 0;
        case CF_DOUBLE:
          for (i=0; i<num; i++) dst[i] = (char)((double *)src)[i];
          return 0;
        default: return illegal_format;
      }
    case CF_INTEGER:
      switch (srcFmt)
      {
        case CF_BYTE:
          for (i=0; i<num; i++) ((short *)dst)[i] = src[i];
          return 0;
        case CF_LONG:
          for (i=0; i<num; i++) ((short *)dst)[i] = (short)((long *)src)[i];
          return 0;
        case CF_FLOAT:
          for (i=0; i<num; i++) ((short *)dst)[i] = (short)((float *)src)[i];
          return 0;
        case CF_DOUBLE:
          for (i=0; i<num; i++) ((short *)dst)[i] = (short)((double *)src)[i];
          return 0;
        default: return illegal_format;
      }
    case CF_LONG:
      switch (srcFmt)
      {
        case CF_BYTE:
          for (i=0; i<num; i++) ((long *)dst)[i] = src[i];
          return 0;
        case CF_INTEGER:
          for (i=0; i<num; i++) ((long *)dst)[i] = ((short *)src)[i];
          return 0;
        case CF_FLOAT:
          for (i=0; i<num; i++) ((long *)dst)[i] = (long)((float *)src)[i];
          return 0;
        case CF_DOUBLE:
          for (i=0; i<num; i++) ((long *)dst)[i] = (long)((double *)src)[i];
          return 0;
        case CF_FLTINT:
          for (i=0; i<num; i++) ((long *)dst)[i] = ((FLTINT *)src)[i].ival;
          return 0;
        case CF_NAME16I:
          for (i=0; i<num; i++) ((long *)dst)[i] = ((NAME16I *)src)[i].ival;
          return 0;
        case CF_NAME32I:
          for (i=0; i<num; i++) ((long *)dst)[i] = ((NAME32I *)src)[i].ival;
          return 0;
        default: return illegal_format;
      }
    case CF_FLOAT:
      switch (srcFmt)
      {
        case CF_BYTE:
          for (i=0; i<num; i++) ((float *)dst)[i] = src[i];
          return 0;
        case CF_INTEGER:
          for (i=0; i<num; i++) ((float *)dst)[i] = ((short *)src)[i];
          return 0;
        case CF_LONG:
          for (i=0; i<num; i++) ((float *)dst)[i] = (float)((long *)src)[i];
          return 0;
        case CF_DOUBLE:
          for (i=0; i<num; i++) ((float *)dst)[i] = (float)((double *)src)[i];
          return 0;
        case CF_SPECTRUM:
          for (i=0; i<num-24; i++) ((float *)dst)[i] = ((float *)src)[i+24];
          return 0;
        case CF_FLTINT:
          for (i=0; i<num; i++) ((float *)dst)[i] = ((FLTINT *)src)[i].fval;
          return 0;
        case CF_TDS:
          for (i=0; i<num; i++) ((float *)dst)[i] = ((INTFLTINT *)src)[i].fval;
          return 0;
        default: return illegal_format;
      }
    case CF_DOUBLE:
      switch (srcFmt)
      {
        case CF_BYTE:
          for (i=0; i<num; i++) ((double *)dst)[i] = src[i];
          return 0;
        case CF_INTEGER:
          for (i=0; i<num; i++) ((double *)dst)[i] = ((short *)src)[i];
          return 0;
        case CF_LONG:
          for (i=0; i<num; i++) ((double *)dst)[i] = ((long *)src)[i];
          return 0;
        case CF_FLOAT:
          for (i=0; i<num; i++) ((double *)dst)[i] = ((float *)src)[i];
          return 0;
        default: return illegal_format;
      }
    case CF_NAME16:
      switch (srcFmt)
      {
        case CF_NAME8:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME8 *)src)[i].name,8);
          return 0;
        case CF_NAME32:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME32 *)src)[i].name,16);
          return 0;
        case CF_NAME48:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME48 *)src)[i].name,16);
          return 0;
        case CF_NAME64:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME64 *)src)[i].name,16);
          return 0;
        case CF_NAME32I:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME32I *)src)[i].name,16);
          return 0;
        case CF_NAME16I:
          for (i=0; i<num; i++) strncpy(((NAME16 *)dst)[i].name,((NAME16I *)src)[i].name,16);
          return 0;
        default: return illegal_format;
      }
    case CF_NAME32:
      switch (srcFmt)
      {
        case CF_NAME8:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME8 *)src)[i].name,8);
          return 0;
        case CF_NAME16:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME16 *)src)[i].name,16);
          return 0;
        case CF_NAME48:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME48 *)src)[i].name,32);
          return 0;
        case CF_NAME64:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME64 *)src)[i].name,32);
          return 0;
        case CF_NAME32I:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME32I *)src)[i].name,32);
          return 0;
        case CF_NAME16I:
          for (i=0; i<num; i++) strncpy(((NAME32 *)dst)[i].name,((NAME16I *)src)[i].name,16);
          return 0;
        default: return illegal_format;
      }
    case CF_NAME48:
      switch (srcFmt)
      {
        case CF_NAME8:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME8 *)src)[i].name,8);
          return 0;
        case CF_NAME16:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME16 *)src)[i].name,16);
          return 0;
        case CF_NAME32:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME32 *)src)[i].name,32);
          return 0;
        case CF_NAME64:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME64 *)src)[i].name,48);
          return 0;
        case CF_NAME32I:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME32I *)src)[i].name,32);
          return 0;
        case CF_NAME16I:
          for (i=0; i<num; i++) strncpy(((NAME48 *)dst)[i].name,((NAME16I *)src)[i].name,16);
          return 0;
        default: return illegal_format;
      }
    case CF_NAME64:
      switch (srcFmt)
      {
        case CF_NAME8:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME8 *)src)[i].name,8);
          return 0;
        case CF_NAME16:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME16 *)src)[i].name,16);
          return 0;
        case CF_NAME32:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME32 *)src)[i].name,32);
          return 0;
        case CF_NAME48:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME48 *)src)[i].name,48);
          return 0;
        case CF_NAME32I:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME32I *)src)[i].name,32);
          return 0;
        case CF_NAME16I:
          for (i=0; i<num; i++) strncpy(((NAME64 *)dst)[i].name,((NAME16I *)src)[i].name,16);
          return 0;
        default: return illegal_format;
      }
  }
  return illegal_format;
}

short reformat(char *data,short num,short dstFmt,short srcFmt)
{
  short i,sizd,sizs;

  if (dstFmt == srcFmt) return  0;

  sizd = getFormatSize(dstFmt);
  sizs = getFormatSize(srcFmt);

  if (sizd < 0 || sizs < 0) return -1;
  if (sizd != 8 && sizs != 8) return -1;  // one must be double

  if (sizd < sizs)
  {
    switch (dstFmt)   // assume src is double
    {
      case CF_INTEGER:
        for (i=0; i<num; i++) ((short *)data)[i] = (short)((double *)data)[i];
        return 0;
      case CF_LONG:
        for (i=0; i<num; i++) ((long *)data)[i] = (long)((double *)data)[i];
        return 0;
      case CF_FLOAT:
        for (i=0; i<num; i++) ((float *)data)[i] = (float)((double *)data)[i];
        return 0;
      case CF_BYTE:
        for (i=0; i<num; i++) ((char *)data)[i] = (char)((double *)data)[i];
        return 0;
      case CF_TEXT:
      case CF_POCALBLK:
      case CF_STRUCT:
        return 0;
      default:
        return -1;
    }
  }
  else
  {
    switch (srcFmt)                           // assume src is double
    {
      case CF_INTEGER:
        for (i=num-1; i>=0; i--) ((double *)data)[i] = ((short *)data)[i];
        return 0;
      case CF_LONG:
        for (i=num-1; i>=0; i--) ((double *)data)[i] = ((long *)data)[i];
        return 0;
      case CF_FLOAT:
        for (i=num-1; i>=0; i--) ((double *)data)[i] = ((float *)data)[i];
        return 0;
      case CF_BYTE:
        for (i=num-1; i>=0; i--) ((double *)data)[i] = ((char *)data)[i];
         return 0;
      case CF_TEXT:
      case CF_POCALBLK:
      case CF_STRUCT:
        return 0;
      default:
        return -1;
    }
  }
}

short findcol(char *str,char *tag)
{
  char *c;
  int i,len;

  for (i=0,c=str; strlen(c); i++,c++)
  {
    while (*c == '\"') { c = strchr(++c,'\"'); if (c++ == NULL) return -1; }
    while (*c == ' ') c++;
    strupr(c);
    len = strcspn(c,",\n"); while (c[len-1] == ' ') len--;
    if (!strncmp(c,tag,(unsigned int)MAX(len,(int)strlen(tag)))) return i;
    if ((c=strpbrk(c,",\n")) == NULL) return -1;
  }
  return -1;
}
char *colptr(short col,char *str)
{
  static char buf[256];
  char *c;
  int i,sslen;

  for (i=0,c=str; i < col && strlen(c); i++,c++)
  {
    while (*c == '\"')
    {
      c = strchr(++c,'\"');
      if (c++ == NULL)
      {
        sprintf(buf,"%s : missing quote in column %d\n",str,col);
        feclog(buf);
        return NULL;
      }
    }
    if ((c=strpbrk(c,",")) == NULL)
    {
      if (strlen(str) > 128) str[128] = 0;
      sprintf(buf,"%s : missing column %d\n",str,col);
      feclog(buf);
      return NULL;
    }
  }
  while (*c == ' ') c++;
  if (*c == '\"') sslen = strcspn(++c,"\"");
  else sslen = strcspn(c,",\n");
  if (sslen > 126)
  {
    sprintf(buf,"sub string \"%s\" too long (%d vs 126)\n",c,sslen);
    feclog(buf); sslen = 126;
  }
  while (sslen > 0 && c[sslen-1] == ' ') sslen--;
  strncpy(buf,c,i=MIN(sizeof(buf)-1,sslen));
  buf[i] = 0;
  return buf;
}

int getUserList(UserListStruct **ul,char *userfile,int *nr)
{
  FILE *fp = NULL;
  int i,cc=0,done=0;
  int user_col,fec_col;
  UserListStruct *usrlst = NULL;
  static char str[256],hdr[256],scratch[16];

  if (nr == NULL) ccerr(code_failure);
  if (*ul != NULL) free(*ul);
  if ((fp=fopen(userfile,"r")) == NULL) ccerr(no_such_file);

  for (i=0; fgets(str,255,fp) != NULL;)
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#",str[0])) continue;      /* comment line(s) */
    i++;
  }
  *nr = i - 1; // subtract header
  if ((usrlst=(UserListStruct *)calloc(*nr,sizeof(UserListStruct))) == NULL) ccerr(out_of_server_memory);
  rewind(fp);
  for (i=0; fgets(str,255,fp) != NULL && i < *nr; )
  {
    strtrm(str);
    if (strlen(str) == 0) continue;
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((user_col=findcol(hdr,"USERNAME")) < 0) 
      {
        sprintf(str,"%s : column USERNAME not found\n",userfile);
        feclog(str);
        ccerr(no_such_line);
      }
      if ((fec_col=findcol(hdr,"FECNAME")) >=0) usersFecnameColumnExists = TRUE;
      done = 1;                              /* read header info */
      continue;
    }
    if (fec_col >= 0)
    {
      strncpy(scratch,colptr((short)fec_col,str),16);
      if (strlen(scratch) > 0 && strncmp(scratch,gFecName,16)) continue;
    }
    strncpy(usrlst[i++].UserName,colptr((short)user_col,str),16);
  }
  *nr = i;  // readjust
  if (*nr == 0) ccerr(not_allowed);
err:
  if (fp != NULL) fclose(fp);
  if (cc == 0) 
  {
    *ul = usrlst;
  }
  return cc;
}
int removeRegisteredUser(char *user)
{
  FILE *fp = NULL, *tmpfp = NULL;
  int i,len,cc=0;
  char *c,str[256],userfilename[80],tmpfilename[80];

  if (user == NULL || strlen(user) == 0) ccerr(argument_list_error);
  for (i=0; i<NrOfUsers;i++)
    if (!strnicmp(UserList[i].UserName,user,16)) break;
  if (i == NrOfUsers) ccerr(0);

  sprintf(userfilename,"%s%s",FecDBpath,USERS_FILE);
  strncpy(tmpfilename,userfilename,80);
  if ((c=strstr(tmpfilename,".csv")) == NULL) ccerr(file_error);
  *c = 0; strcat(tmpfilename,".bak");
  unlink(tmpfilename);
  if (rename(userfilename,tmpfilename)) ccerr(file_error);

  if ((tmpfp=fopen(tmpfilename,"r")) == NULL) ccerr(no_such_file);
  if ((fp=fopen(userfilename,"w")) == NULL) ccerr(no_such_file);

  while (fgets(str,255,tmpfp) != NULL)
  {
    strtrm(str);
    if ((len=strlen(str)-1) <= 0) continue;
    if (!strnicmp(str,user,len)) continue;
    fwrite(str,strlen(str),1,fp);
  }
  fclose(fp); fp = NULL;
  fclose(tmpfp); tmpfp = NULL;
  getUserList(&UserList,userfilename,&NrOfUsers);
err:
  if (fp != NULL) fclose(fp);
  if (tmpfp != NULL) fclose(tmpfp);
  return cc;
}
int appendRegisteredUserList(char *user)
{
  FILE *fp = NULL;
  int cc=0;
  char scratch[64];
  if (user == NULL || strlen(user) == 0) ccerr(argument_list_error);
  FecPath[0] = 0;
  strcat(FecPath,FecDBpath); strcat(FecPath,"users.csv");
  if ((fp=fopen(FecPath,"a")) == NULL) ccerr(no_such_file);
  fseek(fp,0,SEEK_END);
  if (ftell(fp) == 0)
  {
    fprintf(fp,"USERNAME\n");  /* file did not exist before */
  }
  strncpy(scratch,user,16); scratch[16] = 0;
  if (usersFecnameColumnExists)
  {
    strcat(scratch,",");
    strcat(scratch,gFecName);
  }
  fprintf(fp,"%s\n",scratch);
  fclose(fp);
  getUserList(&UserList,FecPath,&NrOfUsers);
err:
  return cc;
}

void getDatabasePaths(void)
{
  char *ptr;
  int len = 0;
  FecDBpath[0] = 0; FecPath[0] = 0;
  if ((ptr=getenv("FECDB")) != NULL && (len=strlen(ptr)) > 0 && len < 79) 
  {
    strncpy(FecDBpath,ptr,len);
    if (FecDBpath[len-1] != '\\') strcat(FecDBpath,"\\");
  }
  memset(gFeclogPath,0,80);
  if ((ptr=getenv("FECLOG")) != NULL) 
  {
    strncpy(gFeclogPath,ptr,64);
    len = strlen(gFeclogPath);
    if (gFeclogPath[len-1] != '\\') strcat(gFeclogPath,"\\");
  }
  else
  {
    strncpy(gFeclogPath,FecDBpath,64);
  }
  ControlDBpath[0] = 0;
  if ((ptr=getenv("TINE.HOME")) == NULL) ptr = getenv("CONTROLDB");
  if (ptr != NULL && (len=strlen(ptr)) > 0 && len < 79) 
  {
    strncpy(ControlDBpath,ptr,len);
    if (ControlDBpath[len-1] != '\\') strcat(ControlDBpath,"\\");
  }
}

RPCFecStruct *RPCFec = NULL,NameServer;

short GetNameServerAddress(void)
{
  FILE *fp = NULL;
  DWORD haddr;
  static short gotNameServerAddress = 0;
  short cc=0,done=0,col;
  char str[256],hdr[256],scratch[16];

  if (gotNameServerAddress) return 0;

  NameServerPath[0] = 0;
  strcat(NameServerPath,ControlDBpath); strcat(NameServerPath,CSHOSTS_FILE);

  if ((fp=fopen(NameServerPath,"r")) == NULL) 
  {
    NameServerPath[0] = 0;  // take this out in '99
    strcat(NameServerPath,ControlDBpath); strcat(NameServerPath,"herans.csv");
	if ((fp=fopen(NameServerPath,"r")) == NULL) 
	  ccerr(no_such_file);
  }
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strcpy(hdr,str);
      if ((col=findcol(hdr,"FECNAME")) < 0) ccerr(no_such_line);
      done = 1;
      continue;
    }
    strncpy(NameServer.Name,colptr(col,str),FEC_NAME_SIZE);
    if ((col=findcol(hdr,"FECNETWORK")) < 0) ccerr(no_such_line);
    strncpy(scratch,colptr(col,str),16);
    HextoBytes(scratch,NameServer.Network,8);
    if ((col=findcol(hdr,"FECNODE")) < 0) ccerr(no_such_line);
    strncpy(scratch,colptr(col,str),16);
    HextoBytes(scratch,NameServer.Node,12);
    if ((col=findcol(hdr,"IPADDR")) < 0) ccerr(no_such_line);
    strncpy(NameServer.IP,colptr(col,str),16);
    haddr = inet_addr(NameServer.IP);
    memcpy(NameServer.IPh_addr,&haddr,4);
    if ((col=findcol(hdr,"PORTOFFSET")) >= 0)
    {
      strncpy(scratch,colptr(col,str),16);
      NameServer.PortOffset = atoi(scratch);
    }
    else NameServer.PortOffset = 0;
	break;  /* a server uses only primary name server for PnP */
  }
  fclose(fp); fp = NULL;
err: if (fp) fclose(fp);
  if (cc == 0) gotNameServerAddress = 1;
  return cc;
}

char gFecName[20] =  "*unknown*";
IPXAddress IPXFecAddress;
short ExportsRegistered = 0;
RPCFecExStruct RPCFecEx;
char gDeviceContext[17] = "";

__declspec(dllexport) short PASCAL RegisterFecName(char *name,char *desc,char *loc,char *hdw,char *resp,short port)
{
  return RegisterFecNameEx(name,desc,loc,hdw,resp,port,NULL);
}
__declspec(dllexport) short PASCAL RegisterFecNameEx(char *name,char *desc,char *loc,char *hdw,char *resp,short port,char *context)
{
  char *c,sub[16] = "";
  if ((c=strchr(desc,'[')) != NULL) 
  {
    strncpy(sub,++c,16);
    if ((c=strchr(sub,']')) != NULL) *c = 0;
  }
  return RegisterFecInformation(name,sub,context,desc,loc,hdw,resp,port);
}

__declspec(dllexport) short PASCAL RegisterFecInformation(char *name,char *sub,char *cntxt,char *dsc,char *loc,char *hdw,char *resp,UINT16 poff)
{
  char s[80];
  if (FecNameResolved) return not_accepted;
  strncpy(gFecName,name,FEC_NAME_SIZE);
  strncpy(RPCFecEx.OS,"WIN32",12);
  strncpy(RPCFecEx.desc,dsc,64);
  strncpy(RPCFecEx.loc,loc,32);
  strncpy(RPCFecEx.hdw,hdw,32);
  strncpy(RPCFecEx.resp,resp,32);
  PortOffset = poff;
  if (cntxt != NULL) 
  {
    strncpy(gDeviceContext,cntxt,EQP_CONTEXT_SIZE);
    strupr(gDeviceContext);
  }
  if (sub != NULL) 
  {
    strncpy(RPCFecEx.sub,sub,16);
    strupr(RPCFecEx.sub);
  }
  sprintf(s,"FEC name from API : %s (Port offset %d)",gFecName,PortOffset);
  dbllog(s);
  FecNameResolved = 1;
  return 0;
}


FILE *devFileOpen(char *eqm)
{
  char fname[20],devFile[80];
  int i;
  FILE *fp=NULL;
  for (i=0; i<2; i++)
  {
    memset(fname,0,sizeof(fname));
    strncpy(fname,eqm,6);
    strcat(strlwr(fname),i == 0 ? "-devices.csv" : "dv.csv");
    sprintf(devFile,"%s%s",FecDBpath,fname);
    if ((fp=fopen(devFile,"r")) != NULL) return fp;
  }
  return NULL;
}
NAME16 *GetEqpModuleNames(char *eqname,int num)
{
  NAME16 *eqpModuleName = NULL;
  int i,cc=0,done=0,name_col=-1,num_col=-1;
  char str[256],hdr[256],scratch[33];
  FILE *fp = NULL;

  if (num <= 0) ccerr(out_of_range);
  if ((eqpModuleName=(NAME16 *)calloc(num,sizeof(NAME16))) == NULL) ccerr(out_of_server_memory);
  if ((fp=devFileOpen(eqname)) == NULL) ccerr(no_such_file);

  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((name_col=findcol(hdr,"MODULE_NAME")) < 0 &&
          (name_col=findcol(hdr,"DEVICE_NAME")) < 0) ccerr(no_such_column);
      if ((num_col=findcol(hdr,"MODULE_NUMBER")) < 0 &&
          (num_col=findcol(hdr,"DEVICE_NUMBER")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(scratch,colptr((short)num_col,str),16);
    scratch[16] = 0;
    if ((i=atoi(scratch)) < num)
      strncpy(eqpModuleName[i].name,colptr((short)name_col,str),16);
  }

err:
  if (fp) fclose(fp);
  sprintf(str,"device file for %s : %s",eqname, cc ? erlst[cc] : "found");
  feclog(str);
  return eqpModuleName;
}
void makeEqpModuleHashTable(ExportListStruct *el)
{
  HashXRefStruct *xref;
  char lclname[33];
  int cc=0,idx,i;

  if (el == NULL || el->eqpModuleXRef == NULL) return;
  lclname[32] = 0;
  for (i=0; i<el->EqpNumModules; i++)
  {
    strncpy(lclname,el->EqpModuleName[i].name,32);
    idx = (int)(ElfHash((unsigned char *)lclname) % MODULE_HASH_SIZE);
    for (xref=el->eqpModuleXRef[idx]; xref && xref->index != i; xref=xref->next);
    if (xref == NULL) /* it's a new one*/
    {
      if ((xref=(HashXRefStruct *)calloc(1,sizeof(HashXRefStruct))) == NULL) ccerr(out_of_server_memory);
      xref->index = i;
      xref->next = el->eqpModuleXRef[idx];
      el->eqpModuleXRef[idx] = xref;
    }
  }
err:
  return;
}
FILE *expFileOpen(char *fecnm,char *expfname)
{
  char fname[32],expFile[80];
  int i;
  FILE *fp=NULL;
  for (i=0; i<2; i++)
  {
    if (fecnm == NULL || strlen(fecnm) == 0) continue;
    memset(fname,0,sizeof(fname));
    if (i > 0) strcpy(fname,"exports.csv");
    else
    {
      strncpy(fname,fecnm,16);
      strcat(fname,"-exports.csv");
    }
    sprintf(expFile,"%s%s",FecDBpath,fname);
    if (expfname != NULL) strncpy(expfname,expFile,80);
    if ((fp=fopen(expFile,"r")) != NULL) return fp;
  }
  return NULL;
}
char *getLocalName(char *exportName)
{
  ExportListStruct *el;
  if (exportName == NULL || strlen(exportName) == 0) return NULL;
  for (el=ExportList; el != NULL; el=el->next) 
    if (!strncmp(exportName,el->tagName,EQP_DEVICE_SIZE)) break;
  return el == NULL ? NULL : el->EqpName;
}
int getExportNameFromFile(char *exportName,char *localName)
{
  int cc=0,done=0;
  int export_name_col=-1,local_name_col=-1;
  FILE *fp;
  char str[256],hdr[256],expfilename[80];

  if (localName == NULL || strlen(localName) == 0) ccerr(argument_list_error);
  if (!FecNameResolved) getFecName(exportName,localName);
  if ((fp=expFileOpen(gFecName,expfilename)) == NULL) ccerr(no_such_file);
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;    /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((export_name_col=findcol(hdr,"EXPORT_NAME")) < 0) ccerr(no_such_column);
      if ((local_name_col=findcol(hdr,"LOCAL_NAME")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    if (strncmp(localName,colptr((short)local_name_col,str),EQP_NAME_SIZE)) continue;
    strncpy(exportName,colptr((short)export_name_col,str),EXPORT_NAME_SIZE);
    sprintf(str,"matched export name %s to %s",exportName,localName);
    feclog(str);
    ccerr(0);
  }
err:
  if (fp) fclose(fp);
  if (cc)
  {
    sprintf(str,"getExportNameFromFile: %s",erlst[cc]);
    feclog(str);
  }
  return cc;
}
int getRegisteredExports(char *fecnm,char *lclname)
{
  FILE *fp;
  int i,idx,cc=0,done=0;
  int export_name_col=-1,local_name_col=-1,property_col=-1,property_size_col=-1,
      num_modules_col=-1,format_col=-1,access_col=-1,description_col=-1,
      property_id_col=-1,property_insize_col=-1,informat_col=-1,redir_col=-1;
  char str[256],hdr[256],scratch[64],expfilename[80], *c;
  int prpId = -1;
  ExportPropertyListStruct *prp;
  ExportListStruct *el;
  PrpRedirBlk *prd = NULL;

  if ((fp=expFileOpen(fecnm,expfilename)) == NULL) ccerr(no_such_file);
  for (done=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;    /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((export_name_col=findcol(hdr,"EXPORT_NAME")) < 0) ccerr(no_such_column);
      if ((local_name_col=findcol(hdr,"LOCAL_NAME")) < 0) ccerr(no_such_column);
      if ((num_modules_col=findcol(hdr,"NUM_MODULES")) < 0) ccerr(no_such_column);
      if ((property_col=findcol(hdr,"PROPERTY")) < 0) ccerr(no_such_column);
      if ((property_size_col=findcol(hdr,"PROPERTY_SIZE")) < 0) ccerr(no_such_column);
      property_insize_col = findcol(hdr,"PROPERTY_INSIZE");
      property_id_col = findcol(hdr,"PROPERTY_ID");
      if ((format_col=findcol(hdr,"FORMAT")) < 0) ccerr(no_such_column);
      informat_col = findcol(hdr,"INFORMAT");
      if ((access_col=findcol(hdr,"ACCESS")) < 0) ccerr(no_such_column);
      if ((description_col=findcol(hdr,"DESCRIPTION")) < 0) ccerr(no_such_column);
      redir_col = findcol(hdr,"REDIRECTION");
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(scratch,colptr((short)export_name_col,str),EXPORT_NAME_SIZE);
    for (el=ExportList; el != NULL; el=el->next) if (!strncmp(scratch,el->tagName,EQP_DEVICE_SIZE)) break;
    if (el == NULL)               /* new eqp type */
    {
      el = (ExportListStruct *)calloc(1,sizeof(ExportListStruct));
      if (el == NULL) ccerr(out_of_server_memory);
      strncpy(el->tagName,scratch,EXPORT_NAME_SIZE);
      strncpy(el->EqpName,colptr((short)local_name_col,str),EQP_NAME_SIZE);
      strncpy(el->Context,gDeviceContext,EQP_CONTEXT_SIZE);
      strncpy(scratch,colptr((short)num_modules_col,str),16);
      el->EqpNumModules = atoi(scratch);
      if (el->EqpNumModules == 0) el->EqpNumModules = 1;
      el->EqpModuleName = GetEqpModuleNames(el->EqpName,el->EqpNumModules);
      makeEqpModuleHashTable(el);
      el->EqpNumProperties = 0;
      if ((el->almLst=(ALARM **)calloc(el->EqpNumModules,sizeof(ALARM *))) == NULL)
        ccerr(out_of_server_memory);
      el->exportId = NrOfEqpTypes++;
      el->next = ExportList;
      ExportList = el;
    }
    if ((prp=(ExportPropertyListStruct *)calloc(1,sizeof(ExportPropertyListStruct))) == NULL) ccerr(out_of_server_memory);
    strncpy(prp->prpName,colptr((short)property_col,str),EQP_PROP_SIZE);
    if ((i=getAliasTableIndexFromName(prp->prpName)) >= 0)
      strncpy(prp->prpAlias,gAliasTable[i].alias,EQP_PROP_SIZE);
    strncpy(scratch,colptr((short)property_size_col,str),16);
    prp->prpSizeOut = atoi(scratch);
    strncpy(scratch,colptr((short)format_col,str),16);
    if (!strncmp(scratch,"STRUCT.",7)) /* tag coming ... */
    {
      if ((c=strstr(scratch,".")) == NULL) ccerr(code_failure);
      *c++ = 0;
      strncpy(prp->strTagOut,c,8);
    }
    prp->prpFormatOut = ftoi(scratch);
    prp->prpArrayType = getArrayType(scratch);
    if (!(prp->prpArrayType & AT_DOUBLE)) prp->prpArrayType |= prp->prpSizeOut > 1 ? AT_SINGLE : AT_SCALAR;
    prp->rowSize = getArrayRowLength(scratch);
    if (prp->rowSize > 0) prp->numRows = (UINT16)(prp->prpSizeOut/prp->rowSize); else prp->rowSize = (short)prp->prpSizeOut;
    strncpy(scratch,colptr((short)access_col,str),16);
    prp->prpAccessMode = mtoi(scratch);
    if (property_insize_col >= 0)
    {
      strncpy(scratch,colptr((short)property_insize_col,str),16);
      prp->prpSizeIn = atoi(scratch);
    }
    else
    {
      if (prp->prpAccessMode & CA_WRITE) prp->prpSizeIn = prp->prpSizeOut;
    }
    prp->prpFormatIn = CF_NULL;
    if (informat_col >= 0)
    {
      strncpy(scratch,colptr((short)informat_col,str),16);
      if (!strncmp(scratch,"STRUCT.",7)) /* tag coming ... */
      {
        if ((c=strstr(scratch,".")) == NULL) ccerr(code_failure);
        *c++ = 0;
        strncpy(prp->strTagIn,c,8);
      }
      prp->prpFormatIn = ftoi(scratch);
    }
    else
    {
      if (prp->prpAccessMode & CA_WRITE) prp->prpFormatIn = prp->prpFormatOut;
    }
    ParsePropertyDescription(colptr((short)description_col,str),prp);
    if (redir_col >= 0)
    {
      strncpy(scratch,colptr((short)redir_col,str),64);
      if (strlen(scratch) > 0)
      {
        if ((prd=(PrpRedirBlk *)calloc(1,sizeof(PrpRedirBlk))) == NULL) ccerr(out_of_server_memory);
        prd->next = prp->prd;
        prp->prd = prd;
        ParseRedirector(scratch,prp);
      }
    }
    if (strchr(prp->prpName,'*')) idx = PROPERTY_HASH_SIZE;
    else idx = (int)(ElfHash((unsigned char *)prp->prpName) % PROPERTY_HASH_SIZE);
    prp->prpId = MAKE_PROPERTY_ID(el->EqpNumProperties,idx);
    prp->next = el->EqpProperty[idx];
    el->EqpProperty[idx] = prp;
    if ((i=strlen(prp->prpName)) > 4 && !strcmp(&prp->prpName[i-4],".NAM")) 
    {
      el->hasPropertyDeviceNames = TRUE;
      acquireDeviceNamesFromProperties = TRUE;
    }
    el->EqpNumProperties++;
  }

  // getAlarmInfo(); <- do this piecemeal in RegisterExport
  // this has assigned names according to the file but has not bound anything to a control!

err:
  if (fp) fclose(fp);
  sprintf(str,"exports file %s : %s\n",expfilename,cc ? erlst[cc] : "found");
  feclog(str);
  return cc;
}

short getFecName(char *expName,char *lclName)
{
  FILE *fp = NULL;
  short cc=0,done=0;
  short name_col,sub_col,desc_col,loc_col,hdw_col,resp_col,port_col,exp_col,con_col,lcl_col;
  char *c,str[256],hdr[256],s[80];

  FecPath[0] = 0;
  strcat(FecPath,FecDBpath); strcat(FecPath,"fecid.csv");

  if ((fp=fopen(FecPath,"r")) == NULL) ccerr(no_such_file);
  PortOffset = 0;
  while (fgets(str,255,fp) != NULL)
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strcpy(hdr,str);
      if ((name_col=findcol(hdr,"FEC_NAME")) < 0) ccerr(no_such_line);
      if ((desc_col=findcol(hdr,"DESCRIPTION")) < 0) ccerr(no_such_line);
      if ((loc_col=findcol(hdr,"LOCATION")) < 0) ccerr(no_such_line);
      if ((hdw_col=findcol(hdr,"HARDWARE")) < 0) ccerr(no_such_line);
      if ((resp_col=findcol(hdr,"RESPONSIBLE")) < 0) ccerr(no_such_line);
      port_col = findcol(hdr,"PORT_OFFSET");
      exp_col = findcol(hdr,"EXPORT_NAME");
      lcl_col = findcol(hdr,"LOCAL_NAME");
      con_col = findcol(hdr,"CONTEXT");
      sub_col = findcol(hdr,"SUBSYSTEM");
      done = 1;                              /* read header info */
      continue;
    }
    if (exp_col >= 0 && expName != NULL && _strnicmp(expName,colptr(exp_col,str),16)) continue;  /* not the target */
    if (lcl_col >= 0 && lclName != NULL && _strnicmp(lclName,colptr(lcl_col,str),6)) continue;  /* not the target */
    strncpy(gFecName,colptr(name_col,str),16);
    strncpy(RPCFecEx.OS,"WIN32",12);
    strncpy(RPCFecEx.desc,colptr(desc_col,str),64);
    strncpy(RPCFecEx.loc,colptr(loc_col,str),32);
    strncpy(RPCFecEx.hdw,colptr(hdw_col,str),32);
    strncpy(RPCFecEx.resp,colptr(resp_col,str),32);
    if (con_col >= 0)
    {
      strncpy(gDeviceContext,colptr(con_col,str),16);
      strupr(gDeviceContext);
    }
    if (sub_col >= 0)
    {
      strncpy(RPCFecEx.sub,colptr(sub_col,str),16);
      strupr(RPCFecEx.sub);
    }
    else if ((c=strchr(RPCFecEx.desc,'[')) != 0)
    {
      strncpy(s,++c,20);
      if ((c=strchr(s,']')) != 0) *c=0;
      strncpy(RPCFecEx.sub,c,16);
      strupr(RPCFecEx.sub);
    }
    if (port_col >= 0)
    {
      strncpy(s,colptr(port_col,str),16);
      PortOffset = atoi(s);
    }
    FecNameResolved = 1;
    break;
  }
  if (!FecNameResolved) ccerr(not_exported);
  sprintf(s,"FEC name from file : %s (Port offset %d)",gFecName,PortOffset);
  dbllog(s);
err:
  if (fp) fclose(fp);
  if (cc)
  {
    sprintf(str,"GetFECName: %s",erlst[cc]);
    dbllog(str);
  }
  return cc;
}
short ftoi(char *fmt)
{
  char *c,fmtstr[64];
  short ifmt;
  strncpy(fmtstr,fmt,64);
  if ((c=strchr(fmtstr,'.')) != NULL) *c = 0;
  strupr(fmtstr);
  if ((ifmt=getSimpleFormatType(fmtstr)) != CF_NULL) return ifmt;
  return getCompoundFormatType(fmtstr);
}
short getArrayType(char *fmt)
{
  char *c,typstr[64];
  short itype = AT_UNKNOWN;
  if ((c=strchr(fmt,'.')) == NULL) return itype;
  c++; strncpy(typstr,c,64); strupr(typstr);
  if (strstr(typstr,"DOUBLE") != NULL) itype = AT_DOUBLE;
  if (strstr(typstr,"CHANNEL") != NULL) itype |= AT_CHANNEL;
  else if (strstr(typstr,"SPECTRUM") != NULL) itype |= AT_SPECTRUM;
  else if (strstr(typstr,"TRACE") != NULL) itype |= AT_SPECTRUM;
  else if (strstr(typstr,"WAVEFORM") != NULL) itype |= AT_SPECTRUM;
  else if (strstr(typstr,"UNSTRUCTURED") != NULL) itype |= AT_UNSTRUCTURED;
  else if (strstr(typstr,"BLOB") != NULL) itype |= AT_UNSTRUCTURED;
  else if (strstr(typstr,"IMAGE") != NULL) itype |= AT_UNSTRUCTURED;
  else if (strstr(typstr,"STRUCTURED") != NULL) itype |= AT_STRUCTURED;
  return itype;
}
short getArrayRowLength(char *fmt)
{
  char *c,typstr[64];
  if ((c=strstr(fmt,"DOUBLE.")) == NULL) return 0;
  c++; strncpy(typstr,c,64); strupr(typstr);
  if ((c=strchr(typstr,'X')) != NULL) c++; else c = typstr;
  return atoi(c);
}
short mtoi(char *mod)
{
  short accessMode = 0;

  if (strstr(strupr(mod),"READ")) accessMode |= 0x01;
  if (strstr(mod,"WRITE"))        accessMode |= 0x02;

  return accessMode;
};

char *itom(short accessMode)
{
  static char accessModeStr[32];

  accessModeStr[0] = 0;
  if (accessMode & 0x01) strcat(accessModeStr,strlen(accessModeStr)?"|READ":"READ");
  if (accessMode & 0x02) strcat(accessModeStr,strlen(accessModeStr)?"|WRITE":"WRITE");
  if (strlen(accessModeStr) == 0) strcat(accessModeStr,"NO");

  strcat(accessModeStr," ACCESS");
  return accessModeStr;
}
short NrOfEqpTypes = 0;
ExportListStruct *ExportList = NULL;

int datacmp(BYTE *d1,BYTE *d2,int siz,int fmt,double t)
/* compares array d1[siz] against d2[siz] with given format
   and tolerance */
{
  int fmtsize,i;

  fmtsize = fmtsizeof(fmt);
  if (!memcmp(d1,d2,siz*fmtsize)) return 0;
  if ((short)t == 0) return 1;

  /* data have changed, but by how much? */
  switch (fmt)
  {
   case CF_DOUBLE:
       for (i=0; i<siz; i++)
	 if ( ((double *)d1)[i] < ((double *)d2)[i] - t ||
	      ((double *)d1)[i] > ((double *)d2)[i] + t ) return 1;
       return 0;
   case CF_SHORT:
       for (i=0; i<siz; i++)
	 if ( ((short *)d1)[i] < ((short *)d2)[i] - (short)t ||
	      ((short *)d1)[i] > ((short *)d2)[i] + (short)t ) return 1;
       return 0;
   case CF_LONG:
       for (i=0; i<siz; i++)
	 if ( ((UINT32 *)d1)[i] < ((UINT32 *)d2)[i] - (UINT32)t ||
	      ((UINT32 *)d1)[i] > ((UINT32 *)d2)[i] + (UINT32)t ) return 1;
       return 0;
   case CF_FLOAT:
       for (i=0; i<siz; i++)
	 if ( ((float *)d1)[i] < ((float *)d2)[i] - (float)t ||
	      ((float *)d1)[i] > ((float *)d2)[i] + (float)t ) return 1;
       return 0;
   case CF_BYTE:
       for (i=0; i<siz; i++)
	 if ( ((BYTE *)d1)[i] < ((BYTE *)d2)[i] - (BYTE)t ||
	      ((BYTE *)d1)[i] > ((BYTE *)d2)[i] + (BYTE)t ) return 1;
       return 0;
  }
  return -1;
}
ALIAS_TABLE *gAliasTable = NULL;
int nAliasTableEntries = 0;
HashXRefStruct **AliasXRefTable = NULL;
int GetAliasTable(void)
{
  int n,i,cc=0,done=0,name_col,alias_col;
  char fn[80],str[256],hdr[256];
  FILE *fp = NULL;

  sprintf(fn,"%s%s",FecDBpath,ALIAS_FILE);
  if ((n=GetDataBaseSize(fn)) < 1) ccerr(-n);
  if ((gAliasTable=(ALIAS_TABLE *)calloc(n,sizeof(ALIAS_TABLE))) == NULL) ccerr(out_of_local_memory);
  if ((fp=fopen(fn,"r")) == NULL) ccerr(no_such_file);
  for (done=0,i=0; fgets(str,255,fp) != NULL;)   /* scan file       */
  {
    strtrm(str);
    if (strlen(str) == 0) continue;          /* carriage return */
    if (strchr("%;#\n",str[0])) continue;      /* comment line(s) */
    if (!done)
    {
      strncpy(hdr,str,255);
      if ((name_col=findcol(hdr,"NAME")) < 0) ccerr(no_such_column);
      if ((alias_col=findcol(hdr,"ALIAS")) < 0) ccerr(no_such_column);
      done = 1;                              /* read header info */
      continue;
    }
    strncpy(gAliasTable[i].alias,colptr((short)alias_col,str),32);
    strncpy(gAliasTable[i].name,colptr((short)name_col,str),32);
    i++;
  }
  nAliasTableEntries = i;
  if (n > ALIAS_HASH_PAYOFF && (cc=makeAliasXRefTable())) ccerr(cc);
err:
  if (fp) fclose(fp);
  sprintf(str,"alias file for : %s", cc ? erlst[cc] : "found");
  feclog(str);
  return cc;
}
int makeAliasXRefTable(void)
{
  int k,idx;
  HashXRefStruct *xref;

  if ((AliasXRefTable=(HashXRefStruct **)calloc(ALIAS_HASH_SIZE,sizeof(HashXRefStruct *))) == NULL)
    return out_of_local_memory;
  for (k=0; k<nAliasTableEntries; k++)
  {
    idx = (int)(ElfHash((unsigned char *)gAliasTable[k].alias) % ALIAS_HASH_SIZE);
    for (xref=AliasXRefTable[idx]; xref && xref->index != k; xref = xref->next)
    {
      if (!strncmp(gAliasTable[k].alias,gAliasTable[xref->index].alias,32)) break;
    }
    if (xref == NULL) 
    {
      if ((xref=(HashXRefStruct *)calloc(1,sizeof(HashXRefStruct))) == NULL) return out_of_server_memory;
      xref->index = k;
      xref->next = AliasXRefTable[idx];
      AliasXRefTable[idx] = xref;
    }
  }
  return 0;
}
int getTableIndex(char *str,HashXRefStruct **xrf,int hshsiz,char *tbl,int tblsiz,int off,int reclen)
{
  int i,idx;
  HashXRefStruct *xref;

  if (tbl == NULL) return -1;
  if (xrf != NULL && hshsiz > 0) /* this should be quick */
  {
    idx = (int)(ElfHash((unsigned char *)str) % hshsiz);
    for (xref=xrf[idx]; xref; xref = xref->next)
    {
      i = xref->index;
      if (!strncmp(str,tbl+reclen*i+off,32)) return i;
    }
  }
  else
  {
    for (i=0; i<tblsiz; i++)
    {
      if (!strncmp(str,tbl+reclen*i+off,32)) return i;
    }
  }
  return -1;
}
void csvStdInit(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return;
  db->cnt = 0;
  if (db->tgt != NULL) free(db->tgt);
  db->tgt = (BYTE *)calloc(db->siz,db->tgt_siz);
  for (i=0; i<db->ncols; i++) 
  {
    db->col[i].start = db->col[i].ptr;
    db->col[i].ptr = db->tgt + (UINT32)db->col[i].ptr;
  }
}
int csvStdIterator(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return -1;  /* don't iterate */
  for (i=0; i<db->ncols; i++) if (db->col[i].fmt != CF_NULL) db->col[i].ptr += db->tgt_siz;
  db->cnt++;
  return 0;
}
void csvStdExit(struct CSVDBStruct *db)
{
  int i;
  if (db->tgt_siz <= 0) return;
  for (i=0; i<db->ncols; i++) db->col[i].ptr = db->col[i].start;
}
int csvAssignValue(void *ptr,int fmt,int siz,char *str)
{
  if (ptr == NULL) return 0;         /* intentionally not assigned */
  if (str == NULL || strlen(str) == 0) return argument_list_error;
  switch (fmt)
  {
    case CF_NULL: break;
    case CF_TEXT: strncpy((char *)ptr,str,siz); break;
    case CF_BYTE: 
      if (siz > 1) HextoBytes(str,(BYTE *)ptr,(short)siz); 
      else *(BYTE *)ptr = (BYTE)atoi(str);
      break;
    case CF_SHORT: *(short *)ptr = atoi(str); break;
    case CF_LONG:  *(long *)ptr = strtol(str,NULL,10); break;
    case CF_FLOAT: *(float *)ptr = (float)strtod(str,NULL); break;
    case CF_DOUBLE: *(double *)ptr = strtod(str,NULL); break;
    default: return illegal_format;
  }
  return 0;
}
int csvReadFile(char *path,char *file,CSVDB *db,void **target)
{
  FILE *fp=NULL;
  int i,cc=0,done=0;
  char *c,str[256],hdr[256],scratch[80],fn[128];

  sprintf(fn,"%s%s",path,file);
  /* find database size */
  if ((db->siz=GetDataBaseSize(fn)) < 0) ccerr(-db->siz);
  if (target != NULL) db->tgt = (BYTE *)*target;
  /* call init routine if there */
  if (db->ini != NULL) db->ini(db);
  if (target != NULL) *target = db->tgt;
  /* read through */
  if ((fp=fopen(fn,"r")) == NULL) ccerr(no_such_file);
  while (fgets(str,255,fp) != NULL)
  {
    strtrm(str);
    if (strlen(str) == 0) continue;             /* carraige return */
    if (strchr("%;#",str[0]) != NULL) continue; /* comment line */
    if (!done)
    {
      strncpy(hdr,str,255);
      for (i=0; i<db->ncols; i++) db->col[i].pos = findcol(hdr,db->col[i].nam);
      done = TRUE;
      continue;
    }
    for (i=0; i<db->ncols; i++)
    {
      if (db->col[i].pos < 0) 
      {
        c = db->col[i].def;
      }
      else
      {
        strncpy(scratch,colptr((short)db->col[i].pos,str),64); scratch[64] = 0;
        c = scratch;
      }
      /* assign value to structure location */
      cc = csvAssignValue(db->col[i].ptr,db->col[i].fmt,db->col[i].siz,c);
      if (cc)
      {
        sprintf(str,"column %s entry %d : %s\n",db->col[i].nam,i,erlst[cc]);
        feclog(str); 
        ccerr(cc);
      }
      /* call action function if there */
      if (db->col[i].fcn != NULL) if (db->col[i].fcn(c) != 0) break;    
    }
    /* call iterator if there */
    if (db->itr != NULL) if (db->itr(db) != 0) break;
  }
  /* call end routine if there */
  if (db->exi != NULL) db->exi(db);
err:
  if (fp) fclose(fp);
  return cc;
}
int csvGetValue(void *ptr,int fmt,int siz,char *str)
{
  if (ptr == NULL) return 0;         /* intentionally not assigned */
  if (str == NULL || strlen(str) == 0) return argument_list_error;
  switch (fmt)
  {
    case CF_TEXT: strncpy(str,(char *)ptr,siz); break;
    case CF_BYTE: BytesToHex((BYTE *)ptr,str,siz); break;
    case CF_SHORT: sprintf(str,"%d",*(short *)ptr); break;
    case CF_LONG:  sprintf(str,"%ld",*(long *)ptr); break;
    case CF_FLOAT: sprintf(str,"%g",*(float *)ptr); break;
    case CF_DOUBLE: sprintf(str,"%g",*(double *)ptr); break;
    default: return illegal_format;
  }
  return 0;
}
int csvWriteFile(char *path,char *file,CSVDB *db)
{
  FILE *fp=NULL;
  int i,j,cc=0;
  char str[256],scratch[64],fn[128];

  sprintf(fn,"%s%s",path,file);
  if ((fp=fopen(fn,"w")) == NULL) ccerr(file_error);
  for (i=0,str[0]=0; i<db->ncols; i++)
  {
    strcat(str,db->col[i].nam);
    if (i<db->ncols-1) strcat(str,",");
  }
  fwrite(str,strlen(str),1,fp);
  for (j=0; j<db->siz; j++)
  {
    str[0] = 0;
    for (i=0; i<db->ncols; i++)
    {
      csvGetValue(db->col[i].ptr,db->col[i].fmt,db->col[i].siz,scratch);
      strcat(str,scratch);
      if (i<db->ncols-1) strcat(str,",");
    }
    fwrite(str,strlen(str),1,fp);
    /* call iterator if there */
    if (db->itr != NULL) if (db->itr(db) != 0) break;
  }
err:
  if (fp != NULL) fclose(fp);
  return cc;
}
