#ifndef _HISTORY_H_
#define _HISTORY_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "winsrv.h"

#ifndef MAX_HISTORY_RECORDS
# define MAX_HISTORY_RECORDS 20
#endif

typedef struct
{
  CONTRACT c;
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthShort;             /* for short term storage */
  int  depthLong;               /* for long term storage */
  int  dataPtr;
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
  int  recordIndex;            /* local index identifier */
  int  recordLength;
  int  interimStatus;
  int  interimCounter;
  time_t lastArchiveTime;
  char ArchiveFileName[16]; /* current working file name */
  struct timeval *timestamp;          /* timestamp array */
  BYTE *data;
} HistoryRecord;

typedef struct
{
  int  pollingRate;               /* polling rate in sec */
  int  archiveRate;               /* archive rate in sec */
  int  depthShort;             /* for short term storage */
  int  depthLong;               /* for long term storage */
  int  heartbeat;            /* archive heartbeat in sec */
  float pTolerance;                 /* percent tolerance */
  float aTolerance;                /* absolute tolerance */
} HistorySpecification;

#ifdef UNIX
# define SH_DENYNONE (S_IWUSR|S_IRUSR|S_IRGRP|S_IROTH)
#endif
#define HISTORY_FILENAME "history.csv"

#define HISTORY_BUFFER_SIZE 1024
#define STS_DEADBAND 2

extern char ArchiveLocation[128];
extern HistoryRecord *historyRecords;
extern int nHistoryRecords;
extern int maxHistoryRecords;

__declspec(dllexport) int PASCAL AppendHistoryInformation(char *eqp,char *prp,char *dev,int len,int fmt,int idx,HistorySpecification *hspec);
int LoadHistoryInformation(char *fname,char *expName,char *lclName);
void historyInit(char *expName,char *lclName);
void historyCycle(void);
long loadLTSData(int i,int index,time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 void *datbuf,int datfmt,int bufsize,int sample);
long getLTSData(char *eqp,char *prp, char *dev,int index,
                 time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 BYTE *data,int fmt,int arraysize,int sample,int recsreadFlag);
long getSTSData(char *eqp,char *prp, char *dev,int index,
                 time_t starttime,time_t endtime,
                 float lwrLimit, float uprLimit,
                 BYTE *data,int fmt,int arraysize,int sample,int recsreadFlag,int startMSEC);
int HISTQueryFunction(CONTRACT *con,char *dataIn,char *data);
void assignNumberOfPoints(BYTE *data,int fmt,int siz,int *npoints,int *nfound,int recsreadFlag);
int isInRange(BYTE *data,int fmt,int number,float lwrLimit,float uprLimit);
long getStartTimeFromDepth(char *eqp,char *prp, int depth);

#endif /* _HISTORY_H_ */
