/** \file pclib.c
 * \brief TINE Server DLL documentation 
 *
 */
#include "project.def"
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <malloc.h>
#include <sys\timeb.h>
#include <winsock.h>
#include "shmalloc.h"
#include "errors.h"
#include "database.h"
#include "winsrv.h"
#include "prolog.h"
#include "history.h"

extern int debugTargetError;
extern int debugTargetErrorMet;

int gtUDPPort = UDP_PORT;
int gtTCPPort = TCP_PORT;
int gtGCastPort = GCAST_PORT;
int gtMCastPort = MCAST_PORT;
int gtUCPClnPort = UDP_CLN_PORT;
char *gtMCastAddr = MCAST_ADDR;
char *gtGCastAddr = GCAST_ADDR;
int gtMCastTTL = MCAST_TTL;

void soperror(char *txt);
int IsMemberLockSet(CLN *cln);
int datacmp(BYTE *d1,BYTE *d2,int siz,int fmt,double t);
int renegotiateContract(ContractListStruct *cl,int cc);
int interimStatus = not_posted;
short NativeFormatIn = -1,NativeFormatOut = -1;
int RequireAcknowledgments = 1; /* global flag to force acknowledge in REFRESH mode */
int ReturnToServerCycle = 0;
int StaleData = 0;
long SysPoll = DEFAULT_POLLRATE;
unsigned short MinPollingRate = MIN_POLLRATE;
unsigned short MaxPollingRate = DEFAULT_POLLRATE;
struct fd_set UDPset;
struct timeval UDPto;
ContractListStruct **ContractList;
CLN **Consumers;
int ConsumerListArraySize = NCONSUMER;
int ContractListArraySize = NCONTRACT;

#ifndef SystemCalloc
typedef struct 
{
  ContractListStruct cls;
  int inuse;
} ContractListStructMemPool;
ContractListStructMemPool ContractListStructMem[NCONTRACT];
typedef struct 
{
  CONSUMER c;
  int inuse;
} ConsumerMemPool;
ConsumerMemPool ConsumerMem[NCONSUMER];
typedef struct 
{
  CLN cln;
  int inuse;
} clnMemPool;
clnMemPool clnMem[NCONSUMER];
typedef struct 
{
  BYTE data[16];
  int inuse;
} dataMemPool;
dataMemPool dataMem[NARRAYELEMS];

int freeFromMemPool(void *ptr)
{
  int i;
  for (i=0; i<NCONTRACT; i++) if (ptr == (void *)&ContractListStructMem[i].cls) 
  {
    ContractListStructMem[i].inuse = FALSE;
    return 0;
  }
  for (i=0; i<NCONSUMER; i++) 
  {
    if (ptr == (void *)&ConsumerMem[i].c) 
    {
      ConsumerMem[i].inuse = FALSE;
      return 0;
    }
    if (ptr == (void *)&clnMem[i].cln) 
    {
      clnMem[i].inuse = FALSE;
      return 0;
    }
  }
  for (i=0; i<NARRAYELEMS; i++) if (ptr == dataMem[i].data) 
  {
    dataMem[i].inuse = FALSE;
    return 0;
  }
  return -1;
}
BYTE *callocFromMemPool(int num, int len)
{
  int i, totalSize = num * len;
  if (num != 1 || totalSize <= 0) return NULL;
  switch (len)
  {
    case sizeof(ContractListStruct):
      for (i=0; i<NCONTRACT && ContractListStructMem[i].inuse; i++);
      if (i < NCONTRACT) 
      {
        ContractListStructMem[i].inuse = TRUE;
        return (BYTE *)&ContractListStructMem[i].cls;
      }
      break;
    case sizeof(CONSUMER):
      for (i=0; i<NCONSUMER && ConsumerMem[i].inuse; i++);
      if (i < NCONSUMER) 
      {
        ConsumerMem[i].inuse = TRUE;
        return (BYTE *)&ConsumerMem[i].c;
      }
      break;
    case sizeof(CLN):
      for (i=0; i<NCONSUMER && clnMem[i].inuse; i++);
      if (i < NCONSUMER) 
      {
        clnMem[i].inuse = TRUE;
        return (BYTE *)&clnMem[i].cln;
      }
      break;
    default:
      for (i=0; i<NARRAYELEMS && dataMem[i].inuse; i++);
      if (i < NARRAYELEMS) 
      {
        dataMem[i].inuse = TRUE;
        return dataMem[i].data;
      }
      break;
  }
  return NULL;
}
BYTE *SystemCalloc(int num, int len)
{
  BYTE *ptr=NULL;
  if ((ptr=callocFromMemPool(num,len)) == NULL) ptr = calloc(num,len);
  return ptr;
}
void SystemFree(void *ptr)
{
  if (freeFromMemPool(ptr)) SystemFree(ptr);
}

#endif // SystemCalloc

int ncontract = 0, nconsumer = 0;
int ProducerSocket;
char txtbuf[128];

int NrOfIPNets = 0;
int NrOfIPBcastNets = 0;
int nbroadcasts = 0;
int nExpiredContracts = 0;
struct sockaddr_in *IPNetList = NULL;
struct sockaddr_in *IPBcastList = NULL;
BYTE *BrdcstBuffer = NULL;
GlobalListStruct *brdcstList = NULL;
AccessLockStruct currentAccessLock;
int putCommandsInFeclog = TRUE;
CLN *LocateConsumerInList(struct sockaddr_in *ip,IPXAddress *nwAddr,BYTE *immAddr,long *hWnd);

// timezone and gettimeofday are not standard in DOS
struct timezone { int dummy; };
unsigned long ElfHash(const unsigned char *key)
{
  unsigned long g, h = 0;
  while (*key)
  {
    h = (h << 4) + *key++;
    g = h & 0xF0000000L;
    if (g) h ^= g >> 24;
    h &= ~g;
  }
  return h;
}
char tRedirectedServer[64];
ExportPropertyListStruct *lookupPropertyName(ExportListStruct *el,char *prpName,char *devName,int *CompCode)
{
  ExportPropertyListStruct *prp = NULL;
  PrpRedirBlk *prd = NULL;
  int idx,cc=illegal_property,redir=0;

  if (CompCode != NULL && *CompCode == server_redirection) ccerr(server_redirection);
  if (el == NULL || prpName == NULL) ccerr(argument_list_error);
  idx = (int)(ElfHash((unsigned char *)prpName) % PROPERTY_HASH_SIZE);
  for (prp=el->EqpProperty[idx]; prp; prp=prp->next)
    if (!strncmp(prp->prpName,prpName,32)) 
    {
      if (prp->prd != NULL)
      {
        for (prd=prp->prd; prd != NULL; prd=prd->next)
        {
          if (prd->prpRedirection[2].name[0] == 0) redir = -1;
          if (prd->prpRedirection[2].name[0] != 0 &&
              !strncmp(prd->prpRedirection[2].name,devName,16)) redir = -1;
          if (redir)
          {
            memcpy(tRedirectedServer,prd->prpRedirection[0].name,16*4);
            ccerr(server_redirection);
          }
        }
      }
      if (CompCode != NULL) *CompCode = 0;
      return prp;
    }
err:
  if (CompCode != NULL) *CompCode = cc;
  return NULL;
}
char *lookupPropertyId(ExportListStruct *el,int id)
{
  ExportPropertyListStruct *prp;
  int idx;

  if (el == NULL) return (char *)NULL;
  idx = PROPERTY_HASH_INDEX(id);
  for (prp=el->EqpProperty[idx]; prp; prp=prp->next)
    if (prp->prpId == id) return prp->prpName;
  return (char *)NULL;
}

struct timeval *gettimeofday(struct timeval *t,struct timezone *tz)
{
  static struct timeb tb;
  ftime(&tb);
  t->tv_sec  = tb.time;
  t->tv_usec = ((unsigned long)tb.millitm)*1000L;
  return t;
}
char RPCLastErrorString[RPCERR_SIZE];
void SetRPCCompletion(char *errstr)
{
  memset(RPCLastErrorString,0,RPCERR_SIZE);
  if (errstr != NULL && strlen(errstr) > 0)
  {
    strncpy(RPCLastErrorString,errstr,RPCERR_SIZE-1);
  }
}
long findpoll(long p1,long p2)
{
  long p3;

  if (p1 < MinPollingRate) return p2 <= MinPollingRate ? MinPollingRate : p2;
  if (p2 < MinPollingRate || p1 == p2) return p1;
  if (p2 < p1) while ((p3 = p1%p2) >= MinPollingRate) { p1 = p2; p2=p3; }
  else if (p1 < p2) while ((p3 = p2%p1) >= MinPollingRate) { p2 = p1; p1=p3; }

  return p1 > p2 ? p2 : p1;
}
int RemoveConsumerFromList(CONSUMER *c,int j)  // from contract j
{
  ContractListStruct *con;
  CONSUMER *p;

  if (j < 0 || j>=ncontract || c == hNULL) return -2;
  c->client->count = MAX(c->client->count-1,0);
  StaleData++;
  con = ContractList[j];
  p = con->consumer;

  if (p == c) con->consumer = c->next;
  else
  {
    while (p->next != c && p->next != NULL) p = p->next;
    p->next = c->next;
  }
  SystemFree((char *)c);
  con->nconsumer = MAX(con->nconsumer-1,0);
  if (con->nconsumer == 0) return 0;

  p = con->consumer;
  con->PollingRate = p->status.PollingRate;
  for (p=p->next; p != NULL; p = p->next)
  {
    con->PollingRate = findpoll(con->PollingRate,p->status.PollingRate);
  }
  return 0;
}
int RemoveConsumer(int i)  // return memory to heap
{
  if (i < 0 || i >= nconsumer) return -2;
  if (Consumers[i] == NULL) return -3;
  SystemFree((char *)Consumers[i]);
  for (;i<nconsumer-1; i++) Consumers[i] = Consumers[i+1];
  Consumers[--nconsumer] = NULL;
  return 0;
}
int RemoveContract(int j)  // return memory to heap
{                          // decrement ncontract
  CONSUMER *c,*p;
  ContractListStruct *con;

  if (j < 0 || j>=ncontract) return -2;
  con = ContractList[j];

  c = con->consumer;
  for (p = c = con->consumer; p != NULL; c=p)
  {
    c->client->count = MAX(c->client->count-1,0);
    StaleData++;
    p = c->next;
    SystemFree((char *)c);
  }
  if (con->expired && nExpiredContracts > 0) nExpiredContracts--;
  if (con->DataIn) SystemFree((char *)con->DataIn);
  if (con->DataOut) SystemFree((char *)con->DataOut);
  SystemFree((char *)ContractList[j]);
  for (;j<ncontract-1; j++) ContractList[j] = ContractList[j+1];
  ContractList[--ncontract] = NULL;
  return 0;
}

CONSUMER *EQPconsumerlist = NULL;                      /* global reference  */
int EQPnconsumers;

__declspec(dllexport) short PASCAL ScheduleProperty(char *eqm, char *prp)
{
  int i;
  ContractListStruct *cl;       /* improve readability by assignment */
  char *p=NULL,*prpstr;

  if (eqm == NULL || prp == NULL) return argument_list_error;
  if ((prpstr=(char *)alloca(strlen(prp)+1)) == NULL) return out_of_server_memory;
  gettimeofday(&gThisCycleTime,(struct timezone *)NULL);/* Cycle time */
  lastCycleTime = gThisCycleTime;

  strcpy(prpstr,prp);
  for (p=strtok(prpstr,",|"); p != NULL; p=strtok(NULL,",|"))
  {
    if (GetPropertyId(eqm,p) < 0) continue;
    for (i=0; i<ncontract; i++)
    {
      if ((cl=ContractList[i]) == NULL) continue;
      if (cl->nconsumer > 0 && !strncmp(cl->contract.EqpProperty,p,EQP_PROP_SIZE))
      {
        /* guarantee immediate execution of any contract using this property */
        cl->lasttime.tv_sec = cl->lasttime.tv_usec = PRP_SCHEDULE_SIGNAL;
      }
    }
  }

  ProduceData();                  /* call the contract */
  while (StaleData) DeliverData();/* send it out */

  gettimeofday(&gThisCycleTime,(struct timezone *)NULL);/* Cycle time */
  gTimeServerCycle += MSECS(gThisCycleTime,lastCycleTime);
  lastCycleTime = gThisCycleTime;
  CycleCounts++;

  return 0;
}

int ProduceData(void)  // Delivery system
{
  ContractListStruct *cl;      /* improve readability by assignment */
  CONSUMER *c;
  short cc,rc,i,dirty,Stale,fmtoutsize;
  UINT32 doutsize;
  int delta_t;
  static struct timeval tv,tv_stamp;
  int isScheduled;

//  gettimeofday(&tv_stamp,(struct timezone *)NULL); tv = tv_stamp;
  tv = gThisCycleTime;
  for (i=0; i<ncontract; i++)
  {
    if ((cl=ContractList[i]) == NULL) continue;
    if (cl->nconsumer == 0 && !cl->expired)
    {
      if (cl->contract.EqpAccess & CA_LAST)    /* going out of scope */
        doContract(&cl->contract,cl->exp,cl->DataIn,WorkArea,i,STD_PROTOCOL);
      cl->pending = 0;
      StaleData++;        /* Ensure pass thru deliverData for contract removal */
      RemoveContract(i--); 
      continue;
    }
    cc = Stale = isScheduled = 0;
    if ((fmtoutsize=fmtsizeof(cl->contract.EqpFormatOut)) != 0)
    {
      doutsize = cl->contract.EqpSizeOut*fmtoutsize;
      if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
    }
    else doutsize = 0;
    if ((delta_t=MSECS(tv,cl->lasttime)) < 0) cl->lasttime = tv; /* system clock set back */
    if (delta_t < cl->PollingRate) continue;
    if (cl->expired && cl->nconsumer == 0)
    {
      if (delta_t > 3*cl->PollingRate) RemoveContract(i--); /* decrements ncontract => decrement i */
      continue;
    }
    if (cl->lasttime.tv_sec == PRP_SCHEDULE_SIGNAL) isScheduled = TRUE;
    if (delta_t < MAXALLOWED_POLLRATE && delta_t >= 2*cl->PollingRate)
    {
      if (debug > 1)
      {
        wsprintf(txtbuf,"[%d] (%s) %s %s missed last schedule (%d vs %d)",i,
          cl->contract.EqpName,
          cl->contract.EqpDeviceName,
          cl->contract.EqpProperty,delta_t,2*cl->PollingRate);
        output(txtbuf);
      }
      cl->misses++;
      gContractMisses++;
    }
    if (cl->pending > 0) 
    {
      if (debug > 2)
      {
        sprintf(txtbuf,"[%d] (%s) %s %s still pending",i,
          cl->contract.EqpName,
          cl->contract.EqpDeviceName,
          cl->contract.EqpProperty);
        output(txtbuf);
      }
      cl->delays++;
      gContractDelays++;
      StaleData++;        /* Ensure pass thru deliverData but ...*/
      continue;           /* still sending out the last round */
    }
    tv_stamp = tv;
    if (!(cl->PollingRate%1000)) tv_stamp.tv_usec = 0;

    gDataTimeStamp = gDataTimeStampOffset + (double)tv.tv_sec + (double)tv.tv_usec/1000000;
    EQPnconsumers = cl->nconsumer;
    EQPconsumerlist = cl->consumer;
    if (cl->nconsumer == 1 && cl->consumer->status.counter == 1)
	    cl->contract.EqpAccess |= CA_LAST; /* dangling client going out of scope ? */
    switch (cc=doContract(&cl->contract,cl->exp,cl->DataIn,WorkArea,i,STD_PROTOCOL))
    {
       case 0: 
         break;
       case not_ready:                         /* don't send out */
       case completion_waiting:
         for (c=cl->consumer; c != NULL; c=c->next) c->status.Stale = 0;
         cl->pending = 0;
         continue;
       default:
         if (cc & 0xf000) /* CE_RENEGOTIATE or CE_SENDDATA */
         {
           if ((rc=renegotiateContract(cl,(cc & 0xfff))) != 0) cc = rc; else cc &= ~CE_RENEGOTIATE;
           if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
         }
         else goto err;
   }

    if (doutsize && !cl->expired && memcmp(cl->DataOut,WorkArea,doutsize))
    {
      memcpy(cl->DataOut,WorkArea,doutsize);
      Stale = dirty = 1;                       /* Data have changed: */
    }                                          /* the cached data are dirty */
    else
    {
      if (MSECS(tv,cl->lastmarked) > (HEARTBEAT-1)*1000L) Stale = 1;
      dirty = 0;                               /* Cached data have not changed */
    }

err:if (cc) Stale = 1;
    if (Stale) cl->lastmarked = tv_stamp;      /* Last marked as Stale */
    cl->lasttime = tv_stamp;                   /* Last polled */
    cl->dtimestamp = RPCtimestamp;
    cl->dtimestampMSEC = RPCtimestampMSEC;
    cl->returnCode = cc;

    for (c=cl->consumer; c != NULL; c=c->next)
    {
      if (c->status.counter == 0)
      {
        StaleData++;
        continue;
      }
      c->status.statusCode = cc;
      if (cc) c->status.Stale = 1;             /* Stale because of error code */
      if (isScheduled) c->status.lasttime.tv_sec = c->status.lasttime.tv_usec = PRP_SCHEDULE_SIGNAL; 
      if (c->status.mode == REFRESH)           
      {
        if (Stale)
        {         
          c->status.Stale = 1;                 /* Stale because of data change or heartbeat */
          if (dirty && RequireAcknowledgments)
            c->status.counter = ACK_REQUEST;   /* Force acknowledgment */
        }
      }
      else                                     /* Honor polling rate   */
      {
        if (MSECS(tv,c->status.lasttime) > c->status.PollingRate - gSystemDelay)
          c->status.Stale = 1;                 /* Stale because of polling rate */
      }
      if (c->status.Stale)                     /* Consumer for this contract */
      {                                        /* is to be refreshed */
        c->status.lasttime = tv_stamp;
        StaleData++;                           /* Ensure pass thru deliverData */
        cl->pending++;
      }
      if (c->status.counter == ACK_REQUEST)    /* Get serious until an */
      {                                        /* acknowledgment comes */
        c->status.mode = ACK_PENDING;          /* Reset by renewal */
        c->status.PollingRate = cl->PollingRate;
      }
    }
  }
  return 0;
}
#define maxDeadTime(m,p) ((m) == CM_REFRESH ? HEARTBEAT + (p) : 2 * (p))
void clearPending(int k)
{
  short i;
  CONSUMER *c;
  ContractListStruct *cl;       /* improve readability by assignment */
  for (i=0; i<ncontract; i++)
  {
    cl = ContractList[i];
    for (c=cl->consumer; c != NULL; c=c->next) if (c->client == Consumers[k]) break;
    if (c == NULL) continue;
    RemoveConsumerFromList(c,i);
    cl->pending--;
  }
}
int DeliverData(void)            /* Delivery system */
{
  ProducerHeaderStruct4 tmpP,*Pheader = &tmpP;
  ProducerHeaderStruct tmpPEx,*PheaderEx = &tmpPEx;
  CONSUMER *c;
  ContractListStruct *cl;       /* improve readability by assignment */
  short i,k,n,passed,snddata,basemode,actionTaken=FALSE;
  long gapsize, gapptr;
  int dsize;
  UINT16 msgsize,totalsize=0,packetmtu;
  UINT32 dptr,delta_t;
  short stepsize;
  BYTE *buf = WorkArea;

  for (k=0; k<nconsumer; k++)
  {
    if (Consumers[k] == NULL) continue;
    if (Consumers[k]->count == 0 || Consumers[k]->sck == 0)
    {
      RemoveConsumer(k--);
      continue;
    }
    passed = 1; totalsize = sizeof(totalsize);
    for (i=0,n=0; i<ncontract && n<Consumers[k]->count; i++)
    {
      cl = ContractList[i];
      for (c=cl->consumer; c != NULL; c=c->next) if (c->client == Consumers[k]) break;
      if (c == NULL) continue;
      n++; snddata = TRUE; gapptr = gapsize = 0;
      basemode = c->status.mode;
      if (c->status.Stale)
      {
        actionTaken = TRUE;
        stepsize = c->client->protocol == 4 ? PRODUCER4_SIZE : PRODUCER_SIZE;
        packetmtu = sizeof(UINT16) + stepsize + c->status.mtu;
        snddata = (c->status.statusCode == 0) ? TRUE : 
                  (c->status.statusCode & CE_SENDDATA) ? TRUE : FALSE;
        if (snddata)
        {
          dsize = c->status.nextDataSize;
          dptr  = c->status.nextDataPtr;
        }
        else
        {
          dsize = gapsize = (c->client->protocol == 4 ? 0 : RPCERR_SIZE);
          dptr  = 0;
        }
        if ((msgsize=dsize+stepsize) > packetmtu) 
        { 
          c->status.counter = 0; continue; 
        }
        if (msgsize % 2 != 0) msgsize++;
        gDataTimeStamp = putDataTimeStamp(0,cl->dtimestamp,cl->dtimestampMSEC);
        PrepPCHeader(&buf[totalsize],c,cl->contract);
        if (snddata && fmtsizeof(cl->contract.EqpFormatOut) != 0)
        {
          if ((c->status.statusCode & 0xfff))
          {
            gapsize = (long)(c->status.nextDataPtr + c->status.nextDataSize -
                             cl->contract.EqpSizeOut*fmtsizeof(cl->contract.EqpFormatOut));
            if (gapsize > RPCERR_SIZE) gapsize = RPCERR_SIZE; /* shouldn't be possible */
            if (gapsize > dsize)                              /* err string remainder  */
            {
              gapsize = dsize;
              gapptr  = RPCERR_SIZE - gapsize;
            }
            if (gapsize > 0) dsize -= gapsize; else gapsize = 0;
          }
          if (dsize > 0)
          {
            memcpy(&buf[totalsize+stepsize],&cl->DataOut[dptr],dsize);
          }
        }
        
        if ((totalsize + msgsize) > packetmtu) { passed = 0; goto bok; }
        totalsize += msgsize;

        if (debug > 1 || (debug && c->status.mode <= SINGLE))
        {
          wsprintf(txtbuf,"id %d (%ld ms) %s %s %s <%d>(%d b %d of %d) to %s (+%d)\n",
                    i,cl->PollingRate,
                    cl->contract.EqpName,
                    cl->contract.EqpDeviceName,
                    cl->contract.EqpProperty,c->status.statusCode,msgsize,
                    c->status.blknum,c->status.numblks,
                    c->client->userName,c->status.counter);
          output(txtbuf);
        }
        if (gapsize > 0)                      /* insert err string */
        {
          if ((c->status.statusCode & 0xfff) < numErr) SetRPCCompletion(erlst[c->status.statusCode & 0xff]);
          memcpy(&buf[totalsize-gapsize],&RPCLastErrorString[gapptr],gapsize);
        }
        if (snddata == 0 || c->status.blknum >= c->status.numblks)
        {                                     /* datablock finished and/or error code */
          c->status.Stale = 0;
          if (c->status.counter > 0 ) c->status.counter--;
          if (c->status.mode == CM_CANCEL) c->status.counter = 0;
          c->status.blknum = 1;
          c->status.nextDataPtr = 0;
          c->status.BytesRemaining = c->status.BytesTotal;
          ReturnToServerCycle = 0;
          cl->pending--;
          if ((delta_t=MSECS(gThisCycleTime,c->status.lasttime)) > (UINT32)maxDeadTime(c->status.mode,c->status.PollingRate))
          {
            if (debug > 1)
            {
              wsprintf(txtbuf,"[%d]: %s %s %s -> returning late data (%d > %d ms)",
                  i,cl->contract.EqpName,
                  cl->contract.EqpDeviceName,
                  cl->contract.EqpProperty,delta_t,c->status.PollingRate);
              output(txtbuf);
            }
            c->status.misses++;
            gClientMisses++;
          }
        }
        else
        {
          ReturnToServerCycle = c->status.numblks - c->status.blknum;
          c->status.blknum++;
          c->status.nextDataPtr += c->status.nextDataSize;
          c->status.BytesRemaining = c->status.BytesTotal - c->status.nextDataPtr;
          StaleData++;
        }
        c->status.nextDataSize = MIN(c->status.mtu,c->status.BytesRemaining);
      }
      if (c->status.counter == 0) 
      { 
        actionTaken = TRUE;
        RemoveConsumerFromList(c,i); 
        n--; 
      }
      if (!cl->nconsumer) 
      {
        actionTaken = TRUE;
        if (gRetardSingleContractRemoval && basemode == CM_SINGLE) 
        {
          cl->expired = TRUE;
          cl->contract.EqpAccess &= ~(CA_FIRST|CA_LAST);
          nExpiredContracts++;
        }
        else 
        {
          RemoveContract(i--); /* decrements ncontract => decrement i */
        }
      }
    
    }

bok:if (totalsize > sizeof(totalsize) && Consumers[k])
    {
      actionTaken = TRUE;
      *(short *)WorkArea = totalsize;
      if (SendToPeer(Consumers[k],WorkArea)) clearPending(k);
    }
    if (!passed) k--;
  }
  if (!actionTaken) StaleData = 0;
  return StaleData = --StaleData > 0 ? StaleData : 0;
}
ContractProgress cp[MAXLONGGRAMS];
void CleanupStrandedDatagrams(void)
{
  int i;

  for (i=0; i<MAXLONGGRAMS; i++)
    if (cp[i].busy && time(NULL) > cp[i].blktime + 5)
  {
    SystemFree(cp[i].data);
    memset(&cp[i],0,sizeof(ContractProgress));
    if (debug)
    {
      wsprintf(txtbuf,"stale long datagram %d",i);
      output(txtbuf);
    }
  }
}

#define SADDR_PTR struct sockaddr *
int getConsumerIPAddress(int sck, int prot)
{
  CLN *cln = NULL;
  struct sockaddr_in IPconsumer;
  char *c;
  static int len = sizeof(IPconsumer);
  short nread = 0,nleft,n,cc = -1;
  static struct timeval to;
  static struct fd_set rset;
  TCPBCKT *bckt;
  BYTE *bufptr=WorkArea;

  memset(&IPconsumer,0,sizeof(IPconsumer));
  if ((nread=recvfrom(sck,(LPSTR)WorkArea,UDP_BUFFER_SIZE,0,(SADDR_PTR)&IPconsumer,(LPINT)&len)) <= 0) 
   ccerr(connection_timeout);
  if ((IPconsumer.sin_addr.S_un.S_addr == 0) &&
       getpeername(sck,(struct sockaddr *)&IPconsumer,&len) != 0) ccerr(tcp_socket_error);

  if ((c=inet_ntoa(IPconsumer.sin_addr)) != NULL) strcpy(clientaddr,c);
  if ((cln=LocateConsumerInList(&IPconsumer,NULL,NULL,NULL)) == NULL) ccerr(out_of_server_memory);
  cln->sck = sck;  cln->transport = prot; cc = 0;
  IPconsumer.sin_family = PF_INET;
  if (prot == TCP)
  {
    TCPcount++; 
    memcpy(&cln->IPaddress,&IPconsumer,sizeof(IPconsumer));
    if (debug)
    {
      wsprintf(txtbuf,"TCP: [%s] ",(char *)clientaddr);
      output(txtbuf);
    }
    if ((bckt=findBucket(sck)) == NULL) ccerr(tcp_socket_error);
    nleft = nread;
    while (nleft > 0)
    {
      if (bckt->ptr == 0) bckt->siz = *(short *)bufptr;
      n = bckt->siz - bckt->ptr;
      if (nleft >= n)
      {
        if (debug > 4)
        {
          wsprintf(txtbuf,"TCP: %d bytes read %d bytes needed [%s:%d] ",
            nleft,n,(char *)clientaddr,sck);
          OutputToDebugFile("getConsumerIPAddress",txtbuf);
        }
        memcpy(&bckt->buf[bckt->ptr],bufptr,n);
        getConsumerRequest(cln,bckt->buf,UDP_BUFFER_SIZE);
        bufptr += n;
        nleft -= n;
        bckt->ptr = 0;
      }
      else
      {
        if (debug > 4)
        {
          wsprintf(txtbuf,"TCP: incomplete -> %d bytes read %d bytes needed [%s:%d] ",
            nleft,n,(char *)clientaddr,sck);
          OutputToDebugFile("getConsumerIPAddress",txtbuf);
        }
        memcpy(&bckt->buf[bckt->ptr],bufptr,nleft);
        bckt->ptr += nleft;
        nleft = 0;
      }
    }
    cln = NULL;
    ccerr(0);
  }
  UDPcount++; 
  if (nread < *(short *)WorkArea) ccerr(net_read_error);
  if (debug)
  {
    wsprintf(txtbuf,"UDP: [%s] ",(char *)clientaddr);
    output(txtbuf);
  }
err:
  if (cc && cc != connection_timeout)
  {
    if (debug)
    {
      wsprintf(txtbuf,"%s\n",(char *)erlst[cc]);
      output(txtbuf);
    }
  }
  if (cln != NULL) getConsumerRequest(cln,WorkArea,UDP_BUFFER_SIZE);
  return (int)nread;
}
#define IsMemberControlNets(c) (\
  IsMemberIPControlNets(NrOfIPNets,IPNetList,&(c)->IPaddress)\
)
#define IsMemberBcastNets(c) (\
  IsMemberIPControlNets(NrOfIPBcastNets,IPBcastList,&(c)->IPaddress)\
)
int IsMemberIPControlNets(int num,struct sockaddr_in *lst,struct sockaddr_in *ipaddr)
{
  long targetnet,targetnode,cnode;
  int n;

  if (num == 0) return TRUE;  /* everything's open */
  if (lst == hNULL) return TRUE;
  if (ipaddr == NULL) return FALSE;
  targetnet  = ipaddr->sin_addr.s_addr&0xffffffL;
  targetnode = (ipaddr->sin_addr.s_addr >> 24) & 0xff;
  for (n=0; n<num; n++)
  {
    if ((lst[n].sin_addr.s_addr&0xffffffL) != (UINT32)targetnet) continue;
    if ((cnode=((lst[n].sin_addr.s_addr >> 24)&0xff)) == 0xff) return TRUE;
    if (cnode == targetnode) return TRUE;
  }
  return FALSE;
}
int IsMemberLockSet(CLN *cln)
{
  if (!currentAccessLock.lockType) return TRUE; /* unlocked */
  if (strncmp(currentAccessLock.cln.userName,cln->userName,16)) return FALSE;
  if (memcmp(&currentAccessLock.cln.IPXaddress,&cln->IPXaddress,sizeof(IPXAddress)))
    return FALSE;
  if (memcmp(&currentAccessLock.cln.IPaddress,&cln->IPaddress,sizeof(struct sockaddr_in)))
    return FALSE;
  return TRUE;
}
int IsDeniedUser(char *usr)
{
  int i;
  for (i=0; i<NrOfDeniedUsers && strnicmp(usr,DeniedUserList[i].UserName,16); i++);
  return i == NrOfDeniedUsers ? 0 : -1;
}
short IsIllegalProperty(CONTRACT *con,CLN *cln,int *EqpNativeFormatIn,int *EqpNativeFormatOut,
						int *MaxSizeIn,int *MaxSizeOut,ExportListStruct **listEntry)
{
  int k,plen=0,ps,modQuery=0,rc=0;
  short prpType,cc=0;
  ExportListStruct *el = *listEntry;
  ExportPropertyListStruct *prp=NULL;
  char *c, Property[32], s[80];

  strncpy(Property,con->EqpProperty,32);
  for (k=0; k<NrStockModifiers; k++)
    if ((c=strstr(Property,StockModifier[k].modName)) != NULL)
    {
      *c = 0;
      modQuery = -1;
      break;
    }

  prpType = EMPTY_PROP;
  if (el == NULL)
  {
    for (el = ExportList; el; el = el->next) 
    {
      if (!strncmp(el->EqpName,con->EqpName,EQP_NAME_SIZE)) break;
      if (!strncmp(el->EqpAlias,con->EqpName,EQP_NAME_SIZE)) break;
    }
    if (el == NULL) ccerr(non_existent_elem);
  }

  if ((prp=lookupPropertyName(el,con->EqpProperty,con->EqpDeviceName,&rc)) == NULL &&
      (prp=lookupPropertyName(el,Property,con->EqpDeviceName,&rc)) == NULL) 
  { /* failed first test */
    if (rc == server_redirection) ccerr(server_redirection);
    /* check for wild cards */
    for (prp=el->EqpProperty[PROPERTY_HASH_SIZE]; prp; prp=prp->next)
    {
      for (ps = 0; ps < (int)strlen(prp->prpName) && prp->prpName[ps] == '*'; ps++);
      plen = (ps < (int)strlen(prp->prpName)) ? strcspn(&prp->prpName[ps],"*") : 0;
      if (plen && !strncmp(&Property[ps],&prp->prpName[ps],plen)) break;
    }
    if (prp == NULL) /* Maybe it's a query */
    {
      if ((prpType=IsStockProperty(Property)) != STOCK_PROP) ccerr(illegal_property);
    }
    else prpType = WILDCARD_PROP;
  }
  else prpType = TARGET_PROP;

  if (prpType == TARGET_PROP || prpType == WILDCARD_PROP)
  {
    if (con->EqpSizeOut > 0)
    {
      *EqpNativeFormatOut = prp->prpFormatOut;
      if (modQuery) ccerr(0);
      if ( (prp->prpFormatOut > CF_FLOAT || LFMT(con->EqpFormatOut) > CF_FLOAT)
            && prp->prpFormatOut != LFMT(con->EqpFormatOut))
      {
        if (((prp->prpFormatOut != CF_SPECTRUM && prp->prpFormatOut != CF_FLTINT) ||
             LFMT(con->EqpFormatOut) != CF_FLOAT) )
        ccerr(illegal_format);
      }
      if (prp->prpFormatOut == CF_STRUCT && strlen(con->strTagOut) > 0 && 
          strncmp(prp->strTagOut,con->strTagOut,8))
        ccerr(invalid_structure_tag);
    }
    if (con->EqpSizeIn > 0)
    {
      if ( ((*EqpNativeFormatIn=prp->prpFormatIn) > CF_FLOAT || LFMT(con->EqpFormatIn) > CF_FLOAT ) 
            && prp->prpFormatIn != LFMT(con->EqpFormatIn))
        ccerr(illegal_format);
	    if (prp->prpFormatIn == CF_STRUCT && strlen(con->strTagIn) > 0 && 
          strncmp(prp->strTagIn,con->strTagIn,8))
        ccerr(invalid_structure_tag);
    }
    if (con->EqpSizeOut > (UINT32)(*MaxSizeOut=prp->prpSizeOut)) ccerr(dimension_error);	  
    if (con->EqpSizeIn > (UINT32)(*MaxSizeIn=prp->prpSizeIn)) ccerr(dimension_error);	  
    prpType = TARGET_PROP;
  }

  if ((con->EqpAccess&CA_WRITE) && strcmp(Property,"MESSAGE"))
  {
    if (cln->transport != LCL) 
    {
      for (k=0;!openToAllUsers && k<NrOfUsers;k++)
        if (!_strnicmp(UserList[k].UserName,cln->userName,16)) break;
      if (!openToAllUsers && k == NrOfUsers) 
      {
        sprintf(s,"COMMAND %s called by %s : user refused access",Property,getCaller());
        feclog(s);
        ccerr(access_denied);
      }
      if (!IsMemberControlNets(cln)) 
      {
        sprintf(s,"COMMAND %s called by %s from addr %s: network refused access",
           Property,getCaller(),inet_ntoa(cln->IPaddress.sin_addr));
        feclog(s);
        ccerr(access_denied);
      }
    }
    if (strcmp("ACCESSLOCK",Property) && !IsMemberLockSet(cln))
    {
      sprintf(s,"COMMAND %s called by %s from addr %s: access lock refused access",
         Property,getCaller(),inet_ntoa(cln->IPaddress.sin_addr));
      feclog(s);
      ccerr(access_denied);
    }
    if (putCommandsInFeclog)
    {
      sprintf(s,"COMMAND %s called by %s",Property,getCaller());
      feclog(s);
    }
  }

err:
  *listEntry = el;
  return cc;
}
void displayData(DTYPE *d,int max,char *txt)
{
  int i,n,len;
  char s[132],tmp[48];

  if (d == NULL || d->dFormat == CF_NULL || d->dArrayLength <= 0) return;
  if (d->dFormat != CF_TEXT) n = MIN((int)d->dArrayLength,max);
  else n = MIN(d->dArrayLength,130);

  sprintf(s,"%s : ",txt);
  if (d->dFormat == CF_TEXT) 
  {
	strncpy(tmp,d->data.cptr,47); tmp[47] = 0;
	sprintf(&s[strlen(s)],"%s",tmp);
  }
  else
  {
	  for (i= 0; i<n; i++)
	  {
		switch (d->dFormat)
		{
		  case CF_SHORT: sprintf(tmp,"%d ",d->data.sptr[i]); break;
		  case CF_LONG: sprintf(tmp,"%d ",d->data.lptr[i]); break;
		  case CF_FLOAT: sprintf(tmp,"%g ",d->data.fptr[i]); break;
		  case CF_DOUBLE: sprintf(tmp,"%g ",d->data.dptr[i]); break;
		  default: sprintf(tmp,"%d ",d->data.bptr[i]); break;
		}
		if ((len=(int)strlen(s)) > 130 - (int)strlen(tmp)) break;
     	sprintf(&s[len],"%s",tmp);
	  }
  }
  sprintf(&s[strlen(s)],"\n");
  output(s);
  return;
}
int renegotiateContract(ContractListStruct *cl,int cc)
{
  CONSUMER *c;
  UINT32 doutsize = 0,fmtoutsize;

  if ((fmtoutsize=fmtsizeof(cl->contract.EqpFormatOut)) != 0)
  {
    doutsize = cl->dataSizeOut*fmtoutsize;
    if (cc) doutsize += RPCERR_SIZE;
    if (LFMT(cl->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
  }
  for (c=cl->consumer; c != NULL; c=c->next)
  {
    c->status.BytesRemaining = (UINT32)doutsize;
    c->status.BytesTotal = (UINT32)doutsize;
    c->status.nextDataSize = c->status.BytesRemaining;
    if (c->status.BytesRemaining > c->status.mtu) c->status.nextDataSize = c->status.mtu;
    c->status.numblks = (UINT16)(doutsize/c->status.mtu + (doutsize%c->status.mtu ? 1 : 0));
    if (doutsize == 0) c->status.numblks = 1;
  }
  return 0;
}
int doContract(CONTRACT *con,ExportListStruct *el,BYTE *dataIn,BYTE *dataOut,int id,int prot)
{
  int i,fmtsiz,dp,cc;
  DTYPE din,dout;
  UINT32 doutlen,doutsize;
  char *c;
  ExportPropertyListStruct *pls;

  doutlen = con->EqpSizeOut; fmtsiz = fmtsizeof(con->EqpFormatOut);
  doutsize = doutlen*fmtsiz; if (LFMT(con->EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
  if (ContractList[id]->expired)
  {
    if (debug > 1)
    {
      wsprintf(txtbuf,"[%d] (%s) %s %s returning expired data",id,con->EqpName,con->EqpDeviceName,con->EqpProperty);
      output(txtbuf);
    }
    con->EqpSizeOut = ContractList[id]->dataSizeOut;
    ccerr(ContractList[id]->returnCode);
  }
  /* check stock properties: */
  if ((i=GetStockPropertyId(con->EqpProperty)) >= 0)
  {
    gDataTimeStamp = gDataTimeStampOffset + startuptime;
    if (doutsize) memset(dataOut,0,doutsize);
    ccerr((*StockProperty[i].fcn)(con,(char *)dataIn,(char *)dataOut));
  }
  /* check stock modifiers: (allow overloads) */
  if (GetPropertyId(con->EqpName,con->EqpProperty) < 0)
  {
    for (i=0; i<NrStockModifiers; i++)
      if (strstr(con->EqpProperty,StockModifier[i].modName))
      {
        if (doutsize) memset(dataOut,0,doutsize);
        ccerr(StockModifier[i].fcn(con,(char *)dataIn,(char *)dataOut));
      }
  }
  if (el == NULL || (el->hEqpFcn == NULL)) return non_existent;

  /* call equipment function: */
  if (el->hEqpFcn != NULL)
  {
    if ((c=strstr(con->EqpProperty,".NAM")) != NULL && // in case this has been overloaded
         strlen(c) == 4 &&
        (pls=lookupPropertyName(el,con->EqpProperty,con->EqpDeviceName,NULL)) != NULL &&
        con->EqpSizeOut > pls->prpSizeOut) 
      con->EqpSizeOut = pls->prpSizeOut;
    dout.dArrayLength = con->EqpSizeOut;
    dout.dFormat = LFMT(con->EqpFormatOut);
    dout.data.vptr = dataOut;
    strncpy(dout.dTag,con->strTagOut,8);
    din.dArrayLength = con->EqpSizeIn;
    din.dFormat = LFMT(con->EqpFormatIn);
    din.data.vptr = dataIn;
    strncpy(din.dTag,con->strTagIn,8);
    if (debug > 1 && din.dFormat != CF_NULL) displayData(&din,10,"input");
    if ((cc=ContractList[id]->interimStatus) != not_posted && cc != not_ready && cc != completion_waiting) 
    {
      ContractList[id]->interimCounter = 0;
      ContractList[id]->interimStatus = not_posted;
      return cc;                                   // was not ready, now it's ready to send out
    }
    if (ContractList[id]->interimStatus != completion_waiting) // fire the event if I haven't already
    {
      ContractList[id]->interimStatus = not_posted;  // let the app. clear this
      if (SystemPresetMemory && doutsize) memset(dataOut,0,doutsize);
      if ((cc=SendMessage(el->hEqpFcn,WM_SYSCOMMAND,MAKEPROTID(id,prot),prot)) > 0) return cc;
    }
    switch (cc = ContractList[id]->interimStatus) // currently set value from SetCompletion
    {
      case not_posted: 
        ContractList[id]->interimStatus = completion_waiting; // go into waiting state 
        break;
      case not_ready:
      case completion_waiting:
        break;
      default:
        ContractList[id]->interimStatus = not_posted; // finished, reset comletion state
    }
    if (debug > 1 && dout.dFormat != CF_NULL && cc == 0) displayData(&dout,10,"output");
  
    if (cc == completion_waiting && ContractList[id]->interimCounter++ > 20) 
    {
      cc = ContractList[id]->interimStatus = not_posted; // finished, reset comletion state
      ContractList[id]->interimCounter = 0;
    }
  }
err:
  if (con->EqpAccess & CA_FIRST) con->EqpAccess &= ~(CA_FIRST);
  ContractList[id]->dataSizeOut = con->EqpSizeOut;
  if (con->EqpSizeOut < doutlen) 
  {
    if (!SystemPresetMemory) /* zero out the remainder for later data comparisons */
    {
      dp = con->EqpSizeOut*fmtsiz;
      memset(&dataOut[dp],0,(doutlen-con->EqpSizeOut)*fmtsiz);
    }
    con->EqpSizeOut = doutlen;  // set it back so as not to disrupt the contract
    cc |= CE_RENEGOTIATE;
  }

  if (debugTargetError > 0 && cc == debugTargetError) debugTargetErrorMet = -1;
  return cc;
}
CLN *LocateConsumerInList(struct sockaddr_in *ip,IPXAddress *nwAddr,BYTE *immAddr,long *hWnd)
{
  int i;

  if (ip == NULL && nwAddr == NULL && hWnd == NULL) return NULL;
  for (i=0; i<nconsumer; i++)        /* is client in client list ?     */
  {
    if (ip != NULL && !memcmp(ip,&Consumers[i]->IPaddress,8)) break;
    if (nwAddr != NULL && !memcmp(&nwAddr,&Consumers[i]->IPXaddress,12)) break;
    if (hWnd != NULL && Consumers[i]->sck == *hWnd) break;
  }
  if (i < nconsumer) return Consumers[i];
  if (i >= NCONSUMER ||
     (Consumers[i]=(CLN *)SystemCalloc(1,sizeof(CLN))) == NULL)
    return NULL;
  nconsumer++;
  if (ip != NULL) memcpy(&Consumers[i]->IPaddress,ip,sizeof(struct sockaddr_in));
  if (nwAddr != NULL) memcpy(&Consumers[i]->IPXaddress,nwAddr,12);
  if (immAddr != NULL) memcpy(Consumers[i]->node,immAddr,6);
  if (hWnd != NULL) Consumers[i]->sck = *hWnd;

  return Consumers[i];
}
int GetDefaultFormatAndSize(CONTRACT *con)
{
  int i, idx, cc = illegal_property;
  ExportListStruct *el;
  ExportPropertyListStruct *prp = NULL;
  char *eqm, *eqpprp;
  BYTE *fmt;
  UINT32 *siz;

  if (con == NULL) ccerr(code_failure);

  eqm = con->EqpName; eqpprp = con->EqpProperty;
  fmt = &con->EqpFormatOut; siz = &con->EqpSizeOut;

  for (el=ExportList; el != NULL; el=el->next) if (!strncmp(eqm,el->EqpName,EQP_NAME_SIZE)) break;
  if (el == NULL) ccerr(non_existent_elem);
  idx = (int)(ElfHash((unsigned char *)eqpprp) % PROPERTY_HASH_SIZE);
  for (prp=el->EqpProperty[idx]; prp; prp=prp->next) if (!strncmp(prp->prpName,eqpprp,32))
  {
    *fmt = BFMT(prp->prpFormatOut);
    *siz = prp->prpSizeOut;
    ccerr(0);
  }
  if ((i=GetStockPropertyId(eqpprp)) >= 0)
  {
    *fmt = BFMT(StockProperty[i].prpFormat);
    *siz = StockProperty[i].prpSize;
    ccerr(0);
  }
err:
  return cc;
}
char *modstr[] = { "CANCEL","SINGLE","REFRESH","POLL","REGISTER" };
int SendMessageToCaller(CLN *cln,struct subscription *s,int cc,BYTE *data,int len)
{
  BYTE dbuf[HEADERSTRUCT_SIZE+PRODUCER_SIZE+RPCERR_SIZE], *buf;
  CONSUMER tmpC,*c = &tmpC;
  HeaderStruct tmpH,*h = &tmpH;
  ProducerHeaderStruct4 tmpP,*Pheader = &tmpP;
  ProducerHeaderStruct tmpPEx,*PheaderEx = &tmpPEx;
  UINT16 msgsize,prot = 5;

  buf = WorkArea;
  h = (HeaderStruct *)buf;

  switch (cc)
  {
    case get_subscription_id:
     if (h->Protocol !=  5) ccerr(illegal_protocol);
     break;
    case server_redirection:
      len = RPCERR_SIZE; /* = 2 * NAME16 */
      data = tRedirectedServer;
      break;
    default:
      if (cc > 0 && cc < numErr) SetRPCCompletion(erlst[cc]);
      if (debugTargetError > 0 && cc == debugTargetError) debugTargetErrorMet = -1;
      break;
  }
  if (debug)
  {
    wsprintf(txtbuf,"%s\n",(char *)erlst[cc]);
    output(txtbuf);
  }
  (c = &tmpC)->status.starttime = s->starttime;
  c->status.id        = s->id;
  c->status.numblks   = 1;
  c->status.statusCode = cc;
  c->client           = cln;
  c->client->protocol = prot;
  gDataTimeStamp = putDataTimeStamp(gDataTimeStampOffset,gThisCycleTime.tv_sec,gThisCycleTime.tv_usec/1000);
  msgsize = prot == 4 ? PRODUCER4_SIZE : PRODUCER_SIZE+RPCERR_SIZE;
  buf = dbuf;
  *(short *)buf = msgsize+sizeof(msgsize);
  PrepPCHeader(&buf[sizeof(msgsize)],c,s->contract);
  if (data != NULL && len > 0)
  {
    memcpy(&buf[sizeof(msgsize)+PRODUCER_SIZE],data,len);
  }
  else if (cc)
  {
    memcpy(&buf[sizeof(msgsize)+PRODUCER_SIZE],RPCLastErrorString,RPCERR_SIZE);
  }
  SendToPeer(cln,buf);
err:
  return cc;
}
int getNextFreeContractID(void)
{
  static UINT16 cid = 0;
  int i;
  for (i=0,cid++; i<ncontract; i++) if (ContractList[i]->contractID == cid) cid++;
  return cid;
}
#define replaceContractAliasNames(cn) { if (nAliasTableEntries > 0) {\
  if ((i=getAliasTableIndex((cn)->EqpProperty)) >= 0) strncpy((cn)->EqpProperty,gAliasTable[i].name,EQP_PROP_SIZE);\
  if ((i=getAliasTableIndex((cn)->EqpDeviceName)) >= 0) strncpy((cn)->EqpDeviceName,gAliasTable[i].name,EQP_DEVICE_SIZE);\
  }\
}
#define ccout(o) { cc = o; goto out; }
int getConsumerRequest(CLN *cln,BYTE *buf,int packetsize)
{
  BYTE *dbuf;
  HeaderStruct *h = (HeaderStruct *)buf;
  CONSUMER *c;
  short i,j=-1,fmtinsize,fmtoutsize,prot,cc=0,subid,isTrueMCast=0,renewalMultiplier;
  UINT16 totalsize,datasize,bytesread=HEADERSTRUCT_SIZE;
  static long dataPtr,dinsize,doutsize,defsize;
  long reallocsize = WorkAreaSize;
  struct subscription4 *reg;
  struct subscription  tmpS, *regEx = &tmpS;
  ProducerHeaderStruct4 *Pheader = (ProducerHeaderStruct4 *)&WorkArea[sizeof(totalsize)];
  ProducerHeaderStruct *PheaderEx = (ProducerHeaderStruct *)&WorkArea[sizeof(totalsize)];
  static UINT16 datamtu;

  if (debug > 4) 
  {
    wsprintf(txtbuf,"handle request from %s (%d bytes)",h->UserName,h->TotalSize);
    OutputToDebugFile("getConsumerRequest",txtbuf);
  }
  if ((totalsize=h->TotalSize) <= 0) ccout(out_of_range);
  if ((prot=h->Protocol) < 4 || prot > 5) ccout(illegal_protocol);
  strncpy(cln->userName,h->UserName,16);                  /* always reset   */
  strncpy(clientName,h->UserName,16);                  /* always reset   */

  PrepSubHeader(prot,&buf[bytesread]);
  replaceContractAliasNames(&regEx->contract);

  while (bytesread < totalsize && bytesread < packetsize) /* unpack request */
  {
    if (NrOfDeniedUsers > 0 && IsDeniedUser(h->UserName)) ccerr(access_denied);
    if ((fmtoutsize=fmtsizeof(regEx->contract.EqpFormatOut)) != 0)
    {
      if ((doutsize=regEx->contract.EqpSizeOut*fmtoutsize) <= 0) ccerr(illegal_data_size);
      if (LFMT(regEx->contract.EqpFormatOut) == CF_SPECTRUM) doutsize += 96;
	    if (doutsize > (long)WorkAreaSize) ccerr(WA_full);
      if (doutsize > reallocsize) reallocsize = doutsize;
    }
    else doutsize = 0;

    if ((fmtinsize=fmtsizeof(regEx->contract.EqpFormatIn)) != 0) /* data input */
    {
      if (regEx->blknum > regEx->numblks) ccerr(link_error);
      dinsize = regEx->contract.EqpSizeIn*fmtinsize;
      dbuf = &buf[bytesread + (prot == 4 ? SUBSCRIPTION4_SIZE : SUBSCRIPTION_SIZE)];
      dataPtr = ((long)(regEx->blknum-1))*regEx->mtu;
      datasize = MIN(dinsize-dataPtr,regEx->mtu);
      if (debug > 4) 
      {
        wsprintf(txtbuf,"input data %d elements of type %d",regEx->contract.EqpSizeIn,regEx->contract.EqpFormatIn);
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      if (dinsize > regEx->mtu) /* incoming long datagram */
      {
        for (i=0; i<MAXLONGGRAMS; i++)
          if (cp[i].haddr == cln->IPaddress.sin_addr.s_addr &&
              cp[i].SubscriptionID == regEx->id &&
              cp[i].SubsStarttime  == regEx->starttime) break;
        if (i == MAXLONGGRAMS) for (i=0; cp[i].busy && i<MAXLONGGRAMS; i++);
        if (i == MAXLONGGRAMS) ccerr(out_of_server_memory);
        if (++(cp[i].blksin) <= regEx->numblks)
        {
          if (cp[i].blksin == 1) cp[i].blkid = regEx->blkid;
          else if (cp[i].blkid < regEx->blkid ||
                   cp[i].blkid > regEx->blkid + 500)
          {
            cp[i].blksin = 1;
            cp[i].blkid = regEx->blkid;
            cp[i].blktime = time(NULL);
            cp[i].SubsStarttime = regEx->starttime;
          }
          if (!cp[i].busy)
          {
            cp[i].busy = 1;
            cp[i].haddr = cln->IPaddress.sin_addr.s_addr;
            cp[i].SubscriptionID = regEx->id;
            cp[i].blktime = time(NULL);
            cp[i].SubsStarttime = regEx->starttime;
            if ((cp[i].data=(BYTE *)SystemCalloc(1,dinsize)) == NULL) ccerr(out_of_server_memory);
          }
          memcpy(&cp[i].data[dataPtr],dbuf,datasize);
          if (debug > 4) 
          {
            wsprintf(txtbuf,"received %d from %d blocks",cp[i].blksin,regEx->numblks);
            OutputToDebugFile("getConsumerRequest",txtbuf);
          }
          if (cp[i].blksin < regEx->numblks) goto bow;
          cp[i].busy = 2;
          dbuf = cp[i].data;
        }
      }
    }
    else { dinsize = 0; dbuf = NULL; }

    if (regEx->PollingRate < MinPollingRate) regEx->PollingRate = MinPollingRate;

    if (LFMT(regEx->contract.EqpFormatOut) == CF_DEFAULT)
    {                                      /* caller wants default settings */
      if (debug > 4) 
      {
        wsprintf(txtbuf,"determine default data type");
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      if ((cc=GetDefaultFormatAndSize(&regEx->contract)) != 0) ccerr(cc);
      if ((fmtoutsize=fmtsizeof(regEx->contract.EqpFormatOut)) != 0)
      {
        if ((defsize=regEx->contract.EqpSizeOut*fmtoutsize) <= 0) ccerr(illegal_data_size);
        if (LFMT(regEx->contract.EqpFormatOut) == CF_SPECTRUM) defsize += 96; /* CF_SPECTRUM HEADER */
        if (defsize > reallocsize) reallocsize = defsize;
        if (defsize > doutsize)           /* caller can't buffer default */
        {
          regEx->contract.EqpSizeOut = doutsize/fmtoutsize;
          defsize = regEx->contract.EqpSizeOut*fmtoutsize;
        }
      }
      else defsize = 0;
      doutsize = defsize;                  /* for allocation purposes */
    }

    for (j=0; j<ncontract; j++)            /* is contract in list ? */
      if (!memcmp(&ContractList[j]->contract,&regEx->contract,sizeof(CONTRACT)))
      {
        if (!dbuf && !ContractList[j]->DataIn) break;          /* no data input */
        if (dbuf && ContractList[j]->DataIn) /* input buffer has been allocated */
        {
          if (!memcmp(ContractList[j]->DataIn,dbuf,(size_t)dinsize)) break;
        }
      }

    if (j == ncontract)                    /* no, add to list  */
    {
      if (debug > 4) 
      {
        wsprintf(txtbuf,"add contract to connection table");
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      if (j >= ContractListArraySize) ccerr(resources_exhausted);
      if ((ContractList[j]=(ContractListStruct *)SystemCalloc(1,sizeof(ContractListStruct))) == NULL)
        ccerr(out_of_server_memory);
      memcpy(&ContractList[j]->contract,&regEx->contract,sizeof(CONTRACT));

      ncontract++;

      if ((cc=IsIllegalProperty(&(ContractList[j]->contract),cln,
                                &ContractList[j]->prp.NativeFormatIn,&ContractList[j]->prp.NativeFormatOut,
                                &ContractList[j]->prp.MaxSizeIn,&ContractList[j]->prp.MaxSizeOut,
                                &ContractList[j]->exp)) != 0 ) ccerr(cc);

      if (debug > 4) 
      {
        wsprintf(txtbuf,"property %s is legal",(char *)ContractList[j]->contract.EqpProperty);
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      if (dinsize && (ContractList[j]->DataIn = (BYTE *)SystemCalloc(1,(size_t)dinsize)) == NULL)
      {
         SystemFree((char *)ContractList[j]);
         ccerr(out_of_server_memory);
      }
      if (doutsize && (ContractList[j]->DataOut = (BYTE *)SystemCalloc(1,(size_t)doutsize)) == NULL)
      {
         if (dinsize) SystemFree((char *)ContractList[j]->DataIn);
         SystemFree((char *)ContractList[j]);
         ccerr(out_of_server_memory);
      }
      if (dbuf) memcpy(ContractList[j]->DataIn,dbuf,dinsize);
      ContractList[j]->contract.EqpAccess |= CA_FIRST; // add FIRST bit
      ContractList[j]->interimStatus = not_posted;
      ContractList[j]->returnCode = not_posted;
      ContractList[j]->contractID = getNextFreeContractID();
    }

    /* is a new link attaching to an expired contract ? */
    if (ContractList[j]->expired && !(regEx->mode & CM_RETRY)) 
    {
      ContractList[j]->expired = 0;
      if (nExpiredContracts > 0) nExpiredContracts--;
    }

    /* is broadcast or multicast request ? */
    if (regEx->mode & (CM_BCAST|CM_MCAST))
    {
      if (debug > 4) 
      {
        wsprintf(txtbuf,"add network subscription");
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      isTrueMCast = regEx->mode & CM_MCAST;
      regEx->mode &= 0x0ff; /* strip off */
      if (regEx->mode == CM_CANCEL) ccerr(0); /* don't do anything */
      if (IsMemberBcastNets(cln) || isTrueMCast)
      {
        /* send subscription ID to caller */
        subid = ContractList[j]->contractID;
        SendMessageToCaller(cln,regEx,get_subscription_id,(BYTE *)&subid,2);
        /* add network to client list and replace cln */
        strncpy(cln->userName,"NETWORK",16);
        if (cln->IPaddress.sin_addr.s_addr != 0)
        {
          cln->IPaddress.sin_port = htons((UINT16)gtMCastPort);
          if (isTrueMCast) cln->IPaddress.sin_addr.s_addr = tMCastAddr.s_addr;
          else             cln->IPaddress.sin_addr.s_addr |= 0xff000000;
          cln = LocateConsumerInList(&cln->IPaddress,NULL,NULL,NULL);
        }
#       ifdef IPX_WSOCK
        else if (*(DWORD far *)cln->IPXaddress.network != 0)
        {
          memset(&cln->IPXaddress.network[10],MCAST_SOCKET,2);
          memset(cln->IPXaddress.node,0xff,6);
          cln = LocateConsumerInList(NULL,&cln->IPXaddress,cln->node,NULL);
        }
#       endif
        regEx->id = ContractList[j]->contractID;
        regEx->starttime = BCAST_ID;
      }
    }

	/* is client in contractList?    */
    for (c=ContractList[j]->consumer; c!=NULL && c->client!=cln; c=c->next);

    if (c == NULL)  /* no, add to list */
    {
      if (debug > 4) 
      {
        wsprintf(txtbuf,"add client to connection table");
        OutputToDebugFile("getConsumerRequest",txtbuf);
      }
      if (regEx->mode == CM_CANCEL) goto bow;
      if ((c=(CONSUMER*)SystemCalloc(1,sizeof(CONSUMER))) == NULL) ccerr(out_of_server_memory);
      c->next = ContractList[j]->consumer;
      ContractList[j]->consumer = c;
      c->client = cln;
      ContractList[j]->nconsumer++;
      c->client->count++;
      c->client->protocol = prot;
      c->status.statusCode = ContractList[j]->returnCode;
      c->status.counter = 1;
    }
    c->status.PollingRate = regEx->PollingRate;
    c->status.id = regEx->id;
    c->status.starttime = regEx->starttime;
    c->status.blknum = 1;
    c->status.nextDataPtr = 0;
    c->status.BytesRemaining = (UINT32)doutsize;
    c->status.BytesTotal = (UINT32)doutsize;
    datamtu = PMTU(*(short *)c->client->IPXaddress.socket);
    if (fmtoutsize > 0) datamtu -= datamtu%fmtoutsize;
    c->status.mtu = datamtu;
    c->status.nextDataSize = c->status.BytesRemaining;
    if (c->status.BytesRemaining > c->status.mtu) c->status.nextDataSize = c->status.mtu;
    c->status.numblks = doutsize/c->status.mtu +
                        (doutsize%c->status.mtu ? 1 : 0);
    if (doutsize == 0) c->status.numblks = 1;
    renewalMultiplier = (regEx->PollingRate < 500) ? 3 : 1;
    if (c->status.mode != ACK_PENDING)         /* Force Contract execution */
    {
      if ((c->status.counter == 0 && regEx->starttime != BCAST_ID) || regEx->mode == CM_REFRESH)
      {
        c->status.Stale = 1;                     
        c->status.lasttime = gThisCycleTime;
      }
      if (c->status.mode != CM_CANCEL && c->status.mode != CM_REFRESH &&
          c->status.counter > ACK_REQUEST*renewalMultiplier &&
          c->status.counter < RENEWAL*renewalMultiplier)
      {
        if (debug > 1)
        {
          wsprintf(txtbuf,"reconnecting to %s %s %s from %s (+%d)",
            regEx->contract.EqpName,regEx->contract.EqpDeviceName,
            regEx->contract.EqpProperty,h->UserName,c->status.counter);
          output(txtbuf);
        }
        c->status.reconnects++;
        gClientReconnects++;
      }
      StaleData++;
    }
    if (regEx->mode & CM_RETRY)
    {
      if (debug > 1)
      {
        wsprintf(txtbuf,"retrying %s %s %s by %s",
          regEx->contract.EqpName,regEx->contract.EqpDeviceName,
          regEx->contract.EqpProperty,h->UserName);
        output(txtbuf);
      }
      c->status.Stale = 1;                     
      StaleData++;   /* if expired, DeliverData() will send the last payload again */
      c->status.isRetry = TRUE;
      gClientRetries++;
      regEx->mode = BASEMODE(regEx->mode);
    }
    switch (c->status.mode = regEx->mode)
    {
      case CM_SINGLE :
        gSingleLinkCount++;
        c->status.counter = 1;
        if (!(ContractList[j]->expired && c->status.isRetry))
          ContractList[j]->lasttime.tv_sec = PRP_RETRY_SIGNAL;  /* a RETRY marked as expired should not be called */
        else
          ContractList[j]->lasttime.tv_sec = time(NULL);
        if (ContractList[j]->PollingRate == 0) ContractList[j]->PollingRate = c->status.PollingRate;
      	if (ContractList[j]->nconsumer == 1)
        {
          ContractList[j]->contract.EqpAccess |= CA_LAST; /* turn on LAST bit */
        }
        break;
      case CM_POLL   :
      case CM_REFRESH:
        c->status.counter = RENEWAL * renewalMultiplier;
        ContractList[j]->PollingRate = findpoll(regEx->PollingRate,ContractList[j]->PollingRate);
        break;
      default     :
      case CM_CANCEL :
        if (!strcmp(cln->userName,"NETWORK")) break;
        if (ContractList[j]->nconsumer == 1 && regEx->starttime != BCAST_ID)
        {
          if (c->status.counter)                     /* was active */
            ContractList[j]->contract.EqpAccess |= CA_LAST; /* turn on LAST bit */
          ContractList[j]->PollingRate = MaxPollingRate;
          ContractList[j]->lasttime.tv_sec = time(NULL);
        }
        break;
    }
    if (debug)
    {
      wsprintf(txtbuf,"%s -> %s %s %d ms [%s]\n",
         (char *)cln->userName,(char *)ContractList[j]->contract.EqpName,
         (char *)ContractList[j]->contract.EqpProperty,c->status.PollingRate,
         (char *)modstr[c->status.mode]);
      output(txtbuf);
    }
err:if (cc)
    {
      SendMessageToCaller(cln,regEx,cc,NULL,0);
      StaleData++;                                          /* force pass thru DeliverData */
      if (j >= 0) RemoveContract(j);
      cc = 0;
    }
    else
    {
      SysPoll = findpoll(SysPoll,ContractList[j]->PollingRate);
    }
    for (i=0; i<MAXLONGGRAMS; i++) if (cp[i].busy == 2)
    {
      SystemFree(cp[i].data);
      memset(&cp[i],0,sizeof(ContractProgress));
    }
bow:if ((bytesread += regEx->msgsize) < totalsize) PrepSubHeader(prot,&buf[bytesread]);
  }                                                             /* continue unpacking  */
/*
  if (reallocsize > WorkAreaSize)
  {
    if ((tmpWorkArea=(BYTE *)realloc((char *)WorkArea,(size_t)reallocsize)) == NULL) ccerr(out_of_server_memory);
    WorkArea = tmpWorkArea;
    WorkAreaSize = reallocsize;
  }
*/
out:
  if (cc) SendMessageToCaller(cln,regEx,cc,NULL,0);
  return cc;
}

int SendIPBroadcast(BYTE *buf,int size)
{
  int cc=0,n, nfds;
  char *c;
  struct fd_set fdwset;
  struct timeval wait_to;
  struct sockaddr *sa;

  for (n=0; n<NrOfIPBcastNets+1; n++)
  {
    if (debug) 
    {
      if (n == 0) strcpy(txtbuf,"UDP broadcast : ");
      if (n == NrOfIPBcastNets) strcpy(txtbuf,"UDP multicast : ");
    }
    FD_ZERO(&fdwset); FD_SET(ProducerSocket,&fdwset);
    wait_to.tv_sec=0;  wait_to.tv_usec=0;
    nfds = ProducerSocket + 1;
    switch (select(nfds,NULL,&fdwset,NULL,&wait_to))
    {
      case 1: break;
      case 0:
      default: ccerr(tcp_socket_error);
    }
    sa = n < NrOfIPBcastNets ? (struct sockaddr *)&IPBcastList[n] : (struct sockaddr *)&tGCastAddrSck;
    if (sendto(ProducerSocket,(char *)buf,size,0,sa,sizeof(struct sockaddr_in)) < 0)
    {
      ccerr(tcp_socket_error);
    }
err:
    if (debug)
    {
      if (cc) strcat(txtbuf,erlst[cc]);
      else
      {
        if ((c=inet_ntoa(IPBcastList[n].sin_addr)) != NULL) strcpy(clientaddr,c);
        strcat(txtbuf,clientaddr);
        output(txtbuf);
      }
    }
  }
  return cc;
}
void doBroadcast(void)
{
  BYTE *buf;
  int n,size=sizeof(HeaderStruct),setsize,BrdcstPacketSize,datasize;
  BrdcstData *brdcstdata;
  GlobalListStruct *bl;
  time_t interim;
  HeaderStruct *brdcstHeader;
  static time_t lasttime = 0;

  if (brdcstList == NULL || nbroadcasts == 0) return;
  interim = time(NULL);
  if (lasttime == interim) return;
  lasttime = interim; 
  buf = WorkArea; BrdcstPacketSize = HEADERSTRUCT_SIZE;
  for (bl=brdcstList, n=0; bl; bl=bl->next)
  {
    if (bl->time > 0 && (bl->minPeriod <= 0 || bl->maxPeriod <= 0)) continue;/* not a scheduled broadcast */
    if (interim < bl->time + bl->minPeriod) continue; /* too soon */
    if (interim < bl->time + bl->maxPeriod)   /* within heartbeat */
    {
      if (!datacmp(bl->olddata,bl->data,bl->size,bl->format,bl->tolerance)) continue;
    }
    datasize = bl->size*fmtsizeof(bl->format);
    setsize = BRDCSTDATA_HEADER_SIZE + datasize;
    if (BrdcstPacketSize + setsize > BRDCST_BUFFER_SIZE)
    {                                          /* send what's there  */
      brdcstHeader = (HeaderStruct *)buf;      /* fill in the header */
      brdcstHeader->EqpSize = n;               /* number of keywords */
      strncpy(brdcstHeader->UserName,gFecName,16);
      brdcstHeader->Protocol = 3;              /* broadcast protocol */
      brdcstHeader->TotalSize = BrdcstPacketSize;
      SendIPBroadcast(buf,BrdcstPacketSize);
      BrdcstPacketSize = HEADERSTRUCT_SIZE;
      n = 0;
    }
    brdcstdata = (BrdcstData *)&buf[BrdcstPacketSize];
    strncpy(brdcstdata->name,bl->keyword,16);
    brdcstdata->size = bl->size;
    brdcstdata->format = bl->format;
    brdcstdata->time = bl->time = interim;
    BrdcstPacketSize += BRDCSTDATA_HEADER_SIZE;
    if (bl->data) 
    {
      memcpy(bl->olddata,bl->data,datasize);
      memcpy(&buf[BrdcstPacketSize],bl->data,datasize);
    }
    BrdcstPacketSize += datasize; n++;
  }
  if (n > 0)                                 /* something left     */
  {
    brdcstHeader = (HeaderStruct *)buf;      /* fill in the header */
    brdcstHeader->EqpSize = n;               /* number of keywords */
    strncpy(brdcstHeader->UserName,gFecName,16);
    brdcstHeader->Protocol = 3;              /* broadcast protocol */
    brdcstHeader->TotalSize = BrdcstPacketSize;
    SendIPBroadcast(buf,BrdcstPacketSize);
  }
  return;
}
int SendLCLData(CLN *cln,BYTE *data)
{
  char shmTag[64];
  HANDLE hShm = 0;
  int msgsize = *(short *)data;
  if (msgsize > SHM_BUFFER_SIZE) return use_stream_transport;
  sprintf(shmTag,"_shmTData:%d",cln->sck);
  return shmSndDataGram(shmTag,data,msgsize,&hShm);
}
int SendIPData(CLN *cln,BYTE *data)
{
  fd_set fdwset;
  struct timeval wait_to;
  int msgsize = *(short *)data;
  int sck, cc = 0,retry = 0, nfds;
  
  if (!TCPloaded) return tcp_not_supported;
  if (msgsize > UDP_BUFFER_SIZE) return use_stream_transport;
  sck = cln->sck;
SendUDPretry:
  FD_ZERO(&fdwset); FD_SET(sck,&fdwset);
  wait_to.tv_sec=0;  wait_to.tv_usec=1000;
  nfds = sck+1;
  if (select(nfds,NULL,&fdwset,NULL,&wait_to) != 1)
  {
    if (retry++ < 5)
    {
      if (cln->transport == TCP) Sleep(1);
      goto SendUDPretry;
    }
    if (cln->transport == TCP) CloseConnectionSocket(sck);
    cln->sck = 0;
    ccerr(cannot_select);		
  }
  if (sendto(sck,(char *)data,msgsize,0,
             (struct sockaddr *)&cln->IPaddress,sizeof(cln->IPaddress)) < 0)
  {
    if (retry++ < 5)
    {
      if (cln->transport == TCP) Sleep(1);
      goto SendUDPretry;
    }
    if (cln->transport == TCP) CloseConnectionSocket(sck);
    cln->sck = 0;
    ccerr(net_write_error);
  }
err:
  if (debug)
  {
    wsprintf(txtbuf,"%s: %d bytes %s\n",
      cln->transport == TCP ? "TCP" : "UDP",
      msgsize, 
      cc ? erlst[cc] : "sent");
    output(txtbuf);
  }
  return cc;
}
short tUDPCall(int fecindex,BYTE *buf,int timout);
/* trivial RPC call: */
int tRPC(int fecindex,char *data,char *name,short EqNo,char *prop,
	   short size,short format,short timout)
{
  static char dev[16];
  sprintf(dev,"#%d",EqNo);
  return tExecLink(fecindex,name,dev,prop,size,data,format,timout);
}
int tExecLink(int fecindex,char *name,char *dev,char *prop,
	   short size,char *data,short format,short timout)
{
  int fmtsize,retry = 2;
  short cc=0,EqNo,dsize,sstepsize,pstepsize;
  static BYTE lclWorkArea[IPX_BUFFER_SIZE];
  static short NameServerProtocol = 5;
  HeaderStruct *h = (HeaderStruct *)lclWorkArea;
  struct subscription4 *s = (struct subscription4 *)&lclWorkArea[HEADERSTRUCT_SIZE];
  struct subscription *sEx = (struct subscription *)&lclWorkArea[HEADERSTRUCT_SIZE];
  ProducerHeaderStruct4 *Pheader;
  ProducerHeaderStruct PheaderTmp, *PheaderEx = &PheaderTmp;
  short sizeIn,sizeOut,formatIn,formatOut,access;
  char devname[16];

  if (debug)
  {
    wsprintf(txtbuf,"trivial RPC: %s %s %s\n",name,dev,prop);
    output(txtbuf);
  }
  format = format ? format : CF_DOUBLE;
  fmtsize = fmtsizeof(format);
  if (!fmtsize || strlen(name) > 15 || strlen(prop) > 12) return string_too_long;
  if ((dsize = abs(size)*fmtsize) < 1) return out_of_range;
  if ((HEADERSTRUCT_SIZE+SUBSCRIPTION4_SIZE+dsize) > IPX_BUFFER_SIZE) return use_stream_transport;
  if (timout < 1) timout = 1;
  EqNo = dev[0] == '#' ? atoi(&dev[1]) : 0;
  sprintf(devname,"#%d",EqNo);

  if (size < 0)
  {
    sizeIn = -size; formatIn = format;
    sizeOut = -size; formatOut = format;
    access = CA_WRITE|CA_READ;
  }
  else
  {
    sizeIn = 0; formatIn = CF_NULL;
    sizeOut = size; formatOut = format;
    access = CA_READ;
  }

  if (NameServerProtocol == 5)
  {
    sstepsize = SUBSCRIPTION_SIZE;
    pstepsize = PRODUCER_SIZE;
  }
  else
  {
    sstepsize = SUBSCRIPTION4_SIZE;
    pstepsize = PRODUCER4_SIZE;
  }
  memset(h,0,sizeof(HeaderStruct));

tRETRY:
  RPCprotocol = NameServerProtocol;
  AssignOutgoingSubscription(NameServerProtocol,&lclWorkArea[HEADERSTRUCT_SIZE]);
  if (size < 0)
  {
    memcpy(&lclWorkArea[HEADERSTRUCT_SIZE+sstepsize],data,dsize);
    s->msgsize = sstepsize + dsize;
  }
  else s->msgsize = sstepsize;

  h->TotalSize = HEADERSTRUCT_SIZE + s->msgsize;
  memcpy(h->UserName,gFecName,16);
  h->Protocol = RPCprotocol;
  h->EqpSize = 1;

  switch (cc = tUDPCall(fecindex,lclWorkArea,timout))
  {
    case 0: break;
    case link_not_open:
      if (--retry) goto tRETRY;
      return cc;
    case illegal_protocol:
      if (NameServerProtocol == 5)
      {
        NameServerProtocol = 4;
        sstepsize = SUBSCRIPTION4_SIZE;
        pstepsize = PRODUCER4_SIZE;
        goto tRETRY;
      }
      return cc;
    default:
      return cc;
  }

  PrepIncomingSubscription(NameServerProtocol,&lclWorkArea[2]);
  if (PheaderEx->SubscriptionID != 0) return connection_timeout;
  if ((cc=PheaderEx->CompletionCode) == 0)
  {
    if (dsize != PheaderEx->msgsize-pstepsize) return illegal_format;
    else memcpy(data,&lclWorkArea[2+pstepsize],dsize);
  }
  return cc;
}

/* trivial RPC call: */
short tUDPCall(int fecindex,BYTE *buf,int timout)
{
  static struct fd_set tset;
  static struct timeval t_to;
  static struct sockaddr_in IPfec;
  static CLN cln;
  static struct sockaddr_in prod_name;
  int msgsize = *(unsigned short *)buf;
  int n,len, nfds;
  RPCFecStruct *rpcfec = &NameServer;

  if (!TCPloaded) return tcp_not_supported;

  memset(&cln,0,sizeof(CLN));
  strncpy(cln.userName,rpcfec->Name,16);
  if (!rpcfec->IPh_addr) return non_existent_fec;
  memset(&prod_name,0,sizeof(prod_name));
  memcpy(&prod_name.sin_addr,&rpcfec->IPh_addr,sizeof(struct in_addr));
  prod_name.sin_family=AF_INET;
  prod_name.sin_port = htons((UINT16)gtUDPPort);
  memcpy(&cln.IPaddress,&prod_name,sizeof(prod_name));

  if (sendto(SyncSocket,(char *)buf,msgsize,0,
	    (struct sockaddr *)&cln.IPaddress,sizeof(cln.IPaddress)) < 0)
    return tcp_socket_error;
  if (debug)
  {
    wsprintf(txtbuf,"tUDP: %d bytes sent\n",msgsize);
    output(txtbuf);
  }

  FD_ZERO(&tset); FD_SET(SyncSocket,&tset);
  t_to.tv_sec=timout; t_to.tv_usec=0; len = sizeof(IPfec);
  nfds = SyncSocket + 1;
  switch (select(nfds,&tset,NULL,NULL,&t_to))
  {
    case -1: return tcp_socket_error;
    case 0:  return link_not_open;
    default: n = recvfrom(SyncSocket,(char *)buf,IPX_BUFFER_SIZE,0,
			     	      (struct sockaddr *)&IPfec,&len);
  }
  if (n != *(short *)buf) return remitted_data_lost;
  return 0;
}
