/** \file winsrv.c
 * \brief TINE Server DLL documentation 
 *
 */
#include "project.def"
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys\timeb.h>
#include <signal.h>
#include <io.h>
#include <winsock.h>
#include "prolog.h"
#include "errors.h"
#include "database.h"
#include "winsrv.h"
#include "almsrv.h"
#include "history.h"
#include "shmalloc.h"

__declspec(dllimport) int GetNetGlobal(char *keyword, void *buf, short size, short format);
__declspec(dllimport) int GetNetGlobalEx(char *keyword, void *buf, short size, short format, double *stale);
__declspec(dllimport) short SetUserName(char *uname);

#ifndef STD_TIME_STR
# define STD_TIME_STR "CET"
#endif
#ifndef DST_TIME_STR
# define DST_TIME_STR "CDT"
#endif
#ifndef STD_TIMEZONE_STR
# define STD_TIMEZONE_STR "TZ=GMT-1GDT"
#endif
#ifndef DLT_TIMEZONE_STR
# define DLT_TIMEZONE_STR "TZ=GMT-2GDT"
#endif
#ifndef GLOBAL_SYNCHRONIZATION
# define GLOBAL_SYNCHRONIZATION TRUE
#endif
char *ltzname[2] = { STD_TIME_STR, DST_TIME_STR };
int useGlobalSynchronization = GLOBAL_SYNCHRONIZATION;
double gSyncTimestamp = 0;
double gSyncDeadband = 800;
int SystemPresetMemory = FALSE;
int leftFeclogTime = FALSE;
char MSGLastMessageString[256] = "";
int isNewMessage = 0;

int getIPControlNets(char *fn,int *nr,struct sockaddr_in **list);
void getDatabasePaths(void);
void initUserList(void);
int initControlStructs(void);
void doBroadcast(void);
void flushTCPBuckets(void);
int isAbsolutePath(char *fname);

extern int NrOfIPNets;
extern int NrOfIPBcastNets;
extern struct sockaddr_in *IPNetList;
extern struct sockaddr_in *IPBcastList;
extern int nbroadcasts;
extern int ConsumerListArraySize,ContractListArraySize;
extern int putCommandsInFeclog;
CLN IncomingTCPClient;

short initAlarmServer(void);
extern unsigned short MinPollingRate;

extern short NativeFormatIn,NativeFormatOut;
extern CONSUMER *EQPconsumerlist;                      /* global reference  */
// instance handle of DLL:
HANDLE hWinSrv;
HANDLE hDLLWnd = hNULL;
// statistics counters:
unsigned long TCPcount=0,UDPcount=0,SPXcount=0,IPXcount=0,BRDcount=0;
int TCPerrcnt=0,TCPcumcnt=0;
int TCPParent=0,TCPloaded=0;
int TCPProducerSocket = 0;
int FecNameResolved = 0;
struct fd_set TCPrset, rwset, exset;
struct timeval TCPto, fec_to;
extern struct fd_set UDPset;
extern struct timeval UDPto;
struct sockaddr TCPaddr[NumTCPChan];
int TCPsocket[NumTCPChan],
    IPaddrsize = sizeof(struct sockaddr);

int SyncSocket = 0;
int nUDPsets;
int UDPstranded = 0,TCPstranded = 0;

int WinSrvTimer(void);
int TimerID=1;
time_t startuptime = 0;
// WRITE access:
UserListStruct *UserList = NULL;
UserListStruct *DeniedUserList = NULL;
int NrOfUsers = 0;
int NrOfDeniedUsers = 0;
int openToAllUsers = 1;
int accessToAllUsers = 0;

PropertyListStruct *PropertyList = NULL;
int NrOfProperties = 0;

int AveBusyTime = 0;
UINT32 gContractMisses = 0;
UINT32 gContractDelays = 0;
UINT32 gClientMisses = 0;
UINT32 gClientReconnects = 0;
UINT32 gClientRetries = 0;
UINT32 gBurstLimitReachedCount = 0;
UINT32 gSingleLinkCount = 0;
int gCycleCounts = 0, gMaxCycleCounts = 1;
int gRetardSingleContractRemoval = TRUE;
UINT32 gTimeServerCycle = 0;
UINT32 gTimeServerIdleLastCycle = 0;
struct timeval gThisCycleTime;

// Registered equipment functions:
int SizeInBytes=0;
int initflag=0;
HCURSOR WaitCurs;

BYTE *Values;
BYTE *WorkArea;
unsigned long WorkAreaSize = WORK_SIZE;
int	WorkAreaBusy = 0; // WorkArea semaphore
extern int isHistoryCall;
int debug = 0;
time_t debugStoptime = 0;
__declspec(dllexport) short PASCAL DebugServer(short level);
void SetRPCCompletion(char *errstr);
int eqpsize;
BYTE *DataBuffer;
int RPCprotocol,TotalMsgSize;
int msgsize = 2;
double gDataTimeStamp;
double gDataTimeStampOffset = 0;
char clientName[17],clientaddr[30],msgbuf[256];
extern char txtbuf[128];
char txtbuffer[BUFLEN+26]; // make sure the last ctime() fits

int nofeclog = TRUE;
int FeclogDepth = NLINES;

struct sockaddr_in tGCastAddrSck;
struct sockaddr_in tMCastAddrSck;
struct in_addr tMCastAddr;
struct in_addr tGCastAddr;

int gSystemDelay = SYSTEM_DELAY;
int gBurstLimit = BURST_LIMIT;
int gCycleDelay = DEFAULT_CYCLE_DELAY;

time_t appmktime = 0;
char appmkversion[64];

int findDaylight(void)
{
  int dl;
  time_t t = time(NULL);
  struct tm *tm = localtime(&t);

  if      (tm->tm_mon<2 || tm->tm_mon>9)       dl = 0;  /* winter */
  else if (tm->tm_mon>2 && tm->tm_mon<9)       dl = 1;  /* summer */
  else if (tm->tm_mon==2)                               /* March */
  { if      (tm->tm_mday < 25)                  dl = 0;
    else if (tm->tm_wday > tm->tm_mday-25)      dl = 0;
    else if (tm->tm_wday == 0 && tm->tm_hour<2) dl = 0; /* Sunday */
    else dl = 1;
  }
  else                                                  /* October */
  { if      (tm->tm_mday < 25)                  dl = 1;
    else if (tm->tm_wday > tm->tm_mday-25)      dl = 1;
    else if (tm->tm_wday == 0 && tm->tm_hour<2) dl = 1; /* Sunday */
    else dl = 0;
  }
  return dl;
}
char *getDataTimeString(double ts,int useLongStringFormat)
{
  static char tsstr[64];
  time_t t = (time_t)ts;
  struct tm *tms = localtime(&t);
  UINT32 ms = (UINT32)((ts-t+0.0005) * 1000.0);
  int isdst = tms->tm_isdst > 0 ? 1 : 0;

  if (tms->tm_isdst < 0) isdst = findDaylight();
  if (useLongStringFormat)
  {
    strncpy(tsstr,ctime(&t),24);
    strncpy(&tsstr[28],&tsstr[20],4); tsstr[32] = 0;
    sprintf(&tsstr[19],".%03d",ms);
    sprintf(&tsstr[23]," %s",ltzname[isdst]);
    tsstr[27] = ' ';
  }
  else
  {
    sprintf(tsstr,"%02d.%02d.%02d %02d:%02d:%02d.%03ld %s",
      tms->tm_mday,tms->tm_mon+1,tms->tm_year%100,
      tms->tm_hour,tms->tm_min,tms->tm_sec,
      ms,ltzname[isdst]);
  }
  return tsstr;
}
struct timeval *gettimeofday(struct timeval *t,struct timezone *tz);
struct timeval *getDataTimeStampAsTimeval(void)
{
  static struct timeval tv;
  tv.tv_sec = RPCtimestamp;
  tv.tv_usec = (RPCtimestampMSEC) * 1000;
  return &tv;
}
__declspec(dllexport) void PASCAL setRetardContractRemoval(int value)
{
  gRetardSingleContractRemoval = value;
}
__declspec(dllexport) void PASCAL setLeftFeclogTime(int value)
{
  leftFeclogTime = value;
}
__declspec(dllexport) void PASCAL setGlobalSynchronization(int value)
{
  useGlobalSynchronization = value;
}
__declspec(dllexport) void PASCAL setSystemPresetMemory(int value)
{
  SystemPresetMemory = value;
}
__declspec(dllexport) double PASCAL getDataTimeStamp(void)
{
  return gDataTimeStamp;
}
__declspec(dllexport) void PASCAL setDataTimeStamp(double ts)
{
  gDataTimeStamp = ts;
}
double putDataTimeStamp(double toffset, time_t tsec, int tmsec)
{
  double ts;
  ts = (double)tsec;
  ts += (double)tmsec/1000.0;
  ts += toffset;
  return ts;
}
__declspec(dllexport) double PASCAL makeDataTimeStamp(void)
{
  double ts;
  static struct timeb tb;
  ftime(&tb);
  ts = (double)tb.time;
  ts += (double)(tb.millitm)/1000.0;
  ts += gDataTimeStampOffset;
  return ts;
}
__declspec(dllexport) int PASCAL SetCycleDelay(int msecs)
{
  gCycleDelay = msecs;
  return gCycleDelay;
}
__declspec(dllexport) int PASCAL SetSystemDelay(int msecs)
{
  if (msecs > 20) gSystemDelay = msecs;
  return gSystemDelay;
}
__declspec(dllexport) int PASCAL SetBurstLimit(int npackets)
{
  if (npackets > 20) gBurstLimit = npackets;
  if (gBurstLimit < 30) gCycleDelay = 5;
  else if (gBurstLimit < 100) gCycleDelay = 1;
  else gCycleDelay = 0;
  return gBurstLimit;
}
__declspec(dllexport) short PASCAL setFecLogDepth(short nlines)
{
  if (nlines > 1) FeclogDepth = nlines;
  return nlines;
}
__declspec(dllexport) short PASCAL feclog(char *text)
{
  int cc=0,len,pos=0;
  FILE *fp = NULL, *fpb = NULL;
  double ts = makeDataTimeStamp();
  char TmpPath[128], str[256];
  static int flen = -1;

  if (nofeclog) ccerr(0);
  if (text == NULL || (len=strlen(text)) == 0) ccerr(0);
  if (text[len-1] == '\n') text[len-1] = 0;
  FecPath[0] = 0; strcat(FecPath,gFeclogPath); strcat(FecPath,"fec.log");
  if ((fp=fopen(FecPath,"a+t")) == NULL) ccerr(file_error);
  sprintf(str,"%s[%s] %s\n",getDataTimeString(ts,0),gFecName,text);
  fwrite(str,strlen(str),1,fp);
  if (flen == -1) flen = ftell(fp);
  if ((flen += len) < FeclogDepth*2*80) ccerr(0);

  fclose(fp); fp = NULL; flen = 0;
  TmpPath[0] = 0; strcat(TmpPath,gFeclogPath); strcat(TmpPath,"fec.bak");
  remove(TmpPath);
  rename(FecPath,TmpPath);
err:
  if (fp) fclose(fp);
  return cc;
}
__declspec(dllexport) short PASCAL appendLog(char *logfilename, char *process, char *text)
{
  int cc=0,len,pos=0,useAbsolutePath=0;
  FILE *fp = NULL, *fpb = NULL;
  double ts = makeDataTimeStamp();
  char *c, TmpPath[128], str[256];
  static int flen = -1;

  if (nofeclog) ccerr(0);
  if (text == NULL || (len=strlen(text)) == 0) ccerr(0);
  if (text[len-1] == '\n') text[len-1] = 0;
  FecPath[0] = 0;
  useAbsolutePath = isAbsolutePath(logfilename);
  if (useAbsolutePath) 
  {
    strncpy(FecPath,logfilename,128); 
  }
  else 
  {
    strcat(FecPath,gFeclogPath); strcat(FecPath,logfilename);
  }
  if ((fp=fopen(FecPath,"a+t")) == NULL) ccerr(file_error);
  sprintf(str,"%s[%s] %s\n",getDataTimeString(ts,0), process == NULL ? "logger" : process, text);
  fwrite(str,strlen(str),1,fp);
  if (flen == -1) flen = ftell(fp);
  if ((flen += len) < FeclogDepth*2*80) ccerr(0);
  fclose(fp); fp = NULL; flen = 0;
  // rotate once to .bak file ...
  strncpy(TmpPath,FecPath,128);
  if ((c=strstr(TmpPath,".log")) != NULL) *c = 0;
  strcat(TmpPath,".bak");
  remove(TmpPath);
  rename(FecPath,TmpPath);
err:
  if (fp) fclose(fp);
  return cc;
}
#define NotALogFile(f) (\
  strstr(f,".log") == NULL &&\
  strstr(f,".txt") == NULL &&\
  strstr(f,".ini") == NULL &&\
  strstr(f,".dat") == NULL &&\
  strstr(f,".cfg") == NULL &&\
  strstr(f,".csv") == NULL)
int isAbsolutePath(char *fname)
{
  if (fname == NULL || strlen(fname) < 2) return FALSE;
  if (fname[1] == ':') return TRUE;
  if (fname[0] == '\\') return TRUE;
  if (fname[0] == '/') return TRUE;
  return FALSE;
}
void applyFecLogPath(char *path,int pathlen)
{
  char root[256];
  int len = strlen(gFeclogPath);
  if (len == 0) return; /* nothing to apply */
  if (path == NULL) return;
  if (isAbsolutePath(path)) return; /* nothing to apply */
  if (len + (int)strlen(path) > pathlen) return;
  strncpy(root,gFeclogPath,80);
  strncat(root,path,128);
  strncpy(path,root,256); /* copy it back */
  return;
}
#define MAX_FILE_SCAN_LIMIT 100
int scanFecLogFiles(char *path,char *subpath,NAME64 *namebuf,int *numfiles,int numoffset)
{
  int n = numoffset, nread = 0;
  char fn[128],fnp[128],tfn[32],spath[64];
  struct _finddata_t fp;
  long hfp;
  static int scancounter = 0;

  if (subpath == NULL) scancounter = 0;
  if (strlen(path) > 128) return 0;
  if (namebuf == NULL || numfiles == NULL || *numfiles == 0) return argument_list_error;
  if (strlen(path) > 0 && path[strlen(path)-1] != '\\') strcat(path,"\\");
  sprintf(fn,"%s*.*",path);
  if ((hfp=_findfirst(fn, &fp)) == -1) return no_such_file;
  do
  {
    if (scancounter++ > MAX_FILE_SCAN_LIMIT) break;
    if (!strcmp(fp.name,".") || !strcmp(fp.name,"..")) continue;
    if (fp.attrib & _A_SUBDIR) 
    {
      if (subpath != NULL) strncpy(spath,subpath,60); else spath[0] = 0;
      strncpy(fnp,path,128);
      if (strlen(fnp) + strlen(fp.name) > 128) continue;
      if (strlen(spath) + strlen(fp.name) > 60) continue;
      strcat(fnp,fp.name); strcat(fnp,"\\");
      strcat(spath,fp.name); strcat(spath,"\\");
      nread = *numfiles;
      scanFecLogFiles(fnp,spath,namebuf,&nread,n);
      n = nread;
    }
    strncpy(tfn,fp.name,32); strlwr(tfn);
    if (NotALogFile(tfn)) continue;
    if (n >= *numfiles) break;
    if (subpath != NULL && strlen(subpath) + strlen(fp.name) < 64)
    {
      sprintf(namebuf[n++].name,"%s%s",subpath,fp.name);
    }
    else
    {
      strncpy(namebuf[n++].name,fp.name,32);
    }
  } while (_findnext(hfp,&fp) != -1);
  *numfiles = n;
  return 0;
}
int readFecLog(char *textbuf,int sizeInBytes)
{
 return readFecLogFile("fec.log",textbuf,sizeInBytes);
}
int readFecLogFile(char *fname,char *textbuf,int sizeInBytes)
{
  int cc=0,sz=0,szb=0,pos=0,szt=0,useAbsolutePath=0;
  FILE *fp = NULL, *fpb = NULL;
  char TmpPath[128],bakname[96], *c;

  if (nofeclog) return 0;
  if (fname == NULL) return no_such_file;
  FecPath[0] = 0; 
  useAbsolutePath = isAbsolutePath(fname);
  if (useAbsolutePath) 
  {
    strncpy(FecPath,fname,128); 
  }
  else 
  {
    strcat(FecPath,gFeclogPath); strcat(FecPath,fname);
  }
  if ((fp=fopen(FecPath,"r+t")) == NULL) goto checkbak;
  fseek(fp,0,SEEK_END); sz = ftell(fp);
checkbak:
  if (sz < sizeInBytes)  /* look for back file */
  {
    szt = 0;
    strncpy(bakname,fname,95); bakname[95] = 0;
    if ((c=strstr(bakname,".log")) != NULL) *c = 0;
    strcat(bakname,".bak");
    TmpPath[0] = 0; 
    if (useAbsolutePath) 
    {
      strncpy(TmpPath,bakname,128); 
    }
    else 
    {
      strcat(TmpPath,gFeclogPath); strcat(TmpPath,bakname);
    }
    if ((fpb=fopen(TmpPath,"r")) == NULL) goto filesread;
    szt = sizeInBytes - sz;
    fseek(fpb,0,SEEK_END); szb = ftell(fpb);
    if (szb > szt) pos = szb - szt;
    fseek(fpb,pos,SEEK_SET);
    szt = fread(textbuf,1,szt,fpb);
    pos = 0;
    fclose(fpb);
  }
filesread:
  if (fp)
  {
    if (sz > sizeInBytes) pos = sz - sizeInBytes;
    fseek(fp,pos,SEEK_SET);
    szb = fread(&textbuf[szt],1,sizeInBytes-szt,fp);
    fclose(fp);
    textbuf[szt+szb] = 0;
  }
  if (sz == 0 && szb == 0) cc = no_such_file;
  return cc;
}
int readSelfTest(char *eqm,char *textbuf,int sizeInBytes)
{
  int cc=0,sz=0;
  FILE *fp = NULL;
  char fn[96];

  textbuf[0] = 0;
  sprintf(fn,"%s%s-selftest.csv",FecDBpath,eqm);
  if ((fp=fopen(fn,"r+t")) == NULL) ccerr(no_such_file);
  sz = fread(textbuf,1,sizeInBytes,fp);
  fclose(fp);
err:
  return sz;
}

void output(char *txt)
{
  static int initialized = 0;
  static HANDLE hDC;
  static RECT Rect;
  static int buflen = 0;
  static time_t t;
  int len;
  char *c;

  if ((len=strlen(txt)) == 0) return;
  while ((buflen=strlen(txtbuffer)) + len > BUFLEN)
  {
    if ((c=strchr(txtbuffer,'\n')) == NULL) return;
    strcpy(txtbuffer,++c);
  }
  strcat(txtbuffer,txt);

  if (txtbuffer[strlen(txtbuffer)-1] != '\n') return;
  if (debug > 4) OutputToDebugFile("output",txt);
  txtbuffer[strlen(txtbuffer)-1] = ' ';
  time(&t);
  strcat(txtbuffer,(char *)ctime(&t));
  if (!IsWindowVisible(hDLLWnd)) return;
  if (!initialized)
  {
    initialized = 1;
    hDC = GetDC(hDLLWnd);
    GetClientRect(hDLLWnd, (LPRECT) &Rect);
  }
  FillRect(hDC,&Rect,GetStockObject(WHITE_BRUSH));
  DrawText(hDC,txtbuffer,-1,&Rect,DT_LEFT);
}
char szDebugFile[82];
char szDebugFileBak[82];
int debugTargetError = 0;
int debugTargetErrorMet = 0;
int gMaxDebugLines = 5000;
void OutputToDebugFile(char *routine, char *txt)
{
  char outputbuffer[512], timestr[48];
  struct timeval tv;
  int len;
  static FILE *fpDebugFile = NULL;
  static int filecreated = 0;
  static int lineswritten = 0;

  if (szDebugFile[0] == 0) return;
  gettimeofday(&tv,(struct timezone *)NULL);
  if (!filecreated)
  {
    if ((fpDebugFile=fopen(szDebugFile,"w")) == NULL) return;
    filecreated = -1;
  }
  else
  {
    if ((fpDebugFile=fopen(szDebugFile,"a")) == NULL) return;
  }
  sprintf(outputbuffer,"%s : %s",routine,txt);
  if (outputbuffer[len=(strlen(outputbuffer)-1)] == '\n') outputbuffer[len] = ' ';
  else strcat(outputbuffer," ");
  strncpy(timestr,ctime(&tv.tv_sec),19); 
  sprintf(&timestr[19],".%03d\n",tv.tv_usec/1000);
  strcat(outputbuffer,&timestr[4]);
  fprintf(fpDebugFile,outputbuffer);
  fclose(fpDebugFile); fpDebugFile = NULL;
  lineswritten++;

  if (lineswritten > gMaxDebugLines)
  {
    lineswritten = 0; filecreated = 0;
    remove(szDebugFileBak);
    rename(szDebugFile,szDebugFileBak);
  }
}
int setDebugFromEnvironment(void)
{
  char *ptr;
  int dbglevel = 0;
  if  ((ptr=getenv("TINE_DEBUG_LEVEL")) != NULL) 
  {
    dbglevel = atoi(ptr);
  }
  if  ((ptr=getenv("TINE_DEBUG_TARGET_ERROR")) != NULL) 
  {
    debugTargetError = atoi(ptr);
  }
  return dbglevel;
}
void GetDebugFilePath(void)
{
  char *ptr;
  if      ((ptr=getenv("SRVDEBUGFILE")) != NULL) strncpy(szDebugFile,ptr,80);
  else if ((ptr=getenv("TEMP")) != NULL) 
  {
    strncpy(szDebugFile,ptr,68); 
    strncpy(szDebugFileBak,ptr,68); 
    strcat(szDebugFile,"\\tinesrv.dbg");
    strcat(szDebugFileBak,"\\tinesrv.bak");
  }
  else 
  {
    strcpy(szDebugFile,"C:\\tinesrv.dbg");
    strcpy(szDebugFileBak,"C:\\tinesrv.bak");
  }
}

// WSA cover routines :
void soperror(char *txt)
{
  static char buf[40];
  int cc = WSAGetLastError();
  static int last_cc = 0;
  wsprintf(buf,"%s: error %d (%x)",(char *)txt,cc,cc);
  if (last_cc != cc)
  {
    last_cc = cc;
    dbllog(buf);
  }
  else 
  {
    output(buf);
  }
  return;
}
int WinsockLoaded(void)
{
  static WSADATA wsaData;
  static int WSAinitialized = -1;
  static WORD wVersionRequested;
  char *ptr, str[128];

  if((ptr=getenv("OS")) != NULL && !stricmp(ptr,"Windows_NT"))
  {
    wVersionRequested = MAKEWORD( 2, 0 );
    strcpy(str,"WinSock 2.0");
  }
  else
  {
    wVersionRequested = MAKEWORD( 1, 1 );
    strcpy(str,"WinSock 1.1");
  }
 
  if (WSAinitialized == -1) WSAinitialized = WSAStartup(wVersionRequested,&wsaData) ? 0 : 1;
  if (WSAinitialized ==  0 && debug) soperror("WSAStartup");
  sprintf(&str[strlen(str)]," %s", WSAinitialized ? "initialized" : "failed");
  dbllog(str);
  return WSAinitialized;
}
int GetErrno(void) { return WSAGetLastError();}
int soioctl(int s,long cmd,char *mode)
{
  return ioctlsocket(s,cmd,(u_long *)mode);
}
int soclose(int s) { return closesocket(s); }
int TCPGetResource(void)
{
  int cc;
  static WSADATA wsaData;
  if ((cc=WSAStartup(0x0101,&wsaData)) != 0 && debug) soperror("WSAStartup");
  return cc;
}
int TCPFreeResource(void)
{
  int cc;
  if ((cc=WSACleanup()) != 0 && debug) soperror("WSACleanup");
  return cc;
}

// end WSA cover routines ...

__declspec(dllexport) LRESULT CALLBACK WinSrvWndProc(HWND hWnd, WORD iMessage, WORD wParam, LONG lParam)
{
  int sck,sckevn,sckerr;
  char shmTag[64];
  shmDataGram *cr;
  HANDLE hCr;
  int siz;
  long hClnWnd;
  CLN *cln;
  switch (iMessage)
  {
    case WM_READ_LCL:
      hClnWnd = lParam;
      if ((cln=LocateConsumerInList(NULL,NULL,NULL,&hClnWnd)) == NULL) break;
      sprintf(shmTag,"_shmTContract:%d",lParam);
      cln->transport = LCL;
      cln->sck = lParam;
      siz = wParam;
      cr = (shmDataGram *)shmGetDataGram(shmTag,siz,&hCr);
      strncpy(cln->userName,cr->dataTag,16);
      getConsumerRequest(cln,cr->payload,cr->size);
      cr->dirty = 0;
      shmFreeDataGram(cr,hCr);
      break;
    case WM_READ_UDP: 
      sck = wParam;
      while (GetIPData(sck,UDP) > 0);
      // the following is necessary to recover from breakpoints in VB:
      WSAAsyncSelect(ProducerSocket,hDLLWnd,WM_READ_UDP,FD_READ);
      break;
    case WM_READ_TCP: 
      sck = wParam;
      sckevn = WSAGETSELECTEVENT(lParam);
      sckerr = WSAGETSELECTERROR(lParam);
      if (sckerr != 0 || sckevn == FD_CLOSE) 
      {
        CloseConnectionSocket(sck);
      }
      else 
      {
        while (GetIPData(sck,TCP) > 0);
        WSAAsyncSelect(sck,hDLLWnd,WM_READ_TCP,FD_READ|FD_CLOSE);
      }
      break;
    case WM_CONNECT_TCP: 
      sck = wParam;
      if (sck == TCPProducerSocket) AssignConnectionSocket();
      else if (sck == TCPParent) AcceptTCPStream();
      break;
    case WM_TIMER:       
      WinSrvTimer(); 
      break;
    case WM_DESTROY:     
      KillTimer(hWnd,1);
    default: 
      return DefWindowProc (hWnd, iMessage, wParam, lParam);
  }
  return 0;
}
int CreateDLLWindow(void)
{
  static WNDCLASS wndclass;
  static char  lpszWN[17];

  strncpy(lpszWN,gFecName,16); lpszWN[16] = 0;
  wndclass.style         = CS_GLOBALCLASS|CS_SAVEBITS;  // essential style!!
  wndclass.lpfnWndProc   = (WNDPROC) WinSrvWndProc;
  wndclass.cbClsExtra    = 0;
  wndclass.cbWndExtra    = 0;
  wndclass.hInstance     = hWinSrv;
  wndclass.hIcon         = LoadIcon (hNULL, IDI_APPLICATION);
  wndclass.hCursor       = LoadCursor (hNULL, IDC_ARROW);
  wndclass.hbrBackground = GetStockObject (WHITE_BRUSH);
  wndclass.lpszMenuName  = hNULL;
  wndclass.lpszClassName = lpszWN;

  if (!RegisterClass(&wndclass)) return -1;

  if (!(hDLLWnd = CreateWindow (
			lpszWN, lpszWN,               // lpClassName, lpWindowName
			WS_BORDER|WS_CAPTION,         // dwStyle
			CW_USEDEFAULT, CW_USEDEFAULT, // X, Y
			CW_USEDEFAULT, CW_USEDEFAULT, // nWidth, nHeight
			hNULL, hNULL,                 // hWndParent, hMenu
			hWinSrv,                      // hInstance
			(LPSTR)""                     // createstruct
      ) ) ) return -1;

  return 0;
}

int PASCAL LibMain(HANDLE hInstance, WORD wDataSegment, WORD wHeapSize, LPSTR lpszCmdLine)
{
  char *ptr;

  tzset();
  strcpy(appmkversion,"1.00.0000");
  FecDBpath[0] = 0; gFeclogPath[0] = 0;
  hWinSrv = hInstance;
  if ((ptr=getenv("LOGDEPTH")) != NULL) FeclogDepth = atoi(ptr);
  if (FeclogDepth > 0) nofeclog = FALSE;
  getDatabasePaths();
  GetDebugFilePath();
  debug = setDebugFromEnvironment();
  startuptime = time(NULL);
  MinPollingRate = 200; // this is a test: later let this be set thru API
  memset(&currentAccessLock,0,sizeof(currentAccessLock));
  return 1;   /* indicate that the DLL was initialized successfully */
}
short InitStatus = not_running;
int InitApplication(char *expName,char *lclName,int InitViaDatabase)
{
  int cc = 0;
  char s[128];
  if (initflag) return 0;
  sprintf(s,"FECDB: [%s]",FecDBpath); dbllog(s);
  sprintf(s,"TINE.HOME: [%s]",ControlDBpath); dbllog(s);
  sprintf(s,"VERSION : %d.%02d.%04d",MAJOR_VERSION,MINOR_VERSION,VERSION_REVISION); dbllog(s);
  sprintf(s,"OS : WIN32"); dbllog(s);
  if (!FecNameResolved && (cc=getFecName(expName,lclName)) != 0) 
  {
    dbllog("Couldn't get Server Name (not initializing)");
    return -1;
  }
  if (FecNameResolved) 
  {
    SetUserName(gFecName);
  }
  if (CreateDLLWindow()) { dbllog("CreateDLLWindow failed !"); return InitStatus = code_failure; }
  if ((InitStatus=AllocateAndAssignMemory()) != 0) return InitStatus;
  GetAliasTable();
  if ((cc=initTCP()) != 0) { dbllog("TCP not loaded!"); return InitStatus = tcp_socket_error; }
  initUserList();
  makeStockPropXRefTable();
//  if (InitViaDatabase) getRegisteredExports(gFecName,lclName);
  getRegisteredExports(gFecName,lclName);
//  historyInit(expName);
  initControlStructs();
  initflag = 1; InitStatus = 0;

# ifdef USE_MULTITHREADED_AT_YOUR_OWN_RISK
  if (useMultiThreadedEquipmentFunctions)
  {
    if ((hEqpFcnThread =eqpCreateThread()) == 0) useMultiThreadedEquipmentFunctions = FALSE;
  }
# endif
  dbllog("Server Initialized");
  return 0;
}
short WEP(int bSystemExit)
{
  static char  lpszWN[17];
  feclog("unloading tinesrv32.dll");
  if (initflag) FreeApplication();
  if (hDLLWnd != hNULL)
  {
    strncpy(lpszWN,gFecName,16); lpszWN[16] = 0;
    feclog("destroying tinesrv32 DLL Window");
    DestroyWindow (hDLLWnd);
    UnregisterClass(lpszWN,hWinSrv);
  }
  return 1;
}
BOOL WINAPI DllMain(HINSTANCE hInstDll,DWORD fdwReason,LPVOID lpvReserved)
{
	switch (fdwReason)
	{
	 case DLL_PROCESS_ATTACH:  // DLL being loaded
			LibMain(hInstDll,0,0,0);
			break;
	 case DLL_THREAD_ATTACH:
			break;
	 case DLL_THREAD_DETACH:
			break;
	 case DLL_PROCESS_DETACH:  // DLL being unloaded
		    WEP (0);
			break;
  }
  return (TRUE);
}
void cleanup(void)
{
  ExportListStruct *el;

  CleanupStrandedDatagrams();

  for (el=ExportList; el != NULL; el=el->next)
    if (el->hEqpFcn != hNULL && !IsWindow(el->hEqpFcn)) break;

  if (el == NULL) return;
}
void checkAccessLocks(time_t t)
{
  if (currentAccessLock.lockDuration < 0) return;  /* forever */
  if (t > currentAccessLock.lockStart + currentAccessLock.lockDuration)
    memset(&currentAccessLock,0,sizeof(currentAccessLock));
}
int CycleCounts = 0;
struct timeval lastCycleTime;
void checkSubscriptions(void)
{
  int i;
  if (WorkAreaBusy) return; else WorkAreaBusy = 1;
  gettimeofday(&gThisCycleTime,(struct timezone *)NULL);/* Cycle time */
  lastCycleTime = gThisCycleTime;
  ProduceData();  // Is it time to service consumers ?
  for (i=0; StaleData && i<gBurstLimit; i++) 
  {
    DeliverData();
  }
  if (StaleData)
  {
    if (gCycleDelay) Sleep(gCycleDelay);
    gBurstLimitReachedCount++;
  }
  if (nbroadcasts > 0) doBroadcast();
  WorkAreaBusy = 0;
  gettimeofday(&gThisCycleTime,(struct timezone *)NULL);/* Cycle time */
  gTimeServerCycle += MSECS(gThisCycleTime,lastCycleTime);
  lastCycleTime = gThisCycleTime;
  CycleCounts++;
}
int acquireDeviceNamesFromProperties = FALSE;
void devNamesInitialization(void)
{
  ExportListStruct *el;
  ExportPropertyListStruct *prp;
  int plen, j, idx;
  char prpName[64];

  for (el=ExportList; el != NULL; el=el->next)
  {
    if (el->hasPropertyDeviceNames) 
    {
      for (j=0,idx=0; idx<PROPERTY_HASH_SIZE; idx++)
        for (prp=el->EqpProperty[idx]; prp; prp=prp->next)
      {
        if ((plen=strlen(prp->prpName)) < 5) continue;
        if (strcmp(&prp->prpName[plen-4],".NAM")) continue;
        strncpy(prpName,prp->prpName,plen-4); prpName[plen-4] = 0;
        GetDeviceListFromProperty(el->EqpName,prpName);
      }
    }
  }
  acquireDeviceNamesFromProperties = FALSE;
}
int WinSrvTimer(void)
{
  static int last_cc = -1, ncorrections = 0;
  static double last_SyncTimestamp = 0;
  static double last_dataTimeOffset = 0, largest_correction = 0;
  static double last_logged = 0;
  int cc;
  double delta_t, msStale = 0;
  char s[128];

  if (isNewMessage)
  {
    isNewMessage = FALSE;
    MessageBox(GetFocus(),MSGLastMessageString,"",IDOK);
  }

  gettimeofday(&gThisCycleTime,(struct timezone *)NULL);/* Cycle time */
  gTimeServerCycle += MSECS(gThisCycleTime,lastCycleTime);
  gTimeServerIdleLastCycle += gSystemDelay;
  lastCycleTime = gThisCycleTime;

  checkSubscriptions();

  if (acquireDeviceNamesFromProperties) devNamesInitialization();
  if (!ExportsRegistered) sndRegisteredExports();
  SurveyAlarms();
  cleanup();
  if (debug && gThisCycleTime.tv_sec > debugStoptime) 
  {
    if (debugTargetError == 0) 
    {
      DebugServer(0); // don't let this stay on by accident !!
    }
    else if (debugTargetErrorMet)
    {
      debugStoptime = gThisCycleTime.tv_sec + 5;
      debugTargetError = 0;
    }
  }
  if (currentAccessLock.lockType) checkAccessLocks(gThisCycleTime.tv_sec);
  if (nHistoryRecords) historyCycle();

  if (gTimeServerCycle >= 1000)
  {
    if (CycleCounts > gMaxCycleCounts) gMaxCycleCounts = CycleCounts;
    gCycleCounts = CycleCounts;
    if (gTimeServerIdleLastCycle > gTimeServerCycle) gTimeServerIdleLastCycle = gTimeServerCycle;
    AveBusyTime = (gTimeServerCycle-gTimeServerIdleLastCycle)*100/gTimeServerCycle;
    CycleCounts = gTimeServerCycle = gTimeServerIdleLastCycle = 0;
  }

  if (useGlobalSynchronization) 
  {
    if ((cc=GetNetGlobalEx("SYSTIME",&gSyncTimestamp,1,CF_DOUBLE,&msStale)) != last_cc)
    {
      last_cc = cc;
      wsprintf(s,"global synchronization : %s",erlst[cc]);
      feclog(s);
      if (debug) output(s);
    }
    if (cc == 0 && gSyncTimestamp > last_SyncTimestamp)
    {
      last_SyncTimestamp = gSyncTimestamp;
      gSyncTimestamp += msStale;
      gDataTimeStamp = makeDataTimeStamp();
      delta_t = gSyncTimestamp - gDataTimeStamp;
      if (fabs(delta_t) > (gSyncDeadband/1000))
      {
        if (debug) 
        {
          sprintf(s,"global synchronization : %g sec",delta_t);
          output(s);
        }
        gDataTimeStampOffset += delta_t;
        ncorrections++;
        if (fabs(delta_t) > largest_correction) largest_correction = fabs(delta_t);
      }
      if (gDataTimeStamp > last_logged + 3600) // once per hour
      {
        if (fabs(gDataTimeStampOffset-last_dataTimeOffset) > 0.01)
        {
          sprintf(s,"global synchronization offset : %g sec",gDataTimeStampOffset);
          feclog(s);
          sprintf(s,"largest correction : %g sec  (nr. corrections %d)",largest_correction,ncorrections);
          feclog(s);
          last_logged = gDataTimeStamp; ncorrections = 0; largest_correction = 0;
          last_dataTimeOffset = gDataTimeStampOffset;
        }
      }
    }
  }
  
  return 0;
}

HANDLE SearchFunction(char *name)
{
  ExportListStruct *el;

  if ((el=findTagID(name)) != NULL) return el->hEqpFcn;
  return hNULL;
}
void initUserList(void)
{
  FecPath[0] = 0; strcat(FecPath,FecDBpath); strcat(FecPath,USERS_FILE);
  openToAllUsers = getUserList(&UserList,FecPath,&NrOfUsers) ? 1 : 0;
  strcpy(txtbuf,"WRITE ACCESS: ");
  strcat(txtbuf,openToAllUsers ? "OPEN\n" : "CONTROLLED\n");
  dbllog(txtbuf);
  FecPath[0] = 0; strcat(FecPath,FecDBpath); strcat(FecPath,DENYUSER_FILE);
  accessToAllUsers = getUserList(&DeniedUserList,FecPath,&NrOfDeniedUsers) ? 1 : 0;
  strcpy(txtbuf,"DENY ACCESS: ");
  strcat(txtbuf,accessToAllUsers ? "NONE\n" : "CONTROLLED\n");
  dbllog(txtbuf);
}
int AllocateAndAssignMemory(void)
{
  int done = 0;

  if (!(ContractList=(ContractListStruct **)calloc(ContractListArraySize,sizeof(ContractListStruct *))))
	  return out_of_server_memory;
  if (!(Consumers=(CLN **)calloc(ConsumerListArraySize,sizeof(CLN *))))
	  return out_of_server_memory;  
  if (!(WorkArea=(BYTE *)calloc(1,WorkAreaSize))) return out_of_server_memory;
  initAlarmServer();
  if (!SetTimer(hDLLWnd,1,gSystemDelay,NULL))
  {
    dbllog("Can't set Timer !");
  }
  return 0;
}
int allowLegacyConnections = FALSE;
UINT16 PortOffset = 0;
UINT16 GetPortOffset(RPCFecStruct *f)
{
  UINT16 poff;
  poff = (f->Port[1] << 8) | f->Port[0];
  if (poff == 0) poff = f->PortOffset;
  return poff;
}
#define soerr(s)	{strcat(msgbuf,s); goto err;}
int setupTCPServerSocket(int port)
{
  struct sockaddr_in sck_name;
  int optval = 1, optlen = sizeof(int), mode = 1;
  int sck = 0;
  struct linger to;
  char str[80],txt[50];

  strcpy(msgbuf,"TCP: ");
  to.l_onoff = 1; to.l_linger = 1;
  memset(&sck_name,0,sizeof(sck_name));
  sck_name.sin_addr.s_addr = 0;
  sck_name.sin_family = PF_INET;
  sck_name.sin_port = htons((UINT16)port);
  sprintf(txt,"TCP PORT %d: ",port);

  if ((sck=socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) soerr("socket");
  if (setsockopt(sck,SOL_SOCKET,SO_REUSEADDR,(char *)&optval,sizeof(optval))) soerr("REUSEADDR");
//  if (setsockopt(sck,SOL_SOCKET,SO_LINGER,(char *)&to,sizeof(to))) soerr("LINGER");
  if (bind(sck,(struct sockaddr *)&sck_name,sizeof(sck_name))) soerr("bind");
  if (listen(sck,NumTCPChan) < 0) soerr("listen");
  if (WSAAsyncSelect(sck,hDLLWnd,WM_CONNECT_TCP,FD_ACCEPT))	soerr("select (parent)");

  sprintf(str,"%s initialized",txt);
  feclog(str);
  return sck;
err:
  soperror(msgbuf);
  if (sck > 0) soclose(sck);
  return -1;
}
int initTCP(void)
{
  static struct sockaddr_in sck_name;
  static int optval = 1;
  static struct linger to;
  int ttl = MCAST_TTL;
  char str[80];

  if (!WinsockLoaded()) return -1;

  sprintf(str,"Using Port Offset %d",PortOffset); feclog(str);
  TCPProducerSocket = setupTCPServerSocket(gtTCPPort+PortOffset);
  if (allowLegacyConnections)
  {
    memset(&IncomingTCPClient,0,sizeof(CLN));
    TCPParent = setupTCPServerSocket(TCP_LEGACYPORT+PortOffset);
  }

  if (listenUDP(&ProducerSocket,(UINT16)(gtUDPPort+PortOffset))) goto err;
  if (setsockopt(ProducerSocket,SOL_SOCKET,SO_BROADCAST,(char *)&optval,sizeof(optval))) soerr("BROADCAST");
  if (setsockopt(ProducerSocket,IPPROTO_IP,IP_MULTICAST_TTL, (char *)&ttl,sizeof(ttl))<0) soerr("MULTICAST");

  if (listenUDP(&SyncSocket,0)) goto err;

  if (WSAAsyncSelect(ProducerSocket,hDLLWnd,WM_READ_UDP,FD_READ)) soerr("select (Producer)");

  getIPControlNets(IPNETS_FILE,&NrOfIPNets,&IPNetList);
  getIPControlNets(IPBCASTNETS_FILE,&NrOfIPBcastNets,&IPBcastList);
  
  TCPloaded = 1;

  return 0;
err:
  soperror(msgbuf);
  if (TCPParent) soclose(TCPParent);
  if (ProducerSocket) soclose(ProducerSocket);
  if (TCPProducerSocket) soclose(TCPProducerSocket);
  return -1;
}
#ifndef UDP_CLN_PORT
# define UDP_CLN_PORT 8050
#endif
#ifndef UDP_CLN_RANGE
# define UDP_CLN_RANGE 400
#endif
int max_rcvbuf = 0x2000;
int max_sndbuf = 0x8000;
__declspec(dllexport) short PASCAL srvAssignBufferSpace(unsigned long rcvBufferSize,unsigned long sndBufferSize)
{
  if (rcvBufferSize > 0)
  {
    if (rcvBufferSize < 0x1000) rcvBufferSize = 0x1000;
    max_rcvbuf = rcvBufferSize;
  }
  if (sndBufferSize > 0)
  {
    if (sndBufferSize < 0x1000) sndBufferSize = 0x1000;
    max_sndbuf = sndBufferSize;
  }
  return 0;
}
int listenUDP(int *s,unsigned short port)
{
  static struct sockaddr_in sck_name;
  static int mode = 1,optval;
  char str[80],txt[40];
  unsigned short udpport;
  static unsigned short nextFreePort = UDP_CLN_PORT;
  static int mCastDone = FALSE;
  long ip;

  if (!mCastDone) /* always fill this in at initialization time for use by server code */
  {
    mCastDone = TRUE;
    ip = inet_addr(gtMCastAddr);
    memcpy(&tMCastAddr,&ip,sizeof(struct in_addr));  /* producer-subscriber */
    ip = inet_addr(gtGCastAddr);
    memcpy(&tGCastAddr,&ip,sizeof(struct in_addr));  /* producer-consumer   */
    tGCastAddrSck.sin_addr = tGCastAddr;
    tGCastAddrSck.sin_family = AF_INET;
    tGCastAddrSck.sin_port = htons((UINT16)gtGCastPort);
  }
  strcpy(msgbuf,"UDP: ");
  sprintf(txt,"UDP PORT %d: ",port);
  if ((*s=socket(PF_INET,SOCK_DGRAM,0)) < 0) soerr("socket");
  optval = max_rcvbuf;
  setsockopt(*s,SOL_SOCKET,SO_RCVBUF,(char *)&optval,sizeof(optval));
  optval = max_sndbuf;
  setsockopt(*s,SOL_SOCKET,SO_SNDBUF,(char *)&optval,sizeof(optval));
  if (port != 0)
  {
    optval = 1;
    if (setsockopt(*s,SOL_SOCKET,SO_REUSEADDR,(char *)&optval,sizeof(optval))) soerr("REUSEADDR");
  }
  memset(&sck_name,0,sizeof(sck_name));
  sck_name.sin_addr.s_addr = 0;
  sck_name.sin_family = AF_INET;
  udpport = port ? port : nextFreePort;
  if (udpport == 0) goto portbound;
  if (port == 0) nextFreePort = udpport + 1;
bindport:
  sck_name.sin_port = htons(udpport);
  if (bind(*s,(struct sockaddr  *)&sck_name,sizeof(sck_name)))
  {
    if (port == 0 && udpport++ < UDP_CLN_PORT + UDP_CLN_RANGE) goto bindport;
    soerr("bind");
  }
  if (port == 0) nextFreePort = udpport + 1;
portbound:
  sprintf(str,"%s initialized",txt); feclog(str);
  return 0;
err: 
  soclose(*s);
  soperror(msgbuf);
  return -1;
}

void FreeApplication()
{
  int i;
  NrOfEqpTypes=0;
  
  feclog("freeing tinesrv32.dll application");
  for (i=0; i<ncontract; i++) RemoveContract(i);
  for (i=0; i<ncontract; i++) RemoveConsumer(i);

  if (TCPloaded)
  {
    if (TCPParent)
    {
      for (i=0; i<NumTCPChan; i++) shutdown(TCPsocket[i],2);
	    shutdown(TCPParent,2);
    }
    if (ProducerSocket) soclose(ProducerSocket);
    if (TCPProducerSocket) 
    {
      flushTCPBuckets();
      shutdown(TCPProducerSocket,2);
      soclose(TCPProducerSocket);
    }
    if (SyncSocket) soclose(SyncSocket);
  }

  free((char *)WorkArea);
  feclog("server stopped");
}

int fulfillTCPRequest(int s)
{
  int cc;

  if (!allowLegacyConnections) return 0;
  msgsize = 2; *clientaddr = *clientName = 0;
  if ((cc=GetTCPStream(s,&IncomingTCPClient)) == 0) 
	cc = execRequest(&IncomingTCPClient);
  if (cc)
  {
    ((short *)WorkArea)[0] = 4;
    ((short *)WorkArea)[1] = cc;
  }
  TCPSendData(s);
  CloseTCPStream(s);
  return 0;
}
int GetTCPStream(int socket,CLN *cln)
{
  static struct sockaddr_in addr;
  int addrlen = sizeof(addr);
  static short size;
  short bytesread=0,d,mode=1;
  char *c;
  clock_t t = clock();
  char strbuf[80];

  if (!allowLegacyConnections) return 0;
  if (socket < 0) soerr("Open Stream");
  strcpy(msgbuf,"TCP: ");
  if (soioctl(socket,FIONBIO,(char *)&mode)) soerr("socket");

  if (getpeername(socket,(struct sockaddr *)&addr,&addrlen)) soerr("peer");
  memcpy(&cln->IPaddress,&addr,sizeof(addr));
  if ((c=inet_ntoa(addr.sin_addr)) != NULL) strcpy(clientaddr,c);

  FD_ZERO(&rwset); FD_SET(socket,&rwset);
  fec_to.tv_sec=1; fec_to.tv_usec=0;
  if (select(32,&rwset,NULL,NULL,&fec_to) <= 0) soerr("select TCP");

  while (bytesread<2)
  {
	if ((d=recv(socket,&((LPSTR)&size)[bytesread],2-bytesread,0)) > 0)
      bytesread += d;
	if (clock() > t+SERVER_TIMEOUT) 
	{
	  sprintf(strbuf,"recv : bytesread %d bytes, size %d (ret %d)",bytesread,size,d);
	  soerr(strbuf);
	}
  }
  if (size <= 0) return invalid_data;

  if (!(DataBuffer=(char *)calloc(1,size))) return out_of_server_memory;
  memcpy(DataBuffer,&size,bytesread);
  while (bytesread<size)
  {
    FD_SET(socket,&rwset);
    if (select(32,&rwset,NULL,NULL,&fec_to) <= 0 || clock() > t+SERVER_TIMEOUT) 
       soerr("select/to");
    if ((d=recv(socket,&DataBuffer[bytesread],size-bytesread,0)) > 0)
       bytesread += d;
  }
  return 0;

err:
  soperror(msgbuf);
  return tcp_connect_error;
}

#define getInfoPtr(i) \
	(EqpInfoStruct *)&DataBuffer[sizeof(HeaderStruct)+sizeof(EqpInfoStruct)*(i)]
int execRequest(CLN *cln)
{
  short cc,len,i;
  short eqpidx,doffset,datasize,rwflag;
  int MaxSizeIn,MaxSizeOut,FmtIn,FmtOut;
  unsigned long worksize = 0, ts;
  BYTE *tmpWorkArea;
  static char eqpmodname[8];
  static CONTRACT contract;
  EqpInfoStruct *e;
  HeaderStruct *h = (HeaderStruct *)DataBuffer;
  ExportListStruct *export;

  if (DataBuffer == NULL) return out_of_server_memory;
  if (cln == NULL) return code_failure;
  if ((RPCprotocol=h->Protocol) > 2) return illegal_protocol;
  for (i=0,len=0; i<abs(h->EqpSize); i++)
  {
    e = getInfoPtr(i);
    worksize += 3*sizeof(short) + e->DataSize*getFormatSize(e->Format) + 8;
    if (e->DataSize > len) len = e->DataSize;
  }
  if (worksize > WorkAreaSize)
  {
    worksize *= 2;  // double the size request for good measure (plan ahead!)
    if ((tmpWorkArea=realloc(WorkArea,worksize)) == NULL) ccerr(out_of_server_memory);
    WorkArea = tmpWorkArea;
    WorkAreaSize = worksize; 
  }

  if ((Values=(BYTE *)calloc(len,sizeof(double))) == NULL) ccerr(out_of_server_memory);

  memset(eqpmodname,0,8);
  rwflag = h->EqpSize/abs(h->EqpSize);
  strncpy(clientName,h->UserName,16);
  TotalMsgSize = h->TotalSize;
  eqpsize = abs(h->EqpSize);
  msgsize = 2;
  strncpy(cln->userName,h->UserName,16);
  for (doffset=0,eqpidx=0; eqpidx<eqpsize; eqpidx++,doffset+=datasize)
  {
    cc=0;
    e = getInfoPtr(eqpidx);
    datasize = e->DataSize*getFormatSize(e->Format);
    if (rwflag < 0)                                          // WRITE command
    {
      memcpy(Values,
             &DataBuffer[sizeof(HeaderStruct)+sizeof(EqpInfoStruct)*abs(h->EqpSize)+doffset],
	         datasize);
    }
    if (strcmp(eqpmodname,e->EqpModName))
    {
      strncpy(eqpmodname,e->EqpModName,EQP_NAME_SIZE);
    }
    time(&ts);

    strncpy(contract.EqpName,eqpmodname,EQP_NAME_SIZE);
    strncpy(contract.EqpProperty,e->Property,32);
    contract.EqpSizeIn = rwflag < 0 ? e->DataSize : 0;
    contract.EqpSizeOut = e->DataSize;
    contract.EqpFormatIn = (BYTE)BFMT(rwflag < 0 ? e->Format : CF_NULL);
    contract.EqpFormatOut = (BYTE)BFMT(e->Format);
    sprintf(contract.EqpDeviceName,"#%d",e->EqpModNo);

    for (export = ExportList; export; export = export->next)
    {
      if (!strncmp(export->EqpName,contract.EqpName,EQP_NAME_SIZE)) break;
    }
    if (export == NULL) ccerr(non_existent_elem);

    FmtIn = NativeFormatIn; FmtOut = NativeFormatOut;
    if ((cc=IsIllegalProperty(&contract,cln,
		                      &FmtIn,&FmtOut,
							  &MaxSizeIn,&MaxSizeOut,
							  &export)) != 0     ) ccerr(cc);

    if (!IsWindow(export->hEqpFcn)) ccerr(not_running);
    cc = doContract((CONTRACT *)&contract,export,Values,Values,eqpidx,1);

err:memcpy(&WorkArea[msgsize],&cc,2); msgsize += 2;
    if (!cc)
    {
      memcpy(&WorkArea[msgsize],&e->Format,2); msgsize += 2;
      datasize = e->DataSize*getFormatSize(e->Format);
      if (datasize%2) datasize++;
      memcpy(&WorkArea[msgsize],&datasize,2); msgsize += 2;
      memcpy(&WorkArea[msgsize],Values,datasize); msgsize += datasize;
      if (RPCprotocol > 1)
      {
        memcpy(&WorkArea[msgsize],&ts,4); msgsize += 8;
      }
    }
    else memset(eqpmodname,0,8);
  }
  *(short *)WorkArea = msgsize;
  free((char *)Values);
  return cc;
}
void CloseTCPStream(int socket)
{
  int i;
  if (!allowLegacyConnections) return;
  for (i=0; i<NumTCPChan; i++) if (TCPsocket[i] == socket) break;
  if (i == NumTCPChan) return;
  soclose(socket);
}
#define MTU 1472
void TCPSendData(int socket)
{
  int dsize;
  int bytes2send,cnt=0;
  time_t interim = time(NULL);

  msgsize = *(short *)WorkArea;
  fec_to.tv_sec=0;  fec_to.tv_usec=100000;

  while (cnt<msgsize)
  {
    bytes2send = MIN(msgsize-cnt,MTU);
    FD_ZERO(&rwset); FD_SET(socket,&rwset);
    switch (select(32,NULL,&rwset,NULL,&fec_to))
    {
      case 1: if ((dsize=send(socket,&WorkArea[cnt],bytes2send,0)) > 0) cnt += dsize;
              break;
      case 0: if (debug) output("TCP: write set not ready\n");
              break;
      default:if (debug) soperror("select TCP send");
              return;
    }
    if (time(NULL)>(interim+SERVER_TIMEOUT+msgsize/MTU)) break;
  }

  return;
}

int ContractListItem = -1;
int useMultiThreadedEquipmentFunctions = FALSE;
int gEqpFcnBusySemaphore = 0;

#ifdef MULTITHREADED
int hEqpFcnThread = 0;
int ServerExitCondition = 0;
DWORD dwBkgThreadId,dwEqpThreadId;
HANDLE hEqpThreadMutex;
THREAD_TYPE bkgTaskThread(THREAD_PARAM lpvThreadParam);
THREAD_TYPE eqpTaskThread(THREAD_PARAM lpvThreadParam);

int bkgCreateThread(BkgThreadFcn *tsk)
{
  return (int)CreateThread(NULL,0,bkgTaskThread,(LPVOID)tsk,0,&dwBkgThreadId);
}
int eqpCreateThread(void)
{
  if ((hEqpThreadMutex=CreateMutex(NULL,TRUE,EqpThreadMutexName)) == NULL)
  {
    feclog("Cannot create Equipment Function Mutex!");
    return 0;
  }
  return (int)CreateThread(NULL,0,eqpTaskThread,NULL,0,&dwEqpThreadId);
}
THREAD_TYPE eqpTaskThread(THREAD_PARAM lpvThreadParam)
{
  ExportListStruct *el;
  ContractListStruct *cl;
  struct timeval tv0, tv1;
  UINT32 delay = gSystemDelay;
  int prot = STD_PROTOCOL, id = -1, wasCalled = FALSE;
  char s[256];

  while (!ServerExitCondition)
  {
    WaitForMutex(hEqpThreadMutex,-1);
    wasCalled = FALSE;
    if (debug > 2) OutputToDebugFile("eqpTaskThread"," got mutex");
    if ((id=ContractListItem) != -1)
    {
      cl = ContractList[id]; 
      if (cl->interimStatus == not_signalled) cl->interimStatus = not_posted;
      if (cl->interimStatus == not_posted && (el=cl->exp)->hEqpFcn != hNULL)
      {
        gettimeofday(&tv0,(struct timezone *)NULL);
        cl->interimStatus = SendMessage(el->hEqpFcn,WM_SYSCOMMAND,MAKEPROTID(id,prot),prot);
        gettimeofday(&tv1,(struct timezone *)NULL);
        delay = (MSECS(tv1,tv0) < gSystemDelay) ? 0 : gSystemDelay;
        wasCalled = TRUE;
      }
      if (debug > 2)
      {
        sprintf(s,"%s %s %s <%d> %s -> delay %d ms",
          cl->contract.EqpName,cl->contract.EqpDeviceName,
          cl->contract.EqpProperty,cl->interimStatus,wasCalled ? "done" : "not called",delay);
        OutputToDebugFile("eqpTaskThread",s);
      }
    }
    else 
    {
      if (debug > 2) 
      {
        sprintf(s,"Contract empty -> delay %d ms",delay);
        OutputToDebugFile("eqpTaskThread",s);
      }
    }
    ContractListItem = -1;
    ReleaseMutex(hEqpThreadMutex); /* let the primary thread catch this */
    millisleep(delay);           /* avoid spinning here if we can help it ! */
    delay += gSystemDelay;
  }
  return 0;
}
THREAD_TYPE bkgTaskThread(THREAD_PARAM lpvThreadParam)
{
  int dly;
  struct timeval tv1,tv2;

  BkgThreadFcn btf;

  btf = *(BkgThreadFcn *)lpvThreadParam;

  while (!ServerExitCondition)
  {
    gettimeofday(&tv1,(struct timezone *)NULL);
    if (InitStatus == 0) btf.tsk();    /* do the registered background task */
    gettimeofday(&tv2,(struct timezone *)NULL);
    dly = btf.rate - MSECS(tv2,tv1);
    if (dly < 0) dly = 0;
    millisleep(dly);
  }
  return 0;
}
#endif

struct structStruct *structList = NULL;
struct structStruct *getStructFromTag(char *tag)
{
  struct structStruct *s;

  if (tag == NULL || strlen(tag) == 0) return NULL;
  for (s=structList; s && strncmp(s->name,tag,8); s=s->next);
  if (s == NULL || s->rawlen == 0 || s->size == 0) return NULL;
  return s;
}
struct structStruct *findstruct(char *tag)
{
  struct structStruct *s;

  for (s=structList; s; s=s->next) if (!strncmp(s->name,tag,8)) return s;
  if ((s=(struct structStruct *)calloc(1,sizeof(struct structStruct))) != NULL)
  {
    strncpy(s->name,tag,8);
    s->rawlen = 0;
    s->next = structList;
    structList = s;
  }
  return s;
}
int addtostruct(void *target,int addr,int size, int fmt)
{
  struct structFormat *tmp,**s = (struct structFormat **)target;
  int offset = (*s) ? (*s)->off + (*s)->siz*fmtsizeof((*s)->fmt) : 0;

  tmp = (struct structFormat *)calloc(1,sizeof(struct structFormat));
  if (tmp == NULL) return out_of_local_memory;
  tmp->off = offset;
  tmp->fmt = fmt;
  tmp->siz = size;
  tmp->addr= addr;
  tmp->nxt = *s;
  *s = tmp;
  return 0;
}
int queryStructFormat(char *tag,INTINT *sfq,int *sfqsiz)
{
  int i,n,sz=0,nq,cc=0;
  struct structStruct *s;
  struct structFormat *sf;
  
  if (sfqsiz == NULL) ccerr(argument_list_error);
  nq = *sfqsiz;
  if ((s=getStructFromTag(tag)) == NULL) ccerr(invalid_structure_tag);
  for (sf=s->s,n=0; sf != NULL; sf=sf->nxt,n++); /* count them first (reverse ordered!) */
  for (i=0,sf=s->s; i<nq && i<n && sf != NULL; i++, sf=sf->nxt)
  {
    sfq[n-i-1].i1val = sf->siz;
    sfq[n-i-1].i2val = sf->fmt;
    sz += sf->siz * fmtsizeof(sf->fmt);
  }
  if (i == nq) i--;
  sfq[i].i1val = sz;
  sfq[i].i2val = CF_NULL;
  *sfqsiz = n+1;
err:
  return cc;
}
short fillstruct(char *tag, short addr, short size, short fmt)
{
  struct structStruct *s;
  if ((s=findstruct(tag)) == NULL) return out_of_local_memory;
  if (fmt == CF_NULL)
  {
    s->num = size;
    s->size = addr;
    s->rawlen = s->size;
    return 0;
  }
  s->rawlen += size*fmtsizeof(fmt);
  return addtostruct(&s->s,addr,size,fmt);
}
int initControlStructs(void)
{
  static int initialized = 0;
  struct structStruct *ss = NULL;
  int cc;

  if (initialized) return 0;
  initialized = 1;
  if ((cc=fillstruct("PQS",
		       OFFSETIN(PropertyQueryStruct,prpName),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQS",
		       OFFSETIN(PropertyQueryStruct,prpDescription),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQS",
		       OFFSETIN(PropertyQueryStruct,prpSize),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("PQS",
		       OFFSETIN(PropertyQueryStruct,prpFormat),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("PQS",
		       OFFSETIN(PropertyQueryStruct,prpAccess),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("PQS",
		       sizeof(PropertyQueryStruct),
		       100,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpName),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpDescription),
		       64,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpRedirection),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpTag),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpUnits),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpMinValue),
		       2,CF_FLOAT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpSize),
		       3,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpHistoryDepthLong),
		       2,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       OFFSETIN(PropertyQueryStructEx,prpFormat),
		       4,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("PQSX",
		       sizeof(PropertyQueryStructEx),
		       100,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpName),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpDescription),
		       64,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpRedirection),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpTag),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpUnits),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpMinValue),
		       2,CF_FLOAT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpSize),
		       2,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpNumOverloads),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpHistoryDepthShort),
		       2,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,prpFormat),
		       4,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,rngUnits),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,rngMinValue),
		       2,CF_FLOAT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       OFFSETIN(XPropertyQueryStruct,numRows),
		       6,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("XPQS",
		       sizeof(XPropertyQueryStruct),
		       10,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,FecName),
		       FEC_NAME_SIZE,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,reserved),
		       4,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,localtime),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,starttime),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,systemPollingRate),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numBkgTsks),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numTotalContracts),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numTotalClients),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numTargetContracts),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numTargetClients),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numConnections),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numConnectionTimeouts),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numConnectionArrivals),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numUDPpkts),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numTCPpkts),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numIPXpkts),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       OFFSETIN(ActivityQueryStruct,numSPXpkts),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AQS",
		       sizeof(ActivityQueryStruct),
		       1,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpProperty),
		       32,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpDeviceName),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpName),
		       8,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpSizeIn),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpSizeOut),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.hEqpName),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpAccess),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpFormatIn),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.EqpFormatOut),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.strTagIn),
		       8,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,contract.strTagOut),
		       8,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,pollrate),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       OFFSETIN(ContractQueryStruct,nconsumers),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("CONQS",
		       sizeof(ContractQueryStruct),
		       1,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.userName),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.IPXaddress),
		       12,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.node),
		       6,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.IPaddress.sin_family),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.IPaddress.sin_port),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.IPaddress.sin_addr.s_addr),
		       4,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.IPaddress.sin_zero),
		       8,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.count),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       OFFSETIN(ClientQueryStruct,cln.protocol),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("CLNQS",
		       sizeof(ClientQueryStruct),
		       1,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,tag),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,device),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmTag),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmCode),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,timestamp),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmMask),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmData),
		       6,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmDataFormat),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmDataArraySize),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,severity),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,descriptor),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       OFFSETIN(AMS,alarmSystem),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("AMS",
		       sizeof(AMS),
		       1,CF_NULL)) != 0) return cc;

  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmTag),
		       16,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmCode),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmMask),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,severity),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmDataFormat),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmDataArraySize),
		       1,CF_BYTE)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,timeRes),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,hitsChannel),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,hitsModule),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmText),
		       40,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,deviceText),
		       38,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,alarmSystem),
		       1,CF_SHORT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,dataText),
		       40,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,url),
		       40,CF_TEXT)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       OFFSETIN(ADS,next),
		       1,CF_LONG)) != 0) return cc;
  if ((cc=fillstruct("ADS",
		       sizeof(ADS),
		       1,CF_NULL)) != 0) return cc;
    
  feclog("Control Structures Swap information registered");
  return 0;
}
/*-------------------------------------------------------------------------*/
/* DLL functions                                                           */
/*-------------------------------------------------------------------------*/
__declspec(dllexport) short PASCAL FILLSRVSTRUCT (char *tag, short addr, short size, short fmt)
{
  return fillstruct(tag,addr,size,fmt);
}
__declspec(dllexport) long PASCAL GETSRVSTRUCTSIZE (char *tag)
{
  struct structStruct *s=getStructFromTag(tag);
  return s == NULL ? 0 : (long) s->size;
}
__declspec(dllexport) short PASCAL DebugServer(short level)
{
  if (level < 0) return debug;
  if (level > 5) level = 5;
  debug = level;
  if (debug) debugStoptime = time(NULL) + 1800;
  ShowWindow(hDLLWnd,debug > 2  && debug < 5 ? SW_SHOW : SW_HIDE);
  return debug;
}
__declspec(dllexport) void SETRPCERROR(short errcode,char *errstr)
{
  return; // deprecated
}

__declspec(dllexport) short SETRPCERROREx(short errcode,char *errstr,short eqpidx)
{
  int i;
  if (errcode < 0) errcode = not_defined;
  i = MAKERPCID(eqpidx);
  if (eqpidx&PROTHIST) // history call
  {
    if (i < 0 || i >= nHistoryRecords) return invalid_index;
    if (historyRecords == NULL) return un_allocated;
    historyRecords[i].interimStatus = errcode;
  }
  else if (eqpidx&PROTALM) // alarm watch call
  {
    if (i < 0 || i >= nAlmWatchList) return invalid_index;
    if (almWatchList == NULL) return un_allocated;
    almWatchList[i].interimStatus = errcode;
  }
  else if (eqpidx&PROTNAM) // devices call
  {
    interimStatus = errcode;
  }
  else
  {
    if (i >= ncontract) return invalid_index;
    ContractList[i]->interimStatus = errcode;
  }
  if (errstr != NULL && strlen(errstr) != 0) SetRPCCompletion(errstr);
  return 0;
}

__declspec(dllexport) short PASCAL SetMinimumPollingRate(short mpr)
{
  if (mpr < 50) mpr = 50;
  if (mpr > 1000) mpr = 1000;
  MinPollingRate = mpr;
  return MinPollingRate;
}
// this has no business being a VB routine do to the Windows message handler ...
__declspec(dllexport) short AttachToDatabase(char *expname,char *lclname,HWND hwnd)
{
  ExportListStruct *el;
  short i,cc=0;
  char *eqm = NULL;
  char str[132];

  if (!initflag) InitApplication(expname,lclname,TRUE); // <- reads exports.csv if there ...
  eqm = (lclname != NULL && strlen(lclname) != 0) ? lclname : getLocalName(expname);
  if (eqm == NULL) ccerr(non_existent_elem);
  for (el = ExportList; el; el = el->next) 
  {
    if (strncmp(el->EqpName,eqm,EQP_NAME_SIZE)) continue;
    if ((i=getAliasTableIndexFromName(el->EqpName)) >= 0)
      strncpy(el->EqpAlias,gAliasTable[i].alias,EQP_NAME_SIZE);
    // if eqm == NULL, this attaches everything in the list to the given window
    el->hEqpFcn = hwnd;  
    if ( el->almLst == NULL &&
        (el->almLst=(ALARM **)calloc(el->EqpNumModules,sizeof(ALARM *))) == NULL) ccerr(out_of_server_memory);
    getAlarmInfoTable(el->EqpName);
    if (lclname != NULL && strlen(lclname) == 0) strncpy(lclname,eqm,EQP_NAME_SIZE);
  }
err:
  sprintf(str,"AttachToDatabase: exp %s lcl %s hwnd %d : %s",
    expname == NULL ? "NULL" : expname,
    lclname == NULL ? "NULL" : lclname,
    hwnd,
    erlst[cc]);
  feclog(str);
  return cc;
}
__declspec(dllexport) short RegisterEQP(char *expname,char *lclname,short numModules,HWND hwnd)
{
  short cc = 0;
  int initViaDatabase = FALSE;
  ExportListStruct *el;
  char expName[32];

  if (hwnd == NULL) ccerr(argument_list_error);
  if (lclname == NULL || strlen(lclname) == 0) ccerr(non_existent_elem);
  if (expname == NULL || strlen(expname) == 0) 
  {
    if ((cc=getExportNameFromFile(expName,lclname)) != 0) ccerr(cc);
    initViaDatabase = TRUE;
  }
  else
  {
    strncpy(expName,expname,16); expName[16] = 0;
  }
  if (!initflag) InitApplication(expName,lclname,initViaDatabase); 

  if ((cc=RegisterExport(expName,lclname,numModules)) != 0) ccerr(cc);
  if ((el=findTagID(lclname)) == NULL) ccerr(non_existent_elem);
  el->hEqpFcn = hwnd;
  historyInit(expName,lclname);
err:
  sprintf(txtbuf,"Equipment Module %s (%s) : %s",expName, lclname, cc ? erlst[cc] : "registered");    
  dbllog(txtbuf);
  return cc;
}
TCPBCKT *bcktList = NULL;

TCPBCKT *findBucket(int sck)
{
  TCPBCKT *bckt=NULL;
  for (bckt=bcktList; bckt != NULL && bckt->sck != sck; bckt=bckt->nxt);
  return bckt;
}
TCPBCKT *getBucket(int sck)
{
  TCPBCKT *bckt=NULL;
  for (bckt=bcktList; bckt != NULL; bckt=bckt->nxt)
	if (bckt->sck == sck) return bckt;
  if ((bckt=(TCPBCKT *)calloc(1,sizeof(TCPBCKT))) != NULL)
  {
    bckt->sck = sck;
    bckt->nxt = bcktList;
    bcktList = bckt;
  }
  return bckt;
}
void freeBucket(int sck)
{
  TCPBCKT *bckt,*b;
  if ((bckt=findBucket(sck)) == NULL) return;
  if (bckt == bcktList) bcktList = bckt->nxt;
  else
  {
    for (b=bcktList; b != NULL && b->nxt != bckt; b = b->nxt);
    if (b == NULL || b->nxt == NULL) return;
    b->nxt = bckt->nxt;
  }
  free(bckt);
}
void flushTCPBuckets(void)
{
  TCPBCKT *bckt=NULL;
  for (bckt=bcktList; bckt != NULL; bckt=bckt->nxt) 
  {
    shutdown(bckt->sck,2);
    soclose(bckt->sck);
  }
}
int AssignConnectionSocket(void)
{
  int sck,len = sizeof(struct sockaddr);
  struct sockaddr addr;
  TCPBCKT *bckt;

  strcpy(msgbuf,"TCP: ");
  if ((sck=accept(TCPProducerSocket,&addr,&len)) != INVALID_SOCKET)
  {
    if ((bckt=getBucket(sck)) != NULL)
    {
      bckt->idx = -1;
      if (WSAAsyncSelect(sck,hDLLWnd,WM_READ_TCP,FD_READ|FD_CLOSE)) soerr("select (child)");
      return sck;
    }
    soclose(sck);
  }
err:
  soperror(msgbuf);
  return -1;
}
void CloseConnectionSocket(int sck)
{
//  int i;
  if (sck == 0) return;
//  for (i=0; i<nconsumer; i++) if (Consumers[i]->sck == sck) break;
//  if (i == nconsumer) return;
//  Consumers[i]->sck = 0;
  freeBucket(sck);
  soclose(sck);
  if (debug)
  {
	  sprintf(txtbuf,"close TCP socket: %d\n",sck);
	  output(txtbuf);
  }
}
void AcceptTCPStream(void)
{
  WORD s;
  if (!allowLegacyConnections) return;
  if ((s=accept(TCPParent,&TCPaddr[0],&IPaddrsize)) == INVALID_SOCKET) return;
  if (WorkAreaBusy) return; else WorkAreaBusy = 1;
  fulfillTCPRequest(s);
  WorkAreaBusy = 0;
  return;
}

int GetIPData(int sck,int prot)
{
  int nread;
  if (WorkAreaBusy) return 0; else WorkAreaBusy = 1;
  nread = getConsumerIPAddress(sck,prot);
  WorkAreaBusy = 0;
  if (nread > 0) checkSubscriptions();
  return nread;
}
__declspec(dllexport) void PASCAL GetCallerInfo(char *ClientName,char *ClientAddress)
{
  char *c;
  if (!isHistoryCall && EQPconsumerlist != NULL)
  {
    if (ClientName != NULL) strncpy(ClientName,EQPconsumerlist->client->userName,16);
    if (ClientAddress != NULL)
    {
      if ((c=inet_ntoa(EQPconsumerlist->client->IPaddress.sin_addr)) != NULL) 
      strcpy(ClientAddress,c);
    }
  }
  else
  {
    if (ClientName != NULL) strncpy(ClientName,clientName,16);
    if (ClientAddress != NULL) strncpy(ClientAddress,clientaddr,16);
  }
}
__declspec(dllexport) short GetCallerFormat(short eqpidx)
{
  int cc,i;
  ContractListStruct *con;
  EqpInfoStruct *e;

  i = MAKERPCID(eqpidx);
  if (eqpidx&PROTHIST) // history call
  {
    if (i < 0 || i >= nHistoryRecords) ccerr(invalid_index);
    if (historyRecords == NULL) ccerr(un_allocated);
    return LFMT(historyRecords[i].c.EqpFormatOut);
  }
  else if (eqpidx&PROTALM) // alarm watch call
  {
    if (i < 0 || i >= nAlmWatchList) ccerr(invalid_index);
    if (almWatchList == NULL) ccerr(un_allocated);
    return LFMT(almWatchList[i].fmt);
  }
  else if (!(eqpidx&PROT1ID)) // protocol 5
  {
    if (i >= ncontract) ccerr(illegal_delimiter);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    return LFMT(con->contract.EqpFormatOut);
  }
  else if (i < eqpsize)  // Client-Server protocol
  {
    e = getInfoPtr(i);
    if (e == NULL) ccerr(un_allocated);
    return e->Format;
  }
err:
  return CF_NULL;
}
__declspec(dllexport) short GetData(void *data, short eqpidx)
{
  int cc=0,i,dsize = 0;
  short fmt;
  ContractListStruct *con;
  EqpInfoStruct *e;

  i = MAKERPCID(eqpidx);
  if (((eqpidx & PROTHIST) != 0) || ((eqpidx & PROTALM) != 0)) // history or alm watch call
  {
    ccerr(un_allocated); // no input data for HISTORY or ALARM calls
  }
  else if (!(eqpidx&PROT1ID)) // protocol 5
  {
    if (i >= ncontract) ccerr(invalid_index);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    Values = con->DataIn;
    dsize = con->contract.EqpSizeIn;
    fmt = LFMT(con->contract.EqpFormatIn);
    NativeFormatIn = con->prp.NativeFormatIn;
  }
  else if (i < eqpsize)  // protocol 1
  {
    e = getInfoPtr(i);
    dsize = abs(e->DataSize);
    fmt = e->Format;
  }
  cc = MoveAndReformat(data,NativeFormatIn,Values,fmt,dsize);
err:if (debug && cc)
	{
	  sprintf(txtbuf,"GetData: %s",erlst[cc]);
	  output(txtbuf);
	}
  return cc;
}
__declspec(dllexport) short AdjustContract(char *devName, char *devProperty, long dataLengthOut, 
										   long dataLengthIn,short eqpidx)
{
  ContractListStruct *con;
  int cc=0,i;
  unsigned short eidx = (unsigned short)eqpidx;
  struct structStruct *s;

  i = MAKERPCID(eidx);
  if (eidx&PROTNAM) // device names call
  {
    if (devName != NULL) strncpy(gLclContract.EqpDeviceName,devName,16);
    if (devProperty != NULL) strncpy(gLclContract.EqpProperty,devProperty,32);
    if (dataLengthOut < (long)gLclContract.EqpSizeOut) gLclContract.EqpSizeOut = dataLengthOut;
  }
  else if (!(eidx&PROT1ID)) // standard protocol 5
  {
    if (i >= ncontract) ccerr(invalid_index);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    if (devName != NULL) strncpy(con->contract.EqpDeviceName,devName,16);
    if (devProperty != NULL) strncpy(con->contract.EqpProperty,devProperty,32);
    if (LFMT(con->contract.EqpFormatOut) == CF_STRUCT) 
    {
      i = con->contract.EqpSizeOut;
      if ((s=getStructFromTag(con->contract.strTagOut)) != NULL)
      {
        if ((i=i/s->rawlen) > s->num) ccerr(out_of_range);
        if (con->contract.EqpSizeOut%s->rawlen) ccerr(argument_list_error);
        dataLengthOut *= s->rawlen;
      }
    }
    if (dataLengthOut < (long)con->contract.EqpSizeOut) con->contract.EqpSizeOut = dataLengthOut;
    //	if (LFMT(con->contract.EqpFormatOut) == CF_SPECTRUM) con->contract.EqpSizeOut += 24;
    con->contract.EqpSizeIn = dataLengthIn;
  }
err:
  if (debug && cc)
  {
    sprintf(txtbuf,"AdjustContract: %s",erlst[cc]);
    output(txtbuf);
  }
  return cc;
}
__declspec(dllexport) void GetContractEx(char *devName, char *devProperty, DTYPE *dout,
										 DTYPE *din,short *access,short eqpidx)
{
  ContractListStruct *con;
  EqpInfoStruct *e;
  HeaderStruct *h = (HeaderStruct *)DataBuffer;
  int cc=0,i,n;
  struct structStruct *s;

  i = MAKERPCID(eqpidx);
  if (eqpidx&PROTHIST) // history call
  {
    if (i < 0 || i >= nHistoryRecords) ccerr(invalid_index);
    if (historyRecords == NULL) ccerr(un_allocated);
    if (historyRecords[i].data == NULL) ccerr(un_allocated);
    n = (historyRecords[i].dataPtr + 1) % historyRecords[i].depthShort;
    strncpy(devName,historyRecords[i].c.EqpDeviceName,16); //devName[15] = 0;
    strncpy(devProperty,historyRecords[i].c.EqpProperty,32);
    *access = CA_READ|CA_HIST;
    dout->dFormat = LFMT(historyRecords[i].c.EqpFormatOut);
    dout->dArrayLength = historyRecords[i].c.EqpSizeOut;
    dout->data.vptr = historyRecords[i].data + n*historyRecords[i].recordLength;
    din->dFormat = CF_NULL;
    din->dArrayLength = 0;
    din->data.vptr = NULL;
  }
  else if (eqpidx&PROTALM) // alarm watch call
  {
    if (i < 0 || i >= nAlmWatchList) ccerr(invalid_index);
    if (almWatchList == NULL) ccerr(un_allocated);
    if (almWatchList[i].data == NULL) ccerr(un_allocated);
    strncpy(devName,almWatchList[i].dev,16); //devName[15] = 0;
    strncpy(devProperty,almWatchList[i].prp,32);
    *access = CA_READ|CA_ALARM;
    dout->dFormat = LFMT(almWatchList[i].fmt);
    dout->dArrayLength = almWatchList[i].siz;
    dout->data.vptr = almWatchList[i].data;
    din->dFormat = CF_NULL;
    din->dArrayLength = 0;
    din->data.vptr = NULL;
  }
  else if (eqpidx&PROTNAM) // device names call
  {
    strncpy(devName,gLclContract.EqpDeviceName,16);
    strncpy(devProperty,gLclContract.EqpProperty,32);
    *access = CA_READ;
    dout->dFormat = LFMT(gLclContract.EqpFormatOut);
    dout->dArrayLength = gLclContract.EqpSizeOut;
    dout->data.vptr = WorkArea;
    din->dFormat = CF_NULL;
    din->dArrayLength = 0;
    din->data.vptr = NULL;
  }
  else if (!(eqpidx&PROT1ID)) // standard protocol 5
  {
    if (i >= ncontract) ccerr(invalid_index);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    strncpy(devName,con->contract.EqpDeviceName,16);
    strncpy(devProperty,con->contract.EqpProperty,32);
    *access = con->contract.EqpAccess;
    dout->dFormat = LFMT(con->contract.EqpFormatOut);
    dout->dArrayLength = con->contract.EqpSizeOut;
    strncpy(dout->dTag,con->contract.strTagOut,8);
    dout->data.vptr = WorkArea;
    if (dout->dFormat == CF_STRUCT) 
    {
      AssignStructDataPtr(dout->data.vptr,WorkArea,NONE);
      dout->dArrayLength = i;
    }
    din->dFormat = LFMT(con->contract.EqpFormatIn);
    din->dArrayLength = con->contract.EqpSizeIn;
    strncpy(din->dTag,con->contract.strTagIn,8);
    din->data.vptr = con->DataIn;
    if (din->dFormat == CF_STRUCT) 
    { 
      AssignStructDataPtr(din->data.vptr,con->DataIn,RTOL);
      din->dArrayLength = i;
    }
  }
  else if (i < eqpsize)  // legacy protocol
  {
    e = getInfoPtr(i);
    sprintf(devName,"#%d",e->EqpModNo);
    strncpy(devProperty,e->Property,32);
    *access = h->EqpSize < 0 ? CA_WRITE|CA_READ : CA_READ;
    dout->dFormat = e->Format;
    dout->dArrayLength = e->DataSize;
    dout->dTag[0] = 0;
    dout->data.vptr = Values;
    din->dFormat = h->EqpSize < 0 ? e->Format : CF_NULL;
    din->dArrayLength = h->EqpSize < 0 ? e->DataSize : 0;
    din->dTag[0] = 0;
    din->data.vptr = Values;
  }
  return;
err:if (debug && cc)
	{
	  sprintf(txtbuf,"GetContractEx: %s",erlst[cc]);
	  output(txtbuf);
	}
  *devProperty = *devName = 0;
  *access = 0;
  return;

}

__declspec(dllexport) void GetContract(short *eqno, char *prop,
                                       short *size,short eqpidx, time_t *eqptime)
{
  ContractListStruct *con;
  EqpInfoStruct *e;
  HeaderStruct *h = (HeaderStruct *)DataBuffer;
  int cc=0,i, rwflag;

  i = MAKERPCID(eqpidx);
  if (eqpidx&PROTHIST) // history call
  {
    if (i < 0 || i >= nHistoryRecords) ccerr(invalid_index);
    if (historyRecords == NULL) ccerr(un_allocated);
    *size = (short)historyRecords[i].c.EqpSizeOut;
    *eqno = historyRecords[i].c.EqpDeviceName[0] == '#' ? atoi(&historyRecords[i].c.EqpDeviceName[1]) : 0;
    *eqptime = RPCtimestamp;
  }
  else if (eqpidx&PROTALM) // alarm watch call
  {
    if (i < 0 || i >= nAlmWatchList) ccerr(invalid_index);
    if (almWatchList == NULL) ccerr(un_allocated);
    *size = (short)almWatchList[i].siz;
    *eqno = almWatchList[i].dev[0] == '#' ? atoi(&almWatchList[i].dev[1]) : 0;
    *eqptime = RPCtimestamp;
  }
  else if (!(eqpidx&PROT1ID)) // protocol 5
  {
    if (i >= ncontract) ccerr(invalid_index);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    rwflag = con->contract.EqpSizeIn > 0 ? -1 : 1;
    *size = (short)con->contract.EqpSizeOut * rwflag;
    *eqno = con->contract.EqpDeviceName[0] == '#' ? atoi(&con->contract.EqpDeviceName[1]) : 0;
    strncpy(prop,con->contract.EqpProperty,32);
    *eqptime = RPCtimestamp;
  }
  else if (i < eqpsize)  // Client-Server protocol
  {
    e = getInfoPtr(i);
    rwflag = h->EqpSize < 0 ? -1 : 1;
    *size = e->DataSize*rwflag;          // read-write coded array size
    *eqno = e->EqpModNo;
    strncpy(prop,e->Property,32);
    *eqptime = RPCtimestamp;
  }
  return;
err:if (debug && cc)
	{
	  sprintf(txtbuf,"GetContract: %s",erlst[cc]);
	  output(txtbuf);
	}
  *prop = 0;
  *eqno = *size = 0;
  return;
}
__declspec(dllexport) short PutDataEx(void *data,double dataTimeStamp,short eqpidx)
{
  int cc=0,i,n;
  UINT16 fmt;
  UINT32 size;
  ContractListStruct *con;
  EqpInfoStruct *e;

  i = MAKERPCID(eqpidx);
  if (eqpidx&PROTHIST) // history call
  {
    if (i < 0 || i >= nHistoryRecords) ccerr(invalid_index);
    if (historyRecords == NULL) ccerr(un_allocated);
    n = (historyRecords[i].dataPtr + 1) % historyRecords[i].depthShort;
    if (historyRecords[i].data == NULL) ccerr(un_allocated);
    fmt = LFMT(historyRecords[i].c.EqpFormatOut);
    size = historyRecords[i].c.EqpSizeOut;
    Values = historyRecords[i].data + n*historyRecords[i].recordLength;
    NativeFormatOut = GetPropertyOutputFormat(historyRecords[i].c.EqpName,historyRecords[i].c.EqpProperty);
  }
  else if (eqpidx&PROTALM) // alarm watch call
  {
    if (i < 0 || i >= nAlmWatchList) ccerr(invalid_index);
    if (almWatchList == NULL) ccerr(un_allocated);
    fmt = LFMT(almWatchList[i].fmt);
    size = almWatchList[i].siz;
    Values = almWatchList[i].data;
    NativeFormatOut = GetPropertyOutputFormat(almWatchList[i].eqm,almWatchList[i].prp);
  }
  else if (eqpidx&PROTNAM) // devices call
  {
    fmt = LFMT(gLclContract.EqpFormatOut);
    size = gLclContract.EqpSizeOut;
    Values = WorkArea;
    NativeFormatOut = GetPropertyOutputFormat(gLclContract.EqpName,gLclContract.EqpProperty);
  }
  else if (!(eqpidx&PROT1ID)) // protocol 5
  {
    if (i >= ncontract) ccerr(invalid_index);
    if ((con=ContractList[i]) == NULL) ccerr(un_allocated);
    size = con->contract.EqpSizeOut;
    fmt  = LFMT(con->contract.EqpFormatOut);
    NativeFormatOut = con->prp.NativeFormatOut;
    Values = WorkArea;
  }
  else if (i < eqpsize)
  {
    e    = getInfoPtr(i);
    size = e->DataSize;
    fmt  = e->Format;
  }
  else return code_failure;
  if (data != NULL)
  {
    if (fmt == CF_SPECTRUM) size += 24;
    cc = MoveAndReformat(Values,fmt,data,NativeFormatOut,size);
    if (dataTimeStamp > 1000) gDataTimeStamp = dataTimeStamp;
  }
err:if (debug && cc)
	{
	  sprintf(txtbuf,"PutData: %s",erlst[cc]);
	  output(txtbuf);
	}
  return cc;
}
__declspec(dllexport) short PutData(void *data,time_t eqptime,short eqpidx)
{
  double ts = 0;
  if (eqptime != 0) ts = putDataTimeStamp(0,eqptime,0);
  return PutDataEx(data,ts,eqpidx);
}
__declspec(dllexport) void PutStatus(short status)
{
  return; // deprecated !
}
__declspec(dllexport) void ContractDone(void *data, short status,short eqpidx, time_t eqptime)
{
  return;  // deprecated !
}
__declspec(dllexport) void RemoveEqpfcn(char *name, HWND hwnd)
{
  ExportListStruct *el;

  if ((el=findTagID(name)) == NULL) return;
  return;
}
__declspec(dllexport) short PASCAL BroadCast(char *name,char *data,int format) { return 0; }
__declspec(dllexport) unsigned long PASCAL Address(void *data) { return (unsigned long)data; }
__declspec(dllexport) unsigned long PASCAL CADDR(void *data) { return (unsigned long)data; }
__declspec(dllexport) double PASCAL ADDRD(unsigned long dataPtr) { return *(double *)(dataPtr); }
__declspec(dllexport) float PASCAL ADDRF(unsigned long dataPtr) { return *(float *)(dataPtr); }
__declspec(dllexport) unsigned long PASCAL ADDRL(unsigned long dataPtr) { return *(long *)(dataPtr); }
__declspec(dllexport) unsigned short PASCAL ADDRS(unsigned long dataPtr) { return *(short *)(dataPtr); }
__declspec(dllexport) unsigned long PASCAL UTIME(void) { return time(NULL); }
__declspec(dllexport) short PASCAL SRVSTATUS(void) { return InitStatus; }

__declspec(dllexport) int PASCAL sendNetGlobal(char *keyword,BYTE *data,int len,int fmt,
		                               int minPeriod,int maxPeriod,double tolerance)
{
  GlobalListStruct *bl;
  UINT16 dsize;
  int isRegistered = FALSE;

  if ((dsize=len*fmtsizeof(fmt)) <= 0) return 0;
  for (bl=brdcstList; bl; bl=bl->next)
  {
    if (!strcmp(bl->keyword,keyword)) break;
  }
  if (bl == NULL)
  {
    if ((bl=calloc(1,sizeof(GlobalListStruct))) == NULL) return -out_of_client_memory;
    strncpy(bl->keyword,keyword,16);
    bl->size = len;
    bl->format = fmt;
    bl->maxPeriod = maxPeriod;
    bl->minPeriod = minPeriod;
    bl->tolerance = tolerance;
    if (bl->data == hNULL && (bl->data=calloc(1,dsize)) == hNULL) return -out_of_server_memory;
    if (bl->olddata == hNULL && (bl->olddata=calloc(1,dsize)) == hNULL) return -out_of_server_memory;
    bl->next = brdcstList;
    brdcstList = bl;
    nbroadcasts++;
  }
  else
  {
    isRegistered = TRUE;
  }
  bl->time = time(NULL);
  memcpy(bl->data,data,dsize);
  if (isRegistered)
  {
    bl->time = 0;
    doBroadcast();                /* send it out immediately if already registered */
  }
  return 0;
}

__declspec(dllexport) int PASCAL SetContractListSize(int siz)
{
	if (siz > 10) ContractListArraySize = siz;
	return ContractListArraySize;
}
__declspec(dllexport) int PASCAL SetConsumerListSize(int siz)
{
	if (siz > 10) ConsumerListArraySize = siz;
	return ConsumerListArraySize;
}
__declspec(dllexport) int PASCAL PutCommandsInLogFile(int LogFlag)
{
  putCommandsInFeclog = LogFlag;
  return 0;
}
__declspec(dllexport) int PASCAL SetWorkAreaSize(int siz)
{
	if (siz > WORK_SIZE) WorkAreaSize = siz;
	return WorkAreaSize;
}
__declspec(dllexport) double PASCAL SetSynchronizationDeadband(double db)
{
	if (db > 100) gSyncDeadband = db;
	return gSyncDeadband;
}
__declspec(dllexport) double PASCAL GetSynchronizationDeadband(void)
{
	return gSyncDeadband;
}
