/** \file winsrv.h
 * \brief TINE Server DLL documentation 
 *
 */
#ifndef _WINSRV_H_
#define _WINSRV_H_

#define SystemCalloc(s,n) calloc(s,n)
#define SystemFree(p) free(p)

#include "rpcdefs.h"
#include "almsrv.h"

#define BUFLEN 1280
extern char txtbuffer[BUFLEN+26]; // make sure the last ctime() fits

#define RPCERR_SIZE 32
extern char RPCLastErrorString[RPCERR_SIZE];

#define BURST_LIMIT 50
#define MIN_POLLRATE        1000
#define DEFAULT_POLLRATE    1000
#define MAXALLOWED_POLLRATE 65535L
#define MSECS(a,b) ((a).tv_sec-(b).tv_sec > MAXALLOWED_POLLRATE ? MAXALLOWED_POLLRATE :\
		    ((a).tv_sec-(b).tv_sec)*1000+((a).tv_usec-(b).tv_usec)/1000)
#define FEC_NAME_SIZE     16
#define EXPORT_NAME_SIZE  16
#define EQP_NAME_SIZE      6
#define EQP_PROP_SIZE     32
#define EQP_DEVICE_SIZE   16
#define EQP_CONTEXT_SIZE  16
#define BYTE        unsigned char
#define MAX_ENTRIES      100  // connection table entries (* 70 Kbytes)
#define FEC_TIMEOUT      1    // seconds
#define SERVER_TIMEOUT   1000 //clock ticks (1000 per second in win32)
#define SYSTEM_DELAY     20   // msecs
#define DEFAULT_CYCLE_DELAY 5 // msecs
#define INTSWAP(w)     ((w<<8)|(w>>8))
#define hNULL             0
#define MAX_EQP_FCN       20

#define CONNECT_MSG    (WM_USER+1)
#define RECEIVE_MSG    (WM_USER+2)
#define ACTIVATE_MSG   (WM_USER+3)

#define WM_READ_UDP        (WM_USER+1)
#define WM_READ_TCP        (WM_USER+2)
#define WM_CONNECT_TCP     (WM_USER+3)
#define WM_CONNECT_IPX     (WM_USER+4)
#define WM_CONNECT_SPX     (WM_USER+5)
#define WM_READ_LCL        (WM_USER+100)

#define ccerr(e) {cc = e; goto err; }
#define DeReference(h)  GlobalLock(h); GlobalUnlock(h);
#define OFFSETIN(struc,field)	((unsigned short)&(((struc *)0L)->field))

#define IPX	    0x01
#define SPX     0x02
#define TCP     0x04
#define UDP     0x08
#define LCL     0x80

// IPX/SPX specific:
#define RPC_SOCKET      0x45 // listening SPX socket
#define BRDCST_SOCKET   0x67 // listening broadcast socket
#define REG_SOCKET      0x55 // listening registration socket
#define NUM_LSN_ECB     2              // allowed simultaneous connections
#define NUM_RCV_ECB    (NUM_LSN_ECB*2) // associated incoming contracts
#define NUM_GLB_ECB     2              // incoming broadcasts
#define NUM_REG_ECB     2              // incoming registration requests
#define NUM_CLN_ECB     3              // client RPC ecbs
#define NUM_CON_ECB     5              // consumer ecbs
#define NUM_MCST_ECB    2              // incoming mcasts
#define IPX_DATA_SIZE   512
#define IPX_BUFFER_SIZE 1466

// TCP/UDP specific:
#define NumTCPChan	  5
#ifndef UDP_PORT
# define UDP_PORT          9003
#endif
#define  TCP_LEGACYPORT    8003
#ifndef TCP_PORT
# define TCP_PORT          8503
#endif
#ifndef GCAST_PORT
# define GCAST_PORT        8004
#endif
#ifndef MCAST_PORT
# define MCAST_PORT        9503
#endif
#ifndef UDP_CLN_PORT
# define UDP_CLN_PORT   8050
#endif
#ifndef UDP_CLN_RANGE
# define UDP_CLN_RANGE (UDP_CLN_PORT+1000)
#endif
#define UDP_BUFFER_SIZE 1472
#define WORK_BUFFER_SIZE 0x2000
#define SHM_BUFFER_SIZE WORK_BUFFER_SIZE
#define BCAST_ID 255
#define PRP_SCHEDULE_SIGNAL 0
#define PRP_RETRY_SIGNAL 1
#define PRP_CANCEL_SIGNAL 2

#ifndef MCAST_ADDR
# define MCAST_ADDR "238.1.1.1"
#endif
#ifndef GCAST_ADDR
# define GCAST_ADDR "238.1.1.0"
#endif

#ifndef MCAST_TTL
# define MCAST_TTL 16
#endif

#define EMPTY_PROP    0
#define TARGET_PROP   1
#define WILDCARD_PROP 2
#define STOCK_PROP    3

extern int gtUDPPort;
extern int gtTCPPort;
extern int gtGCastPort;
extern int gtMCastPort;
extern int gtUCPClnPort;
extern char *gtMCastAddr;
extern char *gtGCastAddr;
extern int gtMCastTTL;

extern int TCPParent,TCPindex,TCPloaded;
extern int TCPerrcnt,TCPcumcnt;
extern struct fd_set TCPrset,rwset,exset;
extern struct timeval TCPto,fec_to;
extern struct sockaddr addr[NumTCPChan];
extern int TCPsocket[NumTCPChan],TCPrc[NumTCPChan],
           TCPclose_rc[NumTCPChan],TCPconnection[NumTCPChan];
extern int addrsize;
extern int nUDPsets;

#define LEGACY_PROTOCOL 2
#define STD_PROTOCOL    5
#define HST_PROTOCOL   10
#define ALM_PROTOCOL   15
#define NAM_PROTOCOL   20

#define PROTNAM  0x8000
#define PROTALM  0x4000
#define PROTHIST 0x2000
#define PROT1ID  0x1000
#define PROT5ID  0x0000

#define MAKEPROTID(i,p) ((p) < LEGACY_PROTOCOL ? ((i)|PROT1ID) :\
                         (p) < HST_PROTOCOL    ? ((i)|PROT5ID) :\
                         (p) == HST_PROTOCOL   ? ((i)|PROTHIST):\
                         (p) == ALM_PROTOCOL   ? ((i)|PROTALM):\
                         ((i)|PROTNAM))

#define MAKERPCID(i)  ((i)&0xfff)

typedef struct
{
  char name[8];
} NAME8;

typedef struct
{
  char name[16];
} NAME16;

typedef struct
{
  char name[16];
  UINT32 ival;
} NAME16I;

typedef struct
{
  char name[32];
} NAME32;

typedef struct
{
  char name[32];
  UINT32 ival;
} NAME32I;

typedef struct
{
  char name[48];
} NAME48;

typedef struct
{
  char name[48];
  UINT32 ival;
} NAME48I;

typedef struct
{
  char name[64];
} NAME64;

typedef struct
{
  char name[64];
  UINT32 ival;
} NAME64I;

typedef struct
{
  char   name[16];
  float  fval;
  UINT32 ival;
} NAME16FI;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
} INTINT;

typedef struct
{
  UINT32 ival;
  double dval;
} INTDBL;

typedef struct
{
  UINT32 lval;
  double dval;
} LNGDBL;

typedef struct
{
  float  fval;
  UINT32 ival;
} FLTINT;

typedef struct
{
  float  fval;
  double dval;
} FLTDBL;

typedef struct
{
  double dval;
  UINT32 ival;
} DBLINT;

typedef struct
{
  double d1val;
  double d2val;
} DBLDBL;

typedef struct
{
  float  fval;
  UINT32 i1val;
  UINT32 i2val;
} FLTINTINT;

typedef struct
{
  UINT32 i1val;
  float  fval;
  UINT32 i2val;
} INTFLTINT;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  time_t tm;
  char   str[80];
} DUSTRING;

typedef struct
{
  time_t t1;
  time_t t2;
  UINT32 i1val;
  UINT32 i2val;
} DWINDOW;

typedef struct
{
  UINT32 i1val;
  UINT32 i2val;
  float  f1val;
  float  f2val;
} FWINDOW;

typedef struct
{
  float  xval;
  float  yval;
  UINT32 ival;
} XYS;

typedef struct
{
  UINT32 ival;
  float  f1val;
  float  f2val;
  float  f3val;
} DFILTER;

typedef struct
{
  char   comment[80];
  time_t tm;
  float  s_start;
  float  s_inc;
  UINT32 status;
} hdrSPECTRUM;

typedef struct
{
  char   comment[80];
  time_t tm;
  float  s_start;
  float  s_inc;
  UINT32 status;
  float  d_spect_array[1024];
} DSPECTRUM;

typedef union
{
  void   *vptr;
  char   *cptr;
  UINT8  *bptr;
  UINT16 *sptr;
  UINT32 *lptr;
  float  *fptr;
  double *dptr;
} DUNION;

typedef struct
{
  UINT32 dArrayLength;
  short  dFormat;
  short  reserved;
  char   dTag[16];
  DUNION data;
} DTYPE;

typedef struct PropertyEGU
{
  char units[16];
  double min;
  double max;
  BYTE graph;
  BYTE reserved[3];
} PropertyEGUStruct;

typedef struct PrpRedirBlkStruct
{
  NAME16 prpRedirection[4];  
  struct PrpRedirBlkStruct *next;
} PrpRedirBlk;

typedef struct ExportPropertyListTag
{
  char  prpName[32];       /* property name */
  char  prpAlias[32];      /* property alias name */
  UINT32 prpSizeIn;        /* maximum supported array size */
  UINT32 prpSizeOut;       /* maximum supported array size */
  int   prpId;             /* property id */
  short prpFormatIn;       /* native format */
  short prpFormatOut;      /* native format */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  short prpArrayType;      /* property array type if array */
  short numRows;           /* num rows if array */
  short rowSize;           /* row size if array */
  char  strTagIn[8]; 	     /* structure tag */
  char  strTagOut[8];      /* structure tag */
  int   structSizeIn;      /* input structure size */
  int   structSizeOut;     /* output structure size */
  PrpRedirBlk *prd;        /* re-director if not NULL */
  NAME16 *devNames;        /* device list if not NULL */
  int numDevices;          /* number of devices in device list */
  int hasQueriedForDeviceNames; /* just ask once !*/
  char  prpDescription[64];/* short description of property */
  PropertyEGUStruct egu;   /* EGU information for display */
  PropertyEGUStruct xegu;  /* x-axis EGU information for spectrum display */
  struct ExportPropertyListTag *next;
} ExportPropertyListStruct;

typedef struct HashXRef
{
  int index;
  struct HashXRef *next;
} HashXRefStruct;

#ifndef PROPERTY_HASH_SIZE
# define PROPERTY_HASH_SIZE 211
#endif
#define PROPERTY_HASH_INDEX(i) ((i) & 0xff)
#define PROPERTY_NUM_INDEX(i) (((i) >> 8) & 0xffff)
#define MAKE_PROPERTY_ID(i,indx) (((i) << 8) | (indx))

#ifndef MODULE_HASH_SIZE
# define MODULE_HASH_SIZE 211
#endif

#ifndef STOCKPROP_HASH_SIZE
# define STOCKPROP_HASH_SIZE 103
#endif

extern HashXRefStruct **StockPropXRefTable;

#ifndef ALIAS_HASH_SIZE
# define ALIAS_HASH_SIZE 103
#endif
#define ALIAS_HASH_PAYOFF (ALIAS_HASH_SIZE/10)
typedef struct
{
  char name[32];
  char alias[32];
} ALIAS_TABLE;
extern ALIAS_TABLE *gAliasTable;
extern int nAliasTableEntries;
extern HashXRefStruct **AliasXRefTable;
int makeAliasXRefTable(void);
int GetAliasTable(void);
int getTableIndex(char *str,HashXRefStruct **xrf,int hshsiz,char *tbl,int tblsiz,int off,int reclen);
#define getAliasTableIndex(str) \
   getTableIndex(str,AliasXRefTable,ALIAS_HASH_SIZE,(char *)gAliasTable,nAliasTableEntries,OFFSETIN(ALIAS_TABLE,alias),sizeof(ALIAS_TABLE))
#define getAliasTableIndexFromName(str) \
   getTableIndex(str,NULL,0,(char *)gAliasTable,nAliasTableEntries,OFFSETIN(ALIAS_TABLE,name),sizeof(ALIAS_TABLE))
#define GetStockPropertyId(str) \
  getTableIndex(str,StockPropXRefTable,STOCKPROP_HASH_SIZE,(char *)StockProperty,NrStockProperties,OFFSETIN(StockPropertyStructEx,prpName),sizeof(StockPropertyStructEx))

typedef struct ExportListTag
{
  char   Context[16];       /* device context */
  char   tagName[16];       /* exported equipment name tag */
  char   EqpName[8];        /* local equipment name tag */
  char   EqpAlias[8];       /* local equipment name tag alias */
  int    exportId;          /* local ID */
  ExportPropertyListStruct  /* pointer to chained equipment property hash list */
         *EqpProperty[PROPERTY_HASH_SIZE+1]; /* Add room for Wild-card list */
  int    hasPropertyDeviceNames;
  HashXRefStruct
	     *eqpModuleXRef[MODULE_HASH_SIZE]; /* ModuleName cross-reference */
  NAME16 *EqpModuleName;    /* pointer to device name list */
  short  EqpNumProperties;  /* total number of properties supported */
  short  EqpNumModules;     /* total number of devices supported */
  short  registered;        /* registered with name server */
  HANDLE hEqpFcn;
  ADS    *adsTable;         /* alarm definition structure table */
  short  alarmSystem;       /* alarm system flag */
  ALARM  **almLst;          /* associated alarm list */
  short  nalarms;           /* number of active alarms */
  time_t almMRT;            /* alarm Most Recent Timestamp */
  time_t almMRCT;           /* alarm Most Recent Complete Timestamp */
  short  almHSV;            /* alarm highest Severity */
  short  nAlmMRT;           /* number of alarms at MRT */
  short  nAlmMRCT;          /* number of alarms at MRCT */
  short  nAlmHSV;           /* number of alarms at HSV */
  short  nclients;          /* number of active clients */
  short  ncontracts;        /* number of active contracts */
  struct ExportListTag *next;
}ExportListStruct;

extern ExportPropertyListStruct *ExportPropertyList;
extern ExportListStruct *ExportList;
extern short NrOfEqpTypes; 

typedef struct
{
  char  prpName[32];
  char  prpDescription[32];
  UINT16 prpSize;
  BYTE prpFormat;
  BYTE prpAccess;
} PropertyQueryStruct;
#define PROPERTYQUERYSTRUCT_SIZE (32+32+2+1+1)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
} PropertyQueryStructEx;
#define PROPERTYQUERYSTRUCTEX_SIZE (32+64+32+8*2+16+4*5+2*2+4)

typedef struct
{
  char  prpName[32];
  char  prpDescription[64];
  char  prpRedirection[32];
  char  prpTag[8];
  char  prpTagIn[8];
  char  prpUnits[16];
  float prpMinValue;
  float prpMaxValue;
  UINT32 prpSize;
  UINT32 prpSizeIn;
  UINT32 prpNumOverloads;
  UINT16 prpHistoryDepthShort;
  UINT16 prpHistoryDepthLong;
  BYTE prpFormat;
  BYTE prpFormatIn;
  BYTE prpAccess;
  BYTE prpGraphType;
  char  rngUnits[16];
  float rngMinValue;
  float rngMaxValue;
  UINT16 numRows;
  UINT16 rowSize;
  UINT16 prpArrayType;
  UINT16 reserved[3];
} XPropertyQueryStruct;
#define XPROPERTYQUERYSTRUCT_SIZE (PROPERTYQUERYSTRUCTEX_SIZE + 16+2*4+6*2)

typedef struct
{
  char FecName[FEC_NAME_SIZE];
  char reserved[4];
  time_t localtime;
  time_t starttime;
  long   systemPollingRate;
  short  numBkgTsks;
  short  numTotalContracts;
  short  numTotalClients;
  short  numTargetContracts;
  short  numTargetClients;
  short  numConnections;
  UINT32 numConnectionTimeouts;
  UINT32 numConnectionArrivals;
  UINT32 numUDPpkts;
  UINT32 numTCPpkts;
  UINT32 numIPXpkts;
  UINT32 numSPXpkts;
} ActivityQueryStruct;
#define ACTIVITYQUERYSTRUCT_SIZE (FEC_NAME_SIZE+4+4*3+2*6+4*6)

typedef struct
{
  char desc[64];
  BYTE dataFmt;
  BYTE dataSiz;
  BYTE data[64];
  time_t timeEntered;
  UINT32 codeNow;
  UINT32 codePrv;
  UINT32 codeNxt;
} ServerStateStruct;
#define SERVERSTATESTRUCT_SIZE (64+2+64+4*4)

typedef struct
{
  UINT32 AveBusyTime;
  UINT32 CycleCounts;
  UINT32 MaxCycleCounts;
  UINT32 SingleLinkCount;
  UINT32 ClientMisses;
  UINT32 ClientReconnects;
  UINT32 ClientRetries;
  UINT32 ContractMisses;
  UINT32 ContractDelays;
  UINT32 BurstLimitReachedCount;
  UINT32 DataTimeStampOffset;
} ServerStatsStruct;
#define SERVERSTATSSTRUCT_SIZE (11*4)

// common definitions:
#define WORK_SIZE  (WORK_BUFFER_SIZE*sizeof(double))
#define BRDCST_BUFFER_SIZE IPX_BUFFER_SIZE

typedef struct
{
  unsigned short TotalSize;
  char UserName[16];
  short Protocol;
  short EqpSize;
} HeaderStruct;
#define HEADERSTRUCT_SIZE  22

typedef struct
{
  char ElemName[16];
  char EqpModName[8];
  short EqpModNo;
  char Property[12];
  char Cycle[22];
  short Format;
  short DataSize;
} EqpInfoStruct;

typedef struct
{
  char UserName[16];
}UserListStruct;

typedef struct
{
  char name[16];       // broadcast element name  (16 bytes)
  unsigned short  size;// number of elements broadcast under "name" (2 bytes)
  short format;        // data format (as per RPC definitions)  (2 bytes)
  long time;           // timestamp of broadcast data (4 bytes)
} BrdcstData;
#define BRDCSTDATA_HEADER_SIZE (16 + 2 + 2 + 4)

typedef struct GlobalListStructTag
{
  char keyword[16];          /* broadcast keyword */
  short size;                /* number of elements in broadcast array */
  short format;              /* broadcast data format */
  time_t time;               /* timestamp of broadcast data */
  BYTE *data;                /* data pointer */
  BYTE *olddata;             /* for comparison */
  int  minPeriod,            /* minimum period between broadcasts */
       maxPeriod;            /* maximum period between broadcasts */
  double tolerance;          /* allowed deviation of data (data units) */
  struct GlobalListStructTag *next;
} GlobalListStruct;

extern GlobalListStruct *brdcstList;

typedef struct
{
  char   dTag[8];
  short  dFormat;
  UINT16 dSize;
} LDS;                     /* Long Data Structure */
extern LDS ldsXRef[];
extern int nLdsXRef;

typedef struct
{
  char  EqpName[8];      // Registered Equipment function name
  char  Property[12];    // property
  unsigned short EqpSize;// maximum supported array size
  short EqpFormat;       // lowest common denominator format
  short EqpType;         // 1 = Read, -1 = Write/Read
  char  Description[50]; // short description of property
}PropertyListStruct;

typedef struct
{
  char  FecName[FEC_NAME_SIZE];
  char  reserved[5];
  char  FullName[EXPORT_NAME_SIZE];
  char  EqName[EQP_NAME_SIZE+1];
  char  Context[16];
  short alarmSystem;
} RPCDataStruct;
#define RPCDATASTRUCT_SIZE (FEC_NAME_SIZE+5+EXPORT_NAME_SIZE+EQP_NAME_SIZE+1+16+2)

typedef struct               /* net: queries */
{
  char Name[FEC_NAME_SIZE];
  BYTE reserved[3];
  BYTE Port[2];
  BYTE Network[4];
  BYTE Node[6];
  BYTE ImmediateAddress[6];
  char IP[16];
  BYTE PortOffset;
  char IPh_addr[4];
  short  Prot;
}RPCFecStruct;
#define RPCFECSTRUCT_SIZE (FEC_NAME_SIZE+3+2+16+21+2)

#undef NONE
#define NONE   -1
#define RTOL    0  /* Swap: Remote to Local */
#define LTOR    1  /* Swap: Local to Remote  */

struct structFormat
{
  int   siz;
  int   fmt;
  int   off;
  int  addr;
  struct structFormat *nxt;
};
struct structStruct
{
  char name[12];
  short rawlen;
  short size;
  short num;
  struct structFormat *s;  /* structure definition */
  struct structStruct *next;
};
extern struct structStruct *structList;
struct structStruct *getStructFromTag(char *tag);
struct structStruct *findstruct(char *tag);
int addtostruct(void *target,int addr,int size, int fmt);
int queryStructFormat(char *tag,INTINT *sfq,int *sfqsiz);

// =============================================================
//
// Subscription oriented Producer-consumer definitions:
//
// =============================================================
#define NARRAYELEMS   800 
#define NCONSUMER     256
#define NCONTRACT     256
#define RENEWAL       60
#define HEARTBEAT     60
#define REMINDER      5
#define ACK_REQUEST   (2*REMINDER)
#define REPLY_PENDING 3
#define SUB_PENDING   0x3fff
#define ACK_PENDING   0x7fff
#define CANCEL        CM_CANCEL
#define SINGLE        CM_SINGLE
#define REFRESH       CM_REFRESH
#define POLL          CM_POLL
#define REGISTER      CM_REGISTER
#define GROUPED       CM_GROUPED

extern char *SubType[];
extern int nconsumer;
extern int ncontract;

typedef struct
{
  char  EqpName[8],
        Property[12];
  short EqpSize,
	EqpFormat,
	EqpNumber;
  HANDLE hVBWnd;
} CONTRACT4;
#define CONTRACT4_SIZE 30

typedef struct             /* net: Protocol 5 (in) */
{
  char   EqpProperty[32];
  char   EqpDeviceName[16];
  char   EqpName[8];
  UINT32 EqpSizeIn;
  UINT32 EqpSizeOut;
  BYTE   hEqpName;
  BYTE   EqpAccess;
  BYTE   EqpFormatIn;
  BYTE   EqpFormatOut;
  char   strTagIn[8];
  char   strTagOut[8];
} CONTRACT;
#define CONTRACT_SIZE (32+16+8+4*2+4*1+2*8)

typedef struct
{
  CONTRACT contract;
  long pollrate;
  long nconsumers;
} ContractQueryStruct;
#define CONTRACT_QUERY_SIZE (CONTRACT_SIZE + 4 + 4)

typedef struct
{
  int NativeFormatIn;
  int NativeFormatOut;
  int MaxSizeIn;
  int MaxSizeOut;
  int MaxBytesIn;
  int MaxBytesOut;
} CONTRACTeX;

typedef struct
{
  BYTE network[4],
       node[6],
       socket[2];
}
IPXAddress;

typedef struct
{
  char               userName[16];
  IPXAddress         IPXaddress;  // if 0 then IP transfer is assumed
  BYTE               node[6];     // IPX routing node (Immediate Address)
  struct sockaddr_in IPaddress;
  short              count;       // number of registrations
  short              protocol;    /* target client protocol */
  short              transport;   /* transport protocol */
  int                sck;         /* communication socket */
} CLN;
#define CLIENT_SIZE (16 + 12 + 6 + 16 + 2 + 2 + 2 + 4)

typedef struct                    /* net: Protocol 5 (query) */
{
  char               userName[16];
  IPXAddress         IPXaddress;  /* if 0 then IP transfer is assumed */
  BYTE               node[6];     /* IPX routing node (Immediate Address) */
  struct sockaddr_in IPaddress;
  short              count;       /* number of registrations */
  short              protocol;    /* target client protocol level */
} CLNQS;

#define LOCK_UNLOCKED 0
#define LOCK_PREEMPTIVE 1
#define LOCK_PERSISTENT 2
#define LOCK_ABORT 3
typedef struct
{
  CLN cln;
  int lockType;
  int lockDuration;
  UINT32 lockToken;
  time_t lockStart;
} AccessLockStruct;

typedef struct                /* local */
{
  short  id;                  /* contract ID as seen by consumer */
  short  mode;                /* SINGLE,REFRESH,POLL,CANCEL */
  long   PollingRate;         /* requested polling rate */
  struct timeval lasttime;    /* time of most recent update */
  time_t starttime;           /* start of initial subscription */
  BYTE   counter;             /* counts remaining in subscription(s) */
  BYTE   Stale;               /* 1 = send new data to consumer */
  UINT16 numblks;             /* total number of message blocks */
  UINT16 blknum;              /* block number of this message */
  UINT32 nextDataSize;        /* next outgoing data size in bytes */
  UINT32 nextDataPtr;         /* next outgoing data array pointer */
  UINT32 BytesRemaining;      /* number of bytes data remaining */
  UINT32 BytesTotal;          /* total number of bytes data */
  UINT16 mtu;                 /* maximal transport unit */
  short  statusCode;          /* interim status */
  UINT32 misses;              /* data return after expiration */
  UINT32 reconnects;          /* client unexpectedly re-establishes link */
  int    isRetry;             /* the contract is being resubmitted by the client */
} ClientSTATUS;

typedef struct
{
  CLNQS cln;
} ClientQueryStruct;
#define CLIENT_QUERY_SIZE (16 + 12 + 6 + 16 + 2 + 2)

#define MAXLONGGRAMS 15
typedef struct
{
  UINT32 haddr;
  UINT16 SubscriptionID;      /* supplied by the consumer upon registration */
  UINT16 blksin;              /* # blocks arrived from long datagram */
  UINT16 blkid;               /* block id of long datagram */
  UINT8  busy,
         reserved;
  UINT8  *data;
  time_t blktime;
  time_t SubsStarttime;
} ContractProgress;

typedef struct ConStruct
{
  CLN *client;
  ClientSTATUS status;
  struct ConStruct *next;
} CONSUMER;

typedef struct
{
  CONSUMER *consumer;            // assciated consumer list
  short    nconsumer;            // number of consumers for this contract
  CONTRACT4 contract;            // Contract structure information
  CONTRACTeX  prp;               // Contract property information
  unsigned short PollingRate;    // in seconds, smallest number wins ...
  struct timeval lasttime,       // most recent data polled
               lastmarked;       // last time data were marked
  time_t dtimestamp;           // data timestamp
  BYTE *data;                    // data to send
} ContractListStruct4;

typedef struct                  /* local */
{
  CONSUMER *consumer;           /* assciated consumer list */
  int      nconsumer;           /* number of consumers for this contract */
  CONTRACT contract;            /* see above */
  CONTRACTeX  prp;              /* Contract property information  */
  long     PollingRate;         /* in mseconds, smallest number wins ... */
  struct   timeval lasttime;    /* most recent data polled */
  struct   timeval lastmarked;  /* last time the data were marked STALE */
  time_t   dtimestamp;          /* data timestamp */
  int      dtimestampMSEC;      /* data timestamp (msec fraction) */
  BYTE     *DataOut;            /* data to send  */
  BYTE     *DataIn;             /* data received */
  UINT32   dataSizeOut;         /* output data size <= contract.dataSizeOut */
  ExportListStruct *exp;        /* associated export element */
  int      interimStatus;       /* interim status code for this contract */
  int      interimCounter;      /* interim counter used for 'not_ready' */
  int      returnCode;          /* last return code for this contract */
  UINT16   contractID;		    	/* ID to pass back to clients for BCAST, MCAST */
  int      pending;             /* pending flag for DeliverData() vs ProduceData() */
  int      expired;             /* expired flag for contract removal */
  UINT32   delays;              /* delayed scheduling due to pending */
  UINT32   misses;              /* missed scheduling (i.e. > 2x polling rate) */
} ContractListStruct;

typedef struct
{
  UINT16 msgsize;
  short SubscriptionID,         // supplied by the consumer upon registration
        CompletionCode,
        counter;                // tell consumer how much is left
  time_t timestamp,             // RPCtimestamp
	 starttime;             // supplied by the consumer upon registration
} ProducerHeaderStruct4;
#define PRODUCER4_SIZE 16

typedef struct                  /* net: Protocol 5 (out) */
{
  UINT16 msgsize;               /* message size in bytes */
  UINT16 SubscriptionID;        /* supplied by the consumer upon registration */
  short  CompletionCode;        /* completion or interim status */
  UINT16 numblks;               /* total number of message blocks */
  UINT16 blknum;                /* block number of this message */
  UINT16 mtu;                   /* maximum transport unit */
  UINT16 timestampMSEC;         /* msec fraction of data timestamp */
  BYTE   EqpFormat;             /* Eqp Format of returned data */
  BYTE   counter;               /* tell consumer how much is left */
  time_t timestamp;             /* data timestamp */
  time_t ClnStarttime;          /* supplied by the consumer upon registration */
} ProducerHeaderStruct;
#define PRODUCER_SIZE (7*2+2*1+2*4)
#define PMTU(s)  (((s) ? IPX_BUFFER_SIZE : UDP_BUFFER_SIZE) - PRODUCER_SIZE - sizeof(UINT16))

struct subscription4
{
   UINT16 msgsize;
   short PollingRate,
         id,
         mode;
   time_t   starttime;
   CONTRACT4 contract;
};
#define SUBSCRIPTION4_SIZE (12+CONTRACT4_SIZE)

struct subscription         /* net: Protocol 5 (in) */
{
  UINT16    msgsize;
  UINT16    mtu;            /* maximum transport unit for incoming data */
  UINT8     numblks;        /* total number of message blocks           */
  UINT8     blknum;         /* block number of this message             */
  UINT16    id;
  short     mode;
  short     blkid;          /* (incremented) block id */
  UINT32    PollingRate;
  time_t    starttime;
  CONTRACT contract;
};
typedef struct subscription SUBSCRIPTION;
#define SUBSCRIPTION_SIZE (6*2+2*4+CONTRACT_SIZE)

typedef struct
{
  char prpName[32];
  UINT16 prpSize;          /* maximum supported array size */
  short prpFormat;         /* native format */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  char  prpDescription[32];/* short description of property */
  int  (*fcn)(CONTRACT *,char *,char *);
} StockPropertyStruct;

typedef struct
{
  char prpName[32];
  UINT32 prpSize;          /* maximum supported array size (output) */
  UINT32 prpSizeIn;        /* maximum supported array size (input)  */
  short prpFormat;         /* format (output) */
  short prpFormatIn;       /* format (input) */
  short prpAccessMode;     /* bit 0: read access, bit 1: write access */
  short prpArrayType;      /* output array type */
  char  prpStructTag[8];   /* output data structure tag */
  char  prpDescription[64];/* short description of property */
  int  (*fcn)(CONTRACT *,char *,char *);
} StockPropertyStructEx;

typedef struct
{
  char modName[16];
  int  (*fcn)(CONTRACT *,char *,char *);
} StockModifierStruct;
extern int NrStockModifiers;
extern StockModifierStruct StockModifier[];

extern AccessLockStruct currentAccessLock;
extern int StaleData;
extern int PCSocket;              // IPX Producer/Consumer send socket
extern int ProducerSocket;        // UDP Producer Socket
extern int ConsumerSocket;        // UDP Consumer Socket
extern int BrdcstSocket;          // UDP Broadcast Socket
extern int GlobalSocket;          // UDP Global listening Socket
extern struct sockaddr_in prod_name;
//#define SendToPeer(c,d)	SendIPData(c,d)
#define SendToPeer(c,d)	c->transport == LCL ? SendLCLData(c,d) : SendIPData(c,d)
// ((cc=SendIPXData(c,d)) == ipx_socket_error ? SendIPData(c,d) : cc )
void doConsumer(void);

typedef struct tcpbckt
{
  int idx;                  // FEC index
  int sck;                  // TCP connection socket
  int ptr;                  // bucket pointer
  int siz;                  // bucket target size
  BYTE buf[UDP_BUFFER_SIZE];// bucket buffer
  struct tcpbckt *nxt;      // this is a linked list
} TCPBCKT;
extern TCPBCKT *bcktList;
TCPBCKT *findBucket(int sck);
TCPBCKT *getBucket(int sck);
void freeBucket(int sck);
void CloseConnectionSocket(int sck);
int AssignConnectionSocket(void);

// =========================================================================

extern BYTE *WorkArea;
extern unsigned long WorkAreaSize;
extern BYTE *Values;
extern IPXAddress FecAddress;

extern int RPCNumRecsRead,RPCNumFecsRead,RPCNumEqpsRead,FECallocNum;
extern unsigned long TCPcount,UDPcount,SPXcount,IPXcount,BRDcount;

// ==================================================================

extern CLN **Consumers;
//extern ContractListStruct *ContractList[NCONTRACT];
extern ContractListStruct **ContractList;
extern UserListStruct *UserList;
extern UserListStruct *DeniedUserList;
extern int openToAllUsers;
extern int NrOfUsers;
extern int NrOfDeniedUsers;
extern int accessToAllUsers;
extern PropertyListStruct *PropertyList;
extern int NrOfProperties;

extern int numEqpFcn;
extern HCURSOR WaitCurs;
extern DWORD IPXTaskID;
extern WORD destConID;
extern int interimStatus;

extern WORD IPXsocket;
extern const BYTE rpc_socket[2];
extern const BYTE brdcst_socket[2];
extern const BYTE reg_socket[2];

extern char *ltzname[2];
extern short ExportsRegistered;
extern int eqpsize;
extern BYTE *DataBuffer;
extern int RPCprotocol,TotalMsgSize;
extern int msgcnt,msgsize;
extern double gDataTimeStamp;
extern double gDataTimeStampOffset;
#define RPCtimestamp ((long)gDataTimeStamp)
#define RPCtimestampMSEC (int)((gDataTimeStamp - (long)gDataTimeStamp) * 1000)
extern char clientName[17],clientaddr[30],msgbuf[256];
extern char *erlst[];
extern char gFecName[20];
extern char gDeviceContext[17];
extern char ControlDBpath[80];
extern char NameServerPath[80];
extern char FecDBpath[80];
extern char gFeclogPath[80];
extern int AveBusyTime;
extern UINT32 gContractMisses;
extern UINT32 gContractDelays;
extern UINT32 gClientMisses;
extern UINT32 gClientReconnects;
extern UINT32 gClientRetries;
extern UINT32 gBurstLimitReachedCount;
extern UINT32 gSingleLinkCount;
extern int gCycleCounts, gMaxCycleCounts;
extern int max_rcvbuf;
extern int gRetardSingleContractRemoval;
extern struct timeval gThisCycleTime;
extern int gSystemDelay;
extern int SystemPresetMemory;
extern time_t startuptime;
extern int debug;
extern RPCFecStruct NameServer; //  "
extern int SyncSocket;
extern UserListStruct *UserList;
extern int NrStockProperties;
extern StockPropertyStructEx StockProperty[];
extern struct sockaddr_in tGCastAddrSck;
extern struct sockaddr_in tMCastAddrSck;
extern struct in_addr tMCastAddr;
extern struct in_addr tGCastAddr;
extern long SysPoll;
extern int NrOfIPNets;
extern int NrOfIPBcastNets;
extern int nbroadcasts;
extern int nExpiredContracts;
extern int putCommandsInFeclog;
extern int CycleCounts;
extern struct timeval lastCycleTime;
extern UINT32 gTimeServerCycle;
extern CONSUMER *EQPconsumerlist;
extern struct sockaddr_in *IPNetList;
extern struct sockaddr_in *IPBcastList;
extern int FecNameResolved;
extern UINT16 PortOffset;
extern CONTRACT gLclContract;
extern int acquireDeviceNamesFromProperties;
extern char MSGLastMessageString[256];
extern int isNewMessage;
extern int ContractListItem;
extern int useMultiThreadedEquipmentFunctions;
extern int gEqpFcnBusySemaphore;
extern time_t appmktime;
extern char appmkversion[64];

#ifdef MULTITHREADED
#define THREAD_TYPE DWORD WINAPI 
#define THREAD_PARAM LPVOID
#define EqpThreadMutexName "tineEqpThreadMutex"
#define millisleep(ms) Sleep(ms)
#define WaitForMutex(mx,to) WaitForSingleObject(mx,(to) < 0 ? INFINITE : to)
extern int hEqpFcnThread;
extern int ServerExitCondition;
extern DWORD dwBkgThreadId,dwEqpThreadId;
extern HANDLE hEqpThreadMutex;
typedef struct 
{
  void (*tsk)(void);
  int rate;
} BkgThreadFcn;
THREAD_TYPE bkgTaskThread(THREAD_PARAM lpvThreadParam);
THREAD_TYPE eqpTaskThread(THREAD_PARAM lpvThreadParam);
#define thread_dbglog(cl,txt) {\
    sprintf(txtbuf," %s %s %s -> %s",\
    (cl)->contract.EqpName,(cl)->contract.EqpDeviceName,\
    (cl)->contract.EqpProperty,(txt));\
    OutputToDebugFile("doContract",txtbuf);\
  }
int eqpCreateThread(void);
int bkgCreateThread(BkgThreadFcn *tsk);
#endif

#ifndef SystemCalloc
BYTE *SystemCalloc(int num, int len);
void SystemFree(void *ptr);
#endif
void OutputToDebugFile(char *routine, char *txt);

#define NLINES 1000
extern int nofeclog;
extern int FeclogDepth;
/** 
 * \brief Puts the supplied text string to the server's log file
 *
 * \b Visual \b Basic :
 * \n Declare Function \b feclog Lib "tinesrv32.dll" (ByVal text As String) As Integer
 *
 * A server will automatically make entries of a certain variety in the server's fec.log
 * file (such as intialization paramters, incoming WRITE commands, etc.)  In addition,
 * a server can enter any text at any time by calling this routine.  Note that the 
 * server's log file can be pulled by any client by calling the stock property "LOGFILE".
 *
 * \param text is the text string to be entered into the server's fec.log file.
 *
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \include eg_feclog.txt
 *
 */
__declspec(dllexport) short PASCAL feclog(char *text);

/** 
 * \brief Determines whether in-coming commands are appended to the server's log file
 *
 * \b Visual \b Basic :
 * \n Declare Function \b PutCommandsInLogFile Lib "tinesrv32.dll" (ByVal flag As Long) As Integer
 *
 * A server will automatically make entries of a certain variety in the server's fec.log
 * file (such as intialization paramters).  By default, in-coming WRITE access commands are
 * also entered into the server's log file. In addition,
 * a server can enter any text at any time by calling feclog().  Note that the 
 * server's log file can be pulled by any client by calling the stock property "LOGFILE".
 *
 * \param flag determines whether in-coming commands are logged.  A non-zero value for
 * 'flag' is interpretted as TRUE (the default).  To turn this feature off, pass a '0'
 * in this call.
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 *
 * \b default: TRUE
 *
 * \include eg_putcommandsinfeclog.txt
 *
 */
__declspec(dllexport) int PASCAL PutCommandsInLogFile(int LogFlag);

/** 
 * \brief Sets the depth of the server's log file to (approximately) the number of lines
 * given.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b setFecLogDepth Lib "tinesrv32.dll" (ByVal nlines As Integer) As Integer
 *
 * A server's log file will contain the last 'nlines' entries based on the average line
 * size being approximately 40 characters.  When this limit is reached the log file 'fec.log'
 * is automatically rotated to the backup file 'fec.bak' and a new log file is started.
 * Calls to the stock property 'LOGFILE' will return the last n characters requested 
 * and will use both files to process the call.
 *
 * \param nlines is the number of lines to keep in the server's log file (where 1 line approx.
 * equals 40 characters).
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \see feclog().
 *
 * \include eg_feclogdepth.txt
 *
 */
__declspec(dllexport) short PASCAL setFecLogDepth(short nlines);
void output(char *txt);
#define dbllog(txt) { output(txt); feclog(txt); }

int makeStockPropXRefTable(void);
void FreeApplication(void);
int InitApplication(char *expName,char *lclName,int InitViaDatabase);
int AllocateAndAssignMemory(void);
int initTCP(void);
int GetTCPStream(int socket,CLN *cln);
int execRequest(CLN *cln);
void TCPSendData(int socket);
void CloseTCPStream(int socket);
int fulfillTCPRequest(int s);
int doContract(CONTRACT *con,ExportListStruct *el,BYTE *dataIn,BYTE *dataOut,int id,int prot);
int SendIPData(CLN *cln,BYTE *data);
int SendLCLData(CLN *cln,BYTE *data);
HANDLE SearchFunction(char *name);
int ProduceData(void);
int DeliverData(void);
int listenUDP(int *s,unsigned short port);
int getConsumerRequest(CLN *cln,BYTE *buf,int packetsize);
int getConsumerIPAddress(int sck, int prot);
int RemoveConsumer(int i);
int RemoveContract(int j);
void AcceptTCPStream(void);
int GetIPData(int sck,int prot);
ExportListStruct *findTagID(char *nam);
int appendRegisteredUserList(char *user);
int getUserList(UserListStruct **ul,char *userfile,int *nr);
int getValuesAsDouble(DTYPE *d,double *dval,short num);
int getValuesAsLong(DTYPE *d,UINT32 *lval,short num);
int getValuesAsFloat(DTYPE *d,float *fval,short num);
int getValuesAsString(DTYPE *d,char *str,UINT32 *dsiz);
void CleanupStrandedDatagrams(void);
short GetPropertyOutputFormat(char *eqm,char *prpName);
ExportPropertyListStruct *GetProperyListStruct(char *eqm,char *prpName);
int removeRegisteredUser(char *user);
int tExecLink(int fecindex,char *name,char *dev,char *prop,
	   short size,char *data,short format,short timout);
int tRPC(int fecindex,char *data,char *name,short EqNo,char *prop,
	   short size,short format,short timout);

NAME16 *GetEqpModuleNames(char *eqname,int num);
int getRegisteredExports(char *fecname,char *lclname);
char *getLocalName(char *exportName);
int getExportNameFromFile(char *exportName,char *localName);
void ParseRedirector(char *str,ExportPropertyListStruct *prp);
void ParsePropertyDescription(char *str,ExportPropertyListStruct *prp);
ExportPropertyListStruct *GetDeviceListFromProperty(char *eqm,char *prp);
short IsIllegalProperty(CONTRACT *con,CLN *cln,int *EqpNativeFormatIn,int *EqpNativeFormatOut,
						int *MaxSizeIn,int *MaxSizeOut,ExportListStruct **listEntry);
char *getCaller(void);
int getNumContracts(char *eqname);
int getNumConsumers(char *eqname);
int getNumCallers(char *eqname);
int historyRecordDepth(char *prpName,UINT16 *shortDepth,UINT16 *longDepth);
int IsMemberLockSet(CLN *cln);
short IsStockProperty(char *Property);

void applyFecLogPath(char *path,int pathlen);
int scanFecLogFiles(char *path,char *subpath,NAME64 *namebuf,int *numfiles,int numoffset);
int readFecLogFile(char *fname,char *textbuf,int sizeInBytes);
int readFecLog(char *textbuf,int sizeInBytes);
int readSelfTest(char *eqm,char *textbuf,int sizeInBytes);
struct timeval *getDataTimeStampAsTimeval(void);
unsigned long ElfHash(const unsigned char *key);
CLN *LocateConsumerInList(struct sockaddr_in *ip,IPXAddress *nwAddr,BYTE *immAddr,long *hWnd);

/** 
 * \brief Sets the server's Front End Controller (FEC) Name.
 * given.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b RegisterFecName Lib "tinesrv32.dll" 
 * (ByVal name As String,ByVal dsc As String,ByVal loc As String,
    ByVal hdw As String,ByVal resp As String,ByVal port As Integer) As Integer
 *
 * 
 * Servers must be assigned to a Front End Controller (FEC), which for many operating systems
 * corresponds to a process running on the computer rather than the computer itself (meaning
 * there can in some cases be more than one servers with different 'FEC' names running on 
 * the same computer.  There can be several device servers attached to the same FEC (i.e.
 * sharing the same address space), so the FEC name is a distinct quantity from the
 * device server name, and this name must also be system-wide 
 * unique.  
 *
 * The FEC name can also be (and frequently is) registered via the presence of an
 * 'fecid.csv' startup database file.
 *
 * \param name is the FEC name indentifying the Front End Controller and to which all registered 
 *  equipment modules are bound.
 * \param dsc is a 32-character desciption of the FECs server duties.  A 'subsystem' can be 
 *  associated with ALL equipment modules found on the FEC by prefixing the description with
 *  a subsystem tag of the form "[<tag>]".  This information will be parsed by the Equipment
 *  name server and used for sub-system specific queries.  For example, the associate the FEC
 *  "HEPHF" with the subsystem "RF", the description field might be "[RF]Hera Proton HF Control".
 * \param loc is a 32-character string giving the phyical location of the FEC.
 * \param hwd is a 32-character brief description of the IO hardware found on the FEC.
 * \param resp is a 32-character string listing the developer(s) responsible for the FEC. Note:
 *  The Equipment Name Server will allow the removal of FECs and associated equipment modules
 *  to the user(s) specified here.
 * \param poff is the 'Port Offset' to be applied to the FEC.  This parameter plays an important
 * role where more that one FEC is to run on a machine running an operating system using virtual 
 * memory.  In such cases 'Front End Controller' is a misnomer, since "FEC" really refers to a 
 * process running on the computer to which one or more equipment modules are bound.  For 
 * Win16 there is in fact only one such processes managing all requistered equipment
 * modules, so this parameter is unimportant there.
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \see RegisterFecInformation(), RegisterFecNameEx().
 *
 * \include eg_RegisterFecName.txt
 *
 */
__declspec(dllexport) short PASCAL RegisterFecName(char *name,char *dsc,char *loc,char *hdw,char *resp,short port);

/** 
 * \brief Sets the server's Front End Controller (FEC) Name. (Extended Call)
 * given.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b RegisterFecNameEx Lib "tinesrv32.dll" 
 * (ByVal name As String,ByVal dsc As String,ByVal loc As String,
    ByVal hdw As String,ByVal resp As String,ByVal port As Integer,ByVal Context As String) As Integer
 *
 * 
 * Servers must be assigned to a Front End Controller (FEC), which for many operating systems
 * corresponds to a process running on the computer rather than the computer itself (meaning
 * there can in some cases be more than one servers with different 'FEC' names running on 
 * the same computer.  There can be several device servers attached to the same FEC (i.e.
 * sharing the same address space), so the FEC name is a distinct quantity from the
 * device server name, and this name must also be system-wide 
 * unique.  The extended call RegisterFecNameEx() differs from RegisterFecName() in that
 * it accepts the server 'Context' as an additional parameter.  The context will be applied
 * to all device servers attaching to the same FEC name.
 *
 * The FEC name can also be (and frequently is) registered via the presence of an
 * 'fecid.csv' startup database file.
 *
 * \param name is the FEC name indentifying the Front End Controller and to which all registered 
 *  equipment modules are bound.
 * \param dsc is a 32-character desciption of the FECs server duties.  A 'subsystem' can be 
 *  associated with ALL equipment modules found on the FEC by prefixing the description with
 *  a subsystem tag of the form "[<tag>]".  This information will be parsed by the Equipment
 *  name server and used for sub-system specific queries.  For example, the associate the FEC
 *  "HEPHF" with the subsystem "RF", the description field might be "[RF]Hera Proton HF Control".
 * \param loc is a 32-character string giving the phyical location of the FEC.
 * \param hwd is a 32-character brief description of the IO hardware found on the FEC.
 * \param resp is a 32-character string listing the developer(s) responsible for the FEC. Note:
 *  The Equipment Name Server will allow the removal of FECs and associated equipment modules
 *  to the user(s) specified here.
 * \param poff is the 'Port Offset' to be applied to the FEC.  This parameter plays an important
 * role where more that one FEC is to run on a machine running an operating system using virtual 
 * memory.  In such cases 'Front End Controller' is a misnomer, since "FEC" really refers to a 
 * process running on the computer to which one or more equipment modules are bound.  For 
 * Win16 there is in fact only one such processes managing all requistered equipment
 * modules, so this parameter is unimportant there.
 * \param context is a 16-character string giving the context of all registered equipment modules.
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \see RegisterFecInformation(), RegisterFecName().
 *
 * \include eg_RegisterFecNameEx.txt
 *
 */
__declspec(dllexport) short PASCAL RegisterFecNameEx(char *name,char *dsc,char *loc,char *hdw,char *resp,short port,char *context);

/** 
 * \brief Sets the server's Front End Controller (FEC) Name and relevant information.
 * given.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b RegisterFecInformation Lib "tinesrv32.dll" 
 * (ByVal name As String,ByVal sub As String,ByVal cntxt As String,ByVal dsc As String,ByVal loc As String,
    ByVal hdw As String,ByVal resp As String,ByVal port As Integer) As Integer
 *
 * 
 * Servers must be assigned to a Front End Controller (FEC), which for many operating systems
 * corresponds to a process running on the computer rather than the computer itself (meaning
 * there can in some cases be more than one servers with different 'FEC' names running on 
 * the same computer.  There can be several device servers attached to the same FEC (i.e.
 * sharing the same address space), so the FEC name is a distinct quantity from the
 * device server name, and this name must also be system-wide 
 * unique.  
 *
 * The FEC name can also be (and frequently is) registered via the presence of an
 * 'fecid.csv' startup database file.
 *
 * \param name is the FEC name indentifying the Front End Controller and to which all registered 
 *  equipment modules are bound.
 * \param sub is the subsystem to which the equipment modules attached to the FEC belong.
 * \param cntxt is the context to which the equipment modules attached to the FEC belong.
 * \param dsc is a 32-character desciption of the FECs server duties.  A 'subsystem' can be 
 *  associated with ALL equipment modules found on the FEC by prefixing the description with
 *  a subsystem tag of the form "[<tag>]".  This information will be parsed by the Equipment
 *  name server and used for sub-system specific queries.  For example, the associate the FEC
 *  "HEPHF" with the subsystem "RF", the description field might be "[RF]Hera Proton HF Control".
 * \param loc is a 32-character string giving the phyical location of the FEC.
 * \param hwd is a 32-character brief description of the IO hardware found on the FEC.
 * \param resp is a 32-character string listing the developer(s) responsible for the FEC. Note:
 *  The Equipment Name Server will allow the removal of FECs and associated equipment modules
 *  to the user(s) specified here.
 * \param poff is the 'Port Offset' to be applied to the FEC.  This parameter plays an important
 * role where more that one FEC is to run on a machine running an operating system using virtual 
 * memory.  In such cases 'Front End Controller' is a misnomer, since "FEC" really refers to a 
 * process running on the computer to which one or more equipment modules are bound.  For 
 * Win16 there is in fact only one such processes managing all requistered equipment
 * modules, so this parameter is unimportant there.
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \see RegisterFec(), RegisterFecNameEx().
 *
 * \include eg_RegisterFecInformation.txt
 *
 */
__declspec(dllexport) short PASCAL RegisterFecInformation(char *name,char *sub,char *cntxt,char *dsc,char *loc,char *hdw,char *resp,UINT16 poff);

/** 
 * \brief Fills in the structure information for a TINE 'tagged' structure
 *
 * \b Visual \b Basic :
 * \n Declare Function \b FILLSRVSTRUCT Lib "tinesrv32.dll" (ByVal tag As String, ByVal addr As Integer, ByVal fmt As Integer) As Integer
 *
 * If a server passes user-defined structures (format CF_STRUCT), then it should make use of 
 * FILLSRVSTRUCT() to register the structure locally.  The calling program should likewise make use
 * of FILLSTRUCT() (or fillstruct() in C or java) to register the stucture at the client side.  A
 * client can 'discover' the structure's structure, but the general idea behind a 'user-defined'
 * structure is that it makes sense for the developer to define a structure for passing data from
 * server to client and thus the developer is likely responsible for both sides of the data 
 * transfer.  The advantages of using FILLSRVSTRUCT to register structures is three-fold.  First,
 * issues of byte-swapping across platforms are automatically taken care of.  Second, the 
 * structure in question is then known by the system and hence an array of such 'things' can
 * be trivially passed by simply passing the number of items as the array size (otherwise the
 * number of total bytes must be used).  Finally, as mentioned, a caller can discover the structure
 * by systematic query (introspection).  The structure registration through FILLSRVSTRUCT should
 * occur only once per structure at initialization.  Call FILLSRVSTRUCT() for each element of the
 * structure in order and terminate the registration by passing a CF_NULL as the data type.
 *
 * \param tag is the (up to) 8-character structure tag to be used by the system kernel to identify
 * the structure.
 * \param addr is the reference inside the structure of the element being registered
 * \param size is the number of elements (array size) of the element being registered
 * \param fmt is the TINE data type of the element being registered
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \note Although conceptually equivalent, Client-side API calls in the Windows GUI world should
 * use FILLSTRUCT() and not FILLSRVSTRUCT().  This is because client and server GUI applications
 * usually work in different address spaces, and the structure registration must occur within the
 * address space of the application in order to function.
 *
 * \include eg_fillsrvstruct.txt
 *
 */
__declspec(dllexport) short PASCAL FILLSRVSTRUCT (char *tag, short addr, short size, short fmt);
__declspec(dllexport) long PASCAL GETSRVSTRUCTSIZE (char *tag);
__declspec(dllexport) short PASCAL DebugServer(short level);
__declspec(dllexport) short GetDeviceNumber(char *eqm,char *devname);
__declspec(dllexport) short GetDeviceNumberEx(char *eqm,char *devname,char *prpname);
__declspec(dllexport) short GetPropertyId(char *eqm,char *prpName);
__declspec(dllexport) void PASCAL GetCallerInfo(char *ClientName,char *ClientAddress);

__declspec(dllexport) double PASCAL getDataTimeStamp(void);
__declspec(dllexport) void PASCAL setDataTimeStamp(double ts);
double putDataTimeStamp(double toffset, time_t tsec, int tmsec);
__declspec(dllexport) double PASCAL makeDataTimeStamp(void);

/** 
 * \brief Sets the delay time between bursts of out-going datagrams from the server.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b SetCycleDelay Lib "tinesrv32.dll" (ByVal msec As Long) As Long
 *
 * A server fulfilling a contract via UDP datagrams does not know a priori what the total
 * transfer rate is between the end points (10baseT or 100baseT) and will insert an cycle
 * delay during a burst of outgoing datagrams.  Although the defaults generally assure that
 * everyone is satisfied, under certain conditions (for instance sending images over a high
 * speed network) it is desireable to configure the Cycle Delay and Burst Limit for 
 * optimal transport.
 *
 * \param msecs specifies the Cycle Delay to be used during bursts.
 *
 * \return The current cycle delay.
 *
 * \b default: 5 msec
 *
 * \see SetBurstLimit()
 *
 * \include eg_setcycledelay.txt
 *
 */
__declspec(dllexport) int PASCAL SetCycleDelay(int msecs);
__declspec(dllexport) int PASCAL SetSystemDelay(int msecs);
/** 
 * \brief Sets the burst limit in use before a Cycle Delay is applied.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b SetBurstLimit Lib "tinesrv32.dll" (ByVal npackets As Long) As Long
 *
 * A server fulfilling a contract via UDP datagrams does not know a priori what the total
 * transfer rate is between the end points (10baseT or 100baseT) and will insert an cycle
 * delay during a burst of outgoing datagrams.  Although the defaults generally assure that
 * everyone is satisfied, under certain conditions (for instance sending images over a high
 * speed network) it is desireable to configure the Cycle Delay and Burst Limit for 
 * optimal transport. 
 *
 * \param npackets specifies the number of consecutive datagrams sent to a client
 * which are allowed prior to applying a Cycle Delay.
 *
 * \return The current burst limit.
 *
 * \b default: 50
 *
 * \see SetCycleDelay()
 *
 * \include eg_setcycledelay.txt
 *
 */
__declspec(dllexport) int PASCAL SetBurstLimit(int npackets);

/** 
 * \brief Determines whether the server should be synchronized with a TINE time server
 *
 * \b Visual \b Basic :
 * \n Declare Function \b setGlobalSynchronization Lib "tinesrv32.dll" (ByVal value As Long)
 *
 * If a TINE time server is in place and providing the current system timestamp as a
 * network global, the a device server can synchronize its data timestamp by enabling
 * global synchronization (the default).  Note that this does not involve setting the 
 * system's clock but applies an offset to the TINE timestamp used for timestamping the 
 * data. If an ntp daemon is in place this feature may or may not be necessary.
 *
 * \param value determines whether sychnronization with a TINE time server is in effect
 * or not.
 *
 * \b default: TRUE
 *
 * \include eg_setglobalsync.txt
 *
 */
__declspec(dllexport) void PASCAL setGlobalSynchronization(int value);
/** 
 * \brief Determines whether the memory buffer holding the outgoing contract
 * data should be zeroed prior to calling the server's equipment functions.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b setSystemPresetMemory Lib "tinesrv32.dll" (ByVal v As Long)
 *
 * If an equipment module is apt to return fewer data points than requested for 
 * certain properties, it is sometimes prudent to make sure those data points not set
 * by the equipment module contain zeroes rather than random values from a prior call.
 * This can be realized by calling this routine with a non-zero value.  This will 
 * ensure that the contents of the output buffer is preset to zero before calling the
 * equipment module.  For large data blocks this could add unnecessary latency to the
 * call and a better approach might be to set the size of the output data to the
 * data size actually returned by calling the EqpSendDataEx() method within the 
 * call to the equipment module.
 *
 * \param value determines whether sychnronization with a TINE time server is in effect
 * or not.
 *
 * \b default: FALSE
 *
 * \include eg_setpresetmem.txt
 *
 */
__declspec(dllexport) void PASCAL setSystemPresetMemory(int value);
/** 
 * \brief Determines the low-level buffer space available for receiving
 * and sending data across the net.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b srvAssignBufferSpace Lib "tinesrv32.dll" (ByVal rcvBufferSize As Long, ByVal sndBufferSize As Long) As Integer
 *
 * If a server is expected to receive or send large amounts of data it is sometimes
 * advisable to increase the low-level socket buffer space available to the server.
 * This is especially true if the server is also a 'busy' server which can only
 * infrequently pull data out of its receive buffers for instance.  More than likely
 * the default settings are fine.  In is more common that this parameters need to be
 * adjusted on the client side rather than the server side, where a client might for 
 * instance be receiving large image frames at a fast rate,  To have an effect, this
 * routine must be called prior to initializing the server.
 *
 * \param rcvBufferSize is the amount of space (in bytes) available for receive buffers
 * for storing incoming data.
 * \param sndBufferSize is the amount of space (in bytes) available for send buffers
 * for storing outgoing data.
 *
 * \b default: \n
 * \n rcvBufferSize = 32768
 * \n sndBufferSize =  8192
 *
 * \include eg_setassignbufspace.txt
 *
 */
__declspec(dllexport) short PASCAL srvAssignBufferSpace(unsigned long rcvBufferSize,unsigned long sndBufferSize);

/** 
 * \brief Sets the server's working area buffer size
 *
 * \b Visual \b Basic :
 * \n Declare Function \b SetWorkAreaSize Lib "tinesrv32.dll" (ByVal siz As Long) As Long
 *
 * The properties supported by a server generally involve the double buffering of the return
 * data.  The data buffer used for the instantaneous call is a common data working area and
 * must be large enough to hold the largest data set which is to be returned by the server.
 * The default size of this working area is 64 KBytes, which is usually large enough for most
 * server applications.  Occasionally, some servers will need to return data sets larger than
 * this (video frames for instance).  In this case the default value of the working area size
 * should be set using a call to SetWorkAreaSize() with the largest value needed.  A call to
 * SetWorkAreaSize() must be made prior to enabling the server.
 *
 * \param siz is the size in bytes which is to correspond to the maximum data transfer 
 * supported. If 'siz' is smaller than the default size of 64 KBytes, the working area size
 * will not be changed.  In this way, a call to SetWorkAreaSize can be made passing a size of
 * '0' in order to determine the current working area size.
 *
 * \return the size in bytes of the working area.
 *
 *
 * \b default: 64 KBytes
 *
 * \include eg_setworkareasize.txt
 *
 */
__declspec(dllexport) int PASCAL SetWorkAreaSize(int siz);

__declspec(dllexport) short PASCAL SetMinimumPollingRate(short mpr);
/** 
 * \brief The server maintains its connection information in the form of a contract
 * table and a client table, both of which are pre-allocated lookup tables as opposed
 * to link lists.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b SetContractListSize Lib "tinesrv32.dll" (ByVal numContracts As Long) As Long
 *
 * If a server's contract table exceeds its allocated size, then a caller asking for a 
 * new contract will receive a 'resources exhausted' error message.  If a server is likely 
 * to deal with many contracts, it is a good idea to increase this value away from the
 * default of 256 entries.  Note that multiple clients requesting the same thing will
 * only introduce a single contract into the table.
 * To have an effect, this routine must be called prior to initializing the server.
 *
 * \param siz is the maximum number of simultaneous contracts which can be
 * serviced by the server.
 *
 * \b default: 256
 *
 * \include eg_setcontractlistsize.txt
 *
 */
__declspec(dllexport) int PASCAL SetContractListSize(int siz);
/** 
 * \brief The server maintains its connection information in the form of a contract
 * table and a client table, both of which are pre-allocated lookup tables as opposed
 * to link lists.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b SetConsumerListSize Lib "tinesrv32.dll" (ByVal numConsumers As Long) As Long
 *
 * If a server's client table exceeds its allocated size, then a new caller asking for a 
 * contract will receive a 'resources exhausted' error message.  If a server is likely 
 * to deal with many clients, it is a good idea to increase this value away from the
 * default of 256 entries.  
 * To have an effect, this routine must be called prior to initializing the server.
 *
 * \param siz is the maximum number of simultaneous contracts which can be
 * serviced by the server.
 *
 * \b default: 256
 *
 * \include eg_setcontractlistsize.txt
 *
 */
__declspec(dllexport) int PASCAL SetConsumerListSize(int siz);
__declspec(dllexport) unsigned long PASCAL CADDR(void *data);
__declspec(dllexport) double PASCAL ADDRD(unsigned long dataPtr);
__declspec(dllexport) float PASCAL ADDRF(unsigned long dataPtr);
__declspec(dllexport) unsigned long PASCAL ADDRL(unsigned long dataPtr);
__declspec(dllexport) unsigned short PASCAL ADDRS(unsigned long dataPtr);
__declspec(dllexport) unsigned long PASCAL UTIME(void);
__declspec(dllexport) short PASCAL SRVSTATUS(void);

/** 
 * \brief Sends a network global to networks and work stations in the ipbcasts.csv
 * list as well as to the global multicast group.
 *
 * \b Visual \b Basic :
 * \n Declare Function \b sendNetGlobal Lib "tinesrv32.dll" (ByVal keyword$, Data As Any, ByVal size As Long, ByVal fmt As Long, ByVal minPeriod As Long, ByVal maxPeriod As Long, ByVal tolerance As Double) As Long
 *
 * A server can send network globals via this routine.  Note that initial call appends 
 * the given parameters to the locally maintained globals list.  Entries in this list
 * are then transmitted to the global multicast group as well as any networks or workstations
 * found in the local ipbcasts.csv file.  
 *
 * \param keyword is the name of the network global.
 * \param data is a reference to the data block to be sent.
 * \param len is the length of the data block to be sent.
 * \param fmt is the TINE format of the data block to be sent.
 * \param minPeriod is the minimum interval in seconds between transmissions
 * \param maxPeriod is the maximum interval in seconds between transmissions
 * \param tolerance is the absolute allowed tolerance between successive data values
 * leading to a transmission
 *
 * \return 0 if successful, otherwise a TINE completion code.
 *
 * \b default: TRUE
 *
 * \include eg_sendnetglobal.txt
 *
 */
__declspec(dllexport) int PASCAL sendNetGlobal(char *keyword,BYTE *data,int len,int fmt,
		                               int minPeriod,int maxPeriod,double tolerance);
__declspec(dllexport) short PASCAL ScheduleProperty(char *eqm, char *prp);
__declspec(dllexport) short AttachToDatabase(char *expname,char *lclname,HWND hwnd);
#endif // _WINSRV_H_
