Attribute VB_Name = "Utils"
'==========================================================
'==                                                      ==
'==         General purpose plotting routines            ==
'==                                                      ==
'==========================================================

' -------------------------------------------------
'   For quick plotting, it is strongly suggested that
'   you use the routine DRAWXY described below
'   Several other routines are also given (for historical
'   reasons)
' -------------------------------------------------

Declare Sub drawtrace Lib "utils32.dll" (ByVal hdc&, l%, ByVal n%, ByVal ym%)

' -------------------------------------------------
' | plots the trace array (e.g. x%()) vs its index |
' | from 0 to NPOINTS - 1                          |
' |  e.g.                                          |
' -------------------------------------------------
'
'    Call DRAWTRACE(picture1.hDC, x%(0), NPOINTS, YMAX)
'

Declare Sub drawtime Lib "utils32.dll" (ByVal hdc&, ByVal X%, ByVal Y%)
Declare Sub HISTOGRAM Lib "utils32.dll" Alias "HISTO" (ByVal hdc&, a!, ByVal aMAX!, ByVal npts%, ByVal wdth%, ByVal offset%, ByVal mode%)

' ---------------------------------------------------
' | mode% = 0 plots positive values only            |
' | mode% = 1 plots negative and positive values    |
' | wdth% gives the spacing between histogram lines |
' | offset% gives the starting position             |
' | e.g.                                            |
' ---------------------------------------------------
'
'    Call HISTO(picture1.hDC, loss(2), ymax, numBLM%, 2, 0, 0)
'
'
Declare Sub PLOTXY Lib "utils32.dll" (ByVal hdc&, Y!, ByVal ymax!, ByVal ymin!, X!, ByVal npts%)

' ----------------------------------------
' | plots array x() vs array y(), e.g.   |
' ----------------------------------------
'
'    Call PLOTXY(picture1.hDC, y(0), ymax, ymin, x(0), npts%)
'
Declare Sub DRAWXY Lib "utils32.dll" (ByVal hdc&, Y!, ByVal ymax!, ByVal ymin!, X!, ByVal npts%, ByVal mode%)
Declare Sub WINDOWXY Lib "utils32.dll" (ByVal hdc&, Y!, ByVal ymax!, ByVal ymin!, X!, ByVal xmax!, ByVal xmin!, ByVal npts%, ByVal mode%)

' ---------------------------------------------------------------
' | plots array x() vs array y() according to given mode, e.g.   |
' | x() and y() both float arrays
' |
' |  MODE:  type of display:
' |     0     connect-the-dots
' |     1     bar-graph
' |     2     histogram
' |     3     individual points
' ---------------------------------------------------------------
'
'    Call DRAWXY(picture1.hDC, y(0), ymax, ymin, x(0), npts%, 0)
'
' ---------------------------------------------------------------
' | WINDOWXY allows you to specify the x-axis window through
' | the xmax and xmin parameters, DRAWXY calculates them for you
' ---------------------------------------------------------------
'
Declare Sub ARRMULT Lib "utils32.dll" (ByVal typ%, a As Any, ByVal v As Double, ByVal num%)
'
' -----------------------------------------------------
' | multiplies given array by given value according to |
' | specified type: CF_INTEGER,CF_FLOAT,CF_LONG,       |
' | CF_DOUBLE                                          |
' -----------------------------------------------------
'
'   Call ARRMULT(CF_FLOAT, f(0), x!, npts%)
'
Declare Sub MEMCPY Lib "utils32.dll" (dst As Any, src As Any, ByVal nBytes%)
Declare Function MEMCMP Lib "utils32.dll" (dst As Any, src As Any, ByVal nBytes%) As Integer
Declare Sub ITOF Lib "utils32.dll" (dst!, src%, ByVal num%, ByVal scl!)
Declare Sub LTOF Lib "utils32.dll" (dst!, src&, ByVal num%, ByVal scl!)
Declare Function MEAN Lib "utils32.dll" (arr!, avearr!, ByVal num%, ByVal depth%) As Integer
Declare Function NORM Lib "utils32.dll" (newarr!, avearr!, ByVal num%) As Integer

' ---------------------------------------
' | scales integer array ix%() (ITOF) or |
' | lx&() (LTOF) by xscale!              |
' | puts the result in float array fx()  |
' ----------------------------------------
'
'    call ITOF(fx!(0), ix%(0), npts%, xscale!)
'    call LTOF(fx!(0), lx&(0), npts%, xscale!)
'
Declare Function GETBIT Lib "utils32.dll" (ByVal wrd%, ByVal BIT%) As Integer
Declare Function GET32BIT Lib "utils32.dll" (ByVal wrd&, ByVal BIT%) As Integer

' --------------------------------------------
' |   returns value of specified bit (0 or 1) |
' --------------------------------------------

Declare Function SETBIT Lib "utils32.dll" (ByVal wrd%, ByVal BIT%) As Integer

' -----------------------------------------------------
' |   sets specified bit to 1 and returns the new word |
' -----------------------------------------------------

Declare Function CLEARBIT Lib "utils32.dll" (ByVal wrd%, ByVal BIT%) As Integer

' -----------------------------------------------------
' |   sets specified bit to 0 and returns the new word |
' -----------------------------------------------------

Type TM
  tm_sec As Long   '0 - 59
  tm_min As Long   '0 - 59
  tm_hour As Long  '0 - 23
  tm_mday As Long  'day of month (1 - 31)
  tm_mon As Long   'use 1 = Jan., 12 = Dec.
  tm_year As Long  'either 1998 or 98 will work
  tm_wday As Long  ' not used
  tm_yday As Long  ' not used
  tm_isdst As Long ' not used
End Type

Declare Function CONVTIME Lib "utils32.dll" (ByVal dat$, ByVal LONGtime&) As Integer
Declare Sub CTIME Lib "utils32.dll" (ByVal dat As String, ByVal LONGtime As Long)
Declare Function MKTIME Lib "utils32.dll" (t As TM) As Long
Declare Sub LCLTIME Lib "utils32.dll" (ByVal t As Long, st As TM)

' -----------------------------------------------------
' | returns ascii time string with current time and date |
' | given the unix time constant LONGtime&               |
' | be sure to reserve space for dat$ in global.bas      |
' -----------------------------------------------------
'      global dat as string * 25
'
'      rc% = CONVTIME(dat, timen&)

Declare Function UNIXTIME Lib "utils32.dll" () As Long


'==========================================================
'==                                                      ==
'==         Network utilities                            ==
'==                                                      ==
'==========================================================

Declare Function PingRemoteHost Lib "ping32.dll" (ByVal hostname$) As Long

'==========================================================
'==                                                      ==
'==            Math routines                             ==
'==                                                      ==
'==========================================================

Declare Function FFT Lib "utils32.dll" (ByVal num%, x_arr!, f_arr!, q_max!, ByVal percent!, ByVal cut!) As Integer

'===========================================================
'==
'==        Some Win32 API Declares ....
'==
'===========================================================
Type OrientStructure
  Orientation As Long
  pad As String * 16
End Type
'
Declare Function Escape% Lib "GDI" (ByVal hdc&, ByVal nEsc%, ByVal nLen%, lpData As OrientStructure, lpOut As Any)

Public Declare Sub MemoryStatus Lib "utils32.dll" (physicalMemory As Long, virtualMemory As Long)
Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
Declare Function FindWindow Lib "USER32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Long
Declare Function FindWindowEx Lib "USER32" Alias "FindWindowExA" (ByVal hWnd1 As Long, ByVal hWnd2 As Long, ByVal lpsz1 As String, ByVal lpsz2 As String) As Long
Declare Function GetWindowText Lib "USER32" Alias "GetWindowTextA" (ByVal hWnd As Long, ByVal lpString As String, ByVal cch As Long) As Long
Declare Function SetForegroundWindow Lib "USER32" (ByVal hWnd As Long) As Long
Declare Function ShowWindow Lib "USER32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Declare Function ShowWindowAsync Lib "USER32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Public Const SW_RESTORE = 9
Public Declare Function GetTickCount Lib "kernel32" () As Long

Sub WaitTime(ByVal spWaitTime As Single)
'------------------------------------------------------------------------------
'---Wait Time fuer angegebene Zeit in Sekunden                              ---
'---'TickCounts' zaehlen vom Rechnerstart an uns sind unabhaengig von der   ---
'---am PC eingestellten Zeit. Umsetzten der Zeit aendert die 'TickCounts'   ---
'---nicht. Pro Millisekunde wir der TickCount um 1 erhoeht.                 ---
'------------------------------------------------------------------------------
' Version   : 3                                                             ---
' Date      : 2001-01-12                                                    ---
'  Version    Datum     Beschreibung                                        ---
'     2     2001-01-25  Funktion leicht angeaendert, wurde bisher nie benutzt--
'     3     2001-05-17  Ueberlauf TickCount behandelt                       ---
'------------------------------------------------------------------------------
' Parameter  Typ  I/O   Description                                         ---
' -------------------------------------------------------------------       ---
' spWaitTime Sgl  I/-   Wartezeit in Sekunden                               ---
'------------------------------------------------------------------------------
Dim MilliNow    As Long
Dim MilliStart  As Long
    
    MilliStart = GetTickCount()
    Do
        DoEvents
        MilliNow = GetTickCount()
        If MilliNow < MilliStart Then MilliStart = MilliNow
    Loop Until MilliNow - MilliStart + 9 > spWaitTime * 1000
    
End Sub

Sub dumpGlobalMemory()
Dim physMem As Long, virtMem As Long
Debug.Print Now
MemoryStatus physMem, virtMem
Debug.Print "available virtual  : " + str$(virtMem)
Debug.Print "available physical : " + str$(physMem)
Debug.Print vbNewLine
End Sub
Function IsCleared(UserFile$, UserName$) As Integer
Dim a$
FileNumber = FreeFile   ' Get unused file
On Error GoTo OpenError
Open App.Path$ + "\" + UserFile$ For Input Access Read Shared As #FileNumber
On Error GoTo 0
Input #1, a$  ' Header
done = False: IsCleared = False
While Not EOF(1) And Not done
  Input #1, a$
  If a$ = UserName$ Then
    IsCleared = True
    done = True
  End If
Wend
Close (FileNumber)
Exit Function
OpenError:
MsgBox "File " + UserFile$ + " does not exist"
Exit Function
End Function
Function ASCTIME(timestamp As Long) As String
Static tmpTime As String * 26
If timestamp < 0 Then Exit Function
CTIME tmpTime, timestamp
ASCTIME = Left$(tmpTime, 24)
End Function

Function MAKETIME(yr As Integer, mo As Integer, da As Integer, hr As Integer, se As Integer) As Long
Static t As TM
t.tm_hour = hr Mod 24
t.tm_mday = da
t.tm_mon = mo
t.tm_year = yr
t.tm_sec = se
MAKETIME = MKTIME(t)
End Function
Sub Paperorient(to_land As Integer)

    Const PORTRAIT = 1
    Const LANDSCAPE = 2
    Const GETSETPAPERORIENT = 30
    Const myNULL = 0&
    
    Dim Orient As OrientStructure

    '* Specify the orientation
    If to_land Then Orient.Orientation = LANDSCAPE Else Orient.Orientation = PORTRAIT

    '* Send escape sequence to change orientation
    Dim dummy
    dummy = Escape(Printer.hdc, GETSETPAPERORIENT, Len(Orient), Orient, myNULL)

    Printer.EndDoc 'printer mu jetzt neue initialisiert werden

End Sub

Function colptr(s$, costr$) As Integer
Dim tmpstr$, colstr$
Dim ncommas As Integer
Dim pos As Long

colstr$ = UCase(costr$)
ncommas = 0: tmpstr$ = UCase(s$)
pos = InStr(tmpstr$, ",") - 1
If pos = -1 Then pos = Len(tmpstr$)
While (pos > 0) And (Left$(tmpstr$, pos) <> colstr$)
  If Len(tmpstr$) = pos Then
    colptr = -1
    Exit Function
  End If
  ncommas = ncommas + 1
  tmpstr$ = Right$(tmpstr$, Len(tmpstr$) - pos - 1)
  If Left$(tmpstr$, 1) = Chr(34) Then ' trap '"'
    pos = InStr(Right$(tmpstr$, Len(tmpstr$) - 1), Chr(34)) + 1
  Else
    pos = InStr(tmpstr$, ",") - 1
  End If
  If pos = -1 Then pos = Len(tmpstr$)
Wend
colptr = ncommas
End Function
Function getcol(s$, ByVal col%) As String
Dim tmpstr$
Dim ncommas As Integer
Dim pos As Long
ncommas = 0: tmpstr$ = s$
pos = InStr(tmpstr$, ",") - 1
If pos = -1 Then pos = Len(tmpstr$)
While (pos >= 0) And (ncommas <> col%)
  ncommas = ncommas + 1
  tmpstr$ = Right$(tmpstr$, Len(tmpstr$) - pos - 1)
  If Left$(tmpstr$, 1) = Chr(34) Then ' trap '"'
    pos = InStr(Right$(tmpstr$, Len(tmpstr$) - 1), Chr(34)) + 1
  Else
    pos = InStr(tmpstr$, ",") - 1
  End If
  If pos = -1 Then pos = Len(tmpstr$)
Wend
getcol = IIf(ncommas = col%, Left$(tmpstr$, pos), "")
End Function
Function ActivatePrevInstance(hThisWnd As Long, WindowText As String) As Boolean
'e.g :
'If ActivatePrevInstance(Me.hWnd, Me.Caption) Then EndApplication ' Found and activated previous instance
Dim i As Integer, hw&
hw& = FindWindowEx(0, hw&, vbNullString, WindowText)
For i = 0 To 1
    If hw& And hw& <> hThisWnd Then
      ShowWindowAsync hw&, SW_RESTORE
      SetForegroundWindow hw&
      ActivatePrevInstance = True
      Exit Function
    End If
    hw& = FindWindowEx(0, hw&, vbNullString, WindowText)
Next
ActivatePrevInstance = False
End Function
Sub EndApplication()
Dim f As Variant
For Each f In Forms
  Unload f
Next f
End
End Sub

